/***************************************************************
 * Name:      dxde.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__

#include <wx/aboutdlg.h>
#include <wx/sysopt.h>
#include <wx/aui/aui.h>
#include <wx/numdlg.h>
#include <wx/splash.h>
#include <wx/stdpaths.h>
#include <wx/filefn.h>
#include <stack>
#ifndef __WXMSW__
    #include "resources/images/dxde.xpm"
#endif
#include "resources/images/dxde64.xpm"
#include "resources/images/splash.xpm"
#include "dxdefs.h"
#include "config.h"
#include "dxde.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "gui/dialogs/firmadialog.h"
#include "gui/dialogs/kontaktydialog.h"
#include "gui/dialogs/predkontacesdialog.h"
#include "gui/dialogs/predvolbydialog.h"
#include "gui/dialogs/fauhradydialog.h"
#include "gui/dialogs/datumdialog.h"
#include "gui/dialogs/odeslanedialog.h"
#include "gui/dialogs/dphdruhydialog.h"
#include "gui/dialogs/zalohadialog.h"
#include "gui/wizards/firmawizard.h"
#include "gui/pages/dxbook.h"
#include "gui/pages/homepage.h"
#include "gui/pages/denikpage.h"
#include "gui/pages/pokladnapage.h"
#include "gui/pages/bankapage.h"
#include "gui/pages/vydanapage.h"
#include "gui/pages/prijatapage.h"
#include "gui/pages/pohledavkapage.h"
#include "gui/pages/zavazekpage.h"
#include "gui/pages/majetekpage.h"
#include "gui/pages/skladpage.h"
#include "gui/pages/jizdapage.h"
#include "gui/template/templateeditor.h"
#include "gui/taxes/seznamdphdialog.h"
#include "gui/taxes/priznanidialog.h"
#include "gui/taxes/dphdp3wizard.h"
#include "gui/taxes/prijmyvydajedialog.h"
#include "gui/taxes/dphshwizard.h"
#include "gui/taxes/dphkh1wizard.h"
#include "gui/taxes/ulozenedialog.h"
#include "gui/dialogs/skladdialog.h"
#include "gui/dialogs/inventurydialog.h"
#include "gui/taxes/majetekzavazkydialog.h"
#ifdef HAVE_CURL
    #include <curl/curlver.h>
#endif
#include "pdf/include/wx/pdfdoc_version.h"

wxSplashScreen *splash = NULL;

IMPLEMENT_APP(dxdeApp)

bool dxdeApp::OnInit()
{
    m_checker = new wxSingleInstanceChecker("dxde");
    if(m_checker->IsAnotherRunning())
    {
        wxLogError(wxString::FromUTF8("dxDE už běží"));
        delete m_checker; // OnExit() won't be called if we return false
        m_checker = NULL;
        return false;
    }
    wxInitAllImageHandlers();
    dxsettings.load();
#if defined (__WXMAC__) || defined (__WXOSX__)
    wxSystemOptions::SetOption(wxT("mac.listctrl.always_use_generic"),1);
#endif
    wxLocale::AddCatalogLookupPathPrefix(LOCALEDIR);
    m_locale = new wxLocale();
    m_locale->Init(wxLANGUAGE_DEFAULT);

    m_frame = new dxdeFrame(0L, "dxDE");
    m_frame->SetIcon(wxICON(dxde)); // To Set App Icon
    m_frame->Show();

    return true;
}

int dxdeApp::OnExit()
{
    if(m_checker)
        delete m_checker;
    return 0;
}

void dxdeApp::ShowPreferencesEditor(wxWindow *parent)
{
    m_preferences.reset(new wxPreferencesEditor);
    m_preferences->AddPage(new FakturacePage());
    m_preferences->AddPage(new RadyPage());
    m_preferences->AddPage(new EmailPage());
    m_preferences->Show(parent);
}

void dxdeApp::DismissPreferencesEditor()
{
    if(m_preferences)
        m_preferences->Dismiss();
}

BEGIN_EVENT_TABLE(dxdeFrame, wxFrame)
    EVT_MENU(wxID_EXIT, dxdeFrame::OnQuit)
    EVT_CLOSE(dxdeFrame::OnClose)
    EVT_MENU(wxID_ABOUT, dxdeFrame::OnAbout)
    EVT_MENU(wxID_HELP, dxdeFrame::OnHelp)
    EVT_MENU(ID_MENU_FIRMA_EDIT, dxdeFrame::OnEditFirma)
    EVT_MENU(ID_MENU_FIRMA_ADD, dxdeFrame::OnAddFirma)
    EVT_MENU(ID_MENU_FIRMA_DELETE, dxdeFrame::OnDeleteFirma)
    EVT_MENU(ID_MENU_FIRMA_CHANGE, dxdeFrame::OnChangeFirma)
    EVT_MENU(ID_MENU_ADRESAR, dxdeFrame::OnAdresar)
    EVT_MENU(ID_MENU_ZALOHA, dxdeFrame::OnZaloha)
    EVT_MENU(ID_MENU_OBNOVA, dxdeFrame::OnObnova)
    EVT_MENU(ID_MENU_OBDOBI_ADD, dxdeFrame::OnAddObdobi)
    EVT_MENU(ID_MENU_OBDOBI_CHANGE, dxdeFrame::OnChangeObdobi)
    EVT_MENU(ID_MENU_OBDOBI_DELETE, dxdeFrame::OnDeleteObdobi)
    EVT_TREE_SEL_CHANGED(ID_NAVIGATIONTREE, dxdeFrame::OnNavigationChanged)
    EVT_COMMAND(wxID_ANY, MODEL_UPDATED, dxdeFrame::OnModelUpdated)
    EVT_COMMAND(wxID_ANY, MODEL_MSG, dxdeFrame::OnModelMsg)
    EVT_MENU(ID_MENU_DENIK, dxdeFrame::OnDenik)
    EVT_MENU(ID_MENU_POKLADNA, dxdeFrame::OnPokladna)
    EVT_MENU(ID_MENU_BANKA, dxdeFrame::OnBanka)
    EVT_MENU(ID_MENU_PREDKONTACE, dxdeFrame::OnPredkontace)
    EVT_MENU(ID_MENU_STAVPOKLADENBANK, dxdeFrame::OnStavPokladenBank)
    EVT_MENU(wxID_PREFERENCES, dxdeFrame::OnPreferences)
    EVT_MENU(ID_MENU_VYSTAVENA, dxdeFrame::OnVystavena)
    EVT_MENU(ID_MENU_PRIJATA, dxdeFrame::OnPrijata)
    EVT_MENU(ID_MENU_POHLEDAVKA, dxdeFrame::OnPohledavka)
    EVT_MENU(ID_MENU_ZAVAZEK, dxdeFrame::OnZavazek)
    EVT_MENU(ID_MENU_MAJETEK, dxdeFrame::OnMajetek)
    EVT_MENU(ID_MENU_SKLAD, dxdeFrame::OnSklad)
    EVT_MENU(ID_MENU_INVETURY, dxdeFrame::OnInventury)
    EVT_MENU(ID_MENU_FAUHRADY, dxdeFrame::OnFauhrady)
    EVT_MENU(ID_MENU_DATUM, dxdeFrame::OnDatum)
    EVT_MENU(ID_MENU_ODESLANE, dxdeFrame::OnOdeslane)
    EVT_MENU(ID_MENU_TEMPLATEEDITOR, dxdeFrame::OnTemplateEditor)
    EVT_MENU(ID_MENU_DPHDRUH, dxdeFrame::OnDphDruh)
    EVT_MENU(ID_MENU_DPHPRIZNANI, dxdeFrame::OnDphPriznani)
    EVT_MENU(ID_MENU_DPHKH, dxdeFrame::OnDphKH)
    EVT_MENU(ID_MENU_DPHSEZNAM, dxdeFrame::OnDphSeznam)
    EVT_MENU(ID_MENU_DPHSH, dxdeFrame::OnDphSH)
    EVT_MENU(ID_MENU_DPPV, dxdeFrame::OnDpPrijmyVydaje)
    EVT_MENU(ID_MENU_DPMZ, dxdeFrame::OnDpMajetekZavazky)
    EVT_MENU(ID_MENU_ULOZENE, dxdeFrame::OnUlozene)
    EVT_MENU(ID_MENU_VERSION, dxdeFrame::OnCheckVersion)
    EVT_MENU(ID_MENU_JIZDY, dxdeFrame::OnJizdy)
END_EVENT_TABLE()

dxdeFrame::dxdeFrame(wxFrame *frame, const wxString& title)
    : wxFrame(frame, -1, title, wxDefaultPosition, wxSize(800,600))
{
#if wxUSE_MENUS
    // create a menu bar
    wxMenuBar *mbar = new wxMenuBar();
    wxMenu *fileMenu = new wxMenu();
#if (defined (__WXMAC__) || defined (__WXOSX__))
    fileMenu->Append(wxID_ABOUT);
    fileMenu->Append(wxID_PREFERENCES);
#endif

    wxMenu *firmaMenu = new wxMenu();
    firmaMenu->Append(ID_MENU_FIRMA_EDIT, "Upravit firmu", wxString::FromUTF8("Otevřít nastavení účtované firmy"));
    firmaMenu->Append(ID_MENU_FIRMA_DELETE, "Smazat firmu", wxString::FromUTF8("Smazat firmu"));
    firmaMenu->AppendSeparator();
    firmaMenu->Append(ID_MENU_FIRMA_ADD, wxString::FromUTF8("Přidat firmu"), wxString::FromUTF8("Přidat firmu"));
    firmaMenu->AppendSeparator();
    m_changeFirma = new wxMenuItem(NULL, ID_MENU_FIRMA_CHANGE, wxString::FromUTF8("Změnit firmu"), wxString::FromUTF8("Změnit firmu"));
    firmaMenu->Append(m_changeFirma);
    m_changeFirma->Enable(false);
    fileMenu->AppendSubMenu(firmaMenu, "Firma");
    wxMenu *obdobiMenu = new wxMenu();
    obdobiMenu->Append(ID_MENU_OBDOBI_ADD, wxString::FromUTF8("Přidat období"), wxString::FromUTF8("Přidat období"));
    obdobiMenu->Append(ID_MENU_OBDOBI_CHANGE, wxString::FromUTF8("Změnit období"), wxString::FromUTF8("Změnit období"));
    m_deleteObdobi = new wxMenuItem(NULL, ID_MENU_OBDOBI_DELETE, wxString::FromUTF8("Smazat období"), wxString::FromUTF8("Smazat období"));
    obdobiMenu->Append(m_deleteObdobi);
    m_deleteObdobi->Enable(false);
    fileMenu->AppendSubMenu(obdobiMenu, wxString::FromUTF8("Období"));
    fileMenu->Append(ID_MENU_ADRESAR, wxString::FromUTF8("Adresář"), wxString::FromUTF8("Otevřít adresář"));
    fileMenu->AppendSeparator();
    wxMenu *daneMenu = new wxMenu();
    wxMenu *dpMenu = new wxMenu();
    dpMenu->Append(ID_MENU_DPPV, wxString::FromUTF8("Přehled příjmů a výdajů"), wxString::FromUTF8("Přehled příjmů a výdajů"));
    dpMenu->Append(ID_MENU_DPMZ, wxString::FromUTF8("Přehled majetku a závázků"), wxString::FromUTF8("Přehled majetku a závázků"));
    daneMenu->AppendSubMenu(dpMenu, wxString::FromUTF8("Daň z příjmů"));
    wxMenu *dphMenu = new wxMenu();
    m_dphPriznani = new wxMenuItem(NULL, ID_MENU_DPHPRIZNANI, wxString::FromUTF8("Přiznání k DPH"), wxString::FromUTF8("Přiznání k DPH"));
    dphMenu->Append(m_dphPriznani);
    m_dphPriznani->Enable(false);
    m_dphKH = new wxMenuItem(NULL, ID_MENU_DPHKH, wxString::FromUTF8("Kontrolní hlášení DPH"), wxString::FromUTF8("Kontrolní hlášení DPH"));
    dphMenu->Append(m_dphKH);
    m_dphKH->Enable(false);
    m_dphSH = new wxMenuItem(NULL, ID_MENU_DPHSH, wxString::FromUTF8("Souhrnné hlášení DPH"), wxString::FromUTF8("Souhrnné hlášení DPH"));
    dphMenu->Append(m_dphSH);
    m_dphSH->Enable(false);
    m_dphSeznam = new wxMenuItem(NULL, ID_MENU_DPHSEZNAM, wxString::FromUTF8("Seznam dokladů k DPH"), wxString::FromUTF8("Seznam dokladů k DPH"));
    dphMenu->Append(m_dphSeznam);
    m_dphSeznam->Enable(false);
    daneMenu->AppendSubMenu(dphMenu, "DPH");
    m_ulozene = new wxMenuItem(NULL, ID_MENU_ULOZENE, wxString::FromUTF8("Uložená přiznání"), wxString::FromUTF8("Uložená přiznání"));
    daneMenu->Append(m_ulozene);
    m_ulozene->Enable(false);
    fileMenu->AppendSubMenu(daneMenu, wxString::FromUTF8("Daně"));
    fileMenu->AppendSeparator();
    fileMenu->Append(ID_MENU_ZALOHA, wxString::FromUTF8("Zálohovat data"), wxString::FromUTF8("Zálohovat data"));
    fileMenu->Append(ID_MENU_OBNOVA, wxString::FromUTF8("Obnovit data ze zálohy"), wxString::FromUTF8("Obnovit data ze zálohy"));
    fileMenu->AppendSeparator();
    fileMenu->Append(wxID_EXIT);
    mbar->Append(fileMenu, wxString::FromUTF8("&Systém"));

    wxMenu* ucetMenu = new wxMenu();
    ucetMenu->Append(ID_MENU_DENIK, wxString::FromUTF8("Peněžní deník"), wxString::FromUTF8("Peněžní deník"));
    ucetMenu->Append(ID_MENU_POKLADNA, "Pokladna", "Pokladna");
    ucetMenu->Append(ID_MENU_BANKA, "Banka", "Banka");
    ucetMenu->AppendSeparator();
    ucetMenu->Append(ID_MENU_PREDKONTACE, wxString::FromUTF8("Předkontace"), wxString::FromUTF8("Předkontace"));
    ucetMenu->Append(ID_MENU_STAVPOKLADENBANK, wxString::FromUTF8("Stav pokladen a bankovních účtů"), wxString::FromUTF8("Stav pokladen a bankovních účtů"));
    mbar->Append(ucetMenu, wxString::FromUTF8("Úče&tnictví"));

    wxMenu *faMenu = new wxMenu();
    faMenu->Append(ID_MENU_VYSTAVENA, wxString::FromUTF8("Faktury vydané"), wxString::FromUTF8("Faktury vydané"));
    faMenu->Append(ID_MENU_PRIJATA, wxString::FromUTF8("Faktury přijaté"), wxString::FromUTF8("Faktury přijaté"));
    faMenu->AppendSeparator();
    faMenu->Append(ID_MENU_POHLEDAVKA, wxString::FromUTF8("Pohledávky"), wxString::FromUTF8("Pohledávky"));
    faMenu->Append(ID_MENU_ZAVAZEK, wxString::FromUTF8("Závazky"), wxString::FromUTF8("Závazky"));
    faMenu->AppendSeparator();
    faMenu->Append(ID_MENU_FAUHRADY, wxString::FromUTF8("Úhrady"), wxString::FromUTF8("Úhrady"));
    faMenu->AppendSeparator();
    faMenu->Append(ID_MENU_ODESLANE, wxString::FromUTF8("Seznam odeslaných faktur"), wxString::FromUTF8("Seznam odeslaných faktur"));
    faMenu->AppendSeparator();
    m_dphDruh = new wxMenuItem(NULL, ID_MENU_DPHDRUH, wxString::FromUTF8("Členění DPH"), wxString::FromUTF8("Členění DPH"));
    faMenu->Append(m_dphDruh);
    m_dphDruh->Enable(false);
    mbar->Append(faMenu, "&Fakturace");

    wxMenu *majMenu = new wxMenu();
    majMenu->Append(ID_MENU_MAJETEK, "Kniha majetku", "Kniha majetku");
    mbar->Append(majMenu, "Ma&jetek");

    wxMenu *sklMenu = new wxMenu();
    sklMenu->Append(ID_MENU_SKLAD, wxString::FromUTF8("Kniha zásob"), wxString::FromUTF8("Kniha zásob"));
    sklMenu->Append(ID_MENU_INVETURY, "Inventury", "Inventury");
    mbar->Append(sklMenu, "Sk&lad");

    wxMenu *editMenu = new wxMenu();
#if !(defined (__WXMAC__) || defined (__WXOSX__))
    editMenu->Append(wxID_PREFERENCES);
    editMenu->AppendSeparator();
#endif
    editMenu->Append(ID_MENU_DATUM, "Datum aplikace\tAlt-D", "Datum aplikace");
    editMenu->AppendSeparator();
    editMenu->Append(ID_MENU_TEMPLATEEDITOR, wxString::FromUTF8("Editor šablon"), wxString::FromUTF8("Editor šablon"));
    mbar->Append(editMenu, wxString::FromUTF8("Ú&pravy"));

    wxMenu *helpMenu = new wxMenu();
    helpMenu->Append(ID_MENU_VERSION, "Zkontrolovat aktualizace", "Zkontrolovat aktualizace");
    helpMenu->Append(wxID_HELP);
#if !(defined (__WXMAC__) || defined (__WXOSX__))
    helpMenu->AppendSeparator();
    helpMenu->Append(wxID_ABOUT);
#endif
    mbar->Append(helpMenu, wxString::FromUTF8("&Nápověda"));

    SetMenuBar(mbar);
#endif // wxUSE_MENUS

#if wxUSE_STATUSBAR
    CreateStatusBar(3);
    SetStatusText("dxDE",0);
#endif // wxUSE_STATUSBAR

    m_navigation = new wxTreeCtrl(this, ID_NAVIGATIONTREE, wxDefaultPosition, wxSize(100,100), wxTR_SINGLE|wxTR_NO_LINES|wxTR_HAS_BUTTONS);
    m_book = new dxBook(this, wxID_ANY);
    m_manager.AddPane(m_navigation, wxAuiPaneInfo().
                      Name("Navigation").Caption("Navigation").
                      BestSize(wxSize(200, 100)).MinSize(wxSize(100, 100)).
                      CaptionVisible(false).CloseButton(false).
                      Left());
    m_manager.AddPane(m_book, wxAuiPaneInfo().
                      Name("Book").Caption("Book").
                      CenterPane().PaneBorder(false));
    m_manager.SetManagedWindow(this);
    m_manager.LoadPerspective(dxsettings.auiperspective());
    m_manager.SetFlags(wxAUI_MGR_DEFAULT | wxAUI_MGR_TRANSPARENT_DRAG | wxAUI_MGR_ALLOW_ACTIVE_PANE);
    m_manager.Update();

    SetSize(dxsettings.x(), dxsettings.y(), dxsettings.w(), dxsettings.h());
    if(dxsettings.maximized())
        Maximize();

    if(loadDatabase())
    {

        createNavigation();

        if(!m_book->HasPage(NAV_HOME))
        {
            HomePage *home = new HomePage(m_book);
            m_book->AppendPage(home);
        }

#if defined (__WXMAC__)
        m_help.SetTempDir(wxStandardPaths::Get().GetResourcesDir());
        m_help.AddBook(wxFileName(wxStandardPaths::Get().GetResourcesDir()+"/help/dxde.hhp"));
#else
        m_help.SetTempDir(wxStandardPaths::Get().GetTempDir());
        m_help.AddBook(wxFileName(wxString(DXDE_DATADIR)+"/help/dxde.hhp"), wxPATH_UNIX);
#endif
    }
    else
    {
        Destroy();
    }
}

dxdeFrame::~dxdeFrame()
{
    dxsettings.setX(GetPosition().x);
    dxsettings.setY(GetPosition().y);
    dxsettings.setW(GetSize().x);
    dxsettings.setH(GetSize().y);
    dxsettings.setMaximized(IsMaximized());
    dxsettings.setAuiperspective(m_manager.SavePerspective());
    dxsettings.save();
    m_model->closeDatabase();
    m_model->shutdownModel();
    m_manager.UnInit();
}

bool dxdeFrame::loadDatabase()
{
    m_model = new DbModel(this);
    m_model->initModel();
    bool newdb = false;
    //prvni spusteni nebo db byla smazana
    if(dxsettings.dbname().IsEmpty())
    {
        FirmaWizard wizard(this, true);
        wizard.RunWizard(wizard.GetFirstPage());
        if(!wizard.success())
        {
            return false;
        }
        newdb = true;
    }
    if(!splash)
    {
        splash = new wxSplashScreen(wxBitmap(splash_xpm), wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_NO_TIMEOUT, 0, NULL, wxID_ANY);
        wxYield();
    }
    if(dxutils::versionToInt(VERSION) >= 1300)
    {
        //update dbfirma aby obsahoval IC
        m_model->updateDbFirma();
    }
    if(!newdb)
    {
        if(wxFileExists(wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+dxsettings.dbname()))
        {
            if(m_model->openDatabase())
            {
                m_model->fillData();
            }
            else
            {
                wxLogError(wxString::FromUTF8("Nepodařilo se otevřít databázi"));
                return false;
            }
        }
        else
        {
            if(dxsettings.dbname().IsEmpty()) dxsettings.setDbname(dxutils::getUniqueDbName());
            m_model->newDatabase();
            m_model->addAdresa(Adresa(1, "", "", "", "", ""));
            wxString firma = "ABC_0";
            int i = 1;
            while(dxsettings.firmaExists(firma))
            {
                firma = wxString::Format("ABC_%d", i);
                i++;
            }
            m_model->setFirma(Firma(firma.Before('_'), firma.After('_'), "", "", false, false, "", "", "", "", "", 1));
            m_model->addObdobi(Obdobi(wxDateTime::Today().GetYear(), true));
            m_book->UpdatePages(ID_MODEL_ALL);
            wxMessageBox(wxString::FromUTF8("Nepodařilo se otevřít databázi\nByla vytvořena nová"));
        }
    }
    if(splash)
    {
        splash->Close(true);
        wxDELETE(splash);
    }
    dxsettings.editDbnamePair(dxsettings.dbname(), m_model->firma().nazev()+"_"+m_model->firma().ic());
    m_changeFirma->Enable(dxsettings.dbcount()>1?true:false);
    m_deleteObdobi->Enable(m_model->getObdobiCount()>1?true:false);
    m_dphDruh->Enable(m_model->isFirmaPlatce()?true:false);
    m_dphPriznani->Enable(m_model->isFirmaPlatce()?true:false);
    m_dphKH->Enable(m_model->isFirmaPlatce()?true:false);
    m_dphSH->Enable(m_model->isFirmaPlatce()?true:false);
    m_dphSeznam->Enable(m_model->isFirmaPlatce()?true:false);
    m_ulozene->Enable(m_model->isFirmaPlatce()?true:false);
    dxsettings.setDatum(dxutils::todayForObdobi());
    createRemoveJizdy();
    updateStatusText();
    return true;
}

void dxdeFrame::createNavigation()
{
    m_navigation->DeleteAllItems();

    wxTreeItemId root = m_navigation->AddRoot(m_model->firma().nazev());
    m_navigation->SetItemData(root, new NavigationItemData(NAV_HOME));
    m_navigation->SetFocus();

    wxTreeItemId ucetnictvi = m_navigation->AppendItem(root, wxString::FromUTF8("Účetnictví"));
    m_navigation->SetItemData(ucetnictvi, new NavigationItemData(NAV_NULL));
    wxTreeItemId denik = m_navigation->AppendItem(ucetnictvi, wxString::FromUTF8("Pěněžní deník"));
    m_navigation->SetItemData(denik, new NavigationItemData(NAV_DENIK));
    wxTreeItemId pokladna = m_navigation->AppendItem(ucetnictvi, "Pokladna");
    m_navigation->SetItemData(pokladna, new NavigationItemData(NAV_POKLADNA));
    wxTreeItemId banka = m_navigation->AppendItem(ucetnictvi, "Banka");
    m_navigation->SetItemData(banka, new NavigationItemData(NAV_BANKA));

    wxTreeItemId fakturace = m_navigation->AppendItem(root, "Fakturace");
    m_navigation->SetItemData(fakturace, new NavigationItemData(NAV_NULL));
    wxTreeItemId vydane = m_navigation->AppendItem(fakturace, wxString::FromUTF8("Faktury vydané"));
    m_navigation->SetItemData(vydane, new NavigationItemData(NAV_VYDANA));
    wxTreeItemId prijate = m_navigation->AppendItem(fakturace, wxString::FromUTF8("Faktury přijaté"));
    m_navigation->SetItemData(prijate, new NavigationItemData(NAV_PRIJATA));

    wxTreeItemId zavpoh = m_navigation->AppendItem(root, wxString::FromUTF8("Pohledávky a závazky"));
    m_navigation->SetItemData(zavpoh, new NavigationItemData(NAV_NULL));
    wxTreeItemId pohledavky = m_navigation->AppendItem(zavpoh, wxString::FromUTF8("Pohledávky"));
    m_navigation->SetItemData(pohledavky, new NavigationItemData(NAV_POHLEDAVKA));
    wxTreeItemId zavazky = m_navigation->AppendItem(zavpoh, wxString::FromUTF8("Závazky"));
    m_navigation->SetItemData(zavazky, new NavigationItemData(NAV_ZAVAZEK));

    wxTreeItemId majetek = m_navigation->AppendItem(root, "Majetek");
    m_navigation->SetItemData(majetek, new NavigationItemData(NAV_NULL));
    wxTreeItemId kmajetku = m_navigation->AppendItem(majetek, "Kniha majetku");
    m_navigation->SetItemData(kmajetku, new NavigationItemData(NAV_MAJETEK));

    wxTreeItemId sklad = m_navigation->AppendItem(root, "Sklad");
    m_navigation->SetItemData(sklad, new NavigationItemData(NAV_NULL));
    wxTreeItemId kszasob = m_navigation->AppendItem(sklad, wxString::FromUTF8("Kniha zásob"));
    m_navigation->SetItemData(kszasob, new NavigationItemData(NAV_SKLAD));

    if(m_model->currentRok() > 2017)
    {
        wxTreeItemId jizdy = m_navigation->AppendItem(root, wxString::FromUTF8("Jízdy"));
        m_navigation->SetItemData(jizdy, new NavigationItemData(NAV_NULL));
        wxTreeItemId kjizd = m_navigation->AppendItem(jizdy, wxString::FromUTF8("Kniha jízd"));
        m_navigation->SetItemData(kjizd, new NavigationItemData(NAV_JIZDA));
    }

    m_navigation->Expand(root);
    m_navigation->SelectItem(root);
}

void dxdeFrame::OnQuit(wxCommandEvent& /*event*/)
{
    wxTimeSpan span = wxDateTime::Today().Subtract(dxsettings.lastBackup());
    if(span.GetDays() > 14)
    {
        ZalohaDialog dialog(this);
        dialog.ShowModal();
    }
    wxGetApp().DismissPreferencesEditor();
    Destroy();
}

void dxdeFrame::OnClose(wxCloseEvent &event)
{
    if(event.CanVeto())
    {
        wxTimeSpan span = wxDateTime::Today().Subtract(dxsettings.lastBackup());
        if(span.GetDays() > 14)
        {
            ZalohaDialog dialog(this);
            dialog.ShowModal();
        }
    }
    wxGetApp().DismissPreferencesEditor();
    Destroy();
}

void dxdeFrame::OnAbout(wxCommandEvent& /*event*/)
{
    wxString description = wxString() << wxVERSION_STRING
#ifdef __WXGTK__
        << " (gtk " << GTKV << ")\n"
#else
        << "\n"
#endif
        << m_model->getVersion() << "\n"
        << PDFDOC_VERSION_STRING
#ifdef HAVE_CURL
        << "\nLibCurl " << LIBCURL_VERSION << "\n"
#endif
        << wxString::FromUTF8("\nProgram používá famfamfam icons\n(http://www.famfamfam.com/lab/icons/)");
    wxAboutDialogInfo info;
    info.SetIcon(wxICON(dxde64));
    info.SetName(PACKAGE);
    info.SetVersion(VERSION);
    info.SetDescription(description);
    info.SetCopyright("(c) 2014 ~");
    info.SetWebSite("https://dxde.dxsolutions.org");
    info.AddDeveloper("David Vachulka <archdvx@dxsolutions.org>");
    info.SetLicence(wxString::FromUTF8("Program je svobodný software;"
                                       "můžete ho šířit a/nebo upravit podle ujednání\n"
                                       "GNU General Public License publikovanou Free Software Foundation;\n"
                                       "verze 3 licence nebo (podle svého uvážení) pozdější verzi.\n"
                                       "\n"
                                       "Program je šířen ve víře v jeho použitelnost, ale BEZ JAKÉKOLIV ZÁRUKY;\n"
                                       "odvozené záruky PRODEJNOSTI nebo VHODNOSTI PRO URČITÝ ÚČEL.\n"
                                       "Další detaily v GNU General Public License.\n"
                                       "\n"
                                       "Měl byste získat kopii GNU General Public License s tímto programem;\n"
                                       "jestliže ne, napište Free Software Foundation, Inc., 51 Franklin Street,\n"
                                       "Fifth Floor, Boston, MA  02110-1301, USA"));
    wxAboutBox(info);
}

void dxdeFrame::OnEditFirma(wxCommandEvent& /*event*/)
{
    FirmaDialog dialog(this, true);
    dialog.ShowModal();
    m_dphDruh->Enable(m_model->isFirmaPlatce()?true:false);
    m_dphPriznani->Enable(m_model->isFirmaPlatce()?true:false);
    m_dphKH->Enable(m_model->isFirmaPlatce()?true:false);
    m_dphSH->Enable(m_model->isFirmaPlatce()?true:false);
    m_dphSeznam->Enable(m_model->isFirmaPlatce()?true:false);
    m_ulozene->Enable(m_model->isFirmaPlatce()?true:false);
}

void dxdeFrame::OnAddFirma(wxCommandEvent& event)
{
    FirmaWizard wizard(this, false);
    wizard.RunWizard(wizard.GetFirstPage());
    if(wizard.success())
    {
        wxCommandEvent dummy;
        OnModelUpdated(dummy);
        m_changeFirma->Enable(dxsettings.dbcount()>1?true:false);
        m_deleteObdobi->Enable(m_model->getObdobiCount()>1?true:false);
        m_dphDruh->Enable(m_model->isFirmaPlatce()?true:false);
        m_dphPriznani->Enable(m_model->isFirmaPlatce()?true:false);
        m_dphKH->Enable(m_model->isFirmaPlatce()?true:false);
        m_dphSH->Enable(m_model->isFirmaPlatce()?true:false);
        m_dphSeznam->Enable(m_model->isFirmaPlatce()?true:false);
        m_ulozene->Enable(m_model->isFirmaPlatce()?true:false);
        createNavigation();
        createRemoveJizdy();
    }
    else
    {
        if(event.GetInt() == 0)
        {
            dxsettings.setDbname();
            wxMessageBox(wxString::FromUTF8("Program bude ukončen"));
            wxCommandEvent dummy;
            OnQuit(dummy);
        }
    }
}

void dxdeFrame::OnDeleteFirma(wxCommandEvent& /*event*/)
{
    wxSingleChoiceDialog dialog(this, wxString::FromUTF8("Vyberte firmu pro smazání:"), "Firma", dxsettings.firmy());
    if(dialog.ShowModal() == wxID_OK)
    {
        wxString nazev = dialog.GetStringSelection().BeforeLast(',');
        wxString ic = dialog.GetStringSelection().AfterLast(' ');
        wxTextEntryDialog dotaz(this, wxString::Format(wxString::FromUTF8("Pro smazání firmy\n%s, IČ: %s\nZadejte ANO"), nazev, ic),
                                "Smazat firmu");
        if(dotaz.ShowModal() == wxID_OK && dotaz.GetValue() == "ANO")
        {
            m_model->closeDatabase();
            if(dxsettings.dbmodel() == DB_SQLITE)
            {
                if(!wxRemoveFile(wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+dxsettings.dbnameByFirma(nazev+"_"+ic)))
                {
                    wxLogDebug(wxString::Format("Nepodarilo se smazat soubor: %s", wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+dxsettings.dbnameByFirma(nazev+"_"+ic)));
                }
            }
            dxsettings.deleteFirma(nazev+"_"+ic);
            wxCommandEvent dummy;
            dummy.SetInt(0);
            if(dxsettings.dbcount())
            {
                OnChangeFirma(dummy);
            }
            else
            {
                OnAddFirma(dummy);
            }
            m_changeFirma->Enable(dxsettings.dbcount()>1?true:false);
        }
    }
}

void dxdeFrame::OnChangeFirma(wxCommandEvent& event)
{
    wxSingleChoiceDialog dialog(this, "Vyberte firmu:", "Firma", dxsettings.firmy());
    if(dialog.ShowModal() == wxID_OK)
    {
        wxString nazev = dialog.GetStringSelection().BeforeLast(',');
        wxString ic = dialog.GetStringSelection().AfterLast(' ');
        if(nazev == m_model->firma().nazev() && ic == m_model->firma().ic())
        {
            return;
        }
        dxsettings.setDbname(dxsettings.dbnameByFirma(nazev+"_"+ic));
        m_model->closeDatabase();
        splash = new wxSplashScreen(wxBitmap(splash_xpm), wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_NO_TIMEOUT, 0, NULL, wxID_ANY);
        wxYield();
        if(wxFileExists(wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+dxsettings.dbname()))
        {
            if(m_model->openDatabase())
            {
                m_model->fillData();
            }
            else
            {
                wxLogError(wxString::FromUTF8("Nepodařilo se otevřít databázi"));
            }
        }
        else
        {
            if(dxsettings.dbname().IsEmpty()) dxsettings.setDbname(dxutils::getUniqueDbName());
            m_model->newDatabase();
            m_model->addAdresa(Adresa(1, "", "", "", "", ""));
            m_model->setFirma(Firma(nazev, ic, "", "", false, false, "", "", "", "", "", 1));
            m_model->addObdobi(Obdobi(wxDateTime::Today().GetYear(), true));
            m_book->UpdatePages(ID_MODEL_ALL);
        }
        createNavigation();
        createRemoveJizdy();
        if(splash)
        {
            splash->Close(true);
            wxDELETE(splash);
        }
        dxsettings.editDbnamePair(dxsettings.dbname(), m_model->firma().nazev()+"_"+m_model->firma().ic());
        m_changeFirma->Enable(dxsettings.dbcount()>1?true:false);
        m_deleteObdobi->Enable(m_model->getObdobiCount()>1?true:false);
        m_dphDruh->Enable(m_model->isFirmaPlatce()?true:false);
        m_dphPriznani->Enable(m_model->isFirmaPlatce()?true:false);
        m_dphKH->Enable(m_model->isFirmaPlatce()?true:false);
        m_dphSH->Enable(m_model->isFirmaPlatce()?true:false);
        m_dphSeznam->Enable(m_model->isFirmaPlatce()?true:false);
        m_ulozene->Enable(m_model->isFirmaPlatce()?true:false);
        dxsettings.setDatum(dxutils::todayForObdobi());
        updateStatusText();
    }
    else
    {
        if(event.GetInt() == 0)
        {
            dxsettings.setDbname();
            wxMessageBox(wxString::FromUTF8("Program bude ukončen"));
            wxCommandEvent dummy;
            OnQuit(dummy);
        }
    }
}

void dxdeFrame::OnAdresar(wxCommandEvent& /*event*/)
{
    KontaktyDialog dialog(this);
    dialog.ShowModal();
}

void dxdeFrame::OnZaloha(wxCommandEvent &/*event*/)
{
    wxDirDialog dialog(this, wxString::FromUTF8("Adresář pro zálohy"), dxsettings.backupPath(), wxDD_DEFAULT_STYLE & ~wxDD_DIR_MUST_EXIST);
    if(dialog.ShowModal() == wxID_OK)
    {
        wxString path = dialog.GetPath();
        if(dxutils::zalohaDat(path))
        {
            dxsettings.setLastBackup(wxDateTime::Today());
            dxsettings.setBackupPath(path);
            wxMessageBox(wxString::FromUTF8("Vytvoření zálohy proběhlo úspěšně"));
        }
        else
        {
            wxMessageBox(wxString::FromUTF8("Vytvoření zálohy se nezdařilo"));
        }
    }
}

void dxdeFrame::OnObnova(wxCommandEvent &/*event*/)
{
    wxString firma = m_model->firma().nazev();
    wxString ic = m_model->firma().ic();
    wxTextEntryDialog dotaz(this, wxString::Format(wxString::FromUTF8("Pro obnovení dat firmy\n%s, IČ: %s\nze zálohy.\nZadejte ANO\n(Přepíše aktuální data)"), firma, ic),
                            wxString::FromUTF8("Obnovení dat"));
    if(dotaz.ShowModal() == wxID_OK && dotaz.GetValue() == "ANO")
    {
        wxFileDialog dialog(m_parent, wxString::FromUTF8("Otevřít soubor databáze"), dxsettings.backupPath(), wxEmptyString, "Soubor databáze (*.db)|*.db", wxFD_OPEN|wxFD_FILE_MUST_EXIST);
        if(dialog.ShowModal() == wxID_OK)
        {
            wxString path = dialog.GetPath();
            if(!path.empty())
            {
                wxSQLite3Database db;
                int err = SQLITE_OK;
                try
                {
                    db.Open(path);
                }
                catch(const wxSQLite3Exception &e)
                {
                    err = e.GetErrorCode();
                }
                if(err == SQLITE_OK)
                {
                    if(!db.TableExists("firma"))
                    {
                        wxMessageBox(wxString::FromUTF8("Data se nepodařilo obnovit"));
                        return;
                    }
                    wxSQLite3ResultSet rs = db.ExecuteQuery("SELECT nazev,ic FROM firma");
                    if(firma != rs.GetString(0) || ic != rs.GetString(1))
                    {
                        wxMessageBox(wxString::FromUTF8("Data se nepodařilo obnovit.\nNesouhlasí název firmy a IČ"));
                        return;
                    }
                    m_model->closeDatabase();
                    wxCopyFile(path, wxStandardPaths::Get().GetUserDataDir()+wxFILE_SEP_PATH+dxsettings.dbname());
                    splash = new wxSplashScreen(wxBitmap(splash_xpm), wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_NO_TIMEOUT, 0, NULL, wxID_ANY);
                    wxYield();
                    if(m_model->openDatabase())
                    {
                        m_model->fillData();
                    }
                    else
                    {
                        wxLogError(wxString::FromUTF8("Nepodařilo se otevřít databázi"));
                    }
                    if(splash)
                    {
                        splash->Close(true);
                        wxDELETE(splash);
                    }
                    dxsettings.editDbnamePair(dxsettings.dbname(), m_model->firma().nazev()+"_"+m_model->firma().ic());
                    m_changeFirma->Enable(dxsettings.dbcount()>1?true:false);
                    m_deleteObdobi->Enable(m_model->getObdobiCount()>1?true:false);
                    updateStatusText();
                    wxMessageBox(wxString::FromUTF8("Data byla obnovena"));
                }
            }
            else
            {
                wxMessageBox(wxString::FromUTF8("Data se nepodařilo obnovit"));
            }
        }
    }
}

void dxdeFrame::OnHelp(wxCommandEvent& /*event*/)
{
    m_help.Display(wxString::FromUTF8("Nápověda aplikace dxde"));
}

void dxdeFrame::updateStatusText()
{
#if wxUSE_STATUSBAR
    SetStatusText(wxString::Format(wxString::FromUTF8("%s, IČ: %s"), m_model->firma().nazev(), m_model->firma().ic()), 0);
    SetStatusText(wxString::Format("%d", m_model->currentRok()), 1);
    SetStatusText(wxString::Format("Datum: %s", dxsettings.datum().Format("%x")), 2);
#endif // wxUSE_STATUSBAR
}

void dxdeFrame::OnAddObdobi(wxCommandEvent& /*event*/)
{
    long ret = wxGetNumberFromUser(wxEmptyString, wxString::FromUTF8("Nové období:"), wxString::FromUTF8("Zadejte období"),
                                   m_model->currentRok()-1, OBDOBI_MIN,
                                   wxDateTime::GetCurrentYear()+1, this);
    if(ret == -1)
        return;
    m_model->setCurrentRok((int)ret);
    m_model->setDefaultRadaCislo();
    m_deleteObdobi->Enable(m_model->getObdobiCount()>1?true:false);
    createNavigation();
    createRemoveJizdy();
}

void dxdeFrame::OnDeleteObdobi(wxCommandEvent& event)
{
    wxSingleChoiceDialog dialog(this, wxString::FromUTF8("Období:"), wxString::FromUTF8("Smazat období"),
                                m_model->getObdobiAsStrings());
    if(dialog.ShowModal() == wxID_OK)
    {
        long rok;
        dialog.GetStringSelection().ToLong(&rok);
        m_model->deleteObdobi(rok);
        OnChangeObdobi(event);
    }
}

void dxdeFrame::OnChangeObdobi(wxCommandEvent& /*event*/)
{
    wxSingleChoiceDialog dialog(this, wxString::FromUTF8("Období:"), wxString::FromUTF8("Vyberte období"),
                                m_model->getObdobiAsStrings());
    if(dialog.ShowModal() == wxID_OK)
    {
        if(!splash)
        {
            splash = new wxSplashScreen(wxBitmap(splash_xpm), wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_NO_TIMEOUT, 0, NULL, wxID_ANY);
            wxYield();
        }
        int val = wxAtoi(dialog.GetStringSelection());
        m_model->setCurrentRok(val);
        dxsettings.setSplatnost(m_model->getSplatnost(val));
        m_model->setDefaultRadaCislo();
        createNavigation();
        createRemoveJizdy();
        if(splash)
        {
            splash->Close(true);
            wxDELETE(splash);
        }
    }
}

void dxdeFrame::OnNavigationChanged(wxTreeEvent &event)
{
    wxTreeItemId id = event.GetItem();
    if(!id) return;
    NavigationItemData *nav = dynamic_cast<NavigationItemData*>(m_navigation->GetItemData(id));
    if(!nav) return;
    switch(nav->navigation())
    {
        case NAV_HOME:
        {
            m_book->SelectPage(NAV_HOME);
            return;
        }
        case NAV_DENIK:
        {
            if(!m_book->HasPage(NAV_DENIK))
            {
                DenikPage *denik = new DenikPage(m_book);
                m_book->AppendPage(denik);
                m_book->UpdatePages(ID_MODEL_POKLADNA);
            }
            else
            {
                m_book->SelectPage(NAV_DENIK);
            }
            return;
        }
        case NAV_POKLADNA:
        {
            if(!m_book->HasPage(NAV_POKLADNA))
            {
                PokladnaPage *pokladna = new PokladnaPage(m_book);
                m_book->AppendPage(pokladna);
                m_book->UpdatePages(ID_MODEL_POKLADNA);
            }
            else
            {
                m_book->SelectPage(NAV_POKLADNA);
            }
            return;
        }
        case NAV_BANKA:
        {
            if(!m_book->HasPage(NAV_BANKA))
            {
                BankaPage *banka = new BankaPage(m_book);
                m_book->AppendPage(banka);
                m_book->UpdatePages(ID_MODEL_BANKA);
            }
            else
            {
                m_book->SelectPage(NAV_BANKA);
            }
            return;
        }
        case NAV_VYDANA:
        {
            if(!m_book->HasPage(NAV_VYDANA))
            {
                VydanaPage *vydana = new VydanaPage(m_book);
                m_book->AppendPage(vydana);
                m_book->UpdatePages(ID_MODEL_VYDANA);
            }
            else
            {
                m_book->SelectPage(NAV_VYDANA);
            }
            return;
        }
        case NAV_PRIJATA:
        {
            if(!m_book->HasPage(NAV_PRIJATA))
            {
                PrijataPage *prijata = new PrijataPage(m_book);
                m_book->AppendPage(prijata);
                m_book->UpdatePages(ID_MODEL_PRIJATA);
            }
            else
            {
                m_book->SelectPage(NAV_PRIJATA);
            }
            return;
        }
        case NAV_POHLEDAVKA:
        {
            if(!m_book->HasPage(NAV_POHLEDAVKA))
            {
                PohledavkaPage *pohledavka = new PohledavkaPage(m_book);
                m_book->AppendPage(pohledavka);
                m_book->UpdatePages(ID_MODEL_POHLEDAVKA);
            }
            else
            {
                m_book->SelectPage(NAV_POHLEDAVKA);
            }
            return;
        }
        case NAV_ZAVAZEK:
        {
            if(!m_book->HasPage(NAV_ZAVAZEK))
            {
                ZavazekPage *zavazek = new ZavazekPage(m_book);
                m_book->AppendPage(zavazek);
                m_book->UpdatePages(ID_MODEL_ZAVAZEK);
            }
            else
            {
                m_book->SelectPage(NAV_ZAVAZEK);
            }
            return;
        }
        case NAV_MAJETEK:
        {
            if(!m_book->HasPage(NAV_MAJETEK))
            {
                MajetekPage *majetek = new MajetekPage(m_book);
                m_book->AppendPage(majetek);
                m_book->UpdatePages(ID_MODEL_MAJETEK);
            }
            else
            {
                m_book->SelectPage(NAV_MAJETEK);
            }
            return;
        }
        case NAV_SKLAD:
        {
            if(!m_book->HasPage(NAV_SKLAD))
            {
                SkladPage *sklad = new SkladPage(m_book);
                m_book->AppendPage(sklad);
                m_book->UpdatePages(ID_MODEL_ZASOBA);
            }
            else
            {
                m_book->SelectPage(NAV_SKLAD);
            }
            return;
        }
        case NAV_JIZDA:
        {
            if(!m_book->HasPage(NAV_JIZDA))
            {
                JizdaPage *jizda = new JizdaPage(m_book);
                m_book->AppendPage(jizda);
                m_book->UpdatePages(ID_MODEL_JIZDA);
            }
            else
            {
                m_book->SelectPage(NAV_JIZDA);
            }
            return;
        }
        default:
        {
            m_book->SelectPage(NAV_HOME);
            return;
        }
    }
}

void dxdeFrame::OnModelUpdated(wxCommandEvent &event)
{
    if(m_navigation->GetCount()) m_navigation->SetItemText(m_navigation->GetRootItem(), m_model->firma().nazev());
    m_book->UpdatePages(event.GetId());
    if(dxsettings.datum().GetYear() != dxutils::todayForObdobi().GetYear())
    {
        dxsettings.setDatum(dxutils::todayForObdobi());
    }
    updateStatusText();
}

void dxdeFrame::OnModelMsg(wxCommandEvent &event)
{
    if(splash)
    {
        splash->Close(true);
        wxDELETE(splash);
    }
    wxMessageDialog dialog(this, event.GetString(), "Informace", wxCENTER|wxOK_DEFAULT|wxOK|wxICON_INFORMATION);
    dialog.ShowModal();
}

void dxdeFrame::OnDenik(wxCommandEvent& /*event*/)
{
    selectNavigation(NAV_DENIK);
}

void dxdeFrame::OnPokladna(wxCommandEvent& /*event*/)
{
    selectNavigation(NAV_POKLADNA);
}

void dxdeFrame::OnBanka(wxCommandEvent& /*event*/)
{
    selectNavigation(NAV_BANKA);
}

void dxdeFrame::OnPredkontace(wxCommandEvent& /*event*/)
{
    PredkontacesDialog dialog(this);
    dialog.ShowModal();
}

void dxdeFrame::OnStavPokladenBank(wxCommandEvent& /*event*/)
{
    wxString msg = "Pokladny:\n";
    dxPokladnaArray pokladny = m_model->pokladny();
    for(size_t i=0; i<pokladny.GetCount(); i++)
    {
        msg.Append(pokladny[i].popis());
        msg.Append(wxString::Format(" (%s): ", pokladny[i].zkratka()));
        msg.Append(wxString::Format("%.2f %s", m_model->getPokladnaStav(pokladny[i].id()), m_model->zkratkaMena(pokladny[i].menaid())));
        msg.Append('\n');
    }
    msg.Append("Banky:\n");
    dxBankaArray banky = m_model->banky();
    for(size_t i=0; i<banky.GetCount(); i++)
    {
        msg.Append(banky[i].popis());
        msg.Append(wxString::Format(" (%s): ", banky[i].zkratka()));
        msg.Append(wxString::Format("%.2f %s", m_model->getBankaStav(banky[i].id()), m_model->zkratkaMena(banky[i].menaid())));
        msg.Append('\n');
    }
    wxMessageDialog dialog(this, msg, wxString::FromUTF8("Stav účtů a pokladen"), wxOK|wxICON_INFORMATION);
    dialog.ShowModal();
}

void dxdeFrame::OnPreferences(wxCommandEvent& /*event*/)
{
    wxGetApp().ShowPreferencesEditor(this);
}

void dxdeFrame::OnVystavena(wxCommandEvent &/*event*/)
{
    selectNavigation(NAV_VYDANA);
}

void dxdeFrame::OnPrijata(wxCommandEvent &/*event*/)
{
    selectNavigation(NAV_PRIJATA);
}

void dxdeFrame::OnPohledavka(wxCommandEvent &/*event*/)
{
    selectNavigation(NAV_POHLEDAVKA);
}

void dxdeFrame::OnZavazek(wxCommandEvent &/*event*/)
{
    selectNavigation(NAV_ZAVAZEK);
}

void dxdeFrame::OnMajetek(wxCommandEvent &/*event*/)
{
    selectNavigation(NAV_MAJETEK);
}

void dxdeFrame::OnSklad(wxCommandEvent &/*event*/)
{
    selectNavigation(NAV_SKLAD);
}

void dxdeFrame::OnJizdy(wxCommandEvent &/*event*/)
{
    selectNavigation(NAV_JIZDA);
}

void dxdeFrame::OnInventury(wxCommandEvent &/*event*/)
{
    if(!m_model->getSkladyCount())
    {
        SkladDialog dialog(this, -1, false);
        if(dialog.ShowModal() != wxID_OK)
        {
            return;
        }
    }
    InventuryDialog dialog(this);
    dialog.ShowModal();
}

void dxdeFrame::OnFauhrady(wxCommandEvent &/*event*/)
{
    FauhradyDialog dialog(this);
    dialog.ShowModal();
}

void dxdeFrame::OnDatum(wxCommandEvent &/*event*/)
{
    DatumDialog dialog(this);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_book->UpdatePages(ID_MODEL_ALL);
        updateStatusText();
    }
}

void dxdeFrame::OnOdeslane(wxCommandEvent &/*event*/)
{
    OdeslaneDialog dialog(this);
    dialog.ShowModal();
}

void dxdeFrame::OnTemplateEditor(wxCommandEvent &/*event*/)
{
    TemplateEditor dialog(this);
    dialog.ShowModal();
}

void dxdeFrame::OnDphDruh(wxCommandEvent &/*event*/)
{
    DphDruhyDialog dialog(this);
    dialog.ShowModal();
}

void dxdeFrame::OnDphPriznani(wxCommandEvent &/*event*/)
{
    if(!dxutils::haveCiselniky())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Je třeba stáhnout číselníky potřebné pro přiznání.\nStáhnout teď?"), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            wxBeginBusyCursor();
#if __WXGTK__
            wxYield();
#endif
            if(!dxutils::downloadCiselniky())
            {
                wxEndBusyCursor();
                return;
            }
            wxEndBusyCursor();
        }
        else
        {
            return;
        }
    }
    if(!m_model->priznani().isValid())
    {
        PriznaniDialog dialog(this, false);
        if(dialog.ShowModal() == wxID_CANCEL)
        {
            return;
        }
    }
    Dphdp3Wizard wizard(this);
    wizard.RunWizard(wizard.GetFirstPage());
}

void dxdeFrame::OnDphKH(wxCommandEvent &/*event*/)
{
    if(!m_model->hasObdobiGreaterThan(2015))
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Kontrolní hlášení se podává až od roku 2016."), "Informace", wxOK|wxICON_INFORMATION);
        dialog.ShowModal();
        return;
    }
    if(!dxutils::haveCiselniky())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Je třeba stáhnout číselníky potřebné pro hlášení.\nStáhnout teď?"), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            wxBeginBusyCursor();
#if __WXGTK__
            wxYield();
#endif
            if(!dxutils::downloadCiselniky())
            {
                wxEndBusyCursor();
                return;
            }
            wxEndBusyCursor();
        }
        else
        {
            return;
        }
    }
    if(!m_model->priznani().isValid())
    {
        PriznaniDialog dialog(this, false);
        if(dialog.ShowModal() == wxID_CANCEL)
        {
            return;
        }
    }
    Dphkh1Wizard wizard(this);
    wizard.RunWizard(wizard.GetFirstPage());
}

void dxdeFrame::OnDphSH(wxCommandEvent &/*event*/)
{
    if(!dxutils::haveCiselniky())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Je třeba stáhnout číselníky potřebné pro hlášení.\nStáhnout teď?"), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            wxBeginBusyCursor();
#if __WXGTK__
            wxYield();
#endif
            if(!dxutils::downloadCiselniky())
            {
                wxEndBusyCursor();
                return;
            }
            wxEndBusyCursor();
        }
        else
        {
            return;
        }
    }
    if(!m_model->priznani().isValid())
    {
        PriznaniDialog dialog(this, false);
        if(dialog.ShowModal() == wxID_CANCEL)
        {
            return;
        }
    }
    DphshWizard wizard(this);
    wizard.RunWizard(wizard.GetFirstPage());
}

void dxdeFrame::OnDphSeznam(wxCommandEvent &/*event*/)
{
    SeznamDphDialog dialog(this);
    dialog.ShowModal();
}

void dxdeFrame::OnDpPrijmyVydaje(wxCommandEvent &/*event*/)
{
    PrijmyVydajeDialog dialog(this);
    dialog.ShowModal();
}

void dxdeFrame::OnDpMajetekZavazky(wxCommandEvent &/*event*/)
{
    MajetekZavazkyDialog dialog(this);
    dialog.ShowModal();
}

void dxdeFrame::OnUlozene(wxCommandEvent &/*event*/)
{
    if(!m_model->getUlozeneCount())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Žádná uložená přiznání"), "Informace", wxOK|wxICON_INFORMATION);
        dialog.ShowModal();
        return;
    }
    UlozeneDialog dialog(this);
    dialog.ShowModal();
}

void dxdeFrame::OnCheckVersion(wxCommandEvent &/*event*/)
{
    wxString availableVersion = dxutils::availableVersion();
    int availableVersionInt = dxutils::versionToInt(availableVersion);
    if(availableVersionInt == 0)
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Aktualizace nebylo možné ověřit.\nZkuste později."), "Informace", wxOK|wxICON_INFORMATION);
        dialog.ShowModal();
        return;
    }
    if(dxutils::versionToInt(VERSION) < availableVersionInt)
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Je dostupná novější verze %s."), availableVersion), "Informace", wxOK|wxICON_INFORMATION);
        dialog.ShowModal();
        return;
    }
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Máte aktuální verzi %s."), availableVersion), "Informace", wxOK|wxICON_INFORMATION);
    dialog.ShowModal();
}

void dxdeFrame::selectNavigation(NAVIGATION nav)
{
    if(nav == NAV_HOME)
    {
        m_navigation->UnselectAll();
        m_navigation->SelectItem(m_navigation->GetRootItem());
        return;
    }
    std::stack<wxTreeItemId> items;
    if(m_navigation->GetRootItem().IsOk())
        items.push(m_navigation->GetRootItem());
    while(!items.empty())
    {
        wxTreeItemId next = items.top();
        items.pop();
        if(next != m_navigation->GetRootItem()
                && dynamic_cast<NavigationItemData*>(m_navigation->GetItemData(next))
                && (static_cast<NavigationItemData*>(m_navigation->GetItemData(next)))->navigation() == nav)
        {
            m_navigation->UnselectAll();
            m_navigation->SelectItem(next);
            m_navigation->EnsureVisible(next);
            return;
        }
        wxTreeItemIdValue cookie;
        wxTreeItemId nextChild = m_navigation->GetFirstChild(next, cookie);
        while(nextChild.IsOk())
        {
            items.push(nextChild);
            nextChild = m_navigation->GetNextSibling(nextChild);
        }
    }
}

void dxdeFrame::createRemoveJizdy()
{
    if(m_model->currentRok()>2017 && GetMenuBar()->GetMenuCount()<8)
    {
        wxMenu *jizMenu = new wxMenu();
        jizMenu->Append(ID_MENU_JIZDY, wxString::FromUTF8("Kniha jízd"), wxString::FromUTF8("Kniha jízd"));
        GetMenuBar()->Insert(5, jizMenu, wxString::FromUTF8("Jízdy"));
        return;
    }
    if(m_model->currentRok()<2018 && GetMenuBar()->GetMenuCount()>7)
    {
        GetMenuBar()->Remove(5);
    }
}


