/***************************************************************
 * Name:      dxde.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef DXDE_H
#define DXDE_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/aui/aui.h>
#include <wx/snglinst.h>
#include <wx/treectrl.h>
#include <wx/preferences.h>
#include <wx/html/helpfrm.h>
#include <wx/html/helpctrl.h>
#include "model/dbmodel.h"

class dxBook;
class dxdeFrame;

class dxdeApp : public wxApp
{
public:
    virtual bool OnInit();
    virtual int OnExit();
    void ShowPreferencesEditor(wxWindow* parent);
    void DismissPreferencesEditor();
protected:
    wxLocale *m_locale;
    wxSingleInstanceChecker *m_checker;
    wxScopedPtr<wxPreferencesEditor> m_preferences;
private:
    dxdeFrame *m_frame;
};

class NavigationItemData : public wxTreeItemData
{
public:
    explicit NavigationItemData(NAVIGATION nav) : m_navigation(nav) {}
    ~NavigationItemData() {}

    NAVIGATION navigation() const { return m_navigation; }
private:
    NAVIGATION m_navigation;
};

class dxdeFrame : public wxFrame
{
public:
    dxdeFrame(wxFrame *frame, const wxString& title);
    ~dxdeFrame();

private:
    wxAuiManager m_manager;
    wxTreeCtrl *m_navigation;
    dxBook *m_book;
    DbModel *m_model;
    wxMenuItem *m_deleteObdobi;
    wxMenuItem *m_changeFirma;
    wxMenuItem *m_dphDruh;
    wxMenuItem *m_dphPriznani;
    wxMenuItem *m_dphKH;
    wxMenuItem *m_dphSH;
    wxMenuItem *m_dphSeznam;
    wxMenuItem *m_ulozene;
    wxHtmlHelpController m_help;

    bool loadDatabase();
    void createNavigation();
    void OnAbout(wxCommandEvent& event);
    void OnQuit(wxCommandEvent& event);
    void OnClose(wxCloseEvent& event);
    void OnEditFirma(wxCommandEvent& event);
    void OnAddFirma(wxCommandEvent& event);
    void OnDeleteFirma(wxCommandEvent& event);
    void OnChangeFirma(wxCommandEvent& event);
    void OnAdresar(wxCommandEvent& event);
    void OnZaloha(wxCommandEvent& event);
    void OnObnova(wxCommandEvent& event);
    void OnHelp(wxCommandEvent& event);
    void updateStatusText();
    void OnAddObdobi(wxCommandEvent& event);
    void OnDeleteObdobi(wxCommandEvent& event);
    void OnChangeObdobi(wxCommandEvent& event);
    void OnNavigationChanged(wxTreeEvent& event);
    void OnModelUpdated(wxCommandEvent& event);
    void OnModelMsg(wxCommandEvent& event);
    void OnDenik(wxCommandEvent& event);
    void OnPokladna(wxCommandEvent& event);
    void OnBanka(wxCommandEvent& event);
    void OnPredkontace(wxCommandEvent& event);
    void OnStavPokladenBank(wxCommandEvent& event);
    void OnPreferences(wxCommandEvent& event);
    void OnVystavena(wxCommandEvent& event);
    void OnPrijata(wxCommandEvent& event);
    void OnPohledavka(wxCommandEvent& event);
    void OnZavazek(wxCommandEvent& event);
    void OnMajetek(wxCommandEvent& event);
    void OnSklad(wxCommandEvent& event);
    void OnJizdy(wxCommandEvent& event);
    void OnInventury(wxCommandEvent& event);
    void OnFauhrady(wxCommandEvent& event);
    void OnDatum(wxCommandEvent& event);
    void OnOdeslane(wxCommandEvent& event);
    void OnTemplateEditor(wxCommandEvent& event);
    void OnDphDruh(wxCommandEvent& event);
    void OnDphPriznani(wxCommandEvent& event);
    void OnDphKH(wxCommandEvent& event);
    void OnDphSH(wxCommandEvent& event);
    void OnDphSeznam(wxCommandEvent& event);
    void OnDpPrijmyVydaje(wxCommandEvent& event);
    void OnDpMajetekZavazky(wxCommandEvent& event);
    void OnUlozene(wxCommandEvent& event);
    void OnCheckVersion(wxCommandEvent& event);
    void selectNavigation(NAVIGATION nav);
    void createRemoveJizdy();
DECLARE_EVENT_TABLE()
};


#endif // DXDE_H
