/***************************************************************
 * Name:      bankadialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "bankadialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/cog.xpm"

IMPLEMENT_DYNAMIC_CLASS(BankaDialog, wxDialog)

BEGIN_EVENT_TABLE(BankaDialog, wxDialog)
    EVT_BUTTON(wxID_OK, BankaDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, BankaDialog::OnCancel)
    EVT_BUTTON(ID_BANKA_KOD, BankaDialog::OnKodSelect)
    EVT_BUTTON(ID_BANKA_IBAN, BankaDialog::OnIbanCalc)
END_EVENT_TABLE()

BankaDialog::BankaDialog(wxWindow *parent, int id, bool edit)
    : wxDialog(parent, wxID_ANY, edit ? wxString::FromUTF8("Bankovní účet") : wxString::FromUTF8("Zadejte bankovní účet")), m_edit(edit)
{
    createControls();
    if(m_edit)
    {
        m_id = id;
    }
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void BankaDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Zkratka"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zkratka = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_zkratka, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popis = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_popis, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Číslo účtu")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cislo = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_cislo, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Kód banky")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kod = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_kod, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);
    m_kodSelect = new wxButton(this, ID_BANKA_KOD, "...", wxDefaultPosition, wxSize(30,-1));
    m_kodSelect->SetToolTip(wxString::FromUTF8("Vyberte kód banky"));
    gridSizer->Add(m_kodSelect, wxGBPosition(3,2), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "IBAN"), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_iban = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_iban, wxGBPosition(4,1), wxDefaultSpan, wxEXPAND);
    m_ibanCalc = new wxBitmapButton(this, ID_BANKA_IBAN, wxBitmap(cog_xpm));
    m_ibanCalc->SetBitmapDisabled(dxutils::GreyScaledIcon(cog_xpm));
    m_ibanCalc->SetToolTip(wxString::FromUTF8("Spočítat IBAN"));
    gridSizer->Add(m_ibanCalc, wxGBPosition(4,2), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "BIC"), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_bic = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_bic, wxGBPosition(5,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Měna")), wxGBPosition(6,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mena = new wxChoice(this, wxID_ANY);
    if(m_edit) m_mena->Disable();
    gridSizer->Add(m_mena, wxGBPosition(6,1), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void BankaDialog::fillControls()
{
    if(m_edit)
    {
        Banka bk = idbmodel->getBanka(m_id);
        if(bk.isValid())
        {
            m_zkratka->SetValue(bk.zkratka());
            m_popis->SetValue(bk.popis());
            m_cislo->SetValue(bk.cislouctu());
            m_kod->SetValue(bk.kodbanky());
            m_iban->SetValue(bk.iban());
            m_bic->SetValue(bk.bic());
            dxMenaArray meny = idbmodel->meny();
            for(size_t i=0; i<meny.GetCount(); i++)
            {
                m_mena->Append(idbmodel->nazevMena(meny[i].id()));
                if(bk.menaid() == meny[i].id())
                    m_mena->SetSelection(static_cast<int>(i));
            }
        }
    }
    else
    {
        dxMenaArray meny = idbmodel->meny();
        for(size_t i=0; i<meny.GetCount(); i++)
            m_mena->Append(idbmodel->nazevMena(meny[i].id()));
        m_mena->SetSelection(0);
    }
}

void BankaDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_zkratka->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Zkratka nemůže být prázdná"), "Informace", m_edit? wxOK|wxICON_INFORMATION : wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    if(zkratkaExists())
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Zkratka '%s' už existuje\nZvolte jinou"), m_zkratka->GetValue()), "Informace", wxOK|wxICON_INFORMATION);
        dialog.ShowModal();
        return;
    }
    if(!m_edit)
    {
        idbmodel->addBanka(Banka(-1, m_zkratka->GetValue(), m_popis->GetValue(), m_cislo->GetValue(), m_kod->GetValue(), m_iban->GetValue(), m_bic->GetValue(),
                                 idbmodel->itemMena(static_cast<size_t>(m_mena->GetSelection())).id()));
    }
    else
    {
        idbmodel->editBanka(Banka(m_id, m_zkratka->GetValue(), m_popis->GetValue(), m_cislo->GetValue(), m_kod->GetValue(), m_iban->GetValue(), m_bic->GetValue(),
                                  idbmodel->itemMena(static_cast<size_t>(m_mena->GetSelection())).id()));
    }
    EndModal(wxID_OK);
}

void BankaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

bool BankaDialog::zkratkaExists()
{
    if(m_edit)
    {
        if(idbmodel->getBankaZkratka(m_id) == m_zkratka->GetValue())
            return false;
        else
            return idbmodel->hasBanka(m_zkratka->GetValue());
    }
    else
    {
        return idbmodel->hasBanka(m_zkratka->GetValue());
    }
}

void BankaDialog::OnKodSelect(wxCommandEvent &/*event*/)
{
    wxSingleChoiceDialog vyber(this, "Vyberte banku", "Banky", idbmodel->kodyBankSNazvem(m_edit==0));
    vyber.SetSelection(0);
    if(vyber.ShowModal() == wxID_OK)
    {
        m_kod->SetValue(vyber.GetStringSelection().Left(4));
        m_bic->SetValue(idbmodel->bicByKod(vyber.GetStringSelection().Left(4)));
    }
}

void BankaDialog::OnIbanCalc(wxCommandEvent &event)
{
    if(m_kod->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Kód banky nemůže být prázdný"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnKodSelect(event);
        return;
    }
    if(m_cislo->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Číslo účtu nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_cislo->SetFocus();
        return;
    }
    m_iban->SetValue(dxutils::iban(m_kod->GetValue(), m_cislo->GetValue()));
}

