/***************************************************************
 * Name:      bankydialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "bankydialog.h"
#include "bankadialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"

IMPLEMENT_DYNAMIC_CLASS(BankyDialog, wxDialog)

BEGIN_EVENT_TABLE(BankyDialog, wxDialog)
    EVT_BUTTON(wxID_OK, BankyDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, BankyDialog::OnCancel)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_BANKY_LIST, BankyDialog::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_BANKY_LIST, BankyDialog::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_BANKY_LIST, BankyDialog::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_BANKY_LIST, BankyDialog::OnItemDeselected)
    EVT_MENU(ID_BANKY_ADD, BankyDialog::OnAddBanka)
    EVT_MENU(ID_BANKY_EDIT, BankyDialog::OnEditBanka)
    EVT_MENU(ID_BANKY_DELETE, BankyDialog::OnDeleteBanka)
END_EVENT_TABLE()

BankyDialog::BankyDialog(wxWindow *parent, bool select)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Bankovní účty")),
      m_select(select), m_selectedId(-1)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_BANKY_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat bankovní účet"));
    m_bar->AddTool(ID_BANKY_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit bankovní účet"));
    m_bar->AddTool(ID_BANKY_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat bankovní účet"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    m_list = new wxListCtrl(this, ID_BANKY_LIST, wxDefaultPosition, wxDefaultSize,
                            wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText("Zkratka");
    col0.SetWidth(80);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText("Popis");
    col1.SetWidth(350);
    m_list->InsertColumn(1, col1);
    wxListItem col2;
    col2.SetId(2);
    col2.SetText(wxString::FromUTF8("Zůstatek"));
    col2.SetAlign(wxLIST_FORMAT_RIGHT);
    col2.SetWidth(120);
    m_list->InsertColumn(2, col2);
    wxListItem col3;
    col3.SetId(3);
    col3.SetText(wxString::FromUTF8("Měna"));
    col3.SetWidth(100);
    m_list->InsertColumn(3, col3);
    fillList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, select? "&Vybrat" : "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void BankyDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void BankyDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void BankyDialog::OnItemRightClick(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    wxMenu popup;
    popup.Append(ID_BANKY_ADD, wxString::FromUTF8("Přidat bankovní účet"));
    if(m_selectedId != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_BANKY_EDIT, wxString::FromUTF8("Upravit bankovní účet"));
        popup.Append(ID_BANKY_DELETE, wxString::FromUTF8("Smazat bankovní účet"));
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void BankyDialog::OnItemActivated(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    if(m_select)
    {
        EndModal(wxID_OK);
    }
    else
    {
        BankaDialog dialog(this, m_selectedId, true);
        if(dialog.ShowModal() == wxID_OK)
            fillList();
    }
}

void BankyDialog::OnItemSelected(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void BankyDialog::OnItemDeselected(wxListEvent& /*event*/)
{
    setSelectedId(-1);
}

void BankyDialog::OnEditBanka(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    BankaDialog dialog(this, m_selectedId, true);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void BankyDialog::OnAddBanka(wxCommandEvent& /*event*/)
{
    BankaDialog dialog(this, -1, false);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void BankyDialog::OnDeleteBanka(wxCommandEvent& /*event*/)
{
    if(idbmodel->hasBanka(m_selectedId))
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat bankovní účet %s?"), idbmodel->getBanka(m_selectedId).popis()), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            idbmodel->deleteBanka(m_selectedId);
            fillList();
        }
    }
}

void BankyDialog::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = -1;
    else
        m_selectedId = m_banky[static_cast<size_t>(selectedIndex)].id();
    enableTools();
}

void BankyDialog::fillList()
{
    m_list->DeleteAllItems();
    m_list->Hide();
    m_banky = idbmodel->banky();
    for(size_t i=0; i < m_banky.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(m_banky[i].zkratka());
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_banky[i].popis());
        m_list->SetItem(static_cast<long>(i), 2, wxString::Format("%.2f", idbmodel->getBankaStav(m_banky[i].id())));
        m_list->SetItem(static_cast<long>(i), 3, idbmodel->zkratkaMena(m_banky[i].menaid()));
    }
    m_list->Show();
    if(m_select && m_banky.GetCount())
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
    else
        setSelectedId(-1);
}

long BankyDialog::selectedID() const
{
    return m_selectedId;
}

void BankyDialog::enableTools()
{
    if(m_selectedId == -1)
    {
        m_bar->EnableTool(ID_BANKY_EDIT, false);
        m_bar->EnableTool(ID_BANKY_DELETE, false);
    }
    else
    {
        m_bar->EnableTool(ID_BANKY_EDIT, true);
        m_bar->EnableTool(ID_BANKY_DELETE, true);
    }
}
