/***************************************************************
 * Name:      fakturadialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef FAKTURADIALOG_H
#define FAKTURADIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/gbsizer.h>
#include <wx/datectrl.h>
#include <wx/dateevt.h>
#include "model/dbmodel.h"
#include "gui/widgets/dxlistctrl.h"

class FakturaDialog;

class FaPolozkaCtrl: public dxListCtrl
{
public:
    FaPolozkaCtrl(FakturaDialog *dialog, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_parent(dialog) {}

    enum PColumn
    {
        PCOL_POPIS = 0,
        PCOL_MNOZSTVI,
        PCOL_MJ,
        PCOL_CENAMJ,
        PCOL_CENACELKEM,
        PCOL_SKLADOVA,
        PCOL_MAX, // pocet sloupcu
    };

    void CreateColumns();
private:
    FakturaDialog *m_parent;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class FakturaDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(FakturaDialog)
    DECLARE_EVENT_TABLE()
public:
    FakturaDialog() : m_id(-1), m_typid(-1), m_edit(0), m_vydana(false), m_platce(false),
    m_mainSizer(NULL), m_popis(NULL), m_cislo(NULL), m_vs(NULL), m_ks(NULL), m_uhrada(NULL),
    m_uhradaAdd(NULL), m_bankapokladnaText(NULL), m_bankapokladnaSelect(NULL), m_predkontaceText(NULL),
    m_predkontaceSelect(NULL), m_vystaveni(NULL), m_duzp(NULL), m_dppd(NULL), m_splatnost(NULL), m_mena(NULL),
    m_celkem(NULL), m_nazev(NULL), m_jmeno(NULL), m_ulice(NULL), m_cp(NULL), m_misto(NULL), m_psc(NULL), m_zeme(NULL),
    m_ic(NULL), m_dic(NULL), m_email(NULL), m_kontaktSelect(NULL), m_kontaktClear(NULL), m_polozkaSelected(-1),
    m_polozkaBar(NULL), m_polozkyList(NULL), m_dph(NULL), m_dphdruhText(NULL), m_dphdruhSelect(NULL), m_kurz(NULL),
    m_kurzDownload(NULL), m_kurzLabel(NULL), m_kurzValue(0.0), m_pausal(NULL), m_pausalAdd(NULL), m_zaokrouhleni(NULL) {}
    //nova faktura
    FakturaDialog(int typid, bool vydana, wxWindow *parent);
    //editace faktury
    FakturaDialog(int id, wxWindow *parent, int edit = 1);

    wxString getPolozkaText(long item, long column) const;
    int getPolozkaImage(long item, long column) const;
private:
    int m_id;
    int m_typid;
    /* 0 - nova faktura, 1 - uprava, 2 - kopie */
    int m_edit;
    bool m_vydana;
    bool m_platce;
    /* pro Cancel editovane faktury */
    dxFaPolozkaArray m_oldPolozky;
    dxFaPolozkaArray m_polozky;
    wxBoxSizer *m_mainSizer;
    wxTextCtrl *m_popis;
    wxTextCtrl *m_cislo;
    wxTextCtrl *m_vs;
    wxTextCtrl *m_ks;
    wxChoice *m_uhrada;
    wxButton *m_uhradaAdd;
    wxTextCtrl *m_bankapokladnaText;
    wxButton *m_bankapokladnaSelect;
    wxTextCtrl *m_predkontaceText;
    wxButton *m_predkontaceSelect;
    wxDatePickerCtrl *m_vystaveni;
    wxDatePickerCtrl *m_duzp;
    wxDatePickerCtrl *m_dppd;
    wxDatePickerCtrl *m_splatnost;
    wxChoice *m_mena;
    wxStaticText *m_celkem;
    wxTextCtrl *m_nazev;
    wxTextCtrl *m_jmeno;
    wxTextCtrl *m_ulice;
    wxTextCtrl *m_cp;
    wxTextCtrl *m_misto;
    wxTextCtrl *m_psc;
    wxTextCtrl *m_zeme;
    wxTextCtrl *m_ic;
    wxTextCtrl *m_dic;
    wxTextCtrl *m_email;
    wxButton *m_kontaktSelect;
    wxBitmapButton *m_kontaktClear;
    long m_polozkaSelected;
    wxToolBar *m_polozkaBar;
    FaPolozkaCtrl *m_polozkyList;
    wxScopedPtr<wxImageList> m_polozkyImageList;
    wxListCtrl *m_dph;
    wxTextCtrl *m_dphdruhText;
    wxButton *m_dphdruhSelect;
    wxTextCtrl *m_kurz;
    wxBitmapButton *m_kurzDownload;
    wxStaticText *m_kurzLabel;
    double m_kurzValue;
    wxChoice *m_pausal;
    wxButton *m_pausalAdd;
    wxButton *m_zaokrouhleni;

    void createControls();
    wxGridBagSizer *createFakturaGrid();
    wxStaticBoxSizer *createKontaktGrid();
    wxStaticBoxSizer *createPolozkyGrid();
    void createDphGrid();
    void fillControls();
    void fillMena();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnChoiceUhrada(wxCommandEvent& event);
    void OnChoiceMena(wxCommandEvent& event);
    void OnAddUhrada(wxCommandEvent& event);
    void OnSelectBankaPokladna(wxCommandEvent& event);
    void OnSelectPredkontace(wxCommandEvent& event);
    void OnSelectDphDruh(wxCommandEvent& event);
    void OnVystaveniChanged(wxDateEvent& event);
    void OnDuzpChanged(wxDateEvent& event);
    void OnDppdChanged(wxDateEvent& event);
    void OnSelectKontakt(wxCommandEvent& event);
    void OnClearKontakt(wxCommandEvent& event);
    void enableDisableKontakt();
    void OnPolozkaRightClick(wxListEvent& event);
    void OnPolozkaActivated(wxListEvent& event);
    void OnPolozkaSelected(wxListEvent& event);
    void OnPolozkaDeselected(wxListEvent& event);
    void enablePolozkyTools();
    void updateData();
    void updateDph();
    void OnEditPolozka(wxCommandEvent& event);
    void OnAddPolozka(wxCommandEvent& event);
    void OnDeletePolozka(wxCommandEvent& event);
    void OnAddZasoba(wxCommandEvent& event);
    int getMenaId() const;
    int getMenaId(unsigned int index) const;
    void showhideKurz();
    void OnKurzDownload(wxCommandEvent& event);
    void OnKurzChanged(wxCommandEvent& event);
    void OnPausalAdd(wxCommandEvent& event);
    wxDateTime datumForPolozky() const;
    wxDateTime datumForPolozky(const wxDateTime &vystaveni);
    bool isPolozkaEditable(int zasobaid);
    bool isPolozkaRight(size_t index);
    void printFaktura();
    void previewInternal();
    void writeUhrada();
    bool isEdited();
    void OnZaokrouhlit(wxCommandEvent& event);
};

#endif  /* FAKTURADIALOG_H */

