/***************************************************************
 * Name:      fauhradadialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "fauhradadialog.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(FauhradaDialog, wxDialog)

BEGIN_EVENT_TABLE(FauhradaDialog, wxDialog)
    EVT_BUTTON(wxID_OK, FauhradaDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, FauhradaDialog::OnCancel)
END_EVENT_TABLE()

FauhradaDialog::FauhradaDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Zadejte úhradu"))
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void FauhradaDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Úhrada")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_uhrada = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_uhrada, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Pokladnou"), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_pokladnou = new wxCheckBox(this, wxID_ANY, "");
    gridSizer->Add(m_pokladnou, wxGBPosition(1,1));

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void FauhradaDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_uhrada->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Popis nemůže být prázdný"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    if(idbmodel->hasFaUhrada(m_uhrada->GetValue()))
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Úhrada '%s' už existuje\nZvolte jinou"), m_uhrada->GetValue()), "Informace", wxOK|wxICON_INFORMATION);
        dialog.ShowModal();
        return;
    }
    idbmodel->addFaUhrada(FaUhrada(idbmodel->getLastId("fauhrada"), m_uhrada->GetValue(), m_pokladnou->IsChecked()));
    EndModal(wxID_OK);
}

void FauhradaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

