/***************************************************************
 * Name:      fauhradydialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "fauhradydialog.h"
#include "fauhradadialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/checked.xpm"
#include "resources/images/unchecked.xpm"

IMPLEMENT_DYNAMIC_CLASS(FauhradyDialog, wxDialog)

BEGIN_EVENT_TABLE(FauhradyDialog, wxDialog)
    EVT_BUTTON(wxID_OK, FauhradyDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, FauhradyDialog::OnCancel)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_FAUHRADY_LIST, FauhradyDialog::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_FAUHRADY_LIST, FauhradyDialog::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_FAUHRADY_LIST, FauhradyDialog::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_FAUHRADY_LIST, FauhradyDialog::OnItemDeselected)
    EVT_MENU(ID_FAUHRADY_ADD, FauhradyDialog::OnAddFauhrada)
    EVT_MENU(ID_FAUHRADY_DELETE, FauhradyDialog::OnDeleteFauhrada)
END_EVENT_TABLE()

FauhradyDialog::FauhradyDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Úhrady")), m_selectedId(-1)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_FAUHRADY_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat úhradu"));
    m_bar->AddTool(ID_FAUHRADY_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat úhradu"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(checked_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(unchecked_xpm).Scale(16, 16));
    m_list = new wxListCtrl(this, ID_FAUHRADY_LIST, wxDefaultPosition, wxDefaultSize,
                            wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText("Popis");
    col0.SetWidth(130);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText("Pokladnou");
    col1.SetWidth(90);
    m_list->InsertColumn(1, col1);
    fillList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void FauhradyDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void FauhradyDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void FauhradyDialog::OnItemRightClick(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    wxMenu popup;
    popup.Append(ID_FAUHRADY_ADD, wxString::FromUTF8("Přidat úhradu"));
    if(m_selectedId != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_FAUHRADY_DELETE, wxString::FromUTF8("Smazat úhradu"));
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void FauhradyDialog::OnItemActivated(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void FauhradyDialog::OnItemSelected(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void FauhradyDialog::OnItemDeselected(wxListEvent& /*event*/)
{
    setSelectedId(-1);
}

void FauhradyDialog::OnAddFauhrada(wxCommandEvent& /*event*/)
{
    FauhradaDialog dialog(this);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void FauhradyDialog::OnDeleteFauhrada(wxCommandEvent& /*event*/)
{
    if(idbmodel->hasFaUhrada(m_selectedId))
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat úhradu %s?"), idbmodel->getFaUhrada(m_selectedId).uhrada()), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            idbmodel->deleteFauhrada(m_selectedId);
            fillList();
        }
    }
}

void FauhradyDialog::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = -1;
    else
        m_selectedId = idbmodel->getFaUhradaByPopis(m_list->GetItemText(selectedIndex,0));
    enableTools();
}

void FauhradyDialog::fillList()
{
    m_list->DeleteAllItems();
    m_list->Hide();
    dxFaUhradaArray fauhrady = idbmodel->fauhrady();
    for(size_t i=0; i < fauhrady.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(fauhrady[i].uhrada());
        item.SetImage(-1);
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, wxEmptyString, fauhrady[i].pokladnou()?0:1);
    }
    m_list->Show();
    if(fauhrady.GetCount())
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
    else
        setSelectedId(-1);
}

long FauhradyDialog::selectedID() const
{
    return m_selectedId;
}

void FauhradyDialog::enableTools()
{
    if(m_selectedId == -1)
    {
        m_bar->EnableTool(ID_FAUHRADY_DELETE, false);
    }
    else
    {
        m_bar->EnableTool(ID_FAUHRADY_DELETE, true);
    }
}
