/***************************************************************
 * Name:      inventurydialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "inventurydialog.h"
#include "inventuradialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/csv.xpm"
#include "resources/images/print.xpm"
#include "resources/images/printdoc.xpm"
#include "gui/prints/inventuraprint.h"
#include "gui/prints/knihainventurprint.h"

IMPLEMENT_DYNAMIC_CLASS(InventuryDialog, wxDialog)

BEGIN_EVENT_TABLE(InventuryDialog, wxDialog)
    EVT_BUTTON(wxID_OK, InventuryDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, InventuryDialog::OnCancel)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_INVENTURY_LIST, InventuryDialog::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_INVENTURY_LIST, InventuryDialog::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_INVENTURY_LIST, InventuryDialog::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_INVENTURY_LIST, InventuryDialog::OnItemDeselected)
    EVT_MENU(ID_INVENTURY_ADD, InventuryDialog::OnAddInventura)
    EVT_MENU(ID_INVENTURY_EDIT, InventuryDialog::OnEditInventura)
    EVT_MENU(ID_INVENTURY_DELETE, InventuryDialog::OnDeleteInventura)
    EVT_MENU(ID_INVENTURY_PRINTDOKLAD, InventuryDialog::OnPrintInventura)
    EVT_MENU(ID_INVENTURY_PRINT, InventuryDialog::OnPrintInventury)
    EVT_MENU(ID_INVENTURY_EXPORT, InventuryDialog::OnExportInventury)
END_EVENT_TABLE()

InventuryDialog::InventuryDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, "Inventury"), m_selectedId(-1)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_INVENTURY_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat inventuru"));
    m_bar->AddTool(ID_INVENTURY_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, "Upravit inventuru");
    m_bar->AddTool(ID_INVENTURY_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, "Smazat inventuru");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_INVENTURY_PRINTDOKLAD, wxEmptyString, wxBitmap(printdoc_xpm), dxutils::GreyScaledIcon(printdoc_xpm), wxITEM_NORMAL, "Vytisknout inventuru");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_INVENTURY_PRINT, wxEmptyString, wxBitmap(print_xpm), dxutils::GreyScaledIcon(print_xpm), wxITEM_NORMAL, "Vytisknout inventury");
    m_bar->AddTool(ID_INVENTURY_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, "Exportovat inventury");
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    m_list = new wxListCtrl(this, ID_INVENTURY_LIST, wxDefaultPosition, wxDefaultSize,
                            wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    m_list->InsertColumn(0, "Datum", wxLIST_FORMAT_LEFT, 150);
    m_list->InsertColumn(1, wxString::FromUTF8("Číslo"), wxLIST_FORMAT_LEFT, 70);
    m_list->InsertColumn(2, "Popis", wxLIST_FORMAT_LEFT, 250);
    m_list->InsertColumn(3, "Provedl", wxLIST_FORMAT_LEFT, 200);
    m_list->InsertColumn(4, wxString::FromUTF8("Evidovaný stav"), wxLIST_FORMAT_RIGHT, 150);
    m_list->InsertColumn(5, wxString::FromUTF8("Zjištěný stav"), wxLIST_FORMAT_RIGHT, 150);
    fillList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void InventuryDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void InventuryDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void InventuryDialog::OnItemRightClick(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    wxMenu popup;
    popup.Append(ID_INVENTURY_ADD, wxString::FromUTF8("Přidat inventuru"));
    if(m_selectedId != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_INVENTURY_EDIT, "Upravit inventuru");
        popup.Append(ID_INVENTURY_DELETE, "Smazat inventuru");
        popup.AppendSeparator();
        popup.Append(ID_INVENTURY_PRINTDOKLAD, "Vytisknout inventuru");
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void InventuryDialog::OnItemActivated(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    InventuraDialog dialog(this, m_selectedId, true);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void InventuryDialog::OnItemSelected(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void InventuryDialog::OnItemDeselected(wxListEvent& /*event*/)
{
    setSelectedId(-1);
}

void InventuryDialog::OnEditInventura(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    InventuraDialog dialog(this, m_selectedId, true);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void InventuryDialog::OnAddInventura(wxCommandEvent& /*event*/)
{
    InventuraDialog dialog(this);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void InventuryDialog::OnDeleteInventura(wxCommandEvent& /*event*/)
{
    if(idbmodel->hasInventura(m_selectedId))
    {
        wxMessageDialog dialog(this, wxString::Format("Opravdu smazat inventuru\n%s\n%s?", idbmodel->getInventura(m_selectedId).cislo(), idbmodel->getInventura(m_selectedId).popis()), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            idbmodel->deleteInventura(m_selectedId);
            fillList();
        }
    }
}

void InventuryDialog::OnPrintInventura(wxCommandEvent &/*event*/)
{
    if(m_selectedId == -1) return;
    InventuraPrint print(this, idbmodel->getInventura(m_selectedId));
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void InventuryDialog::OnPrintInventury(wxCommandEvent &/*event*/)
{
    KnihaInventurPrint print(this);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void InventuryDialog::OnExportInventury(wxCommandEvent &/*event*/)
{
    wxFileDialog dlg(this, "Exportovat knihu inventur do", wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << "Datum" << delim << wxString::FromUTF8("Číslo") << delim << "Popis" << delim
         << "Provedl" << delim << wxString::FromUTF8("Evidovaný stav") << delim
         << wxString::FromUTF8("Zjištěný stav") << endl;
    for(size_t i=0; i < m_inventury.GetCount(); i++)
    {
        text << dxutils::enquotes(m_inventury[i].datum().Format("%d.%m.%Y %H:%M:%S"), delim);
        text << delim;
        text << dxutils::enquotes(m_inventury[i].cislo(), delim);
        text << delim;
        text << dxutils::enquotes(m_inventury[i].popis(), delim);
        text << delim;
        text << dxutils::enquotes(m_inventury[i].provedl(), delim);
        text << delim;
        dxIvPolozkaArray polozky = idbmodel->ivpolozkyByInventuraId(m_inventury[i].id());
        double evidovano = 0.0, zjisteno = 0.0;
        for(size_t j=0; j < polozky.GetCount(); j++)
        {
            evidovano += polozky[j].evidovano();
            zjisteno += polozky[j].zjisteno();
        }
        text << dxutils::enquotes(wxString::Format("%.3f", evidovano), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.3f", zjisteno), delim);
        text << endl;
    }
}

void InventuryDialog::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = -1;
    else
        m_selectedId = m_inventury[selectedIndex].id();
    enableTools();
}

void InventuryDialog::fillList()
{
    m_list->DeleteAllItems();
    m_list->Hide();
    m_inventury = idbmodel->inventury();
    for(size_t i=0; i < m_inventury.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(m_inventury[i].datum().Format("%d.%m.%Y %H:%M:%S"));
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_inventury[i].cislo());
        m_list->SetItem(static_cast<long>(i), 2, m_inventury[i].popis());
        m_list->SetItem(static_cast<long>(i), 3, m_inventury[i].provedl());
        dxIvPolozkaArray polozky = idbmodel->ivpolozkyByInventuraId(m_inventury[i].id());
        double evidovano = 0.0, zjisteno = 0.0;
        for(size_t j=0; j < polozky.GetCount(); j++)
        {
            evidovano += polozky[j].evidovano();
            zjisteno += polozky[j].zjisteno();
        }
        m_list->SetItem(static_cast<long>(i), 4, wxString::Format("%.3f", evidovano));
        m_list->SetItem(static_cast<long>(i), 5, wxString::Format("%.3f", zjisteno));
    }
    m_list->Show();
    setSelectedId(-1);
    if(m_inventury.GetCount())
    {
        m_bar->EnableTool(ID_INVENTURY_PRINT, true);
        m_bar->EnableTool(ID_INVENTURY_EXPORT, true);
    }
    else
    {
        m_bar->EnableTool(ID_INVENTURY_PRINT, false);
        m_bar->EnableTool(ID_INVENTURY_EXPORT, false);
    }
}

void InventuryDialog::enableTools()
{
    if(m_selectedId == -1)
    {
        m_bar->EnableTool(ID_INVENTURY_EDIT, false);
        m_bar->EnableTool(ID_INVENTURY_DELETE, false);
        m_bar->EnableTool(ID_INVENTURY_PRINTDOKLAD, false);
    }
    else
    {
        m_bar->EnableTool(ID_INVENTURY_EDIT, true);
        m_bar->EnableTool(ID_INVENTURY_DELETE, true);
        m_bar->EnableTool(ID_INVENTURY_PRINTDOKLAD, true);
    }
}
