/***************************************************************
 * Name:      kontaktydialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "kontaktydialog.h"
#include "kontaktdialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/copy.xpm"

IMPLEMENT_DYNAMIC_CLASS(KontaktyDialog, wxDialog)

BEGIN_EVENT_TABLE(KontaktyDialog, wxDialog)
    EVT_BUTTON(wxID_OK, KontaktyDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, KontaktyDialog::OnCancel)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_KONTAKTY_LIST, KontaktyDialog::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_KONTAKTY_LIST, KontaktyDialog::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_KONTAKTY_LIST, KontaktyDialog::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_KONTAKTY_LIST, KontaktyDialog::OnItemDeselected)
    EVT_MENU(ID_KONTAKTY_ADD, KontaktyDialog::OnAddKontakt)
    EVT_MENU(ID_KONTAKTY_EDIT, KontaktyDialog::OnEditKontakt)
    EVT_MENU(ID_KONTAKTY_COPY, KontaktyDialog::OnCopyKontakt)
    EVT_MENU(ID_KONTAKTY_DELETE, KontaktyDialog::OnDeleteKontakt)
END_EVENT_TABLE()

KontaktyDialog::KontaktyDialog(wxWindow *parent, bool select)
    : wxDialog(parent, wxID_ANY, "Kontakty"), m_select(select), m_selectedId(-1)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_KONTAKTY_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat kontakt"));
    m_bar->AddTool(ID_KONTAKTY_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, "Upravit kontakt");
    m_bar->AddTool(ID_KONTAKTY_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat kontakt"));
    m_bar->AddTool(ID_KONTAKTY_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, "Smazat kontakt");
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    m_list = new wxListCtrl(this, ID_KONTAKTY_LIST, wxDefaultPosition, wxDefaultSize, wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText(wxString::FromUTF8("Název"));
    col0.SetWidth(150);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText(wxString::FromUTF8("Obchodní jméno"));
    col1.SetWidth(150);
    m_list->InsertColumn(1, col1);
    wxListItem col2;
    col2.SetId(2);
    col2.SetText(wxString::FromUTF8("IČ"));
    col2.SetAlign(wxLIST_FORMAT_RIGHT);
    col2.SetWidth(100);
    m_list->InsertColumn(2, col2);
    wxListItem col3;
    col3.SetId(3);
    col3.SetText("Adresa");
    col3.SetWidth(300);
    m_list->InsertColumn(3, col3);
    fillList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, select? "&Vybrat" : "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void KontaktyDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void KontaktyDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void KontaktyDialog::OnItemRightClick(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    wxMenu popup;
    popup.Append(ID_KONTAKTY_ADD, wxString::FromUTF8("Přidat kontakt"));
    if(m_selectedId != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_KONTAKTY_EDIT, "Upravit kontakt");
        popup.Append(ID_KONTAKTY_COPY, wxString::FromUTF8("Kopírovat kontakt"));
        popup.Append(ID_KONTAKTY_DELETE, "Smazat kontakt");
    }
    PopupMenu(&popup, event.GetPoint());
}

void KontaktyDialog::OnItemActivated(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    if(m_select)
    {
        EndModal(wxID_OK);
    }
    else
    {
        KontaktDialog dialog(this, m_selectedId, true);
        if(dialog.ShowModal() == wxID_OK)
            fillList();
    }
}

void KontaktyDialog::OnItemSelected(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void KontaktyDialog::OnItemDeselected(wxListEvent &/*event*/)
{
    setSelectedId(-1);
}

void KontaktyDialog::OnEditKontakt(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    KontaktDialog dialog(this, m_selectedId, 1);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void KontaktyDialog::OnAddKontakt(wxCommandEvent& /*event*/)
{
    KontaktDialog dialog(this, -1, 0);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void KontaktyDialog::OnCopyKontakt(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    KontaktDialog dialog(this, idbmodel->copyKontakt(m_selectedId), 2);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void KontaktyDialog::OnDeleteKontakt(wxCommandEvent& /*event*/)
{
    Kontakt ko = idbmodel->getKontakt(m_selectedId);
    if(ko.isValid())
    {
        wxMessageDialog dialog(this, wxString::Format("Opravdu smazat kontakt %s?", ko.nazev()), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            int adresaid = ko.adresaid();
            if(idbmodel->deleteKontakt(m_selectedId))
            {
                idbmodel->deleteAdresa(adresaid);
            }
            fillList();
        }
    }
}

void KontaktyDialog::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = -1;
    else
        m_selectedId = m_kontakty[selectedIndex].id();
    enableTools();
}

void KontaktyDialog::fillList()
{
    m_list->DeleteAllItems();
    m_list->Hide();
    m_kontakty = idbmodel->kontaktyview();
    for(size_t i=0; i < m_kontakty.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(m_kontakty[i].nazev());
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_kontakty[i].jmeno());
        m_list->SetItem(static_cast<long>(i), 2, m_kontakty[i].ic());
        m_list->SetItem(static_cast<long>(i), 3, m_kontakty[i].adresa());
    }
    m_list->Show();
    if(m_select && m_kontakty.GetCount())
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
    else
        setSelectedId(-1);
}

long KontaktyDialog::selectedID() const
{
    return m_selectedId;
}

void KontaktyDialog::enableTools()
{
    if(m_selectedId == -1)
    {
        m_bar->EnableTool(ID_KONTAKTY_EDIT, false);
        m_bar->EnableTool(ID_KONTAKTY_COPY, false);
        m_bar->EnableTool(ID_KONTAKTY_DELETE, false);
    }
    else
    {
        m_bar->EnableTool(ID_KONTAKTY_EDIT, m_kontakty.GetCount()?true:false);
        m_bar->EnableTool(ID_KONTAKTY_COPY, m_kontakty.GetCount()?true:false);
        m_bar->EnableTool(ID_KONTAKTY_DELETE, m_kontakty.GetCount()?true:false);
    }
}
