/***************************************************************
 * Name:      majetekdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "majetekdialog.h"
#include <wx/valnum.h>
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(MajetekDialog, wxDialog)

BEGIN_EVENT_TABLE(MajetekDialog, wxDialog)
    EVT_BUTTON(wxID_OK, MajetekDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, MajetekDialog::OnCancel)
    EVT_CHOICE(ID_MAJETEK_TYP, MajetekDialog::OnTyp)
    EVT_CHOICE(ID_MAJETEK_ODPISY, MajetekDialog::OnOdpisy)
    EVT_BUTTON(ID_MAJETEK_VYRADIT, MajetekDialog::OnVyradit)
END_EVENT_TABLE()

//Novy majetek
MajetekDialog::MajetekDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, "Karta majetku"), m_edit(0), m_vyrazen(false)
{
    m_id = idbmodel->getLastId("majetek");
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

//Editace majetku
MajetekDialog::MajetekDialog(wxWindow *parent, int id, int edit)
    : wxDialog(parent, wxID_ANY, "Karta majetku"), m_id(id), m_edit(edit)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void MajetekDialog::createControls()
{
    int i=0; //row

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Evidenční číslo")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cislo = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_cislo, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Název")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_nazev = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_nazev, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popis = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE);
    gridSizer->Add(m_popis, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Typ"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_typ = new wxChoice(this, ID_MAJETEK_TYP);
    m_typ->Append(wxString::FromUTF8("Dlouhodobý hmotný majetek"));
    m_typ->Append(wxString::FromUTF8("Dlouhodobý nehmotný majetek"));
    m_typ->Append(wxString::FromUTF8("Drobný majetek"));
    gridSizer->Add(m_typ, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Odpisy"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_odpisy = new wxChoice(this, ID_MAJETEK_ODPISY);
    m_odpisy->Append(wxString::FromUTF8("Bez odpisů"));
    m_odpisy->Append(wxString::FromUTF8("Rovnoměrné"));
    m_odpisy->Append(wxString::FromUTF8("Zrychlené"));
    m_odpisy->Append(wxString::FromUTF8("Vyloučené"));
    gridSizer->Add(m_odpisy, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    m_skupinaLabel = new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Odpisová skupina"));
    gridSizer->Add(m_skupinaLabel, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_skupina = new wxChoice(this, wxID_ANY);
    dxOdpiskupinaArray skupiny = idbmodel->odpiskupiny();
    for(size_t j=0; j<skupiny.GetCount(); j++)
        m_skupina->Append(skupiny[j].popis(), new Odpiskupina(skupiny[j].id(), skupiny[j].popis(), skupiny[j].odpisovat(), skupiny[j].rov1(), skupiny[j].rov2(), skupiny[j].rovzvy(),
                                                              skupiny[j].zry1(), skupiny[j].zry2(), skupiny[j].zryzvy()));
    m_skupina->SetSelection(0);
    gridSizer->Add(m_skupina, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pořízen")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_porizen = new wxDatePickerCtrl(this, wxID_ANY, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    gridSizer->Add(m_porizen, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Cena"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cena = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxTE_RIGHT,
                                  wxMakeFloatingPointValidator(2, &m_cenaValue, wxNUM_VAL_THOUSANDS_SEPARATOR));
    gridSizer->Add(m_cena, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    m_vyradit = new wxButton(this, ID_MAJETEK_VYRADIT, wxString::FromUTF8("Vyřadit"));
    gridSizer->Add(m_vyradit, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    m_datumvyrazeniLabel = new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Vyřazen"));
    gridSizer->Add(m_datumvyrazeniLabel, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_datumvyrazeni = new wxDatePickerCtrl(this, wxID_ANY, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    gridSizer->Add(m_datumvyrazeni, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);

    gridSizer->SetEmptyCellSize(wxSize(0,0));
    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void MajetekDialog::fillControls()
{
    if(m_edit)
    {
        Majetek ma = idbmodel->getMajetek(m_id);
        if(ma.isValid())
        {
            m_cislo->SetValue(ma.cislo());
            m_nazev->SetValue(ma.nazev());
            m_popis->SetValue(ma.popis());
            m_typ->SetSelection(ma.typ()-1);
            if(m_edit == 1) m_typ->Disable();
            m_odpisy->SetSelection(ma.odpis());
            if(m_edit == 1) m_odpisy->Disable();
            m_porizen->SetValue(ma.porizen());
            if(m_edit == 1) m_porizen->Disable();
            m_cenaValue = ma.cena();
            if(m_edit == 1) m_cena->Disable();
            m_datumvyrazeni->SetValue(ma.datumvyrazeni());
            m_vyrazen = ma.vyrazen();
            if(m_vyrazen)
            {
                m_vyradit->Hide();
            }
            else
            {
                m_datumvyrazeniLabel->Hide();
                m_datumvyrazeni->Hide();
            }
            m_skupina->SetSelection(0);
            for(unsigned int i=0; i<m_skupina->GetCount(); i++)
            {
                if(static_cast<Odpiskupina*>(m_skupina->GetClientData(i))->id() == ma.skupinaid())
                {
                    m_skupina->SetSelection(static_cast<int>(i));
                    break;
                }
            }
            if(ma.odpis() == 0 || ma.odpis() == 3)
            {
                m_skupinaLabel->Hide();
                m_skupina->Hide();
            }
            if(m_edit == 1) m_skupina->Disable();
        }
    }
    else
    {
        m_cislo->SetValue(idbmodel->getCisloDokladu("MA"));
        m_typ->SetSelection(0);
        m_odpisy->SetSelection(1);
        m_cenaValue = 0.0;
        m_vyradit->Hide();
        m_datumvyrazeniLabel->Hide();
        m_datumvyrazeni->Hide();
    }
}

void MajetekDialog::OnOk(wxCommandEvent& /*event*/)
{
    m_cena->GetValidator()->TransferFromWindow();
    if(!m_edit)
    {
        idbmodel->addMajetek(Majetek(m_id, m_cislo->GetValue(), m_nazev->GetValue(), m_popis->GetValue(),
                                         m_porizen->GetValue(), m_vyrazen, wxDateTime(1, wxDateTime::Jan), m_cenaValue, m_typ->GetSelection()+1,
                                         m_odpisy->GetSelection(), static_cast<Odpiskupina*>(m_skupina->GetClientData(m_skupina->GetSelection()))->id()));
    }
    else
    {
        idbmodel->editMajetek(Majetek(m_id, m_cislo->GetValue(), m_nazev->GetValue(), m_popis->GetValue(),
                                      m_porizen->GetValue(), m_vyrazen, m_datumvyrazeni->GetValue(), m_cenaValue, m_typ->GetSelection()+1,
                                      m_odpisy->GetSelection(), static_cast<Odpiskupina*>(m_skupina->GetClientData(m_skupina->GetSelection()))->id()));
    }
    if(m_edit == 0 || m_edit == 2)
    {
        idbmodel->increaseRadaCislo("MA");
    }
    EndModal(wxID_OK);
}

void MajetekDialog::OnCancel(wxCommandEvent& /*event*/)
{
    if(m_edit == 2)
    {
        idbmodel->deleteMajetek(m_id);
    }
    EndModal(wxID_CANCEL);
}

void MajetekDialog::OnTyp(wxCommandEvent &event)
{
    int typ = m_typ->GetSelection();
    if(typ) //nemhotny nebo drobny
    {
        m_odpisy->SetSelection(0);
        if(typ==2) m_odpisy->Disable(); //drobny
        else m_odpisy->Enable(); //nehmotny
    }
    else
    {
        m_odpisy->SetSelection(1);
        m_odpisy->Enable();
    }
    OnOdpisy(event);
}

void MajetekDialog::OnOdpisy(wxCommandEvent &/*event*/)
{
    int odpis = m_odpisy->GetSelection();
    if(odpis == 0 || odpis == 3)
    {
        m_skupinaLabel->Hide();
        m_skupina->Hide();
    }
    else
    {
        m_skupinaLabel->Show();
        m_skupina->Show();
    }
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
}

void MajetekDialog::OnVyradit(wxCommandEvent &/*event*/)
{
    m_vyrazen = true;
    m_vyradit->Hide();
    m_datumvyrazeniLabel->Show();
    m_datumvyrazeni->Show();
    m_datumvyrazeni->SetValue(dxsettings.datum());
    m_datumvyrazeni->SetRange((idbmodel->hasMajetek(m_id)?idbmodel->getMajetek(m_id).porizen():dxsettings.datum()), dxsettings.datum());
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
}
