/***************************************************************
 * Name:      mahradaialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "nahradadialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "gui/prints/nahradaprint.h"

IMPLEMENT_DYNAMIC_CLASS(NahradaDialog, wxDialog)

BEGIN_EVENT_TABLE(NahradaDialog, wxDialog)
    EVT_BUTTON(wxID_OK, NahradaDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, NahradaDialog::OnCancel)
    EVT_BUTTON(wxID_APPLY, NahradaDialog::OnCalculate)
    EVT_BUTTON(wxID_PRINT, NahradaDialog::OnPrint)
    EVT_CHECKBOX(ID_NAHRADA_VOZIK, NahradaDialog::OnVozik)
END_EVENT_TABLE()

NahradaDialog::NahradaDialog(wxWindow *parent, const Jizda &jizda)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Náhrada jízdy")), m_jizda(jizda)
{
    m_vozidlo = idbmodel->getVozidlo(m_jizda.vozidloid());
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void NahradaDialog::createControls()
{
    int i = 0; //row index

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Začátek cesty")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, m_jizda.startcas().Format("%x %X")), wxGBPosition(i,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Konec cesty"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, m_jizda.endcas().Format("%x %X")), wxGBPosition(i,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Ujeto km"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::Format("%d", m_jizda.endkm()-m_jizda.startkm())), wxGBPosition(i,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Vozidlo"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, m_vozidlo.jmeno()+"\n"+m_vozidlo.rz()), wxGBPosition(i,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    i++;

    m_vozik = new wxCheckBox(this, ID_NAHRADA_VOZIK, wxEmptyString);
    if(m_vozidlo.druhvozidla() == 1)
    {
        gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Přívěsný vozík")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        gridSizer->Add(m_vozik, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
        i++;
    }

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Výše náhrady za km")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cenakm = new wxSpinCtrlDouble(this, wxID_ANY);
    m_cenakm->SetDigits(2);
    m_cenakm->SetRange(0.0, 10000.0);
    m_cenakm->SetValue(idbmodel->nahradaKm(m_vozidlo.druhvozidla(), dxsettings.datum().GetYear()));
    gridSizer->Add(m_cenakm, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Cena PHM"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cenaphm = new wxSpinCtrlDouble(this, wxID_ANY);
    m_cenaphm->SetDigits(2);
    m_cenaphm->SetRange(0.0, 10000.0);
    m_cenaphm->SetValue(idbmodel->cenaPhm(m_vozidlo.phm(), dxsettings.datum()));
    gridSizer->Add(m_cenaphm, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    m_calculate = new wxButton(this, wxID_APPLY, wxString::FromUTF8("Spočítat"));
    gridSizer->Add(m_calculate, wxGBPosition(i,0), wxGBSpan(1,2), wxALIGN_CENTER_HORIZONTAL);
    i++;

    m_nahradaZaKm = new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Náhrada za ujeté km"));
    gridSizer->Add(m_nahradaZaKm, wxGBPosition(i,0), wxGBSpan(1,2), wxALIGN_CENTER_HORIZONTAL);
    i++;

    m_nahradaZaPalivo = new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Náhrada za spotřebované palivo"));
    gridSizer->Add(m_nahradaZaPalivo, wxGBPosition(i,0), wxGBSpan(1,2), wxALIGN_CENTER_HORIZONTAL);
    i++;

    m_print = new wxButton(this, wxID_PRINT);
    m_print->Disable();
    gridSizer->Add(m_print, wxGBPosition(i,0), wxGBSpan(1,2), wxALIGN_CENTER_HORIZONTAL);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void NahradaDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void NahradaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void NahradaDialog::OnCalculate(wxCommandEvent &/*event*/)
{
    m_nahradaKm = (m_jizda.endkm()-m_jizda.startkm())*m_cenakm->GetValue();
    m_nahradaPalivo = ((m_jizda.endkm()-m_jizda.startkm())*m_vozidlo.spotreba()*m_cenaphm->GetValue())/100.0;
    m_nahradaZaKm->SetLabel(wxString::Format(wxString::FromUTF8("Náhrada za ujeté km: %.0f Kč"), m_nahradaKm));
    m_nahradaZaPalivo->SetLabel(wxString::Format(wxString::FromUTF8("Náhrada za spotřebované palivo: %.0f Kč"), m_nahradaPalivo));
    m_print->Enable();
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
}

void NahradaDialog::OnPrint(wxCommandEvent &event)
{
    OnCalculate(event);
    NahradaPrint print(this, m_jizda, m_nahradaKm, m_nahradaPalivo, m_cenakm->GetValue(), m_cenaphm->GetValue());
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void NahradaDialog::OnVozik(wxCommandEvent &event)
{
    if(m_vozidlo.druhvozidla()==1)
    {
        if(m_vozik->GetValue()) m_cenakm->SetValue(idbmodel->nahradaKm(0, dxsettings.datum().GetYear()));
        else m_cenakm->SetValue(idbmodel->nahradaKm(1, dxsettings.datum().GetYear()));
    }
    if(m_print->IsEnabled())
    {
        OnCalculate(event);
    }
}
