/***************************************************************
 * Name:      neuhrazenedialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "neuhrazenedialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"
#include "uhradadialog.h"

IMPLEMENT_DYNAMIC_CLASS(NeuhrazeneDialog, wxDialog)

BEGIN_EVENT_TABLE(NeuhrazeneDialog, wxDialog)
    EVT_BUTTON(wxID_OK, NeuhrazeneDialog::OnOk)
END_EVENT_TABLE()

NeuhrazeneDialog::NeuhrazeneDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Neuhrazené"))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    //m_book = new wxChoicebook(this, wxID_ANY);
    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, "Faktury"), 0, wxALL, 5);
    m_fa = new FaPage(this);
    m_mainSizer->Add(m_fa, 1, wxALL|wxEXPAND, 5);
    //m_book->AddPage(fa, "Faktury");
    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pohledávky a závazky")), 0, wxALL, 5);
    m_zp = new ZpPage(this);
    m_mainSizer->Add(m_zp, 1, wxALL|wxEXPAND, 5);
    //m_book->AddPage(zp, wxString::FromUTF8("Pohledávky a závazky"));
    //m_mainSizer->Add(m_book, 1, wxALL|wxEXPAND, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

NeuhrazeneDialog::~NeuhrazeneDialog()
{
    m_fa->clearData();
    m_zp->clearData();
}

void NeuhrazeneDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void FaCtrl::CreateColumns()
{
    InsertColumn(FCOL_DOKLAD, "Doklad");
    InsertColumn(FCOL_POPIS, "Popis");
    SetColumnWidth(FCOL_POPIS, 250);
    InsertColumn(FCOL_PROTISTRANA, "Protistrana");
    SetColumnWidth(FCOL_PROTISTRANA, 110);
    InsertColumn(FCOL_VYSTAVENA, "Vystavena");
    SetColumnWidth(FCOL_VYSTAVENA, 110);
    InsertColumn(FCOL_SPLATNA, wxString::FromUTF8("Splatná"));
    SetColumnWidth(FCOL_SPLATNA, 110);
    InsertColumn(FCOL_CELKEM, "Celkem", wxLIST_FORMAT_RIGHT);
    InsertColumn(FCOL_UHRADIT, "Uhradit", wxLIST_FORMAT_RIGHT);
    setResizeColumn(FCOL_POPIS);
}

wxString FaCtrl::OnGetItemText(long item, long column) const
{
    return m_page->getItemText(item, column);
}

int FaCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *FaCtrl::OnGetItemAttr(long /*item*/) const
{
    return nullptr;
}

IMPLEMENT_DYNAMIC_CLASS(FaPage, wxPanel)

BEGIN_EVENT_TABLE(FaPage, wxPanel)
    EVT_LIST_COL_CLICK(ID_PAGE_FA_LIST, FaPage::OnColumnClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PAGE_FA_LIST, FaPage::OnItemActivated)
END_EVENT_TABLE()

FaPage::FaPage(wxWindow *parent)
    : wxPanel(parent), m_sortCol(FaCtrl::FCOL_DEF_SORT), m_sortAsc(true)
{
    m_faktury = idbmodel->neuhrazeneFakturyView();
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new FaCtrl(this, this, ID_PAGE_FA_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    updateData();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

wxString FaPage::getItemText(long item, long column) const
{
    if(item < 0 || item >= static_cast<long>(m_faktury.GetCount()))
        return "";
    switch(column)
    {
    case FaCtrl::FCOL_DOKLAD:
        return m_faktury[static_cast<size_t>(item)].cislo();
    case FaCtrl::FCOL_POPIS:
        return m_faktury[static_cast<size_t>(item)].popis();
    case FaCtrl::FCOL_PROTISTRANA:
        return m_faktury[static_cast<size_t>(item)].protistrana();
    case FaCtrl::FCOL_VYSTAVENA:
        return m_faktury[static_cast<size_t>(item)].vystaveni().FormatDate();
    case FaCtrl::FCOL_SPLATNA:
        return m_faktury[static_cast<size_t>(item)].splatnost().FormatDate();
    case FaCtrl::FCOL_CELKEM:
        return wxString::Format("%.2f", m_faktury[static_cast<size_t>(item)].celkem());
    case FaCtrl::FCOL_UHRADIT:
        return wxString::Format("%.2f", m_faktury[static_cast<size_t>(item)].celkem()-m_faktury[static_cast<size_t>(item)].uhrazeno());
    default:
        return "";
    }
}

void FaPage::clearData()
{
    m_faktury.Clear();
}

void FaPage::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= FaCtrl::FCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<FaCtrl::FColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    updateData();
}

void FaPage::OnItemActivated(wxListEvent &event)
{
    UhradaDialog dialog(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(event.GetIndex())].id()));
    if(dialog.ShowModal() == wxID_OK)
    {
        if(idbmodel->isFakturaUhrazena(m_faktury[static_cast<size_t>(event.GetIndex())].id()))
            m_faktury.RemoveAt(static_cast<size_t>(event.GetIndex()));
        updateData();
    }
}

void FaPage::setColumnImage(FaCtrl::FColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void FaPage:: updateData()
{
    long oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    idbmodel->sortFakturyView(m_faktury, columnToFakturaSort(m_sortCol), m_sortAsc);
    long cnt = static_cast<long>(m_faktury.GetCount());
    m_list->SetItemCount(cnt);
    if(cnt)
    {
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}

DbModel::FZPSort FaPage::columnToFakturaSort(FaCtrl::FColumn column)
{
    switch(column)
    {
    case FaCtrl::FCOL_DOKLAD: return DbModel::FZP_DOKLAD;
    case FaCtrl::FCOL_POPIS: return DbModel::FZP_POPIS;
    case FaCtrl::FCOL_PROTISTRANA: return DbModel::FZP_PROTISTRANA;
    case FaCtrl::FCOL_VYSTAVENA: return DbModel::FZP_VYSTAVENA;
    case FaCtrl::FCOL_SPLATNA: return DbModel::FZP_SPLATNA;
    case FaCtrl::FCOL_CELKEM: return DbModel::FZP_CELKEM;
    case FaCtrl::FCOL_UHRADIT: return DbModel::FZP_UHRADIT;
    default: return DbModel::FZP_DOKLAD;
    }
}

void ZpCtrl::CreateColumns()
{
    InsertColumn(ZCOL_DOKLAD, "Doklad");
    InsertColumn(ZCOL_POPIS, "Popis");
    SetColumnWidth(ZCOL_POPIS, 250);
    InsertColumn(ZCOL_PROTISTRANA, "Protistrana");
    SetColumnWidth(ZCOL_PROTISTRANA, 110);
    InsertColumn(ZCOL_VYSTAVENA, "Vystavena");
    SetColumnWidth(ZCOL_VYSTAVENA, 110);
    InsertColumn(ZCOL_SPLATNA, wxString::FromUTF8("Splatná"));
    SetColumnWidth(ZCOL_SPLATNA, 110);
    InsertColumn(ZCOL_CELKEM, "Celkem", wxLIST_FORMAT_RIGHT);
    InsertColumn(ZCOL_UHRADIT, "Uhradit", wxLIST_FORMAT_RIGHT);
    setResizeColumn(ZCOL_POPIS);
}

wxString ZpCtrl::OnGetItemText(long item, long column) const
{
    return m_page->getItemText(item, column);
}

int ZpCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *ZpCtrl::OnGetItemAttr(long /*item*/) const
{
    return nullptr;
}

IMPLEMENT_DYNAMIC_CLASS(ZpPage, wxPanel)

BEGIN_EVENT_TABLE(ZpPage, wxPanel)
    EVT_LIST_COL_CLICK(ID_PAGE_ZP_LIST, ZpPage::OnColumnClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PAGE_ZP_LIST, ZpPage::OnItemActivated)
END_EVENT_TABLE()

ZpPage::ZpPage(wxWindow *parent)
    : wxPanel(parent), m_sortCol(ZpCtrl::ZCOL_DEF_SORT), m_sortAsc(true)
{
    m_zavpohy = idbmodel->neuhrazeneZavpohyView();
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new ZpCtrl(this, this, ID_PAGE_ZP_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    updateData();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

wxString ZpPage::getItemText(long item, long column) const
{
    if(item < 0 || item >= static_cast<long>(m_zavpohy.GetCount()))
        return "";
    switch(column)
    {
    case ZpCtrl::ZCOL_DOKLAD:
        return m_zavpohy[static_cast<size_t>(item)].cislo();
    case ZpCtrl::ZCOL_POPIS:
        return m_zavpohy[static_cast<size_t>(item)].popis();
    case ZpCtrl::ZCOL_PROTISTRANA:
        return m_zavpohy[static_cast<size_t>(item)].protistrana();
    case ZpCtrl::ZCOL_VYSTAVENA:
        return m_zavpohy[static_cast<size_t>(item)].vystaveni().FormatDate();
    case ZpCtrl::ZCOL_SPLATNA:
        return m_zavpohy[static_cast<size_t>(item)].splatnost().FormatDate();
    case ZpCtrl::ZCOL_CELKEM:
        return wxString::Format("%.2f", m_zavpohy[static_cast<size_t>(item)].celkem());
    case ZpCtrl::ZCOL_UHRADIT:
        return wxString::Format("%.2f", m_zavpohy[static_cast<size_t>(item)].celkem()-m_zavpohy[static_cast<size_t>(item)].uhrazeno());
    default:
        return "";
    }
}

void ZpPage::clearData()
{
    m_zavpohy.Clear();
}

void ZpPage::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= ZpCtrl::ZCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<ZpCtrl::ZColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    updateData();
}

void ZpPage::OnItemActivated(wxListEvent &event)
{
    UhradaDialog dialog(this, idbmodel->getZavPoh(m_zavpohy[static_cast<size_t>(event.GetIndex())].id()));
    if(dialog.ShowModal() == wxID_OK)
    {
        if(idbmodel->isZavpohUhrazena(m_zavpohy[static_cast<size_t>(event.GetIndex())].id()))
            m_zavpohy.RemoveAt(static_cast<size_t>(event.GetIndex()));
        updateData();
    }
}

void ZpPage::setColumnImage(ZpCtrl::ZColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void ZpPage::updateData()
{
    long oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    idbmodel->sortZavPohView(m_zavpohy, columnToZavpohSort(m_sortCol), m_sortAsc);
    long cnt = static_cast<long>(m_zavpohy.GetCount());
    m_list->SetItemCount(cnt);
    if(cnt)
    {
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}

DbModel::FZPSort ZpPage::columnToZavpohSort(ZpCtrl::ZColumn column)
{
    switch(column)
    {
    case ZpCtrl::ZCOL_DOKLAD: return DbModel::FZP_DOKLAD;
    case ZpCtrl::ZCOL_POPIS: return DbModel::FZP_POPIS;
    case ZpCtrl::ZCOL_PROTISTRANA: return DbModel::FZP_PROTISTRANA;
    case ZpCtrl::ZCOL_VYSTAVENA: return DbModel::FZP_VYSTAVENA;
    case ZpCtrl::ZCOL_SPLATNA: return DbModel::FZP_SPLATNA;
    case ZpCtrl::ZCOL_CELKEM: return DbModel::FZP_CELKEM;
    case ZpCtrl::ZCOL_UHRADIT: return DbModel::FZP_UHRADIT;
    default: return DbModel::FZP_DOKLAD;
    }
}
