/***************************************************************
 * Name:      neuhrazenedialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#ifndef NEUHRAZENEDIALOG_H
#define NEUHRAZENEDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/choicebk.h>
#include "model/dbmodel.h"
#include "gui/widgets/dxlistctrl.h"

class FaPage;

class FaCtrl: public dxListCtrl
{
public:
    FaCtrl(FaPage *page, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_page(page) {}

    enum FColumn
    {
        FCOL_DOKLAD = 0,
        FCOL_POPIS,
        FCOL_PROTISTRANA,
        FCOL_VYSTAVENA,
        FCOL_SPLATNA,
        FCOL_CELKEM,
        FCOL_UHRADIT,
        FCOL_MAX, // pocet sloupcu
        FCOL_DEF_SORT = FCOL_VYSTAVENA
    };

    void CreateColumns();
private:
    FaPage *m_page;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class FaPage: public wxPanel
{
    DECLARE_DYNAMIC_CLASS(FaPage)
    DECLARE_EVENT_TABLE()
public:
    FaPage() : m_mainSizer(nullptr), m_list(nullptr), m_sortCol(FaCtrl::FCOL_DOKLAD), m_sortAsc(true) {}
    explicit FaPage(wxWindow *parent);

    wxString getItemText(long item, long column) const;
    void clearData();
private:
    wxBoxSizer *m_mainSizer;
    FaCtrl *m_list;
    FaCtrl::FColumn m_sortCol;
    bool m_sortAsc;
    wxScopedPtr<wxImageList> m_imageList;
    dxFakturaViewArray m_faktury;

    void OnColumnClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void setColumnImage(FaCtrl::FColumn column, int image);
    void updateData();
    DbModel::FZPSort columnToFakturaSort(FaCtrl::FColumn column);
};

class ZpPage;

class ZpCtrl: public dxListCtrl
{
public:
    ZpCtrl(ZpPage *page, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_page(page) {}

    enum ZColumn
    {
        ZCOL_DOKLAD = 0,
        ZCOL_POPIS,
        ZCOL_PROTISTRANA,
        ZCOL_VYSTAVENA,
        ZCOL_SPLATNA,
        ZCOL_CELKEM,
        ZCOL_UHRADIT,
        ZCOL_MAX, // pocet sloupcu
        ZCOL_DEF_SORT = ZCOL_VYSTAVENA
    };

    void CreateColumns();
private:
    ZpPage *m_page;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class ZpPage: public wxPanel
{
    DECLARE_DYNAMIC_CLASS(ZpPage)
    DECLARE_EVENT_TABLE()
public:
    ZpPage() : m_mainSizer(nullptr), m_list(nullptr), m_sortCol(ZpCtrl::ZCOL_DOKLAD), m_sortAsc(true) {}
    explicit ZpPage(wxWindow *parent);

    wxString getItemText(long item, long column) const;
    void clearData();
private:
    wxBoxSizer *m_mainSizer;
    ZpCtrl *m_list;
    ZpCtrl::ZColumn m_sortCol;
    bool m_sortAsc;
    wxScopedPtr<wxImageList> m_imageList;
    dxZavPohViewArray m_zavpohy;

    void OnColumnClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void setColumnImage(ZpCtrl::ZColumn column, int image);
    void updateData();
    DbModel::FZPSort columnToZavpohSort(ZpCtrl::ZColumn column);
};

class NeuhrazeneDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(NeuhrazeneDialog)
    DECLARE_EVENT_TABLE()
public:
    NeuhrazeneDialog() : m_mainSizer(nullptr)/*, m_book(NULL)*/, m_fa(nullptr), m_zp(nullptr) {}
    explicit NeuhrazeneDialog(wxWindow *parent);
    virtual ~NeuhrazeneDialog();
private:
    wxBoxSizer *m_mainSizer;
//    wxChoicebook *m_book;
    FaPage *m_fa;
    ZpPage *m_zp;

    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
};

#endif  /* NEUHRAZENEDIALOG_H */

