/***************************************************************
 * Name:      odeslanedialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#ifndef ODESLANEDIALOG_H
#define ODESLANEDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/imaglist.h>
#include "gui/widgets/dxlistctrl.h"
#include "gui/widgets/dxtoolbar.h"
#include "model/dbmodel.h"

class OdeslaneDialog;

class OdeslaneCtrl: public dxListCtrl
{
public:
    OdeslaneCtrl(OdeslaneDialog *dialog, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_dialog(dialog) {}

    enum OColumn
    {
        OCOL_CAS = 0,
        OCOL_FAKTURA,
        OCOL_PRIJEMCE,
        OCOL_MAX, // pocet sloupcu
        OCOL_DEF_SORT = OCOL_CAS
    };

    void CreateColumns();
private:
    OdeslaneDialog *m_dialog;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class OdeslaneDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(OdeslaneDialog)
    DECLARE_EVENT_TABLE()
public:
    OdeslaneDialog():m_mainSizer(nullptr),m_list(nullptr),m_sortCol(OdeslaneCtrl::OCOL_CAS),m_sortAsc(true),m_bar(nullptr) {}
    explicit OdeslaneDialog(wxWindow *parent);

    wxString getItemText(long item, long column) const;
private:
    wxBoxSizer *m_mainSizer;
    OdeslaneCtrl *m_list;
    wxScopedPtr<wxImageList> m_imageList;
    OdeslaneCtrl::OColumn m_sortCol;
    bool m_sortAsc;
    dxToolBar *m_bar;
    dxOdeslanyArray m_odeslane;

    void OnOk(wxCommandEvent& event);
    void OnExport(wxCommandEvent& event);
    void OnColumnClick(wxListEvent& event);
    DbModel::OSort columnToOdeslaneSort(OdeslaneCtrl::OColumn column);
    void setColumnImage(OdeslaneCtrl::OColumn column, int image);
    void updateList();
};

#endif  /* ODESLANEDIALOG_H */

