/***************************************************************
 * Name:      pokladnydialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "pokladnydialog.h"
#include "pokladnadialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"

IMPLEMENT_DYNAMIC_CLASS(PokladnyDialog, wxDialog)

BEGIN_EVENT_TABLE(PokladnyDialog, wxDialog)
    EVT_BUTTON(wxID_OK, PokladnyDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, PokladnyDialog::OnCancel)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_POKLADNY_LIST, PokladnyDialog::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_POKLADNY_LIST, PokladnyDialog::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_POKLADNY_LIST, PokladnyDialog::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_POKLADNY_LIST, PokladnyDialog::OnItemDeselected)
    EVT_MENU(ID_POKLADNY_ADD, PokladnyDialog::OnAddPokladna)
    EVT_MENU(ID_POKLADNY_EDIT, PokladnyDialog::OnEditPokladna)
    EVT_MENU(ID_POKLADNY_DELETE, PokladnyDialog::OnDeletePokladna)
END_EVENT_TABLE()

PokladnyDialog::PokladnyDialog(wxWindow *parent, bool select)
    : wxDialog(parent, wxID_ANY, "Pokladny"),
      m_select(select), m_selectedId(-1)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_POKLADNY_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat pokladnu"));
    m_bar->AddTool(ID_POKLADNY_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, "Upravit pokladnu");
    m_bar->AddTool(ID_POKLADNY_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, "Smazat pokladnu");
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    m_list = new wxListCtrl(this, ID_POKLADNY_LIST, wxDefaultPosition, wxDefaultSize,
                            wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText("Zkratka");
    col0.SetWidth(80);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText("Popis");
    col1.SetWidth(350);
    m_list->InsertColumn(1, col1);
    wxListItem col2;
    col2.SetId(2);
    col2.SetText(wxString::FromUTF8("Zůstatek"));
    col2.SetAlign(wxLIST_FORMAT_RIGHT);
    col2.SetWidth(120);
    m_list->InsertColumn(2, col2);
    wxListItem col3;
    col3.SetId(3);
    col3.SetText(wxString::FromUTF8("Měna"));
    col3.SetWidth(100);
    m_list->InsertColumn(3, col3);
    fillList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, select? "&Vybrat" : "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void PokladnyDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void PokladnyDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void PokladnyDialog::OnItemRightClick(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    wxMenu popup;
    popup.Append(ID_POKLADNY_ADD, wxString::FromUTF8("Přidat pokladnu"));
    if(m_selectedId != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_POKLADNY_EDIT, "Upravit pokladnu");
        popup.Append(ID_POKLADNY_DELETE, "Smazat pokladnu");
    }
    PopupMenu(&popup, event.GetPoint());
}

void PokladnyDialog::OnItemActivated(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    if(m_select)
    {
        EndModal(wxID_OK);
    }
    else
    {
        PokladnaDialog dialog(this, m_selectedId, true);
        if(dialog.ShowModal() == wxID_OK)
            fillList();
    }
}

void PokladnyDialog::OnItemSelected(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void PokladnyDialog::OnItemDeselected(wxListEvent& /*event*/)
{
    setSelectedId(-1);
}

void PokladnyDialog::OnEditPokladna(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    PokladnaDialog dialog(this, m_selectedId, true);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void PokladnyDialog::OnAddPokladna(wxCommandEvent& /*event*/)
{
    PokladnaDialog dialog(this, -1, false);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void PokladnyDialog::OnDeletePokladna(wxCommandEvent& /*event*/)
{
    if(idbmodel->hasPokladna(m_selectedId))
    {
        wxMessageDialog dialog(this, wxString::Format("Opravdu smazat pokladnu %s?", idbmodel->getPokladna(m_selectedId).popis()), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            idbmodel->deletePokladna(m_selectedId);
            fillList();
        }
    }
}

void PokladnyDialog::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = -1;
    else
        m_selectedId = m_pokladny[selectedIndex].id();
    enableTools();
}

void PokladnyDialog::fillList()
{
    m_list->DeleteAllItems();
    m_list->Hide();
    m_pokladny = idbmodel->pokladny();
    for(size_t i=0; i < m_pokladny.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(m_pokladny[i].zkratka());
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_pokladny[i].popis());
        m_list->SetItem(static_cast<long>(i), 2, wxString::Format("%.2f", idbmodel->getPokladnaStav(m_pokladny[i].id())));
        m_list->SetItem(static_cast<long>(i), 3, idbmodel->zkratkaMena(m_pokladny[i].menaid()));
    }
    m_list->Show();
    if(m_select && m_pokladny.GetCount())
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
    else
        setSelectedId(-1);
}

long PokladnyDialog::selectedID() const
{
    return m_selectedId;
}

void PokladnyDialog::enableTools()
{
    if(m_selectedId == -1)
    {
        m_bar->EnableTool(ID_POKLADNY_EDIT, false);
        m_bar->EnableTool(ID_POKLADNY_DELETE, false);
    }
    else
    {
        m_bar->EnableTool(ID_POKLADNY_EDIT, true);
        m_bar->EnableTool(ID_POKLADNY_DELETE, true);
    }
}
