/***************************************************************
 * Name:      polozkadialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "polozkadialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include <wx/valnum.h>
#include <wx/numdlg.h>
#include <wx/spinctrl.h>

IMPLEMENT_DYNAMIC_CLASS(PolozkaDialog, wxDialog)

BEGIN_EVENT_TABLE(PolozkaDialog, wxDialog)
    EVT_BUTTON(wxID_OK, PolozkaDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, PolozkaDialog::OnCancel)
    EVT_CHECKBOX(ID_POLOZKA_BEZDPH, PolozkaDialog::OnBezDph)
END_EVENT_TABLE()

//Editace polozky
PolozkaDialog::PolozkaDialog(wxWindow *parent, int id, bool platce, const wxDateTime &datum, const wxDateTime &duzp)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Položka faktury")), m_id(id), m_zasobaid(-1), m_edit(true), m_platce(platce), m_datum(datum),
      m_duzp(duzp)
{
    if(idbmodel->hasFaPolozka(m_id))
    {
        FaPolozka fp = idbmodel->getFaPolozka(m_id);
        m_fakturaid = fp.fakturaid();
        m_zavpohid = fp.zavpohid();
        m_zasobaid = fp.zasobaid();
    }
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    if(idbmodel->hasZavPoh(m_zavpohid)) SetTitle(wxString::FromUTF8("Položka závazku/pohledávky"));
}

//Nova polozka
PolozkaDialog::PolozkaDialog(int fakturaid, int zavpohid, wxWindow *parent, bool platce, const wxDateTime &duzp)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Položka faktury")), m_fakturaid(fakturaid), m_zavpohid(zavpohid), m_zasobaid(-1), m_edit(false),
      m_platce(platce), m_pocetValue(0.0), m_cenamjValue(0.0), m_dph(0.0), m_celkem(0.0), m_duzp(duzp)
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    if(idbmodel->hasZavPoh(m_zavpohid)) SetTitle(wxString::FromUTF8("Položka závazku/pohledávky"));
}

//Nova polozka ze zasob
PolozkaDialog::PolozkaDialog(int zasobaid, wxWindow *parent, int fakturaid, bool platce, const wxDateTime &datum, const wxDateTime &duzp)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Položka faktury")), m_fakturaid(fakturaid), m_zasobaid(zasobaid), m_edit(false),
      m_platce(platce), m_pocetValue(0.0), m_cenamjValue(0.0), m_dph(0.0), m_celkem(0.0), m_datum(datum), m_duzp(duzp)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void PolozkaDialog::createControls()
{
    int i=0; //row

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popis = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(200,100), wxTE_MULTILINE);
    gridSizer->Add(m_popis, wxGBPosition(i,1), wxGBSpan(1,3), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Počet")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_pocetValue = 1.0;
    m_pocet = new FloatCtrl(this, ID_POLOZKA_POCET, "", wxDefaultPosition, wxDefaultSize, wxTE_RIGHT,
                            wxMakeFloatingPointValidator(3, &m_pocetValue, wxNUM_VAL_THOUSANDS_SEPARATOR));
    gridSizer->Add(m_pocet, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "mj"), wxGBPosition(i,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mj = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_mj, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Cena za mj"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cenamj = new FloatCtrl(this, ID_POLOZKA_POCET, "", wxDefaultPosition, wxDefaultSize, wxTE_RIGHT,
                             wxMakeFloatingPointValidator(2, &m_cenamjValue, wxNUM_VAL_THOUSANDS_SEPARATOR));
    gridSizer->Add(m_cenamj, wxGBPosition(i,1), wxGBSpan(1,3), wxEXPAND);
    i++;

    if(m_platce)
    {
        m_bezdph = new wxCheckBox(this, ID_POLOZKA_BEZDPH, "bez DPH");
        m_bezdph->SetValue(true);
        gridSizer->Add(m_bezdph, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
        i++;

        m_sazba = new wxChoice(this, ID_POLOZKA_SAZBA);
        dxDphSazbaArray sazby = idbmodel->dphsazby();
        for(size_t j=0; j<sazby.GetCount(); j++)
        {
            m_sazba->Append(wxString::Format("%d%%", sazby[j].sazba()), new DphSazba(sazby[j].id(), sazby[j].sazba(), sazby[j].rok(), sazby[j].zakladni()));
        }
        m_sazba->Select(0);
        gridSizer->Add(new wxStaticText(this, wxID_ANY, "Sazba DPH"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        gridSizer->Add(m_sazba, wxGBPosition(i,1), wxGBSpan(1,3), wxEXPAND);
        i++;

        m_dphLabel = new wxStaticText(this, wxID_ANY, wxString::Format("%.2f", 0.0));
        gridSizer->Add(new wxStaticText(this, wxID_ANY, "DPH"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        gridSizer->Add(m_dphLabel, wxGBPosition(i,1), wxGBSpan(1,3), wxEXPAND);
        i++;
    }

    wxStaticText *label = new wxStaticText(this, wxID_ANY, "Celkem");
    label->SetFont(this->GetFont().Bold());
    m_celkemLabel = new wxStaticText(this, wxID_ANY, wxString::Format("%.2f", 0.0));
    m_celkemLabel->SetFont(this->GetFont().Bold());
    gridSizer->Add(label, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(m_celkemLabel, wxGBPosition(i,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->SetEmptyCellSize(wxSize(0,0));
    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);

    Connect(ID_POLOZKA_SAZBA, wxEVT_CHOICE, wxCommandEventHandler(PolozkaDialog::OnChoiceSazba), nullptr, this);
}

void PolozkaDialog::fillControls()
{
    if(idbmodel->hasZasoba(m_zasobaid) && !m_edit)
    {
        Zasoba za = idbmodel->getZasoba(m_zasobaid);
        if(za.isValid())
        {
            m_popis->SetValue(za.nazev());
            m_mj->SetValue(za.mj());
            m_cenamjValue = idbmodel->isFakturaVydana(m_fakturaid) ? za.cenamj() : idbmodel->zasobaNakupniCena(m_zasobaid);
            if(m_platce)
            {
                m_bezdph->SetValue(za.bezdph());
                for(unsigned int i=0; i<m_sazba->GetCount(); i++)
                {
                    if(za.dph() == static_cast<DphSazba*>(m_sazba->GetClientData(i))->sazba())
                    {
                        m_sazba->Select(static_cast<int>(i));
                        break;
                    }
                }
            }
        }
        calculateValues();
        return;
    }
    FaPolozka fp = idbmodel->getFaPolozka(m_id);
    if(fp.isValid())
    {
        m_popis->SetValue(fp.popis());
        m_mj->SetValue(fp.mj());
        m_pocetValue = fp.pocet();
        m_cenamjValue = fp.cenamj();
        if(m_platce)
        {
            m_bezdph->SetValue(fp.bezdph());
            for(unsigned int i=0; i<m_sazba->GetCount(); i++)
            {
                if(fp.sazbaid() == static_cast<DphSazba*>(m_sazba->GetClientData(i))->id())
                {
                    m_sazba->Select(static_cast<int>(i));
                    break;
                }
            }
        }
    }
    calculateValues();
}


void PolozkaDialog::OnOk(wxCommandEvent& /*event*/)
{
    m_pocet->GetValidator()->TransferFromWindow();
    if(dxutils::isSameDouble(m_pocetValue, 0.0))
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Počet nemůže být nulový"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_pocet->SetFocus();
        return;
    }
    if(m_pocetValue < 0.0)
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Počet nemůže být záporný"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_pocet->SetFocus();
        return;
    }
    m_cenamj->GetValidator()->TransferFromWindow();
    int sazbaid = 0;
    if(m_platce)
    {
        sazbaid = static_cast<DphSazba*>(m_sazba->GetClientData(static_cast<unsigned int>(m_sazba->GetSelection())))->id();
    }
    if(idbmodel->hasZasoba(m_zasobaid))
    {
        bool vydana = idbmodel->isFakturaVydana(m_fakturaid);
        if(vydana)
        {
            double stav = idbmodel->stavZasobyToDate(m_zasobaid, m_datum);
            if(m_edit) //pricteni puvodniho poctu z polozky
            {
                stav += idbmodel->hasFaPolozka(m_id) ? idbmodel->getFaPolozka(m_id).pocet() : 0.0;
            }
            if(m_pocetValue > stav)
            {
                wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Stav zásoby %.3f na skladě je nižší než požadovaný výdej %.3f"), stav, m_pocetValue),
                                       "Informace", wxOK|wxICON_WARNING);
                dialog.ShowModal();
                m_pocet->SetFocus();
                return;
            }
        }
        if(m_edit)
        {
            Pohybzasob pz = idbmodel->getPohybzasobByFaktura(m_zasobaid, m_fakturaid);
            if(pz.isValid())
            {
                idbmodel->editPohybzasob(Pohybzasob(pz.id(), m_datum, pz.cislo(), m_pocetValue, vydana?0.0:dxutils::cenamj(m_cenamjValue, m_platce?m_bezdph->GetValue():1, sazbaid), !vydana,
                                                    false, m_fakturaid, m_zasobaid, idbmodel->currentRok(), false, -1));
            }
            else
            {
                wxMessageDialog dialog(this, wxString::FromUTF8("Položku nelze přidat"), "Informace", wxOK|wxICON_WARNING);
                dialog.ShowModal();
                EndModal(wxID_OK);
            }
        }
        else
        {
            if(idbmodel->getPohybzasobByFaktura(m_zasobaid, m_fakturaid).isValid())
            {
                wxMessageDialog dialog(this, wxString::FromUTF8("Položku nelze přidat, faktura už obsahuje položku pro vybranou zásobu"), "Informace", wxOK|wxICON_WARNING);
                dialog.ShowModal();
                EndModal(wxID_OK);
            }
            idbmodel->addPohybzasob(Pohybzasob(idbmodel->getLastId("pohybzasob"), m_datum, idbmodel->getCisloDokladu(vydana?"VY":"PR"), m_pocetValue, vydana?0.0:dxutils::cenamj(m_cenamjValue, m_platce?m_bezdph->GetValue():1, sazbaid),
                                               !vydana, false, m_fakturaid, m_zasobaid, idbmodel->currentRok(), false, -1));
            idbmodel->increaseRadaCislo(vydana?"VY":"PR");
        }
    }
    if(m_edit)
    {
        idbmodel->editFaPolozka(FaPolozka(m_id, m_popis->GetValue(), m_mj->GetValue(), m_pocetValue, m_cenamjValue,
                                          sazbaid, m_dph, m_celkem, m_platce?m_bezdph->GetValue():0, m_fakturaid, m_zavpohid, m_zasobaid, false));
    }
    else
    {
        idbmodel->addFaPolozka(FaPolozka(idbmodel->getLastId("fapolozka"), m_popis->GetValue(), m_mj->GetValue(), m_pocetValue,
                                             m_cenamjValue, sazbaid, m_dph, m_celkem, m_platce?m_bezdph->GetValue():0, m_fakturaid,
                                             m_zavpohid, m_zasobaid, false));
    }
    EndModal(wxID_OK);
}

void PolozkaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void PolozkaDialog::OnChoiceSazba(wxCommandEvent &/*event*/)
{
    calculateValues();
}

void PolozkaDialog::OnBezDph(wxCommandEvent &/*event*/)
{
    calculateValues();
}

void PolozkaDialog::calculateValues()
{
    m_pocet->GetValidator()->TransferFromWindow();
    m_cenamj->GetValidator()->TransferFromWindow();
    if(m_platce)
    {
        double sazba = static_cast<DphSazba*>(m_sazba->GetClientData(static_cast<unsigned int>(m_sazba->GetSelection())))->sazba();
        if(m_bezdph->GetValue())
            m_dph = m_pocetValue*m_cenamjValue*(sazba/100.0);
        else
        {
            if(m_duzp.IsLaterThan(wxDateTime(31, wxDateTime::Mar, 2019))) m_dph = m_pocetValue*m_cenamjValue*(sazba/(100.0+sazba));
            else m_dph = m_pocetValue*m_cenamjValue*round4(sazba/(100.0+sazba));
        }
        m_celkem = (m_bezdph->GetValue()? m_dph : 0.0) + m_cenamjValue*m_pocetValue;
        m_dphLabel->SetLabel(wxString::Format("%.2f", m_dph));
    }
    else
    {
        m_celkem = m_cenamjValue*m_pocetValue;
    }
    m_celkemLabel->SetLabel(wxString::Format("%.2f", m_celkem));
}

BEGIN_EVENT_TABLE(FloatCtrl, wxTextCtrl)
    EVT_KEY_UP(FloatCtrl::OnKeyUp)
END_EVENT_TABLE()

void FloatCtrl::OnKeyUp(wxKeyEvent &event)
{
    m_parent->calculateValues();
    event.Skip();
}
