/***************************************************************
 * Name:      predkontacesdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "predkontacesdialog.h"
#include "predkontacedialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/checked.xpm"
#include "resources/images/unchecked.xpm"

IMPLEMENT_DYNAMIC_CLASS(PredkontacesDialog, wxDialog)

BEGIN_EVENT_TABLE(PredkontacesDialog, wxDialog)
    EVT_BUTTON(wxID_OK, PredkontacesDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, PredkontacesDialog::OnCancel)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PREDKONTACES_LIST, PredkontacesDialog::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PREDKONTACES_LIST, PredkontacesDialog::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PREDKONTACES_LIST, PredkontacesDialog::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PREDKONTACES_LIST, PredkontacesDialog::OnItemDeselected)
    EVT_MENU(ID_PREDKONTACES_ADD, PredkontacesDialog::OnAddPredkontace)
    EVT_MENU(ID_PREDKONTACES_EDIT, PredkontacesDialog::OnEditPredkontace)
    EVT_MENU(ID_PREDKONTACES_COPY, PredkontacesDialog::OnCopyPredkontace)
    EVT_MENU(ID_PREDKONTACES_DELETE, PredkontacesDialog::OnDeletePredkontace)
END_EVENT_TABLE()

PredkontacesDialog::PredkontacesDialog(wxWindow *parent, bool select, PType type)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Předkontace")),
      m_select(select), m_type(type), m_selectedId(-1)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_PREDKONTACES_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat předkontaci"));
    m_bar->AddTool(ID_PREDKONTACES_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit předkontaci"));
    m_bar->AddTool(ID_PREDKONTACES_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat předkontaci"));
    m_bar->AddTool(ID_PREDKONTACES_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat předkontaci"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(checked_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(unchecked_xpm).Scale(16, 16));
    m_list = new wxListCtrl(this, ID_PREDKONTACES_LIST, wxDefaultPosition, wxDefaultSize,
                            wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText("Zkratka");
    col0.SetWidth(80);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText("Popis");
    col1.SetWidth(350);
    m_list->InsertColumn(1, col1);
    wxListItem col2;
    col2.SetId(2);
    col2.SetText("Typ");
    col2.SetWidth(100);
    m_list->InsertColumn(2, col2);
    wxListItem col3;
    col3.SetId(3);
    col3.SetText(wxString::FromUTF8("Daňový"));
    col3.SetWidth(80);
    m_list->InsertColumn(3, col3);
    fillList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, select? "&Vybrat" : "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void PredkontacesDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void PredkontacesDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void PredkontacesDialog::OnItemRightClick(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    wxMenu popup;
    popup.Append(ID_PREDKONTACES_ADD, wxString::FromUTF8("Přidat předkontaci"));
    if(m_selectedId != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_PREDKONTACES_EDIT, wxString::FromUTF8("Upravit předkontaci"));
        popup.Append(ID_PREDKONTACES_COPY, wxString::FromUTF8("Kopírovat předkontaci"));
        popup.Append(ID_PREDKONTACES_DELETE, wxString::FromUTF8("Smazat předkontaci"));
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void PredkontacesDialog::OnItemActivated(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    if(m_select)
    {
        EndModal(wxID_OK);
    }
    else
    {
        PredkontaceDialog dialog(this, m_selectedId, true);
        if(dialog.ShowModal() == wxID_OK)
            fillList();
    }
}

void PredkontacesDialog::OnItemSelected(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void PredkontacesDialog::OnItemDeselected(wxListEvent& /*event*/)
{
    setSelectedId(-1);
}

void PredkontacesDialog::OnEditPredkontace(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    PredkontaceDialog dialog(this, m_selectedId, true);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void PredkontacesDialog::OnCopyPredkontace(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    PredkontaceDialog dialog(this, idbmodel->copyPredkontace(m_selectedId), 2);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void PredkontacesDialog::OnAddPredkontace(wxCommandEvent& /*event*/)
{
    PredkontaceDialog dialog(this, -1, false);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void PredkontacesDialog::OnDeletePredkontace(wxCommandEvent& /*event*/)
{
    Predkontace pk = idbmodel->getPredkontace(m_selectedId);
    if(pk.isValid())
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat předkontaci %s\n%s?"), pk.zkratka(),
                                                      pk.popis()), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            idbmodel->deletePredkontace(m_selectedId);
            fillList();
        }
    }
}

void PredkontacesDialog::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = -1;
    else
        m_selectedId = idbmodel->getPredkontaceIdByZkratka(m_list->GetItemText(selectedIndex,0));
    enableTools();
}

void PredkontacesDialog::fillList()
{
    m_list->DeleteAllItems();
    m_list->Hide();
    dxPredkontaceArray predkontace;
    switch(m_type)
    {
    case PK_ALL: predkontace = idbmodel->predkontace(); break;
    case PK_PRIJEM: predkontace = idbmodel->predkontacePrijem(); break;
    case PK_VYDAJ: predkontace = idbmodel->predkontaceVydaj(); break;
    }
    for(size_t i=0; i < predkontace.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(predkontace[i].zkratka());
        item.SetImage(-1);
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, predkontace[i].popis());
        m_list->SetItem(static_cast<long>(i), 2, predkontace[i].prijem()?wxString::FromUTF8("Příjem"):wxString::FromUTF8("Výdaj"));
        m_list->SetItem(static_cast<long>(i), 3, wxEmptyString, idbmodel->isPredkontaceDanova(predkontace[i].id())?0:1);
    }
    m_list->Show();
    if(m_select && predkontace.GetCount())
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
    else
        setSelectedId(-1);
}

long PredkontacesDialog::selectedID() const
{
    return m_selectedId;
}

void PredkontacesDialog::enableTools()
{
    if(m_selectedId == -1)
    {
        m_bar->EnableTool(ID_PREDKONTACES_EDIT, false);
        m_bar->EnableTool(ID_PREDKONTACES_COPY, false);
        m_bar->EnableTool(ID_PREDKONTACES_DELETE, false);
    }
    else
    {
        m_bar->EnableTool(ID_PREDKONTACES_EDIT, true);
        m_bar->EnableTool(ID_PREDKONTACES_COPY, true);
        m_bar->EnableTool(ID_PREDKONTACES_DELETE, true);
    }
}
