/***************************************************************
 * Name:      sendemaildialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include "config.h"
#include <wx/gbsizer.h>
#include <wx/filename.h>
#include <wx/mimetype.h>
#include "sendemaildialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "smtp/wxMailer.h"
#include "gui/prints/fakturaneplatceprint.h"
#include "gui/prints/fakturaplatceprint.h"
#include "gui/prints/fakturaxmlprint.h"
#ifdef HAVE_CURL
    #include <curl/curl.h>
#endif

IMPLEMENT_DYNAMIC_CLASS(SendEmailDialog, wxDialog)

BEGIN_EVENT_TABLE(SendEmailDialog, wxDialog)
    EVT_BUTTON(wxID_OK, SendEmailDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, SendEmailDialog::OnCancel)
    EVT_BUTTON(wxID_PREVIEW, SendEmailDialog::OnPreview)
END_EVENT_TABLE()

SendEmailDialog::SendEmailDialog(wxWindow *parent, const Faktura &faktura)
    : wxDialog(parent, wxID_ANY, "Odeslat fakturu"), m_faktura(faktura), m_useSsl(false), m_select(true)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void SendEmailDialog::createControls()
{
    int i = 0;
#ifdef CURL_VERSION_SSL
    m_useSsl = true;
#endif

    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Faktura"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, m_faktura.cislo()), wxGBPosition(i,1), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "SMTP server"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_server = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_server, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Port"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_port = new wxSpinCtrl(this);
    m_port->SetRange(0,65535);
    gridSizer->Add(m_port, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    if(m_useSsl)
    {
        gridSizer->Add(new wxStaticText(this, wxID_ANY, "SSL"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        m_ssl = new wxChoice(this, wxID_ANY);
        m_ssl->Append(wxString::FromUTF8("Žádné"));
        m_ssl->Append("SSL/TLS");
        gridSizer->Add(m_ssl, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
        i++;
    }

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Použít autentifikaci")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_auth = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_auth, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Uživatelské jméno")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_username = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_username, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Heslo"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_password = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_password, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Odesílatel")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_sender = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_sender, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Příjemce")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_recipient = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_recipient, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Předmět")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_subject = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_subject, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Text"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_text = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(300,70), wxTE_MULTILINE);
    gridSizer->Add(m_text, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL, 5);

    wxFileType* fileType = wxTheMimeTypesManager->GetFileTypeFromExtension("pdf");
    if(fileType != nullptr)
    {
        wxBoxSizer *btnSizer = new wxBoxSizer(wxHORIZONTAL);
        wxButton* okButton = new wxButton(this, wxID_OK, "Odeslat", wxDefaultPosition, wxDefaultSize, 0);
        wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
        wxButton* previewButton = new wxButton(this, wxID_PREVIEW, wxString::FromUTF8("Náhled"), wxDefaultPosition, wxDefaultSize, 0);
        btnSizer->Add(previewButton, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 2);
        btnSizer->Add(okButton, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 2);
        btnSizer->Add(cancelButton, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 2);
        m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
        delete fileType;
    }
    else
    {
        wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
        wxButton* okButton = new wxButton(this, wxID_OK, "Odeslat", wxDefaultPosition, wxDefaultSize, 0);
        btnSizer->AddButton(okButton);
        wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
        btnSizer->AddButton(cancelButton);
        btnSizer->Realize();
        m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    }

    this->SetSizer(m_mainSizer);
}

void SendEmailDialog::fillControls()
{
    Smtp smtp = idbmodel->smtp();
    m_server->SetValue(smtp.server());
    m_port->SetValue(smtp.port());
    if(m_useSsl)
    {
        m_ssl->Select(smtp.ssl());
    }
    m_auth->SetValue(smtp.autentifikace());
    m_username->SetValue(smtp.username());
    m_password->SetValue(smtp.password());
    m_sender->SetValue(smtp.sender());
    m_recipient->SetValue(idbmodel->getKontaktEmail(m_faktura.protistranaid()));
    m_subject->SetValue(smtp.subject());
    m_text->SetValue(smtp.text());
}

void SendEmailDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_server->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Adresa serveru nemůže být prázdná"), wxString::FromUTF8("Varování"), wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    if(m_sender->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Odesílatel nemůže být prázdný"), wxString::FromUTF8("Varování"), wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    if(m_recipient->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Příjemce nemůže být prázdný"), wxString::FromUTF8("Varování"), wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    wxString path = createPdf();
    bool ssl = false;
    if(m_useSsl)
    {
        ssl = m_ssl->GetSelection();
    }
    wxMailer mailer(m_username->GetValue(), m_password->GetValue(), wxString::Format("%s://%s:%d", ssl?"smtps":"smtp", m_server->GetValue(), m_port->GetValue()),
                    m_auth->GetValue(), ssl);
    wxEmailMessage message;
    message.SetFrom(m_sender->GetValue()).SetTo(m_recipient->GetValue()).SetSubject(m_subject->GetValue()).SetMessage(m_text->GetValue());
    if(!path.IsEmpty()) message.AddAttachment(path);
    message.Finalize();
    if(mailer.Send(message))
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Email úspěšně odeslán"), "Informace", wxOK|wxICON_INFORMATION);
        dialog.ShowModal();
        idbmodel->addOdeslany(Odeslany(idbmodel->getLastId("odeslany"), wxDateTime::Now(), m_faktura.cislo(), m_recipient->GetValue()));
    }
    wxRemoveFile(path);
    EndModal(wxID_OK);
}

void SendEmailDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void SendEmailDialog::OnPreview(wxCommandEvent& /*event*/)
{
    bool faktura = m_faktura.typid() == 1;
    wxArrayString choices;
    if(faktura && dxsettings.fakturaSablona() != "internal") choices.Add(wxString::FromUTF8("Poslední použitá šablona"));
    if(!faktura && dxsettings.zalohaSablona() != "internal") choices.Add(wxString::FromUTF8("Poslední použitá šablona"));
    choices.Add(wxString::FromUTF8("Výchozí šablona programu"));
    choices.Add("Ze souboru");
    wxSingleChoiceDialog vyber(this, wxString::FromUTF8("Vyberte typ šablony"), wxString::FromUTF8("Typ šablony"), choices);
    vyber.SetSelection(0);
    if(vyber.ShowModal() == wxID_OK)
    {
        if(vyber.GetStringSelection() == wxString::FromUTF8("Výchozí šablona programu"))
        {
            previewInternal();
        }
        else if(vyber.GetStringSelection() == "Ze souboru")
        {
            wxFileDialog dialog(m_parent, wxString::FromUTF8("Otevřít šablonu"), wxGetHomeDir(), wxEmptyString, "XML soubor (*.xml)|*.xml", wxFD_OPEN|wxFD_FILE_MUST_EXIST);
            if(dialog.ShowModal() == wxID_OK)
            {
                wxString path = dialog.GetPath();
                if(!path.empty())
                {
                    if(faktura) dxsettings.setFakturaSablona(path);
                    else dxsettings.setZalohaSablona(path);
                    FakturaXmlPrint print(this, m_faktura, path);
                    print.Preview(false);
                }
                else
                {
                    previewInternal();
                }
            }
            else
            {
                previewInternal();
            }
        }
        else
        {
            if(wxFileExists(faktura ? dxsettings.fakturaSablona() : dxsettings.zalohaSablona()))
            {
                FakturaXmlPrint print(this, m_faktura, faktura ? dxsettings.fakturaSablona() : dxsettings.zalohaSablona());
                print.Preview(false);
            }
            else
            {
                if(faktura) dxsettings.setFakturaSablona("internal");
                else dxsettings.setZalohaSablona("internal");
                previewInternal();
            }
        }
        m_select = false;
    }
    else
    {
        previewInternal();
    }
}

void SendEmailDialog::previewInternal()
{
    if(m_faktura.platce())
    {
        FakturaPlatcePrint print(this, m_faktura);
        print.Preview(false);
    }
    else
    {
        FakturaNeplatcePrint print(this, m_faktura);
        print.Preview(false);
    }
}

wxString SendEmailDialog::saveInternal()
{
    if(m_faktura.platce())
    {
        FakturaPlatcePrint print(this, m_faktura);
        return print.CreatePDF(m_faktura.cislo(), wxString::Format(wxString::FromUTF8("Faktura č. %s"), m_faktura.cislo()));
    }
    else
    {
        FakturaNeplatcePrint print(this, m_faktura);
        return print.CreatePDF(m_faktura.cislo(), wxString::Format(wxString::FromUTF8("Faktura č. %s"), m_faktura.cislo()));
    }
}

wxString SendEmailDialog::createPdf()
{
    bool faktura = m_faktura.typid() == 1;
    if(m_select)
    {
        wxArrayString choices;
        if(faktura && dxsettings.fakturaSablona() != "internal") choices.Add(wxString::FromUTF8("Poslední použitá šablona"));
        if(!faktura && dxsettings.zalohaSablona() != "internal") choices.Add(wxString::FromUTF8("Poslední použitá šablona"));
        choices.Add(wxString::FromUTF8("Výchozí šablona programu"));
        choices.Add("Ze souboru");
        wxSingleChoiceDialog vyber(this, wxString::FromUTF8("Vyberte typ šablony"), wxString::FromUTF8("Typ šablony"), choices);
        vyber.SetSelection(0);
        if(vyber.ShowModal() == wxID_OK)
        {
            if(vyber.GetStringSelection() == wxString::FromUTF8("Výchozí šablona programu"))
            {
                return saveInternal();
            }
            else if(vyber.GetStringSelection() == "Ze souboru")
            {
                wxFileDialog dialog(m_parent, wxString::FromUTF8("Otevřít šablonu"), wxGetHomeDir(), wxEmptyString, "XML soubor (*.xml)|*.xml", wxFD_OPEN|wxFD_FILE_MUST_EXIST);
                if(dialog.ShowModal() == wxID_OK)
                {
                    wxString path = dialog.GetPath();
                    if(!path.empty())
                    {
                        if(faktura) dxsettings.setFakturaSablona(path);
                        else dxsettings.setZalohaSablona(path);
                        FakturaXmlPrint print(this, m_faktura, path);
                        return print.CreatePDF(m_faktura.cislo(), wxString::Format(wxString::FromUTF8("Faktura č. %s"), m_faktura.cislo()));
                    }
                    else
                    {
                        return saveInternal();
                    }
                }
                else
                {
                    return saveInternal();
                }
            }
            else
            {
                if(wxFileExists(faktura ? dxsettings.fakturaSablona() : dxsettings.zalohaSablona()))
                {
                    FakturaXmlPrint print(this, m_faktura, faktura ? dxsettings.fakturaSablona() : dxsettings.zalohaSablona());
                    return print.CreatePDF(m_faktura.cislo(), wxString::Format(wxString::FromUTF8("Faktura č. %s"), m_faktura.cislo()));
                }
                else
                {
                    if(faktura) dxsettings.setFakturaSablona("internal");
                    else dxsettings.setZalohaSablona("internal");
                    return saveInternal();
                }
            }
        }
        else
        {
            return saveInternal();
        }
    }
    else
    {
        if((faktura && dxsettings.fakturaSablona() != "internal") || (!faktura && dxsettings.zalohaSablona() != "internal"))
        {
            if(wxFileExists(faktura ? dxsettings.fakturaSablona() : dxsettings.zalohaSablona()))
            {
                FakturaXmlPrint print(this, m_faktura, faktura ? dxsettings.fakturaSablona() : dxsettings.zalohaSablona());
                return print.CreatePDF(m_faktura.cislo(), wxString::Format(wxString::FromUTF8("Faktura č. %s"), m_faktura.cislo()));
            }
            else
            {
                if(faktura) dxsettings.setFakturaSablona("internal");
                else dxsettings.setZalohaSablona("internal");
                return saveInternal();
            }
        }
        else
        {
            return saveInternal();
        }
    }
}
