/***************************************************************
 * Name:      skladdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "skladdialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(SkladDialog, wxDialog)

BEGIN_EVENT_TABLE(SkladDialog, wxDialog)
    EVT_BUTTON(wxID_OK, SkladDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, SkladDialog::OnCancel)
END_EVENT_TABLE()

SkladDialog::SkladDialog(wxWindow *parent, int id, bool edit)
    : wxDialog(parent, wxID_ANY, edit ? "Sklad" : "Zadejte sklad"), m_edit(edit)
{
    createControls();
    if(m_edit)
    {
        m_id = id;
    }
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void SkladDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Název")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_nazev = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_nazev, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popis = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE);
    gridSizer->Add(m_popis, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Oceňování\nzásob")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ocenovani = new wxChoice(this, wxID_ANY);
    gridSizer->Add(m_ocenovani, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void SkladDialog::fillControls()
{
    m_ocenovani->Append("FIFO");
    m_ocenovani->Append(wxString::FromUTF8("Vážený aritmetický průměr"));
    if(m_edit)
    {
        Sklad sk = idbmodel->getSklad(m_id);
        if(sk.isValid())
        {
            m_nazev->SetValue(sk.nazev());
            m_popis->SetValue(sk.popis());
            m_ocenovani->SetSelection(sk.ocenovani());
        }
        else
        {
            m_ocenovani->SetSelection(0);
        }
    }
    else
    {
        m_ocenovani->SetSelection(0);
    }
}

void SkladDialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_nazev->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Název nemůže být prázdný"), "Informace", m_edit? wxOK|wxICON_INFORMATION : wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_nazev->SetFocus();
        return;
    }
    if(nazevExists())
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Název '%s' už existuje\nZvolte jiný"), m_nazev->GetValue()), "Informace", wxOK|wxICON_INFORMATION);
        dialog.ShowModal();
        return;
    }
    if(!m_edit)
    {
        idbmodel->addSklad(Sklad(idbmodel->getLastId("sklad"), m_nazev->GetValue(), m_popis->GetValue(), m_ocenovani->GetSelection()));
    }
    else
    {
        idbmodel->editSklad(Sklad(m_id, m_nazev->GetValue(), m_popis->GetValue(), m_ocenovani->GetSelection()));
    }
    EndModal(wxID_OK);
}

void SkladDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

bool SkladDialog::nazevExists()
{
    if(m_edit)
    {
        wxString nazev = idbmodel->hasSklad(m_id) ? idbmodel->getSklad(m_id).nazev() : "";
        if(nazev == m_nazev->GetValue())
            return false;
        else
            return idbmodel->hasSklad(m_nazev->GetValue());
    }
    else
    {
        return idbmodel->hasSklad(m_nazev->GetValue());
    }
}

