/***************************************************************
 * Name:      skladydialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "skladydialog.h"
#include "skladdialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"

IMPLEMENT_DYNAMIC_CLASS(SkladyDialog, wxDialog)

BEGIN_EVENT_TABLE(SkladyDialog, wxDialog)
    EVT_BUTTON(wxID_OK, SkladyDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, SkladyDialog::OnCancel)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_SKLADY_LIST, SkladyDialog::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_SKLADY_LIST, SkladyDialog::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_SKLADY_LIST, SkladyDialog::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_SKLADY_LIST, SkladyDialog::OnItemDeselected)
    EVT_MENU(ID_SKLADY_ADD, SkladyDialog::OnAddSklad)
    EVT_MENU(ID_SKLADY_EDIT, SkladyDialog::OnEditSklad)
    EVT_MENU(ID_SKLADY_DELETE, SkladyDialog::OnDeleteSklad)
END_EVENT_TABLE()

SkladyDialog::SkladyDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, "Sklady"), m_selectedId(-1)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_SKLADY_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat sklad"));
    m_bar->AddTool(ID_SKLADY_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, "Upravit sklad");
    m_bar->AddTool(ID_SKLADY_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, "Smazat sklad");
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    m_list = new wxListCtrl(this, ID_SKLADY_LIST, wxDefaultPosition, wxDefaultSize,
                            wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText(wxString::FromUTF8("Název"));
    col0.SetWidth(120);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText("Popis");
    col1.SetWidth(250);
    m_list->InsertColumn(1, col1);
    wxListItem col2;
    col2.SetId(2);
    col2.SetText(wxString::FromUTF8("Oceňování zásob"));
    col2.SetWidth(200);
    m_list->InsertColumn(2, col2);
    fillList();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void SkladyDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void SkladyDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void SkladyDialog::OnItemRightClick(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    wxMenu popup;
    popup.Append(ID_SKLADY_ADD, wxString::FromUTF8("Přidat sklad"));
    if(m_selectedId != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_SKLADY_EDIT, "Upravit sklad");
        popup.Append(ID_SKLADY_DELETE, "Smazat sklad");
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void SkladyDialog::OnItemActivated(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    SkladDialog dialog(this, m_selectedId, true);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void SkladyDialog::OnItemSelected(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void SkladyDialog::OnItemDeselected(wxListEvent& /*event*/)
{
    setSelectedId(-1);
}

void SkladyDialog::OnEditSklad(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    SkladDialog dialog(this, m_selectedId, true);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void SkladyDialog::OnAddSklad(wxCommandEvent& /*event*/)
{
    SkladDialog dialog(this, -1, false);
    if(dialog.ShowModal() == wxID_OK)
        fillList();
}

void SkladyDialog::OnDeleteSklad(wxCommandEvent& /*event*/)
{
    if(idbmodel->hasSklad(m_selectedId))
    {
        wxMessageDialog dialog(this, wxString::Format("Opravdu smazat sklad %s?", idbmodel->getSklad(m_selectedId).nazev()), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            idbmodel->deleteSklad(m_selectedId);
            fillList();
        }
    }
}

void SkladyDialog::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = -1;
    else
        m_selectedId = m_sklady[selectedIndex].id();
    enableTools();
}

void SkladyDialog::fillList()
{
    m_list->DeleteAllItems();
    m_list->Hide();
    m_sklady = idbmodel->sklady();
    for(size_t i=0; i < m_sklady.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(m_sklady[i].nazev());
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_sklady[i].popis());
        m_list->SetItem(static_cast<long>(i), 2, m_sklady[i].ocenovani() ? wxString::FromUTF8("Vážený aritmetický průměr") : "FIFO");
    }
    m_list->Show();
    setSelectedId(-1);
}

void SkladyDialog::enableTools()
{
    if(m_selectedId == -1)
    {
        m_bar->EnableTool(ID_SKLADY_EDIT, false);
        m_bar->EnableTool(ID_SKLADY_DELETE, false);
    }
    else
    {
        m_bar->EnableTool(ID_SKLADY_EDIT, true);
        m_bar->EnableTool(ID_SKLADY_DELETE, true);
    }
}
