/***************************************************************
 * Name:      UhradyDialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#ifndef UHRADYDIALOG_H
#define UHRADYDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/imaglist.h>
#include "gui/widgets/dxlistctrl.h"
#include "model/dbmodel.h"

class UhradyDialog;

class UhradyCtrl: public dxListCtrl
{
public:
    UhradyCtrl(UhradyDialog *dialog, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_dialog(dialog) {}

    enum UColumn
    {
        UCOL_DATUM = 0,
        UCOL_CISLO,
        UCOL_POPIS,
        UCOL_CASTKA,
        UCOL_MAX, // pocet sloupcu
        UCOL_DEF_SORT = UCOL_DATUM
    };

    void CreateColumns();
private:
    UhradyDialog *m_dialog;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class UhradyDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(UhradyDialog)
    DECLARE_EVENT_TABLE()
public:
    UhradyDialog() : m_fakturaid(-1), m_zavpohid(-1), m_mainSizer(nullptr), m_list(nullptr),
        m_sortCol(UhradyCtrl::UCOL_DATUM), m_sortAsc(true) {}
    UhradyDialog(wxWindow *parent, int fakturaid, int zavpohid);
    virtual ~UhradyDialog();

    wxString getItemText(long item, long column) const;
private:
    int m_fakturaid;
    int m_zavpohid;
    wxBoxSizer *m_mainSizer;
    UhradyCtrl *m_list;
    UhradyCtrl::UColumn m_sortCol;
    bool m_sortAsc;
    wxScopedPtr<wxImageList> m_imageList;
    dxDenikArray m_denik;

    void createControls();
    void updateData();
    void OnOk(wxCommandEvent& event);
    void OnColumnClick(wxListEvent& event);
    DbModel::DSort columnToDenikSort(UhradyCtrl::UColumn column);
    void setColumnImage(UhradyCtrl::UColumn column, int image);
};

#endif  /* UHRADYDIALOG_H */

