/***************************************************************
 * Name:      zalohadialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include "zalohadialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(ZalohaDialog, wxDialog)

BEGIN_EVENT_TABLE(ZalohaDialog, wxDialog)
    EVT_BUTTON(wxID_OK, ZalohaDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, ZalohaDialog::OnCancel)
END_EVENT_TABLE()

ZalohaDialog::ZalohaDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Záloha dat aplikace"))
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void ZalohaDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxTimeSpan span = wxDateTime::Today().Subtract(dxsettings.lastBackup());
    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::Format(wxString::FromUTF8("Zálohy dat aplikace jsou %d dní staré"), span.GetDays())), 0, wxALL|wxEXPAND, 5);
    wxButton* okButton = new wxButton(this, wxID_OK, wxString::FromUTF8("Zálohovat"), wxDefaultPosition, wxDefaultSize, 0);
    m_mainSizer->Add(okButton, 0, wxALL|wxEXPAND, 5);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, wxString::FromUTF8("Nezálohovat"), wxDefaultPosition, wxDefaultSize, 0);
    m_mainSizer->Add(cancelButton, 0, wxALL|wxEXPAND, 5);

    this->SetSizer(m_mainSizer);
}

void ZalohaDialog::OnOk(wxCommandEvent& /*event*/)
{
    wxDirDialog dialog(this, wxString::FromUTF8("Adresář pro zálohy"), dxsettings.backupPath(), wxDD_DEFAULT_STYLE & ~wxDD_DIR_MUST_EXIST);
    if(dialog.ShowModal() == wxID_OK)
    {
        wxString path = dialog.GetPath();
        if(dxutils::zalohaDat(path))
        {
            dxsettings.setLastBackup(wxDateTime::Today());
            dxsettings.setBackupPath(path);
            wxMessageBox(wxString::FromUTF8("Vytvoření zálohy proběhlo úspěšně"));
        }
        else
        {
            wxMessageBox(wxString::FromUTF8("Vytvoření zálohy se nezdařilo"));
        }
    }
    EndModal(wxID_OK);
}

void ZalohaDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}
