/***************************************************************
 * Name:      zasobydialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#ifndef ZASOBYDIALOG_H
#define ZASOBYDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/treectrl.h>
#include <wx/wxhtml.h>
#include "model/dbmodel.h"

class SkladZasobaItemData : public wxTreeItemData
{
public:
    SkladZasobaItemData(bool sklad, int skladid, int zasobaid) : m_sklad(sklad), m_skladid(skladid), m_zasobaid(zasobaid) {}
    ~SkladZasobaItemData() {}

    bool sklad() const { return m_sklad; }
    int skladid() const { return m_skladid; }
    int zasobaid() const { return m_zasobaid; }
private:
    bool m_sklad;
    int m_skladid;
    int m_zasobaid;
};

class HtmlPage: public wxPanel
{
    DECLARE_DYNAMIC_CLASS(HtmlPage)
public:
    HtmlPage() : m_mainSizer(nullptr), m_html(nullptr) {}
    explicit HtmlPage(wxWindow *parent);

    void updateSklad(int skladid);
    void updateZasoba(int zasobaid, const wxDateTime &datum);
private:
    wxBoxSizer *m_mainSizer;
    wxHtmlWindow *m_html;
};

class ZasobyDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(ZasobyDialog)
    DECLARE_EVENT_TABLE()
public:
    ZasobyDialog() : m_mainSizer(nullptr), m_navigation(nullptr), m_page(nullptr), m_ok(nullptr) {}
    ZasobyDialog(wxWindow *parent, const wxDateTime &datum);

    int zasobaId() const;
private:
    wxBoxSizer *m_mainSizer;
    wxTreeCtrl *m_navigation;
    HtmlPage *m_page;
    wxButton *m_ok;
    wxDateTime m_datum;

    void createControls();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnTreeChanged(wxTreeEvent& event);
};

#endif  /* ZASOBYDIALOG_H */

