/***************************************************************
 * Name:      zavpohdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "zavpohdialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "pokladnydialog.h"
#include "bankydialog.h"
#include "predkontacesdialog.h"
#include "kontaktydialog.h"
#include "polozkadialog.h"
#include "fauhradadialog.h"
#include "dphdruhydialog.h"
#include "resources/images/unchecked.xpm"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/download.xpm"
#include <wx/valnum.h>
#include <wx/numdlg.h>
#include "gui/prints/zavpohprint.h"

IMPLEMENT_DYNAMIC_CLASS(ZavpohDialog, wxDialog)

BEGIN_EVENT_TABLE(ZavpohDialog, wxDialog)
    EVT_BUTTON(wxID_OK, ZavpohDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, ZavpohDialog::OnCancel)
    EVT_BUTTON(ID_ZAVPOH_UHRADA_ADD, ZavpohDialog::OnAddUhrada)
    EVT_BUTTON(ID_ZAVPOH_BANKAPOKLADNA_SELECT, ZavpohDialog::OnSelectBankaPokladna)
    EVT_CHOICE(ID_ZAVPOH_UHRADA, ZavpohDialog::OnChoiceUhrada)
    EVT_CHOICE(ID_ZAVPOH_MENA, ZavpohDialog::OnChoiceMena)
    EVT_BUTTON(ID_ZAVPOH_PREDKONTACE_SELECT, ZavpohDialog::OnSelectPredkontace)
    EVT_BUTTON(ID_ZAVPOH_DPHDRUH_SELECT, ZavpohDialog::OnSelectDphDruh)
    EVT_BUTTON(ID_ZAVPOH_KURZ_DOWNLOAD, ZavpohDialog::OnKurzDownload)
    EVT_DATE_CHANGED(ID_ZAVPOH_VYSTAVENI, ZavpohDialog::OnVystaveniChanged)
    EVT_DATE_CHANGED(ID_ZAVPOH_DUZP, ZavpohDialog::OnDuzpChanged)
    EVT_DATE_CHANGED(ID_ZAVPOH_DPPD, ZavpohDialog::OnDppdChanged)
    EVT_BUTTON(ID_ZAVPOH_KONTAKT_SELECT, ZavpohDialog::OnSelectKontakt)
    EVT_BUTTON(ID_ZAVPOH_KONTAKT_CLEAR, ZavpohDialog::OnClearKontakt)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_ZAVPOH_POLOZKY, ZavpohDialog::OnPolozkaRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_ZAVPOH_POLOZKY, ZavpohDialog::OnPolozkaActivated)
    EVT_LIST_ITEM_SELECTED(ID_ZAVPOH_POLOZKY, ZavpohDialog::OnPolozkaSelected)
    EVT_LIST_ITEM_DESELECTED(ID_ZAVPOH_POLOZKY, ZavpohDialog::OnPolozkaDeselected)
    EVT_MENU(ID_ZAVPOH_POLOZKY_ADD, ZavpohDialog::OnAddPolozka)
    EVT_MENU(ID_ZAVPOH_POLOZKY_EDIT, ZavpohDialog::OnEditPolozka)
    EVT_MENU(ID_ZAVPOH_POLOZKY_DELETE, ZavpohDialog::OnDeletePolozka)
    EVT_TEXT(ID_ZAVPOH_KURZ, ZavpohDialog::OnKurzChanged)
    EVT_BUTTON(ID_ZAVPOH_PAUSAL_ADD, ZavpohDialog::OnPausalAdd)
    EVT_BUTTON(ID_ZAVPOH_ZAOKROUHLENI, ZavpohDialog::OnZaokrouhlit)
END_EVENT_TABLE()

//novy zavazek, pohledavka
ZavpohDialog::ZavpohDialog(bool zavazek, wxWindow *parent)
    : wxDialog(parent, wxID_ANY, ""), m_edit(0), m_zavazek(zavazek), m_polozkaSelected(-1), m_kurz(nullptr), m_kurzDownload(nullptr), m_kurzLabel(nullptr), m_kurzValue(1.0)
{
    m_id = idbmodel->getLastId("zavpoh");
    m_platce = idbmodel->isFirmaPlatce();
    idbmodel->addZavPoh(ZavPoh(m_id, 0, "", 0, "", "", 1, -1, -1, 1, 1, idbmodel->currentRok(),
                                   m_zavazek, m_platce, dxsettings.datum(), dxsettings.datum(), dxsettings.datum(), dxsettings.datum(), -1),
                        false);
    createControls();
    updateData();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    SetTitle(m_zavazek?wxString::FromUTF8("Závazek"):wxString::FromUTF8("Pohledávka"));
}

//editace zavazku, pohledavky
ZavpohDialog::ZavpohDialog(int id, wxWindow *parent, int edit)
    : wxDialog(parent, wxID_ANY, ""), m_id(id), m_edit(edit), m_kurz(nullptr), m_kurzDownload(nullptr), m_kurzLabel(nullptr)
{
    ZavPoh zp = idbmodel->getZavPoh(id);
    m_zavazek = zp.isValid() ? zp.zavazek() : false;
    m_platce = zp.isValid() ? zp.platce() : false;
    m_oldPolozky = idbmodel->fapolozkyByZavpohId(id);
    createControls();
    fillControls();
    updateData();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    SetTitle(m_zavazek?wxString::FromUTF8("Závazek"):wxString::FromUTF8("Pohledávka"));
}

wxString ZavpohDialog::getPolozkaText(long item, long column) const
{
    if(item < 0 || item >= static_cast<long>(m_polozky.GetCount()))
        return "";
    switch(column)
    {
    case ZpPolozkaCtrl::PCOL_POPIS:
        return m_polozky[static_cast<size_t>(item)].popis();
    case ZpPolozkaCtrl::PCOL_MNOZSTVI:
        return wxString::Format("%.3f", m_polozky[static_cast<size_t>(item)].pocet());
    case ZpPolozkaCtrl::PCOL_MJ:
        return m_polozky[static_cast<size_t>(item)].mj();
    case ZpPolozkaCtrl::PCOL_CENAMJ:
        return wxString::Format("%.2f", dxutils::cenamj(m_polozky[static_cast<size_t>(item)].cenamj(), m_polozky[static_cast<size_t>(item)].bezdph(), m_polozky[static_cast<size_t>(item)].sazbaid()));
    case ZpPolozkaCtrl::PCOL_CENACELKEM:
        return wxString::Format("%.2f", m_polozky[static_cast<size_t>(item)].celkem());
    default:
        return "";
    }
}

void ZavpohDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxBoxSizer *sizer = new wxBoxSizer(wxHORIZONTAL);
    sizer->Add(createZavpohGrid(), 0, wxALL, 5);
    wxBoxSizer *ksizer = new wxBoxSizer(wxVERTICAL);
    ksizer->Add(createKontaktGrid(), 0, wxALL, 5);
    if(m_platce)
    {
        createDphGrid();
        ksizer->Add(m_dph, 0, wxALL, 5);
    }
    sizer->Add(ksizer, 0, wxALL, 5);
    sizer->Add(createPolozkyGrid(), 0, wxALL|wxGROW, 5);
    m_mainSizer->Add(sizer, 0, wxALL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

wxGridBagSizer *ZavpohDialog::createZavpohGrid()
{
    int i=0; //row

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popis = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_popis, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Číslo")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cislo = new wxTextCtrl(this, wxID_ANY);
    m_cislo->SetValue(idbmodel->getCisloDokladu(m_zavazek?"ZA":"PO"));
    gridSizer->Add(m_cislo, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Variabilní symbol")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_vs = new wxTextCtrl(this, wxID_ANY);
    wxString vs = wxString::Format("%d", idbmodel->currentRok());
    wxString cislo = wxString::Format("%d", idbmodel->getRadaCislo(m_zavazek?"ZA":"PO"));
    int zeroes = 6-static_cast<int>(cislo.Length());
    for(int i=0; i<zeroes; i++)
        vs.Append('0');
    vs.Append(cislo);
    m_vs->SetValue(vs);
    gridSizer->Add(m_vs, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Forma úhrady")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_uhrada = new wxChoice(this, ID_ZAVPOH_UHRADA);
    dxFaUhradaArray uhrady = idbmodel->fauhrady();
    for(size_t j=0; j<uhrady.GetCount(); j++)
    {
        m_uhrada->Append(uhrady[j].uhrada(), new FaUhrada(uhrady[j].id(), uhrady[j].uhrada(), uhrady[j].pokladnou()));
    }
    m_uhrada->Select(0);
    gridSizer->Add(m_uhrada, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_uhradaAdd = new wxButton(this, ID_ZAVPOH_UHRADA_ADD, "...");
    m_uhradaAdd->SetToolTip(wxString::FromUTF8("Přidat úhradu"));
    gridSizer->Add(m_uhradaAdd, wxGBPosition(i,2));
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, m_zavazek?"Uhradit z":"Uhradit na"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_bankapokladnaText = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_bankapokladnaText, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_bankapokladnaSelect = new wxButton(this, ID_ZAVPOH_BANKAPOKLADNA_SELECT, "...");
    gridSizer->Add(m_bankapokladnaSelect, wxGBPosition(i,2));
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Předkontace")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_predkontaceText = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_predkontaceText, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_predkontaceSelect = new wxButton(this, ID_ZAVPOH_PREDKONTACE_SELECT, "...");
    m_predkontaceSelect->SetToolTip(wxString::FromUTF8("Vyberte předkontaci"));
    gridSizer->Add(m_predkontaceSelect, wxGBPosition(i,2));
    i++;

    if(!m_zavazek && idbmodel->isFirmaPausal())
    {
        gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Paušální výdaj %")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        m_pausal = new wxChoice(this, wxID_ANY);
        m_pausal->Append("80");
        m_pausal->Append("60");
        m_pausal->Append("40");
        m_pausal->Append("30");
        m_pausal->Append("0");
        m_pausal->Select(1);
        gridSizer->Add(m_pausal, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
        m_pausalAdd = new wxButton(this, ID_ZAVPOH_PAUSAL_ADD, "...");
        m_pausalAdd->SetToolTip(wxString::FromUTF8("Přidat hodnotu paušálních výdajů"));
        gridSizer->Add(m_pausalAdd, wxGBPosition(i,2));
        i++;
    }

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Datum vystavení")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_vystaveni  = new wxDatePickerCtrl(this, ID_ZAVPOH_VYSTAVENI, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_vystaveni->SetRange(wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok()),wxDateTime(31,wxDateTime::Dec,idbmodel->currentRok()));
    gridSizer->Add(m_vystaveni, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    if(m_platce)
    {
        gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Datum usk.zdan.plnění")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        m_duzp  = new wxDatePickerCtrl(this, ID_ZAVPOH_DUZP, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
        m_duzp->SetRange(wxDateTime(1,wxDateTime::Dec,idbmodel->currentRok()-1),wxDateTime(31,wxDateTime::Jan,idbmodel->currentRok()+1));
        gridSizer->Add(m_duzp, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
        i++;
        if(m_zavazek)
        {
            gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Datum povinnosti přiznat daň")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
            m_dppd  = new wxDatePickerCtrl(this, ID_ZAVPOH_DPPD, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
            m_dppd->SetRange(wxDateTime(1,wxDateTime::Dec,idbmodel->currentRok()-1),wxDateTime(31,wxDateTime::Jan,idbmodel->currentRok()+1));
            gridSizer->Add(m_dppd, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
            i++;
        }
        gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Členění DPH")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        m_dphdruhText = new wxTextCtrl(this, wxID_ANY);
        gridSizer->Add(m_dphdruhText, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
        m_dphdruhSelect = new wxButton(this, ID_ZAVPOH_DPHDRUH_SELECT, "...");
        m_dphdruhSelect->SetToolTip(wxString::FromUTF8("Vyberte členění DPH"));
        gridSizer->Add(m_dphdruhSelect, wxGBPosition(i,2));
        i++;
    }

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Datum splatnosti"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_splatnost  = new wxDatePickerCtrl(this, wxID_ANY, dxsettings.datum()+wxDateSpan(0,0,0,dxsettings.splatnost()), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_splatnost->SetRange(wxDateTime(1,wxDateTime::Dec,idbmodel->currentRok()-1),wxDateTime(31,wxDateTime::Jan,idbmodel->currentRok()+1));
    gridSizer->Add(m_splatnost, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Měna")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mena = new wxChoice(this, ID_ZAVPOH_MENA);
    gridSizer->Add(m_mena, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    if(m_platce)
    {
        m_kurzLabel = new wxStaticText(this, wxID_ANY, "Kurz");
        gridSizer->Add(m_kurzLabel, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
        m_kurz = new wxTextCtrl(this, ID_ZAVPOH_KURZ, "", wxDefaultPosition, wxDefaultSize, wxTE_RIGHT,
                                    wxMakeFloatingPointValidator(6, &m_kurzValue, wxNUM_VAL_THOUSANDS_SEPARATOR));

        m_kurzDownload = new wxBitmapButton(this, ID_ZAVPOH_KURZ_DOWNLOAD, wxBitmap(download_xpm));
        m_kurzDownload->SetBitmapDisabled(dxutils::GreyScaledIcon(download_xpm));
        m_kurzDownload->SetToolTip(wxString::FromUTF8("Stáhnout kurz z webu ČNB"));
#if wxUSE_URL
        gridSizer->Add(m_kurz, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
        gridSizer->Add(m_kurzDownload, wxGBPosition(i,2));
#else
        gridSizer->Add(m_kurz, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
#endif
        i++;
    }

    wxStaticText *text = new wxStaticText(this, wxID_ANY, "Celkem");
    text->SetFont(this->GetFont().Bold());
    gridSizer->Add(text, wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_celkem = new wxStaticText(this, wxID_ANY, wxString::Format("%.2f",0.0));
    m_celkem->SetFont(this->GetFont().Bold());
    gridSizer->Add(m_celkem, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);
    i++;

    m_zaokrouhleni = new wxButton(this, ID_ZAVPOH_ZAOKROUHLENI, "Zaokrouhlit");
    m_zaokrouhleni->Disable();
    gridSizer->Add(m_zaokrouhleni, wxGBPosition(i,1), wxGBSpan(1,2), wxEXPAND);

    gridSizer->SetEmptyCellSize(wxSize(0,0));

    if(!m_edit)
    {
        fillMena();
        showhideKurz();
    }

    return gridSizer;
}

wxStaticBoxSizer *ZavpohDialog::createKontaktGrid()
{
    int i=0; //row

    wxStaticBoxSizer *box = new wxStaticBoxSizer(wxVERTICAL, this, "Protistrana");

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Název")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_nazev = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_nazev, wxGBPosition(i,1), wxGBSpan(1,3), wxEXPAND);
    m_kontaktSelect = new wxButton(this, ID_ZAVPOH_KONTAKT_SELECT, "...");
    gridSizer->Add(m_kontaktSelect, wxGBPosition(i,4));
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Obchodní jméno")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_jmeno = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_jmeno, wxGBPosition(i,1), wxGBSpan(1,3), wxEXPAND);
    m_kontaktClear = new wxBitmapButton(this, ID_ZAVPOH_KONTAKT_CLEAR, wxBitmap(unchecked_xpm));
    m_kontaktClear->SetBitmapDisabled(dxutils::GreyScaledIcon(unchecked_xpm));
    m_kontaktClear->Disable();
    gridSizer->Add(m_kontaktClear, wxGBPosition(i,4));
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Ulice"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ulice = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_ulice, wxGBPosition(i,1));
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("čp")), wxGBPosition(i,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cp = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_cp, wxGBPosition(i,3));
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("PSČ")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_psc = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_psc, wxGBPosition(i,1));
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Místo")), wxGBPosition(i,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_misto = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_misto, wxGBPosition(i,3));
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Země")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zeme = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_zeme, wxGBPosition(i,1), wxGBSpan(1,3), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("IČ")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ic = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_ic, wxGBPosition(i,1), wxGBSpan(1,3), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("DIČ")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dic = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_dic, wxGBPosition(i,1), wxGBSpan(1,3), wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "E-mail"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_email = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_email, wxGBPosition(i,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->SetEmptyCellSize(wxSize(0,0));

    box->Add(gridSizer, 1, wxGROW);

    return box;
}

wxStaticBoxSizer *ZavpohDialog::createPolozkyGrid()
{
    wxStaticBoxSizer *box = new wxStaticBoxSizer(wxVERTICAL, this, wxString::FromUTF8("Položky"));

    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
    m_polozkaBar = new dxToolBar(this);
    m_polozkaBar->AddTool(ID_ZAVPOH_POLOZKY_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat položku"));
    m_polozkaBar->AddTool(ID_ZAVPOH_POLOZKY_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit položku"));
    m_polozkaBar->AddTool(ID_ZAVPOH_POLOZKY_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat položku"));
    m_polozkaBar->Realize();
    sizer->Add(m_polozkaBar, 0, wxALL|wxEXPAND, 5);
    m_polozkyList = new ZpPolozkaCtrl(this, this, ID_ZAVPOH_POLOZKY);
    m_polozkyList->CreateColumns();
    enablePolozkyTools();
    sizer->Add(m_polozkyList, 1, wxALL|wxEXPAND, 5);

    box->Add(sizer, 1, wxGROW);

    return box;
}

void ZavpohDialog::createDphGrid()
{
    m_dph = new wxListCtrl(this, wxID_ANY, wxDefaultPosition, wxDefaultSize,
                           wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    m_dph->InsertColumn(0, "Sazba DPH");
    m_dph->InsertColumn(1, wxString::FromUTF8("Základ"), wxLIST_FORMAT_RIGHT);
    m_dph->InsertColumn(2, "DPH", wxLIST_FORMAT_RIGHT);
    m_dph->InsertColumn(3, wxString::FromUTF8("Celkem vč. DPH"), wxLIST_FORMAT_RIGHT);
    m_dph->SetColumnWidth(0, 100);
    m_dph->SetColumnWidth(1, 140);
    m_dph->SetColumnWidth(2, 140);
    m_dph->SetColumnWidth(3, 140);
}

void ZavpohDialog::fillControls()
{
    ZavPoh zp = idbmodel->getZavPoh(m_id);
    if(zp.isValid())
    {
        m_popis->SetValue(zp.popis());
        m_cislo->SetValue(zp.cislo());
        m_vs->SetValue(zp.vs());
        bool pokladnou = true;
        for(unsigned int i=0; i<m_uhrada->GetCount(); i++)
        {
            if(zp.uhradaid() == static_cast<FaUhrada*>(m_uhrada->GetClientData(i))->id())
            {
                pokladnou = static_cast<FaUhrada*>(m_uhrada->GetClientData(i))->pokladnou();
                m_uhrada->Select(static_cast<int>(i));
                break;
            }
        }
        m_bankapokladnaText->SetValue(pokladnou?idbmodel->getPokladnaZkratka(zp.pokladnaid()):idbmodel->getBankaZkratka(zp.bankaid()));
        m_predkontaceText->SetValue(idbmodel->hasPredkontace(zp.predkontaceid())?idbmodel->getPredkontace(zp.predkontaceid()).zkratka():"");
        m_vystaveni->SetValue(zp.vystaveni());
        if(m_platce)
        {
            m_duzp->SetValue(zp.duzp());
            if(m_zavazek) m_dppd->SetValue(zp.dppd());
            m_dphdruhText->SetValue(idbmodel->hasDphDruh(zp.dphdruhid())?idbmodel->getDphDruh(zp.dphdruhid()).zkratka():"");
        }
        m_kurzValue = m_platce?zp.kurz():1.0;
        m_splatnost->SetValue(zp.splatnost());
        Kontakt ko = idbmodel->getKontakt(zp.protistranaid());
        if(ko.isValid())
        {
            Adresa ad = idbmodel->getAdresa(ko.adresaid());
            if(ad.isValid())
            {
                m_nazev->SetValue(ko.nazev());
                m_jmeno->SetValue(ko.jmeno());
                m_ulice->SetValue(ad.ulice());
                m_cp->SetValue(ad.cp());
                m_misto->SetValue(ad.misto());
                m_psc->SetValue(ad.psc());
                m_zeme->SetValue(ad.zeme());
                m_ic->SetValue(ko.ic());
                m_dic->SetValue(ko.dic());
                m_email->SetValue(ko.email());
            }
        }
        if(zp.protistranaid()) enableDisableKontakt();
        if(!m_zavazek && idbmodel->isFirmaPausal())
        {
            wxString value(wxString::Format("%d", zp.pausal()));
            if(m_pausal->FindString(value) == wxNOT_FOUND)
            {
                wxArrayString values = m_pausal->GetStrings();
                m_pausal->Clear();
                values.Add(value);
                values.Sort(true);
                m_pausal->Append(values);
                for(size_t i=0; i<values.GetCount(); i++)
                {
                    if(values[i] == value)
                    {
                        m_pausal->SetSelection(static_cast<int>(i));
                        break;
                    }
                }
            }
            else
            {
                m_pausal->SetSelection(m_pausal->FindString(value));
            }
        }
    }
    fillMena();
    if(m_kurz)
    {
        wxDateTime datum = m_platce?(m_zavazek?m_dppd->GetValue():m_duzp->GetValue()):m_vystaveni->GetValue();
        if(idbmodel->hasKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), datum)) m_kurzDownload->Disable();
    }
    showhideKurz();
    updateData();
}

void ZavpohDialog::fillMena()
{
    m_mena->Clear();
    if(m_edit)
    {
        ZavPoh zp = idbmodel->getZavPoh(m_id);
        if(zp.isValid())
        {
            Mena mn = idbmodel->getMena(zp.menaid());
            m_mena->Append(idbmodel->nazevMena(zp.menaid()), new Mena(mn.id(), mn.kod(), mn.nazev()));
        }
        else
        {
            dxMenaArray meny = idbmodel->meny();
            for(size_t i=0; i<meny.GetCount(); i++)
                m_mena->Append(idbmodel->nazevMena(meny[i].id()), new Mena(meny[i].id(), meny[i].kod(), meny[i].nazev()));
        }
    }
    else
    {
        bool pokladnou = static_cast<FaUhrada*>(m_uhrada->GetClientData(static_cast<unsigned int>(m_uhrada->GetSelection())))->pokladnou();
        if(pokladnou)
        {
            Pokladna pk = idbmodel->getPokladna(m_bankapokladnaText->GetValue());
            if(pk.isValid() && pk.menaid() != 1)
            {
                Mena mn = idbmodel->getMena(pk.menaid());
                m_mena->Append(idbmodel->nazevMena(pk.menaid()), new Mena(mn.id(), mn.kod(), mn.nazev()));
            }
            else
            {
                dxMenaArray meny = idbmodel->meny();
                for(size_t i=0; i<meny.GetCount(); i++)
                    m_mena->Append(idbmodel->nazevMena(meny[i].id()), new Mena(meny[i].id(), meny[i].kod(), meny[i].nazev()));
            }
        }
        else
        {
            Banka bk = idbmodel->getBanka(m_bankapokladnaText->GetValue());
            if(bk.isValid() && bk.menaid() != 1)
            {
                Mena mn = idbmodel->getMena(bk.menaid());
                m_mena->Append(idbmodel->nazevMena(bk.menaid()), new Mena(mn.id(), mn.kod(), mn.nazev()));
            }
            else
            {
                dxMenaArray meny = idbmodel->meny();
                for(size_t i=0; i<meny.GetCount(); i++)
                    m_mena->Append(idbmodel->nazevMena(meny[i].id()), new Mena(meny[i].id(), meny[i].kod(), meny[i].nazev()));
            }
        }
    }
    m_mena->Select(0);
}

int ZavpohDialog::getMenaId() const
{
    return static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->id();
}

int ZavpohDialog::getMenaId(unsigned int index) const
{
    return static_cast<Mena*>(m_mena->GetClientData(index))->id();
}

void ZavpohDialog::showhideKurz()
{
    if(m_kurz)
    {
        if(getMenaId() == 1)
        {
            m_kurzLabel->Hide();
            m_kurz->Hide();
#if wxUSE_URL
            m_kurzDownload->Hide();
#endif
        }
        else
        {
            m_kurzLabel->Show();
            m_kurz->Show();
#if wxUSE_URL
            m_kurzDownload->Show();
#endif
        }
        m_mainSizer->Layout();
        m_mainSizer->Fit(this);
    }
}

void ZavpohDialog::OnKurzDownload(wxCommandEvent &/*event*/)
{
#if wxUSE_URL
    wxDateTime datum = m_platce?(m_zavazek?m_dppd->GetValue():m_duzp->GetValue()):m_vystaveni->GetValue();
    if(idbmodel->hasKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), datum))
        return;
    dxutils::kurzyDownload(datum);
    m_kurzDownload->Disable();
    m_kurzValue = idbmodel->getKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), datum).hodnota();
    m_kurz->GetValidator()->TransferToWindow();
    updateDph();
#endif
}

void ZavpohDialog::OnKurzChanged(wxCommandEvent &/*event*/)
{
    m_kurz->GetValidator()->TransferFromWindow();
    updateDph();
}

void ZavpohDialog::OnPausalAdd(wxCommandEvent &/*event*/)
{
    long pausal = wxGetNumberFromUser(wxString::FromUTF8("Nová hodnota paušálních výdajů"), "%", wxString::FromUTF8("Paušální výdaj"), 1, 1);
    if(pausal == -1)
    {
        return;
    }
    wxString value(wxString::Format("%lu", pausal));
    if(m_pausal->FindString(value) == wxNOT_FOUND)
    {
        wxArrayString values = m_pausal->GetStrings();
        m_pausal->Clear();
        values.Add(value);
        values.Sort(true);
        m_pausal->Append(values);
        for(size_t i=0; i<values.GetCount(); i++)
        {
            if(values[i] == value)
            {
                m_pausal->SetSelection(static_cast<int>(i));
                break;
            }
        }
    }
}

void ZavpohDialog::printZavpoh()
{
    ZavpohPrint print(this->GetParent(), idbmodel->getZavPoh(m_id));
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

//Jen pro EET pokladnou
void ZavpohDialog::writeUhrada()
{
    double dph = m_platce?idbmodel->dphZavPoh(m_id):0.0;
    double celkem = 0.0;
    for(size_t i=0; i<m_polozky.GetCount(); i++)
        celkem += m_polozky[i].celkem();
    //z celkem zaklad dph
    celkem -= dph;
    //write zaklad - start
    wxString cislodokladu = idbmodel->getCisloDokladu("PP");
    int protistranaid = 0;
    if(!m_nazev->GetValue().IsEmpty() && idbmodel->hasKontakt(m_nazev->GetValue()))
    {
        protistranaid = idbmodel->getKontaktIdByNazev(m_nazev->GetValue());
    }
    int pausal = 0;
    if(idbmodel->isFirmaPausal())
    {
        pausal = wxAtoi(m_pausal->GetStringSelection());
    }
    wxString popis = wxString::Format(wxString::FromUTF8("Úhrada pohledávky %s"), m_cislo->GetValue());
    int iddenik = idbmodel->getLastId("denik");
    idbmodel->addDenik(Denik(iddenik, m_vystaveni->GetValue(), cislodokladu, popis, getMenaId()==1?celkem:celkem*m_kurzValue, celkem, m_kurzValue, m_cislo->GetValue(),
                             getMenaId(), idbmodel->getPredkontaceIdByZkratka(m_predkontaceText->GetValue()), idbmodel->getPokladnaIdByZkratka(m_bankapokladnaText->GetValue()),
                             -1, -1, protistranaid, idbmodel->currentRok(), pausal));
    idbmodel->addUhrada(Uhrada(idbmodel->getLastId("uhrada"), celkem, iddenik, -1, m_id, idbmodel->currentRok()));
    idbmodel->increaseRadaCislo("PP");
    //write zaklad - end
    if(dxutils::isSameDouble(dph, 0.0))
        return;
    //write dph - start
    cislodokladu = idbmodel->getCisloDokladu("PP");
    popis = wxString::FromUTF8("DPH příjem");
    iddenik = idbmodel->getLastId("denik");
    idbmodel->addDenik(Denik(iddenik, m_vystaveni->GetValue(), cislodokladu, popis, getMenaId()==1?dph:dph*m_kurzValue, dph, m_kurzValue, m_cislo->GetValue(),
                             getMenaId(), idbmodel->getPredkontaceIdByZkratka("P102"), idbmodel->getPokladnaIdByZkratka(m_bankapokladnaText->GetValue()),
                             -1, -1, protistranaid, idbmodel->currentRok()));
    idbmodel->addUhrada(Uhrada(idbmodel->getLastId("uhrada"), dph, iddenik, -1, m_id, idbmodel->currentRok()));
    idbmodel->increaseRadaCislo("PP");
    //write dph - end
}

// true pro smazani uhrad
bool ZavpohDialog::isEdited()
{
    if(m_edit == 0 || m_edit == 2)
    {
        return false;
    }
    if(idbmodel->someUhradaForZavpoh(m_id))
    {
        bool pokladna = static_cast<FaUhrada*>(m_uhrada->GetClientData(static_cast<unsigned int>(m_uhrada->GetSelection())))->pokladnou();
        ZavPoh zp = idbmodel->getZavPoh(m_id);
        if(m_cislo->GetValue() != zp.cislo()) return true;
        if(getMenaId() != zp.menaid()) return true;
        if(static_cast<FaUhrada*>(m_uhrada->GetClientData(static_cast<unsigned int>(m_uhrada->GetSelection())))->id() != zp.uhradaid()) return true;
        if(pokladna && idbmodel->getPokladnaIdByZkratka(m_bankapokladnaText->GetValue()) != zp.pokladnaid()) return true;
        if(!pokladna && idbmodel->getBankaIdByZkratka(m_bankapokladnaText->GetValue()) != zp.bankaid()) return true;
        if(idbmodel->getPredkontaceIdByZkratka(m_predkontaceText->GetValue()) != zp.predkontaceid()) return true;
        if(!m_vystaveni->GetValue().IsSameDate(zp.vystaveni())) return true;
        if(m_platce)
        {
            if(m_zavazek)
            {
                if(!m_dppd->GetValue().IsSameDate(zp.duzp())) return true;
            }
            else
            {
                if(!m_duzp->GetValue().IsSameDate(zp.dppd())) return true;
            }
            if(idbmodel->getDphDruhIdByZkratka(m_dphdruhText->GetValue()) != zp.dphdruhid()) return true;
        }
        int pausal = 0;
        if(!m_zavazek && idbmodel->isFirmaPausal())
        {
            pausal = wxAtoi(m_pausal->GetStringSelection());
        }
        if(pausal != zp.pausal()) return true;
        if(!dxutils::isSameDouble(m_kurzValue,zp.kurz())) return true;
        double celkem = 0.0, dph = 0.0;
        for(size_t i=0; i<m_polozky.GetCount(); i++)
        {
            celkem += m_polozky[i].celkem();
            dph += m_polozky[i].dph();
        }
        double oldCelkem = 0.0, oldDph = 0.0;
        for(size_t i=0; i<m_oldPolozky.GetCount(); i++)
        {
            oldCelkem += m_oldPolozky[i].celkem();
            oldDph += m_oldPolozky[i].dph();
        }
        if(!dxutils::isSameDouble(celkem,oldCelkem)) return true;
        if(!dxutils::isSameDouble(dph,oldDph)) return true;
    }
    return false;
}

void ZavpohDialog::OnZaokrouhlit(wxCommandEvent &/*event*/)
{
    if(m_platce)
    {
        double celkem = 0.0;
        int idZaokrouleni = idbmodel->getLastId("fapolozka");
        bool pokladna = static_cast<FaUhrada*>(m_uhrada->GetClientData(m_uhrada->GetSelection()))->pokladnou();
        int sazba = 0;
        for(size_t i=0; i<m_polozky.GetCount(); i++)
        {
            if(!m_polozky[i].zaokrouhleni())
            {
                celkem += m_polozky[i].celkem();
                sazba = wxMax(sazba, idbmodel->getDphSazbaValue(m_polozky[i].sazbaid()));
            }
            else idZaokrouleni = m_polozky[i].id();
        }
        if(sazba == 0 && !pokladna) sazba = idbmodel->zakladniSazba();
        if(pokladna) sazba = 0;
        if(dxutils::isSameDouble(ceil(celkem),celkem))
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Zaokrouhlení není potřeba"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            idbmodel->deleteFaPolozka(idZaokrouleni);
            updateData();
            return;
        }
        double zaokrouhleni = ceil(celkem)-celkem;
        double dph = 0.0;
        if(!m_zavazek)
        {
            if(m_duzp->GetValue().IsLaterThan(wxDateTime(31, wxDateTime::Mar, 2019))) dph = zaokrouhleni*(sazba/(100.0+sazba));
            else dph = zaokrouhleni*round4(sazba/(100.0+sazba));
        }
        else
        {
            if(m_dppd->GetValue().IsLaterThan(wxDateTime(31, wxDateTime::Mar, 2019))) dph = zaokrouhleni*(sazba/(100.0+sazba));
            else dph = zaokrouhleni*round4(sazba/(100.0+sazba));
        }
        if(idbmodel->hasFaPolozka(idZaokrouleni))
        {
            idbmodel->editFaPolozka(FaPolozka(idZaokrouleni, wxString::FromUTF8("Zaokrouhlení"), wxEmptyString, 1.0, zaokrouhleni, idbmodel->getDphSazbaId(sazba),
                                              dph, zaokrouhleni, false, -1, m_id, -1, true));
        }
        else
        {
            idbmodel->addFaPolozka(FaPolozka(idZaokrouleni, wxString::FromUTF8("Zaokrouhlení"), wxEmptyString, 1.0, zaokrouhleni, idbmodel->getDphSazbaId(sazba),
                                             dph, zaokrouhleni, false, -1, m_id, -1, true));
        }
    }
    else
    {
        double celkem = 0.0;
        int idZaokrouleni = idbmodel->getLastId("fapolozka");
        for(size_t i=0; i<m_polozky.GetCount(); i++)
        {
            if(!m_polozky[i].zaokrouhleni()) celkem += m_polozky[i].celkem();
            else idZaokrouleni = m_polozky[i].id();
        }
        if(dxutils::isSameDouble(ceil(celkem),celkem))
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Zaokrouhlení není potřeba"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            idbmodel->deleteFaPolozka(idZaokrouleni);
            updateData();
            return;
        }
        double zaokrouhleni = ceil(celkem)-celkem;
        if(idbmodel->hasFaPolozka(idZaokrouleni))
        {
            idbmodel->editFaPolozka(FaPolozka(idZaokrouleni, wxString::FromUTF8("Zaokrouhlení"), wxEmptyString, 1.0, zaokrouhleni, 0,
                                              0.0, zaokrouhleni, true, -1, m_id, -1, true));
        }
        else
        {
            idbmodel->addFaPolozka(FaPolozka(idZaokrouleni, wxString::FromUTF8("Zaokrouhlení"), wxEmptyString, 1.0, zaokrouhleni, 0,
                                             0.0, zaokrouhleni, true, -1, m_id, -1, true));
        }
    }
    updateData();
}

void ZavpohDialog::OnOk(wxCommandEvent& event)
{
    bool pokladna = static_cast<FaUhrada*>(m_uhrada->GetClientData(m_uhrada->GetSelection()))->pokladnou();
    if(m_bankapokladnaText->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Úhrada nemůže být prázdná"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectBankaPokladna(event);
        return;
    }
    if(pokladna && !idbmodel->hasPokladna(m_bankapokladnaText->GetValue()))
    {
        wxMessageDialog dialog(this, wxString::Format("Pokladna %s neexistuje", m_bankapokladnaText->GetValue()), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectBankaPokladna(event);
        return;
    }
    if(!pokladna && !idbmodel->hasBanka(m_bankapokladnaText->GetValue()))
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Bankovní účet %s neexistuje"), m_bankapokladnaText->GetValue()), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectBankaPokladna(event);
        return;
    }
    if(m_predkontaceText->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Předkontace nemůže být prázdná"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectPredkontace(event);
        return;
    }
    if(!idbmodel->hasPredkontace(m_predkontaceText->GetValue()))
    {
        wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Předkontace %s neexistuje"), m_predkontaceText->GetValue()), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        OnSelectPredkontace(event);
        return;
    }
    if(m_platce)
    {
        if(m_dphdruhText->GetValue().IsEmpty())
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Členění DPH nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            OnSelectDphDruh(event);
            return;
        }
        if(!idbmodel->hasDphDruh(m_dphdruhText->GetValue()))
        {
            wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Členění DPH %s neexistuje"), m_dphdruhText->GetValue()), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            OnSelectDphDruh(event);
            return;
        }
        if(!idbmodel->isDphDruhValidForDatum(m_dphdruhText->GetValue(), m_zavazek?m_dppd->GetValue():m_duzp->GetValue()))
        {
            wxString text;
            if(m_zavazek) text = wxString::Format(wxString::FromUTF8("Členění DPH %s není platné pro DPPD %s"), m_dphdruhText->GetValue(), m_dppd->GetValue().FormatDate());
            else text = wxString::Format(wxString::FromUTF8("Členění DPH %s není platné pro DUZP %s"), m_dphdruhText->GetValue(), m_duzp->GetValue().FormatDate());
            wxMessageDialog dialog(this, text, "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            OnSelectDphDruh(event);
            return;
        }
    }
    if(isEdited())
    {
        wxMessageDialog dialog(this, m_zavazek?wxString::FromUTF8("Závazek byl změněn\nPoužít nové hodnoty?\n(Úhrady budou smazány)"):
                                               wxString::FromUTF8("Pohledávka byla změněna\nPoužít nové hodnoty?\n(Úhrady budou smazány)"),
                               "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            idbmodel->deleteUhradaByZavpohId(m_id);
        }
        else
        {
            OnCancel(event);
            return;
        }
    }
    for(size_t i=0; i<m_polozky.GetCount(); i++)
    {
        if(m_polozky[i].zaokrouhleni())
        {
            OnZaokrouhlit(event);
        }
    }
    int protistranaid = 0;
    if(!m_nazev->GetValue().IsEmpty())
    {
        if(idbmodel->hasKontakt(m_nazev->GetValue()))
        {
            protistranaid = idbmodel->getKontaktIdByNazev(m_nazev->GetValue());
        }
        else
        {
            protistranaid = idbmodel->getLastId("kontakt");
            int adresaid = idbmodel->getLastId("adresa");
            idbmodel->addAdresa(Adresa(adresaid, m_ulice->GetValue(), m_cp->GetValue(), m_misto->GetValue(), m_psc->GetValue(), m_zeme->GetValue()));
            idbmodel->addKontakt(Kontakt(protistranaid, m_nazev->GetValue(), m_jmeno->GetValue(), m_ic->GetValue(), m_dic->GetValue(), m_email->GetValue(), true, adresaid));
        }
    }
    int pausal = 0;
    if(!m_zavazek && idbmodel->isFirmaPausal())
    {
        pausal = wxAtoi(m_pausal->GetStringSelection());
    }
    double celkem = 0.0;
    for(size_t i=0; i<m_polozky.GetCount(); i++)
        celkem += m_polozky[i].celkem();
    idbmodel->editZavPoh(ZavPoh(m_id, protistranaid, m_popis->GetValue(), celkem, m_cislo->GetValue(), m_vs->GetValue(),
                                  static_cast<FaUhrada*>(m_uhrada->GetClientData(static_cast<unsigned int>(m_uhrada->GetSelection())))->id(),
                                  pokladna?idbmodel->getPokladnaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                  !pokladna?idbmodel->getBankaIdByZkratka(m_bankapokladnaText->GetValue()):-1,
                                  getMenaId(),
                                  idbmodel->getPredkontaceIdByZkratka(m_predkontaceText->GetValue()),
                                  idbmodel->currentRok(), m_zavazek, m_platce, m_vystaveni->GetValue(),
                                  m_platce?m_duzp->GetValue():m_vystaveni->GetValue(),
                                  m_platce?(m_zavazek?m_dppd->GetValue():m_duzp->GetValue()):m_vystaveni->GetValue(),
                                  m_splatnost->GetValue(),m_platce?idbmodel->getDphDruhIdByZkratka(m_dphdruhText->GetValue()):-1, m_kurzValue, pausal));
    if(m_edit == 0 || m_edit == 2)
    {
        if(m_zavazek) idbmodel->increaseRadaCislo("ZA");
        else idbmodel->increaseRadaCislo("PO");
    }
    EndModal(wxID_OK);
}

void ZavpohDialog::OnCancel(wxCommandEvent& /*event*/)
{
    if(m_edit == 0 || m_edit == 2)
    {
        idbmodel->deleteZavPoh(m_id, m_edit);
    }
    else
    {
        bool polozkySame = true;
        if(m_polozky.GetCount() == m_oldPolozky.GetCount())
        {
            idbmodel->sortFaPolozky(m_polozky);
            idbmodel->sortFaPolozky(m_oldPolozky);
            for(size_t i=0; i<m_oldPolozky.GetCount(); i++)
            {
                if(m_polozky[i] != m_oldPolozky[i])
                {
                    polozkySame = false;
                    break;
                }
            }
        }
        else
        {
            polozkySame = false;
        }
        if(!polozkySame)
        {
            idbmodel->deleteFaPolozkyByZavpohId(m_id);
            for(size_t i=0; i<m_oldPolozky.GetCount(); i++)
            {
                idbmodel->addFaPolozka(FaPolozka(m_oldPolozky[i].id(), m_oldPolozky[i].popis(), m_oldPolozky[i].mj(), m_oldPolozky[i].pocet(), m_oldPolozky[i].cenamj(),
                                                 m_oldPolozky[i].sazbaid(), m_oldPolozky[i].dph(), m_oldPolozky[i].celkem(), m_oldPolozky[i].bezdph(), m_oldPolozky[i].fakturaid(),
                                                 m_oldPolozky[i].zavpohid(), m_oldPolozky[i].zasobaid(), m_oldPolozky[i].zaokrouhleni()));
            }
        }
    }
    EndModal(wxID_CANCEL);
}

void ZavpohDialog::OnChoiceUhrada(wxCommandEvent &/*event*/)
{
    if(static_cast<FaUhrada*>(m_uhrada->GetClientData(static_cast<unsigned int>(m_uhrada->GetSelection())))->pokladnou())
    {
        if(!idbmodel->hasPokladna(m_bankapokladnaText->GetValue()))
        {
            m_bankapokladnaText->SetValue("");
        }
    }
    else
    {
        if(!idbmodel->hasBanka(m_bankapokladnaText->GetValue()))
        {
            m_bankapokladnaText->SetValue("");
        }
    }
    if(m_mena->GetCount() == 1)
    {
        fillMena();
        wxCommandEvent dummy;
        dummy.SetInt(0);
        OnChoiceMena(dummy);
    }
}

void ZavpohDialog::OnChoiceMena(wxCommandEvent &event)
{
    if(m_platce)
    {
        int menaid = getMenaId(event.GetInt());
        if(menaid == 1) //CZK;
        {
            m_kurzValue = 1.0;
        }
        else
        {
            wxDateTime datum = m_platce?(m_zavazek?m_dppd->GetValue():m_duzp->GetValue()):m_vystaveni->GetValue();
            if(idbmodel->hasKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(event.GetInt())))->kod(), datum)) m_kurzDownload->Disable();
            else m_kurzDownload->Enable();
            m_kurzValue = idbmodel->getKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(event.GetInt())))->kod(), datum).hodnota();
        }
        m_kurz->GetValidator()->TransferToWindow();
        showhideKurz();
        updateDph();
        m_mainSizer->Layout();
        m_mainSizer->Fit(this);
    }
}

void ZavpohDialog::OnAddUhrada(wxCommandEvent &/*event*/)
{
    FauhradaDialog dialog(this);
    if(dialog.ShowModal() == wxID_OK)
    {
        FaUhrada fu = idbmodel->getLastFaUhrada();
        m_uhrada->Append(fu.uhrada(), new FaUhrada(fu.id(), fu.uhrada(), fu.pokladnou()));
        m_uhrada->Select(static_cast<int>(m_uhrada->GetCount())-1);
        wxCommandEvent dummy;
        OnChoiceUhrada(dummy);
    }
}

void ZavpohDialog::OnSelectBankaPokladna(wxCommandEvent &/*event*/)
{
    if(static_cast<FaUhrada*>(m_uhrada->GetClientData(static_cast<unsigned int>(m_uhrada->GetSelection())))->pokladnou())
    {
        PokladnyDialog dialog(this, true);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_bankapokladnaText->SetValue(idbmodel->getPokladnaZkratka(dialog.selectedID()));
            fillMena();
            wxCommandEvent dummy;
            dummy.SetInt(0);
            OnChoiceMena(dummy);
        }
    }
    else
    {
        BankyDialog dialog(this, true);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_bankapokladnaText->SetValue(idbmodel->getBankaZkratka(dialog.selectedID()));
            fillMena();
            wxCommandEvent dummy;
            dummy.SetInt(0);
            OnChoiceMena(dummy);
        }
    }
}

void ZavpohDialog::OnSelectPredkontace(wxCommandEvent &/*event*/)
{
    PredkontacesDialog dialog(this, true, m_zavazek?PredkontacesDialog::PK_VYDAJ:PredkontacesDialog::PK_PRIJEM);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_predkontaceText->SetValue(idbmodel->hasPredkontace(dialog.selectedID())?idbmodel->getPredkontace(dialog.selectedID()).zkratka():"");
    }
}

void ZavpohDialog::OnSelectDphDruh(wxCommandEvent &/*event*/)
{
    DphDruhyDialog dialog(this, true, m_zavazek?DphDruhyDialog::DD_VYDAJ:DphDruhyDialog::DD_PRIJEM, m_zavazek?m_dppd->GetValue():m_duzp->GetValue());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_dphdruhText->SetValue(idbmodel->hasDphDruh(dialog.selectedID())?idbmodel->getDphDruh(dialog.selectedID()).zkratka():"");
    }
}

void ZavpohDialog::OnVystaveniChanged(wxDateEvent &/*event*/)
{
    if(m_platce)
    {
        m_duzp->SetValue(m_vystaveni->GetValue());
        if(m_zavazek) m_dppd->SetValue(m_vystaveni->GetValue());
        if(idbmodel->hasKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), m_duzp->GetValue()))
            m_kurzDownload->Disable();
        else
            m_kurzDownload->Enable();
        m_kurzValue = idbmodel->getKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), m_duzp->GetValue()).hodnota();
        m_kurz->GetValidator()->TransferToWindow();
        updateDph();
    }
    m_splatnost->SetValue(m_vystaveni->GetValue()+wxDateSpan(0,0,0,dxsettings.splatnost()));
}

void ZavpohDialog::OnDuzpChanged(wxDateEvent &/*event*/)
{
    if(m_platce)
    {
        if(m_zavazek) m_dppd->SetValue(m_duzp->GetValue());
        if(getMenaId() == 1)
        {
            m_kurzDownload->Disable();
            return;
        }
        if(idbmodel->hasKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), m_duzp->GetValue()))
            m_kurzDownload->Disable();
        else
            m_kurzDownload->Enable();
        m_kurzValue = idbmodel->getKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), m_duzp->GetValue()).hodnota();
        m_kurz->GetValidator()->TransferToWindow();
        updateDph();
    }
}

void ZavpohDialog::OnDppdChanged(wxDateEvent &/*event*/)
{
    if(m_platce && m_zavazek)
    {
        if(getMenaId() == 1)
        {
            m_kurzDownload->Disable();
            return;
        }
        if(idbmodel->hasKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), m_dppd->GetValue()))
            m_kurzDownload->Disable();
        else
            m_kurzDownload->Enable();
        m_kurzValue = idbmodel->getKurz(static_cast<Mena*>(m_mena->GetClientData(static_cast<unsigned int>(m_mena->GetSelection())))->kod(), m_dppd->GetValue()).hodnota();
        m_kurz->GetValidator()->TransferToWindow();
        updateDph();
    }
}

void ZavpohDialog::OnSelectKontakt(wxCommandEvent &/*event*/)
{
    KontaktyDialog dialog(this, true);
    if(dialog.ShowModal() == wxID_OK)
    {
        Kontakt ko = idbmodel->getKontakt(dialog.selectedID());
        if(ko.isValid())
        {
            Adresa ad = idbmodel->getAdresa(ko.adresaid());
            if(ad.isValid())
            {
                m_nazev->SetValue(ko.nazev());
                m_jmeno->SetValue(ko.jmeno());
                m_ulice->SetValue(ad.ulice());
                m_cp->SetValue(ad.cp());
                m_misto->SetValue(ad.misto());
                m_psc->SetValue(ad.psc());
                m_zeme->SetValue(ad.zeme());
                m_ic->SetValue(ko.ic());
                m_dic->SetValue(ko.dic());
                m_email->SetValue(ko.email());
            }
        }
        enableDisableKontakt();
    }
}

void ZavpohDialog::OnClearKontakt(wxCommandEvent &/*event*/)
{
    m_nazev->Clear();
    m_jmeno->Clear();
    m_ulice->Clear();
    m_cp->Clear();
    m_misto->Clear();
    m_psc->Clear();
    m_zeme->Clear();
    m_ic->Clear();
    m_dic->Clear();
    m_email->Clear();
    enableDisableKontakt();
}

void ZavpohDialog::enableDisableKontakt()
{
    if(m_nazev->IsEnabled())
    {
        m_nazev->Disable();
        m_jmeno->Disable();
        m_ulice->Disable();
        m_cp->Disable();
        m_misto->Disable();
        m_psc->Disable();
        m_zeme->Disable();
        m_ic->Disable();
        m_dic->Disable();
        m_email->Disable();
        m_kontaktSelect->Disable();
        m_kontaktClear->Enable();
    }
    else
    {
        m_nazev->Enable();
        m_jmeno->Enable();
        m_ulice->Enable();
        m_cp->Enable();
        m_misto->Enable();
        m_psc->Enable();
        m_zeme->Enable();
        m_ic->Enable();
        m_dic->Enable();
        m_email->Enable();
        m_kontaktSelect->Enable();
        m_kontaktClear->Disable();
    }
}

void ZavpohDialog::OnPolozkaRightClick(wxListEvent &event)
{
    m_polozkaSelected = event.GetIndex();
    enablePolozkyTools();
    wxMenu popup;
    popup.Append(ID_ZAVPOH_POLOZKY_ADD, wxString::FromUTF8("Nová položka"));
    if(m_polozkaSelected != -1)
    {
        popup.AppendSeparator();
        if(!m_polozky[static_cast<size_t>(m_polozkaSelected)].zaokrouhleni()) popup.Append(ID_ZAVPOH_POLOZKY_EDIT, wxString::FromUTF8("Upravit položku"));
        popup.Append(ID_ZAVPOH_POLOZKY_DELETE, wxString::FromUTF8("Smazat položku"));
    }
    PopupMenu(&popup, event.GetPoint()+m_polozkyList->GetPosition());
}

void ZavpohDialog::OnPolozkaActivated(wxListEvent &event)
{
    m_polozkaSelected = event.GetIndex();
    enablePolozkyTools();
    if(m_polozky[static_cast<size_t>(m_polozkaSelected)].zaokrouhleni())
    {
        return;
    }
    FaPolozka po = m_polozky[static_cast<size_t>(m_polozkaSelected)];
    PolozkaDialog dialog(this, po.id(), m_platce, wxDateTime::Now(), m_platce?(m_zavazek?m_dppd->GetValue():m_duzp->GetValue()):m_vystaveni->GetValue());
    if(dialog.ShowModal() == wxID_OK)
        updateData();
}

void ZavpohDialog::OnPolozkaSelected(wxListEvent &event)
{
    m_polozkaSelected = event.GetIndex();
    enablePolozkyTools();
}

void ZavpohDialog::OnPolozkaDeselected(wxListEvent &/*event*/)
{
    m_polozkaSelected = -1;
    enablePolozkyTools();
}

void ZavpohDialog::enablePolozkyTools()
{
    if(m_polozkaSelected == -1 || !m_polozky.GetCount())
    {
        m_polozkaBar->EnableTool(ID_ZAVPOH_POLOZKY_EDIT, false);
        m_polozkaBar->EnableTool(ID_ZAVPOH_POLOZKY_DELETE, false);
    }
    else
    {
        m_polozkaBar->EnableTool(ID_ZAVPOH_POLOZKY_EDIT, m_polozky[static_cast<size_t>(m_polozkaSelected)].zaokrouhleni()?false:true);
        m_polozkaBar->EnableTool(ID_ZAVPOH_POLOZKY_DELETE, true);
    }
}

void ZavpohDialog::updateData()
{
    size_t oldcnt = m_polozkyList->GetItemCount();
    m_polozky = idbmodel->fapolozkyByZavpohId(m_id);
    size_t cnt = m_polozky.GetCount();
    m_polozkyList->SetItemCount(static_cast<long>(cnt));
    m_zaokrouhleni->Disable();
    if(cnt)
    {
        if(static_cast<size_t>(m_polozkaSelected) >= cnt)
        {
            m_polozkyList->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_polozkaSelected = 0;
        }
        if(oldcnt <= cnt) m_polozkyList->RefreshItems(0, static_cast<long>(cnt)-1);
        else m_polozkyList->Refresh();
        if(m_popis->IsEmpty())
        {
            m_popis->SetValue(m_polozky[0].popis());
        }
        for(size_t i=0; i<m_polozky.GetCount(); i++)
        {
            if(!m_polozky[i].zaokrouhleni())
            {
                m_zaokrouhleni->Enable();
                break;
            }
        }
    }
    else
    {
        m_polozkyList->Refresh();
        m_polozkaBar->EnableTool(ID_ZAVPOH_POLOZKY_EDIT, false);
        m_polozkaBar->EnableTool(ID_ZAVPOH_POLOZKY_DELETE, false);
    }
    double celkem = 0.0;
    for(size_t i=0; i<m_polozky.GetCount(); i++)
        celkem += m_polozky[i].celkem();
    m_celkem->SetLabel(wxString::Format("%.2f", celkem));
    if(m_platce) updateDph();
}

void ZavpohDialog::updateDph()
{
    if(m_platce)
    {
        m_dph->DeleteAllItems();
        m_dph->Hide();
        dxDphSazbaArray sazby = idbmodel->dphsazby();
        for(size_t i=0; i<sazby.GetCount(); i++)
        {
            wxListItem item;
            item.SetId(static_cast<long>(i));
            item.SetText(wxString::Format("%d%%", sazby[i].sazba()));
            m_dph->InsertItem(item);
            double dph = 0.0;
            double celkem = 0.0;
            for(size_t j=0; j<m_polozky.GetCount(); j++)
            {
                if(m_polozky[j].sazbaid() == sazby[i].id())
                {
                    dph += m_polozky[j].dph();
                    celkem += m_polozky[j].celkem();
                }
            }
            dph = dph*m_kurzValue;
            celkem = celkem*m_kurzValue;
            m_dph->SetItem(static_cast<long>(i), 1, wxString::Format("%.2f", celkem-dph));
            m_dph->SetItem(static_cast<long>(i), 2, wxString::Format("%.2f", dph));
            m_dph->SetItem(static_cast<long>(i), 3, wxString::Format("%.2f", celkem));
        }
        m_dph->Show();
    }
}

void ZavpohDialog::OnEditPolozka(wxCommandEvent &/*event*/)
{
    if(m_polozky[static_cast<size_t>(m_polozkaSelected)].zaokrouhleni())
    {
        return;
    }
    PolozkaDialog dialog(this, m_polozky[static_cast<size_t>(m_polozkaSelected)].id(), m_platce, wxDateTime::Now(), m_platce?(m_zavazek?m_dppd->GetValue():m_duzp->GetValue()):m_vystaveni->GetValue());
    if(dialog.ShowModal() == wxID_OK)
        updateData();
}

void ZavpohDialog::OnAddPolozka(wxCommandEvent &/*event*/)
{
    PolozkaDialog dialog(-1, m_id, this, m_platce, m_platce?(m_zavazek?m_dppd->GetValue():m_duzp->GetValue()):m_vystaveni->GetValue());
    if(dialog.ShowModal() == wxID_OK)
        updateData();
}

void ZavpohDialog::OnDeletePolozka(wxCommandEvent &/*event*/)
{
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat položku:\n%s?"), m_polozky[static_cast<size_t>(m_polozkaSelected)].popis()),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deleteFaPolozka(m_polozky[static_cast<size_t>(m_polozkaSelected)].id());
        updateData();
    }
}

void ZpPolozkaCtrl::CreateColumns()
{
    InsertColumn(PCOL_POPIS, "Popis");
    InsertColumn(PCOL_MNOZSTVI, wxString::FromUTF8("Množství"), wxLIST_FORMAT_RIGHT);
    InsertColumn(PCOL_MJ, "mj");
    InsertColumn(PCOL_CENAMJ, "Cena za mj", wxLIST_FORMAT_RIGHT);
    InsertColumn(PCOL_CENACELKEM, "Cena celkem", wxLIST_FORMAT_RIGHT);
    setResizeColumn(PCOL_POPIS);
    SetColumnWidth(3, 90);
    SetColumnWidth(4, 120);
}

wxString ZpPolozkaCtrl::OnGetItemText(long item, long column) const
{
    return m_parent->getPolozkaText(item, column);
}

int ZpPolozkaCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *ZpPolozkaCtrl::OnGetItemAttr(long /*item*/) const
{
    return nullptr;
}
