/***************************************************************
 * Name:      dphdruhfilter.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2019
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "dphdruhfilter.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(DphDruhFilter, wxDialog)

BEGIN_EVENT_TABLE(DphDruhFilter, wxDialog)
    EVT_BUTTON(wxID_OK, DphDruhFilter::OnOk)
    EVT_BUTTON(wxID_CANCEL, DphDruhFilter::OnCancel)
END_EVENT_TABLE()

DphDruhFilter::DphDruhFilter(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Filtr pro členění DPH"))
{
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, 0));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, 0));
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

DphDruhFilter::DphDruhFilter(wxWindow *parent, dxFilterArray filters)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Filtr pro členění DPH")), m_filters(filters)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

dxFilterArray DphDruhFilter::filters() const
{
    return m_filters;
}

void DphDruhFilter::createControls()
{
    int i = 0; //row

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Zkratka"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zkratkaAction = new wxChoice(this, wxID_ANY);
    m_zkratkaAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_zkratkaAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_zkratkaAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_zkratkaValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_zkratkaValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_zkratkaUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_zkratkaUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Popis"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_popisAction = new wxChoice(this, wxID_ANY);
    m_popisAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_popisAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_popisAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_popisValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_popisValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_popisUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_popisUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Plnění")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_plneniAction = new wxChoice(this, wxID_ANY);
    m_plneniAction->Append("Rovno", new FilterAction(FA_EQUAL));
    gridSizer->Add(m_plneniAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_plneniValue = new wxChoice(this, wxID_ANY);
    m_plneniValue->Append(wxString::FromUTF8("Uskutečněné"));
    m_plneniValue->Append(wxString::FromUTF8("Přijaté"));
    m_plneniValue->SetSelection(0);
    gridSizer->Add(m_plneniValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_plneniUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_plneniUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Řádky v přiznání")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_radkyAction = new wxChoice(this, wxID_ANY);
    m_radkyAction->Append("Rovno", new FilterAction(FA_EQUAL));
    gridSizer->Add(m_radkyAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_radkyValue = new wxChoice(this, wxID_ANY);
    m_radkyValue->Append(idbmodel->getDphDruhRadky());
    m_radkyValue->SetSelection(0);
    gridSizer->Add(m_radkyValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_radkyUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_radkyUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Předmět plnění")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kppAction = new wxChoice(this, wxID_ANY);
    m_kppAction->Append("Rovno", new FilterAction(FA_EQUAL));
    gridSizer->Add(m_kppAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_kppValue = new wxChoice(this, wxID_ANY);
    m_kppValue->Append(wxString::FromUTF8("Prázdné"));
    m_kppValue->Append(idbmodel->getDphDruhKpp());
    m_kppValue->SetSelection(0);
    gridSizer->Add(m_kppValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_kppUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_kppUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Oddíl v KH")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_khAction = new wxChoice(this, wxID_ANY);
    m_khAction->Append("Rovno", new FilterAction(FA_EQUAL));
    gridSizer->Add(m_khAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_khValue = new wxChoice(this, wxID_ANY);
    m_khValue->Append(idbmodel->getDphDruhKh());
    m_khValue->SetSelection(0);
    gridSizer->Add(m_khValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_khUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_khUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Nabízet")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_nabizetAction = new wxChoice(this, wxID_ANY);
    m_nabizetAction->Append("Rovno", new FilterAction(FA_EQUAL));
    gridSizer->Add(m_nabizetAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_nabizetValue = new wxChoice(this, wxID_ANY);
    m_nabizetValue->Append("Ne");
    m_nabizetValue->Append("Ano");
    m_nabizetValue->SetSelection(1);
    gridSizer->Add(m_nabizetValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_nabizetUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_nabizetUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void DphDruhFilter::fillControls()
{
    if(m_filters.GetCount() == 7)
    {
        if(m_filters[0].type() == FT_TEXT)
        {
            m_zkratkaAction->SetSelection(m_filters[0].action()==FA_EQUAL?0:1);
            m_zkratkaValue->SetValue(m_filters[0].text());
            m_zkratkaUsage->SetValue(m_filters[0].active());
        }
        if(m_filters[1].type() == FT_TEXT)
        {
            m_popisAction->SetSelection(m_filters[1].action()==FA_EQUAL?0:1);
            m_popisValue->SetValue(m_filters[1].text());
            m_popisUsage->SetValue(m_filters[1].active());

        }
        if(m_filters[2].type() == FT_INTEGER)
        {
            m_plneniAction->SetSelection(0);
            m_plneniValue->SetSelection(m_filters[2].intvalue());
            m_plneniUsage->SetValue(m_filters[2].active());
        }
        if(m_filters[3].type() == FT_TEXT)
        {
            m_radkyAction->SetSelection(0);
            m_radkyValue->SetStringSelection(m_filters[3].text());
            m_radkyUsage->SetValue(m_filters[3].active());
        }
        if(m_filters[4].type() == FT_TEXT)
        {
            m_kppAction->SetSelection(0);
            m_kppValue->SetStringSelection(m_filters[4].text());
            m_kppUsage->SetValue(m_filters[4].active());
        }
        if(m_filters[5].type() == FT_TEXT)
        {
            m_khAction->SetSelection(0);
            m_khValue->SetStringSelection(m_filters[5].text());
            m_khUsage->SetValue(m_filters[5].active());
        }
        if(m_filters[6].type() == FT_INTEGER)
        {
            m_nabizetAction->SetSelection(0);
            m_nabizetValue->SetSelection(m_filters[6].intvalue());
            m_nabizetUsage->SetValue(m_filters[6].active());
        }
    }
}

void DphDruhFilter::OnOk(wxCommandEvent& /*event*/)
{
    m_filters[0].setAction(*static_cast<FilterAction*>(m_zkratkaAction->GetClientData(static_cast<unsigned int>(m_zkratkaAction->GetSelection()))));
    m_filters[0].setText(m_zkratkaValue->GetValue());
    m_filters[0].setActive(m_zkratkaUsage->GetValue());

    m_filters[1].setAction(*static_cast<FilterAction*>(m_popisAction->GetClientData(static_cast<unsigned int>(m_popisAction->GetSelection()))));
    m_filters[1].setText(m_popisValue->GetValue());
    m_filters[1].setActive(m_popisUsage->GetValue());

    m_filters[2].setAction(FA_EQUAL);
    m_filters[2].setIntvalue(m_plneniValue->GetSelection());
    m_filters[2].setActive(m_plneniUsage->GetValue());

    m_filters[3].setAction(FA_EQUAL);
    m_filters[3].setText(m_radkyValue->GetStringSelection());
    m_filters[3].setActive(m_radkyUsage->GetValue());

    m_filters[4].setAction(FA_EQUAL);
    m_filters[4].setText(m_kppValue->GetStringSelection());
    m_filters[4].setActive(m_kppUsage->GetValue());

    m_filters[5].setAction(FA_EQUAL);
    m_filters[5].setText(m_khValue->GetStringSelection());
    m_filters[5].setActive(m_khUsage->GetValue());

    m_filters[6].setAction(FA_EQUAL);
    m_filters[6].setIntvalue(m_nabizetValue->GetSelection());
    m_filters[6].setActive(m_nabizetUsage->GetValue());

    EndModal(wxID_OK);
}

void DphDruhFilter::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

