/***************************************************************
 * Name:      filter.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#ifndef FILTER_H
#define FILTER_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/dynarray.h>

enum FilterType {
    FT_DATUM,
    FT_TEXT,
    FT_INTEGER,
    FT_DOUBLE
};

enum FilterAction {
    FA_LESS = 0,
    FA_LESSEQUAL,
    FA_EQUAL,
    FA_GREATEREQUAL,
    FA_GREATER,
    FA_CONTAINS
};

class Filter
{
public:
    Filter(const FilterAction &action, bool active, const wxDateTime &datum);
    Filter(const FilterAction &action, bool active, const wxString &text);
    Filter(const FilterAction &action, bool active, int value);
    Filter(const FilterAction &action, bool active, double value);

    bool matchFilter(const wxDateTime &datum);
    bool matchFilter(const wxString &text);
    bool matchFilter(int value);
    bool matchFilter(double value);
    FilterType type() const;
    void setType(const FilterType &type);
    FilterAction action() const;
    void setAction(const FilterAction &action);
    wxDateTime datum() const;
    void setDatum(const wxDateTime &datum);
    wxString text() const;
    void setText(const wxString &text);
    int intvalue() const;
    void setIntvalue(int intvalue);
    double doublevalue() const;
    void setDoublevalue(double doublevalue);
    bool active() const;
    void setActive(bool active);
private:
    FilterType m_type;
    FilterAction m_action;
    bool m_active;
    wxDateTime m_datum;
    wxString m_text;
    int m_intvalue;
    double m_doublevalue;
};

WX_DECLARE_OBJARRAY(Filter, dxFilterArray);

#endif /* FILTER_H */
