/***************************************************************
 * Name:      majetekfilter.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2018
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "majetekfilter.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(MajetekFilter, wxDialog)

BEGIN_EVENT_TABLE(MajetekFilter, wxDialog)
    EVT_BUTTON(wxID_OK, MajetekFilter::OnOk)
    EVT_BUTTON(wxID_CANCEL, MajetekFilter::OnCancel)
END_EVENT_TABLE()

MajetekFilter::MajetekFilter(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, "Filtr pro knihu majetku")
{
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

MajetekFilter::MajetekFilter(wxWindow *parent, dxFilterArray filters)
    : wxDialog(parent, wxID_ANY, "Filtr pro knihu majetku"), m_filters(filters)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

dxFilterArray MajetekFilter::filters() const
{
    return m_filters;
}

void MajetekFilter::createControls()
{
    int i = 0; //row

    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Číslo")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cisloAction = new wxChoice(this, wxID_ANY);
    m_cisloAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_cisloAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_cisloAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_cisloValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_cisloValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_cisloUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_cisloUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Název")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_nazevAction = new wxChoice(this, wxID_ANY);
    m_nazevAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_nazevAction->Append("Obsahuje", new FilterAction(FA_CONTAINS));
    gridSizer->Add(m_nazevAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_nazevValue = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_nazevValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_nazevUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_nazevUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Typ"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_typAction = new wxChoice(this, wxID_ANY);
    m_typAction->Append("Rovno", new FilterAction(FA_EQUAL));
    gridSizer->Add(m_typAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_typValue = new wxChoice(this, wxID_ANY);
    m_typValue->Append(idbmodel->majetekTypAsString(1));
    m_typValue->Append(idbmodel->majetekTypAsString(2));
    m_typValue->Append(idbmodel->majetekTypAsString(3));
    m_typValue->SetSelection(0);
    gridSizer->Add(m_typValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_typUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_typUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Odpis"), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_odpisAction = new wxChoice(this, wxID_ANY);
    m_odpisAction->Append("Rovno", new FilterAction(FA_EQUAL));
    gridSizer->Add(m_odpisAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_odpisValue = new wxChoice(this, wxID_ANY);
    m_odpisValue->Append(idbmodel->majetekOdpisAsString(1));
    m_odpisValue->Append(idbmodel->majetekOdpisAsString(2));
    m_odpisValue->Append(idbmodel->majetekOdpisAsString(0));
    m_odpisValue->Append(idbmodel->majetekOdpisAsString(3));
    m_odpisValue->SetSelection(0);
    gridSizer->Add(m_odpisValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_odpisUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_odpisUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pořízen")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_porizenAction = new wxChoice(this, wxID_ANY);
    m_porizenAction->Append(wxString::FromUTF8("Starší než"), new FilterAction(FA_LESS));
    m_porizenAction->Append(wxString::FromUTF8("Starší než nebo rovné"), new FilterAction(FA_LESSEQUAL));
    m_porizenAction->Append(wxString::FromUTF8("Rovné"), new FilterAction(FA_EQUAL));
    m_porizenAction->Append(wxString::FromUTF8("Novější než nebo rovné"), new FilterAction(FA_GREATEREQUAL));
    m_porizenAction->Append(wxString::FromUTF8("Novější než"), new FilterAction(FA_GREATER));
    gridSizer->Add(m_porizenAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_porizenValue = new wxDatePickerCtrl(this, wxID_ANY, dxsettings.datum(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_porizenValue->SetRange(wxDateTime(1,wxDateTime::Jan,idbmodel->currentRok()), wxDateTime(31,wxDateTime::Dec,idbmodel->currentRok()));
    gridSizer->Add(m_porizenValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_porizenUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_porizenUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pořizovací cena")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_porizovaciAction = new wxChoice(this, wxID_ANY);
    m_porizovaciAction->Append(wxString::FromUTF8("Menší než"), new FilterAction(FA_LESS));
    m_porizovaciAction->Append(wxString::FromUTF8("Menší než nebo rovno"), new FilterAction(FA_LESSEQUAL));
    m_porizovaciAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_porizovaciAction->Append(wxString::FromUTF8("Větší než nebo rovno"), new FilterAction(FA_GREATEREQUAL));
    m_porizovaciAction->Append(wxString::FromUTF8("Větší než"), new FilterAction(FA_GREATER));
    gridSizer->Add(m_porizovaciAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_porizovaciValue = new wxSpinCtrlDouble(this, wxID_ANY);
    m_porizovaciValue->SetDigits(2);
    m_porizovaciValue->SetRange(0.0, 10000000.0);
    gridSizer->Add(m_porizovaciValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_porizovaciUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_porizovaciUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);
    i++;

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Zůstatková cena")), wxGBPosition(i,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zustatkovaAction = new wxChoice(this, wxID_ANY);
    m_zustatkovaAction->Append(wxString::FromUTF8("Menší než"), new FilterAction(FA_LESS));
    m_zustatkovaAction->Append(wxString::FromUTF8("Menší než nebo rovno"), new FilterAction(FA_LESSEQUAL));
    m_zustatkovaAction->Append("Rovno", new FilterAction(FA_EQUAL));
    m_zustatkovaAction->Append(wxString::FromUTF8("Větší než nebo rovno"), new FilterAction(FA_GREATEREQUAL));
    m_zustatkovaAction->Append(wxString::FromUTF8("Větší než"), new FilterAction(FA_GREATER));
    gridSizer->Add(m_zustatkovaAction, wxGBPosition(i,1), wxDefaultSpan, wxEXPAND);
    m_zustatkovaValue = new wxSpinCtrlDouble(this, wxID_ANY);
    m_zustatkovaValue->SetDigits(2);
    m_zustatkovaValue->SetRange(0.0, 10000000.0);
    gridSizer->Add(m_zustatkovaValue, wxGBPosition(i,2), wxDefaultSpan, wxEXPAND);
    m_zustatkovaUsage = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_zustatkovaUsage, wxGBPosition(i,3), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void MajetekFilter::fillControls()
{
    if(m_filters.GetCount() == 7)
    {
        if(m_filters[0].type() == FT_TEXT)
        {
            m_cisloAction->SetSelection(m_filters[0].action()==FA_EQUAL?0:1);
            m_cisloValue->ChangeValue(m_filters[0].text());
            m_cisloUsage->SetValue(m_filters[0].active());
        }
        if(m_filters[1].type() == FT_TEXT)
        {
            m_nazevAction->SetSelection(m_filters[1].action()==FA_EQUAL?0:1);
            m_nazevValue->ChangeValue(m_filters[1].text());
            m_nazevUsage->SetValue(m_filters[1].active());
        }
        if(m_filters[2].type() == FT_TEXT)
        {
            m_typAction->SetSelection(0);
            m_typValue->SetStringSelection(m_filters[2].text());
            m_typUsage->SetValue(m_filters[2].active());
        }
        if(m_filters[3].type() == FT_TEXT)
        {
            m_odpisAction->SetSelection(0);
            m_odpisValue->SetStringSelection(m_filters[3].text());
            m_odpisUsage->SetValue(m_filters[3].active());
        }
        if(m_filters[4].type() == FT_DATUM)
        {
            m_porizenAction->SetSelection(m_filters[4].action());
            m_porizenValue->SetValue(m_filters[4].datum());
            m_porizenUsage->SetValue(m_filters[4].active());
        }
        if(m_filters[5].type() == FT_DOUBLE)
        {
            m_porizovaciAction->SetSelection(m_filters[5].action());
            m_porizovaciValue->SetValue(m_filters[5].doublevalue());
            m_porizovaciUsage->SetValue(m_filters[5].active());
        }
        if(m_filters[6].type() == FT_DOUBLE)
        {
            m_zustatkovaAction->SetSelection(m_filters[6].action());
            m_zustatkovaValue->SetValue(m_filters[6].doublevalue());
            m_zustatkovaUsage->SetValue(m_filters[6].active());
        }
    }
}

void MajetekFilter::OnOk(wxCommandEvent& /*event*/)
{
    m_filters[0].setAction(*static_cast<FilterAction*>(m_cisloAction->GetClientData(static_cast<unsigned int>(m_cisloAction->GetSelection()))));
    m_filters[0].setText(m_cisloValue->GetValue());
    m_filters[0].setActive(m_cisloUsage->GetValue());

    m_filters[1].setAction(*static_cast<FilterAction*>(m_nazevAction->GetClientData(static_cast<unsigned int>(m_nazevAction->GetSelection()))));
    m_filters[1].setText(m_nazevValue->GetValue());
    m_filters[1].setActive(m_nazevUsage->GetValue());

    m_filters[2].setAction(FA_EQUAL);
    m_filters[2].setText(m_typValue->GetStringSelection());
    m_filters[2].setActive(m_typUsage->GetValue());

    m_filters[3].setAction(FA_EQUAL);
    m_filters[3].setText(m_odpisValue->GetStringSelection());
    m_filters[3].setActive(m_odpisUsage->GetValue());

    m_filters[4].setAction(*static_cast<FilterAction*>(m_porizenAction->GetClientData(static_cast<unsigned int>(m_porizenAction->GetSelection()))));
    m_filters[4].setDatum(m_porizenValue->GetValue());
    m_filters[4].setActive(m_porizenUsage->GetValue());

    m_filters[5].setAction(*static_cast<FilterAction*>(m_porizovaciAction->GetClientData(static_cast<unsigned int>(m_porizovaciAction->GetSelection()))));
    m_filters[5].setDoublevalue(m_porizovaciValue->GetValue());
    m_filters[5].setActive(m_porizovaciUsage->GetValue());

    m_filters[6].setAction(*static_cast<FilterAction*>(m_zustatkovaAction->GetClientData(static_cast<unsigned int>(m_zustatkovaAction->GetSelection()))));
    m_filters[6].setDoublevalue(m_zustatkovaValue->GetValue());
    m_filters[6].setActive(m_zustatkovaUsage->GetValue());

    EndModal(wxID_OK);
}

void MajetekFilter::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

