/***************************************************************
 * Name:      bankapage.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "bankapage.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/prijem.xpm"
#include "resources/images/vydaj.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/csv.xpm"
#include "resources/images/print.xpm"
#include "resources/images/seznam.xpm"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"
#include "resources/images/printdoc.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/filter.xpm"
#include "resources/images/filter_clear.xpm"
#include "gui/dialogs/prijemvydejdialog.h"
#include "gui/dialogs/bankydialog.h"
#include "gui/prints/bankaknihaprint.h"
#include "gui/prints/bankovnidokladprint.h"
#include "gui/filters/bankafilter.h"

void BankaCtrl::CreateColumns()
{
    InsertColumn(BCOL_DATUM, "Datum");
    InsertColumn(BCOL_CISLO, "Doklad");
    InsertColumn(BCOL_BANKA, wxString::FromUTF8("Účet"));
    InsertColumn(BCOL_VYPIS, wxString::FromUTF8("Výpis"));
    InsertColumn(BCOL_TYP, "Typ");
    InsertColumn(BCOL_POPIS, "Popis");
    InsertColumn(BCOL_CASTKA, wxString::FromUTF8("Částka"), wxLIST_FORMAT_RIGHT);
    setResizeColumn(BCOL_POPIS);
}

wxString BankaCtrl::OnGetItemText(long item, long column) const
{
    return m_page->getItemText(item, column);
}

int BankaCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *BankaCtrl::OnGetItemAttr(long /*item*/) const
{
    return nullptr;
}

IMPLEMENT_DYNAMIC_CLASS(BankaPage, wxPanel)

BEGIN_EVENT_TABLE(BankaPage, wxPanel)
    EVT_MENU(ID_PAGE_BANKA_PRIJEM, BankaPage::OnPrijem)
    EVT_MENU(ID_PAGE_BANKA_VYDAJ, BankaPage::OnVydaj)
    EVT_MENU(ID_PAGE_BANKA_EDIT, BankaPage::OnEditBanka)
    EVT_MENU(ID_PAGE_BANKA_COPY, BankaPage::OnCopyBanka)
    EVT_MENU(ID_PAGE_BANKA_DELETE, BankaPage::OnDeleteBanka)
    EVT_MENU(ID_PAGE_BANKA_PRINTDOKLAD, BankaPage::OnPrintDoklad)
    EVT_MENU(ID_PAGE_BANKA_PRINT, BankaPage::OnPrintBanka)
    EVT_MENU(ID_PAGE_BANKA_EXPORT, BankaPage::OnExportBanka)
    EVT_MENU(ID_PAGE_BANKA_SEZNAM, BankaPage::OnSeznam)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PAGE_BANKA_LIST, BankaPage::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PAGE_BANKA_LIST, BankaPage::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PAGE_BANKA_LIST, BankaPage::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PAGE_BANKA_LIST, BankaPage::OnItemDeselected)
    EVT_LIST_COL_CLICK(ID_PAGE_BANKA_LIST, BankaPage::OnColumnClick)
    EVT_MENU(ID_PAGE_BANKA_FILTER, BankaPage::OnFilter)
    EVT_MENU(ID_PAGE_BANKA_FILTERCLEAR, BankaPage::OnFilterClear)
END_EVENT_TABLE()

BankaPage::BankaPage(wxWindow *parent)
    : dxPage(parent, NAV_BANKA), m_selected(-1), m_sortCol(BankaCtrl::BCOL_DEF_SORT), m_sortAsc(true), m_filter(false)
{
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, 1));
    m_filters.Add(Filter(FA_EQUAL, false, 1));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_PAGE_BANKA_PRIJEM, wxEmptyString, wxBitmap(prijem_xpm), dxutils::GreyScaledIcon(prijem_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nový příjem"));
    m_bar->AddTool(ID_PAGE_BANKA_VYDAJ, wxEmptyString, wxBitmap(vydaj_xpm), dxutils::GreyScaledIcon(vydaj_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nový výdaj"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_BANKA_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit záznam"));
    m_bar->AddTool(ID_PAGE_BANKA_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat záznam"));
    m_bar->AddTool(ID_PAGE_BANKA_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat záznam"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_BANKA_PRINTDOKLAD, wxEmptyString, wxBitmap(printdoc_xpm), dxutils::GreyScaledIcon(printdoc_xpm), wxITEM_NORMAL, "Vytisknout doklad");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_BANKA_PRINT, wxEmptyString, wxBitmap(print_xpm), dxutils::GreyScaledIcon(print_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vytisknout bankovní knihu"));
    m_bar->AddTool(ID_PAGE_BANKA_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, wxString::FromUTF8("Exportovat bankovní knihu"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_BANKA_SEZNAM, wxEmptyString, wxBitmap(seznam_xpm), dxutils::GreyScaledIcon(seznam_xpm), wxITEM_NORMAL, wxString::FromUTF8("Seznam bankovních účtů"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_BANKA_FILTER, wxEmptyString, wxBitmap(filter_xpm), dxutils::GreyScaledIcon(filter_xpm), wxITEM_CHECK, "Filtrovat");
    m_bar->AddTool(ID_PAGE_BANKA_FILTERCLEAR, wxEmptyString, wxBitmap(filter_clear_xpm), dxutils::GreyScaledIcon(filter_clear_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zrušit filtr"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new BankaCtrl(this, this, ID_PAGE_BANKA_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    enableTools();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void BankaPage::updateData(int id)
{
    if(!(id==ID_MODEL_ALL || id==ID_MODEL_BANKA)) return;
    long oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    getData();
    idbmodel->sortDenikView(m_denik, columnToDenikSort(m_sortCol), m_sortAsc);
    long cnt = static_cast<long>(m_denik.GetCount());
    m_list->SetItemCount(cnt);
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_BANKA_PRINT, true);
        m_bar->EnableTool(ID_PAGE_BANKA_EXPORT, true);
        m_bar->EnableTool(ID_PAGE_BANKA_FILTER, true);
        m_bar->EnableTool(ID_PAGE_BANKA_FILTERCLEAR, m_filter?true:false);
    }
    else
    {
        m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_BANKA_EDIT, false);
        m_bar->EnableTool(ID_PAGE_BANKA_COPY, false);
        m_bar->EnableTool(ID_PAGE_BANKA_DELETE, false);
        m_bar->EnableTool(ID_PAGE_BANKA_PRINTDOKLAD, false);
        m_bar->EnableTool(ID_PAGE_BANKA_PRINT, false);
        m_bar->EnableTool(ID_PAGE_BANKA_EXPORT, false);
        m_bar->EnableTool(ID_PAGE_BANKA_FILTER, m_filter?true:false);
        m_bar->EnableTool(ID_PAGE_BANKA_FILTERCLEAR, m_filter?true:false);
    }
}

wxString BankaPage::getItemText(long item, long column) const
{
    if(item < 0 || item >= static_cast<long>(m_denik.GetCount()))
        return "";
    switch(column)
    {
    case BankaCtrl::BCOL_DATUM:
        return m_denik[static_cast<size_t>(item)].datum().FormatDate();
    case BankaCtrl::BCOL_CISLO:
        return m_denik[static_cast<size_t>(item)].cislo();
    case BankaCtrl::BCOL_BANKA:
        return m_denik[static_cast<size_t>(item)].pokladnabanka();
    case BankaCtrl::BCOL_VYPIS:
        return wxString::Format("%d", m_denik[static_cast<size_t>(item)].vypis());
    case BankaCtrl::BCOL_TYP:
        return m_denik[static_cast<size_t>(item)].prijem() ? wxString::FromUTF8("Příjem") : wxString::FromUTF8("Výdaj");
    case BankaCtrl::BCOL_POPIS:
        return m_denik[static_cast<size_t>(item)].popis();
    case BankaCtrl::BCOL_CASTKA:
        return wxString::Format("%.2f", m_denik[static_cast<size_t>(item)].castka());
    default:
        return "";
    }
}

void BankaPage::OnPrijem(wxCommandEvent& /*event*/)
{
    PrijemVydejDialog dialog(this, -1, false, true, false, true);
    dialog.ShowModal();
}

void BankaPage::OnVydaj(wxCommandEvent& /*event*/)
{
    PrijemVydejDialog dialog(this, -1, false, false, false, true);
    dialog.ShowModal();
}

void BankaPage::OnEditBanka(wxCommandEvent& /*event*/)
{
    DenikView dk = m_denik[static_cast<size_t>(m_selected)];
    PrijemVydejDialog dialog(this, dk.id(), true, dk.prijem(), idbmodel->hasPokladna(dk.pokladnabanka()), idbmodel->hasBanka(dk.pokladnabanka()));
    dialog.ShowModal();
}

void BankaPage::OnCopyBanka(wxCommandEvent& /*event*/)
{
    Denik dk = idbmodel->getDenik(idbmodel->copyDenik(m_denik[static_cast<size_t>(m_selected)].id()));
    if(dk.isValid())
    {
        PrijemVydejDialog dialog(this, dk.id(), 2, idbmodel->isPredkontacePrijem(dk.predkontaceid()),
                                 idbmodel->hasPokladna(dk.pokladnaid()), idbmodel->hasBanka(dk.bankaid()));
        dialog.ShowModal();
    }
}

void BankaPage::OnDeleteBanka(wxCommandEvent& /*event*/)
{
    DenikView dk = m_denik[static_cast<size_t>(m_selected)];
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat záznam\n%s\n%s?"), dk.popis(), dk.cislo()),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deleteDenik(dk.id());
    }
}

void BankaPage::OnPrintDoklad(wxCommandEvent &/*event*/)
{
    BankovniDokladPrint print(this, idbmodel->getDenik(m_denik[static_cast<size_t>(m_selected)].id()));
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void BankaPage::OnPrintBanka(wxCommandEvent& /*event*/)
{
    BankaKnihaPrint print(this);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void BankaPage::OnExportBanka(wxCommandEvent& /*event*/)
{
    wxFileDialog dlg(this, wxString::FromUTF8("Exportovat bankovní knihu do"), wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << "Datum" << delim << "Doklad" << delim << wxString::FromUTF8("Účet") << wxString::FromUTF8("Výpis") << delim << "Typ" << delim << "Popis" << delim << wxString::FromUTF8("Částka") << delim << endl;
    dxDenikViewArray denik = idbmodel->denikviewBanka();
    for(size_t i=0; i<denik.GetCount(); i++)
    {
        text << denik[i].datum().FormatDate();
        text << delim;
        text << dxutils::enquotes(denik[i].cislo(), delim);
        text << delim;
        text << dxutils::enquotes(denik[i].pokladnabanka(), delim);
        text << delim;
        text << wxString::Format("%d", denik[i].vypis());
        text << delim;
        text << (denik[i].prijem() ? wxString::FromUTF8("Příjem") : wxString::FromUTF8("Výdaj"));
        text << delim;
        text << dxutils::enquotes(denik[i].popis(), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format(wxString::FromUTF8("%.2f Kč"), denik[i].castka()), delim);
        text << delim;
        text << endl;
    }
}

void BankaPage::OnSeznam(wxCommandEvent& /*event*/)
{
    BankyDialog dialog(this);
    dialog.ShowModal();
}

void BankaPage::OnItemRightClick(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    wxMenu popup;
    popup.Append(ID_PAGE_BANKA_PRIJEM, wxString::FromUTF8("Nový příjem"));
    popup.Append(ID_PAGE_BANKA_VYDAJ, wxString::FromUTF8("Nový výdaj"));
    if(m_selected != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_PAGE_BANKA_EDIT, wxString::FromUTF8("Upravit záznam"));
        popup.Append(ID_PAGE_BANKA_COPY, wxString::FromUTF8("Kopírovat záznam"));
        popup.Append(ID_PAGE_BANKA_DELETE, wxString::FromUTF8("Smazat záznam"));
        popup.AppendSeparator();
        popup.Append(ID_PAGE_BANKA_PRINTDOKLAD, "Vytisknout doklad");
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void BankaPage::OnItemActivated(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    DenikView dk = m_denik[static_cast<size_t>(m_selected)];
    PrijemVydejDialog dialog(this, dk.id(), true, dk.prijem(), idbmodel->hasPokladna(dk.pokladnabanka()), idbmodel->hasBanka(dk.pokladnabanka()));
    dialog.ShowModal();
}

void BankaPage::OnItemSelected(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
}

void BankaPage::OnItemDeselected(wxListEvent& /*event*/)
{
    m_selected = -1;
    enableTools();
}

void BankaPage::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= BankaCtrl::BCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<BankaCtrl::BColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    if(m_denik.GetCount()) idbmodel->sortDenikView(m_denik, columnToDenikSort(m_sortCol), m_sortAsc);
    refreshItems();
}

void BankaPage::enableTools()
{
    if(m_selected == -1)
    {
        m_bar->EnableTool(ID_PAGE_BANKA_EDIT, false);
        m_bar->EnableTool(ID_PAGE_BANKA_COPY, false);
        m_bar->EnableTool(ID_PAGE_BANKA_DELETE, false);
        m_bar->EnableTool(ID_PAGE_BANKA_PRINTDOKLAD, false);
    }
    else
    {
        m_bar->EnableTool(ID_PAGE_BANKA_EDIT, true);
        m_bar->EnableTool(ID_PAGE_BANKA_COPY, true);
        m_bar->EnableTool(ID_PAGE_BANKA_DELETE, true);
        m_bar->EnableTool(ID_PAGE_BANKA_PRINTDOKLAD, true);
    }
}

DbModel::DSort BankaPage::columnToDenikSort(BankaCtrl::BColumn column)
{
    switch(column)
    {
    case BankaCtrl::BCOL_CASTKA: return DbModel::DS_CASTKA;
    case BankaCtrl::BCOL_CISLO: return DbModel::DS_CISLO;
    case BankaCtrl::BCOL_DATUM: return DbModel::DS_DATUM;
    case BankaCtrl::BCOL_POPIS: return DbModel::DS_POPIS;
    case BankaCtrl::BCOL_TYP: return DbModel::DS_PRIJEM;
    case BankaCtrl::BCOL_BANKA: return DbModel::DS_BANKA;
    case BankaCtrl::BCOL_VYPIS: return DbModel::DS_VYPIS;
    default: return DbModel::DS_DATUM;
    }
}

void BankaPage::setColumnImage(BankaCtrl::BColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void BankaPage::OnFilter(wxCommandEvent &/*event*/)
{
    BankaFilter filter(this,m_filters);
    if(filter.ShowModal() == wxID_OK)
    {
        m_filters = filter.filters();
        m_filter = false;
        for(size_t i=0; i<m_filters.GetCount(); i++)
        {
            if(m_filters[i].active())
            {
                m_filter = true;
                break;
            }
        }
        m_bar->ToggleTool(ID_PAGE_BANKA_FILTER, m_filter);
        m_bar->EnableTool(ID_PAGE_BANKA_FILTERCLEAR, m_filter);
        if(m_list->GetColumnCount() == static_cast<int>(m_filters.GetCount()))
        {
            for(size_t i=0; i<m_filters.GetCount(); i++)
            {
                m_list->SetColumnHeaderFiltered(static_cast<int>(i), m_filters[i].active());
            }
            m_list->Update();
        }
    }
    else
    {
        m_filter = false;
        m_bar->ToggleTool(ID_PAGE_BANKA_FILTER, false);
        m_bar->EnableTool(ID_PAGE_BANKA_FILTERCLEAR, false);
        for(int i=0; i<m_list->GetColumnCount(); i++)
        {
            m_list->SetColumnHeaderFiltered(i, false);
        }
        m_list->Update();
    }
    updateData(ID_MODEL_ALL);
}

void BankaPage::OnFilterClear(wxCommandEvent &/*event*/)
{
    m_bar->ToggleTool(ID_PAGE_BANKA_FILTER, false);
    m_filter = false;
    m_bar->EnableTool(ID_PAGE_BANKA_FILTERCLEAR, false);
    for(int i=0; i<m_list->GetColumnCount(); i++)
    {
        m_list->SetColumnHeaderFiltered(i, false);
    }
    m_list->Update();
    updateData(ID_MODEL_ALL);
}

void BankaPage::getData()
{
    if(m_filter)
    {
        m_denik.Clear();
        dxDenikViewArray denik = idbmodel->denikviewBanka();
        for(size_t i=0; i<denik.GetCount(); i++)
        {
            if(m_filters[0].matchFilter(denik[i].datum()) && m_filters[1].matchFilter(denik[i].cislo()) && m_filters[2].matchFilter(denik[i].pokladnabanka()) && m_filters[3].matchFilter(denik[i].vypis())
                    && m_filters[4].matchFilter(denik[i].prijem()) && m_filters[5].matchFilter(denik[i].popis()) && m_filters[6].matchFilter(denik[i].castka()))
            {
                m_denik.Add(denik[i]);
            }
        }
    }
    else
    {
        m_denik = idbmodel->denikviewBanka();
    }
}

void BankaPage::refreshItems()
{
    long oldcnt = m_list->GetItemCount();
    long cnt = static_cast<long>(m_denik.GetCount());
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}

