/***************************************************************
 * Name:      denikpage.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef DENIKPAGE_H
#define DENIKPAGE_H

#include <wx/imaglist.h>
#include "gui/widgets/dxlistctrl.h"
#include "dxpage.h"
#include "gui/filters/filter.h"
#include "gui/widgets/dxtoolbar.h"

class DenikPage;

class DenikCtrl: public dxListCtrl
{
public:
    DenikCtrl(DenikPage *page, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_page(page) {}

    enum DColumn
    {
        DCOL_DATUM = 0,
        DCOL_CISLO,
        DCOL_TYP,
        DCOL_POPIS,
        DCOL_CASTKA,
        DCOL_MAX, // pocet sloupcu
        DCOL_DEF_SORT = DCOL_DATUM
    };

    void CreateColumns();
private:
    DenikPage *m_page;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class DenikPage : public dxPage
{
    DECLARE_DYNAMIC_CLASS(DenikPage)
    DECLARE_EVENT_TABLE()
public:
    DenikPage() : dxPage(nullptr, NAV_DENIK), m_mainSizer(nullptr), m_list(nullptr), m_selected(-1),
      m_bar(nullptr), m_sortCol(DenikCtrl::DCOL_DATUM), m_sortAsc(true), m_filter(false) {}
    explicit DenikPage(wxWindow *parent);

    void updateData(int id) override;
    wxString getItemText(long item, long column) const;
private:
    wxBoxSizer *m_mainSizer;
    DenikCtrl *m_list;
    long m_selected;
    dxToolBar *m_bar;
    DenikCtrl::DColumn m_sortCol;
    bool m_sortAsc;
    wxScopedPtr<wxImageList> m_imageList;
    dxDenikViewArray m_denik;
    bool m_filter;
    dxFilterArray m_filters;

    void OnPrijem(wxCommandEvent& event);
    void OnVydaj(wxCommandEvent& event);
    void OnEditDenik(wxCommandEvent& event);
    void OnCopyDenik(wxCommandEvent& event);
    void OnDeleteDenik(wxCommandEvent& event);
    void OnPrintDoklad(wxCommandEvent& event);
    void OnPrintDenik(wxCommandEvent& event);
    void OnExportDenik(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnColumnClick(wxListEvent& event);
    void enableTools();
    DbModel::DSort columnToDenikSort(DenikCtrl::DColumn column);
    void setColumnImage(DenikCtrl::DColumn column, int image);
    void OnUhrada(wxCommandEvent& event);
    void OnFilter(wxCommandEvent& event);
    void OnFilterClear(wxCommandEvent& event);
    void getData();
    void refreshItems();
};

#endif  /* DENIKPAGE_H */

