/***************************************************************
 * Name:      dxbook.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef DXBOOK_H
#define DXBOOK_H

#include <wx/bookctrl.h>
#include "dxdefs.h"

class dxBook : public wxBookCtrlBase
{
    DECLARE_EVENT_TABLE()
public:
    dxBook() { Init(); }
    dxBook(wxWindow *parent,
             wxWindowID id,
             const wxPoint& pos = wxDefaultPosition,
             const wxSize& size = wxDefaultSize,
             long style = 0,
             const wxString& name = wxEmptyString)
    {
        Init();
        (void)Create(parent, id, pos, size, style, name);
    }

    bool Create(wxWindow *parent,
                wxWindowID id,
                const wxPoint& pos = wxDefaultPosition,
                const wxSize& size = wxDefaultSize,
                long style = 0,
                const wxString& name = wxEmptyString);
    virtual bool SetPageText(size_t n, const wxString& strText);
    virtual wxString GetPageText(size_t n) const;
    virtual int GetPageImage(size_t n) const;
    virtual bool SetPageImage(size_t n, int imageId);
    virtual bool InsertPage(size_t n,
                            wxWindow *page,
                            const wxString& text,
                            bool bSelect = false,
                            int imageId = -1);
    virtual int SetSelection(size_t n) { return DoSetSelection(n, SetSelection_SendEvent); }
    virtual int ChangeSelection(size_t n) { return DoSetSelection(n); }
    virtual int GetSelection() const;
    virtual wxSize CalcSizeFromPage(const wxSize& sizePage) const;
    virtual wxBookCtrlBaseEvent* CreatePageChangingEvent() const;
    virtual void MakeChangedEvent(wxBookCtrlEvent& event);
    virtual void UpdateSelectedPage(size_t newsel);
    void OnSize(wxSizeEvent& event);
    bool AppendPage(wxWindow *page);
    bool HasPage(NAVIGATION nav) const;
    void SelectPage(NAVIGATION nav);
    void UpdatePages(int id);
protected:
    int m_selected;

    virtual wxWindow *DoRemovePage(size_t page);
private:
    void Init();
};

#endif  /* DXBOOK_H */

