/***************************************************************
 * Name:      majetekpage.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "majetekpage.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/csv.xpm"
#include "resources/images/print.xpm"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"
#include "resources/images/calc.xpm"
#include "resources/images/printdoc.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/filter.xpm"
#include "resources/images/filter_clear.xpm"
#include "gui/dialogs/majetekdialog.h"
#include "gui/prints/knihamajetkuprint.h"
#include "gui/prints/kartamajetkuprint.h"
#include "gui/filters/majetekfilter.h"

void MajetekCtrl::CreateColumns()
{
    InsertColumn(MCOL_CISLO, wxString::FromUTF8("Číslo"));
    InsertColumn(MCOL_NAZEV, wxString::FromUTF8("Název"));
    InsertColumn(MCOL_TYP, "Typ");
    InsertColumn(MCOL_ODPIS, "Odpis");
    InsertColumn(MCOL_PORIZEN, wxString::FromUTF8("Pořízen"));
    InsertColumn(MCOL_PORCENA, wxString::FromUTF8("Pořizovací cena"), wxLIST_FORMAT_RIGHT);
    InsertColumn(MCOL_ZUSCENA, wxString::FromUTF8("Zůstatková cena"), wxLIST_FORMAT_RIGHT);
    setResizeColumn(MCOL_NAZEV);
    SetColumnWidth(5, 120);
    SetColumnWidth(6, 120);
}

wxString MajetekCtrl::OnGetItemText(long item, long column) const
{
    return m_page->getItemText(item, column);
}

int MajetekCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *MajetekCtrl::OnGetItemAttr(long item) const
{
    return m_page->getItemAttr(item);
}

IMPLEMENT_DYNAMIC_CLASS(MajetekPage, wxPanel)

BEGIN_EVENT_TABLE(MajetekPage, wxPanel)
    EVT_MENU(ID_PAGE_MAJETEK_ADD, MajetekPage::OnAddMajetek)
    EVT_MENU(ID_PAGE_MAJETEK_EDIT, MajetekPage::OnEditMajetek)
    EVT_MENU(ID_PAGE_MAJETEK_COPY, MajetekPage::OnCopyMajetek)
    EVT_MENU(ID_PAGE_MAJETEK_DELETE, MajetekPage::OnDeleteMajetek)
    EVT_MENU(ID_PAGE_MAJETEK_PRINT, MajetekPage::OnPrintKnihaMajetku)
    EVT_MENU(ID_PAGE_MAJETEK_EXPORT, MajetekPage::OnExportKnihaMajetku)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PAGE_MAJETEK_LIST, MajetekPage::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PAGE_MAJETEK_LIST, MajetekPage::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PAGE_MAJETEK_LIST, MajetekPage::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PAGE_MAJETEK_LIST, MajetekPage::OnItemDeselected)
    EVT_LIST_COL_CLICK(ID_PAGE_MAJETEK_LIST, MajetekPage::OnColumnClick)
    EVT_MENU(ID_PAGE_MAJETEK_MAPRINT, MajetekPage::OnPrintMajetek)
    EVT_MENU(ID_PAGE_MAJETEK_ODPISY, MajetekPage::OnCalcOdpisy)
    EVT_MENU(ID_PAGE_MAJETEK_FILTER, MajetekPage::OnFilter)
    EVT_MENU(ID_PAGE_MAJETEK_FILTERCLEAR, MajetekPage::OnFilterClear)
END_EVENT_TABLE()

MajetekPage::MajetekPage(wxWindow *parent)
    : dxPage(parent, NAV_MAJETEK), m_selected(-1), m_sortCol(MajetekCtrl::MCOL_DEF_SORT), m_sortAsc(true), m_filter(false)
{
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_PAGE_MAJETEK_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nový majetek"));
    m_bar->AddTool(ID_PAGE_MAJETEK_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, "Upravit majetek");
    m_bar->AddTool(ID_PAGE_MAJETEK_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat majetek"));
    m_bar->AddTool(ID_PAGE_MAJETEK_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, "Smazat majetek");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_MAJETEK_MAPRINT, wxEmptyString, wxBitmap(printdoc_xpm), dxutils::GreyScaledIcon(printdoc_xpm), wxITEM_NORMAL, "Vytisknout majetkovou kartu");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_MAJETEK_ODPISY, wxEmptyString, wxBitmap(calc_xpm), dxutils::GreyScaledIcon(calc_xpm), wxITEM_NORMAL, wxString::FromUTF8("Spočítat odpisy"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_MAJETEK_PRINT, wxEmptyString, wxBitmap(print_xpm), dxutils::GreyScaledIcon(print_xpm), wxITEM_NORMAL, "Vytisknout knihu majetku");
    m_bar->AddTool(ID_PAGE_MAJETEK_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, "Exportovat knihu majetku");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_MAJETEK_FILTER, wxEmptyString, wxBitmap(filter_xpm), dxutils::GreyScaledIcon(filter_xpm), wxITEM_CHECK, "Filtrovat");
    m_bar->AddTool(ID_PAGE_MAJETEK_FILTERCLEAR, wxEmptyString, wxBitmap(filter_clear_xpm), dxutils::GreyScaledIcon(filter_clear_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zrušit filtr"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new MajetekCtrl(this, this, ID_PAGE_MAJETEK_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    enableTools();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void MajetekPage::updateData(int id)
{
    if(!(id==ID_MODEL_MAJETEK || id==ID_MODEL_ALL)) return;
    long oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    getData();
    idbmodel->sortMajetekView(m_majetek, columnToMajetekSort(m_sortCol), m_sortAsc);
    long cnt = static_cast<long>(m_majetek.GetCount());
    m_list->SetItemCount(cnt);
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_MAJETEK_ODPISY, true);
        m_bar->EnableTool(ID_PAGE_MAJETEK_PRINT, true);
        m_bar->EnableTool(ID_PAGE_MAJETEK_EXPORT, true);
        m_bar->EnableTool(ID_PAGE_MAJETEK_FILTER, true);
        m_bar->EnableTool(ID_PAGE_MAJETEK_FILTERCLEAR, m_filter?true:false);
    }
    else
    {
        m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_MAJETEK_EDIT, false);
        m_bar->EnableTool(ID_PAGE_MAJETEK_COPY, false);
        m_bar->EnableTool(ID_PAGE_MAJETEK_DELETE, false);
        m_bar->EnableTool(ID_PAGE_MAJETEK_MAPRINT, false);
        m_bar->EnableTool(ID_PAGE_MAJETEK_ODPISY, false);
        m_bar->EnableTool(ID_PAGE_MAJETEK_PRINT, false);
        m_bar->EnableTool(ID_PAGE_MAJETEK_EXPORT, false);
        m_bar->EnableTool(ID_PAGE_MAJETEK_FILTER, m_filter?true:false);
        m_bar->EnableTool(ID_PAGE_MAJETEK_FILTERCLEAR, m_filter?true:false);
    }
}

wxString MajetekPage::getItemText(long item, long column) const
{
    if(item < 0 || item >= static_cast<long>(m_majetek.GetCount()))
        return "";
    switch(column)
    {
    case MajetekCtrl::MCOL_CISLO:
    {
        return m_majetek[static_cast<size_t>(item)].cislo();
    }
    case MajetekCtrl::MCOL_NAZEV:
    {
        return m_majetek[static_cast<size_t>(item)].nazev();
    }
    case MajetekCtrl::MCOL_TYP:
    {
        return m_majetek[static_cast<size_t>(item)].typ();
    }
    case MajetekCtrl::MCOL_ODPIS:
    {
        return m_majetek[static_cast<size_t>(item)].odpis();
    }
    case MajetekCtrl::MCOL_PORIZEN:
    {
        return m_majetek[static_cast<size_t>(item)].porizen().FormatDate();
    }
    case MajetekCtrl::MCOL_PORCENA:
    {
        return wxString::Format("%.2f", m_majetek[static_cast<size_t>(item)].cena());
    }
    case MajetekCtrl::MCOL_ZUSCENA:
    {
        return wxString::Format("%.2f", m_majetek[static_cast<size_t>(item)].zustatkovaCena());
    }
    default:
    {
        return "";
    }
    }
}

wxListItemAttr *MajetekPage::getItemAttr(long item) const
{
    if(item < 0 || item >= static_cast<long>(m_majetek.GetCount()))
        return nullptr;
    if(m_majetek[static_cast<size_t>(item)].vyrazen())
    {
        static wxListItemAttr red(*wxRED, wxNullColour, wxNullFont);
        return &red;
    }
    return nullptr;
}

void MajetekPage::OnAddMajetek(wxCommandEvent& /*event*/)
{
    MajetekDialog dialog(this);
    dialog.ShowModal();
}

void MajetekPage::OnEditMajetek(wxCommandEvent& /*event*/)
{
    MajetekDialog dialog(this, m_majetek[static_cast<size_t>(m_selected)].id());
    dialog.ShowModal();
}

void MajetekPage::OnCopyMajetek(wxCommandEvent& /*event*/)
{
    MajetekDialog dialog(this, idbmodel->copyMajetek(m_majetek[static_cast<size_t>(m_selected)].id()), 2);
    dialog.ShowModal();
}

void MajetekPage::OnDeleteMajetek(wxCommandEvent& /*event*/)
{
    wxMessageDialog dialog(this, wxString::Format("Opravdu smazat majetek\n%s\n%s?", m_majetek[static_cast<size_t>(m_selected)].nazev(), m_majetek[static_cast<size_t>(m_selected)].cislo()),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deleteMajetek(m_majetek[static_cast<size_t>(m_selected)].id());
    }
}

void MajetekPage::OnPrintKnihaMajetku(wxCommandEvent& /*event*/)
{
    KnihaMajetkuPrint print(this);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void MajetekPage::OnExportKnihaMajetku(wxCommandEvent& /*event*/)
{
    wxFileDialog dlg(this, "Exportovat knihu majetku do", wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << wxString::FromUTF8("Číslo") << delim << wxString::FromUTF8("Název") << delim << "Typ" << delim
         << "Odpis" << delim << wxString::FromUTF8("Pořízen") << delim << wxString::FromUTF8("Vyřazen") << delim
         << wxString::FromUTF8("Pořizovací cena") << delim << wxString::FromUTF8("Zůstatková cena") << endl;
    dxMajetekArray majetek = idbmodel->majetek();
    for(size_t i=0; i<majetek.GetCount(); i++)
    {
        text << dxutils::enquotes(majetek[i].cislo(), delim);
        text << delim;
        text << dxutils::enquotes(majetek[i].nazev(), delim);
        text << delim;
        text << dxutils::enquotes(majetek[i].typAsString(), delim);
        text << delim;
        text << dxutils::enquotes(majetek[i].odpisAsString(), delim);
        text << delim;
        text << majetek[i].porizen().FormatDate();
        text << delim;
        text << (majetek[i].vyrazen() ? majetek[i].datumvyrazeni().FormatDate() : "---");
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.2f", majetek[i].cena()), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.2f", idbmodel->getZustatkovaCenaMajetku(majetek[i].id())), delim);
        text << endl;
    }
}

void MajetekPage::OnItemRightClick(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    wxMenu popup;
    popup.Append(ID_PAGE_MAJETEK_ADD, wxString::FromUTF8("Nový majetek"));
    if(m_selected != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_PAGE_MAJETEK_EDIT, "Upravit majetek");
        popup.Append(ID_PAGE_MAJETEK_COPY, wxString::FromUTF8("Kopírovat majetek"));
        popup.Append(ID_PAGE_MAJETEK_DELETE, "Smazat majetek");
        popup.AppendSeparator();
        popup.Append(ID_PAGE_MAJETEK_MAPRINT, "Vytisknout majetkovou kartu");
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void MajetekPage::OnItemActivated(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    MajetekDialog dialog(this, m_majetek[static_cast<size_t>(m_selected)].id());
    dialog.ShowModal();
}

void MajetekPage::OnItemSelected(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
}

void MajetekPage::OnItemDeselected(wxListEvent& /*event*/)
{
    m_selected = -1;
    enableTools();
}

void MajetekPage::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= MajetekCtrl::MCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<MajetekCtrl::MColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    if(m_majetek.GetCount()) idbmodel->sortMajetekView(m_majetek, columnToMajetekSort(m_sortCol), m_sortAsc);
    refreshItems();
}

void MajetekPage::enableTools()
{
    if(m_selected == -1)
    {
        m_bar->EnableTool(ID_PAGE_MAJETEK_EDIT, false);
        m_bar->EnableTool(ID_PAGE_MAJETEK_COPY, false);
        m_bar->EnableTool(ID_PAGE_MAJETEK_DELETE, false);
        m_bar->EnableTool(ID_PAGE_MAJETEK_MAPRINT, false);
    }
    else
    {
        m_bar->EnableTool(ID_PAGE_MAJETEK_EDIT, true);
        m_bar->EnableTool(ID_PAGE_MAJETEK_COPY, true);
        m_bar->EnableTool(ID_PAGE_MAJETEK_DELETE, true);
        m_bar->EnableTool(ID_PAGE_MAJETEK_MAPRINT, true);
    }
}

DbModel::MSort MajetekPage::columnToMajetekSort(MajetekCtrl::MColumn column)
{
    switch(column)
    {
    case MajetekCtrl::MCOL_CISLO: return DbModel::MS_CISLO;
    case MajetekCtrl::MCOL_NAZEV: return DbModel::MS_NAZEV;
    case MajetekCtrl::MCOL_TYP: return DbModel::MS_TYP;
    case MajetekCtrl::MCOL_ODPIS: return DbModel::MS_ODPIS;
    case MajetekCtrl::MCOL_PORIZEN: return DbModel::MS_PORIZEN;
    case MajetekCtrl::MCOL_PORCENA: return DbModel::MS_PORCENA;
    case MajetekCtrl::MCOL_ZUSCENA: return DbModel::MS_ZUSCENA;
    default: return DbModel::MS_CISLO;
    }
}

void MajetekPage::setColumnImage(MajetekCtrl::MColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void MajetekPage::OnPrintMajetek(wxCommandEvent &/*event*/)
{
    KartaMajetkuPrint print(this, idbmodel->getMajetek(m_majetek[static_cast<size_t>(m_selected)].id()));
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void MajetekPage::OnCalcOdpisy(wxCommandEvent &/*event*/)
{
    idbmodel->calculateOdpisy();
}

void MajetekPage::OnFilter(wxCommandEvent &/*event*/)
{
    MajetekFilter filter(this,m_filters);
    if(filter.ShowModal() == wxID_OK)
    {
        m_filters = filter.filters();
        m_filter = false;
        for(size_t i=0; i<m_filters.GetCount(); i++)
        {
            if(m_filters[i].active())
            {
                m_filter = true;
                break;
            }
        }
        m_bar->ToggleTool(ID_PAGE_MAJETEK_FILTER, m_filter);
        m_bar->EnableTool(ID_PAGE_MAJETEK_FILTERCLEAR, m_filter);
        if(m_list->GetColumnCount() == static_cast<int>(m_filters.GetCount()))
        {
            for(size_t i=0; i<m_filters.GetCount(); i++)
            {
                m_list->SetColumnHeaderFiltered(static_cast<int>(i), m_filters[i].active());
            }
            m_list->Update();
        }
    }
    else
    {
        m_filter = false;
        m_bar->ToggleTool(ID_PAGE_MAJETEK_FILTER, false);
        m_bar->EnableTool(ID_PAGE_MAJETEK_FILTERCLEAR, false);
        for(int i=0; i<m_list->GetColumnCount(); i++)
        {
            m_list->SetColumnHeaderFiltered(i, false);
        }
        m_list->Update();
    }
    updateData(ID_MODEL_ALL);
}

void MajetekPage::OnFilterClear(wxCommandEvent &/*event*/)
{
    m_bar->ToggleTool(ID_PAGE_MAJETEK_FILTER, false);
    m_filter = false;
    m_bar->EnableTool(ID_PAGE_MAJETEK_FILTERCLEAR, false);
    for(int i=0; i<m_list->GetColumnCount(); i++)
    {
        m_list->SetColumnHeaderFiltered(i, false);
    }
    m_list->Update();
    updateData(ID_MODEL_ALL);
}

void MajetekPage::getData()
{
    if(m_filter)
    {
        m_majetek.Clear();
        dxMajetekViewArray majetek = idbmodel->majetekview();
        for(size_t i=0; i<majetek.GetCount(); i++)
        {
            if(m_filters[0].matchFilter(majetek[i].cislo()) && m_filters[1].matchFilter(majetek[i].nazev()) && m_filters[2].matchFilter(majetek[i].typ()) && m_filters[3].matchFilter(majetek[i].odpis())
                    && m_filters[4].matchFilter(majetek[i].porizen()) && m_filters[5].matchFilter(majetek[i].cena()) && m_filters[6].matchFilter(majetek[i].zustatkovaCena()))
            {
                m_majetek.Add(majetek[i]);
            }
        }
    }
    else
    {
        m_majetek = idbmodel->majetekview();
    }
}

void MajetekPage::refreshItems()
{
    long oldcnt = m_list->GetItemCount();
    long cnt = static_cast<long>(m_majetek.GetCount());
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}
