/***************************************************************
 * Name:      pokladnapage.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include "pokladnapage.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/prijem.xpm"
#include "resources/images/vydaj.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/csv.xpm"
#include "resources/images/print.xpm"
#include "resources/images/seznam.xpm"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"
#include "resources/images/printdoc.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/filter.xpm"
#include "resources/images/filter_clear.xpm"
#include "gui/dialogs/prijemvydejdialog.h"
#include "gui/dialogs/pokladnydialog.h"
#include "gui/prints/pokladniknihaprint.h"
#include "gui/prints/pokladnidokladprint.h"
#include "gui/filters/pokladnafilter.h"

void PokladnaCtrl::CreateColumns()
{
    InsertColumn(PCOL_DATUM, "Datum");
    InsertColumn(PCOL_CISLO, "Doklad");
    InsertColumn(PCOL_POKLADNA, "Pokladna");
    InsertColumn(PCOL_TYP, "Typ");
    InsertColumn(PCOL_POPIS, "Popis");
    InsertColumn(PCOL_CASTKA, wxString::FromUTF8("Částka"), wxLIST_FORMAT_RIGHT);
    setResizeColumn(PCOL_POPIS);
}

wxString PokladnaCtrl::OnGetItemText(long item, long column) const
{
    return m_page->getItemText(item, column);
}

int PokladnaCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *PokladnaCtrl::OnGetItemAttr(long /*item*/) const
{
    return nullptr;
}

IMPLEMENT_DYNAMIC_CLASS(PokladnaPage, wxPanel)

BEGIN_EVENT_TABLE(PokladnaPage, wxPanel)
    EVT_MENU(ID_PAGE_POKLADNA_PRIJEM, PokladnaPage::OnPrijem)
    EVT_MENU(ID_PAGE_POKLADNA_VYDAJ, PokladnaPage::OnVydaj)
    EVT_MENU(ID_PAGE_POKLADNA_EDIT, PokladnaPage::OnEditPokladna)
    EVT_MENU(ID_PAGE_POKLADNA_COPY, PokladnaPage::OnCopyPokladna)
    EVT_MENU(ID_PAGE_POKLADNA_DELETE, PokladnaPage::OnDeletePokladna)
    EVT_MENU(ID_PAGE_POKLADNA_PRINTDOKLAD, PokladnaPage::OnPrintDoklad)
    EVT_MENU(ID_PAGE_POKLADNA_PRINT, PokladnaPage::OnPrintPokladna)
    EVT_MENU(ID_PAGE_POKLADNA_EXPORT, PokladnaPage::OnExportPokladna)
    EVT_MENU(ID_PAGE_POKLADNA_SEZNAM, PokladnaPage::OnSeznam)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PAGE_POKLADNA_LIST, PokladnaPage::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PAGE_POKLADNA_LIST, PokladnaPage::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PAGE_POKLADNA_LIST, PokladnaPage::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PAGE_POKLADNA_LIST, PokladnaPage::OnItemDeselected)
    EVT_LIST_COL_CLICK(ID_PAGE_POKLADNA_LIST, PokladnaPage::OnColumnClick)
    EVT_MENU(ID_PAGE_POKLADNA_FILTER, PokladnaPage::OnFilter)
    EVT_MENU(ID_PAGE_POKLADNA_FILTERCLEAR, PokladnaPage::OnFilterClear)
END_EVENT_TABLE()

PokladnaPage::PokladnaPage(wxWindow *parent)
    : dxPage(parent, NAV_POKLADNA), m_selected(-1), m_sortCol(PokladnaCtrl::PCOL_DEF_SORT), m_sortAsc(true), m_filter(false)
{
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, 1));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_PAGE_POKLADNA_PRIJEM, wxEmptyString, wxBitmap(prijem_xpm), dxutils::GreyScaledIcon(prijem_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nový příjem"));
    m_bar->AddTool(ID_PAGE_POKLADNA_VYDAJ, wxEmptyString, wxBitmap(vydaj_xpm), dxutils::GreyScaledIcon(vydaj_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nový výdaj"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_POKLADNA_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit záznam"));
    m_bar->AddTool(ID_PAGE_POKLADNA_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat záznam"));
    m_bar->AddTool(ID_PAGE_POKLADNA_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat záznam"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_POKLADNA_PRINTDOKLAD, wxEmptyString, wxBitmap(printdoc_xpm), dxutils::GreyScaledIcon(printdoc_xpm), wxITEM_NORMAL, "Vytisknout doklad");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_POKLADNA_PRINT, wxEmptyString, wxBitmap(print_xpm), dxutils::GreyScaledIcon(print_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vytisknout pokladní knihu"));
    m_bar->AddTool(ID_PAGE_POKLADNA_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, wxString::FromUTF8("Exportovat pokladní knihu"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_POKLADNA_SEZNAM, wxEmptyString, wxBitmap(seznam_xpm), dxutils::GreyScaledIcon(seznam_xpm), wxITEM_NORMAL, "Seznam pokladen");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_POKLADNA_FILTER, wxEmptyString, wxBitmap(filter_xpm), dxutils::GreyScaledIcon(filter_xpm), wxITEM_CHECK, "Filtrovat");
    m_bar->AddTool(ID_PAGE_POKLADNA_FILTERCLEAR, wxEmptyString, wxBitmap(filter_clear_xpm), dxutils::GreyScaledIcon(filter_clear_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zrušit filtr"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new PokladnaCtrl(this, this, ID_PAGE_POKLADNA_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    enableTools();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void PokladnaPage::updateData(int id)
{
    if(!(id==ID_MODEL_ALL || id==ID_MODEL_POKLADNA)) return;
    long oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    getData();
    idbmodel->sortDenikView(m_denik, columnToDenikSort(m_sortCol), m_sortAsc);
    long cnt = static_cast<long>(m_denik.GetCount());
    m_list->SetItemCount(cnt);
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_POKLADNA_PRINT, true);
        m_bar->EnableTool(ID_PAGE_POKLADNA_EXPORT, true);
        m_bar->EnableTool(ID_PAGE_POKLADNA_FILTER, true);
        m_bar->EnableTool(ID_PAGE_POKLADNA_FILTERCLEAR, m_filter?true:false);
    }
    else
    {
        m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_POKLADNA_EDIT, false);
        m_bar->EnableTool(ID_PAGE_POKLADNA_COPY, false);
        m_bar->EnableTool(ID_PAGE_POKLADNA_DELETE, false);
        m_bar->EnableTool(ID_PAGE_POKLADNA_PRINTDOKLAD, false);
        m_bar->EnableTool(ID_PAGE_POKLADNA_PRINT, false);
        m_bar->EnableTool(ID_PAGE_POKLADNA_EXPORT, false);
        m_bar->EnableTool(ID_PAGE_POKLADNA_FILTER, m_filter?true:false);
        m_bar->EnableTool(ID_PAGE_POKLADNA_FILTERCLEAR, m_filter?true:false);
    }
}

wxString PokladnaPage::getItemText(long item, long column) const
{
    if(item < 0 || item >= static_cast<long>(m_denik.GetCount()))
        return "";
    switch(column)
    {
    case PokladnaCtrl::PCOL_DATUM:
        return m_denik[static_cast<size_t>(item)].datum().FormatDate();
    case PokladnaCtrl::PCOL_CISLO:
        return m_denik[static_cast<size_t>(item)].cislo();
    case PokladnaCtrl::PCOL_POKLADNA:
        return m_denik[static_cast<size_t>(item)].pokladnabanka();
    case PokladnaCtrl::PCOL_TYP:
        return m_denik[static_cast<size_t>(item)].prijem() ? wxString::FromUTF8("Příjem") : wxString::FromUTF8("Výdaj");
    case PokladnaCtrl::PCOL_POPIS:
        return m_denik[static_cast<size_t>(item)].popis();
    case PokladnaCtrl::PCOL_CASTKA:
        return wxString::Format("%.2f", m_denik[static_cast<size_t>(item)].castka());
    default:
        return "";
    }
}

void PokladnaPage::OnPrijem(wxCommandEvent& /*event*/)
{
    PrijemVydejDialog dialog(this, -1, false, true, true, false);
    dialog.ShowModal();
}

void PokladnaPage::OnVydaj(wxCommandEvent& /*event*/)
{
    PrijemVydejDialog dialog(this, -1, false, false, true, false);
    dialog.ShowModal();
}

void PokladnaPage::OnEditPokladna(wxCommandEvent& /*event*/)
{
    DenikView dk = m_denik[static_cast<size_t>(m_selected)];
    PrijemVydejDialog dialog(this, dk.id(), true, dk.prijem(), idbmodel->hasPokladna(dk.pokladnabanka()), idbmodel->hasBanka(dk.pokladnabanka()));
    dialog.ShowModal();
}

void PokladnaPage::OnCopyPokladna(wxCommandEvent& /*event*/)
{
    Denik dk = idbmodel->getDenik(idbmodel->copyDenik(m_denik[static_cast<size_t>(m_selected)].id()));
    if(dk.isValid())
    {
        PrijemVydejDialog dialog(this, dk.id(), 2, idbmodel->isPredkontacePrijem(dk.predkontaceid()),
                                 idbmodel->hasPokladna(dk.pokladnaid()), idbmodel->hasBanka(dk.bankaid()));
        dialog.ShowModal();
    }
}

void PokladnaPage::OnDeletePokladna(wxCommandEvent& /*event*/)
{
    DenikView dk = m_denik[static_cast<size_t>(m_selected)];
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Opravdu smazat záznam\n%s\n%s?"), dk.popis(), dk.cislo()),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deleteDenik(dk.id());
    }
}

void PokladnaPage::OnPrintDoklad(wxCommandEvent &/*event*/)
{
    PokladniDokladPrint print(this, idbmodel->getDenik(m_denik[static_cast<size_t>(m_selected)].id()));
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void PokladnaPage::OnPrintPokladna(wxCommandEvent& /*event*/)
{
    PokladniKnihaPrint print(this);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void PokladnaPage::OnExportPokladna(wxCommandEvent& /*event*/)
{
    wxFileDialog dlg(this, wxString::FromUTF8("Exportovat pokladní knihu do"), wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << "Datum" << delim << "Doklad" << delim << "Pokladna" << delim << "Typ" << delim << "Popis" << delim << wxString::FromUTF8("Částka") << delim << endl;
    dxDenikViewArray denik = idbmodel->denikviewPokladna();
    for(size_t i=0; i<denik.GetCount(); i++)
    {
        text << denik[i].datum().FormatDate();
        text << delim;
        text << dxutils::enquotes(denik[i].cislo(), delim);
        text << delim;
        text << dxutils::enquotes(denik[i].pokladnabanka(), delim);
        text << delim;
        text << (denik[i].prijem() ? wxString::FromUTF8("Příjem") : wxString::FromUTF8("Výdaj"));
        text << delim;
        text << dxutils::enquotes(denik[i].popis(), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format(wxString::FromUTF8("%.2f Kč"), denik[i].castka()), delim);
        text << delim;
        text << endl;
    }
}

void PokladnaPage::OnSeznam(wxCommandEvent& /*event*/)
{
    PokladnyDialog dialog(this);
    dialog.ShowModal();
}

void PokladnaPage::OnItemRightClick(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    wxMenu popup;
    popup.Append(ID_PAGE_POKLADNA_PRIJEM, wxString::FromUTF8("Nový příjem"));
    popup.Append(ID_PAGE_POKLADNA_VYDAJ, wxString::FromUTF8("Nový výdaj"));
    if(m_selected != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_PAGE_POKLADNA_EDIT, wxString::FromUTF8("Upravit záznam"));
        popup.Append(ID_PAGE_POKLADNA_COPY, wxString::FromUTF8("Kopírovat záznam"));
        popup.Append(ID_PAGE_POKLADNA_DELETE, wxString::FromUTF8("Smazat záznam"));
        popup.AppendSeparator();
        popup.Append(ID_PAGE_POKLADNA_PRINTDOKLAD, "Vytisknout doklad");
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void PokladnaPage::OnItemActivated(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    DenikView dk = m_denik[static_cast<size_t>(m_selected)];
    PrijemVydejDialog dialog(this, dk.id(), true, dk.prijem(), idbmodel->hasPokladna(dk.pokladnabanka()), idbmodel->hasBanka(dk.pokladnabanka()));
    dialog.ShowModal();
}

void PokladnaPage::OnItemSelected(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
}

void PokladnaPage::OnItemDeselected(wxListEvent& /*event*/)
{
    m_selected = -1;
    enableTools();
}

void PokladnaPage::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= PokladnaCtrl::PCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<PokladnaCtrl::PColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    if(m_denik.GetCount()) idbmodel->sortDenikView(m_denik, columnToDenikSort(m_sortCol), m_sortAsc);
    refreshItems();
}

void PokladnaPage::enableTools()
{
    if(m_selected == -1)
    {
        m_bar->EnableTool(ID_PAGE_POKLADNA_EDIT, false);
        m_bar->EnableTool(ID_PAGE_POKLADNA_COPY, false);
        m_bar->EnableTool(ID_PAGE_POKLADNA_DELETE, false);
        m_bar->EnableTool(ID_PAGE_POKLADNA_PRINTDOKLAD, false);
    }
    else
    {
        m_bar->EnableTool(ID_PAGE_POKLADNA_EDIT, true);
        m_bar->EnableTool(ID_PAGE_POKLADNA_COPY, true);
        m_bar->EnableTool(ID_PAGE_POKLADNA_DELETE, true);
        m_bar->EnableTool(ID_PAGE_POKLADNA_PRINTDOKLAD, true);
    }
}

DbModel::DSort PokladnaPage::columnToDenikSort(PokladnaCtrl::PColumn column)
{
    switch(column)
    {
    case PokladnaCtrl::PCOL_CASTKA: return DbModel::DS_CASTKA;
    case PokladnaCtrl::PCOL_CISLO: return DbModel::DS_CISLO;
    case PokladnaCtrl::PCOL_DATUM: return DbModel::DS_DATUM;
    case PokladnaCtrl::PCOL_POPIS: return DbModel::DS_POPIS;
    case PokladnaCtrl::PCOL_TYP: return DbModel::DS_PRIJEM;
    case PokladnaCtrl::PCOL_POKLADNA: return DbModel::DS_POKLADNA;
    default: return DbModel::DS_DATUM;
    }
}

void PokladnaPage::setColumnImage(PokladnaCtrl::PColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void PokladnaPage::OnFilter(wxCommandEvent &/*event*/)
{
    PokladnaFilter filter(this,m_filters);
    if(filter.ShowModal() == wxID_OK)
    {
        m_filters = filter.filters();
        m_filter = false;
        for(size_t i=0; i<m_filters.GetCount(); i++)
        {
            if(m_filters[i].active())
            {
                m_filter = true;
                break;
            }
        }
        m_bar->ToggleTool(ID_PAGE_POKLADNA_FILTER, m_filter);
        m_bar->EnableTool(ID_PAGE_POKLADNA_FILTERCLEAR, m_filter);
        if(m_list->GetColumnCount() == static_cast<int>(m_filters.GetCount()))
        {
            for(size_t i=0; i<m_filters.GetCount(); i++)
            {
                m_list->SetColumnHeaderFiltered(static_cast<int>(i), m_filters[i].active());
            }
            m_list->Update();
        }
    }
    else
    {
        m_filter = false;
        m_bar->ToggleTool(ID_PAGE_POKLADNA_FILTER, false);
        m_bar->EnableTool(ID_PAGE_POKLADNA_FILTERCLEAR, false);
        for(int i=0; i<m_list->GetColumnCount(); i++)
        {
            m_list->SetColumnHeaderFiltered(i, false);
        }
        m_list->Update();
    }
    updateData(ID_MODEL_ALL);
}

void PokladnaPage::OnFilterClear(wxCommandEvent &/*event*/)
{
    m_bar->ToggleTool(ID_PAGE_POKLADNA_FILTER, false);
    m_filter = false;
    m_bar->EnableTool(ID_PAGE_POKLADNA_FILTERCLEAR, false);
    for(int i=0; i<m_list->GetColumnCount(); i++)
    {
        m_list->SetColumnHeaderFiltered(i, false);
    }
    m_list->Update();
    updateData(ID_MODEL_ALL);
}

void PokladnaPage::getData()
{
    if(m_filter)
    {
        m_denik.Clear();
        dxDenikViewArray denik = idbmodel->denikviewPokladna();
        for(size_t i=0; i<denik.GetCount(); i++)
        {
            if(m_filters[0].matchFilter(denik[i].datum()) && m_filters[1].matchFilter(denik[i].cislo()) && m_filters[2].matchFilter(denik[i].pokladnabanka())
                    && m_filters[3].matchFilter(denik[i].prijem()) && m_filters[4].matchFilter(denik[i].popis()) && m_filters[5].matchFilter(denik[i].castka()))
            {
                m_denik.Add(denik[i]);
            }
        }
    }
    else
    {
        m_denik = idbmodel->denikviewPokladna();
    }
}

void PokladnaPage::refreshItems()
{
    long oldcnt = m_list->GetItemCount();
    long cnt = static_cast<long>(m_denik.GetCount());
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}
