/***************************************************************
 * Name:      pokladnapage.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef POKLADNAPAGE_H
#define POKLADNAPAGE_H

#include <wx/imaglist.h>
#include "gui/widgets/dxlistctrl.h"
#include "dxpage.h"
#include "gui/filters/filter.h"
#include "gui/widgets/dxtoolbar.h"

class PokladnaPage;

class PokladnaCtrl: public dxListCtrl
{
public:
    PokladnaCtrl(PokladnaPage *page, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_page(page) {}

    enum PColumn
    {
        PCOL_DATUM = 0,
        PCOL_CISLO,
        PCOL_POKLADNA,
        PCOL_TYP,
        PCOL_POPIS,
        PCOL_CASTKA,
        PCOL_MAX, // pocet sloupcu
        PCOL_DEF_SORT = PCOL_DATUM
    };

    void CreateColumns();
private:
    PokladnaPage *m_page;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class PokladnaPage : public dxPage
{
    DECLARE_DYNAMIC_CLASS(PokladnaPage)
    DECLARE_EVENT_TABLE()
public:
    PokladnaPage() : dxPage(nullptr, NAV_POKLADNA), m_mainSizer(nullptr), m_list(nullptr), m_selected(-1),
    m_bar(nullptr), m_sortCol(PokladnaCtrl::PCOL_DATUM), m_sortAsc(true), m_filter(false) {}
    explicit PokladnaPage(wxWindow *parent);

    void updateData(int id) override;
    wxString getItemText(long item, long column) const;
private:
    wxBoxSizer *m_mainSizer;
    PokladnaCtrl *m_list;
    long m_selected;
    dxToolBar *m_bar;
    dxDenikViewArray m_denik;
    PokladnaCtrl::PColumn m_sortCol;
    bool m_sortAsc;
    wxScopedPtr<wxImageList> m_imageList;
    bool m_filter;
    dxFilterArray m_filters;

    void OnPrijem(wxCommandEvent& event);
    void OnVydaj(wxCommandEvent& event);
    void OnEditPokladna(wxCommandEvent& event);
    void OnCopyPokladna(wxCommandEvent& event);
    void OnDeletePokladna(wxCommandEvent& event);
    void OnPrintDoklad(wxCommandEvent& event);
    void OnPrintPokladna(wxCommandEvent& event);
    void OnExportPokladna(wxCommandEvent& event);
    void OnSeznam(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnColumnClick(wxListEvent& event);
    void enableTools();
    DbModel::DSort columnToDenikSort(PokladnaCtrl::PColumn column);
    void setColumnImage(PokladnaCtrl::PColumn column, int image);
    void OnFilter(wxCommandEvent& event);
    void OnFilterClear(wxCommandEvent& event);
    void getData();
    void refreshItems();
};

#endif  /* POKLADNAPAGE_H */

