/***************************************************************
 * Name:      prijatapage.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef PRIJATAPAGE_H
#define PRIJATAPAGE_H

#include <wx/imaglist.h>
#include "gui/widgets/dxlistctrl.h"
#include "dxpage.h"
#include "gui/filters/filter.h"
#include "gui/widgets/dxtoolbar.h"

class PrijataPage;

class PrijataCtrl: public dxListCtrl
{
public:
    PrijataCtrl(PrijataPage *page, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_page(page) {}

    enum PColumn
    {
        PCOL_DOKLAD = 0,
        PCOL_POPIS,
        PCOL_ODBERRATEL,
        PCOL_VYSTAVENA,
        PCOL_SPLATNA,
        PCOL_CELKEM,
        PCOL_UHRADIT,
        PCOL_MAX, // pocet sloupcu
        PCOL_DEF_SORT = PCOL_DOKLAD
    };

    void CreateColumns();
private:
    PrijataPage *m_page;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class PrijataPage : public dxPage
{
    DECLARE_DYNAMIC_CLASS(PrijataPage)
    DECLARE_EVENT_TABLE()
public:
    PrijataPage() : dxPage(nullptr, NAV_PRIJATA), m_mainSizer(nullptr), m_list(nullptr), m_selected(-1),
    m_bar(nullptr), m_sortCol(PrijataCtrl::PCOL_DOKLAD), m_sortAsc(true), m_filter(false) {}
    explicit PrijataPage(wxWindow *parent);

    void updateData(int id) override;
    wxString getItemText(long item, long column) const;
    wxListItemAttr *getItemAttr(long item) const;
private:
    wxBoxSizer *m_mainSizer;
    PrijataCtrl *m_list;
    long m_selected;
    dxToolBar *m_bar;
    PrijataCtrl::PColumn m_sortCol;
    bool m_sortAsc;
    wxScopedPtr<wxImageList> m_imageList;
    dxFakturaViewArray m_faktury;
    bool m_filter;
    dxFilterArray m_filters;

    void OnAddFaktura(wxCommandEvent& event);
    void OnEditFaktura(wxCommandEvent& event);
    void OnCopyFaktura(wxCommandEvent& event);
    void OnDeleteFaktura(wxCommandEvent& event);
    void OnUhraditFaktura(wxCommandEvent& event);
    void OnPrintKnihaFaktur(wxCommandEvent& event);
    void OnExportKnihaFaktur(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnColumnClick(wxListEvent& event);
    void enableTools();
    DbModel::FZPSort columnToFakturaSort(PrijataCtrl::PColumn column);
    void setColumnImage(PrijataCtrl::PColumn column, int image);
    void OnPrintFaktura(wxCommandEvent& event);
    void OnUhrady(wxCommandEvent& event);
    void OnFilter(wxCommandEvent& event);
    void OnFilterClear(wxCommandEvent& event);
    void getData();
    void refreshItems();
};

#endif  /* PRIJATAPAGE_H */

