/***************************************************************
 * Name:      skladpage.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#ifndef SKLADPAGE_H
#define SKLADPAGE_H

#include <wx/imaglist.h>
#include "gui/widgets/dxlistctrl.h"
#include "dxpage.h"
#include "gui/filters/filter.h"
#include "gui/widgets/dxtoolbar.h"

class SkladPage;

class SkladCtrl: public dxListCtrl
{
public:
    SkladCtrl(SkladPage *page, wxWindow *parent, const wxWindowID id)
        : dxListCtrl(parent, id), m_page(page) {}

    enum SColumn
    {
        SCOL_CISLO = 0,
        SCOL_NAZEV,
        SCOL_STAV,
        SCOL_HODNOTA,
        SCOL_SKLAD,
        SCOL_MAX, // pocet sloupcu
        SCOL_DEF_SORT = SCOL_CISLO
    };

    void CreateColumns();
private:
    SkladPage *m_page;

    virtual wxString OnGetItemText(long item, long column) const;
    virtual int OnGetItemColumnImage(long item, long column) const;
    virtual wxListItemAttr *OnGetItemAttr(long item) const;
};

class SkladPage : public dxPage
{
    DECLARE_DYNAMIC_CLASS(SkladPage)
    DECLARE_EVENT_TABLE()
public:
    SkladPage() : dxPage(nullptr, NAV_DENIK), m_mainSizer(nullptr), m_list(nullptr), m_selected(-1),
    m_bar(nullptr), m_sortCol(SkladCtrl::SCOL_CISLO), m_sortAsc(true), m_filter(false) {}
    explicit SkladPage(wxWindow *parent);

    void updateData(int id) override;
    wxString getItemText(long item, long column) const;
private:
    wxBoxSizer *m_mainSizer;
    SkladCtrl *m_list;
    long m_selected;
    dxToolBar *m_bar;
    SkladCtrl::SColumn m_sortCol;
    bool m_sortAsc;
    wxScopedPtr<wxImageList> m_imageList;
    dxZasobaViewArray m_zasoby;
    bool m_filter;
    dxFilterArray m_filters;

    void OnAddZasoba(wxCommandEvent& event);
    void OnEditZasoba(wxCommandEvent& event);
    void OnCopyZasoba(wxCommandEvent& event);
    void OnDeleteZasoba(wxCommandEvent& event);
    void OnPrintKnihaZasob(wxCommandEvent& event);
    void OnExportKnihaZasob(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    void OnItemSelected(wxListEvent& event);
    void OnItemDeselected(wxListEvent& event);
    void OnColumnClick(wxListEvent& event);
    void enableTools();
    DbModel::ZSort columnToZasobaSort(SkladCtrl::SColumn column);
    void setColumnImage(SkladCtrl::SColumn column, int image);
    void OnPrintZasoba(wxCommandEvent& event);
    void OnInventura(wxCommandEvent& event);
    void OnPrijemka(wxCommandEvent& event);
    void OnVydejka(wxCommandEvent& event);
    void OnSklady(wxCommandEvent& event);
    void OnFilter(wxCommandEvent& event);
    void OnFilterClear(wxCommandEvent& event);
    void getData();
    void refreshItems();
};

#endif  /* SKLADPAGE_H */

