/***************************************************************
 * Name:      vydanapage.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/txtstrm.h>
#include <wx/filefn.h>
#include "vydanapage.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/csv.xpm"
#include "resources/images/print.xpm"
#include "resources/images/asc.xpm"
#include "resources/images/desc.xpm"
#include "resources/images/uhrada.xpm"
#include "resources/images/printdoc.xpm"
#include "resources/images/email.xpm"
#include "resources/images/pdf.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/seznam.xpm"
#include "resources/images/filter.xpm"
#include "resources/images/filter_clear.xpm"
#include "gui/dialogs/fakturadialog.h"
#include "gui/dialogs/uhradadialog.h"
#include "gui/dialogs/sendemaildialog.h"
#include "gui/prints/knihafakturprint.h"
#include "gui/prints/fakturaneplatceprint.h"
#include "gui/prints/fakturaplatceprint.h"
#include "gui/prints/fakturaxmlprint.h"
#include "gui/dialogs/uhradydialog.h"
#include "gui/filters/vydanafilter.h"

void VydanaCtrl::CreateColumns()
{
    InsertColumn(VCOL_DOKLAD, "Doklad");
    InsertColumn(VCOL_POPIS, "Popis");
    InsertColumn(VCOL_ODBERRATEL, wxString::FromUTF8("Odběratel"));
    InsertColumn(VCOL_VYSTAVENA, "Vystavena");
    InsertColumn(VCOL_SPLATNA, wxString::FromUTF8("Splatná"));
    InsertColumn(VCOL_CELKEM, "Celkem", wxLIST_FORMAT_RIGHT);
    InsertColumn(VCOL_UHRADIT, "Uhradit", wxLIST_FORMAT_RIGHT);
    setResizeColumn(VCOL_POPIS);
}

wxString VydanaCtrl::OnGetItemText(long item, long column) const
{
    return m_page->getItemText(item, column);
}

int VydanaCtrl::OnGetItemColumnImage(long /*item*/, long /*column*/) const
{
    return -1;
}

wxListItemAttr *VydanaCtrl::OnGetItemAttr(long item) const
{
    return m_page->getItemAttr(item);
}

IMPLEMENT_DYNAMIC_CLASS(VydanaPage, wxPanel)

BEGIN_EVENT_TABLE(VydanaPage, wxPanel)
    EVT_MENU(ID_PAGE_VYDANA_ADD, VydanaPage::OnAddFaktura)
    EVT_MENU(ID_PAGE_VYDANA_EDIT, VydanaPage::OnEditFaktura)
    EVT_MENU(ID_PAGE_VYDANA_COPY, VydanaPage::OnCopyFaktura)
    EVT_MENU(ID_PAGE_VYDANA_DELETE, VydanaPage::OnDeleteFaktura)
    EVT_MENU(ID_PAGE_VYDANA_PRINT, VydanaPage::OnPrintKnihaFaktur)
    EVT_MENU(ID_PAGE_VYDANA_EXPORT, VydanaPage::OnExportKnihaFaktur)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_PAGE_VYDANA_LIST, VydanaPage::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_PAGE_VYDANA_LIST, VydanaPage::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_PAGE_VYDANA_LIST, VydanaPage::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_PAGE_VYDANA_LIST, VydanaPage::OnItemDeselected)
    EVT_LIST_COL_CLICK(ID_PAGE_VYDANA_LIST, VydanaPage::OnColumnClick)
    EVT_MENU(ID_PAGE_VYDANA_UHRADA, VydanaPage::OnUhraditFaktura)
    EVT_MENU(ID_PAGE_VYDANA_FAPRINT, VydanaPage::OnPrintFaktura)
    EVT_MENU(ID_PAGE_VYDANA_FAEMAIL, VydanaPage::OnEmailFaktura)
    EVT_MENU(ID_PAGE_VYDANA_FAPDF, VydanaPage::OnSaveFaktura)
    EVT_MENU(ID_PAGE_VYDANA_UHRADY, VydanaPage::OnUhrady)
    EVT_MENU(ID_PAGE_VYDANA_FILTER, VydanaPage::OnFilter)
    EVT_MENU(ID_PAGE_VYDANA_FILTERCLEAR, VydanaPage::OnFilterClear)
END_EVENT_TABLE()

VydanaPage::VydanaPage(wxWindow *parent)
    : dxPage(parent, NAV_VYDANA), m_selected(-1), m_sortCol(VydanaCtrl::VCOL_DEF_SORT), m_sortAsc(true), m_filter(false)
{
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_EQUAL, false, wxEmptyString));
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_LESS, false, dxsettings.datum()));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_filters.Add(Filter(FA_LESS, false, 0.0));
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_PAGE_VYDANA_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Nová faktura"));
    m_bar->AddTool(ID_PAGE_VYDANA_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, "Upravit fakturu");
    m_bar->AddTool(ID_PAGE_VYDANA_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat fakturu"));
    m_bar->AddTool(ID_PAGE_VYDANA_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, "Smazat fakturu");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_VYDANA_UHRADA, wxEmptyString, wxBitmap(uhrada_xpm), dxutils::GreyScaledIcon(uhrada_xpm), wxITEM_NORMAL, "Uhradit fakturu");
    m_bar->AddTool(ID_PAGE_VYDANA_UHRADY, wxEmptyString, wxBitmap(seznam_xpm), dxutils::GreyScaledIcon(seznam_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zobrazit úhrady"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_VYDANA_FAPRINT, wxEmptyString, wxBitmap(printdoc_xpm), dxutils::GreyScaledIcon(printdoc_xpm), wxITEM_NORMAL, "Vytisknout fakturu");
    m_bar->AddTool(ID_PAGE_VYDANA_FAPDF, wxEmptyString, wxBitmap(pdf_xpm), dxutils::GreyScaledIcon(pdf_xpm), wxITEM_NORMAL, wxString::FromUTF8("Uložit fakturu jako pdf"));
    m_bar->AddTool(ID_PAGE_VYDANA_FAEMAIL, wxEmptyString, wxBitmap(email_xpm), dxutils::GreyScaledIcon(email_xpm), wxITEM_NORMAL, "Poslat fakturu e-mailem");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_VYDANA_PRINT, wxEmptyString, wxBitmap(print_xpm), dxutils::GreyScaledIcon(print_xpm), wxITEM_NORMAL, "Vytisknout knihu faktur");
    m_bar->AddTool(ID_PAGE_VYDANA_EXPORT, wxEmptyString, wxBitmap(csv_xpm), dxutils::GreyScaledIcon(csv_xpm), wxITEM_NORMAL, "Exportovat knihu faktur");
    m_bar->AddSeparator();
    m_bar->AddTool(ID_PAGE_VYDANA_FILTER, wxEmptyString, wxBitmap(filter_xpm), dxutils::GreyScaledIcon(filter_xpm), wxITEM_CHECK, "Filtrovat");
    m_bar->AddTool(ID_PAGE_VYDANA_FILTERCLEAR, wxEmptyString, wxBitmap(filter_clear_xpm), dxutils::GreyScaledIcon(filter_clear_xpm), wxITEM_NORMAL, wxString::FromUTF8("Zrušit filtr"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxSize imageSize(16, 16);
    m_imageList.reset(new wxImageList(imageSize.GetWidth(), imageSize.GetHeight()));
    m_imageList->Add(wxImage(asc_xpm).Scale(16, 16));
    m_imageList->Add(wxImage(desc_xpm).Scale(16, 16));
    m_list = new VydanaCtrl(this, this, ID_PAGE_VYDANA_LIST);
    m_list->SetImageList(m_imageList.get(), wxIMAGE_LIST_SMALL);
    m_list->CreateColumns();
    enableTools();
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void VydanaPage::updateData(int id)
{
    if(!(id==ID_MODEL_ALL || id==ID_MODEL_VYDANA)) return;
    long oldcnt = m_list->GetItemCount();
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    getData();
    idbmodel->sortFakturyView(m_faktury, columnToFakturaSort(m_sortCol), m_sortAsc);
    long cnt = static_cast<long>(m_faktury.GetCount());
    m_list->SetItemCount(cnt);
    if(cnt)
    {
        if(m_selected >= cnt || m_selected < 0)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_VYDANA_PRINT, true);
        m_bar->EnableTool(ID_PAGE_VYDANA_EXPORT, true);
        m_bar->EnableTool(ID_PAGE_VYDANA_UHRADA, idbmodel->isFakturaUhrazena(m_faktury[static_cast<size_t>(m_selected)].id())?false:true);
        m_bar->EnableTool(ID_PAGE_VYDANA_FILTER, true);
        m_bar->EnableTool(ID_PAGE_VYDANA_FILTERCLEAR, m_filter?true:false);
    }
    else
    {
        m_list->Refresh();
        m_bar->EnableTool(ID_PAGE_VYDANA_EDIT, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_COPY, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_DELETE, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_FAPRINT, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_FAPDF, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_FAEMAIL, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_UHRADA, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_PRINT, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_EXPORT, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_FILTER, m_filter?true:false);
        m_bar->EnableTool(ID_PAGE_VYDANA_FILTERCLEAR, m_filter?true:false);
    }
}

wxString VydanaPage::getItemText(long item, long column) const
{
    if(item < 0 || item >= static_cast<long>(m_faktury.GetCount()))
        return "";
    switch(column)
    {
    case VydanaCtrl::VCOL_DOKLAD:
        return m_faktury[static_cast<size_t>(item)].cislo();
    case VydanaCtrl::VCOL_POPIS:
        return m_faktury[static_cast<size_t>(item)].popis();
    case VydanaCtrl::VCOL_ODBERRATEL:
        return m_faktury[static_cast<size_t>(item)].protistrana();
    case VydanaCtrl::VCOL_VYSTAVENA:
        return m_faktury[static_cast<size_t>(item)].vystaveni().FormatDate();
    case VydanaCtrl::VCOL_SPLATNA:
        return m_faktury[static_cast<size_t>(item)].splatnost().FormatDate();
    case VydanaCtrl::VCOL_CELKEM:
        return wxString::Format("%.2f", m_faktury[static_cast<size_t>(item)].celkem());
    case VydanaCtrl::VCOL_UHRADIT:
        return wxString::Format("%.2f", dxutils::isSameDouble(m_faktury[static_cast<size_t>(item)].celkem(), m_faktury[static_cast<size_t>(item)].uhrazeno()) ?
                                    0.0 : m_faktury[static_cast<size_t>(item)].celkem()-m_faktury[static_cast<size_t>(item)].uhrazeno());
    default:
        return "";
    }
}

wxListItemAttr *VydanaPage::getItemAttr(long item) const
{
    if(item < 0 || item >= static_cast<long>(m_faktury.GetCount()))
        return nullptr;
    if(idbmodel->isFakturaUhrazena(m_faktury[static_cast<size_t>(item)].id()))
        return nullptr;
    else
    {
        if(m_faktury[static_cast<size_t>(item)].splatnost().IsLaterThan(dxsettings.datum()))
        {
            static wxListItemAttr green(wxColour(64,157,64), wxNullColour, wxNullFont);
            return &green;
        }
        else
        {
            static wxListItemAttr red(*wxRED, wxNullColour, wxNullFont);
            return &red;
        }
    }
}

void VydanaPage::OnAddFaktura(wxCommandEvent& /*event*/)
{
    int typid = 0;
    wxSingleChoiceDialog vyber(this, "Vyberte typ faktury", "Typ faktury", idbmodel->getFaTypyAsStrings());
    if(vyber.ShowModal() == wxID_OK)
    {
        typid = idbmodel->itemFaTyp(static_cast<size_t>(vyber.GetSelection())).id();
    }
    else
    {
        return;
    }
    FakturaDialog dialog(typid, true, this);
    dialog.ShowModal();
}

void VydanaPage::OnEditFaktura(wxCommandEvent& /*event*/)
{
    FakturaDialog dialog(m_faktury[static_cast<size_t>(m_selected)].id(), this);
    dialog.ShowModal();
}

void VydanaPage::OnCopyFaktura(wxCommandEvent& /*event*/)
{
    FakturaDialog dialog(idbmodel->copyFaktura(m_faktury[static_cast<size_t>(m_selected)].id()), this, 2);
    dialog.ShowModal();
}

void VydanaPage::OnDeleteFaktura(wxCommandEvent& /*event*/)
{
    wxMessageDialog dialog(this, wxString::Format("Opravdu smazat fakturu\n%s\n%s?", m_faktury[static_cast<size_t>(m_selected)].popis(), m_faktury[static_cast<size_t>(m_selected)].cislo()),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        idbmodel->deleteFaktura(m_faktury[static_cast<size_t>(m_selected)].id());
    }
}

void VydanaPage::OnUhraditFaktura(wxCommandEvent &/*event*/)
{
    UhradaDialog dialog(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()));
    dialog.ShowModal();
}

void VydanaPage::OnPrintKnihaFaktur(wxCommandEvent& /*event*/)
{
    KnihaFakturPrint print(this, true);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void VydanaPage::OnExportKnihaFaktur(wxCommandEvent& /*event*/)
{
    wxFileDialog dlg(this, "Exportovat knihu faktur do", wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxString delim = ",";
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << "Doklad" << delim << "Popis" << delim << wxString::FromUTF8("Odběratel") << delim << "Vystavena" << delim
         << wxString::FromUTF8("Splatná") << delim << "Celkem" << delim << "Uhradit" << delim << wxString::FromUTF8("Měna") << endl;
    dxFakturaArray faktury = idbmodel->vystavene();
    for(size_t i=0; i<faktury.GetCount(); i++)
    {
        text << dxutils::enquotes(faktury[i].cislo(), delim);
        text << delim;
        text << dxutils::enquotes(faktury[i].popis(), delim);
        text << delim;
        text << dxutils::enquotes(idbmodel->getKontakt(faktury[i].protistranaid()).jmeno(), delim);
        text << delim;
        text << faktury[i].vystaveni().FormatDate();
        text << delim;
        text << faktury[i].splatnost().FormatDate();
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.2f", faktury[i].celkem()), delim);
        text << delim;
        text << dxutils::enquotes(wxString::Format("%.2f", faktury[i].celkem()-idbmodel->uhrazenoByFakturaId(faktury[i].id())), delim);
        text << delim;
        text << idbmodel->zkratkaMena(faktury[i].menaid());
        text << endl;
    }
}

void VydanaPage::OnItemRightClick(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    wxMenu popup;
    popup.Append(ID_PAGE_VYDANA_ADD, wxString::FromUTF8("Nová faktura"));
    if(m_selected != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_PAGE_VYDANA_EDIT, "Upravit fakturu");
        popup.Append(ID_PAGE_VYDANA_COPY, wxString::FromUTF8("Kopírovat fakturu"));
        popup.Append(ID_PAGE_VYDANA_DELETE, "Smazat fakturu");
        if(!idbmodel->isFakturaUhrazena(m_faktury[static_cast<size_t>(m_selected)].id()))
        {
            popup.AppendSeparator();
            popup.Append(ID_PAGE_VYDANA_UHRADA, "Uhradit fakturu");
        }
        if(idbmodel->someUhradaForFaktura(m_faktury[static_cast<size_t>(m_selected)].id()))
        {
            popup.AppendSeparator();
            popup.Append(ID_PAGE_VYDANA_UHRADY, wxString::FromUTF8("Zobrazit úhrady"));
        }
        popup.AppendSeparator();
        popup.Append(ID_PAGE_VYDANA_FAPRINT, "Vytisknout fakturu");
        popup.Append(ID_PAGE_VYDANA_FAEMAIL, "Poslat fakturu e-mailem");
    }
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void VydanaPage::OnItemActivated(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
    FakturaDialog dialog(m_faktury[static_cast<size_t>(m_selected)].id(), this);
    dialog.ShowModal();
}

void VydanaPage::OnItemSelected(wxListEvent &event)
{
    m_selected = event.GetIndex();
    enableTools();
}

void VydanaPage::OnItemDeselected(wxListEvent& /*event*/)
{
    m_selected = -1;
    enableTools();
}

void VydanaPage::OnColumnClick(wxListEvent &event)
{
    if(0 > event.GetColumn() || event.GetColumn() >= VydanaCtrl::VCOL_MAX) return;
    setColumnImage(m_sortCol, -1);
    m_sortCol = static_cast<VydanaCtrl::VColumn>(event.GetColumn());
    m_sortAsc = !m_sortAsc;
    setColumnImage(m_sortCol, m_sortAsc?0:1);
    if(m_faktury.GetCount()) idbmodel->sortFakturyView(m_faktury, columnToFakturaSort(m_sortCol), m_sortAsc);
    refreshItems();
}

void VydanaPage::enableTools()
{
    if(m_selected == -1)
    {
        m_bar->EnableTool(ID_PAGE_VYDANA_EDIT, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_COPY, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_DELETE, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_FAPRINT, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_FAPDF, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_FAEMAIL, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_UHRADA, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_UHRADY, false);
    }
    else
    {
        m_bar->EnableTool(ID_PAGE_VYDANA_EDIT, true);
        m_bar->EnableTool(ID_PAGE_VYDANA_COPY, true);
        m_bar->EnableTool(ID_PAGE_VYDANA_DELETE, true);
        m_bar->EnableTool(ID_PAGE_VYDANA_FAPRINT, true);
        m_bar->EnableTool(ID_PAGE_VYDANA_FAPDF, true);
        m_bar->EnableTool(ID_PAGE_VYDANA_FAEMAIL, true);
        m_bar->EnableTool(ID_PAGE_VYDANA_UHRADA, idbmodel->isFakturaUhrazena(m_faktury[static_cast<size_t>(m_selected)].id())?false:true);
        m_bar->EnableTool(ID_PAGE_VYDANA_UHRADY, idbmodel->someUhradaForFaktura(m_faktury[static_cast<size_t>(m_selected)].id()));
    }
}

DbModel::FZPSort VydanaPage::columnToFakturaSort(VydanaCtrl::VColumn column)
{
    switch(column)
    {
    case VydanaCtrl::VCOL_DOKLAD: return DbModel::FZP_DOKLAD;
    case VydanaCtrl::VCOL_POPIS: return DbModel::FZP_POPIS;
    case VydanaCtrl::VCOL_ODBERRATEL: return DbModel::FZP_PROTISTRANA;
    case VydanaCtrl::VCOL_VYSTAVENA: return DbModel::FZP_VYSTAVENA;
    case VydanaCtrl::VCOL_SPLATNA: return DbModel::FZP_SPLATNA;
    case VydanaCtrl::VCOL_CELKEM: return DbModel::FZP_CELKEM;
    case VydanaCtrl::VCOL_UHRADIT: return DbModel::FZP_UHRADIT;
    default: return DbModel::FZP_DOKLAD;
    }
}

void VydanaPage::setColumnImage(VydanaCtrl::VColumn column, int image)
{
    wxListItem item;
    item.SetMask(wxLIST_MASK_IMAGE);
    item.SetImage(image);
    m_list->SetColumn(column, item);
}

void VydanaPage::OnPrintFaktura(wxCommandEvent &/*event*/)
{
    bool faktura = m_faktury[static_cast<size_t>(m_selected)].typid() == 1;
    wxArrayString choices;
    if(faktura && dxsettings.fakturaSablona() != "internal") choices.Add(wxString::FromUTF8("Poslední použitá šablona"));
    if(!faktura && dxsettings.zalohaSablona() != "internal") choices.Add(wxString::FromUTF8("Poslední použitá šablona"));
    choices.Add(wxString::FromUTF8("Výchozí šablona programu"));
    choices.Add("Ze souboru");
    wxSingleChoiceDialog vyber(this, wxString::FromUTF8("Vyberte typ šablony"), wxString::FromUTF8("Typ šablony"), choices);
    vyber.SetSelection(0);
    if(vyber.ShowModal() == wxID_OK)
    {
        if(vyber.GetStringSelection() == wxString::FromUTF8("Výchozí šablona programu"))
        {
            if(faktura) dxsettings.setFakturaSablona("internal");
            else dxsettings.setZalohaSablona("internal");
            previewInternal();
        }
        else if(vyber.GetStringSelection() == "Ze souboru")
        {
            wxFileDialog dialog(m_parent, wxString::FromUTF8("Otevřít šablonu"), wxGetHomeDir(), wxEmptyString, "XML soubor (*.xml)|*.xml", wxFD_OPEN|wxFD_FILE_MUST_EXIST);
            if(dialog.ShowModal() == wxID_OK)
            {
                wxString path = dialog.GetPath();
                if(!path.empty())
                {
                    if(faktura) dxsettings.setFakturaSablona(path);
                    else dxsettings.setZalohaSablona(path);
                    FakturaXmlPrint print(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()), path);
                    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
                    print.Print();
#else
                    print.Preview();
#endif
                }
                else
                {
                    if(faktura) dxsettings.setFakturaSablona("internal");
                    else dxsettings.setZalohaSablona("internal");
                    previewInternal();
                }
            }
            else
            {
                if(faktura) dxsettings.setFakturaSablona("internal");
                else dxsettings.setZalohaSablona("internal");
                previewInternal();
            }
        }
        else
        {
            if(wxFileExists(faktura ? dxsettings.fakturaSablona() : dxsettings.zalohaSablona()))
            {
                FakturaXmlPrint print(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()), faktura ? dxsettings.fakturaSablona() : dxsettings.zalohaSablona());
                //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
                print.Print();
#else
                print.Preview();
#endif
            }
            else
            {
                if(faktura) dxsettings.setFakturaSablona("internal");
                else dxsettings.setZalohaSablona("internal");
                previewInternal();
            }
        }
    }
    else
    {
        if(faktura) dxsettings.setFakturaSablona("internal");
        else dxsettings.setZalohaSablona("internal");
        previewInternal();
    }
}

void VydanaPage::OnEmailFaktura(wxCommandEvent &/*event*/)
{
    SendEmailDialog dialog(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()));
    dialog.ShowModal();
}

void VydanaPage::OnSaveFaktura(wxCommandEvent &/*event*/)
{
    bool faktura = m_faktury[static_cast<size_t>(m_selected)].typid() == 1;
    wxArrayString choices;
    if(faktura && dxsettings.fakturaSablona() != "internal") choices.Add(wxString::FromUTF8("Poslední použitá šablona"));
    if(!faktura && dxsettings.zalohaSablona() != "internal") choices.Add(wxString::FromUTF8("Poslední použitá šablona"));
    choices.Add(wxString::FromUTF8("Výchozí šablona programu"));
    choices.Add("Ze souboru");
    wxSingleChoiceDialog vyber(this, wxString::FromUTF8("Vyberte typ šablony"), wxString::FromUTF8("Typ šablony"), choices);
    vyber.SetSelection(0);
    if(vyber.ShowModal() == wxID_OK)
    {
        if(vyber.GetStringSelection() == wxString::FromUTF8("Výchozí šablona programu"))
        {
            if(faktura) dxsettings.setFakturaSablona("internal");
            else dxsettings.setZalohaSablona("internal");
            saveInternal();
        }
        else if(vyber.GetStringSelection() == "Ze souboru")
        {
            wxFileDialog dialog(m_parent, wxString::FromUTF8("Otevřít šablonu"), wxGetHomeDir(), wxEmptyString, "XML soubor (*.xml)|*.xml", wxFD_OPEN|wxFD_FILE_MUST_EXIST);
            if(dialog.ShowModal() == wxID_OK)
            {
                wxString path = dialog.GetPath();
                if(!path.empty())
                {
                    if(faktura) dxsettings.setFakturaSablona(path);
                    else dxsettings.setZalohaSablona(path);
                    FakturaXmlPrint print(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()), path);
                    print.CreatePDF(m_faktury[static_cast<size_t>(m_selected)].cislo(), wxString::Format(wxString::FromUTF8("Faktura č. %s"), m_faktury[static_cast<size_t>(m_selected)].cislo()), true);
                }
                else
                {
                    if(faktura) dxsettings.setFakturaSablona("internal");
                    else dxsettings.setZalohaSablona("internal");
                    saveInternal();
                }
            }
            else
            {
                if(faktura) dxsettings.setFakturaSablona("internal");
                else dxsettings.setZalohaSablona("internal");
                saveInternal();
            }
        }
        else
        {
            if(wxFileExists(faktura ? dxsettings.fakturaSablona() : dxsettings.zalohaSablona()))
            {
                FakturaXmlPrint print(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()), faktura ? dxsettings.fakturaSablona() : dxsettings.zalohaSablona());
                print.CreatePDF(m_faktury[static_cast<size_t>(m_selected)].cislo(), wxString::Format(wxString::FromUTF8("Faktura č. %s"), m_faktury[static_cast<size_t>(m_selected)].cislo()), true);
            }
            else
            {
                if(faktura) dxsettings.setFakturaSablona("internal");
                else dxsettings.setZalohaSablona("internal");
                saveInternal();
            }
        }
    }
    else
    {
        if(faktura) dxsettings.setFakturaSablona("internal");
        else dxsettings.setZalohaSablona("internal");
        saveInternal();
    }
}

void VydanaPage::OnUhrady(wxCommandEvent &/*event*/)
{
    UhradyDialog dialog(this, m_faktury[static_cast<size_t>(m_selected)].id(), -1);
    dialog.ShowModal();
}

void VydanaPage::previewInternal()
{
    if(m_faktury[static_cast<size_t>(m_selected)].platce())
    {
        FakturaPlatcePrint print(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()));
        //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
        print.Print();
#else
        print.Preview();
#endif
    }
    else
    {
        FakturaNeplatcePrint print(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()));
        //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
        print.Print();
#else
        print.Preview();
#endif
    }
}

void VydanaPage::saveInternal()
{
    if(m_faktury[static_cast<size_t>(m_selected)].platce())
    {
        FakturaPlatcePrint print(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()));
        print.CreatePDF(m_faktury[static_cast<size_t>(m_selected)].cislo(), wxString::Format(wxString::FromUTF8("Faktura č. %s"), m_faktury[static_cast<size_t>(m_selected)].cislo()), true);
    }
    else
    {
        FakturaNeplatcePrint print(this, idbmodel->getFaktura(m_faktury[static_cast<size_t>(m_selected)].id()));
        print.CreatePDF(m_faktury[static_cast<size_t>(m_selected)].cislo(), wxString::Format(wxString::FromUTF8("Faktura č. %s"), m_faktury[static_cast<size_t>(m_selected)].cislo()), true);
    }
}

void VydanaPage::OnFilter(wxCommandEvent &/*event*/)
{
    VydanaFilter filter(this,m_filters);
    if(filter.ShowModal() == wxID_OK)
    {
        m_filters = filter.filters();
        m_filter = false;
        for(size_t i=0; i<m_filters.GetCount(); i++)
        {
            if(m_filters[i].active())
            {
                m_filter = true;
                break;
            }
        }
        m_bar->ToggleTool(ID_PAGE_VYDANA_FILTER, m_filter);
        m_bar->EnableTool(ID_PAGE_VYDANA_FILTERCLEAR, m_filter);
        if(m_list->GetColumnCount() == static_cast<int>(m_filters.GetCount()))
        {
            for(size_t i=0; i<m_filters.GetCount(); i++)
            {
                m_list->SetColumnHeaderFiltered(static_cast<int>(i), m_filters[i].active());
            }
            m_list->Update();
        }
    }
    else
    {
        m_filter = false;
        m_bar->ToggleTool(ID_PAGE_VYDANA_FILTER, false);
        m_bar->EnableTool(ID_PAGE_VYDANA_FILTERCLEAR, false);
        for(int i=0; i<m_list->GetColumnCount(); i++)
        {
            m_list->SetColumnHeaderFiltered(i, false);
        }
        m_list->Update();
    }
    updateData(ID_MODEL_ALL);
}

void VydanaPage::OnFilterClear(wxCommandEvent &/*event*/)
{
    m_bar->ToggleTool(ID_PAGE_VYDANA_FILTER, false);
    m_filter = false;
    m_bar->EnableTool(ID_PAGE_VYDANA_FILTERCLEAR, false);
    for(int i=0; i<m_list->GetColumnCount(); i++)
    {
        m_list->SetColumnHeaderFiltered(i, false);
    }
    m_list->Update();
    updateData(ID_MODEL_ALL);
}

void VydanaPage::getData()
{
    if(m_filter)
    {
        m_faktury.Clear();
        dxFakturaViewArray faktury = idbmodel->vystaveneview();
        for(size_t i=0; i<faktury.GetCount(); i++)
        {
            if(m_filters[0].matchFilter(faktury[i].cislo()) && m_filters[1].matchFilter(faktury[i].popis()) && m_filters[2].matchFilter(faktury[i].protistrana()) && m_filters[3].matchFilter(faktury[i].vystaveni())
                    && m_filters[4].matchFilter(faktury[i].splatnost()) && m_filters[5].matchFilter(faktury[i].celkem()) && m_filters[6].matchFilter(faktury[i].celkem()-faktury[i].uhrazeno()))
            {
                m_faktury.Add(faktury[i]);
            }
        }
    }
    else
    {
        m_faktury = idbmodel->vystaveneview();
    }
}

void VydanaPage::refreshItems()
{
    long oldcnt = m_list->GetItemCount();
    long cnt = static_cast<long>(m_faktury.GetCount());
    if(cnt)
    {
        if(m_selected >= cnt)
        {
            m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
            m_selected = 0;
        }
        if(oldcnt <= cnt) m_list->RefreshItems(0, --cnt);
        else m_list->Refresh();
    }
    else
    {
        m_list->Refresh();
    }
}

