/***************************************************************
 * Name:      bankaknihaprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "bankaknihaprint.h"

BankaKnihaPrint::BankaKnihaPrint(wxWindow *parent)
    : MultiTablePrint(parent, wxString::FromUTF8("Bankovní kniha"))
{
    createTable();
}

void BankaKnihaPrint::createTable()
{
    m_headerLeftText = idbmodel->firma().nazev();
    m_headerCenterText = wxString::FromUTF8("Bankovní kniha");
    m_headerRightText = wxString::Format(wxString::FromUTF8("Účetní rok %d"), idbmodel->currentRok());
    m_headerLine = true;
    dxBankaArray banky = idbmodel->banky();
    for(size_t i=0; i<banky.GetCount(); i++)
    {
        bool cizi = banky[i].menaid() != 1;
        double prijem = 0.0;
        double vydaj = 0.0;
        double zustatek = idbmodel->getBankaZustatek(banky[i].id());
        Table table(7);
        table.setTableTitle(TableTitle(banky[i].zkratka()+", "+banky[i].popis()+", "+idbmodel->zkratkaMena(banky[i].menaid())));
        table.addCellToHeader("Datum");
        table.addCellToHeader("Doklad");
        table.addCellToHeader(wxString::FromUTF8("Výpis"));
        table.addCellToHeader("Popis");
        table.addCellToHeader(wxString::FromUTF8("Příjem"));
        table.addCellToHeader(wxString::FromUTF8("Výdaj"));
        table.addCellToHeader(wxString::FromUTF8("Zůstatek"));
        dxDenikArray denik = idbmodel->denikBankaById(banky[i].id());
        idbmodel->sortDenik(denik, DbModel::DS_DATUM, true);
        for(size_t j=0; j<denik.GetCount(); ++j)
        {
            table.addRow();
            Denik de = denik.Item(j);
            table.addCell(de.datum().FormatDate());
            table.addCell(de.cislo());
            table.addCell(wxString::Format("%d", de.vypis()), wxTEXT_ALIGNMENT_RIGHT);
            table.addCell(de.popis());
            if(idbmodel->isPredkontacePrijem(de.predkontaceid()))
            {
                table.addCell(wxString::Format("%.2f", cizi?de.castkacizi():de.castka()), wxTEXT_ALIGNMENT_RIGHT);
                table.addCell("");
                prijem += cizi?de.castkacizi():de.castka();
                zustatek += cizi?de.castkacizi():de.castka();
            }
            else
            {
                table.addCell("");
                table.addCell(wxString::Format("%.2f", cizi?de.castkacizi():de.castka()), wxTEXT_ALIGNMENT_RIGHT);
                vydaj += cizi?de.castkacizi():de.castka();
                zustatek -= cizi?de.castkacizi():de.castka();
            }
            table.addCell(wxString::Format("%.2f", zustatek), wxTEXT_ALIGNMENT_RIGHT);
        }
        TextStyle style(wxFont(wxFontInfo(8).FaceName(FACENAME).Bold()));
        table.addRow();
        table.addCell("Celkem", wxTEXT_ALIGNMENT_LEFT, style);
        table.addCell("");
        table.addCell("");
        table.addCell("");
        table.addCell(wxString::Format("%.2f", prijem), wxTEXT_ALIGNMENT_RIGHT, style);
        table.addCell(wxString::Format("%.2f", vydaj), wxTEXT_ALIGNMENT_RIGHT, style);
        table.addCell(wxString::Format("%.2f", zustatek), wxTEXT_ALIGNMENT_RIGHT, style);
        table.setColumnWidth(3,75);
        table.refreshCellsSize();
        m_tables->Add(table);
    }
}
