/***************************************************************
 * Name:      fulldenikprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "fulldenikprint.h"

FullDenikPrint::FullDenikPrint(wxWindow *parent)
    : SplitTablePrint(parent, wxString::FromUTF8("Peněžní deník"))
{
    createTable();
}

void FullDenikPrint::createTable()
{
    m_headerLeftText = idbmodel->firma().nazev();
    m_headerCenterText = wxString::FromUTF8("Peněžní deník");
    m_headerRightText = wxString::Format(wxString::FromUTF8("Účetní rok %d"), idbmodel->currentRok());
    m_headerLine = true;
    m_table = new Table(36);
    m_table->setStyle(TextStyle(wxFont(wxFontInfo(7).FaceName(FACENAME))));
    dxTableColumnsSplitArray splits;
    m_table->addCellToHeader("Datum");
    m_table->addCellToHeader("Doklad");
    m_table->addCellToHeader("Popis");
    m_table->addCellToHeader(wxString::FromUTF8("Pokladna\nPříjem"));
    m_table->addCellToHeader(wxString::FromUTF8("Pokladna\nVýdaj"));
    m_table->addCellToHeader(wxString::FromUTF8("Banka\nPříjem"));
    m_table->addCellToHeader(wxString::FromUTF8("Banka\nVýdaj"));
    m_table->addCellToHeader(wxString::FromUTF8("Daňový\nPříjem"));
    m_table->addCellToHeader(wxString::FromUTF8("Daňový\nVýdaj"));
    splits.Add(TableColumnsSplit(0,8));
    m_table->addCellToHeader("Datum");
    m_table->addCellToHeader("Doklad");
    m_table->addCellToHeader(wxString::FromUTF8("Prodej\nzboží"));
    m_table->addCellToHeader(wxString::FromUTF8("Prodej\nvýrobků a\nslužeb"));
    m_table->addCellToHeader(wxString::FromUTF8("Ostatní"));
    m_table->addCellToHeader(wxString::FromUTF8("Nákup\nmateriálu"));
    m_table->addCellToHeader(wxString::FromUTF8("Nákup\nzboží"));
    m_table->addCellToHeader(wxString::FromUTF8("Nákup\ndrobného\nmajetku"));
    m_table->addCellToHeader("Mzdy");
    m_table->addCellToHeader(wxString::FromUTF8("Daň ze\nmzdy"));
    m_table->addCellToHeader(wxString::FromUTF8("Sociální\npojištění"));
    m_table->addCellToHeader(wxString::FromUTF8("Zdravotní\npojištění"));
    m_table->addCellToHeader(wxString::FromUTF8("Režie"));
    m_table->addCellToHeader(wxString::FromUTF8("Ostatní"));
    splits.Add(TableColumnsSplit(9,22));
    m_table->addCellToHeader("Datum");
    m_table->addCellToHeader("Doklad");
    m_table->addCellToHeader("Vklad\npodnikatele");
    m_table->addCellToHeader(wxString::FromUTF8("DPH\nPříjem"));
    m_table->addCellToHeader(wxString::FromUTF8("Ostatní"));
    m_table->addCellToHeader(wxString::FromUTF8("Nákup\ndlouhodobého\nmajetku"));
    m_table->addCellToHeader(wxString::FromUTF8("DPH\nVýdaj"));
    m_table->addCellToHeader(wxString::FromUTF8("Sociální a\nzdravotní\npojištění"));
    m_table->addCellToHeader(wxString::FromUTF8("Daň z\npříjmu"));
    m_table->addCellToHeader(wxString::FromUTF8("Osobní"));
    m_table->addCellToHeader(wxString::FromUTF8("Ostatní"));
    m_table->addCellToHeader(wxString::FromUTF8("Průběžné\npoložky\nPříjem"));
    m_table->addCellToHeader(wxString::FromUTF8("Průběžné\npoložky\nVýdaj"));
    splits.Add(TableColumnsSplit(23,35));
    m_table->setColumnsSplit(splits);
    double sloupce[29] = {};
    dxDenikArray denik = idbmodel->denik();
    for(size_t i=0; i<denik.GetCount(); i++)
    {
        m_table->addRow();
        Denik de = denik[i];
        m_table->addCell(de.datum().FormatDate());
        m_table->addCell(de.cislo());
        m_table->addCell(de.popis());
        if(idbmodel->hasPokladna(de.pokladnaid()))
        {
            if(idbmodel->isPredkontacePrijem(de.predkontaceid()))
            {
                sloupce[0] += de.castka();
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
                m_table->addCell("");
            }
            else
            {
                sloupce[1] += de.castka();
                m_table->addCell("");
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
            }
            m_table->addCell("");
            m_table->addCell("");
        }
        else
        {
            m_table->addCell("");
            m_table->addCell("");
            if(idbmodel->isPredkontacePrijem(de.predkontaceid()))
            {
                sloupce[2] += de.castka();
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
                m_table->addCell("");
            }
            else
            {
                sloupce[3] += de.castka();
                m_table->addCell("");
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
            }
        }
        if(idbmodel->isPredkontaceDanova(de.predkontaceid()))
        {
            if(idbmodel->isPredkontacePrijem(de.predkontaceid()))
            {
                sloupce[4] += de.castka();
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
                m_table->addCell("");
            }
            else
            {
                sloupce[5] += de.castka();
                m_table->addCell("");
                m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
            }
        }
        else
        {
            m_table->addCell("");
            m_table->addCell("");
        }
        m_table->addCell(de.datum().FormatDate());
        m_table->addCell(de.cislo());
        Predkontace pr = idbmodel->getPredkontace(de.predkontaceid());
        if(pr.isValid())
        {
            for(int j=6; j<18; j++)
            {
                if(pr.deniksloupecid() == j+4)
                {
                    sloupce[j] += de.castka();
                    m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
                }
                else
                {
                    m_table->addCell("");
                }
            }
        }
        m_table->addCell(de.datum().FormatDate());
        m_table->addCell(de.cislo());
        if(pr.isValid())
        {
            for(int j=18; j<29; j++)
            {
                if(pr.deniksloupecid() == j+4)
                {
                    sloupce[j] += de.castka();
                    m_table->addCell(wxString::Format("%.2f", de.castka()), wxTEXT_ALIGNMENT_RIGHT);
                }
                else
                {
                    m_table->addCell("");
                }
            }
        }
    }
    TextStyle style(wxFont(wxFontInfo(7).FaceName(FACENAME).Bold()));
    m_table->addRow();
    m_table->addCell("Celkem", wxTEXT_ALIGNMENT_LEFT, style);
    m_table->addCell("");
    m_table->addCell("");
    for(int i=0; i<6; i++)
    {
        m_table->addCell(sloupce[i] ? wxString::Format("%.2f", sloupce[i]) : "", wxTEXT_ALIGNMENT_RIGHT, style);
    }
    m_table->addCell("Celkem", wxTEXT_ALIGNMENT_LEFT, style);
    m_table->addCell("");
    for(int i=6; i<18; i++)
    {
        m_table->addCell(sloupce[i] ? wxString::Format("%.2f", sloupce[i]) : "", wxTEXT_ALIGNMENT_RIGHT, style);
    }
    m_table->addCell("Celkem", wxTEXT_ALIGNMENT_LEFT, style);
    m_table->addCell("");
    for(int i=18; i<29; i++)
    {
        m_table->addCell(sloupce[i] ? wxString::Format("%.2f", sloupce[i]) : "", wxTEXT_ALIGNMENT_RIGHT, style);
    }
    m_table->setColumnWidth(2,40);
    m_table->refreshCellsSize();
}
