/***************************************************************
 * Name:      kartazasobyprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "kartazasobyprint.h"
#include "utils/dxutils.h"

KartaZasobyPrint::KartaZasobyPrint(wxWindow *parent, const Zasoba &zasoba)
    : DocumentPrint(parent, wxString::FromUTF8("Karta zásoby")), m_zasoba(zasoba)
{
    m_marginLeft = 10-PAGE_OFFSET;
    m_marginRight = 10+PAGE_OFFSET;
    m_marginTop = 10-PAGE_OFFSET;
    m_marginBottom = 10;
    createDocument();
}

void KartaZasobyPrint::createDocument()
{
    int pageHeight = m_pageHeight-m_marginTop-m_marginBottom-10;
    int height = 0;
    int page = 1;
    m_document = new Document;
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    TextElement text = TextElement(1, wxString::Format(wxString::FromUTF8("Karta zásoby č.%s"), m_zasoba.cislo()),
                                wxPoint(m_marginLeft,m_marginTop), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_RIGHT,
                                TextStyle(wxFont(wxFontInfo(16).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    height += text.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    Firma fi = idbmodel->firma();
    Adresa ad = idbmodel->getAdresa(fi.adresaid());
    wxArrayString firma;
    firma.Add(fi.nazev());
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.psc(), ad.misto()) : " ");
    firma.Add(ad.isValid() ? ad.zeme() : " ");
    firma.Add(" ");
    firma.Add(wxString::Format(wxString::FromUTF8("IČ: %s"), fi.ic()));
    firma.Add(wxString::Format(wxString::FromUTF8("DIČ: %s"), fi.dic()));
    TextParagraphElement paragraph = TextParagraphElement(1, firma, wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    paragraph.setStyle(m_document->style());
    m_document->addElement(paragraph);
    height += paragraph.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    text = TextElement(1, wxString::Format(wxString::FromUTF8("Název: %s"), m_zasoba.nazev()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    m_document->addElement(text);
    height += text.height();
    wxArrayString texty;
    texty.Add("Popis:");
    texty.Add(m_zasoba.popis());
    wxArrayInt width;
    width.Add(15);
    width.Add(m_pageWidth-m_marginLeft-m_marginRight-15);
    TableRowElement row = TableRowElement(1, texty, wxPoint(m_marginLeft,m_marginTop+height), width);
    row.setStyle(m_document->style());
    height += row.height();
    if(height > pageHeight)
    {
        height -= row.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        row.setPage(page);
        row.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(row);
        height = row.height();
    }
    else
    {
        m_document->addElement(row);
    }
    text = TextElement(page, wxString::Format(wxString::FromUTF8("Zásoba: %.3f %s"), m_zasoba.pocet(), m_zasoba.mj()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    height += text.height();
    if(height > pageHeight)
    {
        height -= text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        text.setPage(page);
        text.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(text);
        height = text.height();
    }
    else
    {
        m_document->addElement(text);
    }
    text = TextElement(page, wxString::Format(wxString::FromUTF8("Hodnota zásoby: %.2f Kč"), m_zasoba.celkem()), wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    height += text.height();
    if(height > pageHeight)
    {
        height -= text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        text.setPage(page);
        text.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(text);
        height = text.height();
    }
    else
    {
        m_document->addElement(text);
    }
    text = TextElement(page, wxString::Format("Sklad: %s", idbmodel->getSkladNazevById(m_zasoba.skladid())), wxPoint(m_marginLeft,m_marginTop+height),
                       m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    height += text.height();
    if(height > pageHeight)
    {
        height -= text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        text.setPage(page);
        text.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(text);
        height = text.height();
    }
    else
    {
        m_document->addElement(text);
    }
    text = TextElement(page, m_zasoba.ocenovani() ? wxString::FromUTF8("Oceňování: Vážený aritmetický průměr") : wxString::FromUTF8("Oceňování: FIFO"), wxPoint(m_marginLeft,m_marginTop+height),
                       m_pageWidth-m_marginLeft-m_marginRight);
    text.setStyle(m_document->style());
    height += text.height();
    if(height > pageHeight)
    {
        height -= text.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        text.setPage(page);
        text.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(text);
        height = text.height();
    }
    else
    {
        m_document->addElement(text);
    }
    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    texty.Clear();
    texty.Add("Datum");
    texty.Add(wxString::FromUTF8("Číslo\ndokladu"));
    texty.Add(wxString::FromUTF8("Číslo\nfaktury/\ninventury"));
    texty.Add("Popis");
    texty.Add(wxString::Format("Pohyb (%s)", m_zasoba.mj()));
    texty.Add(wxString::Format("Cena za %s", m_zasoba.mj()));
    width.Clear();
    width.Add(30);
    width.Add(20);
    width.Add(25);
    width.Add(m_pageWidth-m_marginLeft-m_marginRight-135);
    width.Add(30);
    width.Add(30);
    wxArrayInt alig;
    alig.Add(wxTEXT_ALIGNMENT_LEFT);
    alig.Add(wxTEXT_ALIGNMENT_LEFT);
    alig.Add(wxTEXT_ALIGNMENT_LEFT);
    alig.Add(wxTEXT_ALIGNMENT_LEFT);
    alig.Add(wxTEXT_ALIGNMENT_RIGHT);
    alig.Add(wxTEXT_ALIGNMENT_RIGHT);
    row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
    row.setStyle(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Italic())));
    row.setBorder(true);
    height += row.height();
    if(height > pageHeight)
    {
        height -= row.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        row.setPage(page);
        row.setPos(wxPoint(m_marginLeft,m_marginTop));
        m_document->addElement(row);
        height = row.height();
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    }
    else
    {
        m_document->addElement(row);
    }
    dxPohybzasobArray pohyby = idbmodel->pohybyzasobByZasobaId(m_zasoba.id());
    for(size_t i=0; i < pohyby.GetCount(); i++)
    {
        texty.Clear();
        texty.Add(pohyby[i].datum().Format("%d.%m.%Y %H:%M:%S"));
        texty.Add(pohyby[i].cislo());
        if(pohyby[i].inventura()) texty.Add(idbmodel->getInventuraCisloById(pohyby[i].inventuraid()));
        else texty.Add(idbmodel->getFakturaCisloById(pohyby[i].fakturaid()));
        wxString popis;
        if(pohyby[i].pocatek())
        {
            popis = wxString::FromUTF8("Počáteční stav");
        }
        else
        {
            if(pohyby[i].inventura())
            {
                popis = wxString::FromUTF8("Zaevidování inventurního stavu");
            }
            else
            {
                if(pohyby[i].prijemka()) popis = wxString::FromUTF8("Příjem na sklad");
                else popis = wxString::FromUTF8("Výdej ze skladu");
            }
        }
        texty.Add(popis);
        texty.Add(pohyby[i].prijemka() ? wxString::Format("+%.3f", pohyby[i].pocet()) : wxString::Format("-%.3f", pohyby[i].pocet()));
        texty.Add(pohyby[i].prijemka() ? wxString::Format("%.2f", pohyby[i].cenamj()): "");
        row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
        row.setStyle(m_document->style());
        row.setBorder(true);
        height += row.height();
        if(height > pageHeight)
        {
            height -= row.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            page++;
            row.setPage(page);
            row.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(row);
            height = row.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
        }
        else
        {
            m_document->addElement(row);
        }
    }
    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->setPages(page);
}
