/***************************************************************
 * Name:      zavpohprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "zavpohprint.h"
#include "utils/dxutils.h"

ZavpohPrint::ZavpohPrint(wxWindow *parent, const ZavPoh &zavpoh)
    : DocumentPrint(parent, zavpoh.zavazek()?wxString::FromUTF8("Závazek"):wxString::FromUTF8("Pohledávka")), m_zavpoh(zavpoh)
{
    m_marginLeft = 10-PAGE_OFFSET;
    m_marginRight = 10+PAGE_OFFSET;
    m_marginTop = 10-PAGE_OFFSET;
    m_marginBottom = 10;
    createDocument();
}

void ZavpohPrint::createDocument()
{
    int halfPage = wxRound(((double)m_pageWidth)/2.0);
    int pageHeight = m_pageHeight-m_marginTop-m_marginBottom-10;
    int height = 0;
    int page = 1;
    m_document = new Document;
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    wxString titul;
    if(m_zavpoh.zavazek())
    {
        titul = wxString::Format(wxString::FromUTF8("Závazek č. %s"), m_zavpoh.cislo());
    }
    else
    {
        titul = wxString::Format(wxString::FromUTF8("Pohledávka č. %s"), m_zavpoh.cislo());
    }
    TextElement text = TextElement(1, titul, wxPoint(m_marginLeft,m_marginTop), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_RIGHT,
                                   TextStyle(wxFont(wxFontInfo(16).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    height += text.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height),
                                       wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    Firma fi = idbmodel->firma();
    int firmaHeight = 0;
    Adresa ad = idbmodel->getAdresa(fi.adresaid());
    if(!m_zavpoh.zavazek())
    {
        text = TextElement(1, "Dodavatel:", wxPoint(m_marginLeft,m_marginTop+height+firmaHeight), halfPage-m_marginLeft,
                           wxTEXT_ALIGNMENT_LEFT,
                           TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        firmaHeight += text.height();
    }
    else
    {
        text = TextElement(1, wxString::FromUTF8("Odběratel:"),
                           wxPoint(m_marginLeft,m_marginTop+height+firmaHeight), halfPage-m_marginLeft, wxTEXT_ALIGNMENT_LEFT,
                           TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        firmaHeight += text.height();
    }
    wxArrayString firma;
    firma.Add(fi.nazev());
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.psc(), ad.misto()) : " ");
    firma.Add(ad.isValid() ? ad.zeme() : " ");
    firma.Add(wxString::Format(wxString::FromUTF8("IČ: %s"), fi.ic()));
    firma.Add(wxString::Format(wxString::FromUTF8("DIČ: %s"), fi.dic()));
    firma.Add(wxString::Format("Telefon: %s", fi.telefon()));
    firma.Add(wxString::Format("E-mail: %s", fi.email()));
    firma.Add(wxString::Format("WWW: %s", fi.www()));
    if(!m_zavpoh.platce()) firma.Add(wxString::FromUTF8("Neplátce DPH"));
    firma.Add(wxString::Format("Registrace: %s", fi.registrace()));
    TextParagraphElement paragraph = TextParagraphElement(1, firma, wxPoint(m_marginLeft,m_marginTop+height+firmaHeight), halfPage-m_marginLeft);
    paragraph.setStyle(m_document->style());
    m_document->addElement(paragraph);
    firmaHeight += paragraph.height();
    int kontaktHeight = 0;
    if(!m_zavpoh.zavazek())
    {
        text = TextElement(1, wxString::FromUTF8("Odběratel:"), wxPoint(halfPage,m_marginTop+height), halfPage-m_marginRight,
                           wxTEXT_ALIGNMENT_LEFT,
                           TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        kontaktHeight += text.height();
    }
    else
    {
        text = TextElement(1, "Dodavatel:", wxPoint(halfPage,m_marginTop+height), halfPage-m_marginRight, wxTEXT_ALIGNMENT_LEFT,
                           TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        kontaktHeight += text.height();
    }
    wxArrayString kontakt;
    Kontakt ko = idbmodel->getKontakt(m_zavpoh.protistranaid());
    ad = idbmodel->getAdresa(ko.isValid() ? ko.adresaid() : -1);
    kontakt.Add(ko.isValid() ? ko.jmeno() : " ");
    kontakt.Add(ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    kontakt.Add(ad.isValid() ? wxString::Format("%s %s", ad.psc(), ad.misto()) : " ");
    kontakt.Add(ad.isValid() ? ad.zeme() : " ");
    kontakt.Add(" ");
    kontakt.Add(" ");
    kontakt.Add(ko.isValid() ? wxString::Format(wxString::FromUTF8("IČ: %s"), ko.ic()) : " ");
    kontakt.Add(ko.isValid() ? wxString::Format(wxString::FromUTF8("DIČ: %s"), ko.dic()) : " ");
    paragraph = TextParagraphElement(1, kontakt, wxPoint(halfPage,m_marginTop+height+kontaktHeight), halfPage-m_marginRight);
    paragraph.setStyle(m_document->style());
    m_document->addElement(paragraph);
    kontaktHeight = paragraph.height();
    m_document->addElement(LineElement(1, wxPoint(halfPage,m_marginTop+height), wxPoint(halfPage,m_marginTop+wxMax(height+firmaHeight,kontaktHeight))));
    height = wxMax(height+firmaHeight,kontaktHeight);
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    text = TextElement(1, wxString::FromUTF8("Platební podmínky:"), wxPoint(m_marginLeft,m_marginTop+height), halfPage-m_marginLeft,
                       wxTEXT_ALIGNMENT_LEFT,
                       TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    height += text.height();
    FaUhrada uh = idbmodel->getFaUhrada(m_zavpoh.uhradaid());
    wxArrayString podminky;
    podminky.Add(wxString::Format(wxString::FromUTF8("Datum vystavení: %s"), m_zavpoh.vystaveni().FormatDate()));
    podminky.Add(wxString::Format("Datum splatnosti: %s", m_zavpoh.splatnost().FormatDate()));
    if(m_zavpoh.platce()) podminky.Add(wxString::Format(wxString::FromUTF8("Datum uskutečnění zdanitelného plnění: %s"), m_zavpoh.duzp().FormatDate()));
    podminky.Add(uh.isValid()?wxString::Format(wxString::FromUTF8("Forma úhrady: %s"), uh.uhrada()):" ");
    if(uh.isValid() && !uh.pokladnou())
    {
        Banka ba = idbmodel->getBanka(m_zavpoh.bankaid());
        podminky.Add(ba.isValid()?wxString::Format(wxString::FromUTF8("Bankovní spojení: %s/%s"), ba.cislouctu(), ba.kodbanky()):" ");
        podminky.Add(wxString::Format(wxString::FromUTF8("Variabilní symbol: %s"), m_zavpoh.vs()));
    }
    paragraph = TextParagraphElement(1, podminky, wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight);
    paragraph.setStyle(m_document->style());
    m_document->addElement(paragraph);
    height += paragraph.height();
    m_document->addElement(LineElement(1, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    text = TextElement(1, wxString::FromUTF8("Položky:"), wxPoint(m_marginLeft,m_marginTop+height), halfPage-m_marginLeft,
                       wxTEXT_ALIGNMENT_LEFT,
                       TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    height += text.height();
    if(m_zavpoh.platce())
    {
        wxArrayString texty;
        texty.Add("Popis");
        texty.Add(wxString::FromUTF8("Množství (mj)"));
        texty.Add(wxString::Format("Cena za mj bez DPH (%s)", idbmodel->zkratkaMena(m_zavpoh.menaid())));
        texty.Add(wxString::FromUTF8("Sazba DPH (%)"));
        texty.Add(wxString::Format("DPH (%s)", idbmodel->zkratkaMena(m_zavpoh.menaid())));
        texty.Add(wxString::Format("Cena celkem (%s)", idbmodel->zkratkaMena(m_zavpoh.menaid())));
        wxArrayInt width;
        width.Add(m_pageWidth-m_marginLeft-m_marginRight-130);
        width.Add(20);
        width.Add(30);
        width.Add(20);
        width.Add(25);
        width.Add(35);
        wxArrayInt alig;
        alig.Add(wxTEXT_ALIGNMENT_LEFT);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        TableRowElement row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
        row.setStyle(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Italic())));
        height += row.height();
        if(height > pageHeight)
        {
            height -= row.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            page++;
            row.setPage(page);
            row.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(row);
            height = row.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
        }
        else
        {
            m_document->addElement(row);
        }
        dxFaPolozkaArray polozky = idbmodel->fapolozkyByZavpohId(m_zavpoh.id());
        for(size_t i=0; i<polozky.GetCount(); i++)
        {
            texty.Clear();
            texty.Add(polozky[i].popis());
            texty.Add(wxString::Format("%.3f%s", polozky[i].pocet(), polozky[i].mj()));
            texty.Add(wxString::Format("%.2f", dxutils::cenamj(polozky[i].cenamj(), polozky[i].bezdph(), polozky[i].sazbaid())));
            texty.Add(idbmodel->hasDphSazba(polozky[i].sazbaid()) ? wxString::Format("%d", idbmodel->getDphSazba(polozky[i].sazbaid()).sazba()) : " ");
            texty.Add(wxString::Format("%.2f", polozky[i].dph()));
            texty.Add(wxString::Format("%.2f", polozky[i].celkem()));
            row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
            row.setStyle(m_document->style());
            height += row.height();
            if(height > pageHeight)
            {
                height -= row.height();
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                page++;
                row.setPage(page);
                row.setPos(wxPoint(m_marginLeft,m_marginTop));
                m_document->addElement(row);
                height = row.height();
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
            }
            else
            {
                m_document->addElement(row);
            }
        }
        height += 3;
        text = TextElement(page, wxString::Format(wxString::FromUTF8("CELKEM K ÚHRADĚ: %.2f %s"), m_zavpoh.celkem(), idbmodel->zkratkaMena(m_zavpoh.menaid())),
                           wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_CENTER,
                           TextStyle(wxFont(wxFontInfo(14).FaceName(FACENAME).Bold())));
        height += text.height();
        if(height > pageHeight)
        {
            height -= text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            page++;
            text.setPage(page);
            text.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(text);
            height = text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
        }
        else
        {
            m_document->addElement(text);
        }
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
        text = TextElement(page, wxString::FromUTF8("Rekapitulace DPH:"), wxPoint(m_marginLeft,m_marginTop+height), halfPage-m_marginLeft,
                           wxTEXT_ALIGNMENT_LEFT,
                           TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        m_document->addElement(text);
        height += text.height();
        if(height > pageHeight)
        {
            height -= text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            page++;
            text.setPage(page);
            text.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(text);
            height = text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
        }
        else
        {
            m_document->addElement(text);
        }
        texty.Clear();
        width.Clear();
        alig.Clear();
        texty.Add("Sazba DPH");
        texty.Add(wxString::Format(wxString::FromUTF8("Základ (%s)"), idbmodel->zkratkaMena(m_zavpoh.menaid())));
        texty.Add(wxString::Format("DPH (%s)", idbmodel->zkratkaMena(m_zavpoh.menaid())));
        width.Add(63);
        width.Add(63);
        width.Add(m_pageWidth-m_marginLeft-m_marginRight-126);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
        row.setStyle(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Italic())));
        height += row.height();
        if(height > pageHeight)
        {
            height -= row.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            page++;
            row.setPage(page);
            row.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(row);
            height = row.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
        }
        else
        {
            m_document->addElement(row);
        }
        dxFaDphSazbaArray souhrn = idbmodel->dphZavPohSouhrn(m_zavpoh.id());
        for(size_t i=0; i<souhrn.GetCount(); i++)
        {
            if(souhrn[i].zaklad())
            {
                texty.Clear();
                texty.Add(wxString::Format("%d%%", souhrn[i].sazba()));
                texty.Add(wxString::Format("%.2f", souhrn[i].zaklad()));
                texty.Add(wxString::Format("%.2f", souhrn[i].dph()));
                row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
                row.setStyle(m_document->style());
                height += row.height();
                if(height > pageHeight)
                {
                    height -= row.height();
                    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                    m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                    page++;
                    row.setPage(page);
                    row.setPos(wxPoint(m_marginLeft,m_marginTop));
                    m_document->addElement(row);
                    height = row.height();
                    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
                }
                else
                {
                    m_document->addElement(row);
                }
            }
        }
        if(idbmodel->dphPlatiZakaznik(m_zavpoh.dphdruhid()))
        {
            text = TextElement(page, wxString::FromUTF8("Daň odvede odběratel."), wxPoint(m_marginLeft,m_marginTop+height), halfPage-m_marginLeft,
                               wxTEXT_ALIGNMENT_LEFT,
                               TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
            height += text.height();
            if(height > pageHeight)
            {
                height -= text.height();
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                page++;
                text.setPage(page);
                text.setPos(wxPoint(m_marginLeft,m_marginTop));
                m_document->addElement(text);
                height = text.height();
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
            }
            else
            {
                m_document->addElement(text);
            }
        }
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    }
    else
    {
        wxArrayString texty;
        texty.Add("Popis");
        texty.Add(wxString::FromUTF8("Množství (mj)"));
        texty.Add(wxString::Format("Cena za mj (%s)", idbmodel->zkratkaMena(m_zavpoh.menaid())));
        texty.Add(wxString::Format("Cena celkem (%s)", idbmodel->zkratkaMena(m_zavpoh.menaid())));
        wxArrayInt width;
        width.Add(m_pageWidth-m_marginLeft-m_marginRight-95);
        width.Add(30);
        width.Add(30);
        width.Add(35);
        wxArrayInt alig;
        alig.Add(wxTEXT_ALIGNMENT_LEFT);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        alig.Add(wxTEXT_ALIGNMENT_CENTER);
        TableRowElement row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
        row.setStyle(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
        height += row.height();
        if(height > pageHeight)
        {
            height -= row.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            page++;
            row.setPage(page);
            row.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(row);
            height = row.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
        }
        else
        {
            m_document->addElement(row);
        }
        dxFaPolozkaArray polozky = idbmodel->fapolozkyByZavpohId(m_zavpoh.id());
        for(size_t i=0; i<polozky.GetCount(); i++)
        {
            texty.Clear();
            texty.Add(polozky[i].popis());
            texty.Add(wxString::Format("%.3f%s", polozky[i].pocet(), polozky[i].mj()));
            texty.Add(wxString::Format("%.2f", dxutils::cenamj(polozky[i].cenamj(), polozky[i].bezdph(), polozky[i].sazbaid())));
            texty.Add(wxString::Format("%.2f", polozky[i].celkem()));
            row = TableRowElement(page, texty, wxPoint(m_marginLeft,m_marginTop+height), width, alig);
            row.setStyle(m_document->style());
            height += row.height();
            if(height > pageHeight)
            {
                height -= row.height();
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                page++;
                row.setPage(page);
                row.setPos(wxPoint(m_marginLeft,m_marginTop));
                m_document->addElement(row);
                height = row.height();
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
            }
            else
            {
                m_document->addElement(row);
            }
        }
        height += 3;
        text = TextElement(page, wxString::Format(wxString::FromUTF8("CELKEM K ÚHRADĚ: %.2f %s"), m_zavpoh.celkem(), idbmodel->zkratkaMena(m_zavpoh.menaid())),
                           wxPoint(m_marginLeft,m_marginTop+height), m_pageWidth-m_marginLeft-m_marginRight, wxTEXT_ALIGNMENT_CENTER,
                           TextStyle(wxFont(wxFontInfo(14).FaceName(FACENAME).Bold())));
        height += text.height();
        if(height > pageHeight)
        {
            height -= text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
            page++;
            text.setPage(page);
            text.setPos(wxPoint(m_marginLeft,m_marginTop));
            m_document->addElement(text);
            height = text.height();
            m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
        }
        else
        {
            m_document->addElement(text);
        }
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height)));
    }
    bool razitkoMezera = true;
    int qrheight = 0;
    if(uh.isValid() && !uh.pokladnou() && !m_zavpoh.zavazek())
    {
        Banka ba = idbmodel->getBanka(m_zavpoh.bankaid());
        if(!ba.iban().IsEmpty())
        {
            wxString qrcode = "SPD*1.0";
            if(ba.bic().IsEmpty()) qrcode << "*ACC:" << ba.iban();
            else qrcode << "*ACC:" << ba.iban() << "+" << ba.bic();
            qrcode << "*AM:" << wxString::FromCDouble(m_zavpoh.celkem());
            qrcode << "*CC:" << idbmodel->kodMena(m_zavpoh.menaid());
            qrcode << "*DT:" << m_zavpoh.splatnost().Format("%Y%m%d");
            qrcode << "*X-VS:" << m_zavpoh.vs();
            ImageElement image = ImageElement(page, qrcode, wxPoint(m_marginLeft+2,m_marginTop+height+2));
            qrheight = image.height()+1;
            if(height+qrheight > pageHeight)
            {
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                page++;
                image.setPage(page);
                image.setPos(wxPoint(m_marginLeft+2,m_marginTop+2));
                m_document->addElement(image);
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
            }
            else
            {
                m_document->addElement(image);
            }
            razitkoMezera = false;
        }
    }
    if(!m_zavpoh.zavazek())
    {
        if(wxFileExists(fi.razitko()))
        {
            ImageElement image = ImageElement(page, fi.razitko(), wxPoint(halfPage,m_marginTop+height+1), halfPage-m_marginRight, wxTEXT_ALIGNMENT_CENTER);
            if(height+wxMax(image.height()+1,qrheight) > pageHeight)
            {
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
                page++;
                height = wxMax(image.height()+1,qrheight);
                image.setPage(page);
                image.setPos(wxPoint(halfPage,m_marginTop+height-1-image.height()));
                m_document->addElement(image);
                m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
            }
            else
            {
                height += wxMax(image.height()+1,qrheight);
                image.setPos(wxPoint(halfPage,m_marginTop+height-1-image.height()));
                m_document->addElement(image);
            }
            razitkoMezera = false;
        }
        else
        {
            height += qrheight;
        }
        if(razitkoMezera)
        {
            height += 30;
        }
        text = TextElement(page, wxString::FromUTF8("razítko, podpis"), wxPoint(halfPage,m_marginTop+height), halfPage-m_marginRight, wxTEXT_ALIGNMENT_CENTER);
        text.setStyle(m_document->style());
        height += text.height();
    }
    if(height > pageHeight)
    {
        height -= text.height();
        if(!wxFileExists(fi.razitko())) height -= 30;
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
        page++;
        text.setPage(page);
        text.setPos(wxPoint(halfPage,m_marginTop+30));
        m_document->addElement(text);
        height = text.height()+(wxFileExists(fi.razitko())?0:30);
        m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop), LineStyle(*wxBLACK, BOLD_WIDTH)));
    }
    else
    {
        m_document->addElement(text);
    }
    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop+height), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->addElement(LineElement(page, wxPoint(m_marginLeft,m_marginTop), wxPoint(m_marginLeft,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->addElement(LineElement(page, wxPoint(m_pageWidth-m_marginRight,m_marginTop), wxPoint(m_pageWidth-m_marginRight,m_marginTop+height), LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document->setPages(page);
}
