/***************************************************************
 * Name:      atribut.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "atribut.h"
#include <wx/arrimpl.cpp>

WX_DEFINE_OBJARRAY(dxAtributArray)

Atribut::Atribut()
    : m_type(A_NONE), m_name(wxEmptyString), m_popis(wxEmptyString), m_required(false), m_length(0), m_fraction(0), m_format(wxEmptyString),
      m_value(wxEmptyString)
{
}

Atribut::Atribut(ATRIBUTTYPE type, const wxString &name, bool required, int length, int fraction, const wxString &format)
    : m_type(type), m_name(name), m_popis(wxEmptyString), m_required(required), m_length(length), m_fraction(fraction), m_format(format),
      m_value(wxEmptyString)
{
}

ATRIBUTTYPE Atribut::type() const
{
    return m_type;
}

void Atribut::setType(const ATRIBUTTYPE &type)
{
    m_type = type;
}

wxString Atribut::name() const
{
    return m_name;
}

void Atribut::setName(const wxString &name)
{
    m_name = name;
}

wxString Atribut::popis() const
{
    return m_popis;
}

void Atribut::setPopis(const wxString &popis)
{
    m_popis = popis;
}

bool Atribut::required() const
{
    return m_required;
}

void Atribut::setRequired(bool required)
{
    m_required = required;
}

int Atribut::length() const
{
    return m_length;
}

void Atribut::setLength(int length)
{
    m_length = length;
}

int Atribut::fraction() const
{
    return m_fraction;
}

void Atribut::setFraction(int fraction)
{
    m_fraction = fraction;
}

wxString Atribut::format() const
{
    return m_format;
}

void Atribut::setFormat(const wxString &format)
{
    m_format = format;
}

wxString Atribut::value() const
{
    return m_value;
}

int Atribut::valueInt() const
{
    if(m_type == A_INT)
    {
        return wxAtoi(m_value);
    }
    return 0;
}

double Atribut::valueDouble() const
{
    if(m_type == A_DOUBLE)
    {
        double value = 0.0;
        m_value.ToCDouble(&value);
        return value;
    }
    return 0.0;
}

wxDateTime Atribut::valueDatum() const
{
    if(m_type == A_DATUM)
    {
        wxDateTime dt;
        wxString::const_iterator end;
        if(!dt.ParseFormat(m_value, m_format, &end))
        {
            wxDateTime();
        }
        else if(end == m_value.end())
        {
            return dt;
        }
        else
        {
            wxDateTime();
        }
    }
    return wxDateTime();
}

void Atribut::setValue(const wxString &value)
{
    if(value.Length() <= static_cast<size_t>(m_length)) m_value = value;
}

void Atribut::setValue(int value)
{
    if(m_type == A_INT)
    {
        wxString str = wxString::Format("%d", value);
        if(str.Length() <= static_cast<size_t>(m_length)) m_value = str;
    }
}

void Atribut::setValue(double value)
{
    if(m_type == A_DOUBLE)
    {
        wxString str = wxString::FromCDouble(value, m_fraction);
        if(str.Length() <= static_cast<size_t>(m_length)) m_value = str;
    }
}

void Atribut::setValue(const wxDateTime &value)
{
    if(m_type == A_DATUM)
    {
        wxString str = value.Format(m_format);
        if(str.Length() <= static_cast<size_t>(m_length)) m_value = str;
    }
}

bool Atribut::isForXml()
{
    return !m_value.IsEmpty();
}

void Atribut::clearValue()
{
    m_value = wxEmptyString;
}
