/***************************************************************
 * Name:      dphdp3wizard.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/hyperlink.h>
#include "dphdp3wizard.h"
#include "priznanidialog.h"
#include "dphdp3print.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "config.h"

BEGIN_EVENT_TABLE(Dp3Page1, wxWizardPageSimple)
    EVT_BUTTON(ID_DPHDP3_PAGE1_PRIZNANI, Dp3Page1::OnPriznani)
END_EVENT_TABLE()

Dp3Page1::Dp3Page1(wxWizard *parent)
    : wxWizardPageSimple(parent)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Průvodce Vás provede vytvořením\nPřiznání k DPH")));
    m_mainSizer->AddStretchSpacer();
    m_priznani = new wxButton(this, ID_DPHDP3_PAGE1_PRIZNANI, wxString::FromUTF8("Nastavení plátce"));
    m_mainSizer->Add(m_priznani, 0, wxALL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void Dp3Page1::OnPriznani(wxCommandEvent &/*event*/)
{
    PriznaniDialog dialog(this, true);
    dialog.ShowModal();
}

BEGIN_EVENT_TABLE(Dp3Page2, wxWizardPageSimple)
    EVT_RADIOBUTTON(ID_DPHDP3_PAGE2_MESIC, Dp3Page2::OnMesic)
    EVT_RADIOBUTTON(ID_DPHDP3_PAGE2_CTVRTLETI, Dp3Page2::OnCtvrtleti)
    EVT_RADIOBUTTON(ID_DPHDP3_PAGE2_ODDO, Dp3Page2::OnOddo)
    EVT_DATE_CHANGED(ID_DPHDP3_PAGE2_OD, Dp3Page2::OnOd)
    EVT_DATE_CHANGED(ID_DPHDP3_PAGE2_DO, Dp3Page2::OnDo)
    EVT_CHOICE(ID_DPHDP3_PAGE2_MESICSELECT, Dp3Page2::OnMesicChoice)
    EVT_CHOICE(ID_DPHDP3_PAGE2_CTVRTLETISELECT, Dp3Page2::OnCtvrtletiChoice)
    EVT_BUTTON(ID_DPHDP3_PAGE2_KOEF, Dp3Page2::OnKoefP20Vypor)
END_EVENT_TABLE()

Dp3Page2::Dp3Page2(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphdp3Wizard*>(parent)), m_koefP20VyporValue(0.0), m_vyporOdp(0.0)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    m_mesicBtn = new wxRadioButton(this, ID_DPHDP3_PAGE2_MESIC, wxString::FromUTF8("Měsíc"));
    m_mesicBtn->SetValue(false);
    gridSizer->Add(m_mesicBtn, wxGBPosition(0,0), wxDefaultSpan, wxEXPAND);
    m_mesicSelect = new wxChoice(this, ID_DPHDP3_PAGE2_MESICSELECT);
    m_mesicSelect->Append(dxutils::mesicString(1));
    m_mesicSelect->Append(dxutils::mesicString(2));
    m_mesicSelect->Append(dxutils::mesicString(3));
    m_mesicSelect->Append(dxutils::mesicString(4));
    m_mesicSelect->Append(dxutils::mesicString(5));
    m_mesicSelect->Append(dxutils::mesicString(6));
    m_mesicSelect->Append(dxutils::mesicString(7));
    m_mesicSelect->Append(dxutils::mesicString(8));
    m_mesicSelect->Append(dxutils::mesicString(9));
    m_mesicSelect->Append(dxutils::mesicString(10));
    m_mesicSelect->Append(dxutils::mesicString(11));
    m_mesicSelect->Append(dxutils::mesicString(12));
    m_mesicSelect->SetSelection(wxMax(wxDateTime::Today().GetMonth()-1,0));
    m_mesicSelect->Enable(false);
    gridSizer->Add(m_mesicSelect, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    m_mesicRok = new wxChoice(this, wxID_ANY);
    m_mesicRok->Append(idbmodel->getObdobiAsStrings());
    m_mesicRok->SetStringSelection(wxString::Format("%d", idbmodel->currentRok()));
    m_mesicRok->Enable(false);
    gridSizer->Add(m_mesicRok, wxGBPosition(0,2), wxDefaultSpan, wxEXPAND);

    m_ctvrtletiBtn = new wxRadioButton(this, ID_DPHDP3_PAGE2_CTVRTLETI, wxString::FromUTF8("Čtvrtletí"));
    m_ctvrtletiBtn->SetValue(true);
    gridSizer->Add(m_ctvrtletiBtn, wxGBPosition(1,0), wxDefaultSpan, wxEXPAND);
    m_ctvrtletiSelect = new wxChoice(this, ID_DPHDP3_PAGE2_CTVRTLETISELECT);
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(1));
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(2));
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(3));
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(4));
    m_ctvrtletiSelect->SetSelection(wxMax(wxDateTime::Today().GetMonth()/3-1,0));
    gridSizer->Add(m_ctvrtletiSelect, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    m_ctvrtletiRok = new wxChoice(this, wxID_ANY);
    m_ctvrtletiRok->Append(idbmodel->getObdobiAsStrings());
    m_ctvrtletiRok->SetStringSelection(wxString::Format("%d", idbmodel->currentRok()));
    gridSizer->Add(m_ctvrtletiRok, wxGBPosition(1,2), wxDefaultSpan, wxEXPAND);

    m_oddoBtn = new wxRadioButton(this, ID_DPHDP3_PAGE2_ODDO, wxString::FromUTF8("Období"));
    m_oddoBtn->SetValue(false);
    gridSizer->Add(m_oddoBtn, wxGBPosition(2,0), wxDefaultSpan, wxEXPAND);
    m_od = new wxDatePickerCtrl(this, ID_DPHDP3_PAGE2_OD, wxDateTime(1, wxDateTime::Jan, idbmodel->currentRok()), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_od->Enable(false);
    gridSizer->Add(m_od, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    m_do = new wxDatePickerCtrl(this, ID_DPHDP3_PAGE2_DO, wxDateTime(31, wxDateTime::Jan, idbmodel->currentRok()), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_do->Enable(false);
    gridSizer->Add(m_do, wxGBPosition(2,2), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Datum vyhotovení")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_vyhotoveni = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Today(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    gridSizer->Add(m_vyhotoveni, wxGBPosition(3,1), wxGBSpan(1,2), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Kód zdaňovacího období\nnásledujícího roku")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kodZo = new wxChoice(this, wxID_ANY);
    m_kodZo->Append(wxString::FromUTF8("Beze změny"));
    m_kodZo->Append(wxString::FromUTF8("Q - čtvrletně"));
    m_kodZo->Append(wxString::FromUTF8("M - měsíčně"));
    m_kodZo->SetSelection(0);
    m_kodZo->Enable(false);
    gridSizer->Add(m_kodZo, wxGBPosition(4,1), wxGBSpan(1,2), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("ř.52 - Koeficient (%)")), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_koefP20Nov = new wxSpinCtrlDouble(this);
    m_koefP20Nov->SetValue(100.0);
    m_koefP20Nov->SetDigits(2);
    gridSizer->Add(m_koefP20Nov, wxGBPosition(5,1), wxGBSpan(1,2), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("ř.53 - Vypořádací koeficient(%)")), wxGBPosition(6,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_koefP20Vypor = new wxButton(this, ID_DPHDP3_PAGE2_KOEF, wxString::FromUTF8("Vypočítat z evidence"));
    m_koefP20Vypor->Enable(false);
    gridSizer->Add(m_koefP20Vypor, wxGBPosition(6,1), wxGBSpan(1,2), wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

wxDateTime Dp3Page2::vyhotoveni() const
{
    return m_vyhotoveni->GetValue();
}

//0 - mesic, 1 - ctvrtleti, 2 - oddo
int Dp3Page2::obdobi() const
{
    if(m_mesicBtn->GetValue()) return 0;
    if(m_ctvrtletiBtn->GetValue()) return 1;
    return 2;
}

int Dp3Page2::mesic() const
{
    return m_mesicBtn->GetValue()?m_mesicSelect->GetSelection()+1:0;
}

int Dp3Page2::ctvrtleti() const
{
    return m_ctvrtletiBtn->GetValue()?m_ctvrtletiSelect->GetSelection()+1:0;
}

int Dp3Page2::rok() const
{
    if(obdobi() == 0) return wxAtoi(m_mesicRok->GetStringSelection());
    if(obdobi() == 1) return wxAtoi(m_ctvrtletiRok->GetStringSelection());
    return m_do->GetValue().GetYear();
}

wxDateTime Dp3Page2::start() const
{
    if(obdobi() == 0) return wxDateTime(1, static_cast<wxDateTime::Month>(mesic()-1), rok());
    if(obdobi() == 1) return wxDateTime(1, dxutils::ctvrtletiMesicStart(ctvrtleti()), rok());
    return m_od->GetValue();
}

wxDateTime Dp3Page2::end() const
{
    if(obdobi() == 0) return wxDateTime(start().GetLastMonthDay(static_cast<wxDateTime::Month>(mesic()-1), rok()));
    if(obdobi() == 1) return wxDateTime(start().GetLastMonthDay(dxutils::ctvrtletiMesicEnd(ctvrtleti()), rok()));
    return m_do->GetValue();
}

wxString Dp3Page2::kodZo() const
{
    if(m_kodZo->IsEnabled())
    {
        if(m_kodZo->GetSelection() == 1) return "Q";
        if(m_kodZo->GetSelection() == 2) return "M";
        return "";
    }
    return "";
}

double Dp3Page2::koefP20Nov() const
{
    return m_koefP20Nov->GetValue();
}

double Dp3Page2::koefP20Vypor() const
{
    return m_koefP20VyporValue;
}

double Dp3Page2::vyporOdp() const
{
    return m_vyporOdp;
}

void Dp3Page2::OnMesic(wxCommandEvent &/*event*/)
{
    m_ctvrtletiSelect->Enable(false);
    m_ctvrtletiRok->Enable(false);
    m_mesicSelect->Enable(true);
    m_mesicRok->Enable(true);
    m_od->Enable(false);
    m_do->Enable(false);
    m_mesicSelect->SetSelection(0);
    m_kodZo->Enable(false);
    m_koefP20Vypor->Enable(false);
}

void Dp3Page2::OnCtvrtleti(wxCommandEvent &/*event*/)
{
    m_mesicSelect->Enable(false);
    m_mesicRok->Enable(false);
    m_ctvrtletiSelect->Enable(true);
    m_ctvrtletiRok->Enable(true);
    m_od->Enable(false);
    m_do->Enable(false);
    m_ctvrtletiSelect->SetSelection(0);
    m_kodZo->Enable(false);
    m_koefP20Vypor->Enable(false);
}

void Dp3Page2::OnOddo(wxCommandEvent &/*event*/)
{
    m_mesicSelect->Enable(false);
    m_mesicRok->Enable(false);
    m_ctvrtletiSelect->Enable(false);
    m_ctvrtletiRok->Enable(false);
    m_od->Enable(true);
    m_do->Enable(true);
    m_kodZo->Enable(false);
    m_koefP20Vypor->Enable(false);
}

void Dp3Page2::OnOd(wxDateEvent &/*event*/)
{
    if(m_do->GetValue().IsEarlierThan(m_od->GetValue())) m_do->SetValue(m_od->GetValue().GetLastMonthDay(m_od->GetValue().GetMonth(), m_od->GetValue().GetYear()));
}

void Dp3Page2::OnDo(wxDateEvent &/*event*/)
{
    if(m_od->GetValue().IsLaterThan(m_do->GetValue())) m_od->SetValue(wxDateTime(1, m_do->GetValue().GetMonth(), m_do->GetValue().GetYear()));
}

void Dp3Page2::OnMesicChoice(wxCommandEvent &/*event*/)
{
    if(m_mesicSelect->GetSelection() == 11)
    {
        m_kodZo->Enable(true);
        m_koefP20Vypor->Enable(true);
    }
    else
    {
        m_kodZo->Enable(false);
        m_koefP20Vypor->Enable(false);
    }
}

void Dp3Page2::OnCtvrtletiChoice(wxCommandEvent &/*event*/)
{
    if(m_ctvrtletiSelect->GetSelection() == 3)
    {
        m_kodZo->Enable(true);
        m_koefP20Vypor->Enable(true);
    }
    else
    {
        m_kodZo->Enable(false);
        m_koefP20Vypor->Enable(false);
    }
}

void Dp3Page2::OnKoefP20Vypor(wxCommandEvent &/*event*/)
{
    if(idbmodel->someUlozeneDphDp3(rok(), false))
    {
        dxUlozeneArray ulozene = idbmodel->ulozeneDphDp3(rok(), false);
        wxArrayString choices;
        for(size_t i=0; i < ulozene.GetCount(); i++)
        {
            choices.Add(ulozene[i].popis());
        }
        wxArrayInt selections;
        int const count = wxGetSelectedChoices(selections, wxString::FromUTF8("Vyberte přiznání k DPH,\nzahrnuté do výpočtu vypořádácího koeficientu"), wxString::FromUTF8("Přiznání k DPH"), choices);
        if(count)
        {
            wxBeginBusyCursor();
#if __WXGTK__
            wxYield();
#endif
            double citatel = 0.0, jmenovatel = 0.0, odp_uprav_kf = 0.0, odp_sum_kr = 0.0;
            for(size_t i=0; i < selections.GetCount(); i++)
            {
                Dphdp3 dp;
                dp.readFromXml(ulozene[i].xml());
                odp_sum_kr += dp.atributValueDouble("odp_sum_kr");
                odp_uprav_kf += dp.atributValueDouble("odp_uprav_kf");
                citatel += dp.atributValueDouble("obrat23"); //radek 1
                citatel += dp.atributValueDouble("obrat5"); //radek 2
                citatel += dp.atributValueDouble("dod_zb"); //radek 20
                citatel += dp.atributValueDouble("pln_sluzby"); //radek 21
                citatel += dp.atributValueDouble("pln_vyvoz"); //radek 22
                citatel += dp.atributValueDouble("dod_dop_nrg"); //radek 23
                citatel += dp.atributValueDouble("pln_zaslani"); //radek 24
                citatel += dp.atributValueDouble("pln_rez_pren"); //radek 25
                citatel += dp.atributValueDouble("pln_ost"); //radek 26
                citatel += dp.atributValueDouble("tri_dozb"); //radek 31
                citatel += dp.atributValueDouble("pln_nkf"); //radek 51 sloupec „S nárokem na odpočet daně“
                jmenovatel += dp.atributValueDouble("plnosv_kf"); //radek 50
                jmenovatel += dp.atributValueDouble("plnosv_nkf"); //radek 51 sloupec „Bez nároku na odpočet“
            }
            m_wizard->fillDataForKoefP20Vypor();
            Dphdp3 dp = m_wizard->dphdp3();
            odp_sum_kr += dp.atributValueDouble("odp_sum_kr");
            odp_uprav_kf += wxRound(dp.atributValueDouble("odp_sum_kr")*m_koefP20Nov->GetValue()/100.0);
            citatel += dp.atributValueDouble("obrat23"); //radek 1
            citatel += dp.atributValueDouble("obrat5"); //radek 2
            citatel += dp.atributValueDouble("dod_zb"); //radek 20
            citatel += dp.atributValueDouble("pln_sluzby"); //radek 21
            citatel += dp.atributValueDouble("pln_vyvoz"); //radek 22
            citatel += dp.atributValueDouble("dod_dop_nrg"); //radek 23
            citatel += dp.atributValueDouble("pln_zaslani"); //radek 24
            citatel += dp.atributValueDouble("pln_rez_pren"); //radek 25
            citatel += dp.atributValueDouble("pln_ost"); //radek 26
            citatel += dp.atributValueDouble("tri_dozb"); //radek 31
            citatel += dp.atributValueDouble("pln_nkf"); //radek 51 sloupec „S nárokem na odpočet daně“
            jmenovatel += citatel;
            jmenovatel += dp.atributValueDouble("plnosv_kf"); //radek 50
            jmenovatel += dp.atributValueDouble("plnosv_nkf"); //radek 51 sloupec „Bez nároku na odpočet“
            m_koefP20VyporValue = citatel/jmenovatel;
            m_vyporOdp = m_koefP20VyporValue*odp_sum_kr - odp_uprav_kf;
            m_koefP20VyporValue *= 100.0;
            wxEndBusyCursor();
        }
    }
    else
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Nejsou žádná uložená přiznání k DPH"), "Informace", wxOK|wxICON_INFORMATION);
        dialog.ShowModal();
        return;
    }
}

BEGIN_EVENT_TABLE(Dp3Page3, wxWizardPageSimple)
    EVT_RADIOBUTTON(ID_DPHDP3_PAGE3_RADNE, Dp3Page3::OnRadne)
    EVT_RADIOBUTTON(ID_DPHDP3_PAGE3_RADNEOPRAVNE, Dp3Page3::OnRadne)
    EVT_RADIOBUTTON(ID_DPHDP3_PAGE3_DODATECNE, Dp3Page3::OnDodatecne)
    EVT_RADIOBUTTON(ID_DPHDP3_PAGE3_DODATECNEOPRAVNE, Dp3Page3::OnDodatecne)
END_EVENT_TABLE()

Dp3Page3::Dp3Page3(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphdp3Wizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_radne = new wxRadioButton(this, ID_DPHDP3_PAGE3_RADNE, wxString::FromUTF8("Řádné přiznání"));
    m_radne->SetValue(true);
    m_mainSizer->Add(m_radne);

    m_radneOpravne = new wxRadioButton(this, ID_DPHDP3_PAGE3_RADNEOPRAVNE, wxString::FromUTF8("Řádné opravné přiznání"));
    m_mainSizer->Add(m_radneOpravne);

    m_dodatecne = new wxRadioButton(this, ID_DPHDP3_PAGE3_DODATECNE, wxString::FromUTF8("Dodatečné přiznání"));
    m_dodatecne->Enable(idbmodel->someUlozeneDphDp3(m_wizard->rok())?true:false);
    m_mainSizer->Add(m_dodatecne);

    m_dodatecneOpravne = new wxRadioButton(this, ID_DPHDP3_PAGE3_DODATECNEOPRAVNE, wxString::FromUTF8("Dodatečné opravné přiznání"));
    m_dodatecneOpravne->Enable(idbmodel->someUlozeneDphDp3(m_wizard->rok())?true:false);
    m_mainSizer->Add(m_dodatecneOpravne);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Dodatečné k:")));
    m_ulozene = new wxChoice(this, wxID_ANY);
    if(idbmodel->someUlozeneDphDp3(m_wizard->rok()))
    {
        dxUlozeneArray dp3 = idbmodel->ulozeneDphDp3(m_wizard->rok());
        idbmodel->sortUlozene(dp3);
        for(size_t i=0; i < dp3.GetCount(); i++)
        {
            m_ulozene->Append(dp3[i].popis(), new Ulozene(dp3[i].id(), dp3[i].typ(), dp3[i].popis(), dp3[i].forma(), dp3[i].obdobi(), dp3[i].mesic(), dp3[i].ctvrtleti(), dp3[i].rok(),
                                                          dp3[i].from(), dp3[i].to(), dp3[i].xml()));
        }
        m_ulozene->SetSelection(0);
    }
    m_ulozene->Enable(false);
    m_mainSizer->Add(m_ulozene);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Datum zjištění pro podání dodatečného přiznání:")));
    m_dzjist = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Today(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_dzjist->Enable(false);
    m_mainSizer->Add(m_dzjist);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Důvody pro podání dodatečného daňového přiznání:")));
    m_duvod = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxSize(350,80), wxTE_MULTILINE|wxHSCROLL);
    m_duvod->Enable(false);
    m_mainSizer->Add(m_duvod);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

wxString Dp3Page3::forma() const
{
    if(m_radne->GetValue()) return "B";
    if(m_radneOpravne->GetValue()) return "O";
    if(m_dodatecne->GetValue()) return "D";
    return "E";
}

wxDateTime Dp3Page3::start() const
{
    if(m_radne->GetValue() || m_radneOpravne->GetValue()) return wxDateTime::Today();
    else return static_cast<Ulozene*>(m_ulozene->GetClientData(static_cast<unsigned int>(m_ulozene->GetSelection())))->from();
}

wxDateTime Dp3Page3::end() const
{
    if(m_radne->GetValue() || m_radneOpravne->GetValue()) return wxDateTime::Today();
    else return static_cast<Ulozene*>(m_ulozene->GetClientData(static_cast<unsigned int>(m_ulozene->GetSelection())))->to();
}

int Dp3Page3::ulozeneID()
{
    if(m_radne->GetValue() || m_radneOpravne->GetValue()) return -1;
    else return static_cast<Ulozene*>(m_ulozene->GetClientData(static_cast<unsigned int>(m_ulozene->GetSelection())))->id();
}

wxDateTime Dp3Page3::dzjist() const
{
    return m_dzjist->GetValue();
}

wxString Dp3Page3::duvod() const
{
    return m_duvod->GetValue();
}

void Dp3Page3::OnRadne(wxCommandEvent &/*event*/)
{
    m_dodatecne->Enable(idbmodel->someUlozeneDphDp3(m_wizard->rok())?true:false);
    m_dodatecneOpravne->Enable(idbmodel->someUlozeneDphDp3(m_wizard->rok())?true:false);
    m_ulozene->Enable(false);
    m_dzjist->Enable(false);
    m_duvod->Enable(false);
}

void Dp3Page3::OnDodatecne(wxCommandEvent &/*event*/)
{
    m_ulozene->Enable(true);
    m_dzjist->Enable(true);
    m_duvod->Enable(true);
}

BEGIN_EVENT_TABLE(Dp3Page4, wxWizardPageSimple)
    EVT_BUTTON(ID_DPHDP3_PRINT, Dp3Page4::OnPrint)
    EVT_BUTTON(ID_DPHDP3_EXPORT, Dp3Page4::OnExport)
END_EVENT_TABLE()

Dp3Page4::Dp3Page4(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphdp3Wizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Přiznání k DHP můžete vytisknout nebo\nvyexportovat jako xml soubor.\n\nXml soubor můžete odeslat pomocí datové schránky"
                                                                         "\nnebo podat pomocí")));
    m_mainSizer->Add(new wxHyperlinkCtrl(this, wxID_ANY, "www.daneelektronicky.cz", "http://www.daneelektronicky.cz/"));

    m_mainSizer->AddStretchSpacer();

    wxBoxSizer *buttons = new wxBoxSizer(wxHORIZONTAL);
    m_print = new wxButton(this, ID_DPHDP3_PRINT, wxString::FromUTF8("Tisk přiznání"));
    buttons->Add(m_print);
    buttons->AddSpacer(5);
    m_export = new wxButton(this, ID_DPHDP3_EXPORT, wxString::FromUTF8("Export přiznání"));
    buttons->Add(m_export);
    m_mainSizer->Add(buttons, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void Dp3Page4::OnPrint(wxCommandEvent &/*event*/)
{
    m_wizard->fillData();
    Dphdp3Print print(this, m_wizard->atributy());
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void Dp3Page4::OnExport(wxCommandEvent &/*event*/)
{
    wxFileDialog dlg(this, wxString::FromUTF8("Exportovat přiznání do"), wxEmptyString, wxEmptyString, "XML soubory (*.xml)|*.xml",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("xml", path);
    m_wizard->fillData();
    m_wizard->saveXml(path);
}

IMPLEMENT_DYNAMIC_CLASS(Dphdp3Wizard, wxWizard)

BEGIN_EVENT_TABLE(Dphdp3Wizard, wxWizard)
    EVT_WIZARD_PAGE_CHANGED(wxID_ANY, Dphdp3Wizard::OnPageChanged)
    EVT_WIZARD_FINISHED(wxID_ANY, Dphdp3Wizard::OnFinished)
END_EVENT_TABLE()

Dphdp3Wizard::Dphdp3Wizard(wxWindow *parent)
    : wxWizard(parent, wxID_ANY, wxString::FromUTF8("Průvodce přiznáním k DPH"))
{
    m_page1 = new Dp3Page1(this);

    m_page2 = new Dp3Page2(this);
    m_page1->Chain(m_page2);

    m_page3 = new Dp3Page3(this);
    m_page2->Chain(m_page3);

    m_page4 = new Dp3Page4(this);
    m_page3->Chain(m_page4);

    GetPageAreaSizer()->Add(m_page1);
    GetPageAreaSizer()->Add(m_page2);
    GetPageAreaSizer()->Add(m_page3);
    GetPageAreaSizer()->Add(m_page4);
}

void Dphdp3Wizard::fillData()
{
    if(!m_dphdp3.isEmpty()) return;

    Priznani pr = idbmodel->priznani();

    //VetaD
    m_dphdp3.setAtributValue("c_okec", pr.c_okec());
    m_dphdp3.setAtributValue("d_poddp", m_page2->vyhotoveni());
    m_dphdp3.setAtributValue("dapdph_forma", m_page3->forma());
    m_dphdp3.setAtributValue("dokument", "DP3");
    m_dphdp3.setAtributValue("k_uladis", "DPH");
    m_dphdp3.setAtributValue("kod_zo", m_page2->kodZo());
    if(!idbmodel->someDokladForDph(m_page2->start(), m_page2->end()) && wxIsNullDouble(m_page2->vyporOdp())) m_dphdp3.setAtributValue("trans", "N");
    m_dphdp3.setAtributValue("typ_platce", "P");
    if(m_page3->forma() == "B" || m_page3->forma() == "O")
    {
        if(m_page2->obdobi() == 1) m_dphdp3.setAtributValue("ctvrt", m_page2->ctvrtleti());
        if(m_page2->obdobi() == 0) m_dphdp3.setAtributValue("mesic", m_page2->mesic());
        if(m_page2->obdobi() == 2) m_dphdp3.setAtributValue("zdobd_do", m_page2->end());
        if(m_page2->obdobi() == 2) m_dphdp3.setAtributValue("zdobd_od", m_page2->start());
        m_dphdp3.setAtributValue("rok", m_page2->rok());
        fillVeta1to6(m_page2->start(), m_page2->end());
        if(!wxIsNullDouble(m_page2->vyporOdp()))
        {
            m_dphdp3.setAtributValue("koef_p20_vypor", m_page2->koefP20Vypor());
            m_dphdp3.setAtributValue("vypor_odp", m_page2->vyporOdp());
        }
        fillVeta1to6Summary(m_page2->start(), m_page2->end());
    }
    else
    {
        m_dphdp3.setAtributValue("d_zjist", m_page3->dzjist());
        Dphdp3 dp3;
        dp3.readFromXml(idbmodel->ulozeneXml(m_page3->ulozeneID()));
        m_dphdp3.setAtributValue("ctvrt", dp3.atributValue("ctvrt"));
        m_dphdp3.setAtributValue("mesic", dp3.atributValue("mesic"));
        m_dphdp3.setAtributValue("zdobd_do", dp3.atributValue("zdobd_do"));
        m_dphdp3.setAtributValue("zdobd_od", dp3.atributValue("zdobd_od"));
        m_dphdp3.setAtributValue("rok", dp3.atributValue("rok"));
        fillVeta1to6(m_page3->start(), m_page3->end());
        m_dphdp3.fillDodatecne(dp3);
        m_dphdp3.addVetaR(m_page3->duvod());
    }

    //VetaP
    m_dphdp3.setAtributValue("c_orient", pr.c_orient());
    m_dphdp3.setAtributValue("c_pop", pr.c_pop());
    m_dphdp3.setAtributValue("c_pracufo", pr.c_pracufo());
    m_dphdp3.setAtributValue("c_telef", pr.c_telef());
    m_dphdp3.setAtributValue("c_ufo", pr.c_ufo());
    m_dphdp3.setAtributValue("dic", pr.dic());
    m_dphdp3.setAtributValue("email", pr.email());
    m_dphdp3.setAtributValue("jmeno", pr.jmeno());
    m_dphdp3.setAtributValue("naz_obce", pr.naz_obce());
    m_dphdp3.setAtributValue("opr_jmeno", pr.opr_jmeno());
    m_dphdp3.setAtributValue("opr_postaveni", pr.opr_postaveni());
    m_dphdp3.setAtributValue("opr_prijmeni", pr.opr_prijmeni());
    m_dphdp3.setAtributValue("prijmeni", pr.prijmeni());
    m_dphdp3.setAtributValue("psc", pr.psc());
    m_dphdp3.setAtributValue("stat", pr.stat());
    m_dphdp3.setAtributValue("titul", pr.titul());
    m_dphdp3.setAtributValue("typ_ds", "F");
    m_dphdp3.setAtributValue("ulice", pr.ulice());
    if(pr.zast_typ() == "F") m_dphdp3.setAtributValue("zast_dat_nar", pr.zast_dat_nar());
    m_dphdp3.setAtributValue("zast_ev_cislo", pr.zast_ev_cislo());
    if(pr.zast_typ() == "P") m_dphdp3.setAtributValue("zast_ic", pr.zast_ic());
    m_dphdp3.setAtributValue("zast_jmeno", pr.zast_jmeno());
    m_dphdp3.setAtributValue("zast_kod", pr.zast_kod());
    m_dphdp3.setAtributValue("zast_nazev", pr.zast_nazev());
    m_dphdp3.setAtributValue("zast_prijmeni", pr.zast_prijmeni());
    m_dphdp3.setAtributValue("zast_typ", pr.zast_typ());
}

dxAtributArray Dphdp3Wizard::atributy() const
{
    return m_dphdp3.atributy();
}

void Dphdp3Wizard::saveXml(const wxString &path)
{
    m_dphdp3.saveToXml(path);
}

int Dphdp3Wizard::rok() const
{
    return m_page2->rok();
}

void Dphdp3Wizard::fillDataForKoefP20Vypor()
{
    fillVeta1to6(m_page2->start(), m_page2->end());
    fillVeta1to6Summary(m_page2->start(), m_page2->end());
}

Dphdp3 Dphdp3Wizard::dphdp3() const
{
    return m_dphdp3;
}

void Dphdp3Wizard::setDphdp3(const Dphdp3 &dphdp3)
{
    m_dphdp3 = dphdp3;
}


void Dphdp3Wizard::OnPageChanged(wxWizardEvent &event)
{
    if(!event.GetDirection()) m_dphdp3.clearData();
}

void Dphdp3Wizard::OnFinished(wxWizardEvent &/*event*/)
{
    if(idbmodel->someDokladForDph(m_page2->start(), m_page2->end()) && (m_page3->forma() == "B" || m_page3->forma() == "O"))
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Uložit daňové přiznání?"), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_dphdp3.clearData();
            fillData();
            wxString popis = "";
            if(m_page3->forma() == "B") popis = wxString::FromUTF8("Řádné přiznání ");
            else popis = wxString::FromUTF8("Opravné přiznání ");
            if(m_page2->obdobi()==0)
            {
                popis.Append(dxutils::mesicString(m_page2->mesic()));
                popis.Append(wxString::Format("/%d", m_page2->rok()));
            }
            else if(m_page2->obdobi()==1)
            {
                popis.Append(dxutils::ctvrtletiString(m_page2->ctvrtleti()));
                popis.Append(wxString::Format("/%d", m_page2->rok()));
            }
            else popis.Append(m_page2->start().FormatDate()+"-"+m_page2->end().FormatDate());
            if(idbmodel->hasUlozene(0, "B", m_page2->obdobi(), m_page2->start(), m_page2->end()))
            {
                int id = idbmodel->getUlozeneId(0, "B", m_page2->obdobi(), m_page2->start(), m_page2->end());
                idbmodel->editUlozene(Ulozene(id, 0, popis,
                                              "B", m_page2->obdobi(), m_page2->mesic(), m_page2->ctvrtleti(), m_page2->rok(),
                                              m_page2->start(), m_page2->end(), idbmodel->ulozeneXml(id)));
                m_dphdp3.saveToXml(idbmodel->ulozeneXml(id));
            }
            else if(idbmodel->hasUlozene(0, "O", m_page2->obdobi(), m_page2->start(), m_page2->end()))
            {
                int id = idbmodel->getUlozeneId(0, "O", m_page2->obdobi(), m_page2->start(), m_page2->end());
                idbmodel->editUlozene(Ulozene(id, 0, popis,
                                              "O", m_page2->obdobi(), m_page2->mesic(), m_page2->ctvrtleti(), m_page2->rok(),
                                              m_page2->start(), m_page2->end(), idbmodel->ulozeneXml(id)));
                m_dphdp3.saveToXml(idbmodel->ulozeneXml(id));
            }
            else
            {
                wxString xmlpath = dxutils::getUniqueXmlPath("DPHDP3");
                m_dphdp3.saveToXml(xmlpath);
                idbmodel->addUlozene(Ulozene(idbmodel->getLastId("ulozene"), 0, popis,
                                                 m_page3->forma(), m_page2->obdobi(), m_page2->mesic(), m_page2->ctvrtleti(), m_page2->rok(),
                                                 m_page2->start(), m_page2->end(), xmlpath));
            }
        }
    }
    if(m_page3->forma() == "D" || m_page3->forma() == "E")
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Uložit daňové přiznání?"), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_OK)
        {
            if(idbmodel->hasUlozene(m_page3->ulozeneID()))
            {
                Ulozene dp = idbmodel->getUlozene(m_page3->ulozeneID());
                m_dphdp3.clearData();
                fillData();
                wxString popis = "";
                if(m_page3->forma() == "D") popis = wxString::FromUTF8("Dodatečné přiznání ");
                else popis = wxString::FromUTF8("Dodatečné/opravné přiznání ");
                if(dp.obdobi()==0)
                {
                    popis.Append(dxutils::mesicString(dp.mesic()));
                    popis.Append(wxString::Format("/%d", dp.rok()));
                }
                else if(dp.obdobi()==1)
                {
                    popis.Append(dxutils::ctvrtletiString(dp.ctvrtleti()));
                    popis.Append(wxString::Format("/%d", dp.rok()));
                }
                else popis.Append(dp.from().FormatDate()+"-"+dp.to().FormatDate());
                if(idbmodel->hasUlozene(0, "D", dp.obdobi(), dp.from(), dp.to()))
                {
                    int id = idbmodel->getUlozeneId(0, "D", dp.obdobi(), dp.from(), dp.to());
                    idbmodel->editUlozene(Ulozene(id, 0, popis,
                                                  "D", dp.obdobi(), dp.mesic(), dp.ctvrtleti(), dp.rok(),
                                                  dp.from(), dp.to(), idbmodel->ulozeneXml(id)));
                    m_dphdp3.saveToXml(idbmodel->ulozeneXml(id));
                }
                else if(idbmodel->hasUlozene(0, "E", dp.obdobi(), dp.from(), dp.to()))
                {
                    int id = idbmodel->getUlozeneId(0, "E", dp.obdobi(), dp.from(), dp.to());
                    idbmodel->editUlozene(Ulozene(id, 0, popis,
                                                  "E", dp.obdobi(), dp.mesic(), dp.ctvrtleti(), dp.rok(),
                                                  dp.from(), dp.to(), idbmodel->ulozeneXml(id)));
                    m_dphdp3.saveToXml(idbmodel->ulozeneXml(id));
                }
                else
                {
                    wxString xmlpath = dxutils::getUniqueXmlPath("DPHDP3");
                    m_dphdp3.saveToXml(xmlpath);
                    idbmodel->addUlozene(Ulozene(idbmodel->getLastId("ulozene"), 0, popis,
                                                     m_page3->forma(), dp.obdobi(), dp.mesic(), dp.ctvrtleti(), dp.rok(),
                                                     dp.from(), dp.to(), xmlpath));
                }
            }
        }
    }
}

void Dphdp3Wizard::fillVeta1to6(const wxDateTime &start, const wxDateTime &end)
{
    if(idbmodel->someDokladForDph(start, end))
    {
        //Veta1
        fillVeta1(wxSplit("1,2",'|'), "obrat23", "dan23", "obrat5", "dan5", start, end);
        fillVeta1(wxSplit("3,4|3,4,43,44",'|'), "p_zb23", "dan_pzb23", "p_zb5", "dan_pzb5", start, end);
        fillVeta1(wxSplit("5,6|5,6,43,44",'|'), "p_sl23_e", "dan_psl23_e", "p_sl5_e", "dan_psl5_e", start, end);
        fillVeta1(wxSplit("7,8|7,8,42",'|'), "dov_zb23", "dan_dzb23", "dov_zb5", "dan_dzb5", start, end);
        fillVeta1(wxSplit("9|9,43,44",'|'), "p_dop_nrg", "dan_pdop_nrg", "", "", start, end);
        fillVeta1(wxSplit("10,11|10,11,43,44",'|'), "rez_pren23", "dan_rpren23", "rez_pren5", "dan_rpren5", start, end);
        fillVeta1(wxSplit("12,13|12,13,43,44",'|'), "p_sl23_z", "dan_psl23_z", "p_sl5_z", "dan_psl5_z", start, end);
        //Veta2
        fillVeta2("20", "dod_zb", start, end);
        fillVeta2("21", "pln_sluzby", start, end);
        fillVeta2("22", "pln_vyvoz", start, end);
        fillVeta2("23", "dod_dop_nrg", start, end);
        fillVeta2("24", "pln_zaslani", start, end);
        fillVeta2("25", "pln_rez_pren", start, end);
        fillVeta2("26", "pln_ost", start, end);
        //Veta3
        fillVeta3("30", "tri_pozb", start, end, true);
        fillVeta3("31", "tri_dozb", start, end, true);
        fillVeta3("32", "dov_osv", start, end, true);
        fillVeta3("33", "opr_verit", start, end, false);
        fillVeta3("34", "opr_dluz", start, end, false);
        //Veta4
        fillVeta4(wxSplit("40,41|40,41,47",'|'), "pln23", "odp_tuz23_nar", "odp_tuz23", "pln5", "odp_tuz5_nar", "odp_tuz5", start, end);
        fillVeta4Radek42(wxSplit("7,8,42",'|'), "dov_cu", "odp_cu_nar", "odp_cu", start, end);
        fillVeta4(wxSplit("3,4,43,44|5,6,43,44|9,43,44|10,11,43,44|12,13,43,44",'|'), "nar_zdp23", "od_zdp23", "odkr_zdp23", "nar_zdp5", "od_zdp5", "odkr_zdp5", start, end);
        fillVeta4Radek42(wxSplit("40,41,47",'|'), "nar_maj", "od_maj", "odkr_maj", start, end); //Radek 47
        //Veta5
        fillVeta2("50", "plnosv_kf", start, end); //Radek 50
        m_dphdp3.setAtributValue("koef_p20_nov", m_page2->koefP20Nov());
        //Veta6
        fillVeta3("61", "dan_vrac", start, end, false); //Radek 61
    }
}

void Dphdp3Wizard::fillVeta1to6Summary(const wxDateTime &start, const wxDateTime &end)
{
    if(idbmodel->someDokladForDph(start, end) || !wxIsNullDouble(m_page2->vyporOdp()))
    {
        m_dphdp3.fillRadek46();
        m_dphdp3.fillRadek52();
        m_dphdp3.fillVeta6();
    }
}

void Dphdp3Wizard::fillVeta1(const wxArrayString &radky, const wxString &obrat23Name, const wxString &dan23Name, const wxString &obrat5Name, const wxString &dan5Name,
                             const wxDateTime &start, const wxDateTime &end)
{
    double obrat23 = 0.0, dan23 = 0.0, obrat5 = 0.0, dan5 = 0.0;
    wxArrayInt ids = idbmodel->getDphDruhIdByRadek(radky);
    if(idbmodel->someDokladForDph(ids, start, end))
    {
        dxDphDokladArray doklady = idbmodel->dokladyForDph(ids, start, end, false);
        for(size_t i=0; i<doklady.GetCount(); i++)
        {
            dxDphForDokladArray dph = doklady[i].dph();
            for(size_t j=0; j<dph.GetCount(); j++)
            {
                if(dph[j].zakladni())
                {
                    obrat23 += dph[j].zaklad();
//                    dan23 += round(dph[j].zaklad())*(dph[j].sazba()/100.0);
                    dan23 += dph[j].dph();
                }
                else
                {
                    if(dph[j].sazba())
                    {
                        obrat5 += dph[j].zaklad();
//                        dan5 += round(dph[j].zaklad())*(dph[j].sazba()/100.0);
                        dan5 += dph[j].dph();
                    }
                }
            }
        }
    }
    if(obrat23 != 0.0 && !obrat23Name.IsEmpty()) m_dphdp3.setAtributValue(obrat23Name, round(obrat23));
    if(dan23 != 0.0 && !dan23Name.IsEmpty()) m_dphdp3.setAtributValue(dan23Name, round(dan23));
    if(obrat5 != 0.0 && !obrat5Name.IsEmpty()) m_dphdp3.setAtributValue(obrat5Name, round(obrat5));
    if(dan5 != 0.0 && !dan5Name.IsEmpty()) m_dphdp3.setAtributValue(dan5Name, round(dan5));
}

void Dphdp3Wizard::fillVeta2(const wxString &radky, const wxString &obratName, const wxDateTime &start, const wxDateTime &end)
{
    double obrat = 0.0;
    wxArrayInt ids = idbmodel->getDphDruhIdByRadek(radky);
    if(idbmodel->someDokladForDph(ids, start, end))
    {
        dxDphDokladArray doklady = idbmodel->dokladyForDph(ids, start, end, false);
        for(size_t i=0; i<doklady.GetCount(); i++)
        {
            dxDphForDokladArray dph = doklady[i].dph();
            for(size_t j=0; j<dph.GetCount(); j++)
            {
                obrat += dph[j].zaklad();
            }
        }
    }
    if(obrat != 0.0 && !obratName.IsEmpty()) m_dphdp3.setAtributValue(obratName, round(obrat));
}

void Dphdp3Wizard::fillVeta3(const wxString &radky, const wxString &obratName, const wxDateTime &start, const wxDateTime &end, bool obrat)
{
    double obratsum = 0.0;
    wxArrayInt ids = idbmodel->getDphDruhIdByRadek(radky);
    if(idbmodel->someDokladForDph(ids, start, end))
    {
        dxDphDokladArray doklady = idbmodel->dokladyForDph(ids, start, end, false);
        for(size_t i=0; i<doklady.GetCount(); i++)
        {
            dxDphForDokladArray dph = doklady[i].dph();
            for(size_t j=0; j<dph.GetCount(); j++)
            {
                obratsum += obrat?dph[j].zaklad():dph[j].dph();
            }
        }
    }
    if(obratsum != 0.0 && !obratName.IsEmpty()) m_dphdp3.setAtributValue(obratName, round(obratsum));
}

void Dphdp3Wizard::fillVeta4(const wxArrayString &radky, const wxString &obrat23Name, const wxString &dan23Name, const wxString &krac23Name, const wxString &obrat5Name, const wxString &dan5Name, const wxString &krac5Name, const wxDateTime &start, const wxDateTime &end)
{
    double obrat23 = 0.0, dan23 = 0.0, krac23 = 0.0, obrat5 = 0.0, dan5 = 0.0, krac5 = 0.0;
    wxArrayInt ids = idbmodel->getDphDruhIdByRadek(radky);
    if(idbmodel->someDokladForDph(ids, start, end))
    {
        for(size_t i=0; i<ids.GetCount(); i++)
        {
            bool kraceny = idbmodel->kracenyDphDruh(ids[i]);
            dxDphDokladArray doklady = idbmodel->dokladyForDph(ids[i], start, end, false);
            for(size_t j=0; j<doklady.GetCount(); j++)
            {
                dxDphForDokladArray dph = doklady[j].dph();
                for(size_t k=0; k<dph.GetCount(); k++)
                {
                    if(dph[k].zakladni())
                    {
                        obrat23 += dph[k].zaklad();
//                        if(kraceny) krac23 += round(dph[k].zaklad())*(dph[k].sazba()/100.0);
//                        else dan23 += round(dph[k].zaklad())*(dph[k].sazba()/100.0);
                        if(kraceny) krac23 += dph[k].dph();
                        else dan23 += dph[k].dph();
                    }
                    else
                    {
                        if(dph[k].sazba())
                        {
                            obrat5 += dph[k].zaklad();
//                            if(kraceny) krac5 += round(dph[k].zaklad())*(dph[k].sazba()/100.0);
//                            else dan5 += round(dph[k].zaklad())*(dph[k].sazba()/100.0);
                            if(kraceny) krac5 += dph[k].dph();
                            else dan5 += dph[k].dph();
                        }
                    }
                }
            }
        }
    }
    if(obrat23 != 0.0 && !obrat23Name.IsEmpty()) m_dphdp3.setAtributValue(obrat23Name, round(obrat23));
    if(dan23 != 0.0 && !dan23Name.IsEmpty()) m_dphdp3.setAtributValue(dan23Name, round(dan23));
    if(krac23 != 0.0 && !krac23Name.IsEmpty()) m_dphdp3.setAtributValue(krac23Name, round(krac23));
    if(obrat5 != 0.0 && !obrat5Name.IsEmpty()) m_dphdp3.setAtributValue(obrat5Name, round(obrat5));
    if(dan5 != 0.0 && !dan5Name.IsEmpty()) m_dphdp3.setAtributValue(dan5Name, round(dan5));
    if(krac5 != 0.0 && !krac5Name.IsEmpty()) m_dphdp3.setAtributValue(krac5Name, round(krac5));
}

void Dphdp3Wizard::fillVeta4Radek42(const wxArrayString &radky, const wxString &obrat23Name, const wxString &dan23Name, const wxString &krac23Name, const wxDateTime &start, const wxDateTime &end)
{
    double obrat23 = 0.0, dan23 = 0.0, krac23 = 0.0;
    wxArrayInt ids = idbmodel->getDphDruhIdByRadek(radky);
    if(idbmodel->someDokladForDph(ids, start, end))
    {
        for(size_t i=0; i<ids.GetCount(); i++)
        {
            bool kraceny = idbmodel->kracenyDphDruh(ids[i]);
            dxDphDokladArray doklady = idbmodel->dokladyForDph(ids[i], start, end, false);
            for(size_t j=0; j<doklady.GetCount(); j++)
            {
                dxDphForDokladArray dph = doklady[j].dph();
                for(size_t k=0; k<dph.GetCount(); k++)
                {
                    obrat23 += dph[k].zaklad();
//                        if(kraceny) krac23 += round(dph[k].zaklad())*(dph[k].sazba()/100.0);
//                        else dan23 += round(dph[k].zaklad())*(dph[k].sazba()/100.0);
                    if(kraceny) krac23 += dph[k].dph();
                    else dan23 += dph[k].dph();
                }
            }
        }
    }
    if(obrat23 != 0.0 && !obrat23Name.IsEmpty()) m_dphdp3.setAtributValue(obrat23Name, round(obrat23));
    if(dan23 != 0.0 && !dan23Name.IsEmpty()) m_dphdp3.setAtributValue(dan23Name, round(dan23));
    if(krac23 != 0.0 && !krac23Name.IsEmpty()) m_dphdp3.setAtributValue(krac23Name, round(krac23));
}
