/***************************************************************
 * Name:      dphkh1vetaa1dialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "dphkh1vetaa1dialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(Dphkh1Vetaa1Dialog, wxDialog)

BEGIN_EVENT_TABLE(Dphkh1Vetaa1Dialog, wxDialog)
    EVT_BUTTON(wxID_OK, Dphkh1Vetaa1Dialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, Dphkh1Vetaa1Dialog::OnCancel)
END_EVENT_TABLE()

Dphkh1Vetaa1Dialog::Dphkh1Vetaa1Dialog(wxWindow *parent, Veta veta)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Upravit řádek A1")), m_veta(veta)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("DIČ odběratele")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    wxTextValidator dicodbval(wxFILTER_NUMERIC);
    m_dicOdb = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, dicodbval);
    m_dicOdb->SetMaxLength(10);
    m_dicOdb->SetValue(m_veta.atributValue("dic_odb"));
    gridSizer->Add(m_dicOdb, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Ev. číslo daňového dokladu")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cEvidDd = new wxTextCtrl(this, wxID_ANY);
    m_cEvidDd->SetMaxLength(60);
    m_cEvidDd->SetValue(m_veta.atributValue("c_evid_dd"));
    gridSizer->Add(m_cEvidDd, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "DUZP"), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_duzp = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Today(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    if(!m_veta.atributValue("duzp").IsEmpty()) m_duzp->SetValue(m_veta.atributValueDatum("duzp"));
    gridSizer->Add(m_duzp, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane1 = new wxSpinCtrlDouble(this, wxID_ANY);
    m_zaklDane1->SetDigits(2);
    m_zaklDane1->SetRange(1.0, 99999999999999.0);
    if(!m_veta.atributValue("zakl_dane1").IsEmpty()) m_zaklDane1->SetValue(m_veta.atributValueDouble("zakl_dane1"));
    gridSizer->Add(m_zaklDane1, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Kód předmětu plnění")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kodPredPl = new wxChoice(this, wxID_ANY);
    m_kodPredPl->Append(idbmodel->getDphDruhKpp());
    if(!m_veta.atributValue("kod_pred_pl").IsEmpty()) m_kodPredPl->SetStringSelection(m_veta.atributValue("kod_pred_pl"));
    else m_kodPredPl->SetSelection(0);
    gridSizer->Add(m_kodPredPl, wxGBPosition(4,1), wxDefaultSpan, wxEXPAND);
    m_mainSizer->Add(gridSizer, 0, wxALL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

Veta Dphkh1Vetaa1Dialog::veta() const
{
    return m_veta;
}

void Dphkh1Vetaa1Dialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_dicOdb->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("DIČ odběratele nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_dicOdb->SetFocus();
        return;
    }
    if(m_cEvidDd->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Ev. číslo daňového dokladu nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_cEvidDd->SetFocus();
        return;
    }
    m_veta.setAtributValue("dic_odb", m_dicOdb->GetValue());
    m_veta.setAtributValue("c_evid_dd", m_cEvidDd->GetValue());
    m_veta.setAtributValue("duzp", m_duzp->GetValue());
    m_veta.setAtributValue("kod_pred_pl", m_kodPredPl->GetStringSelection());
    m_veta.setAtributValue("zakl_dane1", m_zaklDane1->GetValue());
    EndModal(wxID_OK);
}

void Dphkh1Vetaa1Dialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}
