/***************************************************************
 * Name:      dphkh1vetaa3dialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "dphkh1vetaa3dialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "model/dbmodel.h"

IMPLEMENT_DYNAMIC_CLASS(Dphkh1Vetaa3Dialog, wxDialog)

BEGIN_EVENT_TABLE(Dphkh1Vetaa3Dialog, wxDialog)
    EVT_BUTTON(wxID_OK, Dphkh1Vetaa3Dialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, Dphkh1Vetaa3Dialog::OnCancel)
    EVT_CHECKBOX(ID_DPHKH1VETAA3_DIC, Dphkh1Vetaa3Dialog::OnDic)
    EVT_CHECKBOX(ID_DPHKH1VETAA3_FYZICKA, Dphkh1Vetaa3Dialog::OnFyzicka)
END_EVENT_TABLE()

Dphkh1Vetaa3Dialog::Dphkh1Vetaa3Dialog(wxWindow *parent, Veta veta)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Upravit řádek A3")), m_veta(veta)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    bool dic = !m_veta.atributValue("k_stat").IsEmpty();
    bool fyzicka = !m_veta.atributValue("d_narozeni").IsEmpty();
    m_dic = new wxCheckBox(this, ID_DPHKH1VETAA3_DIC, wxString::FromUTF8("Odběratel s DIČ/VAT ID"));
    m_dic->SetValue(dic);
    gridSizer->Add(m_dic, wxGBPosition(0,0), wxGBSpan(1,2), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Kód státu")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kStat = new wxTextCtrl(this, wxID_ANY);
    m_kStat->SetMaxLength(2);
    m_kStat->SetValue(m_veta.atributValue("k_stat"));
    m_kStat->Enable(dic);
    gridSizer->Add(m_kStat, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("DIČ/VAT ID")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_vatidOdb = new wxTextCtrl(this, wxID_ANY);
    m_vatidOdb->SetMaxLength(12);
    m_vatidOdb->SetValue(m_veta.atributValue("vatid_odb"));
    m_vatidOdb->Enable(dic);
    gridSizer->Add(m_vatidOdb, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Jméno a příjmení / obchodní jméno")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_jmPrijmObch = new wxTextCtrl(this, wxID_ANY);
    m_jmPrijmObch->SetMaxLength(57);
    m_jmPrijmObch->SetValue(m_veta.atributValue("jm_prijm_obch"));
    m_jmPrijmObch->Enable(!dic);
    gridSizer->Add(m_jmPrijmObch, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Místo pobytu / sídlo")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mPobytuSidlo = new wxTextCtrl(this, wxID_ANY);
    m_mPobytuSidlo->SetMaxLength(100);
    m_mPobytuSidlo->SetValue(m_veta.atributValue("m_pobytu_sidlo"));
    m_mPobytuSidlo->Enable(!dic);
    gridSizer->Add(m_mPobytuSidlo, wxGBPosition(4,1), wxDefaultSpan, wxEXPAND);
    m_fyzicka = new wxCheckBox(this, ID_DPHKH1VETAA3_FYZICKA, wxString::FromUTF8("Odběratel - fyzická osoba"));
    m_fyzicka->SetValue(fyzicka);
    m_fyzicka->Enable(!dic);
    gridSizer->Add(m_fyzicka, wxGBPosition(5,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Datum narození")), wxGBPosition(6,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dNarozeni = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Today(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    if(!m_veta.atributValue("d_narozeni").IsEmpty()) m_dNarozeni->SetValue(m_veta.atributValueDatum("d_narozeni"));
    m_dNarozeni->Enable(!dic && fyzicka);
    gridSizer->Add(m_dNarozeni, wxGBPosition(6,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Ev. číslo daňového dokladu")), wxGBPosition(7,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cEvidDd = new wxTextCtrl(this, wxID_ANY);
    m_cEvidDd->SetMaxLength(60);
    m_cEvidDd->SetValue(m_veta.atributValue("c_evid_dd"));
    gridSizer->Add(m_cEvidDd, wxGBPosition(7,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "DUP"), wxGBPosition(8,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dup = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Today(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    if(!m_veta.atributValue("dup").IsEmpty()) m_dup->SetValue(m_veta.atributValueDatum("dup"));
    gridSizer->Add(m_dup, wxGBPosition(8,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Hodnota osvoboz. plnění")), wxGBPosition(9,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_osvPlneni = new wxSpinCtrlDouble(this, ID_DPHKH1VETAA2_ZAKLAD1);
    m_osvPlneni->SetDigits(2);
    m_osvPlneni->SetRange(1.0, 99999999999999.0);
    if(!m_veta.atributValue("osv_plneni").IsEmpty()) m_osvPlneni->SetValue(m_veta.atributValueDouble("osv_plneni"));
    gridSizer->Add(m_osvPlneni, wxGBPosition(9,1), wxDefaultSpan, wxEXPAND);
    m_mainSizer->Add(gridSizer, 0, wxALL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

Veta Dphkh1Vetaa3Dialog::veta() const
{
    return m_veta;
}

void Dphkh1Vetaa3Dialog::OnOk(wxCommandEvent& /*event*/)
{
    if(m_dic->GetValue())
    {
        if(m_kStat->GetValue().IsEmpty())
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Kód státu nemůže být prázdný"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            m_kStat->SetFocus();
            return;
        }
        if(m_vatidOdb->GetValue().IsEmpty())
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Identifikace DIČ/VAT ID nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            m_vatidOdb->SetFocus();
            return;
        }
    }
    else
    {
        if(m_jmPrijmObch->GetValue().IsEmpty())
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Jméno a příjmení / obchodní jméno nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            m_jmPrijmObch->SetFocus();
            return;
        }
        if(m_mPobytuSidlo->GetValue().IsEmpty())
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Místo pobytu / sídlo nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            m_mPobytuSidlo->SetFocus();
            return;
        }
    }
    if(m_cEvidDd->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Ev. číslo daňového dokladu nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_cEvidDd->SetFocus();
        return;
    }
    m_veta.setAtributValue("c_evid_dd", m_cEvidDd->GetValue());
    m_veta.setAtributValue("dup", m_dup->GetValue());
    m_veta.setAtributValue("osv_plneni", m_osvPlneni->GetValue());
    if(m_dic->GetValue())
    {
        m_veta.setAtributValue("k_stat", m_kStat->GetValue());
        m_veta.setAtributValue("vatid_odb", m_vatidOdb->GetValue());
        m_veta.setAtributValue("jm_prijm_obch", "");
        m_veta.setAtributValue("m_pobytu_sidlo", "");
        m_veta.setAtributValue("d_narozeni", "");
    }
    else
    {
        m_veta.setAtributValue("k_stat", "");
        m_veta.setAtributValue("vatid_odb", "");
        m_veta.setAtributValue("jm_prijm_obch", m_jmPrijmObch->GetValue());
        m_veta.setAtributValue("m_pobytu_sidlo", m_mPobytuSidlo->GetValue());
        if(m_fyzicka->GetValue()) m_veta.setAtributValue("d_narozeni", m_dNarozeni->GetValue());
        else m_veta.setAtributValue("d_narozeni", "");
    }
    EndModal(wxID_OK);
}

void Dphkh1Vetaa3Dialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void Dphkh1Vetaa3Dialog::OnDic(wxCommandEvent &/*event*/)
{
    if(m_dic->GetValue())
    {
        m_kStat->Enable();
        m_vatidOdb->Enable();
        m_jmPrijmObch->Disable();
        m_mPobytuSidlo->Disable();
        m_fyzicka->Disable();
        m_dNarozeni->Disable();
    }
    else
    {
        m_kStat->Disable();
        m_vatidOdb->Disable();
        m_jmPrijmObch->Enable();
        m_mPobytuSidlo->Enable();
        m_fyzicka->Enable();
        if(m_fyzicka->GetValue()) m_dNarozeni->Enable();
        else m_dNarozeni->Disable();
    }
}

void Dphkh1Vetaa3Dialog::OnFyzicka(wxCommandEvent &/*event*/)
{
    if(m_fyzicka->GetValue()) m_dNarozeni->Enable();
    else m_dNarozeni->Disable();
}
