/***************************************************************
 * Name:      dphkh1wizard.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/hyperlink.h>
#include "dphkh1wizard.h"
#include "priznanidialog.h"
#include "dphkh1print.h"
#include "dphkh1vetaa1dialog.h"
#include "dphkh1vetaa2dialog.h"
#include "dphkh1vetaa3dialog.h"
#include "dphkh1vetaa4dialog.h"
#include "dphkh1vetab1dialog.h"
#include "dphkh1vetab2dialog.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "config.h"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/delete.xpm"

BEGIN_EVENT_TABLE(Kh1Page1, wxWizardPageSimple)
    EVT_BUTTON(ID_DPHKH1_PAGE1_PRIZNANI, Kh1Page1::OnPriznani)
END_EVENT_TABLE()

Kh1Page1::Kh1Page1(wxWizard *parent)
    : wxWizardPageSimple(parent)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Průvodce Vás provede vytvořením\nKontrolního hlášení DPH")));
    m_mainSizer->AddStretchSpacer();
    m_priznani = new wxButton(this, ID_DPHKH1_PAGE1_PRIZNANI, wxString::FromUTF8("Nastavení plátce"));
    m_mainSizer->Add(m_priznani, 0, wxALL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void Kh1Page1::OnPriznani(wxCommandEvent &/*event*/)
{
    PriznaniDialog dialog(this, true);
    dialog.ShowModal();
}

BEGIN_EVENT_TABLE(Kh1Page2, wxWizardPageSimple)
    EVT_RADIOBUTTON(ID_DPHKH1_PAGE2_MESIC, Kh1Page2::OnMesic)
    EVT_RADIOBUTTON(ID_DPHKH1_PAGE2_CTVRTLETI, Kh1Page2::OnCtvrtleti)
    EVT_RADIOBUTTON(ID_DPHKH1_PAGE2_ODDO, Kh1Page2::OnOddo)
    EVT_DATE_CHANGED(ID_DPHKH1_PAGE2_OD, Kh1Page2::OnOd)
    EVT_DATE_CHANGED(ID_DPHKH1_PAGE2_DO, Kh1Page2::OnDo)
END_EVENT_TABLE()

Kh1Page2::Kh1Page2(wxWizard *parent)
    : wxWizardPageSimple(parent)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    m_mesicBtn = new wxRadioButton(this, ID_DPHKH1_PAGE2_MESIC, wxString::FromUTF8("Měsíc"));
    m_mesicBtn->SetValue(false);
    gridSizer->Add(m_mesicBtn, wxGBPosition(0,0), wxDefaultSpan, wxEXPAND);
    m_mesicSelect = new wxChoice(this, wxID_ANY);
    m_mesicSelect->Append(dxutils::mesicString(1));
    m_mesicSelect->Append(dxutils::mesicString(2));
    m_mesicSelect->Append(dxutils::mesicString(3));
    m_mesicSelect->Append(dxutils::mesicString(4));
    m_mesicSelect->Append(dxutils::mesicString(5));
    m_mesicSelect->Append(dxutils::mesicString(6));
    m_mesicSelect->Append(dxutils::mesicString(7));
    m_mesicSelect->Append(dxutils::mesicString(8));
    m_mesicSelect->Append(dxutils::mesicString(9));
    m_mesicSelect->Append(dxutils::mesicString(10));
    m_mesicSelect->Append(dxutils::mesicString(11));
    m_mesicSelect->Append(dxutils::mesicString(12));
    m_mesicSelect->SetSelection(wxMax(wxDateTime::Today().GetMonth()-1,0));
    m_mesicSelect->Enable(false);
    gridSizer->Add(m_mesicSelect, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    m_mesicRok = new wxChoice(this, wxID_ANY);
    m_mesicRok->Append(idbmodel->getObdobiAsStrings(2015));
    if(m_mesicRok->FindString(wxString::Format("%d", idbmodel->currentRok())) != wxNOT_FOUND) m_mesicRok->SetStringSelection(wxString::Format("%d", idbmodel->currentRok()));
    else m_mesicRok->SetSelection(0);
    m_mesicRok->Enable(false);
    gridSizer->Add(m_mesicRok, wxGBPosition(0,2), wxDefaultSpan, wxEXPAND);

    m_ctvrtletiBtn = new wxRadioButton(this, ID_DPHKH1_PAGE2_CTVRTLETI, wxString::FromUTF8("Čtvrtletí"));
    m_ctvrtletiBtn->SetValue(true);
    gridSizer->Add(m_ctvrtletiBtn, wxGBPosition(1,0), wxDefaultSpan, wxEXPAND);
    m_ctvrtletiSelect = new wxChoice(this, wxID_ANY);
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(1));
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(2));
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(3));
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(4));
    m_ctvrtletiSelect->SetSelection(wxMax(wxDateTime::Today().GetMonth()/3-1,0));
    gridSizer->Add(m_ctvrtletiSelect, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    m_ctvrtletiRok = new wxChoice(this, wxID_ANY);
    m_ctvrtletiRok->Append(idbmodel->getObdobiAsStrings(2015));
    if(m_ctvrtletiRok->FindString(wxString::Format("%d", idbmodel->currentRok())) != wxNOT_FOUND) m_ctvrtletiRok->SetStringSelection(wxString::Format("%d", idbmodel->currentRok()));
    else m_ctvrtletiRok->SetSelection(0);
    gridSizer->Add(m_ctvrtletiRok, wxGBPosition(1,2), wxDefaultSpan, wxEXPAND);

    m_oddoBtn = new wxRadioButton(this, ID_DPHKH1_PAGE2_ODDO, wxString::FromUTF8("Období"));
    m_oddoBtn->SetValue(false);
    gridSizer->Add(m_oddoBtn, wxGBPosition(2,0), wxDefaultSpan, wxEXPAND);
    m_od = new wxDatePickerCtrl(this, ID_DPHKH1_PAGE2_OD, wxDateTime(1, wxDateTime::Jan, wxMax(idbmodel->currentRok(), 2016)), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_od->SetRange(wxDateTime(1, wxDateTime::Jan, wxMax(idbmodel->currentRok(), 2016)), wxDateTime(31, wxDateTime::Dec, idbmodel->maxRok()));
    m_od->Enable(false);
    gridSizer->Add(m_od, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    m_do = new wxDatePickerCtrl(this, ID_DPHKH1_PAGE2_DO, wxDateTime(31, wxDateTime::Jan, wxMax(idbmodel->currentRok(), 2016)), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_do->Enable(false);
    gridSizer->Add(m_do, wxGBPosition(2,2), wxDefaultSpan, wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Datum vyhotovení")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_vyhotoveni = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Today(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    gridSizer->Add(m_vyhotoveni, wxGBPosition(3,1), wxGBSpan(1,2), wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL, 5);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

wxDateTime Kh1Page2::vyhotoveni() const
{
    return m_vyhotoveni->GetValue();
}

//0 - mesic, 1 - ctvrtleti, 2 - oddo
int Kh1Page2::obdobi() const
{
    if(m_mesicBtn->GetValue()) return 0;
    if(m_ctvrtletiBtn->GetValue()) return 1;
    return 2;
}

int Kh1Page2::mesic() const
{
    return m_mesicBtn->GetValue()?m_mesicSelect->GetSelection()+1:0;
}

int Kh1Page2::ctvrtleti() const
{
    return m_ctvrtletiBtn->GetValue()?m_ctvrtletiSelect->GetSelection()+1:0;
}

int Kh1Page2::rok() const
{
    if(obdobi() == 0) return wxAtoi(m_mesicRok->GetStringSelection());
    if(obdobi() == 1) return wxAtoi(m_ctvrtletiRok->GetStringSelection());
    return m_do->GetValue().GetYear();
}

wxDateTime Kh1Page2::start() const
{
    if(obdobi() == 0) return wxDateTime(1, static_cast<wxDateTime::Month>(mesic()-1), rok());
    if(obdobi() == 1) return wxDateTime(1, dxutils::ctvrtletiMesicStart(ctvrtleti()), rok());
    return m_od->GetValue();
}

wxDateTime Kh1Page2::end() const
{
    if(obdobi() == 0) return wxDateTime(start().GetLastMonthDay(static_cast<wxDateTime::Month>(mesic()-1), rok()));
    if(obdobi() == 1) return wxDateTime(start().GetLastMonthDay(dxutils::ctvrtletiMesicEnd(ctvrtleti()), rok()));
    return m_do->GetValue();
}

void Kh1Page2::OnMesic(wxCommandEvent &/*event*/)
{
    m_ctvrtletiSelect->Enable(false);
    m_ctvrtletiRok->Enable(false);
    m_mesicSelect->Enable(true);
    m_mesicRok->Enable(true);
    m_od->Enable(false);
    m_do->Enable(false);
    m_mesicSelect->SetSelection(0);
}

void Kh1Page2::OnCtvrtleti(wxCommandEvent &/*event*/)
{
    m_mesicSelect->Enable(false);
    m_mesicRok->Enable(false);
    m_ctvrtletiSelect->Enable(true);
    m_ctvrtletiRok->Enable(true);
    m_od->Enable(false);
    m_do->Enable(false);
    m_ctvrtletiSelect->SetSelection(0);
}

void Kh1Page2::OnOddo(wxCommandEvent &/*event*/)
{
    m_mesicSelect->Enable(false);
    m_mesicRok->Enable(false);
    m_ctvrtletiSelect->Enable(false);
    m_ctvrtletiRok->Enable(false);
    m_od->Enable(true);
    m_do->Enable(true);
}

void Kh1Page2::OnOd(wxDateEvent &/*event*/)
{
    if(m_do->GetValue().IsEarlierThan(m_od->GetValue())) m_do->SetValue(m_od->GetValue().GetLastMonthDay(m_od->GetValue().GetMonth(), m_od->GetValue().GetYear()));
}

void Kh1Page2::OnDo(wxDateEvent &/*event*/)
{
    if(m_od->GetValue().IsLaterThan(m_do->GetValue())) m_od->SetValue(wxDateTime(1, m_do->GetValue().GetMonth(), m_do->GetValue().GetYear()));
}

BEGIN_EVENT_TABLE(Kh1Page3, wxWizardPageSimple)
    EVT_CHECKBOX(ID_DPHKH1_PAGE3_VYZVA, Kh1Page3::OnVyzva)
    EVT_CHECKBOX(ID_DPHKH1_PAGE3_RYCHLA, Kh1Page3::OnRychla)
    EVT_CHOICE(ID_DPHKH1_PAGE3_VYZVAODP, Kh1Page3::OnVyzvaOdpoved)
END_EVENT_TABLE()

Kh1Page3::Kh1Page3(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphkh1Wizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_radne = new wxRadioButton(this, wxID_ANY, wxString::FromUTF8("Řádné hlášení"));
    m_radne->SetValue(true);
    m_mainSizer->Add(m_radne);

    m_radneOpravne = new wxRadioButton(this, wxID_ANY, wxString::FromUTF8("Řádné opravné hlášení"));
    m_mainSizer->Add(m_radneOpravne);

    m_nasledne = new wxRadioButton(this, wxID_ANY, wxString::FromUTF8("Následné hlášení"));
    m_mainSizer->Add(m_nasledne);

    m_nasledneOpravne = new wxRadioButton(this, wxID_ANY, wxString::FromUTF8("Následné opravné hlášení"));
    m_mainSizer->Add(m_nasledneOpravne);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Datum zjištění pro podání následného kontrolního hlášení:")));
    m_dzjist = new wxDatePickerCtrl(this, wxID_ANY, wxDateTime::Today(), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_mainSizer->Add(m_dzjist);

    m_vyzva = new wxCheckBox(this, ID_DPHKH1_PAGE3_VYZVA, wxString::FromUTF8("Číslo jednací výzvy :"));
    m_vyzva->SetValue(false);
    m_mainSizer->Add(m_vyzva);
    wxBoxSizer *vyzvaSizer = new wxBoxSizer(wxHORIZONTAL);
    wxTextValidator vyzvaval(wxFILTER_NUMERIC);
    m_vyzva1 = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, vyzvaval);
    m_vyzva1->SetMaxLength(8);
    m_vyzva1->Disable();
    vyzvaSizer->Add(m_vyzva1);
    vyzvaSizer->Add(new wxStaticText(this, wxID_ANY, "/"));
    m_vyzva2 = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, vyzvaval);
    m_vyzva2->SetMaxLength(2);
    m_vyzva2->Disable();
    vyzvaSizer->Add(m_vyzva2);
    vyzvaSizer->Add(new wxStaticText(this, wxID_ANY, "/"));
    m_vyzva3 = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, vyzvaval);
    m_vyzva3->SetMaxLength(4);
    m_vyzva3->Disable();
    vyzvaSizer->Add(m_vyzva3);
    vyzvaSizer->Add(new wxStaticText(this, wxID_ANY, "-"));
    m_vyzva4 = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, vyzvaval);
    m_vyzva4->SetMaxLength(5);
    m_vyzva4->Disable();
    vyzvaSizer->Add(m_vyzva4);
    vyzvaSizer->Add(new wxStaticText(this, wxID_ANY, "-"));
    m_vyzva5 = new wxTextCtrl(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, 0, vyzvaval);
    m_vyzva5->SetMaxLength(6);
    m_vyzva5->Disable();
    vyzvaSizer->Add(m_vyzva5);
    m_mainSizer->Add(vyzvaSizer);

    m_rychla = new wxCheckBox(this, ID_DPHKH1_PAGE3_RYCHLA, wxString::FromUTF8("Rychlá odpověď na výzvu"));
    m_rychla->SetValue(false);
    m_rychla->Disable();
    m_mainSizer->Add(m_rychla);
    m_vyzvaOdp = new wxChoice(this, ID_DPHKH1_PAGE3_VYZVAODP);
    m_vyzvaOdp->Append("Ne");
    m_vyzvaOdp->Append(wxString::FromUTF8("Nemám povinnost podat KH (nulové KH)"));
    m_vyzvaOdp->Append(wxString::FromUTF8("Potvrzuji správnost naposledy podaného KH"));
    m_vyzvaOdp->SetSelection(0);
    m_vyzvaOdp->Disable();
    m_mainSizer->Add(m_vyzvaOdp);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

wxString Kh1Page3::forma() const
{
    if(m_radne->GetValue()) return "B";
    if(m_radneOpravne->GetValue()) return "O";
    if(m_nasledne->GetValue()) return "N";
    return "E";
}

wxDateTime Kh1Page3::dzjist() const
{
    return m_dzjist->GetValue();
}

wxString Kh1Page3::cJedVyzvy() const
{
    if(m_vyzva->GetValue())
    {
        return dxutils::fillByZeroes(m_vyzva1->GetValue(),1)+"/"+dxutils::fillByZeroes(m_vyzva2->GetValue(),2)+"/"+dxutils::fillByZeroes(m_vyzva3->GetValue(),4)+"-"
                +dxutils::fillByZeroes(m_vyzva4->GetValue(),5)+"-"+dxutils::fillByZeroes(m_vyzva5->GetValue(),6);
    }
    return "";
}

wxString Kh1Page3::vyzvaOdp() const
{
    if(m_vyzva->GetValue() && m_rychla->GetValue())
    {
        switch (m_vyzvaOdp->GetSelection()) {
        case 0:
            return "";
        case 1:
            return "B";
        case 2:
            return "P";
        default:
            return "";
        }
    }
    return "";
}

void Kh1Page3::OnVyzva(wxCommandEvent &/*event*/)
{
    if(m_vyzva->GetValue())
    {
        m_vyzva1->Enable();
        m_vyzva2->Enable();
        m_vyzva3->Enable();
        m_vyzva4->Enable();
        m_vyzva5->Enable();
        m_rychla->Enable();
        if(m_rychla->GetValue()) m_vyzvaOdp->Enable();
    }
    else
    {
        m_vyzva1->Disable();
        m_vyzva2->Disable();
        m_vyzva3->Disable();
        m_vyzva4->Disable();
        m_vyzva5->Disable();
        m_rychla->Disable();
        m_vyzvaOdp->Disable();
    }
    m_wizard->skipDataPages(m_vyzva->GetValue() && m_rychla->GetValue() && m_vyzvaOdp->GetSelection());
}

void Kh1Page3::OnRychla(wxCommandEvent &/*event*/)
{
    if(m_rychla->GetValue())
    {
        m_vyzvaOdp->Enable();
    }
    else
    {
        m_vyzvaOdp->Disable();
    }
    m_wizard->skipDataPages(m_vyzva->GetValue() && m_rychla->GetValue() && m_vyzvaOdp->GetSelection());
}

void Kh1Page3::OnVyzvaOdpoved(wxCommandEvent &/*event*/)
{
    m_wizard->skipDataPages(m_vyzva->GetValue() && m_rychla->GetValue() && m_vyzvaOdp->GetSelection());
}

BEGIN_EVENT_TABLE(Kh1PageEP, wxWizardPageSimple)
    EVT_BUTTON(ID_DPHKH1_PRINT, Kh1PageEP::OnPrint)
    EVT_BUTTON(ID_DPHKH1_EXPORT, Kh1PageEP::OnExport)
END_EVENT_TABLE()

Kh1PageEP::Kh1PageEP(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphkh1Wizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Kontrolní hlášení DPH můžete vytisknout nebo\nvyexportovat jako xml soubor.\n\nPodat lze jen jako xml soubor, můžete ho odeslat pomocí datové schránky"
                                                                         "\nnebo podat pomocí")));
    m_mainSizer->Add(new wxHyperlinkCtrl(this, wxID_ANY, "www.daneelektronicky.cz", "http://www.daneelektronicky.cz/"));

    m_mainSizer->AddStretchSpacer();

    wxBoxSizer *buttons = new wxBoxSizer(wxHORIZONTAL);
    m_print = new wxButton(this, ID_DPHKH1_PRINT, wxString::FromUTF8("Tisk přiznání"));
    buttons->Add(m_print);
    buttons->AddSpacer(5);
    m_export = new wxButton(this, ID_DPHKH1_EXPORT, wxString::FromUTF8("Export přiznání"));
    buttons->Add(m_export);
    m_mainSizer->Add(buttons, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void Kh1PageEP::OnPrint(wxCommandEvent &/*event*/)
{
    m_wizard->print();
}

void Kh1PageEP::OnExport(wxCommandEvent &/*event*/)
{
    m_wizard->saveToXml();
}

IMPLEMENT_DYNAMIC_CLASS(Dphkh1Wizard, wxWizard)

BEGIN_EVENT_TABLE(Dphkh1Wizard, wxWizard)
    EVT_WIZARD_PAGE_CHANGING(wxID_ANY, Dphkh1Wizard::OnPageChanging)
END_EVENT_TABLE()

Dphkh1Wizard::Dphkh1Wizard(wxWindow *parent)
    : wxWizard(parent, wxID_ANY, wxString::FromUTF8("Průvodce kontrolním hlášením DPH"))
{
    m_page1 = new Kh1Page1(this);

    m_page2 = new Kh1Page2(this);
    m_page1->Chain(m_page2);

    m_page3 = new Kh1Page3(this);
    m_page2->Chain(m_page3);

    m_pageA1 = new Kh1PageA1(this);
    m_page3->SetNext(m_pageA1);
    m_pageA1->SetPrev(m_page3);

    m_pageA2 = new Kh1PageA2(this);
    m_pageA1->SetNext(m_pageA2);
    m_pageA2->SetPrev(m_pageA1);

    m_pageA3 = new Kh1PageA3(this);
    m_pageA2->SetNext(m_pageA3);
    m_pageA3->SetPrev(m_pageA2);

    m_pageA4 = new Kh1PageA4(this);
    m_pageA3->SetNext(m_pageA4);
    m_pageA4->SetPrev(m_pageA3);

    m_pageA5 = new Kh1PageA5(this);
    m_pageA4->SetNext(m_pageA5);
    m_pageA5->SetPrev(m_pageA4);

    m_pageB1 = new Kh1PageB1(this);
    m_pageA5->SetNext(m_pageB1);
    m_pageB1->SetPrev(m_pageA5);

    m_pageB2 = new Kh1PageB2(this);
    m_pageB1->SetNext(m_pageB2);
    m_pageB2->SetPrev(m_pageB1);

    m_pageB3 = new Kh1PageB3(this);
    m_pageB2->SetNext(m_pageB3);
    m_pageB3->SetPrev(m_pageB2);

    m_pageEP = new Kh1PageEP(this);
    m_pageB3->SetNext(m_pageEP);
    m_pageEP->SetPrev(m_pageB3);

    wxSize size(-1,450);
    for(wxWizardPage *page = m_page1->GetNext(); page; page = page->GetNext())
    {
        wxSize tmp = page->GetBestSize();
        if(tmp.GetHeight() > size.GetHeight())
            size.SetHeight(tmp.GetHeight());
        if(tmp.GetWidth() > size.GetWidth())
            size.SetWidth(tmp.GetWidth());
    }
    SetPageSize(size);

    GetPageAreaSizer()->Add(m_page1);
    GetPageAreaSizer()->Add(m_page2);
    GetPageAreaSizer()->Add(m_page3);
    GetPageAreaSizer()->Add(m_pageA1);
    GetPageAreaSizer()->Add(m_pageA2);
    GetPageAreaSizer()->Add(m_pageA3);
    GetPageAreaSizer()->Add(m_pageA4);
    GetPageAreaSizer()->Add(m_pageA5);
    GetPageAreaSizer()->Add(m_pageB1);
    GetPageAreaSizer()->Add(m_pageB2);
    GetPageAreaSizer()->Add(m_pageB3);
    GetPageAreaSizer()->Add(m_pageEP);
}

void Dphkh1Wizard::fillData()
{
    if(!m_dphkh1.isEmpty()) return;

    Priznani pr = idbmodel->priznani();

    //VetaD
    m_dphkh1.setAtributValue("d_poddp", m_page2->vyhotoveni());
    m_dphkh1.setAtributValue("khdph_forma", m_page3->forma());
    m_dphkh1.setAtributValue("dokument", "KH1");
    m_dphkh1.setAtributValue("k_uladis", "DPH");
    m_dphkh1.setAtributValue("typ_platce", "P");
    if(m_page2->obdobi() == 1) m_dphkh1.setAtributValue("ctvrt", m_page2->ctvrtleti());
    if(m_page2->obdobi() == 0) m_dphkh1.setAtributValue("mesic", m_page2->mesic());
    if(m_page2->obdobi() == 2) m_dphkh1.setAtributValue("zdobd_do", m_page2->end());
    if(m_page2->obdobi() == 2) m_dphkh1.setAtributValue("zdobd_od", m_page2->start());
    m_dphkh1.setAtributValue("rok", m_page2->rok());
    if(m_page3->forma() == "N" || m_page3->forma() == "E")
    {
        m_dphkh1.setAtributValue("d_zjist", m_page3->dzjist());
    }
    m_dphkh1.setAtributValue("c_jed_vyzvy", m_page3->cJedVyzvy());
    m_dphkh1.setAtributValue("vyzva_odp", m_page3->vyzvaOdp());

    //VetaP
    m_dphkh1.setAtributValue("c_orient", pr.c_orient());
    m_dphkh1.setAtributValue("c_pop", pr.c_pop());
    m_dphkh1.setAtributValue("c_pracufo", pr.c_pracufo());
    m_dphkh1.setAtributValue("c_telef", pr.c_telef());
    m_dphkh1.setAtributValue("c_ufo", pr.c_ufo());
    m_dphkh1.setAtributValue("dic", pr.dic());
    m_dphkh1.setAtributValue("email", pr.email());
    m_dphkh1.setAtributValue("id_dats", pr.id_dats());
    m_dphkh1.setAtributValue("jmeno", pr.jmeno());
    m_dphkh1.setAtributValue("naz_obce", pr.naz_obce());
    m_dphkh1.setAtributValue("opr_jmeno", pr.opr_jmeno());
    m_dphkh1.setAtributValue("opr_postaveni", pr.opr_postaveni());
    m_dphkh1.setAtributValue("opr_prijmeni", pr.opr_prijmeni());
    m_dphkh1.setAtributValue("prijmeni", pr.prijmeni());
    m_dphkh1.setAtributValue("psc", pr.psc());
    m_dphkh1.setAtributValue("stat", pr.stat());
    m_dphkh1.setAtributValue("titul", pr.titul());
    m_dphkh1.setAtributValue("typ_ds", "F");
    m_dphkh1.setAtributValue("ulice", pr.ulice());
    if(pr.zast_typ() == "F") m_dphkh1.setAtributValue("zast_dat_nar", pr.zast_dat_nar());
    m_dphkh1.setAtributValue("zast_ev_cislo", pr.zast_ev_cislo());
    if(pr.zast_typ() == "P") m_dphkh1.setAtributValue("zast_ic", pr.zast_ic());
    m_dphkh1.setAtributValue("zast_jmeno", pr.zast_jmeno());
    m_dphkh1.setAtributValue("zast_kod", pr.zast_kod());
    m_dphkh1.setAtributValue("zast_nazev", pr.zast_nazev());
    m_dphkh1.setAtributValue("zast_prijmeni", pr.zast_prijmeni());
    m_dphkh1.setAtributValue("zast_typ", pr.zast_typ());

    if(m_dphkh1.atributValue("vyzva_odp").IsEmpty())
    {
        //VetaA1
        if(m_pageA1->getVetyCount())
        {
            dxVetaArray vetyA1 = m_pageA1->vetyA1();
            for(size_t i=0; i < vetyA1.GetCount(); i++)
            {
                m_dphkh1.addVeta(vetyA1[i]);
            }
        }
        else
        {
            m_dphkh1.removeVety("VetaA1");
        }

        //VetaA2
        if(m_pageA2->getVetyCount())
        {
            dxVetaArray vetyA2 = m_pageA2->vetyA2();
            for(size_t i=0; i < vetyA2.GetCount(); i++)
            {
                m_dphkh1.addVeta(vetyA2[i]);
            }
        }
        else
        {
            m_dphkh1.removeVety("VetaA2");
        }

        //VetaA3
        if(m_pageA3->getVetyCount())
        {
            dxVetaArray vetyA3 = m_pageA3->vetyA3();
            for(size_t i=0; i < vetyA3.GetCount(); i++)
            {
                m_dphkh1.addVeta(vetyA3[i]);
            }
        }
        else
        {
            m_dphkh1.removeVety("VetaA3");
        }

        //VetaA4
        if(m_pageA4->getVetyCount())
        {
            dxVetaArray vetyA4 = m_pageA4->vetyA4();
            for(size_t i=0; i < vetyA4.GetCount(); i++)
            {
                m_dphkh1.addVeta(fixVetaA4B2(vetyA4[i]));
            }
        }
        else
        {
            m_dphkh1.removeVety("VetaA4");
        }

        //VetaA5
        m_dphkh1.removeVety("VetaA5");
        m_dphkh1.addVeta(m_pageA5->vetaA5());

        //VetaB1
        if(m_pageB1->getVetyCount())
        {
            dxVetaArray vetyB1 = m_pageB1->vetyB1();
            for(size_t i=0; i < vetyB1.GetCount(); i++)
            {
                m_dphkh1.addVeta(vetyB1[i]);
            }
        }
        else
        {
            m_dphkh1.removeVety("VetaB1");
        }

        //VetaB2
        if(m_pageB2->getVetyCount())
        {
            dxVetaArray vetyB2 = m_pageB2->vetyB2();
            for(size_t i=0; i < vetyB2.GetCount(); i++)
            {
                m_dphkh1.addVeta(fixVetaA4B2(vetyB2[i]));
            }
        }
        else
        {
            m_dphkh1.removeVety("VetaB2");
        }

        //VetaB3
        m_dphkh1.removeVety("VetaB3");
        m_dphkh1.addVeta(m_pageB3->vetaB3());

        m_dphkh1.fillVetaC();
    }
    else
    {
        m_dphkh1.removeVety("VetaA1");
        m_dphkh1.removeVety("VetaA2");
        m_dphkh1.removeVety("VetaA3");
        m_dphkh1.removeVety("VetaA4");
        m_dphkh1.removeVety("VetaA5");
        m_dphkh1.removeVety("VetaB1");
        m_dphkh1.removeVety("VetaB2");
        m_dphkh1.removeVety("VetaB3");
        m_dphkh1.removeVety("VetaC");
    }
}

dxAtributArray Dphkh1Wizard::atributy() const
{
    return m_dphkh1.atributy();
}

dxVetaArray Dphkh1Wizard::vety() const
{
    return m_dphkh1.vety();
}

void Dphkh1Wizard::saveXml(const wxString &path)
{
    m_dphkh1.saveToXml(path);
}

wxDateTime Dphkh1Wizard::start() const
{
    return m_page2->start();
}

wxDateTime Dphkh1Wizard::end() const
{
    return m_page2->end();
}

bool Dphkh1Wizard::newXML() const
{
    return end().IsLaterThan(wxDateTime(29, wxDateTime::Sep, 2019));
}

void Dphkh1Wizard::skipDataPages(bool skip)
{
    if(skip)
    {
        m_page3->SetNext(m_pageEP);
        m_pageEP->SetPrev(m_page3);
    }
    else
    {
        m_page3->SetNext(m_pageA1);
        m_pageEP->SetPrev(m_pageB3);
    }
}

void Dphkh1Wizard::print()
{
    fillData();
    if(m_dphkh1.isNulove())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Nulové kontrolní hlášení se nepodává."), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    Dphkh1Print print(this, vety());
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void Dphkh1Wizard::saveToXml()
{
    fillData();
    if(m_dphkh1.isNulove())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Nulové kontrolní hlášení se nepodává."), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    wxFileDialog dlg(this, wxString::FromUTF8("Exportovat hlášení do"), wxEmptyString, wxEmptyString, "XML soubory (*.xml)|*.xml",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("xml", path);
    saveXml(path);
}

void Dphkh1Wizard::fillPages()
{
    m_pageA1->fillList();
    m_pageA2->fillList();
    m_pageA3->fillList();
    m_pageA4->fillList();
    m_pageA5->fillData();
    m_pageB1->fillList();
    m_pageB2->fillList();
    m_pageB3->fillData();
}

Veta Dphkh1Wizard::fixVetaA4B2(Veta veta)
{
    if(veta.atributValue("zdph_44") != "N")
    {
        if(dxutils::isSameDouble(veta.atributValueDouble("zakl_dane1"),0.0)) veta.clearAtributValue("zakl_dane1");
        if(dxutils::isSameDouble(veta.atributValueDouble("dan1"),0.0)) veta.clearAtributValue("dan1");
        if(dxutils::isSameDouble(veta.atributValueDouble("zakl_dane2"),0.0)) veta.clearAtributValue("zakl_dane2");
        if(dxutils::isSameDouble(veta.atributValueDouble("dan2"),0.0)) veta.clearAtributValue("dan2");
        if(dxutils::isSameDouble(veta.atributValueDouble("zakl_dane3"),0.0)) veta.clearAtributValue("zakl_dane3");
        if(dxutils::isSameDouble(veta.atributValueDouble("dan3"),0.0)) veta.clearAtributValue("dan3");
        if(veta.atributValue("zdph_44") == "A")
        {
            veta.clearAtributValue("zakl_dane1");
            veta.clearAtributValue("zakl_dane2");
            veta.clearAtributValue("zakl_dane3");
        }
    }
    return veta;
}

void Dphkh1Wizard::OnPageChanging(wxWizardEvent &event)
{
    if(event.GetPage() == m_page3 && event.GetDirection())
    {
        if(m_page3->vyzvaOdp().IsEmpty())
        {
            fillPages();
        }
    }
    if((event.GetPage() == m_page2 && event.GetDirection()) ||
            (event.GetPage() == m_pageEP && !event.GetDirection()))
    {
        m_dphkh1.clearData();
    }
}

BEGIN_EVENT_TABLE(Kh1PageA1, wxWizardPageSimple)
    EVT_MENU(ID_DPHKH1_PAGEA1_ADD, Kh1PageA1::OnAdd)
    EVT_MENU(ID_DPHKH1_PAGEA1_EDIT, Kh1PageA1::OnEdit)
    EVT_MENU(ID_DPHKH1_PAGEA1_DELETE, Kh1PageA1::OnDelete)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_DPHKH1_PAGEA1_LIST, Kh1PageA1::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_DPHKH1_PAGEA1_LIST, Kh1PageA1::OnItemActivated)
    EVT_BUTTON(ID_DPHKH1_PAGEA1_RELOAD, Kh1PageA1::OnReload)
END_EVENT_TABLE()

Kh1PageA1::Kh1PageA1(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphkh1Wizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("A. 1. - Uskutečněná zdanitelná plnění v režimu přenesení daňové povinnosti,u kterých je povinen přiznat daň příjemce plnění podle § 92a")), 0, wxALL|wxEXPAND, 5);

    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_DPHKH1_PAGEA1_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat řádek"));
    m_bar->AddTool(ID_DPHKH1_PAGEA1_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit řádek"));
    m_bar->AddTool(ID_DPHKH1_PAGEA1_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat řádek"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);

    m_list = new dxListCtrl(this, ID_DPHKH1_PAGEA1_LIST, false);
    m_list->InsertColumn(0, wxString::FromUTF8("DIČ odběratele"));
    m_list->SetColumnWidth(0, 110);
    m_list->InsertColumn(1, wxString::FromUTF8("Ev. číslo daňového dokladu"));
    m_list->setResizeColumn(1);
    m_list->setResizeColumnMinWidth(180);
    m_list->InsertColumn(2, "DUZP");
    m_list->SetColumnWidth(2, 90);
    m_list->InsertColumn(3, wxString::FromUTF8("Základ daně"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(3, 140);
    m_list->InsertColumn(4, wxString::FromUTF8("Kód předmětu plnění"));
    m_list->SetColumnWidth(4, 140);
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);

    m_mainSizer->Add(new wxButton(this, ID_DPHKH1_PAGEA1_RELOAD, wxString::FromUTF8("Znovu načíst data z evidence")));

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void Kh1PageA1::fillList(bool vety)
{
    m_list->DeleteAllItems();
    m_list->Hide();
    if(vety) fillVety();
    for(size_t i=0; i<m_vetyA1.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(wxString("CZ")+m_vetyA1[i].atributValue("dic_odb"));
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_vetyA1[i].atributValue("c_evid_dd"));
        m_list->SetItem(static_cast<long>(i), 2, m_vetyA1[i].atributValue("duzp"));
        m_list->SetItem(static_cast<long>(i), 3, m_vetyA1[i].atributValue("zakl_dane1"));
        m_list->SetItem(static_cast<long>(i), 4, m_vetyA1[i].atributValue("kod_pred_pl"));
    }
    m_list->Show();
    if(m_vetyA1.GetCount())
    {
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
        m_bar->EnableTool(ID_DPHKH1_PAGEA1_EDIT, true);
        m_bar->EnableTool(ID_DPHKH1_PAGEA1_DELETE, true);
    }
    else
    {
        m_bar->EnableTool(ID_DPHKH1_PAGEA1_EDIT, false);
        m_bar->EnableTool(ID_DPHKH1_PAGEA1_DELETE, false);
    }
}

dxVetaArray Kh1PageA1::vetyA1() const
{
    return m_vetyA1;
}

size_t Kh1PageA1::getVetyCount() const
{
    return m_vetyA1.GetCount();
}

void Kh1PageA1::fillVety()
{
    m_vetyA1.Clear();
    dxDphDokladArray doklady = idbmodel->dokladyForDph(idbmodel->getDphDruhIdByKh("A1"), m_wizard->start(), m_wizard->end());
    for(size_t i=0; i < doklady.GetCount(); i++)
    {
        Veta vetaa1(static_cast<int>(m_vetyA1.GetCount()), "VetaA1", 1, 1);
        dxAtributArray avetaa1;
        Atribut c_evid_dd(A_STRING, "c_evid_dd", true, 60);
        c_evid_dd.setValue(doklady[i].cislo());
        avetaa1.Add(c_evid_dd);
        Atribut c_radku(A_INT, "c_radku", false, 6);
        c_radku.setValue(static_cast<int>(m_vetyA1.GetCount())+1);
        avetaa1.Add(c_radku);
        Atribut dic_odb(A_STRING, "dic_odb", true, 10);
        dic_odb.setValue(doklady[i].dic().Mid(2));
        avetaa1.Add(dic_odb);
        Atribut duzp(A_DATUM, "duzp", true, 10, 0, "%d.%m.%Y");
        duzp.setValue(doklady[i].duzp());
        avetaa1.Add(duzp);
        Atribut kod_pred_pl(A_STRING, "kod_pred_pl", true, 3);
        kod_pred_pl.setValue(doklady[i].kpp());
        avetaa1.Add(kod_pred_pl);
        Atribut zakl_dane1(A_DOUBLE, "zakl_dane1", true, 17, 2);
        double zaklad = 0.0;
        dxDphForDokladArray dph = doklady[i].dph();
        for(size_t j=0; j < dph.GetCount(); j++)
        {
            zaklad += dph[j].zaklad();
        }
        zakl_dane1.setValue(zaklad);
        avetaa1.Add(zakl_dane1);
        vetaa1.setAtributy(avetaa1);
        m_vetyA1.Add(vetaa1);
    }
}

void Kh1PageA1::OnAdd(wxCommandEvent &/*event*/)
{
    Veta vetaa1(static_cast<int>(m_vetyA1.GetCount()), "VetaA1", 1, 1);
    dxAtributArray avetaa1;
    Atribut c_evid_dd(A_STRING, "c_evid_dd", true, 60);
    avetaa1.Add(c_evid_dd);
    Atribut c_radku(A_INT, "c_radku", false, 6);
    c_radku.setValue(static_cast<int>(m_vetyA1.GetCount())+1);
    avetaa1.Add(c_radku);
    Atribut dic_odb(A_STRING, "dic_odb", true, 10);
    avetaa1.Add(dic_odb);
    Atribut duzp(A_DATUM, "duzp", true, 10, 0, "%d.%m.%Y");
    avetaa1.Add(duzp);
    Atribut kod_pred_pl(A_STRING, "kod_pred_pl", true, 3);
    avetaa1.Add(kod_pred_pl);
    Atribut zakl_dane1(A_DOUBLE, "zakl_dane1", true, 17, 2);
    avetaa1.Add(zakl_dane1);
    vetaa1.setAtributy(avetaa1);
    Dphkh1Vetaa1Dialog dialog(this, vetaa1);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA1.Add(dialog.veta());
        fillList(false);
    }
}

void Kh1PageA1::OnEdit(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    Dphkh1Vetaa1Dialog dialog(this, m_vetyA1[static_cast<size_t>(index)]);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA1[static_cast<size_t>(index)] = dialog.veta();
        fillList(false);
    }
}

void Kh1PageA1::OnDelete(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Smazat řádek pro CZ%s,\ndoklad %s?"), m_vetyA1[static_cast<size_t>(index)].atributValue("dic_odb"),
                                                  m_vetyA1[static_cast<size_t>(index)].atributValue("c_evid_dd")),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA1.RemoveAt(static_cast<size_t>(index));
        fillList(false);
    }
}

void Kh1PageA1::OnReload(wxCommandEvent &/*event*/)
{
    fillList();
}

void Kh1PageA1::OnItemRightClick(wxListEvent &event)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMenu popup;
    popup.Append(ID_DPHKH1_PAGEA1_ADD, wxString::FromUTF8("Přidat řádek"));
    popup.Append(ID_DPHKH1_PAGEA1_EDIT, wxString::FromUTF8("Upravit řádek"));
    popup.Append(ID_DPHKH1_PAGEA1_DELETE, wxString::FromUTF8("Smazat řádek"));
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void Kh1PageA1::OnItemActivated(wxListEvent &event)
{
    Dphkh1Vetaa1Dialog dialog(this, m_vetyA1[static_cast<size_t>(event.GetIndex())]);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA1[static_cast<size_t>(event.GetIndex())] = dialog.veta();
        fillList(false);
    }
}

long Kh1PageA1::selectedIndex()
{
    long index = -1;
    while((index = m_list->GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != wxNOT_FOUND)
    {
        return index;
    }
    return index;
}

BEGIN_EVENT_TABLE(Kh1PageA2, wxWizardPageSimple)
    EVT_MENU(ID_DPHKH1_PAGEA2_ADD, Kh1PageA2::OnAdd)
    EVT_MENU(ID_DPHKH1_PAGEA2_EDIT, Kh1PageA2::OnEdit)
    EVT_MENU(ID_DPHKH1_PAGEA2_DELETE, Kh1PageA2::OnDelete)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_DPHKH1_PAGEA2_LIST, Kh1PageA2::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_DPHKH1_PAGEA2_LIST, Kh1PageA2::OnItemActivated)
    EVT_BUTTON(ID_DPHKH1_PAGEA2_RELOAD, Kh1PageA2::OnReload)
END_EVENT_TABLE()

Kh1PageA2::Kh1PageA2(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphkh1Wizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_mainLabel = new wxStaticText(this, wxID_ANY, wxEmptyString);
    m_mainSizer->Add(m_mainLabel, 0, wxALL|wxEXPAND, 5);

    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_DPHKH1_PAGEA2_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat řádek"));
    m_bar->AddTool(ID_DPHKH1_PAGEA2_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit řádek"));
    m_bar->AddTool(ID_DPHKH1_PAGEA2_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat řádek"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);

    m_list = new dxListCtrl(this, ID_DPHKH1_PAGEA2_LIST, false);
    m_list->InsertColumn(0, wxString::FromUTF8("Identifikace dodavatele"));
    m_list->SetColumnWidth(0, 160);
    m_list->InsertColumn(1, wxString::FromUTF8("Ev. číslo daňového dokladu"));
    m_list->setResizeColumn(1);
    m_list->setResizeColumnMinWidth(180);
    m_list->InsertColumn(2, "DPPD");
    m_list->SetColumnWidth(2, 90);
    m_list->InsertColumn(3, wxString::FromUTF8("Základ daně 1"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(3, 110);
    m_list->InsertColumn(4, wxString::FromUTF8("Daň 1"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(4, 110);
    m_list->InsertColumn(5, wxString::FromUTF8("Základ daně 2"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(5, 110);
    m_list->InsertColumn(6, wxString::FromUTF8("Daň 2"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(6, 110);
    m_list->InsertColumn(7, wxString::FromUTF8("Základ daně 3"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(7, 110);
    m_list->InsertColumn(8, wxString::FromUTF8("Daň 3"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(8, 110);
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);

    m_mainSizer->Add(new wxButton(this, ID_DPHKH1_PAGEA2_RELOAD, wxString::FromUTF8("Znovu načíst data z evidence")));

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void Kh1PageA2::fillList(bool vety)
{
    m_mainLabel->SetLabel(m_wizard->newXML()?wxString::FromUTF8("A. 2. - Přijatá zdanitelná plnění, u kterých je povinen přiznat daň příjemce dle § 108 odst. 2 a 3 (§ 24, § 25)"):
                                             wxString::FromUTF8("A. 2. - Přijatá zdanitelná plnění, u kterých je povinen přiznat daň příjemce dle § 108 odst. 1 písm. b) a c) (§ 24, § 25)"));
    m_list->DeleteAllItems();
    m_list->Hide();
    if(vety) fillVety();
    for(size_t i=0; i<m_vetyA2.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(m_vetyA2[i].atributValue("k_stat")+m_vetyA2[i].atributValue("vatid_dod"));
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_vetyA2[i].atributValue("c_evid_dd"));
        m_list->SetItem(static_cast<long>(i), 2, m_vetyA2[i].atributValue("dppd"));
        m_list->SetItem(static_cast<long>(i), 3, m_vetyA2[i].atributValue("zakl_dane1"));
        m_list->SetItem(static_cast<long>(i), 4, m_vetyA2[i].atributValue("dan1"));
        m_list->SetItem(static_cast<long>(i), 5, m_vetyA2[i].atributValue("zakl_dane2"));
        m_list->SetItem(static_cast<long>(i), 6, m_vetyA2[i].atributValue("dan2"));
        m_list->SetItem(static_cast<long>(i), 7, m_vetyA2[i].atributValue("zakl_dane3"));
        m_list->SetItem(static_cast<long>(i), 8, m_vetyA2[i].atributValue("dan3"));
    }
    m_list->Show();
    if(m_vetyA2.GetCount())
    {
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
        m_bar->EnableTool(ID_DPHKH1_PAGEA2_EDIT, true);
        m_bar->EnableTool(ID_DPHKH1_PAGEA2_DELETE, true);
    }
    else
    {
        m_bar->EnableTool(ID_DPHKH1_PAGEA2_EDIT, false);
        m_bar->EnableTool(ID_DPHKH1_PAGEA2_DELETE, false);
    }
}

dxVetaArray Kh1PageA2::vetyA2() const
{
    return m_vetyA2;
}

size_t Kh1PageA2::getVetyCount() const
{
    return m_vetyA2.GetCount();
}

void Kh1PageA2::fillVety()
{
    m_vetyA2.Clear();
    dxDphDokladArray doklady = idbmodel->dokladyForDph(idbmodel->getDphDruhIdByKh("A2"), m_wizard->start(), m_wizard->end());
    for(size_t i=0; i < doklady.GetCount(); i++)
    {
        dxDphForDokladArray dph = doklady[i].dph();
        Veta vetaa2(static_cast<int>(m_vetyA2.GetCount()), "VetaA2", 1, 1);
        dxAtributArray avetaa2;
        Atribut c_evid_dd(A_STRING, "c_evid_dd", true, 60);
        c_evid_dd.setValue(doklady[i].cislo());
        avetaa2.Add(c_evid_dd);
        Atribut c_radku(A_INT, "c_radku", false, 6);
        c_radku.setValue(static_cast<int>(m_vetyA2.GetCount())+1);
        avetaa2.Add(c_radku);
        Atribut dan1(A_DOUBLE, "dan1", true, 17, 2);
        if(dph.GetCount() > 0) dan1.setValue(dph[0].dph());
        avetaa2.Add(dan1);
        Atribut dan2(A_DOUBLE, "dan2", true, 17, 2);
        if(dph.GetCount() > 1) dan2.setValue(dph[1].dph());
        avetaa2.Add(dan2);
        Atribut dan3(A_DOUBLE, "dan3", true, 17, 2);
        if(dph.GetCount() > 2) dan3.setValue(dph[2].dph());
        avetaa2.Add(dan3);
        Atribut dppd(A_DATUM, "dppd", true, 10, 0, "%d.%m.%Y");
        dppd.setValue(doklady[i].dppd());
        avetaa2.Add(dppd);
        Atribut k_stat(A_STRING, "k_stat", true, 2);
        k_stat.setValue(doklady[i].dic().Left(2));
        avetaa2.Add(k_stat);
        Atribut vatid_dod(A_STRING, "vatid_dod", true, 12);
        vatid_dod.setValue(doklady[i].dic().Mid(2));
        avetaa2.Add(vatid_dod);
        Atribut zakl_dane1(A_DOUBLE, "zakl_dane1", true, 17, 2);
        if(dph.GetCount() > 0) zakl_dane1.setValue(dph[0].zaklad());
        avetaa2.Add(zakl_dane1);
        Atribut zakl_dane2(A_DOUBLE, "zakl_dane2", true, 17, 2);
        if(dph.GetCount() > 1) zakl_dane2.setValue(dph[1].zaklad());
        avetaa2.Add(zakl_dane2);
        Atribut zakl_dane3(A_DOUBLE, "zakl_dane3", true, 17, 2);
        if(dph.GetCount() > 2) zakl_dane3.setValue(dph[2].zaklad());
        avetaa2.Add(zakl_dane3);
        vetaa2.setAtributy(avetaa2);
        m_vetyA2.Add(vetaa2);
    }
}

void Kh1PageA2::OnAdd(wxCommandEvent &/*event*/)
{
    Veta vetaa2(static_cast<int>(m_vetyA2.GetCount()), "VetaA2", 1, 1);
    dxAtributArray avetaa2;
    Atribut c_evid_dd(A_STRING, "c_evid_dd", true, 60);
    avetaa2.Add(c_evid_dd);
    Atribut c_radku(A_INT, "c_radku", false, 6);
    c_radku.setValue(static_cast<int>(m_vetyA2.GetCount())+1);
    avetaa2.Add(c_radku);
    Atribut dan1(A_DOUBLE, "dan1", true, 17, 2);
    avetaa2.Add(dan1);
    Atribut dan2(A_DOUBLE, "dan2", true, 17, 2);
    avetaa2.Add(dan2);
    Atribut dan3(A_DOUBLE, "dan3", true, 17, 2);
    avetaa2.Add(dan3);
    Atribut dppd(A_DATUM, "dppd", true, 10, 0, "%d.%m.%Y");
    avetaa2.Add(dppd);
    Atribut k_stat(A_STRING, "k_stat", true, 2);
    avetaa2.Add(k_stat);
    Atribut vatid_dod(A_STRING, "vatid_dod", true, 12);
    avetaa2.Add(vatid_dod);
    Atribut zakl_dane1(A_DOUBLE, "zakl_dane1", true, 17, 2);
    avetaa2.Add(zakl_dane1);
    Atribut zakl_dane2(A_DOUBLE, "zakl_dane2", true, 17, 2);
    avetaa2.Add(zakl_dane2);
    Atribut zakl_dane3(A_DOUBLE, "zakl_dane3", true, 17, 2);
    avetaa2.Add(zakl_dane3);
    vetaa2.setAtributy(avetaa2);
    Dphkh1Vetaa2Dialog dialog(this, vetaa2, m_wizard->end().GetYear());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA2.Add(dialog.veta());
        fillList(false);
    }
}

void Kh1PageA2::OnEdit(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    Dphkh1Vetaa2Dialog dialog(this, m_vetyA2[static_cast<size_t>(index)], m_wizard->end().GetYear());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA2[static_cast<size_t>(index)] = dialog.veta();
        fillList(false);
    }
}

void Kh1PageA2::OnDelete(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Smazat řádek pro %s%s,\ndoklad %s?"), m_vetyA2[static_cast<size_t>(index)].atributValue("k_stat"),
                                                  m_vetyA2[static_cast<size_t>(index)].atributValue("vatid_dod"), m_vetyA2[static_cast<size_t>(index)].atributValue("c_evid_dd")),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA2.RemoveAt(static_cast<size_t>(index));
        fillList(false);
    }
}

void Kh1PageA2::OnReload(wxCommandEvent &/*event*/)
{
    fillList();
}

void Kh1PageA2::OnItemRightClick(wxListEvent &event)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMenu popup;
    popup.Append(ID_DPHKH1_PAGEA2_ADD, wxString::FromUTF8("Přidat řádek"));
    popup.Append(ID_DPHKH1_PAGEA2_EDIT, wxString::FromUTF8("Upravit řádek"));
    popup.Append(ID_DPHKH1_PAGEA2_DELETE, wxString::FromUTF8("Smazat řádek"));
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void Kh1PageA2::OnItemActivated(wxListEvent &event)
{
    Dphkh1Vetaa2Dialog dialog(this, m_vetyA2[static_cast<size_t>(event.GetIndex())], m_wizard->end().GetYear());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA2[static_cast<size_t>(event.GetIndex())] = dialog.veta();
        fillList(false);
    }
}

long Kh1PageA2::selectedIndex()
{
    long index = -1;
    while((index = m_list->GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != wxNOT_FOUND)
    {
        return index;
    }
    return index;
}

BEGIN_EVENT_TABLE(Kh1PageA3, wxWizardPageSimple)
    EVT_MENU(ID_DPHKH1_PAGEA3_ADD, Kh1PageA3::OnAdd)
    EVT_MENU(ID_DPHKH1_PAGEA3_EDIT, Kh1PageA3::OnEdit)
    EVT_MENU(ID_DPHKH1_PAGEA3_DELETE, Kh1PageA3::OnDelete)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_DPHKH1_PAGEA3_LIST, Kh1PageA3::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_DPHKH1_PAGEA3_LIST, Kh1PageA3::OnItemActivated)
    EVT_BUTTON(ID_DPHKH1_PAGEA3_RELOAD, Kh1PageA3::OnReload)
END_EVENT_TABLE()

Kh1PageA3::Kh1PageA3(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphkh1Wizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("A. 3. - Uskutečněná plnění ve zvláštním režimu pro investiční zlato podle § 101c odst. 1 písm. c) bod 2")), 0, wxALL|wxEXPAND, 5);

    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_DPHKH1_PAGEA3_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat řádek"));
    m_bar->AddTool(ID_DPHKH1_PAGEA3_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit řádek"));
    m_bar->AddTool(ID_DPHKH1_PAGEA3_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat řádek"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);

    m_list = new dxListCtrl(this, ID_DPHKH1_PAGEA3_LIST, false);
    m_list->InsertColumn(0, wxString::FromUTF8("Identifikace odběratele"));
    m_list->SetColumnWidth(0, 160);
    m_list->InsertColumn(1, wxString::FromUTF8("Jméno, příjmení / obchodní jméno"));
    m_list->SetColumnWidth(1, 230);
    m_list->InsertColumn(2, wxString::FromUTF8("Datum narození"));
    m_list->SetColumnWidth(2, 110);
    m_list->InsertColumn(3, wxString::FromUTF8("Místo pobytu / Sídlo"));
    m_list->SetColumnWidth(3, 150);
    m_list->InsertColumn(4, wxString::FromUTF8("Ev. číslo daňového dokladu"));
    m_list->setResizeColumn(4);
    m_list->setResizeColumnMinWidth(160);
    m_list->InsertColumn(5, "DUP");
    m_list->SetColumnWidth(5, 80);
    m_list->InsertColumn(6, wxString::FromUTF8("Hodnota osvoboz. plnění"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(6, 170);
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);

    m_mainSizer->Add(new wxButton(this, ID_DPHKH1_PAGEA3_RELOAD, wxString::FromUTF8("Znovu načíst data z evidence")));

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void Kh1PageA3::fillList(bool vety)
{
    m_list->DeleteAllItems();
    m_list->Hide();
    if(vety) fillVety();
    for(size_t i=0; i<m_vetyA3.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(m_vetyA3[i].atributValue("k_stat")+m_vetyA3[i].atributValue("vatid_odb"));
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_vetyA3[i].atributValue("jm_prijm_obch"));
        m_list->SetItem(static_cast<long>(i), 2, m_vetyA3[i].atributValue("d_narozeni"));
        m_list->SetItem(static_cast<long>(i), 3, m_vetyA3[i].atributValue("m_pobytu_sidlo"));
        m_list->SetItem(static_cast<long>(i), 4, m_vetyA3[i].atributValue("c_evid_dd"));
        m_list->SetItem(static_cast<long>(i), 5, m_vetyA3[i].atributValue("dup"));
        m_list->SetItem(static_cast<long>(i), 6, m_vetyA3[i].atributValue("osv_plneni"));
    }
    m_list->Show();
    if(m_vetyA3.GetCount())
    {
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
        m_bar->EnableTool(ID_DPHKH1_PAGEA3_EDIT, true);
        m_bar->EnableTool(ID_DPHKH1_PAGEA3_DELETE, true);
    }
    else
    {
        m_bar->EnableTool(ID_DPHKH1_PAGEA3_EDIT, false);
        m_bar->EnableTool(ID_DPHKH1_PAGEA3_DELETE, false);
    }
}

dxVetaArray Kh1PageA3::vetyA3()
{
    for(size_t i=0; i<m_vetyA3.GetCount(); i++)
    {
        if(m_vetyA3[i].atributValue("k_stat").IsEmpty() && (m_vetyA3[i].atributValue("jm_prijm_obch").IsEmpty()
                                                            || m_vetyA3[i].atributValue("m_pobytu_sidlo").IsEmpty())) //remove neplatnych radku, neproslo by KH
        {
            m_vetyA3.RemoveAt(i);
            if(i) i--;
        }
    }
    //precislovani radku
    for(size_t i=0; i<m_vetyA3.GetCount(); i++)
    {
        m_vetyA3[i].setAtributValue("c_radku", static_cast<int>(i)+1);
    }
    return m_vetyA3;
}

size_t Kh1PageA3::getVetyCount() const
{
    return m_vetyA3.GetCount();
}

void Kh1PageA3::fillVety()
{
    m_vetyA3.Clear();
    dxDphDokladArray doklady = idbmodel->dokladyForDph(idbmodel->getDphDruhIdByKh("A3"), m_wizard->start(), m_wizard->end(), false);
    for(size_t i=0; i < doklady.GetCount(); i++)
    {
        dxDphForDokladArray dph = doklady[i].dph();
        Veta vetaa3(static_cast<int>(m_vetyA3.GetCount()), "VetaA3", 1, 1);
        dxAtributArray avetaa3;
        Atribut c_evid_dd(A_STRING, "c_evid_dd", true, 60);
        c_evid_dd.setValue(doklady[i].cislo());
        avetaa3.Add(c_evid_dd);
        Atribut c_radku(A_INT, "c_radku", false, 6);
        c_radku.setValue(static_cast<int>(m_vetyA3.GetCount())+1);
        avetaa3.Add(c_radku);
        Atribut d_narozeni(A_DATUM, "d_narozeni", true, 10, 0, "%d.%m.%Y");
        avetaa3.Add(d_narozeni);
        Atribut dup(A_DATUM, "dup", true, 10, 0, "%d.%m.%Y");
        dup.setValue(doklady[i].duzp());
        avetaa3.Add(dup);
        Atribut jm_prijm_obch(A_STRING, "jm_prijm_obch", true, 57);
        Atribut k_stat(A_STRING, "k_stat", true, 2);
        Atribut m_pobytu_sidlo(A_STRING, "m_pobytu_sidlo", true, 100);
        Atribut osv_plneni(A_DOUBLE, "osv_plneni", true, 17, 2);
        double zaklad = 0.0;
        for(size_t j=0; j < dph.GetCount(); j++)
        {
            zaklad += dph[j].zaklad();
        }
        osv_plneni.setValue(zaklad);
        avetaa3.Add(osv_plneni);
        Atribut vatid_odb(A_STRING, "vatid_odb", true, 12);
        Kontakt ko = idbmodel->getKontakt(doklady[i].kontaktid());
        Adresa ad = ko.isValid() ? idbmodel->getAdresa(ko.adresaid()) : Adresa();
        if(idbmodel->hasKontaktDic(doklady[i].kontaktid()))
        {
            k_stat.setValue(ko.dic().Left(2));
            vatid_odb.setValue(ko.dic().Mid(2));
        }
        else
        {
            if(ad.isValid())
            {
                jm_prijm_obch.setValue(ko.isValid()?ko.jmeno():"");
                m_pobytu_sidlo.setValue(ad.id()?ad.ulice()+" "+ad.cp()+","+ad.psc()+" "+ad.misto()+","+ad.zeme():"");
            }
        }
        avetaa3.Add(k_stat);
        avetaa3.Add(vatid_odb);
        avetaa3.Add(jm_prijm_obch);
        avetaa3.Add(m_pobytu_sidlo);
        vetaa3.setAtributy(avetaa3);
        m_vetyA3.Add(vetaa3);
    }
}

void Kh1PageA3::OnAdd(wxCommandEvent &/*event*/)
{
    Veta vetaa3(static_cast<int>(m_vetyA3.GetCount()), "VetaA3", 1, 1);
    dxAtributArray avetaa3;
    Atribut c_evid_dd(A_STRING, "c_evid_dd", true, 60);
    avetaa3.Add(c_evid_dd);
    Atribut c_radku(A_INT, "c_radku", false, 6);
    c_radku.setValue(static_cast<int>(m_vetyA3.GetCount())+1);
    avetaa3.Add(c_radku);
    Atribut d_narozeni(A_DATUM, "d_narozeni", true, 10, 0, "%d.%m.%Y");
    avetaa3.Add(d_narozeni);
    Atribut dup(A_DATUM, "dup", true, 10, 0, "%d.%m.%Y");
    avetaa3.Add(dup);
    Atribut jm_prijm_obch(A_STRING, "jm_prijm_obch", true, 57);
    avetaa3.Add(jm_prijm_obch);
    Atribut k_stat(A_STRING, "k_stat", true, 2);
    k_stat.setValue("CZ");
    avetaa3.Add(k_stat);
    Atribut m_pobytu_sidlo(A_STRING, "m_pobytu_sidlo", true, 100);
    avetaa3.Add(m_pobytu_sidlo);
    Atribut osv_plneni(A_DOUBLE, "osv_plneni", true, 17, 2);
    avetaa3.Add(osv_plneni);
    Atribut vatid_odb(A_STRING, "vatid_odb", true, 12);
    avetaa3.Add(vatid_odb);
    vetaa3.setAtributy(avetaa3);
    Dphkh1Vetaa3Dialog dialog(this, vetaa3);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA3.Add(dialog.veta());
        fillList(false);
    }
}

void Kh1PageA3::OnEdit(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    Dphkh1Vetaa3Dialog dialog(this, m_vetyA3[static_cast<size_t>(index)]);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA3[static_cast<size_t>(index)] = dialog.veta();
        fillList(false);
    }
}

void Kh1PageA3::OnDelete(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Smazat řádek pro doklad %s?"), m_vetyA3[static_cast<size_t>(index)].atributValue("c_evid_dd")),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA3.RemoveAt(static_cast<size_t>(index));
        fillList(false);
    }
}

void Kh1PageA3::OnReload(wxCommandEvent &/*event*/)
{
    fillList();
}

void Kh1PageA3::OnItemRightClick(wxListEvent &event)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMenu popup;
    popup.Append(ID_DPHKH1_PAGEA3_ADD, wxString::FromUTF8("Přidat řádek"));
    popup.Append(ID_DPHKH1_PAGEA3_EDIT, wxString::FromUTF8("Upravit řádek"));
    popup.Append(ID_DPHKH1_PAGEA3_DELETE, wxString::FromUTF8("Smazat řádek"));
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void Kh1PageA3::OnItemActivated(wxListEvent &event)
{
    Dphkh1Vetaa3Dialog dialog(this, m_vetyA3[static_cast<size_t>(event.GetIndex())]);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA3[static_cast<size_t>(event.GetIndex())] = dialog.veta();
        fillList(false);
    }
}

long Kh1PageA3::selectedIndex()
{
    long index = -1;
    while((index = m_list->GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != wxNOT_FOUND)
    {
        return index;
    }
    return index;
}

BEGIN_EVENT_TABLE(Kh1PageA4, wxWizardPageSimple)
    EVT_MENU(ID_DPHKH1_PAGEA4_ADD, Kh1PageA4::OnAdd)
    EVT_MENU(ID_DPHKH1_PAGEA4_EDIT, Kh1PageA4::OnEdit)
    EVT_MENU(ID_DPHKH1_PAGEA4_DELETE, Kh1PageA4::OnDelete)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_DPHKH1_PAGEA4_LIST, Kh1PageA4::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_DPHKH1_PAGEA4_LIST, Kh1PageA4::OnItemActivated)
    EVT_BUTTON(ID_DPHKH1_PAGEA4_RELOAD, Kh1PageA4::OnReload)
END_EVENT_TABLE()

Kh1PageA4::Kh1PageA4(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphkh1Wizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_mainLabel = new wxStaticText(this, wxID_ANY, wxEmptyString);
    m_mainSizer->Add(m_mainLabel, 0, wxALL|wxEXPAND, 5);

    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_DPHKH1_PAGEA4_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat řádek"));
    m_bar->AddTool(ID_DPHKH1_PAGEA4_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit řádek"));
    m_bar->AddTool(ID_DPHKH1_PAGEA4_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat řádek"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);

    m_list = new dxListCtrl(this, ID_DPHKH1_PAGEA4_LIST, false);
    m_list->InsertColumn(0, wxString::FromUTF8("DIČ odběratele"));
    m_list->SetColumnWidth(0, 110);
    m_list->InsertColumn(1, wxString::FromUTF8("Ev. číslo daňového dokladu"));
    m_list->setResizeColumn(1);
    m_list->setResizeColumnMinWidth(180);
    m_list->InsertColumn(2, "DPPD");
    m_list->SetColumnWidth(2, 90);
    m_list->InsertColumn(3, wxString::FromUTF8("Základ daně 1"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(3, 110);
    m_list->InsertColumn(4, wxString::FromUTF8("Daň 1"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(4, 110);
    m_list->InsertColumn(5, wxString::FromUTF8("Základ daně 2"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(5, 110);
    m_list->InsertColumn(6, wxString::FromUTF8("Daň 2"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(6, 110);
    m_list->InsertColumn(7, wxString::FromUTF8("Základ daně 3"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(7, 110);
    m_list->InsertColumn(8, wxString::FromUTF8("Daň 3"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(8, 110);
    m_list->InsertColumn(9, wxString::FromUTF8("Kód režimu plnění"));
    m_list->SetColumnWidth(9, 130);
    m_list->InsertColumn(10, wxString::FromUTF8("§ 44 ZDPH"));
    m_list->SetColumnWidth(10, 90);
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);

    m_mainSizer->Add(new wxButton(this, ID_DPHKH1_PAGEA4_RELOAD, wxString::FromUTF8("Znovu načíst data z evidence")));

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void Kh1PageA4::fillList(bool vety)
{
    m_mainLabel->SetLabel(m_wizard->newXML()?wxString::FromUTF8("A. 4. - Uskutečněná zdanitelná plnění a přijaté úplaty s povinností přiznat daň dle § 108 odst. 1 s hodnotou nad 10.000,- Kč včetně daně\na všechny provedené opravy v souvislosti s nedobytnými pohledávkami bez ohledu na limit"):
                                             wxString::FromUTF8("A. 4. - Uskutečněná zdanitelná plnění a přijaté úplaty s povinností přiznat daň dle § 108 odst. 1 písm. a) s hodnotou nad 10.000,- Kč včetně daně\na všechny provedené opravy podle § 44 bez ohledu na limit"));
    wxListItem item;
    m_list->GetColumn(10, item);
    item.SetText(m_wizard->newXML()?wxString::FromUTF8("Opravy u nedobytné pohledávky"):wxString::FromUTF8("§ 44 ZDPH"));
    m_list->SetColumn(10, item);
    if(m_wizard->newXML()) m_list->SetColumnWidth(10, 210);
    m_list->DeleteAllItems();
    m_list->Hide();
    if(vety) fillVety();
    for(size_t i=0; i<m_vetyA4.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(wxString("CZ")+m_vetyA4[i].atributValue("dic_odb"));
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_vetyA4[i].atributValue("c_evid_dd"));
        m_list->SetItem(static_cast<long>(i), 2, m_vetyA4[i].atributValue("dppd"));
        m_list->SetItem(static_cast<long>(i), 3, m_vetyA4[i].atributValue("zakl_dane1"));
        m_list->SetItem(static_cast<long>(i), 4, m_vetyA4[i].atributValue("dan1"));
        m_list->SetItem(static_cast<long>(i), 5, m_vetyA4[i].atributValue("zakl_dane2"));
        m_list->SetItem(static_cast<long>(i), 6, m_vetyA4[i].atributValue("dan2"));
        m_list->SetItem(static_cast<long>(i), 7, m_vetyA4[i].atributValue("zakl_dane3"));
        m_list->SetItem(static_cast<long>(i), 8, m_vetyA4[i].atributValue("dan3"));
        m_list->SetItem(static_cast<long>(i), 9, m_vetyA4[i].atributValue("kod_rezim_pl"));
        m_list->SetItem(static_cast<long>(i), 10, m_vetyA4[i].atributValue("zdph_44"));
    }
    m_list->Show();
    if(m_vetyA4.GetCount())
    {
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
        m_bar->EnableTool(ID_DPHKH1_PAGEA4_EDIT, true);
        m_bar->EnableTool(ID_DPHKH1_PAGEA4_DELETE, true);
    }
    else
    {
        m_bar->EnableTool(ID_DPHKH1_PAGEA4_EDIT, false);
        m_bar->EnableTool(ID_DPHKH1_PAGEA4_DELETE, false);
    }
}

dxVetaArray Kh1PageA4::vetyA4() const
{
    return m_vetyA4;
}

size_t Kh1PageA4::getVetyCount() const
{
    return m_vetyA4.GetCount();
}

void Kh1PageA4::fillVety()
{
    m_vetyA4.Clear();
    dxDphDokladArray doklady = idbmodel->dokladyForDph(idbmodel->getDphDruhIdByKh("A4,A5"), m_wizard->start(), m_wizard->end());
    for(size_t i=0; i < doklady.GetCount(); i++)
    {
        if(doklady[i].celkem() < 10000.0)
        {
            continue;
        }
        dxDphForDokladArray dph = doklady[i].dph();
        Veta vetaa4(static_cast<int>(m_vetyA4.GetCount()), "VetaA4", 1, 1);
        dxAtributArray avetaa4;
        Atribut c_evid_dd(A_STRING, "c_evid_dd", true, 60);
        c_evid_dd.setValue(doklady[i].cislo());
        avetaa4.Add(c_evid_dd);
        Atribut c_radku(A_INT, "c_radku", false, 6);
        c_radku.setValue(static_cast<int>(m_vetyA4.GetCount())+1);
        avetaa4.Add(c_radku);
        Atribut dan1(A_DOUBLE, "dan1", true, 17, 2);
        if(dph.GetCount() > 0) dan1.setValue(dph[0].dph());
        avetaa4.Add(dan1);
        Atribut dan2(A_DOUBLE, "dan2", true, 17, 2);
        if(dph.GetCount() > 1) dan2.setValue(dph[1].dph());
        avetaa4.Add(dan2);
        Atribut dan3(A_DOUBLE, "dan3", true, 17, 2);
        if(dph.GetCount() > 2) dan3.setValue(dph[2].dph());
        avetaa4.Add(dan3);
        Atribut dppd(A_DATUM, "dppd", true, 10, 0, "%d.%m.%Y");
        dppd.setValue(doklady[i].dppd());
        avetaa4.Add(dppd);
        Atribut vatid_dod(A_STRING, "dic_odb", true, 10);
        vatid_dod.setValue(doklady[i].dic().Mid(2));
        avetaa4.Add(vatid_dod);
        Atribut zakl_dane1(A_DOUBLE, "zakl_dane1", true, 17, 2);
        if(dph.GetCount() > 0) zakl_dane1.setValue(dph[0].zaklad());
        avetaa4.Add(zakl_dane1);
        Atribut zakl_dane2(A_DOUBLE, "zakl_dane2", true, 17, 2);
        if(dph.GetCount() > 1) zakl_dane2.setValue(dph[1].zaklad());
        avetaa4.Add(zakl_dane2);
        Atribut zakl_dane3(A_DOUBLE, "zakl_dane3", true, 17, 2);
        if(dph.GetCount() > 2) zakl_dane3.setValue(dph[2].zaklad());
        avetaa4.Add(zakl_dane3);
        Atribut kod_rezim_pl(A_STRING, "kod_rezim_pl", true, 1);
        kod_rezim_pl.setValue(doklady[i].krp());
        avetaa4.Add(kod_rezim_pl);
        Atribut zdph_44(A_STRING, "zdph_44", true, 1);
        zdph_44.setValue("N");
        avetaa4.Add(zdph_44);
        vetaa4.setAtributy(avetaa4);
        m_vetyA4.Add(vetaa4);
    }
}

void Kh1PageA4::OnAdd(wxCommandEvent &/*event*/)
{
    Veta vetaa4(static_cast<int>(m_vetyA4.GetCount()), "VetaA4", 1, 1);
    dxAtributArray avetaa4;
    Atribut c_evid_dd(A_STRING, "c_evid_dd", true, 60);
    avetaa4.Add(c_evid_dd);
    Atribut c_radku(A_INT, "c_radku", false, 6);
    c_radku.setValue(static_cast<int>(m_vetyA4.GetCount())+1);
    avetaa4.Add(c_radku);
    Atribut dan1(A_DOUBLE, "dan1", true, 17, 2);
    avetaa4.Add(dan1);
    Atribut dan2(A_DOUBLE, "dan2", true, 17, 2);
    avetaa4.Add(dan2);
    Atribut dan3(A_DOUBLE, "dan3", true, 17, 2);
    avetaa4.Add(dan3);
    Atribut dppd(A_DATUM, "dppd", true, 10, 0, "%d.%m.%Y");
    avetaa4.Add(dppd);
    Atribut vatid_dod(A_STRING, "dic_odb", true, 12);
    avetaa4.Add(vatid_dod);
    Atribut zakl_dane1(A_DOUBLE, "zakl_dane1", true, 17, 2);
    avetaa4.Add(zakl_dane1);
    Atribut zakl_dane2(A_DOUBLE, "zakl_dane2", true, 17, 2);
    avetaa4.Add(zakl_dane2);
    Atribut zakl_dane3(A_DOUBLE, "zakl_dane3", true, 17, 2);
    avetaa4.Add(zakl_dane3);
    Atribut kod_rezim_pl(A_STRING, "kod_rezim_pl", true, 1);
    kod_rezim_pl.setValue("0");
    avetaa4.Add(kod_rezim_pl);
    Atribut zdph_44(A_STRING, "zdph_44", true, 1);
    zdph_44.setValue("N");
    avetaa4.Add(zdph_44);
    vetaa4.setAtributy(avetaa4);
    Dphkh1Vetaa4Dialog dialog(this, vetaa4, m_wizard->end().GetYear(), m_wizard->newXML());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA4.Add(dialog.veta());
        fillList(false);
    }
}

void Kh1PageA4::OnEdit(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    Dphkh1Vetaa4Dialog dialog(this, m_vetyA4[static_cast<size_t>(index)], m_wizard->end().GetYear(), m_wizard->newXML());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA4[static_cast<size_t>(index)] = dialog.veta();
        fillList(false);
    }
}

void Kh1PageA4::OnDelete(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Smazat řádek pro CZ%s,\ndoklad %s?"), m_vetyA4[static_cast<size_t>(index)].atributValue("dic_odb"),
                           m_vetyA4[static_cast<size_t>(index)].atributValue("c_evid_dd")), "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA4.RemoveAt(static_cast<size_t>(index));
        fillList(false);
    }
}

void Kh1PageA4::OnReload(wxCommandEvent &/*event*/)
{
    fillList();
}

void Kh1PageA4::OnItemRightClick(wxListEvent &event)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMenu popup;
    popup.Append(ID_DPHKH1_PAGEA4_ADD, wxString::FromUTF8("Přidat řádek"));
    popup.Append(ID_DPHKH1_PAGEA4_EDIT, wxString::FromUTF8("Upravit řádek"));
    popup.Append(ID_DPHKH1_PAGEA4_DELETE, wxString::FromUTF8("Smazat řádek"));
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void Kh1PageA4::OnItemActivated(wxListEvent &event)
{
    Dphkh1Vetaa4Dialog dialog(this, m_vetyA4[static_cast<size_t>(event.GetIndex())], m_wizard->end().GetYear(), m_wizard->newXML());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyA4[static_cast<size_t>(event.GetIndex())] = dialog.veta();
        fillList(false);
    }
}

long Kh1PageA4::selectedIndex()
{
    long index = -1;
    while((index = m_list->GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != wxNOT_FOUND)
    {
        return index;
    }
    return index;
}

BEGIN_EVENT_TABLE(Kh1PageA5, wxWizardPageSimple)
    EVT_SPINCTRLDOUBLE(ID_DPHKH1_PAGEA5_ZAKLAD1, Kh1PageA5::OnZaklad1)
    EVT_SPINCTRLDOUBLE(ID_DPHKH1_PAGEA5_ZAKLAD2, Kh1PageA5::OnZaklad2)
    EVT_SPINCTRLDOUBLE(ID_DPHKH1_PAGEA5_ZAKLAD3, Kh1PageA5::OnZaklad3)
    EVT_BUTTON(ID_DPHKH1_PAGEA5_RELOAD, Kh1PageA5::OnReload)
END_EVENT_TABLE()

Kh1PageA5::Kh1PageA5(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphkh1Wizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_mainLabel = new wxStaticText(this, wxID_ANY, wxEmptyString);
    m_mainSizer->Add(m_mainLabel, 0, wxALL|wxEXPAND, 5);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně\npro základní sazbu")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane1 = new wxSpinCtrlDouble(this, ID_DPHKH1_PAGEA5_ZAKLAD1);
    m_zaklDane1->SetDigits(2);
    m_zaklDane1->SetRange(0.0, 99999999999999.0);
    gridSizer->Add(m_zaklDane1, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Daň\npro základní sazbu")), wxGBPosition(0,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dan1 = new wxStaticText(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    gridSizer->Add(m_dan1, wxGBPosition(0,3), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně\npro 1. sníženou sazbu")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane2 = new wxSpinCtrlDouble(this, ID_DPHKH1_PAGEA5_ZAKLAD2);
    m_zaklDane2->SetDigits(2);
    m_zaklDane2->SetRange(0.0, 99999999999999.0);
    gridSizer->Add(m_zaklDane2, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Daň\npro 1. sníženou sazbu")), wxGBPosition(1,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dan2 = new wxStaticText(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    gridSizer->Add(m_dan2, wxGBPosition(1,3), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně\npro 2. sníženou sazbu")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane3 = new wxSpinCtrlDouble(this, ID_DPHKH1_PAGEA5_ZAKLAD3);
    m_zaklDane3->SetDigits(2);
    m_zaklDane3->SetRange(0.0, 99999999999999.0);
    gridSizer->Add(m_zaklDane3, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Daň\npro 2. sníženou sazbu")), wxGBPosition(2,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dan3 = new wxStaticText(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    gridSizer->Add(m_dan3, wxGBPosition(2,3), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mainSizer->Add(gridSizer, 0, wxALL, 5);

    m_mainSizer->Add(new wxButton(this, ID_DPHKH1_PAGEA5_RELOAD, wxString::FromUTF8("Znovu načíst data z evidence")));

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);

    m_vetaA5 = Veta(1, "VetaA5", 1, 1);
    dxAtributArray avetaa5;
    Atribut dan1(A_DOUBLE, "dan1", true, 17, 2);
    avetaa5.Add(dan1);
    Atribut dan2(A_DOUBLE, "dan2", true, 17, 2);
    avetaa5.Add(dan2);
    Atribut dan3(A_DOUBLE, "dan3", true, 17, 2);
    avetaa5.Add(dan3);
    Atribut zakl_dane1(A_DOUBLE, "zakl_dane1", true, 17, 2);
    avetaa5.Add(zakl_dane1);
    Atribut zakl_dane2(A_DOUBLE, "zakl_dane2", true, 17, 2);
    avetaa5.Add(zakl_dane2);
    Atribut zakl_dane3(A_DOUBLE, "zakl_dane3", true, 17, 2);
    avetaa5.Add(zakl_dane3);
    m_vetaA5.setAtributy(avetaa5);
}

void Kh1PageA5::fillData()
{
    m_mainLabel->SetLabel(m_wizard->newXML()?wxString::FromUTF8("A. 5. - Ostatní uskutečněná zdanitelná plnění a přijaté úplaty s povinností přiznat daň dle § 108 odst. 1 s hodnotou do 10.000,- Kč včetně daně,\nnebo plnění, u nichž nevznikla povinnost vystavit daňový doklad"):
                                             wxString::FromUTF8("A. 5. - Ostatní uskutečněná zdanitelná plnění a přijaté úplaty s povinností přiznat daň dle § 108 odst. 1 písm. a) s hodnotou do 10.000,- Kč včetně daně,\nnebo plnění, u nichž nevznikla povinnost vystavit daňový doklad"));
    double zakl_dane1 = 0.0, dan1 = 0.0, zakl_dane2 = 0.0, dan2 = 0.0, zakl_dane3 = 0.0, dan3 = 0.0;
    dxDphDokladArray doklady = idbmodel->dokladyForDph(idbmodel->getDphDruhIdByKh("A4,A5"), m_wizard->start(), m_wizard->end(), false);
    for(size_t i=0; i < doklady.GetCount(); i++)
    {
        if(doklady[i].celkem() > 10000.0 && idbmodel->hasKontaktDic(doklady[i].kontaktid()))
        {
            continue;
        }
        dxDphForDokladArray dph = doklady[i].dph();
        if(dph.GetCount() > 0) dan1 += dph[0].dph();
        if(dph.GetCount() > 1) dan2 += dph[1].dph();
        if(dph.GetCount() > 2) dan3 += dph[2].dph();
        if(dph.GetCount() > 0) zakl_dane1 += dph[0].zaklad();
        if(dph.GetCount() > 1) zakl_dane2 += dph[1].zaklad();
        if(dph.GetCount() > 2) zakl_dane3 += dph[2].zaklad();
    }
    m_vetaA5.setAtributValue("dan1", dan1);
    m_vetaA5.setAtributValue("dan2", dan2);
    m_vetaA5.setAtributValue("dan3", dan3);
    m_vetaA5.setAtributValue("zakl_dane1", zakl_dane1);
    m_vetaA5.setAtributValue("zakl_dane2", zakl_dane2);
    m_vetaA5.setAtributValue("zakl_dane3", zakl_dane3);
    m_dan1->SetLabel(m_vetaA5.atributValue("dan1"));
    m_dan2->SetLabel(m_vetaA5.atributValue("dan2"));
    m_dan3->SetLabel(m_vetaA5.atributValue("dan3"));
    m_zaklDane1->SetValue(m_vetaA5.atributValueDouble("zakl_dane1"));
    m_zaklDane2->SetValue(m_vetaA5.atributValueDouble("zakl_dane2"));
    m_zaklDane3->SetValue(m_vetaA5.atributValueDouble("zakl_dane3"));
}

Veta Kh1PageA5::vetaA5() const
{
    return m_vetaA5;
}

void Kh1PageA5::OnReload(wxCommandEvent &/*event*/)
{
    fillData();
}

void Kh1PageA5::OnZaklad1(wxSpinDoubleEvent &/*event*/)
{
    m_dan1->SetLabel(wxString::FromCDouble(m_zaklDane1->GetValue()*(idbmodel->zakladniSazba(m_wizard->end().GetYear())/100.0),2));
    m_vetaA5.setAtributValue("zakl_dane1", m_zaklDane1->GetValue());
    m_vetaA5.setAtributValue("dan1", m_dan1->GetLabel());
}

void Kh1PageA5::OnZaklad2(wxSpinDoubleEvent &/*event*/)
{
    m_dan2->SetLabel(wxString::FromCDouble(m_zaklDane2->GetValue()*(idbmodel->prvniSnizenaSazba(m_wizard->end().GetYear())/100.0),2));
    m_vetaA5.setAtributValue("zakl_dane2", m_zaklDane2->GetValue());
    m_vetaA5.setAtributValue("dan2", m_dan2->GetLabel());
}

void Kh1PageA5::OnZaklad3(wxSpinDoubleEvent &/*event*/)
{
    m_dan3->SetLabel(wxString::FromCDouble(m_zaklDane3->GetValue()*(idbmodel->druhaSnizenaSazba(m_wizard->end().GetYear())/100.0),2));
    m_vetaA5.setAtributValue("zakl_dane3", m_zaklDane3->GetValue());
    m_vetaA5.setAtributValue("dan3", m_dan3->GetLabel());
}

BEGIN_EVENT_TABLE(Kh1PageB1, wxWizardPageSimple)
    EVT_MENU(ID_DPHKH1_PAGEB1_ADD, Kh1PageB1::OnAdd)
    EVT_MENU(ID_DPHKH1_PAGEB1_EDIT, Kh1PageB1::OnEdit)
    EVT_MENU(ID_DPHKH1_PAGEB1_DELETE, Kh1PageB1::OnDelete)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_DPHKH1_PAGEB1_LIST, Kh1PageB1::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_DPHKH1_PAGEB1_LIST, Kh1PageB1::OnItemActivated)
    EVT_BUTTON(ID_DPHKH1_PAGEB1_RELOAD, Kh1PageB1::OnReload)
END_EVENT_TABLE()

Kh1PageB1::Kh1PageB1(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphkh1Wizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("B. 1. - Přijatá zdanitelná plnění v režimu přenesení daňové povinnosti, u kterých je povinen přiznat daň příjemce podle § 92a")), 0, wxALL|wxEXPAND, 5);

    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_DPHKH1_PAGEB1_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat řádek"));
    m_bar->AddTool(ID_DPHKH1_PAGEB1_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit řádek"));
    m_bar->AddTool(ID_DPHKH1_PAGEB1_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat řádek"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);

    m_list = new dxListCtrl(this, ID_DPHKH1_PAGEB1_LIST, false);
    m_list->InsertColumn(0, wxString::FromUTF8("DIČ dodavatele"));
    m_list->SetColumnWidth(0, 160);
    m_list->InsertColumn(1, wxString::FromUTF8("Ev. číslo daňového dokladu"));
    m_list->setResizeColumn(1);
    m_list->setResizeColumnMinWidth(180);
    m_list->InsertColumn(2, "DUZP");
    m_list->SetColumnWidth(2, 90);
    m_list->InsertColumn(3, wxString::FromUTF8("Základ daně 1"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(3, 110);
    m_list->InsertColumn(4, wxString::FromUTF8("Daň 1"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(4, 110);
    m_list->InsertColumn(5, wxString::FromUTF8("Základ daně 2"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(5, 110);
    m_list->InsertColumn(6, wxString::FromUTF8("Daň 2"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(6, 110);
    m_list->InsertColumn(7, wxString::FromUTF8("Základ daně 3"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(7, 110);
    m_list->InsertColumn(8, wxString::FromUTF8("Daň 3"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(8, 110);
    m_list->InsertColumn(9, wxString::FromUTF8("Kód předmětu plnění"));
    m_list->SetColumnWidth(9, 140);
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);

    m_mainSizer->Add(new wxButton(this, ID_DPHKH1_PAGEB1_RELOAD, wxString::FromUTF8("Znovu načíst data z evidence")));

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void Kh1PageB1::fillList(bool vety)
{
    m_list->DeleteAllItems();
    m_list->Hide();
    if(vety) fillVety();
    for(size_t i=0; i<m_vetyB1.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(wxString("CZ")+m_vetyB1[i].atributValue("dic_dod"));
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_vetyB1[i].atributValue("c_evid_dd"));
        m_list->SetItem(static_cast<long>(i), 2, m_vetyB1[i].atributValue("duzp"));
        m_list->SetItem(static_cast<long>(i), 3, m_vetyB1[i].atributValue("zakl_dane1"));
        m_list->SetItem(static_cast<long>(i), 4, m_vetyB1[i].atributValue("dan1"));
        m_list->SetItem(static_cast<long>(i), 5, m_vetyB1[i].atributValue("zakl_dane2"));
        m_list->SetItem(static_cast<long>(i), 6, m_vetyB1[i].atributValue("dan2"));
        m_list->SetItem(static_cast<long>(i), 7, m_vetyB1[i].atributValue("zakl_dane3"));
        m_list->SetItem(static_cast<long>(i), 8, m_vetyB1[i].atributValue("dan3"));
        m_list->SetItem(static_cast<long>(i), 9, m_vetyB1[i].atributValue("kod_pred_pl"));
    }
    m_list->Show();
    if(m_vetyB1.GetCount())
    {
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
        m_bar->EnableTool(ID_DPHKH1_PAGEB1_EDIT, true);
        m_bar->EnableTool(ID_DPHKH1_PAGEB1_DELETE, true);
    }
    else
    {
        m_bar->EnableTool(ID_DPHKH1_PAGEB1_EDIT, false);
        m_bar->EnableTool(ID_DPHKH1_PAGEB1_DELETE, false);
    }
}

dxVetaArray Kh1PageB1::vetyB1() const
{
    return m_vetyB1;
}

size_t Kh1PageB1::getVetyCount() const
{
    return m_vetyB1.GetCount();
}

void Kh1PageB1::fillVety()
{
    m_vetyB1.Clear();
    dxDphDokladArray doklady = idbmodel->dokladyForDph(idbmodel->getDphDruhIdByKh("B1"), m_wizard->start(), m_wizard->end());
    for(size_t i=0; i < doklady.GetCount(); i++)
    {
        dxDphForDokladArray dph = doklady[i].dph();
        Veta vetaB1(static_cast<int>(m_vetyB1.GetCount()), "VetaB1", 1, 1);
        dxAtributArray avetab1;
        Atribut c_evid_dd(A_STRING, "c_evid_dd", true, 60);
        c_evid_dd.setValue(doklady[i].cislo());
        avetab1.Add(c_evid_dd);
        Atribut c_radku(A_INT, "c_radku", false, 6);
        c_radku.setValue(static_cast<int>(m_vetyB1.GetCount())+1);
        avetab1.Add(c_radku);
        Atribut dan1(A_DOUBLE, "dan1", true, 17, 2);
        if(dph.GetCount() > 0) dan1.setValue(dph[0].dph());
        avetab1.Add(dan1);
        Atribut dan2(A_DOUBLE, "dan2", true, 17, 2);
        if(dph.GetCount() > 1) dan2.setValue(dph[1].dph());
        avetab1.Add(dan2);
        Atribut dan3(A_DOUBLE, "dan3", true, 17, 2);
        if(dph.GetCount() > 2) dan3.setValue(dph[2].dph());
        avetab1.Add(dan3);
        Atribut dppd(A_DATUM, "duzp", true, 10, 0, "%d.%m.%Y");
        dppd.setValue(doklady[i].duzp());
        avetab1.Add(dppd);
        Atribut vatid_dod(A_STRING, "dic_dod", true, 12);
        vatid_dod.setValue(doklady[i].dic().Mid(2));
        avetab1.Add(vatid_dod);
        Atribut zakl_dane1(A_DOUBLE, "zakl_dane1", true, 17, 2);
        if(dph.GetCount() > 0) zakl_dane1.setValue(dph[0].zaklad());
        avetab1.Add(zakl_dane1);
        Atribut zakl_dane2(A_DOUBLE, "zakl_dane2", true, 17, 2);
        if(dph.GetCount() > 1) zakl_dane2.setValue(dph[1].zaklad());
        avetab1.Add(zakl_dane2);
        Atribut zakl_dane3(A_DOUBLE, "zakl_dane3", true, 17, 2);
        if(dph.GetCount() > 2) zakl_dane3.setValue(dph[2].zaklad());
        avetab1.Add(zakl_dane3);
        Atribut kod_pred_pl(A_STRING, "kod_pred_pl", true, 3);
        kod_pred_pl.setValue(doklady[i].kpp());
        avetab1.Add(kod_pred_pl);
        vetaB1.setAtributy(avetab1);
        m_vetyB1.Add(vetaB1);
    }
}

void Kh1PageB1::OnAdd(wxCommandEvent &/*event*/)
{
    Veta vetaB1(static_cast<int>(m_vetyB1.GetCount()), "VetaB1", 1, 1);
    dxAtributArray avetaB1;
    Atribut c_evid_dd(A_STRING, "c_evid_dd", true, 60);
    avetaB1.Add(c_evid_dd);
    Atribut c_radku(A_INT, "c_radku", false, 6);
    c_radku.setValue(static_cast<int>(m_vetyB1.GetCount())+1);
    avetaB1.Add(c_radku);
    Atribut dan1(A_DOUBLE, "dan1", true, 17, 2);
    avetaB1.Add(dan1);
    Atribut dan2(A_DOUBLE, "dan2", true, 17, 2);
    avetaB1.Add(dan2);
    Atribut dan3(A_DOUBLE, "dan3", true, 17, 2);
    avetaB1.Add(dan3);
    Atribut dppd(A_DATUM, "duzp", true, 10, 0, "%d.%m.%Y");
    avetaB1.Add(dppd);
    Atribut vatid_dod(A_STRING, "dic_dod", true, 12);
    avetaB1.Add(vatid_dod);
    Atribut zakl_dane1(A_DOUBLE, "zakl_dane1", true, 17, 2);
    avetaB1.Add(zakl_dane1);
    Atribut zakl_dane2(A_DOUBLE, "zakl_dane2", true, 17, 2);
    avetaB1.Add(zakl_dane2);
    Atribut zakl_dane3(A_DOUBLE, "zakl_dane3", true, 17, 2);
    avetaB1.Add(zakl_dane3);
    Atribut kod_pred_pl(A_STRING, "kod_pred_pl", true, 3);
    avetaB1.Add(kod_pred_pl);
    vetaB1.setAtributy(avetaB1);
    Dphkh1Vetab1Dialog dialog(this, vetaB1, m_wizard->end().GetYear());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyB1.Add(dialog.veta());
        fillList(false);
    }
}

void Kh1PageB1::OnEdit(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    Dphkh1Vetab1Dialog dialog(this, m_vetyB1[static_cast<size_t>(index)], m_wizard->end().GetYear());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyB1[static_cast<size_t>(index)] = dialog.veta();
        fillList(false);
    }
}

void Kh1PageB1::OnDelete(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Smazat řádek pro CZ%s,\ndoklad %s?"),
                                                  m_vetyB1[static_cast<size_t>(index)].atributValue("vatid_dod"), m_vetyB1[static_cast<size_t>(index)].atributValue("c_evid_dd")),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyB1.RemoveAt(static_cast<size_t>(index));
        fillList(false);
    }
}

void Kh1PageB1::OnReload(wxCommandEvent &/*event*/)
{
    fillList();
}

void Kh1PageB1::OnItemRightClick(wxListEvent &event)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMenu popup;
    popup.Append(ID_DPHKH1_PAGEB1_ADD, wxString::FromUTF8("Přidat řádek"));
    popup.Append(ID_DPHKH1_PAGEB1_EDIT, wxString::FromUTF8("Upravit řádek"));
    popup.Append(ID_DPHKH1_PAGEB1_DELETE, wxString::FromUTF8("Smazat řádek"));
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void Kh1PageB1::OnItemActivated(wxListEvent &event)
{
    Dphkh1Vetab1Dialog dialog(this, m_vetyB1[static_cast<size_t>(event.GetIndex())], m_wizard->end().GetYear());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyB1[static_cast<size_t>(event.GetIndex())] = dialog.veta();
        fillList(false);
    }
}

long Kh1PageB1::selectedIndex()
{
    long index = -1;
    while((index = m_list->GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != wxNOT_FOUND)
    {
        return index;
    }
    return index;
}

BEGIN_EVENT_TABLE(Kh1PageB2, wxWizardPageSimple)
    EVT_MENU(ID_DPHKH1_PAGEB2_ADD, Kh1PageB2::OnAdd)
    EVT_MENU(ID_DPHKH1_PAGEB2_EDIT, Kh1PageB2::OnEdit)
    EVT_MENU(ID_DPHKH1_PAGEB2_DELETE, Kh1PageB2::OnDelete)
    EVT_LIST_ITEM_RIGHT_CLICK(ID_DPHKH1_PAGEB2_LIST, Kh1PageB2::OnItemRightClick)
    EVT_LIST_ITEM_ACTIVATED(ID_DPHKH1_PAGEB2_LIST, Kh1PageB2::OnItemActivated)
    EVT_BUTTON(ID_DPHKH1_PAGEB2_RELOAD, Kh1PageB2::OnReload)
END_EVENT_TABLE()

Kh1PageB2::Kh1PageB2(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphkh1Wizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_mainLabel = new wxStaticText(this, wxID_ANY, wxEmptyString);
    m_mainSizer->Add(m_mainLabel, 0, wxALL|wxEXPAND, 5);

    m_bar = new dxToolBar(this);
    m_bar->AddTool(ID_DPHKH1_PAGEB2_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat řádek"));
    m_bar->AddTool(ID_DPHKH1_PAGEB2_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, wxString::FromUTF8("Upravit řádek"));
    m_bar->AddTool(ID_DPHKH1_PAGEB2_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, wxString::FromUTF8("Smazat řádek"));
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);

    m_list = new dxListCtrl(this, ID_DPHKH1_PAGEB2_LIST, false);
    m_list->InsertColumn(0, wxString::FromUTF8("DIČ dodavatele"));
    m_list->SetColumnWidth(0, 110);
    m_list->InsertColumn(1, wxString::FromUTF8("Ev. číslo daňového dokladu"));
    m_list->setResizeColumn(1);
    m_list->setResizeColumnMinWidth(180);
    m_list->InsertColumn(2, "DPPD");
    m_list->SetColumnWidth(2, 90);
    m_list->InsertColumn(3, wxString::FromUTF8("Základ daně 1"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(3, 110);
    m_list->InsertColumn(4, wxString::FromUTF8("Daň 1"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(4, 110);
    m_list->InsertColumn(5, wxString::FromUTF8("Základ daně 2"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(5, 110);
    m_list->InsertColumn(6, wxString::FromUTF8("Daň 2"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(6, 110);
    m_list->InsertColumn(7, wxString::FromUTF8("Základ daně 3"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(7, 110);
    m_list->InsertColumn(8, wxString::FromUTF8("Daň 3"), wxLIST_FORMAT_RIGHT);
    m_list->SetColumnWidth(8, 110);
    m_list->InsertColumn(9, wxString::FromUTF8("Použit poměr"));
    m_list->SetColumnWidth(9, 100);
    m_list->InsertColumn(10, wxString::FromUTF8("§ 44 ZDPH"));
    m_list->SetColumnWidth(10, 90);
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);

    m_mainSizer->Add(new wxButton(this, ID_DPHKH1_PAGEB2_RELOAD, wxString::FromUTF8("Znovu načíst data z evidence")));

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void Kh1PageB2::fillList(bool vety)
{
    m_mainLabel->SetLabel(m_wizard->newXML()?wxString::FromUTF8("B. 2. - Přijatá zdanitelná plnění a poskytnuté úplaty, u kterých příjemce uplatňuje nárok na odpočet daně dle § 73 odst. 1 písm. a) s hodnotou nad 10.000,- Kč včetně daně \na všechny opravy odpočtu v souvislosti s nedobytnými pohledávkami bez ohledu na limit"):
                                             wxString::FromUTF8("B. 2. - Přijatá zdanitelná plnění a poskytnuté úplaty, u kterých příjemce uplatňuje nárok na odpočet daně dle § 73 odst. 1 písm. a) s hodnotou nad 10.000,- Kč včetně daně \na všechny přijaté opravy podle § 44 bez ohledu na limit"));
    wxListItem item;
    m_list->GetColumn(10, item);
    item.SetText(m_wizard->newXML()?wxString::FromUTF8("Opravy u nedobytné pohledávky"):wxString::FromUTF8("§ 44 ZDPH"));
    m_list->SetColumn(10, item);
    if(m_wizard->newXML()) m_list->SetColumnWidth(10, 210);
    m_list->DeleteAllItems();
    m_list->Hide();
    if(vety) fillVety();
    for(size_t i=0; i<m_vetyB2.GetCount(); i++)
    {
        wxListItem item;
        item.SetId(static_cast<long>(i));
        item.SetText(wxString("CZ")+m_vetyB2[i].atributValue("dic_dod"));
        m_list->InsertItem(item);
        m_list->SetItem(static_cast<long>(i), 1, m_vetyB2[i].atributValue("c_evid_dd"));
        m_list->SetItem(static_cast<long>(i), 2, m_vetyB2[i].atributValue("dppd"));
        m_list->SetItem(static_cast<long>(i), 3, m_vetyB2[i].atributValue("zakl_dane1"));
        m_list->SetItem(static_cast<long>(i), 4, m_vetyB2[i].atributValue("dan1"));
        m_list->SetItem(static_cast<long>(i), 5, m_vetyB2[i].atributValue("zakl_dane2"));
        m_list->SetItem(static_cast<long>(i), 6, m_vetyB2[i].atributValue("dan2"));
        m_list->SetItem(static_cast<long>(i), 7, m_vetyB2[i].atributValue("zakl_dane3"));
        m_list->SetItem(static_cast<long>(i), 8, m_vetyB2[i].atributValue("dan3"));
        m_list->SetItem(static_cast<long>(i), 9, m_vetyB2[i].atributValue("pomer"));
        m_list->SetItem(static_cast<long>(i), 10, m_vetyB2[i].atributValue("zdph_44"));
    }
    m_list->Show();
    if(m_vetyB2.GetCount())
    {
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
        m_bar->EnableTool(ID_DPHKH1_PAGEB2_EDIT, true);
        m_bar->EnableTool(ID_DPHKH1_PAGEB2_DELETE, true);
    }
    else
    {
        m_bar->EnableTool(ID_DPHKH1_PAGEB2_EDIT, false);
        m_bar->EnableTool(ID_DPHKH1_PAGEB2_DELETE, false);
    }
}

dxVetaArray Kh1PageB2::vetyB2() const
{
    return m_vetyB2;
}

size_t Kh1PageB2::getVetyCount() const
{
    return m_vetyB2.GetCount();
}

void Kh1PageB2::fillVety()
{
    m_vetyB2.Clear();
    dxDphDokladArray doklady = idbmodel->dokladyForDph(idbmodel->getDphDruhIdByKh("B2,B3"), m_wizard->start(), m_wizard->end());
    for(size_t i=0; i < doklady.GetCount(); i++)
    {
        if(doklady[i].celkem() < 10000.0)
        {
            continue;
        }
        dxDphForDokladArray dph = doklady[i].dph();
        Veta vetab2(static_cast<int>(m_vetyB2.GetCount()), "VetaB2", 1, 1);
        dxAtributArray avetab2;
        Atribut c_evid_dd(A_STRING, "c_evid_dd", true, 60);
        c_evid_dd.setValue(doklady[i].cislo());
        avetab2.Add(c_evid_dd);
        Atribut c_radku(A_INT, "c_radku", false, 6);
        c_radku.setValue(static_cast<int>(m_vetyB2.GetCount())+1);
        avetab2.Add(c_radku);
        Atribut dan1(A_DOUBLE, "dan1", true, 17, 2);
        if(dph.GetCount() > 0) dan1.setValue(dph[0].dph());
        avetab2.Add(dan1);
        Atribut dan2(A_DOUBLE, "dan2", true, 17, 2);
        if(dph.GetCount() > 1) dan2.setValue(dph[1].dph());
        avetab2.Add(dan2);
        Atribut dan3(A_DOUBLE, "dan3", true, 17, 2);
        if(dph.GetCount() > 2) dan3.setValue(dph[2].dph());
        avetab2.Add(dan3);
        Atribut dppd(A_DATUM, "dppd", true, 10, 0, "%d.%m.%Y");
        dppd.setValue(doklady[i].dppd());
        avetab2.Add(dppd);
        Atribut vatid_dod(A_STRING, "dic_dod", true, 10);
        vatid_dod.setValue(doklady[i].dic().Mid(2));
        avetab2.Add(vatid_dod);
        Atribut zakl_dane1(A_DOUBLE, "zakl_dane1", true, 17, 2);
        if(dph.GetCount() > 0) zakl_dane1.setValue(dph[0].zaklad());
        avetab2.Add(zakl_dane1);
        Atribut zakl_dane2(A_DOUBLE, "zakl_dane2", true, 17, 2);
        if(dph.GetCount() > 1) zakl_dane2.setValue(dph[1].zaklad());
        avetab2.Add(zakl_dane2);
        Atribut zakl_dane3(A_DOUBLE, "zakl_dane3", true, 17, 2);
        if(dph.GetCount() > 2) zakl_dane3.setValue(dph[2].zaklad());
        avetab2.Add(zakl_dane3);
        Atribut pomer(A_STRING, "pomer", true, 1);
        pomer.setValue("N");
        avetab2.Add(pomer);
        Atribut zdph_44(A_STRING, "zdph_44", true, 1);
        zdph_44.setValue("N");
        avetab2.Add(zdph_44);
        vetab2.setAtributy(avetab2);
        m_vetyB2.Add(vetab2);
    }
}

void Kh1PageB2::OnAdd(wxCommandEvent &/*event*/)
{
    Veta vetab2(static_cast<int>(m_vetyB2.GetCount()), "VetaB2", 1, 1);
    dxAtributArray avetab2;
    Atribut c_evid_dd(A_STRING, "c_evid_dd", true, 60);
    avetab2.Add(c_evid_dd);
    Atribut c_radku(A_INT, "c_radku", false, 6);
    c_radku.setValue(static_cast<int>(m_vetyB2.GetCount())+1);
    avetab2.Add(c_radku);
    Atribut dan1(A_DOUBLE, "dan1", true, 17, 2);
    avetab2.Add(dan1);
    Atribut dan2(A_DOUBLE, "dan2", true, 17, 2);
    avetab2.Add(dan2);
    Atribut dan3(A_DOUBLE, "dan3", true, 17, 2);
    avetab2.Add(dan3);
    Atribut dppd(A_DATUM, "dppd", true, 10, 0, "%d.%m.%Y");
    avetab2.Add(dppd);
    Atribut vatid_dod(A_STRING, "dic_dod", true, 12);
    avetab2.Add(vatid_dod);
    Atribut zakl_dane1(A_DOUBLE, "zakl_dane1", true, 17, 2);
    avetab2.Add(zakl_dane1);
    Atribut zakl_dane2(A_DOUBLE, "zakl_dane2", true, 17, 2);
    avetab2.Add(zakl_dane2);
    Atribut zakl_dane3(A_DOUBLE, "zakl_dane3", true, 17, 2);
    avetab2.Add(zakl_dane3);
    Atribut pomer(A_STRING, "pomer", true, 1);
    pomer.setValue("N");
    avetab2.Add(pomer);
    Atribut zdph_44(A_STRING, "zdph_44", true, 1);
    zdph_44.setValue("N");
    avetab2.Add(zdph_44);
    vetab2.setAtributy(avetab2);
    Dphkh1Vetab2Dialog dialog(this, vetab2, m_wizard->end().GetYear(), m_wizard->newXML());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyB2.Add(dialog.veta());
        fillList(false);
    }
}

void Kh1PageB2::OnEdit(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    Dphkh1Vetab2Dialog dialog(this, m_vetyB2[static_cast<size_t>(index)], m_wizard->end().GetYear(), m_wizard->newXML());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyB2[static_cast<size_t>(index)] = dialog.veta();
        fillList(false);
    }
}

void Kh1PageB2::OnDelete(wxCommandEvent &/*event*/)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMessageDialog dialog(this, wxString::Format(wxString::FromUTF8("Smazat řádek pro CZ%s,\ndoklad %s?"), m_vetyB2[static_cast<size_t>(index)].atributValue("dic_dod"),
                                                  m_vetyB2[static_cast<size_t>(index)].atributValue("c_evid_dd")),
                           "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyB2.RemoveAt(static_cast<size_t>(index));
        fillList(false);
    }
}

void Kh1PageB2::OnReload(wxCommandEvent &/*event*/)
{
    fillList();
}

void Kh1PageB2::OnItemRightClick(wxListEvent &event)
{
    long index = selectedIndex();
    if(index == -1) return;
    wxMenu popup;
    popup.Append(ID_DPHKH1_PAGEB2_ADD, wxString::FromUTF8("Přidat řádek"));
    popup.Append(ID_DPHKH1_PAGEB2_EDIT, wxString::FromUTF8("Upravit řádek"));
    popup.Append(ID_DPHKH1_PAGEB2_DELETE, wxString::FromUTF8("Smazat řádek"));
    PopupMenu(&popup, event.GetPoint()+m_list->GetPosition());
}

void Kh1PageB2::OnItemActivated(wxListEvent &event)
{
    Dphkh1Vetab2Dialog dialog(this, m_vetyB2[static_cast<size_t>(event.GetIndex())], m_wizard->end().GetYear(), m_wizard->newXML());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_vetyB2[static_cast<size_t>(event.GetIndex())] = dialog.veta();
        fillList(false);
    }
}

long Kh1PageB2::selectedIndex()
{
    long index = -1;
    while((index = m_list->GetNextItem(index, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED)) != wxNOT_FOUND)
    {
        return index;
    }
    return index;
}

BEGIN_EVENT_TABLE(Kh1PageB3, wxWizardPageSimple)
    EVT_SPINCTRLDOUBLE(ID_DPHKH1_PAGEB3_ZAKLAD1, Kh1PageB3::OnZaklad1)
    EVT_SPINCTRLDOUBLE(ID_DPHKH1_PAGEB3_ZAKLAD2, Kh1PageB3::OnZaklad2)
    EVT_SPINCTRLDOUBLE(ID_DPHKH1_PAGEB3_ZAKLAD3, Kh1PageB3::OnZaklad3)
    EVT_BUTTON(ID_DPHKH1_PAGEB3_RELOAD, Kh1PageB3::OnReload)
END_EVENT_TABLE()

Kh1PageB3::Kh1PageB3(wxWizard *parent)
    : wxWizardPageSimple(parent), m_wizard(static_cast<Dphkh1Wizard*>(parent))
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    m_mainSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("B. 3. - Přijatá zdanitelná plnění a poskytnuté úplaty, u kterých příjemce uplatňuje nárok na odpočet daně dle § 73 odst. 1 písm.a) s hodnotou do 10.000,- Kč včetně daně")), 0, wxALL|wxEXPAND, 5);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně\npro základní sazbu")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane1 = new wxSpinCtrlDouble(this, ID_DPHKH1_PAGEB3_ZAKLAD1);
    m_zaklDane1->SetDigits(2);
    m_zaklDane1->SetRange(0.0, 99999999999999.0);
    gridSizer->Add(m_zaklDane1, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Daň\npro základní sazbu")), wxGBPosition(0,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dan1 = new wxStaticText(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    gridSizer->Add(m_dan1, wxGBPosition(0,3), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně\npro 1. sníženou sazbu")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane2 = new wxSpinCtrlDouble(this, ID_DPHKH1_PAGEB3_ZAKLAD2);
    m_zaklDane2->SetDigits(2);
    m_zaklDane2->SetRange(0.0, 99999999999999.0);
    gridSizer->Add(m_zaklDane2, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Daň\npro 1. sníženou sazbu")), wxGBPosition(1,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dan2 = new wxStaticText(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    gridSizer->Add(m_dan2, wxGBPosition(1,3), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Základ daně\npro 2. sníženou sazbu")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zaklDane3 = new wxSpinCtrlDouble(this, ID_DPHKH1_PAGEB3_ZAKLAD3);
    m_zaklDane3->SetDigits(2);
    m_zaklDane3->SetRange(0.0, 99999999999999.0);
    gridSizer->Add(m_zaklDane3, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Daň\npro 2. sníženou sazbu")), wxGBPosition(2,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dan3 = new wxStaticText(this, wxID_ANY, "", wxDefaultPosition, wxDefaultSize, wxALIGN_RIGHT);
    gridSizer->Add(m_dan3, wxGBPosition(2,3), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mainSizer->Add(gridSizer, 0, wxALL, 5);

    m_mainSizer->Add(new wxButton(this, ID_DPHKH1_PAGEB3_RELOAD, wxString::FromUTF8("Znovu načíst data z evidence")));

    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);

    m_vetaB3 = Veta(1, "VetaB3", 1, 1);
    dxAtributArray avetaa5;
    Atribut dan1(A_DOUBLE, "dan1", true, 17, 2);
    avetaa5.Add(dan1);
    Atribut dan2(A_DOUBLE, "dan2", true, 17, 2);
    avetaa5.Add(dan2);
    Atribut dan3(A_DOUBLE, "dan3", true, 17, 2);
    avetaa5.Add(dan3);
    Atribut zakl_dane1(A_DOUBLE, "zakl_dane1", true, 17, 2);
    avetaa5.Add(zakl_dane1);
    Atribut zakl_dane2(A_DOUBLE, "zakl_dane2", true, 17, 2);
    avetaa5.Add(zakl_dane2);
    Atribut zakl_dane3(A_DOUBLE, "zakl_dane3", true, 17, 2);
    avetaa5.Add(zakl_dane3);
    m_vetaB3.setAtributy(avetaa5);
}

void Kh1PageB3::fillData()
{
    double zakl_dane1 = 0.0, dan1 = 0.0, zakl_dane2 = 0.0, dan2 = 0.0, zakl_dane3 = 0.0, dan3 = 0.0;
    dxDphDokladArray doklady = idbmodel->dokladyForDph(idbmodel->getDphDruhIdByKh("B2,B3"), m_wizard->start(), m_wizard->end(), false);
    for(size_t i=0; i < doklady.GetCount(); i++)
    {
        if(doklady[i].celkem() > 10000.0 && idbmodel->hasKontaktDic(doklady[i].kontaktid()))
        {
            continue;
        }
        dxDphForDokladArray dph = doklady[i].dph();
        if(dph.GetCount() > 0) dan1 += dph[0].dph();
        if(dph.GetCount() > 1) dan2 += dph[1].dph();
        if(dph.GetCount() > 2) dan3 += dph[2].dph();
        if(dph.GetCount() > 0) zakl_dane1 += dph[0].zaklad();
        if(dph.GetCount() > 1) zakl_dane2 += dph[1].zaklad();
        if(dph.GetCount() > 2) zakl_dane3 += dph[2].zaklad();
    }
    m_vetaB3.setAtributValue("dan1", dan1);
    m_vetaB3.setAtributValue("dan2", dan2);
    m_vetaB3.setAtributValue("dan3", dan3);
    m_vetaB3.setAtributValue("zakl_dane1", zakl_dane1);
    m_vetaB3.setAtributValue("zakl_dane2", zakl_dane2);
    m_vetaB3.setAtributValue("zakl_dane3", zakl_dane3);
    m_dan1->SetLabel(m_vetaB3.atributValue("dan1"));
    m_dan2->SetLabel(m_vetaB3.atributValue("dan2"));
    m_dan3->SetLabel(m_vetaB3.atributValue("dan3"));
    m_zaklDane1->SetValue(m_vetaB3.atributValueDouble("zakl_dane1"));
    m_zaklDane2->SetValue(m_vetaB3.atributValueDouble("zakl_dane2"));
    m_zaklDane3->SetValue(m_vetaB3.atributValueDouble("zakl_dane3"));
}

Veta Kh1PageB3::vetaB3() const
{
    return m_vetaB3;
}

void Kh1PageB3::OnReload(wxCommandEvent &/*event*/)
{
    fillData();
}

void Kh1PageB3::OnZaklad1(wxSpinDoubleEvent &/*event*/)
{
    m_dan1->SetLabel(wxString::FromCDouble(m_zaklDane1->GetValue()*(idbmodel->zakladniSazba(m_wizard->end().GetYear())/100.0),2));
    m_vetaB3.setAtributValue("zakl_dane1", m_zaklDane1->GetValue());
    m_vetaB3.setAtributValue("dan1", m_dan1->GetLabel());
}

void Kh1PageB3::OnZaklad2(wxSpinDoubleEvent &/*event*/)
{
    m_dan2->SetLabel(wxString::FromCDouble(m_zaklDane2->GetValue()*(idbmodel->prvniSnizenaSazba(m_wizard->end().GetYear())/100.0),2));
    m_vetaB3.setAtributValue("zakl_dane2", m_zaklDane2->GetValue());
    m_vetaB3.setAtributValue("dan2", m_dan2->GetLabel());
}

void Kh1PageB3::OnZaklad3(wxSpinDoubleEvent &/*event*/)
{
    m_dan3->SetLabel(wxString::FromCDouble(m_zaklDane3->GetValue()*(idbmodel->druhaSnizenaSazba(m_wizard->end().GetYear())/100.0),2));
    m_vetaB3.setAtributValue("zakl_dane3", m_zaklDane3->GetValue());
    m_vetaB3.setAtributValue("dan3", m_dan3->GetLabel());
}
