/***************************************************************
 * Name:      dphkh1wizard.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#ifndef DPHKH1WIZARD_H
#define DPHKH1WIZARD_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/wizard.h>
#include <wx/datectrl.h>
#include <wx/dateevt.h>
#include <wx/spinctrl.h>
#include "gui/widgets/dxlistctrl.h"
#include "dphkh1.h"
#include "model/dbmodel.h"
#include "gui/widgets/dxtoolbar.h"

class Dphkh1Wizard;

class Kh1Page1 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Kh1Page1(wxWizard *parent);

private:
    wxBoxSizer *m_mainSizer;
    wxButton *m_priznani;

    void OnPriznani(wxCommandEvent &event);
};

class Kh1Page2 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Kh1Page2(wxWizard *parent);

    wxDateTime vyhotoveni() const;
    int obdobi() const; //0 - mesic, 1 - ctvrtleti, 2 - oddo
    int mesic() const;
    int ctvrtleti() const;
    int rok() const;
    wxDateTime start() const;
    wxDateTime end() const;
private:
    wxBoxSizer *m_mainSizer;
    wxRadioButton *m_mesicBtn;
    wxChoice *m_mesicSelect;
    wxChoice *m_mesicRok;
    wxRadioButton *m_ctvrtletiBtn;
    wxChoice *m_ctvrtletiSelect;
    wxChoice *m_ctvrtletiRok;
    wxRadioButton *m_oddoBtn;
    wxDatePickerCtrl *m_od;
    wxDatePickerCtrl *m_do;
    wxDatePickerCtrl *m_vyhotoveni;

    void OnMesic(wxCommandEvent& event);
    void OnCtvrtleti(wxCommandEvent& event);
    void OnOddo(wxCommandEvent& event);
    void OnOd(wxDateEvent& event);
    void OnDo(wxDateEvent& event);
};

class Kh1Page3 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Kh1Page3(wxWizard *parent);

    wxString forma() const;
    wxDateTime dzjist() const;
    wxString cJedVyzvy() const;
    wxString vyzvaOdp() const;
private:
    Dphkh1Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxRadioButton *m_radne;
    wxRadioButton *m_radneOpravne;
    wxRadioButton *m_nasledne;
    wxRadioButton *m_nasledneOpravne;
    wxDatePickerCtrl *m_dzjist;
    wxCheckBox *m_vyzva;
    wxTextCtrl *m_vyzva1;
    wxTextCtrl *m_vyzva2;
    wxTextCtrl *m_vyzva3;
    wxTextCtrl *m_vyzva4;
    wxTextCtrl *m_vyzva5;
    wxCheckBox *m_rychla;
    wxChoice *m_vyzvaOdp;

    void OnVyzva(wxCommandEvent& event);
    void OnRychla(wxCommandEvent& event);
    void OnVyzvaOdpoved(wxCommandEvent& event);
};

class Kh1PageA1 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Kh1PageA1(wxWizard *parent);

    // vety = true, naplni se m_vetyA1
    void fillList(bool vety = true);
    dxVetaArray vetyA1() const;
    size_t getVetyCount() const;
private:
    Dphkh1Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    dxListCtrl *m_list;
    dxToolBar *m_bar;
    dxVetaArray m_vetyA1;

    void fillVety();
    void OnAdd(wxCommandEvent& event);
    void OnEdit(wxCommandEvent& event);
    void OnDelete(wxCommandEvent& event);
    void OnReload(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    long selectedIndex();
};

class Kh1PageA2 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Kh1PageA2(wxWizard *parent);

    // vety = true, naplni se m_vetyA2
    void fillList(bool vety = true);
    dxVetaArray vetyA2() const;
    size_t getVetyCount() const;
private:
    Dphkh1Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxStaticText *m_mainLabel;
    dxListCtrl *m_list;
    dxToolBar *m_bar;
    dxVetaArray m_vetyA2;

    void fillVety();
    void OnAdd(wxCommandEvent& event);
    void OnEdit(wxCommandEvent& event);
    void OnDelete(wxCommandEvent& event);
    void OnReload(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    long selectedIndex();
};

class Kh1PageA3 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Kh1PageA3(wxWizard *parent);

    // vety = true, naplni se m_vetyA3
    void fillList(bool vety = true);
    dxVetaArray vetyA3();
    size_t getVetyCount() const;
private:
    Dphkh1Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    dxListCtrl *m_list;
    dxToolBar *m_bar;
    dxVetaArray m_vetyA3;

    void fillVety();
    void OnAdd(wxCommandEvent& event);
    void OnEdit(wxCommandEvent& event);
    void OnDelete(wxCommandEvent& event);
    void OnReload(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    long selectedIndex();
};

class Kh1PageA4 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Kh1PageA4(wxWizard *parent);

    // vety = true, naplni se m_vetyA4
    void fillList(bool vety = true);
    dxVetaArray vetyA4() const;
    size_t getVetyCount() const;
private:
    Dphkh1Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxStaticText *m_mainLabel;
    dxListCtrl *m_list;
    dxToolBar *m_bar;
    dxVetaArray m_vetyA4;

    void fillVety();
    void OnAdd(wxCommandEvent& event);
    void OnEdit(wxCommandEvent& event);
    void OnDelete(wxCommandEvent& event);
    void OnReload(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    long selectedIndex();
};

class Kh1PageA5 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Kh1PageA5(wxWizard *parent);

    void fillData();
    Veta vetaA5() const;
private:
    Dphkh1Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxStaticText *m_mainLabel;
    wxSpinCtrlDouble *m_zaklDane1;
    wxSpinCtrlDouble *m_zaklDane2;
    wxSpinCtrlDouble *m_zaklDane3;
    wxStaticText *m_dan1;
    wxStaticText *m_dan2;
    wxStaticText *m_dan3;
    Veta m_vetaA5;

    void OnReload(wxCommandEvent& event);
    void OnZaklad1(wxSpinDoubleEvent& event);
    void OnZaklad2(wxSpinDoubleEvent& event);
    void OnZaklad3(wxSpinDoubleEvent& event);
};

class Kh1PageB1 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Kh1PageB1(wxWizard *parent);

    // vety = true, naplni se m_vetyB1
    void fillList(bool vety = true);
    dxVetaArray vetyB1() const;
    size_t getVetyCount() const;
private:
    Dphkh1Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    dxListCtrl *m_list;
    dxToolBar *m_bar;
    dxVetaArray m_vetyB1;

    void fillVety();
    void OnAdd(wxCommandEvent& event);
    void OnEdit(wxCommandEvent& event);
    void OnDelete(wxCommandEvent& event);
    void OnReload(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    long selectedIndex();
};

class Kh1PageB2 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Kh1PageB2(wxWizard *parent);

    // vety = true, naplni se m_vetyB2
    void fillList(bool vety = true);
    dxVetaArray vetyB2() const;
    size_t getVetyCount() const;
private:
    Dphkh1Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxStaticText *m_mainLabel;
    dxListCtrl *m_list;
    dxToolBar *m_bar;
    dxVetaArray m_vetyB2;

    void fillVety();
    void OnAdd(wxCommandEvent& event);
    void OnEdit(wxCommandEvent& event);
    void OnDelete(wxCommandEvent& event);
    void OnReload(wxCommandEvent& event);
    void OnItemRightClick(wxListEvent& event);
    void OnItemActivated(wxListEvent& event);
    long selectedIndex();
};

class Kh1PageB3 : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Kh1PageB3(wxWizard *parent);

    void fillData();
    Veta vetaB3() const;
private:
    Dphkh1Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxSpinCtrlDouble *m_zaklDane1;
    wxSpinCtrlDouble *m_zaklDane2;
    wxSpinCtrlDouble *m_zaklDane3;
    wxStaticText *m_dan1;
    wxStaticText *m_dan2;
    wxStaticText *m_dan3;
    Veta m_vetaB3;

    void OnReload(wxCommandEvent& event);
    void OnZaklad1(wxSpinDoubleEvent& event);
    void OnZaklad2(wxSpinDoubleEvent& event);
    void OnZaklad3(wxSpinDoubleEvent& event);
};

class Kh1PageEP : public wxWizardPageSimple
{
    DECLARE_EVENT_TABLE()
public:
    explicit Kh1PageEP(wxWizard *parent);

private:
    Dphkh1Wizard *m_wizard;
    wxBoxSizer *m_mainSizer;
    wxButton *m_print;
    wxButton *m_export;

    void OnPrint(wxCommandEvent& event);
    void OnExport(wxCommandEvent& event);
};

class Dphkh1Wizard : public wxWizard
{
    DECLARE_DYNAMIC_CLASS(Dphkh1Wizard)
    DECLARE_EVENT_TABLE()
public:
    Dphkh1Wizard() : m_page1(nullptr), m_page2(nullptr), m_page3(nullptr), m_pageA1(nullptr), m_pageA2(nullptr),
        m_pageA3(nullptr), m_pageA4(nullptr), m_pageA5(nullptr), m_pageB1(nullptr), m_pageB2(nullptr), m_pageB3(nullptr), m_pageEP(nullptr) {}
    explicit Dphkh1Wizard(wxWindow *parent);

    wxWizardPage *GetFirstPage() const { return m_page1; }
    void fillData();
    dxAtributArray atributy() const;
    dxVetaArray vety() const;
    void saveXml(const wxString &path);
    wxDateTime start() const;
    wxDateTime end() const;
    /*
     * true = pouzit novou strukturu XML od 1.10.2019
    */
    bool newXML() const;
    void skipDataPages(bool skip);
    void print();
    void saveToXml();
private:
    Kh1Page1 *m_page1;
    Kh1Page2 *m_page2;
    Kh1Page3 *m_page3;
    Kh1PageA1 *m_pageA1;
    Kh1PageA2 *m_pageA2;
    Kh1PageA3 *m_pageA3;
    Kh1PageA4 *m_pageA4;
    Kh1PageA5 *m_pageA5;
    Kh1PageB1 *m_pageB1;
    Kh1PageB2 *m_pageB2;
    Kh1PageB3 *m_pageB3;
    Kh1PageEP *m_pageEP;
    Dphkh1 m_dphkh1;

    void OnPageChanging(wxWizardEvent& event);
    void fillPages();
    Veta fixVetaA4B2(Veta veta);
};

#endif  /* DPHKH1WIZARD_H */

