/***************************************************************
 * Name:      dphshprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/stdpaths.h>
#include <wx/filename.h>
#include "dphshprint.h"
#include "utils/dxutils.h"

DphshPrint::DphshPrint(wxWindow *parent, dxVetaArray vety)
    : DocumentPrint(parent, wxString::FromUTF8("Souhrnné hlášení VIES")), m_vety(vety)
{
    m_marginLeft = 10-PAGE_OFFSET;
    m_marginRight = 10+PAGE_OFFSET;
    m_marginTop = 10-PAGE_OFFSET;
    m_marginBottom = 10;
    createDocument();
}

void DphshPrint::Preview(bool print)
{
    DocumentPrintout *print1 = new DocumentPrintout(printTitle());
    print1->setDocument(m_document);
    print1->setHeaderLeftText(m_headerLeftText);
    print1->setHeaderCenterText(m_headerCenterText);
    print1->setHeaderRightText(m_headerRightText);
    print1->setHeaderLine(m_headerLine);
    print1->setMarginLeft(m_marginLeft);
    print1->setMarginRight(m_marginRight);
    print1->setMarginTop(m_marginTop);
    print1->setMarginBottom(m_marginBottom);
    print1->setPages(m_document->pages());
    print1->setFooter(dxPrint::F_NONE);
    print1->performPageSetup();
    wxPrintDialogData data(print1->printData());
    data.SetMinPage(1);
    data.SetMaxPage(m_document->pages());
    data.SetFromPage(1);
    data.SetToPage(m_document->pages());
    wxPrintPreview *preview = nullptr;
    if(print)
    {
        DocumentPrintout *print2 = new DocumentPrintout(printTitle());
        print2->setDocument(m_document);
        print2->setHeaderLeftText(m_headerLeftText);
        print2->setHeaderCenterText(m_headerCenterText);
        print2->setHeaderRightText(m_headerRightText);
        print2->setHeaderLine(m_headerLine);
        print2->setMarginLeft(m_marginLeft);
        print2->setMarginRight(m_marginRight);
        print2->setMarginTop(m_marginTop);
        print2->setMarginBottom(m_marginBottom);
        print2->setPages(m_document->pages());
        print2->setFooter(dxPrint::F_NONE);
        print2->performPageSetup();
        preview = new wxPrintPreview(print1, print2, &data);
    }
    else
    {
        preview = new wxPrintPreview(print1, nullptr, &data);
    }
    if (!preview->IsOk())
    {
        delete preview;
        wxMessageBox("There was a problem with previewing.\nPerhaps your current printer is not set correctly?", "Previewing", wxOK);
        return;
    }
    print1->GetPageSizeMM(&m_pageWidth, &m_pageHeight);
    preview->SetZoom(PREVIEW_ZOOM);
    wxPreviewFrame *frame = new wxPreviewFrame(preview, m_parent, m_title, wxDefaultPosition, wxSize(PREVIEW_WIDTH,PREVIEW_HEIGHT));
    frame->Centre(wxBOTH);
    frame->Initialize();
    frame->Show();
}

void DphshPrint::createDocument()
{
    bool nasledne = atributValue("shvies_forma")=="N";
    int ml = nasledne?10:24, mt = 1, mr = 10;
    m_document = new Document;
    m_document->setStyle(TextStyle(wxFont(wxFontInfo(8).FaceName(FACENAME))));
    TextElement text(1, wxString::FromUTF8("A. ODDÍL"), wxPoint(ml,mt+11), 60, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Finančnímu úřadu pro / Specializovanému finančnímu úřadu"), wxPoint(ml,mt+17), 150);
    m_document->addElement(text);
    RectangleElement rect(1, wxPoint(ml+1,mt+22), wxSize(80,6), LineStyle(), 1);
    m_document->addElement(rect);
    wxString c_ufo = atributValue("c_ufo");
    wxString nazu_ufo = "";
    wxXmlDocument ufo;
    ufo.Load(wxStandardPaths::Get().GetUserDataDir()+"/ufo.xml");
    if(ufo.IsOk() && ufo.GetRoot()->GetName() == "Ciselnik")
    {
        wxXmlNode *child = ufo.GetRoot()->GetChildren();
        while(child)
        {
            if(c_ufo == child->GetAttribute("c_ufo"))
            {
                nazu_ufo = child->GetAttribute("nazu_ufo");
                break;
            }
            child = child->GetNext();
        }
    }
    text = TextElement(1, nazu_ufo, wxPoint(ml,mt+23), 80, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Územní pracoviště v, ve, pro"), wxPoint(ml,mt+28), 150);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+33), wxSize(80,6), LineStyle(), 1);
    m_document->addElement(rect);
    wxString c_pracufo = atributValue("c_pracufo");
    wxString nazu_pracufo = "";
    wxXmlDocument pracufo;
    pracufo.Load(wxStandardPaths::Get().GetUserDataDir()+"/pracufo.xml");
    if(pracufo.IsOk() && pracufo.GetRoot()->GetName() == "Ciselnik")
    {
        wxXmlNode *child = pracufo.GetRoot()->GetChildren();
        while(child)
        {
            if(c_pracufo == child->GetAttribute("c_pracufo"))
            {
                nazu_pracufo = child->GetAttribute("nazu_pracufo");
                break;
            }
            child = child->GetNext();
        }
    }
    text = TextElement(1, nazu_pracufo, wxPoint(ml,mt+34), 80, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Daňové identifikační číslo"), wxPoint(ml,mt+39), 150);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+44), wxSize(60,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, "CZ"+atributValue("dic"), wxPoint(ml,mt+45), 60, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Pořadové číslo strany / Celkový počet stran"), wxPoint(0,mt+14), m_pageWidth-mr-22, wxTEXT_ALIGNMENT_RIGHT);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(m_pageWidth-mr-22,mt+13), wxSize(22,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, m_vety.GetCount()>22?"1/2":"1/1", wxPoint(m_pageWidth-mr-22,mt+14), 22, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(m_pageWidth-mr-80,mt+22), wxSize(80,35), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, wxString::FromUTF8("otisk podacího razítka finančního úřadu"), wxPoint(m_pageWidth-mr-80,mt+52), 80, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(7).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("SOUHRNNÉ HLÁŠENÍ"), wxPoint(ml,mt+58), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(22).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("k dani z přidané hodnoty"), wxPoint(ml,mt+67), m_pageWidth-ml-mr, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(14).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("podle § 102 zákona č. 235/2004 Sb., o dani z přidané hodnoty"), wxPoint(ml,mt+74), m_pageWidth-ml-mr,
                       wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+48,mt+80), wxSize(20,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("mesic"), wxPoint(ml+48,mt+81), 20, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+112,mt+80), wxSize(20,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("ctvrt"), wxPoint(ml+112,mt+81), 20, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+146,mt+80), wxSize(20,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("rok"), wxPoint(ml+146,mt+81), 20, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, "rok", wxPoint(ml+132,mt+81), 14, wxTEXT_ALIGNMENT_RIGHT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("za kalendářní čtvrtletí"), wxPoint(ml+68,mt+81), 44, wxTEXT_ALIGNMENT_RIGHT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("za kalendářní měsíc"), wxPoint(ml+4,mt+81), 44, wxTEXT_ALIGNMENT_RIGHT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Právnická osoba:"), wxPoint(ml,mt+85), 45, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(8).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Obchodní jméno"), wxPoint(ml,mt+89), 45);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+93), wxSize(m_pageWidth-ml-1-mr,6), LineStyle(), 1);
    m_document->addElement(rect);
    rect = RectangleElement(1, wxPoint(ml+1,mt+100), wxSize(115,6), LineStyle(), 1);
    m_document->addElement(rect);
    rect = RectangleElement(1, wxPoint(ml+121,mt+100), wxSize(m_pageWidth-ml-121-mr,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, wxString::FromUTF8("Fyzická osoba:"), wxPoint(ml,mt+107), 45, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(8).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Příjmení"), wxPoint(ml,mt+111), 45);
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Jméno"), wxPoint(ml+100,mt+111), 45);
    m_document->addElement(text);
    text = TextElement(1, "Titul", wxPoint(ml+155,mt+111), 45);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+115), wxSize(95,6), LineStyle(), 1);
    m_document->addElement(rect);
    rect = RectangleElement(1, wxPoint(ml+101,mt+115), wxSize(50,6), LineStyle(), 1);
    m_document->addElement(rect);
    rect = RectangleElement(1, wxPoint(ml+156,mt+115), wxSize(m_pageWidth-ml-156-mr,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("prijmeni"), wxPoint(ml,mt+116), 95, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, atributValue("jmeno"), wxPoint(ml+100,mt+116), 50, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, atributValue("titul"), wxPoint(ml+155,mt+116), 20, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Trvalé bydliště fyzické osoby nebo sídlo právnické osoby:"), wxPoint(ml,mt+122), 145,
                       wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(8).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    text = TextElement(1, "Obec", wxPoint(ml,mt+126), 45);
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("PSČ"), wxPoint(ml+150,mt+126), 45);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+130), wxSize(145,6), LineStyle(), 1);
    m_document->addElement(rect);
    rect = RectangleElement(1, wxPoint(ml+151,mt+130), wxSize(m_pageWidth-ml-151-mr,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("naz_obce"), wxPoint(ml,mt+131), 145, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, atributValue("psc"), wxPoint(ml+150,mt+131), 25, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Ulice, část obce (číslo popisné / číslo orientační)"), wxPoint(ml,mt+136), 145);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+140), wxSize(m_pageWidth-ml-1-mr,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("ulice")+" "+atributValue("c_pop")+(atributValue("c_orient").IsEmpty()?"":"/"+atributValue("c_orient")), wxPoint(ml,mt+141),
                       m_pageWidth-ml-1-mr, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Stát"), wxPoint(ml,mt+146), 145);
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml+1,mt+150), wxSize(75,6), LineStyle(), 1);
    m_document->addElement(rect);
    text = TextElement(1, atributValue("stat"), wxPoint(ml,mt+151), 75, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
    m_document->addElement(text);
    LineElement line(1, wxPoint(ml,158), wxPoint(m_pageWidth-mr,158), LineStyle(wxColour(*wxBLACK), 2));
    m_document->addElement(line);
    text = TextElement(1, wxString::FromUTF8("B. ODDÍL"), wxPoint(ml,mt+157), 60, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    rect = RectangleElement(1, wxPoint(ml,mt+162), wxSize((nasledne?5:4)*15+60+55,7+21*5), LineStyle(), 1);
    m_document->addElement(rect);
    line = LineElement(1, wxPoint(ml,mt+169), wxPoint(ml+(nasledne?5:4)*15+60+55,mt+169));
    m_document->addElement(line);
    for(int i=1; i<21; i++)
    {
        line = LineElement(1, wxPoint(ml,mt+169+i*5), wxPoint(ml+(nasledne?5:4)*15+60+55,mt+169+i*5));
        m_document->addElement(line);
        text = TextElement(1, wxString::Format("%d", i), wxPoint(ml,mt+165+i*5), 15, wxTEXT_ALIGNMENT_CENTER);
        m_document->addElement(text);
    }
    line = LineElement(1, wxPoint(ml+15,mt+162), wxPoint(ml+15,mt+269));
    m_document->addElement(line);
    line = LineElement(1, wxPoint(ml+30,mt+162), wxPoint(ml+30,mt+269));
    m_document->addElement(line);
    if(nasledne)
    {
        line = LineElement(1, wxPoint(ml+45,mt+162), wxPoint(ml+45,mt+269));
        m_document->addElement(line);
    }
    line = LineElement(1, wxPoint(ml+90+nasledne*15,mt+162), wxPoint(ml+90+nasledne*15,mt+269));
    m_document->addElement(line);
    line = LineElement(1, wxPoint(ml+105+nasledne*15,mt+162), wxPoint(ml+105+nasledne*15,mt+269));
    m_document->addElement(line);
    line = LineElement(1, wxPoint(ml+120+nasledne*15,mt+162), wxPoint(ml+120+nasledne*15,mt+274));
    m_document->addElement(line);
    text = TextElement(1, wxString::FromUTF8("Číslo\nřádku"), wxPoint(ml,mt+162), 15, wxTEXT_ALIGNMENT_CENTER);
    m_document->addElement(text);
    if(nasledne)
    {
        text = TextElement(1, wxString::FromUTF8("Storno\nřádek"), wxPoint(ml+15,mt+162), 15, wxTEXT_ALIGNMENT_CENTER);
        m_document->addElement(text);
    }
    text = TextElement(1, wxString::FromUTF8("Kód\nzemě"), wxPoint(ml+15+nasledne*15,mt+162), 15, wxTEXT_ALIGNMENT_CENTER);
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("DIČ pořizovatele zboží"), wxPoint(ml+30+nasledne*15,mt+164), 60, wxTEXT_ALIGNMENT_CENTER);
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Kód\nplnění"), wxPoint(ml+90+nasledne*15,mt+162), 15, wxTEXT_ALIGNMENT_CENTER);
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Počet\nplnění"), wxPoint(ml+105+nasledne*15,mt+162), 15, wxTEXT_ALIGNMENT_CENTER);
    m_document->addElement(text);
    text = TextElement(1, wxString::FromUTF8("Celková hodnota plnění v Kč"), wxPoint(ml+120+nasledne*15,mt+164), 55, wxTEXT_ALIGNMENT_CENTER);
    m_document->addElement(text);
    long soucet = 0;
    for(size_t i=0; i<wxMin(22,m_vety.GetCount()); i++)
    {
        if(m_vety[i].name() == "VetaR")
        {
            Veta v = m_vety[i];
            int rad = v.atributValueInt("c_rad")-1;
            if(nasledne)
            {
                text = TextElement(1, v.atributValue("k_storno").IsEmpty()?"":"X", wxPoint(ml+15,mt+170+rad*5), 15, wxTEXT_ALIGNMENT_CENTER);
                m_document->addElement(text);
            }
            text = TextElement(1, v.atributValue("k_stat"), wxPoint(ml+15+nasledne*15,mt+170+rad*5), 15, wxTEXT_ALIGNMENT_CENTER);
            m_document->addElement(text);
            text = TextElement(1, v.atributValue("c_vat"), wxPoint(ml+30+nasledne*15,mt+170+rad*5), 60);
            m_document->addElement(text);
            text = TextElement(1, v.atributValue("k_pln_eu"), wxPoint(ml+90+nasledne*15,mt+170+rad*5), 15, wxTEXT_ALIGNMENT_CENTER);
            m_document->addElement(text);
            text = TextElement(1, v.atributValue("pln_pocet"), wxPoint(ml+105+nasledne*15,mt+170+rad*5), 15, wxTEXT_ALIGNMENT_CENTER);
            m_document->addElement(text);
            text = TextElement(1, v.atributValue("pln_hodnota"), wxPoint(ml+120+nasledne*15,mt+170+rad*5), 55, wxTEXT_ALIGNMENT_RIGHT);
            m_document->addElement(text);
            soucet += v.atributValueInt("pln_hodnota");
        }
    }
    text = TextElement(1, wxString::FromUTF8("Kontrolní součet řádků 1–20:"), wxPoint(ml,mt+270), 90);
    m_document->addElement(text);
    text = TextElement(1, wxString::Format("%ld", soucet), wxPoint(ml+120+nasledne*15,mt+270), 55, wxTEXT_ALIGNMENT_RIGHT);
    m_document->addElement(text);
    if(m_vety.GetCount()>22)
    {
        ml = 10;
        text = TextElement(2, wxString::FromUTF8("Pořadové číslo strany / Celkový počet stran"), wxPoint(0,mt+14), m_pageWidth-mr-22, wxTEXT_ALIGNMENT_RIGHT);
        m_document->addElement(text);
        rect = RectangleElement(2, wxPoint(m_pageWidth-mr-22,mt+13), wxSize(22,6), LineStyle(), 1);
        m_document->addElement(rect);
        text = TextElement(2, "2/2", wxPoint(m_pageWidth-mr-22,mt+14), 22, wxTEXT_ALIGNMENT_CENTER, TextStyle(wxFont(wxFontInfo(10).FaceName(FACENAME))));
        m_document->addElement(text);
        rect = RectangleElement(2, wxPoint(ml,mt+22), wxSize((nasledne?5:4)*15+60+55,7+21*5), LineStyle(), 1);
        m_document->addElement(rect);
        line = LineElement(2, wxPoint(ml,mt+29), wxPoint(ml+(nasledne?5:4)*15+60+55,mt+29));
        m_document->addElement(line);
        for(int i=1; i<21; i++)
        {
            line = LineElement(2, wxPoint(ml,mt+29+i*5), wxPoint(ml+(nasledne?5:4)*15+60+55,mt+29+i*5));
            m_document->addElement(line);
            text = TextElement(2, wxString::Format("%d", i), wxPoint(ml,mt+25+i*5), 15, wxTEXT_ALIGNMENT_CENTER);
            m_document->addElement(text);
        }
        line = LineElement(2, wxPoint(ml+15,mt+22), wxPoint(ml+15,mt+129));
        m_document->addElement(line);
        line = LineElement(2, wxPoint(ml+30,mt+22), wxPoint(ml+30,mt+129));
        m_document->addElement(line);
        if(nasledne)
        {
            line = LineElement(2, wxPoint(ml+45,mt+22), wxPoint(ml+45,mt+129));
            m_document->addElement(line);
        }
        line = LineElement(2, wxPoint(ml+90+nasledne*15,mt+22), wxPoint(ml+90+nasledne*15,mt+129));
        m_document->addElement(line);
        line = LineElement(2, wxPoint(ml+105+nasledne*15,mt+22), wxPoint(ml+105+nasledne*15,mt+129));
        m_document->addElement(line);
        line = LineElement(2, wxPoint(ml+120+nasledne*15,mt+22), wxPoint(ml+120+nasledne*15,mt+134));
        m_document->addElement(line);
        text = TextElement(2, wxString::FromUTF8("Číslo\nřádku"), wxPoint(ml,mt+22), 15, wxTEXT_ALIGNMENT_CENTER);
        m_document->addElement(text);
        if(nasledne)
        {
            text = TextElement(2, wxString::FromUTF8("Storno\nřádek"), wxPoint(ml+15,mt+22), 15, wxTEXT_ALIGNMENT_CENTER);
            m_document->addElement(text);
        }
        text = TextElement(2, wxString::FromUTF8("Kód\nzemě"), wxPoint(ml+15+nasledne*15,mt+22), 15, wxTEXT_ALIGNMENT_CENTER);
        m_document->addElement(text);
        text = TextElement(2, wxString::FromUTF8("DIČ pořizovatele zboží"), wxPoint(ml+30+nasledne*15,mt+24), 60, wxTEXT_ALIGNMENT_CENTER);
        m_document->addElement(text);
        text = TextElement(2, wxString::FromUTF8("Kód\nplnění"), wxPoint(ml+90+nasledne*15,mt+22), 15, wxTEXT_ALIGNMENT_CENTER);
        m_document->addElement(text);
        text = TextElement(2, wxString::FromUTF8("Počet\nplnění"), wxPoint(ml+105+nasledne*15,mt+22), 15, wxTEXT_ALIGNMENT_CENTER);
        m_document->addElement(text);
        text = TextElement(2, wxString::FromUTF8("Celková hodnota plnění v Kč"), wxPoint(ml+120+nasledne*15,mt+24), 55, wxTEXT_ALIGNMENT_CENTER);
        m_document->addElement(text);
        long soucet = 0;
        for(size_t i=22; i<wxMin(42,m_vety.GetCount()); i++)
        {
            if(m_vety[i].name() == "VetaR")
            {
                Veta v = m_vety[i];
                int rad = v.atributValueInt("c_rad")-21;
                if(nasledne)
                {
                    text = TextElement(2, v.atributValue("k_storno").IsEmpty()?"":"X", wxPoint(ml+15,mt+30+rad*5), 15, wxTEXT_ALIGNMENT_CENTER);
                    m_document->addElement(text);
                }
                text = TextElement(2, v.atributValue("k_stat"), wxPoint(ml+15+nasledne*15,mt+30+rad*5), 15, wxTEXT_ALIGNMENT_CENTER);
                m_document->addElement(text);
                text = TextElement(2, v.atributValue("c_vat"), wxPoint(ml+30+nasledne*15,mt+30+rad*5), 60);
                m_document->addElement(text);
                text = TextElement(2, v.atributValue("k_pln_eu"), wxPoint(ml+90+nasledne*15,mt+30+rad*5), 15, wxTEXT_ALIGNMENT_CENTER);
                m_document->addElement(text);
                text = TextElement(2, v.atributValue("pln_pocet"), wxPoint(ml+105+nasledne*15,mt+30+rad*5), 15, wxTEXT_ALIGNMENT_CENTER);
                m_document->addElement(text);
                text = TextElement(2, v.atributValue("pln_hodnota"), wxPoint(ml+120+nasledne*15,mt+30+rad*5), 55, wxTEXT_ALIGNMENT_RIGHT);
                m_document->addElement(text);
                soucet += v.atributValueInt("pln_hodnota");
            }
        }
        text = TextElement(2, wxString::FromUTF8("Kontrolní součet řádků 1–20:"), wxPoint(ml,mt+130), 90);
        m_document->addElement(text);
        text = TextElement(2, wxString::Format("%ld", soucet), wxPoint(ml+120+nasledne*15,mt+130), 55, wxTEXT_ALIGNMENT_RIGHT);
        m_document->addElement(text);
        line = LineElement(2, wxPoint(ml,mt+135), wxPoint(m_pageWidth-mr,mt+135), LineStyle(wxColour(*wxBLACK), 2));
        m_document->addElement(line);
    }
    m_document->setPages(m_vety.GetCount()>22?2:1);
}

wxString DphshPrint::atributValue(const wxString &name) const
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray atr = m_vety[i].atributy();
        for(size_t j=0; j<atr.GetCount(); j++)
        {
            if(atr[j].name() == name)
            {
                return atr[j].value();
            }
        }
    }
    return wxEmptyString;
}
