/***************************************************************
 * Name:      Dphshv.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/wfstream.h>
#include <wx/sstream.h>
#include "dphshv.h"
#include "utils/dxutils.h"
#include "config.h"

Dphshv::Dphshv()
{
    Veta vetad(0, "VetaD", 1, 1);
    dxAtributArray avetad;
    Atribut ctvrt(A_INT, "ctvrt", false, 1);
    avetad.Add(ctvrt);
    Atribut d_poddp(A_DATUM, "d_poddp", true, 10, 0, "%d.%m.%Y");
    avetad.Add(d_poddp);
    Atribut dokument(A_STRING, "dokument", true, 3);
    avetad.Add(dokument);
    Atribut k_uladis(A_STRING, "k_uladis", true, 3);
    avetad.Add(k_uladis);
    Atribut mesic(A_INT, "mesic", false, 2);
    avetad.Add(mesic);
    Atribut rok(A_INT, "rok", true, 4);
    avetad.Add(rok);
    Atribut shvies_forma(A_STRING, "shvies_forma", true, 1);
    avetad.Add(shvies_forma);
    vetad.setAtributy(avetad);
    m_vety.Add(vetad);

    Veta vetap(1, "VetaP", 1, 1);
    dxAtributArray avetap;
    Atribut c_orient(A_STRING, "c_orient", false, 4);
    avetap.Add(c_orient);
    Atribut c_pop(A_INT, "c_pop", false, 6);
    avetap.Add(c_pop);
    Atribut c_pracufo(A_INT, "c_pracufo", false, 4);
    avetap.Add(c_pracufo);
    Atribut c_telef(A_STRING, "c_telef", false, 14);
    avetap.Add(c_telef);
    Atribut c_ufo(A_INT, "c_ufo", true, 3);
    avetap.Add(c_ufo);
    Atribut dic(A_STRING, "dic", true, 10);
    avetap.Add(dic);
    Atribut dodobchjm(A_STRING, "dodobchjm", false, 11);
    avetap.Add(dodobchjm);
    Atribut jmeno(A_STRING, "jmeno", false, 20);
    avetap.Add(jmeno);
    Atribut naz_obce(A_STRING, "naz_obce", false, 48);
    avetap.Add(naz_obce);
    Atribut opr_jmeno(A_STRING, "opr_jmeno", false, 20);
    avetap.Add(opr_jmeno);
    Atribut opr_postaveni(A_STRING, "opr_postaveni", false, 40);
    avetap.Add(opr_postaveni);
    Atribut opr_prijmeni(A_STRING, "opr_prijmeni", false, 36);
    avetap.Add(opr_prijmeni);
    Atribut prijmeni(A_STRING, "prijmeni", false, 36);
    avetap.Add(prijmeni);
    Atribut psc(A_STRING, "psc", false, 5);
    avetap.Add(psc);
    Atribut sest_jmeno(A_STRING, "sest_jmeno", false, 20);
    avetap.Add(sest_jmeno);
    Atribut sest_prijmeni(A_STRING, "sest_prijmeni", false, 36);
    avetap.Add(sest_prijmeni);
    Atribut sest_telef(A_STRING, "sest_telef", false, 14);
    avetap.Add(sest_telef);
    Atribut stat(A_STRING, "stat", false, 25);
    avetap.Add(stat);
    Atribut titul(A_STRING, "titul", false, 10);
    avetap.Add(titul);
    Atribut typ_ds(A_STRING, "typ_ds", true, 1);
    avetap.Add(typ_ds);
    Atribut ulice(A_STRING, "ulice", false, 38);
    avetap.Add(ulice);
    Atribut zast_dat_nar(A_DATUM, "zast_dat_nar", false, 10, 0, "%d.%m.%Y");
    avetap.Add(zast_dat_nar);
    Atribut zast_ev_cislo(A_STRING, "zast_ev_cislo", false, 36);
    avetap.Add(zast_ev_cislo);
    Atribut zast_ic(A_STRING, "zast_ic", false, 10);
    avetap.Add(zast_ic);
    Atribut zast_jmeno(A_STRING, "zast_jmeno", false, 20);
    avetap.Add(zast_jmeno);
    Atribut zast_kod(A_STRING, "zast_kod", false, 2);
    avetap.Add(zast_kod);
    Atribut zast_nazev(A_STRING, "zast_nazev", false, 255);
    avetap.Add(zast_nazev);
    Atribut zast_prijmeni(A_STRING, "zast_prijmeni", false, 36);
    avetap.Add(zast_prijmeni);
    Atribut zast_typ(A_STRING, "zast_typ", false, 1);
    avetap.Add(zast_typ);
    Atribut zkrobchjm(A_STRING, "zkrobchjm", false, 255);
    avetap.Add(zkrobchjm);
    vetap.setAtributy(avetap);
    m_vety.Add(vetap);
}

void Dphshv::setAtributValue(int vetaId, const wxString &name, const wxString &value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(vetaId == m_vety[i].id())
        {
            m_vety[i].setAtributValue(name, value);
        }
    }
}

void Dphshv::setAtributValue(int vetaId, const wxString &name, int value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(vetaId == m_vety[i].id())
        {
            m_vety[i].setAtributValue(name, value);
        }
    }
}

void Dphshv::setAtributValue(int vetaId, const wxString &name, double value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(vetaId == m_vety[i].id())
        {
            m_vety[i].setAtributValue(name, value);
        }
    }
}

void Dphshv::setAtributValue(int vetaId, const wxString &name, const wxDateTime &value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(vetaId == m_vety[i].id())
        {
            m_vety[i].setAtributValue(name, value);
        }
    }
}

void Dphshv::setAtributValue(const wxString &name, const wxString &value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        m_vety[i].setAtributValue(name, value);
    }
}

void Dphshv::setAtributValue(const wxString &name, int value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        m_vety[i].setAtributValue(name, value);
    }
}

void Dphshv::setAtributValue(const wxString &name, double value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        m_vety[i].setAtributValue(name, value);
    }
}

void Dphshv::setAtributValue(const wxString &name, const wxDateTime &value)
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        m_vety[i].setAtributValue(name, value);
    }
}

wxString Dphshv::atributValue(const wxString &name) const
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray atr = m_vety[i].atributy();
        for(size_t j=0; j<atr.GetCount(); j++)
        {
            if(atr[j].name() == name)
            {
                return atr[j].value();
            }
        }
    }
    return wxEmptyString;
}

int Dphshv::atributValueInt(const wxString &name) const
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray atr = m_vety[i].atributy();
        for(size_t j=0; j<atr.GetCount(); j++)
        {
            if(atr[j].name() == name)
            {
                return atr[j].valueInt();
            }
        }
    }
    return 0;
}

double Dphshv::atributValueDouble(const wxString &name) const
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray atr = m_vety[i].atributy();
        for(size_t j=0; j<atr.GetCount(); j++)
        {
            if(atr[j].name() == name)
            {
                return atr[j].valueDouble();
            }
        }
    }
    return 0.0;
}

wxDateTime Dphshv::atributValueDatum(const wxString &name) const
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray atr = m_vety[i].atributy();
        for(size_t j=0; j<atr.GetCount(); j++)
        {
            if(atr[j].name() == name)
            {
                return atr[j].valueDatum();
            }
        }
    }
    return wxDateTime();
}

bool Dphshv::saveToXml(const wxString &path)
{
    wxFileOutputStream fout(path);
    if(fout.IsOk())
    {
        return createXml().Save(fout);
    }
    else
    {
        return false;
    }
}

wxString Dphshv::saveToString()
{
    wxStringOutputStream fout;
    if(fout.IsOk())
    {
        createXml().Save(fout);
        return fout.GetString();
    }
    else
    {
        return wxEmptyString;
    }
}

bool Dphshv::readFromXml(const wxString &path)
{
    clearData();
    if(!wxFile::Exists(path)) return false;
    wxFileInputStream fin(path);
    if(fin.IsOk())
    {
        wxXmlDocument xml;
        if(!xml.Load(fin)) return false;
        if(xml.GetRoot()->GetName() != "Pisemnost") return false;
        if(!xml.GetRoot()->GetChildren()) return false;
        if(xml.GetRoot()->GetChildren()->GetName() != "DPHSHV") return false;
        wxXmlNode *dph = xml.GetRoot()->GetChildren();
        wxXmlNode *veta = dph->GetChildren();
        int id = 2;
        while(veta)
        {
            if(veta->GetName() == "VetaR")
            {
                Veta v(id, "VetaR", 1, 1);
                dxAtributArray avetar;
                Atribut c_rad(A_INT, "c_rad", false, 2);
                avetar.Add(c_rad);
                Atribut c_vat(A_STRING, "c_vat", false, 12);
                avetar.Add(c_vat);
                Atribut k_pln_eu(A_INT, "k_pln_eu", false, 1);
                avetar.Add(k_pln_eu);
                Atribut k_stat(A_STRING, "k_stat", false, 2);
                avetar.Add(k_stat);
                Atribut k_storno(A_STRING, "k_storno", false, 1);
                avetar.Add(k_storno);
                Atribut pln_hodnota(A_INT, "pln_hodnota", false, 14);
                avetar.Add(pln_hodnota);
                Atribut pln_pocet(A_INT, "pln_pocet", false, 6);
                avetar.Add(pln_pocet);
                v.setAtributy(avetar);
                wxXmlAttribute *atribut = veta->GetAttributes();
                while(atribut)
                {
                    v.setAtributValue(atribut->GetName(), atribut->GetValue());
                    atribut = atribut->GetNext();
                }
                m_vety.Add(v);
                id++;
            }
            else
            {
                wxXmlAttribute *atribut = veta->GetAttributes();
                while(atribut)
                {
                    setAtributValue(atribut->GetName(), atribut->GetValue());
                    atribut = atribut->GetNext();
                }
            }
            veta = veta->GetNext();
        }
        return true;
    }
    else
    {
        return false;
    }
}

bool Dphshv::readFromString(const wxString &data)
{
    clearData();
    if(data.IsEmpty()) return false;
    wxStringInputStream fin(data);
    if(fin.IsOk())
    {
        wxXmlDocument xml;
        if(!xml.Load(fin)) return false;
        if(xml.GetRoot()->GetName() != "Pisemnost") return false;
        if(!xml.GetRoot()->GetChildren()) return false;
        if(xml.GetRoot()->GetChildren()->GetName() != "DPHSHV") return false;
        wxXmlNode *dph = xml.GetRoot()->GetChildren();
        wxXmlNode *veta = dph->GetChildren();
        int id = 2;
        while(veta)
        {
            if(veta->GetName() == "VetaR")
            {
                Veta v(id, "VetaR", 1, 1);
                dxAtributArray avetar;
                Atribut c_rad(A_INT, "c_rad", false, 2);
                avetar.Add(c_rad);
                Atribut c_vat(A_STRING, "c_vat", false, 12);
                avetar.Add(c_vat);
                Atribut k_pln_eu(A_INT, "k_pln_eu", false, 1);
                avetar.Add(k_pln_eu);
                Atribut k_stat(A_STRING, "k_stat", false, 2);
                avetar.Add(k_stat);
                Atribut k_storno(A_STRING, "k_storno", false, 1);
                avetar.Add(k_storno);
                Atribut pln_hodnota(A_INT, "pln_hodnota", false, 14);
                avetar.Add(pln_hodnota);
                Atribut pln_pocet(A_INT, "pln_pocet", false, 6);
                avetar.Add(pln_pocet);
                v.setAtributy(avetar);
                wxXmlAttribute *atribut = veta->GetAttributes();
                while(atribut)
                {
                    v.setAtributValue(atribut->GetName(), atribut->GetValue());
                    atribut = atribut->GetNext();
                }
                m_vety.Add(v);
                id++;
            }
            else
            {
                wxXmlAttribute *atribut = veta->GetAttributes();
                while(atribut)
                {
                    setAtributValue(atribut->GetName(), atribut->GetValue());
                    atribut = atribut->GetNext();
                }
            }
            veta = veta->GetNext();
        }
        return true;
    }
    else
    {
        return false;
    }
}

void Dphshv::clearData()
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        m_vety[i].clearData();
    }
}

dxAtributArray Dphshv::atributy() const
{
    dxAtributArray atr;
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray a = m_vety[i].atributy();
        for(size_t j=0; j<a.GetCount(); j++)
        {
            atr.Add(a[j]);
        }
    }
    return atr;
}

bool Dphshv::isEmpty()
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray a = m_vety[i].atributy();
        for(size_t j=0; j<a.GetCount(); j++)
        {
            if(a[j].isForXml()) return false;
        }
    }
    return true;
}

ATRIBUTTYPE Dphshv::atributType(const wxString &name) const
{
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        dxAtributArray atr = m_vety[i].atributy();
        for(size_t j=0; j<atr.GetCount(); j++)
        {
            if(atr[j].name() == name)
            {
                return atr[j].type();
            }
        }
    }
    return A_NONE;
}

void Dphshv::addVetaR(Veta veta)
{
    m_vety.Add(veta);
}

dxVetaArray Dphshv::vety() const
{
    return m_vety;
}

void Dphshv::setVety(const dxVetaArray &vety)
{
    m_vety = vety;
}

wxXmlDocument Dphshv::createXml()
{
    wxXmlDocument xml;
    wxXmlNode *rootNode = new wxXmlNode(wxXML_ELEMENT_NODE, "Pisemnost");
    rootNode->AddAttribute("nazevSW", PACKAGE);
    rootNode->AddAttribute("verzeSW", VERSION);
    wxXmlNode *dphNode = new wxXmlNode(wxXML_ELEMENT_NODE, "DPHSHV");
    for(size_t i=0; i<m_vety.GetCount(); i++)
    {
        if(m_vety[i].isForXml())
        {
            dphNode->AddChild(m_vety[i].createXmlNode());
        }
    }
    rootNode->AddChild(dphNode);
    xml.SetRoot(rootNode);
    return xml;
}
