/***************************************************************
 * Name:      majetekzavazkydialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "majetekzavazkydialog.h"
#include "majetekzavazkyprint.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(MajetekZavazkyDialog, wxDialog)

BEGIN_EVENT_TABLE(MajetekZavazkyDialog, wxDialog)
    EVT_BUTTON(ID_MAJETEKZAVAZKY_PRINT, MajetekZavazkyDialog::OnPrint)
    EVT_BUTTON(ID_MAJETEKZAVAZKY_RELOAD, MajetekZavazkyDialog::OnReload)
    EVT_CHOICE(ID_MAJETEKZAVAZKY_ROK, MajetekZavazkyDialog::OnRok)
    EVT_BUTTON(wxID_CLOSE, MajetekZavazkyDialog::OnClose)
END_EVENT_TABLE()

MajetekZavazkyDialog::MajetekZavazkyDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Přehled majetku a závázků"))
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    fillData();
}

void MajetekZavazkyDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Rok"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_rok = new wxChoice(this, ID_MAJETEKZAVAZKY_ROK);
    m_rok->Append(idbmodel->getObdobiAsStrings());
    m_rok->SetStringSelection(wxString::Format("%d", idbmodel->currentRok()));
    gridSizer->Add(m_rok, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxGridBagSizer *valueSizer = new wxGridBagSizer(5, 5);

    valueSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Hmotný majetek")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kc_dpfmz02 = new wxSpinCtrlDouble(this);
    m_kc_dpfmz02->SetDigits(0);
    m_kc_dpfmz02->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_dpfmz02, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    m_kc_z_dpfmz02 = new wxSpinCtrlDouble(this);
    m_kc_z_dpfmz02->SetDigits(0);
    m_kc_z_dpfmz02->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_z_dpfmz02, wxGBPosition(0,2), wxDefaultSpan, wxEXPAND);

    valueSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Peněžní prostředky v hotovosti")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kc_dpfmz05a = new wxSpinCtrlDouble(this);
    m_kc_dpfmz05a->SetDigits(0);
    m_kc_dpfmz05a->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_dpfmz05a, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    m_kc_z_dpfmz05a = new wxSpinCtrlDouble(this);
    m_kc_z_dpfmz05a->SetDigits(0);
    m_kc_z_dpfmz05a->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_z_dpfmz05a, wxGBPosition(1,2), wxDefaultSpan, wxEXPAND);

    valueSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Peněžní prostředky na bankovních účtech")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kc_dpfmz06 = new wxSpinCtrlDouble(this);
    m_kc_dpfmz06->SetDigits(0);
    m_kc_dpfmz06->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_dpfmz06, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    m_kc_z_dpfmz06 = new wxSpinCtrlDouble(this);
    m_kc_z_dpfmz06->SetDigits(0);
    m_kc_z_dpfmz06->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_z_dpfmz06, wxGBPosition(2,2), wxDefaultSpan, wxEXPAND);

    valueSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Zásoby")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kc_dpfmz03 = new wxSpinCtrlDouble(this);
    m_kc_dpfmz03->SetDigits(0);
    m_kc_dpfmz03->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_dpfmz03, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);
    m_kc_z_dpfmz03 = new wxSpinCtrlDouble(this);
    m_kc_z_dpfmz03->SetDigits(0);
    m_kc_z_dpfmz03->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_z_dpfmz03, wxGBPosition(3,2), wxDefaultSpan, wxEXPAND);

    valueSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pohledávky včetně poskytnutých úvěrů a zápůjček")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kc_dpfmz04 = new wxSpinCtrlDouble(this);
    m_kc_dpfmz04->SetDigits(0);
    m_kc_dpfmz04->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_dpfmz04, wxGBPosition(4,1), wxDefaultSpan, wxEXPAND);
    m_kc_z_dpfmz04 = new wxSpinCtrlDouble(this);
    m_kc_z_dpfmz04->SetDigits(0);
    m_kc_z_dpfmz04->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_z_dpfmz04, wxGBPosition(4,2), wxDefaultSpan, wxEXPAND);

    valueSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Ostatní majetek")), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kc_dpfmz08 = new wxSpinCtrlDouble(this);
    m_kc_dpfmz08->SetDigits(0);
    m_kc_dpfmz08->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_dpfmz08, wxGBPosition(5,1), wxDefaultSpan, wxEXPAND);
    m_kc_z_dpfmz08 = new wxSpinCtrlDouble(this);
    m_kc_z_dpfmz08->SetDigits(0);
    m_kc_z_dpfmz08->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_z_dpfmz08, wxGBPosition(5,2), wxDefaultSpan, wxEXPAND);

    valueSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Dluhy včetně přijatých úvěrů a zápůjček")), wxGBPosition(6,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kc_dpfmz10 = new wxSpinCtrlDouble(this);
    m_kc_dpfmz10->SetDigits(0);
    m_kc_dpfmz10->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_dpfmz10, wxGBPosition(6,1), wxDefaultSpan, wxEXPAND);
    m_kc_z_dpfmz10 = new wxSpinCtrlDouble(this);
    m_kc_z_dpfmz10->SetDigits(0);
    m_kc_z_dpfmz10->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_z_dpfmz10, wxGBPosition(6,2), wxDefaultSpan, wxEXPAND);

    valueSizer->Add(new wxStaticText(this, wxID_ANY, "Rezervy"), wxGBPosition(7,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kc_dpfmz11 = new wxSpinCtrlDouble(this);
    m_kc_dpfmz11->SetDigits(0);
    m_kc_dpfmz11->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_dpfmz11, wxGBPosition(7,1), wxDefaultSpan, wxEXPAND);
    m_kc_z_dpfmz11 = new wxSpinCtrlDouble(this);
    m_kc_z_dpfmz11->SetDigits(0);
    m_kc_z_dpfmz11->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_z_dpfmz11, wxGBPosition(7,2), wxDefaultSpan, wxEXPAND);

    valueSizer->Add(new wxStaticText(this, wxID_ANY, "Mzdy"), wxGBPosition(8,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_kc_dpfmz18 = new wxSpinCtrlDouble(this);
    m_kc_dpfmz18->SetDigits(0);
    m_kc_dpfmz18->SetRange(0.0, 99999999999999.0);
    valueSizer->Add(m_kc_dpfmz18, wxGBPosition(8,1), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(valueSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    m_mainSizer->Add(new wxButton(this, ID_MAJETEKZAVAZKY_RELOAD, wxString::FromUTF8("Znovu načíst data z evidence")), 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    m_mainSizer->Add(new wxButton(this, ID_MAJETEKZAVAZKY_PRINT, wxString::FromUTF8("Vytisknout přehled")), 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_CLOSE, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void MajetekZavazkyDialog::OnPrint(wxCommandEvent& /*event*/)
{
    Veta veta(1, "VetaU", 0, 1);
    dxAtributArray aveta;
    Atribut kc_dpfmz02(A_DOUBLE, "kc_dpfmz02", false, 14, 0);
    kc_dpfmz02.setValue(m_kc_dpfmz02->GetValue());
    aveta.Add(kc_dpfmz02);
    Atribut kc_z_dpfmz02(A_DOUBLE, "kc_z_dpfmz02", false, 14, 0);
    kc_z_dpfmz02.setValue(m_kc_z_dpfmz02->GetValue());
    aveta.Add(kc_z_dpfmz02);
    Atribut kc_dpfmz05a(A_DOUBLE, "kc_dpfmz05a", false, 14, 0);
    kc_dpfmz05a.setValue(m_kc_dpfmz05a->GetValue());
    aveta.Add(kc_dpfmz05a);
    Atribut kc_z_dpfmz05a(A_DOUBLE, "kc_z_dpfmz05a", false, 14, 0);
    kc_z_dpfmz05a.setValue(m_kc_z_dpfmz05a->GetValue());
    aveta.Add(kc_z_dpfmz05a);
    Atribut kc_dpfmz06(A_DOUBLE, "kc_dpfmz06", false, 14, 0);
    kc_dpfmz06.setValue(m_kc_dpfmz06->GetValue());
    aveta.Add(kc_dpfmz06);
    Atribut kc_z_dpfmz06(A_DOUBLE, "kc_z_dpfmz06", false, 14, 0);
    kc_z_dpfmz06.setValue(m_kc_z_dpfmz06->GetValue());
    aveta.Add(kc_z_dpfmz06);
    Atribut kc_dpfmz03(A_DOUBLE, "kc_dpfmz03", false, 14, 0);
    kc_dpfmz03.setValue(m_kc_dpfmz03->GetValue());
    aveta.Add(kc_dpfmz03);
    Atribut kc_z_dpfmz03(A_DOUBLE, "kc_z_dpfmz03", false, 14, 0);
    kc_z_dpfmz03.setValue(m_kc_z_dpfmz03->GetValue());
    aveta.Add(kc_z_dpfmz03);
    Atribut kc_dpfmz04(A_DOUBLE, "kc_dpfmz04", false, 14, 0);
    kc_dpfmz04.setValue(m_kc_dpfmz04->GetValue());
    aveta.Add(kc_dpfmz04);
    Atribut kc_z_dpfmz04(A_DOUBLE, "kc_z_dpfmz04", false, 14, 0);
    kc_z_dpfmz04.setValue(m_kc_z_dpfmz04->GetValue());
    aveta.Add(kc_z_dpfmz04);
    Atribut kc_dpfmz08(A_DOUBLE, "kc_dpfmz08", false, 14, 0);
    kc_dpfmz08.setValue(m_kc_dpfmz08->GetValue());
    aveta.Add(kc_dpfmz08);
    Atribut kc_z_dpfmz08(A_DOUBLE, "kc_z_dpfmz08", false, 14, 0);
    kc_z_dpfmz08.setValue(m_kc_z_dpfmz08->GetValue());
    aveta.Add(kc_z_dpfmz08);
    Atribut kc_dpfmz10(A_DOUBLE, "kc_dpfmz10", false, 14, 0);
    kc_dpfmz10.setValue(m_kc_dpfmz10->GetValue());
    aveta.Add(kc_dpfmz10);
    Atribut kc_z_dpfmz10(A_DOUBLE, "kc_z_dpfmz10", false, 14, 0);
    kc_z_dpfmz10.setValue(m_kc_z_dpfmz10->GetValue());
    aveta.Add(kc_z_dpfmz10);
    Atribut kc_dpfmz11(A_DOUBLE, "kc_dpfmz11", false, 14, 0);
    kc_dpfmz11.setValue(m_kc_dpfmz11->GetValue());
    aveta.Add(kc_dpfmz11);
    Atribut kc_z_dpfmz11(A_DOUBLE, "kc_z_dpfmz11", false, 14, 0);
    kc_z_dpfmz11.setValue(m_kc_z_dpfmz11->GetValue());
    aveta.Add(kc_z_dpfmz11);
    Atribut kc_dpfmz18(A_DOUBLE, "kc_dpfmz18", false, 14, 0);
    kc_dpfmz18.setValue(m_kc_dpfmz18->GetValue());
    aveta.Add(kc_dpfmz18);
    veta.setAtributy(aveta);
    MajetekZavazkyPrint print(this, wxAtoi(m_rok->GetStringSelection()), veta);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void MajetekZavazkyDialog::OnReload(wxCommandEvent &/*event*/)
{
    fillData();
}

void MajetekZavazkyDialog::OnRok(wxCommandEvent &/*event*/)
{
    fillData();
}

void MajetekZavazkyDialog::OnClose(wxCommandEvent &/*event*/)
{
    EndModal(wxID_OK);
}

void MajetekZavazkyDialog::fillData()
{
    int rok = wxAtoi(m_rok->GetStringSelection());
    if(idbmodel->hasPohledavkyDluhyCizi(rok))
    {
        bool download = false;
        if(!idbmodel->hasJednotneKurzyForPohledavkyDluhy(rok-1))
        {
            download = true;
        }
        if(!idbmodel->hasJednotneKurzyForPohledavkyDluhy(rok))
        {
            download = true;
        }
        if(download)
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Jsou nějaké pohledávky a dluhy v cizí měně,\nje třeba stáhnout jednotné kurzy"), "Informace", wxOK|wxICON_INFORMATION);
            dialog.ShowModal();
            if(!dxutils::jednotnekurzyDownload(rok-1))
            {
                wxMessageDialog dialog1(this, wxString::Format(wxString::FromUTF8("Nepodařilo se stáhnout jednotné kurzy\npro rok %d"), rok-1),
                                        wxString::FromUTF8("Varování"), wxOK|wxICON_WARNING);
                dialog1.ShowModal();
            }
            if(rok != wxDateTime::Today().GetYear())
            {
                if(!dxutils::jednotnekurzyDownload(rok))
                {
                    wxMessageDialog dialog1(this, wxString::Format(wxString::FromUTF8("Nepodařilo se stáhnout jednotné kurzy\npro rok %d"), rok),
                                            wxString::FromUTF8("Varování"), wxOK|wxICON_WARNING);
                    dialog1.ShowModal();
                }
            }
        }
    }
    wxBeginBusyCursor();
    //Penize v hotovosti
    m_kc_dpfmz05a->SetValue(round(idbmodel->getPokladnyStav(wxDateTime(31,wxDateTime::Dec,rok-1))));
    m_kc_z_dpfmz05a->SetValue(round(idbmodel->getPokladnyStav(wxDateTime(31,wxDateTime::Dec,rok))));
    //Penize na ucech
    m_kc_dpfmz06->SetValue(round(idbmodel->getBankyStav(wxDateTime(31,wxDateTime::Dec,rok-1))));
    m_kc_z_dpfmz06->SetValue(round(idbmodel->getBankyStav(wxDateTime(31,wxDateTime::Dec,rok))));
    //Hmotny majetek
    m_kc_dpfmz02->SetValue(round(idbmodel->getMajetekStav(wxDateTime(31,wxDateTime::Dec,rok-1), true)));
    m_kc_z_dpfmz02->SetValue(round(idbmodel->getMajetekStav(wxDateTime(31,wxDateTime::Dec,rok), true)));
    //Ostatni majetek
    m_kc_dpfmz08->SetValue(round(idbmodel->getMajetekStav(wxDateTime(31,wxDateTime::Dec,rok-1), false)));
    m_kc_z_dpfmz08->SetValue(round(idbmodel->getMajetekStav(wxDateTime(31,wxDateTime::Dec,rok), false)));
    //Rezervy
    m_kc_dpfmz11->SetValue(0.0);
    m_kc_z_dpfmz11->SetValue(0.0);
    //Mzdy
    m_kc_dpfmz18->SetValue(0.0);
    //Zasoby
    m_kc_dpfmz03->SetValue(round(idbmodel->getZasobyStav(wxDateTime(31,wxDateTime::Dec,rok-1))));
    m_kc_z_dpfmz03->SetValue(round(idbmodel->getZasobyStav(wxDateTime(31,wxDateTime::Dec,rok))));
    //Pohledavky
    m_kc_dpfmz04->SetValue(round(idbmodel->getPohledavkyStav(rok-1)));
    m_kc_z_dpfmz04->SetValue(round(idbmodel->getPohledavkyStav(rok)));
    //Zavazky
    m_kc_dpfmz10->SetValue(round(idbmodel->getZavazkyStav(rok-1)));
    m_kc_z_dpfmz10->SetValue(round(idbmodel->getZavazkyStav(rok)));
    wxEndBusyCursor();
}
