/***************************************************************
 * Name:      majetekzavazkyprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "majetekzavazkyprint.h"
#include "utils/dxutils.h"

MajetekZavazkyPrint::MajetekZavazkyPrint(wxWindow *parent, int rok, Veta veta)
    : DocumentPrint(parent, wxString::FromUTF8("Přehled majetku a závázků")), m_rok(rok), m_veta(veta)
{
    m_marginLeft = 10;
    m_marginRight = 10;
    m_marginTop = 10;
    m_marginBottom = 10;
    createDocument();
}

void MajetekZavazkyPrint::createDocument()
{
    int height = m_marginTop;
    int page = 1;
    m_document = new Document;
    TextElement text = TextElement(page, wxString::Format(wxString::FromUTF8("Přehled majetku a závázků za rok %d"), m_rok), wxPoint(m_marginLeft,height), m_pageWidth-m_marginLeft-m_marginRight,
                                   wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(14).FaceName(FACENAME).Bold())));
    m_document->addElement(text);
    height += text.height();
    Firma fi = idbmodel->firma();
    Adresa ad = idbmodel->getAdresa(fi.adresaid());
    wxArrayString firma;
    firma.Add(fi.nazev());
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.ulice(), ad.cp()) : " ");
    firma.Add(ad.isValid() ? wxString::Format("%s %s", ad.psc(), ad.misto()) : " ");
    firma.Add(ad.isValid() ? ad.zeme() : " ");
    firma.Add(wxString::Format(wxString::FromUTF8("IČ: %s"), fi.ic()));
    TextParagraphElement paragraph = TextParagraphElement(page, firma, wxPoint(m_marginLeft,height), m_pageWidth-m_marginLeft-m_marginRight);
    paragraph.setStyle(m_document->style());
    m_document->addElement(paragraph);
    height += paragraph.height();
    text = TextElement(page, " ", wxPoint(m_marginLeft,height), 25);
    m_document->addElement(text);
    height += text.height();
    wxArrayInt width;
    width.Add(45);
    width.Add(45);
    wxArrayInt alignment;
    alignment.Add(wxTEXT_ALIGNMENT_LEFT);
    alignment.Add(wxTEXT_ALIGNMENT_LEFT);
    wxArrayString values;
    values.Add(wxString::FromUTF8("Na začátku období"));
    values.Add(wxString::FromUTF8("Na konci období"));
    TableRowElement row = TableRowElement(page, values, wxPoint(m_marginLeft+100,height), width, alignment);
    row.setBorder(true);
    row.setStyle(m_document->style());
    m_document->addElement(row);
    height += row.height();
    width.Clear();
    width.Add(100);
    width.Add(45);
    width.Add(45);
    alignment.Clear();
    alignment.Add(wxTEXT_ALIGNMENT_LEFT);
    alignment.Add(wxTEXT_ALIGNMENT_RIGHT);
    alignment.Add(wxTEXT_ALIGNMENT_RIGHT);
    values.Clear();
    values.Add(wxString::FromUTF8("Hmotný majetek"));
    values.Add(m_veta.atributValue("kc_dpfmz02"));
    values.Add(m_veta.atributValue("kc_z_dpfmz02"));
    row = TableRowElement(page, values, wxPoint(m_marginLeft,height), width, alignment);
    row.setBorder(true);
    row.setStyle(m_document->style());
    m_document->addElement(row);
    height += row.height();
    values.Clear();
    values.Add(wxString::FromUTF8("Peněžní prostředky v hotovosti"));
    values.Add(m_veta.atributValue("kc_dpfmz05a"));
    values.Add(m_veta.atributValue("kc_z_dpfmz05a"));
    row = TableRowElement(page, values, wxPoint(m_marginLeft,height), width, alignment);
    row.setBorder(true);
    row.setStyle(m_document->style());
    m_document->addElement(row);
    height += row.height();
    values.Clear();
    values.Add(wxString::FromUTF8("Peněžní prostředky na bankovních účtech"));
    values.Add(m_veta.atributValue("kc_dpfmz06"));
    values.Add(m_veta.atributValue("kc_z_dpfmz06"));
    row = TableRowElement(page, values, wxPoint(m_marginLeft,height), width, alignment);
    row.setBorder(true);
    row.setStyle(m_document->style());
    m_document->addElement(row);
    height += row.height();
    values.Clear();
    values.Add(wxString::FromUTF8("Zásoby"));
    values.Add(m_veta.atributValue("kc_dpfmz03"));
    values.Add(m_veta.atributValue("kc_z_dpfmz03"));
    row = TableRowElement(page, values, wxPoint(m_marginLeft,height), width, alignment);
    row.setBorder(true);
    row.setStyle(m_document->style());
    m_document->addElement(row);
    height += row.height();
    values.Clear();
    values.Add(wxString::FromUTF8("Pohledávky včetně poskytnutých úvěrů a zápůjček"));
    values.Add(m_veta.atributValue("kc_dpfmz04"));
    values.Add(m_veta.atributValue("kc_z_dpfmz04"));
    row = TableRowElement(page, values, wxPoint(m_marginLeft,height), width, alignment);
    row.setBorder(true);
    row.setStyle(m_document->style());
    m_document->addElement(row);
    height += row.height();
    values.Clear();
    values.Add(wxString::FromUTF8("Ostatní majetek"));
    values.Add(m_veta.atributValue("kc_dpfmz08"));
    values.Add(m_veta.atributValue("kc_z_dpfmz08"));
    row = TableRowElement(page, values, wxPoint(m_marginLeft,height), width, alignment);
    row.setBorder(true);
    row.setStyle(m_document->style());
    m_document->addElement(row);
    height += row.height();
    values.Clear();
    values.Add(wxString::FromUTF8("Dluhy včetně přijatých úvěrů a zápůjček"));
    values.Add(m_veta.atributValue("kc_dpfmz10"));
    values.Add(m_veta.atributValue("kc_z_dpfmz10"));
    row = TableRowElement(page, values, wxPoint(m_marginLeft,height), width, alignment);
    row.setBorder(true);
    row.setStyle(m_document->style());
    m_document->addElement(row);
    height += row.height();
    values.Clear();
    values.Add("Rezervy");
    values.Add(m_veta.atributValue("kc_dpfmz11"));
    values.Add(m_veta.atributValue("kc_z_dpfmz11"));
    row = TableRowElement(page, values, wxPoint(m_marginLeft,height), width, alignment);
    row.setBorder(true);
    row.setStyle(m_document->style());
    m_document->addElement(row);
    height += row.height();
    text = TextElement(page, " ", wxPoint(m_marginLeft,height), 25);
    m_document->addElement(text);
    height += text.height();
    width.Clear();
    width.Add(100);
    width.Add(45);
    alignment.Clear();
    alignment.Add(wxTEXT_ALIGNMENT_LEFT);
    alignment.Add(wxTEXT_ALIGNMENT_RIGHT);
    values.Clear();
    values.Add("Mzdy");
    values.Add(m_veta.atributValue("kc_dpfmz18"));
    row = TableRowElement(page, values, wxPoint(m_marginLeft,height), width, alignment);
    row.setBorder(true);
    row.setStyle(m_document->style());
    m_document->addElement(row);
}
