/***************************************************************
 * Name:      okecdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "okecdialog.h"
#include "dxdefs.h"
#include <wx/arrimpl.cpp>
#include <wx/file.h>
#include <wx/stdpaths.h>

WX_DEFINE_OBJARRAY(dxOkecArray)

static int dxSortOkec(Okec ***o1, Okec ***o2)
{
    if((**o1)->nace() == (**o2)->nace())
        return 0;
    if((**o1)->nace() < (**o2)->nace())
        return -1;
    return 1;
}

Okec::Okec(const wxString &nazev, int nace)
    : m_nazev(nazev), m_nace(nace)
{
    m_lower = nazev.Lower();
}

wxString Okec::nazev() const
{
    return m_nazev;
}

void Okec::setNazev(const wxString &nazev)
{
    m_nazev = nazev;
}

int Okec::nace() const
{
    return m_nace;
}

void Okec::setNace(int nace)
{
    m_nace = nace;
}

wxString Okec::lower() const
{
    return m_lower;
}

void Okec::setLower(const wxString &lower)
{
    m_lower = lower;
}

IMPLEMENT_DYNAMIC_CLASS(OkecDialog, wxDialog)

BEGIN_EVENT_TABLE(OkecDialog, wxDialog)
EVT_BUTTON(wxID_OK, OkecDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, OkecDialog::OnCancel)
    EVT_LIST_ITEM_ACTIVATED(ID_OKEC_LIST, OkecDialog::OnItemActivated)
    EVT_LIST_ITEM_SELECTED(ID_OKEC_LIST, OkecDialog::OnItemSelected)
    EVT_LIST_ITEM_DESELECTED(ID_OKEC_LIST, OkecDialog::OnItemDeselected)
    EVT_TEXT(ID_OKEC_SEARCH, OkecDialog::OnSearch)
END_EVENT_TABLE()

OkecDialog::OkecDialog(wxWindow *parent, int c_okec)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Hlavní ekonomická činnost")),
      m_selectedId(-1)
{
    wxXmlDocument okec;
    okec.Load(wxStandardPaths::Get().GetUserDataDir()+"/okec.xml");
    if(okec.IsOk() && okec.GetRoot()->GetName() == "Ciselnik")
    {
        wxXmlNode *child = okec.GetRoot()->GetChildren();
        while(child)
        {
            m_okecArray.Add(new Okec(child->GetAttribute("naz_nace"), wxAtoi(child->GetAttribute("c_nace"))));
            child = child->GetNext();
        }
    }
    m_okecArray.Sort(dxSortOkec);
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    wxBoxSizer *bsearch = new wxBoxSizer(wxHORIZONTAL);
    bsearch->Add(new wxStaticText(this, wxID_ANY, "Hledat"), 0, wxALIGN_CENTER_VERTICAL, 0);
    m_search = new wxTextCtrl(this, ID_OKEC_SEARCH);
    bsearch->Add(m_search, 1, wxALL|wxEXPAND, 5);
    m_mainSizer->Add(bsearch, 0, wxALL|wxEXPAND, 0);
    m_list = new wxListCtrl(this, ID_OKEC_LIST, wxDefaultPosition, wxDefaultSize,
                            wxLC_REPORT|wxLC_HRULES|wxLC_VRULES|wxLC_SINGLE_SEL);
    wxListItem col0;
    col0.SetId(0);
    col0.SetText(wxString::FromUTF8("NACE"));
    col0.SetWidth(80);
    m_list->InsertColumn(0, col0);
    wxListItem col1;
    col1.SetId(1);
    col1.SetText(wxString::FromUTF8("Činnost"));
    col1.SetWidth(550);
    m_list->InsertColumn(1, col1);
    fillList(wxEmptyString);
    if(c_okec)
    {
        for(size_t i=0; i < m_okecArray.GetCount(); i++)
        {
            if(c_okec == m_okecArray[i]->nace())
            {
                m_list->SetItemState(static_cast<long>(i), wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
                m_list->EnsureVisible(static_cast<long>(i));
                break;
            }
        }
    }
    m_mainSizer->Add(m_list, 1, wxALL|wxEXPAND, 5);
    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

Okec OkecDialog::selectedOkec() const
{
    if(m_selectedId == -1)
    {
        return Okec("", 0);
    }
    int okec = static_cast<int>(m_list->GetItemData(m_selectedId));
    for(size_t i=0; i < m_okecArray.GetCount(); i++)
    {
        if(okec == m_okecArray[i]->nace())
        {
            return *m_okecArray[i];
        }
    }
    return Okec("", 0);
}

void OkecDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}

void OkecDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void OkecDialog::OnItemActivated(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
    EndModal(wxID_OK);
}

void OkecDialog::OnItemSelected(wxListEvent &event)
{
    setSelectedId(event.GetIndex());
}

void OkecDialog::OnItemDeselected(wxListEvent& /*event*/)
{
    setSelectedId(-1);
}

void OkecDialog::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = -1;
    else
        m_selectedId = selectedIndex;
}

void OkecDialog::fillList(const wxString &search)
{
    m_list->DeleteAllItems();
    m_list->Hide();
    for(size_t i=0, j=0; i < m_okecArray.GetCount(); i++)
    {
        if(search.IsEmpty())
        {
            wxListItem item;
            item.SetId(static_cast<long>(i));
            item.SetText(wxString::Format("%d", m_okecArray[i]->nace()));
            m_list->InsertItem(item);
            m_list->SetItem(static_cast<long>(i), 1, m_okecArray[i]->nazev());
            m_list->SetItemData(static_cast<long>(i), m_okecArray[i]->nace());
        }
        else
        {
            if(m_okecArray[i]->lower().Contains(search))
            {
                wxListItem item;
                item.SetId(static_cast<long>(j));
                item.SetText(wxString::Format("%d", m_okecArray[i]->nace()));
                m_list->InsertItem(item);
                m_list->SetItem(static_cast<long>(j), 1, m_okecArray[i]->nazev());
                m_list->SetItemData(static_cast<long>(j), m_okecArray[i]->nace());
                j++;
            }
        }
    }
    m_list->Show();
    if(m_list->GetItemCount())
        m_list->SetItemState(0, wxLIST_STATE_SELECTED, wxLIST_STATE_SELECTED);
    else
        setSelectedId(-1);
}

void OkecDialog::OnSearch(wxCommandEvent &/*event*/)
{
    if(m_search->GetValue().Len()>2)
    {
        fillList(m_search->GetValue().MakeLower());
    }
    else
    {
        fillList(wxEmptyString);
    }
}
