/***************************************************************
 * Name:      priznanidialog.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#ifndef PRIZNANIDIALOG_H
#define PRIZNANIDIALOG_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include <wx/dynarray.h>
#include <wx/gbsizer.h>
#include <wx/datectrl.h>
#include <wx/dateevt.h>
#include <wx/xml/xml.h>
#include "model/dbmodel.h"

class Obec
{
public:
    Obec(const wxString &nazev, int psc, int fu) : m_nazev(nazev), m_psc(psc), m_fu(fu) {}

    wxString nazev() const;
    void setNazev(const wxString &nazev);
    int psc() const;
    void setPsc(int psc);
    int fu() const;
    void setFu(int fu);
private:
    wxString m_nazev;
    int m_psc;
    int m_fu;
};

class Ufo
{
public:
    Ufo(const wxString &nazev, int ufo) : m_nazev(nazev), m_ufo(ufo) {}

    wxString nazev() const;
    void setNazev(const wxString &nazev);
    int ufo() const;
    void setUfo(int ufo);
private:
    wxString m_nazev;
    int m_ufo;
};

class Pracufo
{
public:
    Pracufo(const wxString &nazev, int pracufo, int pufo, int ufo) : m_nazev(nazev), m_pracufo(pracufo), m_pufo(pufo), m_ufo(ufo) {}

    wxString nazev() const;
    void setNazev(const wxString &nazev);
    int pracufo() const;
    void setPracufo(int pracufo);
    int pufo() const;
    void setPufo(int pufo);
    int ufo() const;
    void setUfo(int ufo);
private:
    wxString m_nazev;
    int m_pracufo;
    int m_pufo;
    int m_ufo;
};

WX_DECLARE_OBJARRAY(Obec*, dxObecArray);
WX_DECLARE_OBJARRAY(Ufo*, dxUfoArray);
WX_DECLARE_OBJARRAY(Pracufo*, dxPracufoArray);

class PriznaniDialog: public wxDialog
{
    DECLARE_DYNAMIC_CLASS(PriznaniDialog)
    DECLARE_EVENT_TABLE()
public:
    PriznaniDialog() : m_edit(false), m_mainSizer(nullptr), m_jmeno(nullptr), m_prijmeni(nullptr), m_titul(nullptr), m_ulice(nullptr), m_cpop(0),
    m_cpopText(nullptr), m_corient(nullptr), m_psc(0), m_pscText(nullptr), m_nazobce(nullptr), m_dicText(nullptr), m_email(nullptr), m_iddats(nullptr),
    m_ctelef(nullptr), m_cufo(nullptr), m_cpracufo(nullptr), m_cokec(0), m_cokecText(nullptr), m_cokecSelect(nullptr), m_zast_typ(nullptr), m_zast_kod(nullptr),
    m_zast_jmeno(nullptr), m_zast_prijmeni(nullptr), m_zast_dat_nar(nullptr), m_zast_ev_cislo(nullptr), m_zast_nazev(nullptr), m_zast_ic(0), m_zast_icText(nullptr),
    m_opr_jmeno(nullptr), m_opr_prijmeni(nullptr), m_opr_postaveni(nullptr) {}
    PriznaniDialog(wxWindow *parent, bool edit);
private:
    bool m_edit;
    dxObecArray m_obceArray;
    dxUfoArray m_ufoArray;
    dxPracufoArray m_pracufoArray;
    wxBoxSizer *m_mainSizer;
    wxTextCtrl *m_jmeno;
    wxTextCtrl *m_prijmeni;
    wxTextCtrl *m_titul;
    wxTextCtrl *m_ulice;
    int m_cpop;
    wxTextCtrl *m_cpopText;
    wxTextCtrl *m_corient;
    int m_psc;
    wxTextCtrl *m_pscText;
    wxComboBox *m_nazobce;
    wxString m_dic;
    wxTextCtrl *m_dicText;
    wxTextCtrl *m_email;
    wxTextCtrl *m_iddats;
    wxTextCtrl *m_ctelef;
    wxChoice *m_cufo;
    wxChoice *m_cpracufo;
    int m_cokec;
    wxTextCtrl *m_cokecText;
    wxButton *m_cokecSelect;
    wxChoice *m_zast_typ;
    wxChoice *m_zast_kod;
    wxTextCtrl *m_zast_jmeno;
    wxTextCtrl *m_zast_prijmeni;
    wxDatePickerCtrl *m_zast_dat_nar;
    wxTextCtrl *m_zast_ev_cislo;
    wxTextCtrl *m_zast_nazev;
    int m_zast_ic;
    wxTextCtrl *m_zast_icText;
    wxTextCtrl *m_opr_jmeno;
    wxTextCtrl *m_opr_prijmeni;
    wxTextCtrl *m_opr_postaveni;

    void fillArrays();
    void createControls();
    wxGridBagSizer *createFirmaGrid();
    wxGridBagSizer *createZastupceGrid();
    void fillControls();
    void OnOk(wxCommandEvent& event);
    void OnCancel(wxCommandEvent& event);
    void OnPsc(wxCommandEvent &event);
    void OnCufo(wxCommandEvent& event);
    void pscChanged();
    void OnCokecSelect(wxCommandEvent& event);
    void OnZastTyp(wxCommandEvent& event);
};

#endif  /* PRIZNANIDIALOG_H */

