/***************************************************************
 * Name:      seznamdphdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include <wx/wfstream.h>
#include "seznamdphdialog.h"
#include "seznamdphprint.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(SeznamDphDialog, wxDialog)

BEGIN_EVENT_TABLE(SeznamDphDialog, wxDialog)
    EVT_BUTTON(wxID_CLOSE, SeznamDphDialog::OnOk)
    EVT_RADIOBUTTON(ID_SEZNAMDPH_MESIC, SeznamDphDialog::OnMesic)
    EVT_RADIOBUTTON(ID_SEZNAMDPH_CTVRTLETI, SeznamDphDialog::OnCtvrtleti)
    EVT_RADIOBUTTON(ID_SEZNAMDPH_ODDO, SeznamDphDialog::OnOddo)
    EVT_DATE_CHANGED(ID_SEZNAMDPH_OD, SeznamDphDialog::OnOd)
    EVT_DATE_CHANGED(ID_SEZNAMDPH_DO, SeznamDphDialog::OnDo)
    EVT_BUTTON(ID_SEZNAMDPH_PRINT, SeznamDphDialog::OnPrint)
    EVT_BUTTON(ID_SEZNAMDPH_EXPORT, SeznamDphDialog::OnExport)
END_EVENT_TABLE()

SeznamDphDialog::SeznamDphDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Seznam dokladů DPH"))
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void SeznamDphDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    m_mesicBtn = new wxRadioButton(this, ID_SEZNAMDPH_MESIC, wxString::FromUTF8("Měsíc"));
    m_mesicBtn->SetValue(false);
    gridSizer->Add(m_mesicBtn, wxGBPosition(0,0), wxDefaultSpan, wxEXPAND);
    m_mesicSelect = new wxChoice(this, wxID_ANY);
    m_mesicSelect->Append(dxutils::mesicString(1));
    m_mesicSelect->Append(dxutils::mesicString(2));
    m_mesicSelect->Append(dxutils::mesicString(3));
    m_mesicSelect->Append(dxutils::mesicString(4));
    m_mesicSelect->Append(dxutils::mesicString(5));
    m_mesicSelect->Append(dxutils::mesicString(6));
    m_mesicSelect->Append(dxutils::mesicString(7));
    m_mesicSelect->Append(dxutils::mesicString(8));
    m_mesicSelect->Append(dxutils::mesicString(9));
    m_mesicSelect->Append(dxutils::mesicString(10));
    m_mesicSelect->Append(dxutils::mesicString(11));
    m_mesicSelect->Append(dxutils::mesicString(12));
    m_mesicSelect->SetSelection(0);
    m_mesicSelect->Enable(false);
    gridSizer->Add(m_mesicSelect, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    m_mesicRok = new wxChoice(this, wxID_ANY);
    m_mesicRok->Append(idbmodel->getObdobiAsStrings());
    m_mesicRok->SetStringSelection(wxString::Format("%d", idbmodel->currentRok()));
    m_mesicRok->Enable(false);
    gridSizer->Add(m_mesicRok, wxGBPosition(0,2), wxDefaultSpan, wxEXPAND);

    m_ctvrtletiBtn = new wxRadioButton(this, ID_SEZNAMDPH_CTVRTLETI, wxString::FromUTF8("Čtvrtletí"));
    m_ctvrtletiBtn->SetValue(true);
    gridSizer->Add(m_ctvrtletiBtn, wxGBPosition(1,0), wxDefaultSpan, wxEXPAND);
    m_ctvrtletiSelect = new wxChoice(this, wxID_ANY);
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(1));
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(2));
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(3));
    m_ctvrtletiSelect->Append(dxutils::ctvrtletiString(4));
    m_ctvrtletiSelect->SetSelection(0);
    gridSizer->Add(m_ctvrtletiSelect, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    m_ctvrtletiRok = new wxChoice(this, wxID_ANY);
    m_ctvrtletiRok->Append(idbmodel->getObdobiAsStrings());
    m_ctvrtletiRok->SetStringSelection(wxString::Format("%d", idbmodel->currentRok()));
    gridSizer->Add(m_ctvrtletiRok, wxGBPosition(1,2), wxDefaultSpan, wxEXPAND);

    m_oddoBtn = new wxRadioButton(this, ID_SEZNAMDPH_ODDO, wxString::FromUTF8("Období"));
    m_oddoBtn->SetValue(false);
    gridSizer->Add(m_oddoBtn, wxGBPosition(2,0), wxDefaultSpan, wxEXPAND);
    m_od = new wxDatePickerCtrl(this, ID_SEZNAMDPH_OD, wxDateTime(1, wxDateTime::Jan, idbmodel->currentRok()), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_od->Enable(false);
    gridSizer->Add(m_od, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    m_do = new wxDatePickerCtrl(this, ID_SEZNAMDPH_DO, wxDateTime(31, wxDateTime::Jan, idbmodel->currentRok()), wxDefaultPosition, wxDefaultSize, wxDP_DROPDOWN|wxDP_SHOWCENTURY);
    m_do->Enable(false);
    gridSizer->Add(m_do, wxGBPosition(2,2), wxDefaultSpan, wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxBoxSizer *buttons = new wxBoxSizer(wxHORIZONTAL);
    m_print = new wxButton(this, ID_SEZNAMDPH_PRINT, "Tisk seznamu");
    buttons->Add(m_print);
    m_export = new wxButton(this, ID_SEZNAMDPH_EXPORT, "Export seznamu");
    buttons->Add(m_export);
    m_mainSizer->Add(buttons, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_CLOSE, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void SeznamDphDialog::OnMesic(wxCommandEvent &/*event*/)
{
    m_ctvrtletiSelect->Enable(false);
    m_ctvrtletiRok->Enable(false);
    m_mesicSelect->Enable(true);
    m_mesicRok->Enable(true);
    m_od->Enable(false);
    m_do->Enable(false);
}

void SeznamDphDialog::OnCtvrtleti(wxCommandEvent &/*event*/)
{
    m_mesicSelect->Enable(false);
    m_mesicRok->Enable(false);
    m_ctvrtletiSelect->Enable(true);
    m_ctvrtletiRok->Enable(true);
    m_od->Enable(false);
    m_do->Enable(false);
}

void SeznamDphDialog::OnOddo(wxCommandEvent &/*event*/)
{
    m_mesicSelect->Enable(false);
    m_mesicRok->Enable(false);
    m_ctvrtletiSelect->Enable(false);
    m_ctvrtletiRok->Enable(false);
    m_od->Enable(true);
    m_do->Enable(true);
}

void SeznamDphDialog::OnPrint(wxCommandEvent &/*event*/)
{
    if(m_oddoBtn->GetValue())
    {
        if(!idbmodel->someDokladForDph(m_od->GetValue(), m_do->GetValue()))
        {
            wxMessageDialog dialog(this, wxString::FromUTF8("Nejsou žádné doklady pro vybrané období"), "Informace", wxOK|wxICON_WARNING);
            dialog.ShowModal();
            return;
        }
        SeznamDphPrint print(this, m_od->GetValue(), m_do->GetValue());
        //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
        print.Print();
#else
        print.Preview();
#endif
        return;
    }
    int mesic = m_mesicBtn->GetValue()?m_mesicSelect->GetSelection()+1:0;
    int ctvrtleti = m_ctvrtletiBtn->GetValue()?m_ctvrtletiSelect->GetSelection()+1:0;
    int rok = m_mesicBtn->GetValue()?wxAtoi(m_mesicRok->GetStringSelection()):wxAtoi(m_ctvrtletiRok->GetStringSelection());
    wxDateTime start(1, mesic?static_cast<wxDateTime::Month>(mesic-1):dxutils::ctvrtletiMesicStart(ctvrtleti), rok);
    wxDateTime end(start.GetLastMonthDay(mesic?static_cast<wxDateTime::Month>(mesic-1):dxutils::ctvrtletiMesicEnd(ctvrtleti), rok));
    if(!idbmodel->someDokladForDph(start, end))
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Nejsou žádné doklady pro vybrané období"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    SeznamDphPrint print(this, mesic, ctvrtleti, rok);
    //TODO hnusnej hack aby pri Gtk3 nebylo preview blbe, takze rovnou print dialog
#ifdef __WXGTK3__
    print.Print();
#else
    print.Preview();
#endif
}

void SeznamDphDialog::OnExport(wxCommandEvent &/*event*/)
{
    int mesic = m_mesicBtn->GetValue()?m_mesicSelect->GetSelection()+1:0;
    int ctvrtleti = m_ctvrtletiBtn->GetValue()?m_ctvrtletiSelect->GetSelection()+1:0;
    int rok = m_mesicBtn->GetValue()?wxAtoi(m_mesicRok->GetStringSelection()):wxAtoi(m_ctvrtletiRok->GetStringSelection());
    wxDateTime start(1, mesic?static_cast<wxDateTime::Month>(mesic-1):dxutils::ctvrtletiMesicStart(ctvrtleti), rok);
    wxDateTime end(start.GetLastMonthDay(mesic?static_cast<wxDateTime::Month>(mesic-1):dxutils::ctvrtletiMesicEnd(ctvrtleti), rok));
    if(!idbmodel->someDokladForDph(m_oddoBtn->GetValue()?m_od->GetValue():start, m_oddoBtn->GetValue()?m_do->GetValue():end))
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Nejsou žádné doklady pro vybrané období"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        return;
    }
    wxFileDialog dlg(this, wxString::FromUTF8("Exportovat seznam dokladů DPH do"), wxEmptyString, wxEmptyString, "CSV soubory (*.csv)|*.csv",
                     wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() != wxID_OK) return;
    wxString path = dlg.GetPath();
    dxutils::correctEmptyFileExt("csv", path);
    wxFileOutputStream output(path);
    wxTextOutputStream text(output);
    text << createRowsForRadky(wxSplit("1,2",'|'), start, end, wxString::FromUTF8("Dodání zboží nebo poskytnutí služby s místem plnění v tuzemsku (např. §13, §14, §8)"), wxString::FromUTF8("Řádky v přiznání: 1,2"));
    text << createRowsForRadky(wxSplit("3,4|3,4,43,44",'|'), start, end, wxString::FromUTF8("Pořízení zboží z jiného členského státu (§16; §17 odst.6 písm e); §19 odst.3)"), wxString::FromUTF8("Řádky v přiznání: 3,4"));
    text << createRowsForRadky(wxSplit("5,6|5,6,43,44",'|'), start, end, wxString::FromUTF8("Přijetí služby s místem plnění podle §9 odst.1 od osoby registrované k dani v jiném členském státě"), wxString::FromUTF8("Řádky v přiznání: 5,6"));
    text << createRowsForRadky(wxSplit("7,8|7,8,42",'|'), start, end, wxString::FromUTF8("Dovoz zboží (§23 odst.3 - 5)"), wxString::FromUTF8("Řádky v přiznání: 7,8"));
    text << createRowsForRadky(wxSplit("9|9,43,44",'|'), start, end, wxString::FromUTF8("Pořízení nového dopravního prostředku (§19 odst.4)"), wxString::FromUTF8("Řádek v přiznání: 9"));
    text << createRowsForRadky(wxSplit("10,11|10,11,43,44",'|'), start, end, wxString::FromUTF8("Režim přenesení daňové povinnosti (§92a) - odběratel zboží nebo příjemce služeb"), wxString::FromUTF8("Řádky v přiznání: 10,11"));
    text << createRowsForRadky(wxSplit("12,13|12,13,43,44",'|'), start, end, wxString::FromUTF8("Ostatní zdanitelná plnění, u kterých je povinen přiznat daň plátce při jejich přijetí (§108)"), wxString::FromUTF8("Řádky v přiznání: 12,13"));
    text << createRowsForRadky(wxSplit("20",'|'), start, end, wxString::FromUTF8("Dodání zboží do jiného členského státu (§64)"), wxString::FromUTF8("Řádek v přiznání: 20"));
    text << createRowsForRadky(wxSplit("21",'|'), start, end, wxString::FromUTF8("Poskytnutí služeb s místem plnění v jiném členském státě vymezených v §102 odst.1 písm.d) a odst.2"), wxString::FromUTF8("Řádek v přiznání: 21"));
    text << createRowsForRadky(wxSplit("22",'|'), start, end, wxString::FromUTF8("Vývoz zboží (§66)"), wxString::FromUTF8("Řádek v přiznání: 22"));
    text << createRowsForRadky(wxSplit("23",'|'), start, end, wxString::FromUTF8("Dodání nového dopravního prostředku osobě neregistrované k dani v jiném členském státě (§19 odst.4)"), wxString::FromUTF8("Řádek v přiznání: 23"));
    text << createRowsForRadky(wxSplit("24",'|'), start, end, wxString::FromUTF8("Zasílání zboží do jiného členského státu (§8)"), wxString::FromUTF8("Řádek v přiznání: 24"));
    text << createRowsForRadky(wxSplit("25",'|'), start, end, wxString::FromUTF8("Režim přenesení daňové povinnosti (§92a) - dodavatel zboží nebo poskytovatel služeb"), wxString::FromUTF8("Řádek v přiznání: 25"));
    text << createRowsForRadky(wxSplit("26",'|'), start, end, wxString::FromUTF8("Ostatní uskutečněná plnění s nárokem na odpočet daně (např. §24a, §67, §68, §69, §70, §89, §90, §92)"), wxString::FromUTF8("Řádek v přiznání: 26"));
    text << createRowsForRadky(wxSplit("30",'|'), start, end, wxString::FromUTF8("Zjednodušený postup při dodání zboží formou třístranného obchodu (§17) prostřední osobou - Pořízení zboží"), wxString::FromUTF8("Řádek v přiznání: 30"));
    text << createRowsForRadky(wxSplit("31",'|'), start, end, wxString::FromUTF8("Zjednodušený postup při dodání zboží formou třístranného obchodu (§17) prostřední osobou - Dodání zboží"), wxString::FromUTF8("Řádek v přiznání: 31"));
    text << createRowsForRadky(wxSplit("32",'|'), start, end, wxString::FromUTF8("Dovoz zboží osvobozený podle §71g"), wxString::FromUTF8("Řádek v přiznání: 32"));
    text << createRowsForRadky(wxSplit("33",'|'), start, end, wxString::FromUTF8("Oprava výše daně u pohledávek za dlužníky v insolvenčním řízení (§44) - Věřitel"), wxString::FromUTF8("Řádek v přiznání: 33"));
    text << createRowsForRadky(wxSplit("34",'|'), start, end, wxString::FromUTF8("Oprava výše daně u pohledávek za dlužníky v insolvenčním řízení (§44) - Dlužník"), wxString::FromUTF8("Řádek v přiznání: 34"));
    text << createRowsForRadky(wxSplit("40,41|40,41,47",'|'), start, end, wxString::FromUTF8("Z přijatých zdanitelných plnění od plátců"), wxString::FromUTF8("Řádky v přiznání: 40,41"));
    text << createRowsForRadky(wxSplit("7,8,42",'|'), start, end, wxString::FromUTF8("Při dovozu zboží, kdy je správcem daně celní úřad"), wxString::FromUTF8("Řádek v přiznání: 42"));
    text << createRowsForRadky(wxSplit("3,4,43,44|5,6,43,44|9,43,44|10,11,43,44|12,13,43,44",'|'), start, end, wxString::FromUTF8("Ze zdanitelných plnění vykázaných na řádcích 3 až 13"), wxString::FromUTF8("Řádky v přiznání: 43,44"));
    text << createRowsForRadky(wxSplit("40,41,47",'|'), start, end, wxString::FromUTF8("Hodnota pořízeného majetku vymezeného v § 4 odst. 4 písm. d) a e)"), wxString::FromUTF8("Řádek v přiznání: 47"));
    text << createRowsForRadky(wxSplit("50",'|'), start, end, wxString::FromUTF8("Plnění osvobozená od daně bez nároku na odpočet daně"), wxString::FromUTF8("Řádek v přiznání: 50"));
    text << createRowsForRadky(wxSplit("61",'|'), start, end, wxString::FromUTF8("Vrácení daně (§ 84)"), wxString::FromUTF8("Řádek v přiznání: 61"));
}

void SeznamDphDialog::OnOd(wxDateEvent &/*event*/)
{
    if(m_do->GetValue().IsEarlierThan(m_od->GetValue())) m_do->SetValue(m_od->GetValue().GetLastMonthDay(m_od->GetValue().GetMonth(), m_od->GetValue().GetYear()));
}

void SeznamDphDialog::OnDo(wxDateEvent &/*event*/)
{
    if(m_od->GetValue().IsLaterThan(m_do->GetValue())) m_od->SetValue(wxDateTime(1, m_do->GetValue().GetMonth(), m_do->GetValue().GetYear()));
}

wxString SeznamDphDialog::createRowsForRadky(const wxArrayString &radky, const wxDateTime &start, const wxDateTime &end, const wxString &popis1, const wxString &popis2)
{
    wxArrayInt ids = idbmodel->getDphDruhIdByRadek(radky);
    if(!idbmodel->someDokladForDph(ids, start, end)) return wxEmptyString;
    wxString text;
    wxString delim = ",";
    wxArrayDouble celkem;
    DphDruh dphd = idbmodel->getDphDruh(ids[0]);
    text << dxutils::enquotes(popis1, delim) << "\r\n";
    text << dxutils::enquotes(popis2, delim) << "\r\n";
    text << ((dphd.isValid()?dphd.plneni():false)?wxString::FromUTF8("Přijaté plnění"):wxString::FromUTF8("Uskutečněné plnění")) << "\r\n";
    dxDphDokladArray doklady = idbmodel->dokladyForDph(ids, start, end, false);
    idbmodel->sortDphDoklad(doklady);
    dxDphForDokladArray dphdef = doklady[0].dph();
    celkem.SetCount(2*dphdef.GetCount());
    for(size_t i=0; i < doklady.GetCount(); i++)
    {
        dxDphForDokladArray dph = doklady[i].dph();
        if(!i)
        {
            text << createHeader(dph);
        }
        if(!idbmodel->isDphSazbySameForDoklad(dph, dphdef))
        {
            text << "Celkem" << delim << delim << delim << delim << delim;
            for(size_t j=0; j<celkem.GetCount(); j++)
            {
                if(j%2 == 0) text << delim;
                text << dxutils::enquotes(wxString::Format("%.2f", celkem[j]), delim) << delim;
            }
            text << "\r\n";
            dphdef = dph;
            text << createHeader(dph);
            celkem.Shrink();
            celkem.SetCount(2*dph.GetCount());
        }
        text << dxutils::enquotes(doklady[i].cislo(), delim) << delim;
        text << doklady[i].duzp().FormatDate() << delim;
        text << doklady[i].dppd().FormatDate() << delim;
        text << dxutils::enquotes(doklady[i].popis(), delim) << delim;
        text << doklady[i].zdroj() << delim;
        text << dxutils::enquotes(doklady[i].dic(), delim) << delim;
        for(size_t k=0; k<dph.GetCount(); k++)
        {
            if(dph[k].sazba())
            {
                text << dxutils::enquotes(wxString::Format("%.2f", dph[k].zaklad()), delim) << delim;
                celkem[2*k] += dph[k].zaklad();
                text << dxutils::enquotes(wxString::Format("%.2f", dph[k].dph()), delim) << delim;
                celkem[2*k+1] += dph[k].dph();
                text << dxutils::enquotes(wxString::Format("%d%%", dph[k].sazba()), delim) << delim;
            }
            else
            {
                text << dxutils::enquotes(wxString::Format("%.2f", dph[k].zaklad()), delim) << delim;
                celkem[2*k] += dph[k].zaklad();
            }
        }
        text << dxutils::enquotes(wxString::Format("%.2f", doklady[i].celkem()), delim) << delim;
        celkem[2*dph.GetCount()-1] += doklady[i].celkem();
        text << "\r\n";
    }
    text << "Celkem" << delim << delim << delim << delim << delim;
    for(size_t j=0; j<celkem.GetCount(); j++)
    {
        if(j%2 == 0) text << delim;
        text << dxutils::enquotes(wxString::Format("%.2f", celkem[j]), delim) << delim;
    }
    text << "\r\n";
    return text;
}

wxString SeznamDphDialog::createHeader(dxDphForDokladArray arr)
{
    wxString text = "";
    wxString delim = ",";
    text << "Doklad" << delim << "DUZP" << delim << "DPPD" << delim << "Popis" << delim << "Zdroj" << delim << wxString::FromUTF8("DIČ") << delim;
    for(size_t i=0; i<arr.GetCount(); i++)
    {
        if(arr[i].sazba())
        {
            text << wxString::FromUTF8("Základ") << delim;
            text << "DPH" << delim;
            text << "Sazba" << delim;
        }
        else
        {
            text << wxString::FromUTF8("Základ DPH 0%") << delim;
            text << "Celkem" << delim;
        }
    }
    text << "\r\n";
    return text;
}

void SeznamDphDialog::OnOk(wxCommandEvent& /*event*/)
{
    EndModal(wxID_OK);
}
