/***************************************************************
 * Name:      seznamdphprint.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "seznamdphprint.h"
#include "utils/dxutils.h"

SeznamDphPrint::SeznamDphPrint(wxWindow *parent, int mesic, int ctvrtleti, int rok)
    : MultiTablePrint(parent, wxString::FromUTF8("Seznam dokladů DPH"))
{
    m_pageWidth = 297;
    m_pageHeight = 210;
    m_paperOrientation = wxLANDSCAPE;
    wxDateTime start(1, mesic?static_cast<wxDateTime::Month>(mesic-1):dxutils::ctvrtletiMesicStart(ctvrtleti), rok);
    createTable(mesic?dxutils::mesicString(mesic)+wxString::Format("/%d", rok):dxutils::ctvrtletiString(ctvrtleti)+wxString::Format("/%d", rok),
                start,
                wxDateTime(start.GetLastMonthDay(mesic?static_cast<wxDateTime::Month>(mesic-1):dxutils::ctvrtletiMesicEnd(ctvrtleti), rok)));
}

SeznamDphPrint::SeznamDphPrint(wxWindow *parent, const wxDateTime &start, const wxDateTime &end)
    : MultiTablePrint(parent, wxString::FromUTF8("Seznam dokladů DPH"))
{
    m_pageWidth = 297;
    m_pageHeight = 210;
    m_paperOrientation = wxLANDSCAPE;
    createTable(start.FormatDate()+"-"+end.FormatDate(), start, end);
}

void SeznamDphPrint::createTable(const wxString &headerRightText, const wxDateTime &start, const wxDateTime &end)
{
    m_headerLeftText = idbmodel->firma().nazev();
    m_headerCenterText = wxString::FromUTF8("Seznam dokladů DPH");
    m_headerRightText = headerRightText;
    m_headerLine = true;
    createTableForRadky(wxSplit("1,2",'|'), start, end, wxString::FromUTF8("Dodání zboží nebo poskytnutí služby s místem plnění v tuzemsku (např. §13, §14, §8)"), wxString::FromUTF8("Řádky v přiznání: 1,2"));
    createTableForRadky(wxSplit("3,4|3,4,43,44",'|'), start, end, wxString::FromUTF8("Pořízení zboží z jiného členského státu (§16; §17 odst.6 písm e); §19 odst.3)"), wxString::FromUTF8("Řádky v přiznání: 3,4"));
    createTableForRadky(wxSplit("5,6|5,6,43,44",'|'), start, end, wxString::FromUTF8("Přijetí služby s místem plnění podle §9 odst.1 od osoby registrované k dani v jiném členském státě"), wxString::FromUTF8("Řádky v přiznání: 5,6"));
    createTableForRadky(wxSplit("7,8|7,8,42",'|'), start, end, wxString::FromUTF8("Dovoz zboží (§23 odst.3 - 5)"), wxString::FromUTF8("Řádky v přiznání: 7,8"));
    createTableForRadky(wxSplit("9|9,43,44",'|'), start, end, wxString::FromUTF8("Pořízení nového dopravního prostředku (§19 odst.4)"), wxString::FromUTF8("Řádek v přiznání: 9"));
    createTableForRadky(wxSplit("10,11|10,11,43,44",'|'), start, end, wxString::FromUTF8("Režim přenesení daňové povinnosti (§92a) - odběratel zboží nebo příjemce služeb"), wxString::FromUTF8("Řádky v přiznání: 10,11"));
    createTableForRadky(wxSplit("12,13|12,13,43,44",'|'), start, end, wxString::FromUTF8("Ostatní zdanitelná plnění, u kterých je povinen přiznat daň plátce při jejich přijetí (§108)"), wxString::FromUTF8("Řádky v přiznání: 12,13"));
    createTableForRadky(wxSplit("20",'|'), start, end, wxString::FromUTF8("Dodání zboží do jiného členského státu (§64)"), wxString::FromUTF8("Řádek v přiznání: 20"));
    createTableForRadky(wxSplit("21",'|'), start, end, wxString::FromUTF8("Poskytnutí služeb s místem plnění v jiném členském státě vymezených v §102 odst.1 písm.d) a odst.2"), wxString::FromUTF8("Řádek v přiznání: 21"));
    createTableForRadky(wxSplit("22",'|'), start, end, wxString::FromUTF8("Vývoz zboží (§66)"), wxString::FromUTF8("Řádek v přiznání: 22"));
    createTableForRadky(wxSplit("23",'|'), start, end, wxString::FromUTF8("Dodání nového dopravního prostředku osobě neregistrované k dani v jiném členském státě (§19 odst.4)"), wxString::FromUTF8("Řádek v přiznání: 23"));
    createTableForRadky(wxSplit("24",'|'), start, end, wxString::FromUTF8("Zasílání zboží do jiného členského státu (§8)"), wxString::FromUTF8("Řádek v přiznání: 24"));
    createTableForRadky(wxSplit("25",'|'), start, end, wxString::FromUTF8("Režim přenesení daňové povinnosti (§92a) - dodavatel zboží nebo poskytovatel služeb"), wxString::FromUTF8("Řádek v přiznání: 25"));
    createTableForRadky(wxSplit("26",'|'), start, end, wxString::FromUTF8("Ostatní uskutečněná plnění s nárokem na odpočet daně (např. §24a, §67, §68, §69, §70, §89, §90, §92)"), wxString::FromUTF8("Řádek v přiznání: 26"));
    createTableForRadky(wxSplit("30",'|'), start, end, wxString::FromUTF8("Zjednodušený postup při dodání zboží formou třístranného obchodu (§17) prostřední osobou - Pořízení zboží"), wxString::FromUTF8("Řádek v přiznání: 30"));
    createTableForRadky(wxSplit("31",'|'), start, end, wxString::FromUTF8("Zjednodušený postup při dodání zboží formou třístranného obchodu (§17) prostřední osobou - Dodání zboží"), wxString::FromUTF8("Řádek v přiznání: 31"));
    createTableForRadky(wxSplit("32",'|'), start, end, wxString::FromUTF8("Dovoz zboží osvobozený podle §71g"), wxString::FromUTF8("Řádek v přiznání: 32"));
    createTableForRadky(wxSplit("33",'|'), start, end, wxString::FromUTF8("Oprava výše daně u pohledávek za dlužníky v insolvenčním řízení (§44) - Věřitel"), wxString::FromUTF8("Řádek v přiznání: 33"));
    createTableForRadky(wxSplit("34",'|'), start, end, wxString::FromUTF8("Oprava výše daně u pohledávek za dlužníky v insolvenčním řízení (§44) - Dlužník"), wxString::FromUTF8("Řádek v přiznání: 34"));
    createTableForRadky(wxSplit("40,41|40,41,47",'|'), start, end, wxString::FromUTF8("Z přijatých zdanitelných plnění od plátců"), wxString::FromUTF8("Řádky v přiznání: 40,41"));
    createTableForRadky(wxSplit("7,8,42",'|'), start, end, wxString::FromUTF8("Při dovozu zboží, kdy je správcem daně celní úřad"), wxString::FromUTF8("Řádek v přiznání: 42"));
    createTableForRadky(wxSplit("3,4,43,44|5,6,43,44|9,43,44|10,11,43,44|12,13,43,44",'|'), start, end, wxString::FromUTF8("Ze zdanitelných plnění vykázaných na řádcích 3 až 13"), wxString::FromUTF8("Řádky v přiznání: 43,44"));
    createTableForRadky(wxSplit("40,41,47",'|'), start, end, wxString::FromUTF8("Hodnota pořízeného majetku vymezeného v § 4 odst. 4 písm. d) a e)"), wxString::FromUTF8("Řádek v přiznání: 47"));
    createTableForRadky(wxSplit("50",'|'), start, end, wxString::FromUTF8("Plnění osvobozená od daně bez nároku na odpočet daně"), wxString::FromUTF8("Řádek v přiznání: 50"));
    createTableForRadky(wxSplit("61",'|'), start, end, wxString::FromUTF8("Vrácení daně (§ 84)"), wxString::FromUTF8("Řádek v přiznání: 61"));
}

void SeznamDphPrint::createTableForRadky(const wxArrayString &radky, const wxDateTime &start, const wxDateTime &end, const wxString &popis1, const wxString &popis2)
{
    wxArrayInt ids = idbmodel->getDphDruhIdByRadek(radky);
    if(!idbmodel->someDokladForDph(ids, start, end)) return;
    dxDphDokladArray doklady = idbmodel->dokladyForDph(ids, start, end, false);
    idbmodel->sortDphDoklad(doklady);
    size_t sazeb = doklady[0].dph().GetCount();
    wxArrayDouble celkem(2*sazeb);
    DphDruh dphd = idbmodel->getDphDruh(ids[0]);
    Table table(6+3*static_cast<int>(sazeb)-1);
    table.setTableTitle(TableTitle(popis1+"\n"
                                   +popis2+"\n"
                                   +((dphd.isValid()?dphd.plneni():false)?wxString::FromUTF8("Přijaté plnění"):wxString::FromUTF8("Uskutečněné plnění")),
                                   wxTEXT_ALIGNMENT_LEFT));
    table.addCellToHeader("Doklad");
    table.addCellToHeader("DUZP");
    table.addCellToHeader("DPPD");
    table.addCellToHeader("Popis");
    table.addCellToHeader("Zdroj");
    table.addCellToHeader(wxString::FromUTF8("DIČ"));
    for(size_t j=0; j<sazeb; j++)
    {
        if(j+1 != sazeb)
        {
            table.addCellToHeader(wxString::FromUTF8("Základ"));
            table.addCellToHeader("DPH");
            table.addCellToHeader("Sazba");
        }
        else
        {
            table.addCellToHeader(wxString::FromUTF8("Základ DPH 0%"));
            table.addCellToHeader("Celkem");
        }
    }
    dxDphForDokladArray dphdef = doklady[0].dph();
    for(size_t i=0; i<doklady.GetCount(); i++)
    {
        dxDphForDokladArray dph = doklady[i].dph();
        if(!idbmodel->isDphSazbySameForDoklad(dphdef, dph))
        {
            table.setColumnWidth(3, 30);
            table.refreshCellsSize();
            m_tables->Add(table);
            dphdef = doklady[i].dph();
            celkem.Shrink();
            celkem.SetCount(2*dph.GetCount());
            table = Table(6+3*static_cast<int>(dph.GetCount())-1);
            table.setTableTitle(TableTitle(popis1+"\n"
                                           +popis2+"\n"
                                           +((dphd.isValid()?dphd.plneni():false)?wxString::FromUTF8("Přijaté plnění"):wxString::FromUTF8("Uskutečněné plnění")),
                                           wxTEXT_ALIGNMENT_LEFT));
            table.addCellToHeader("Doklad");
            table.addCellToHeader("DUZP");
            table.addCellToHeader("DPPD");
            table.addCellToHeader("Popis");
            table.addCellToHeader("Zdroj");
            table.addCellToHeader(wxString::FromUTF8("DIČ"));
            for(size_t j=0; j<dph.GetCount(); j++)
            {
                if(j+1 != dph.GetCount())
                {
                    table.addCellToHeader(wxString::FromUTF8("Základ"));
                    table.addCellToHeader("DPH");
                    table.addCellToHeader("Sazba");
                }
                else
                {
                    table.addCellToHeader(wxString::FromUTF8("Základ DPH 0%"));
                    table.addCellToHeader("Celkem");
                }
            }
        }
        table.addRow();
        table.addCell(doklady[i].cislo());
        table.addCell(doklady[i].duzp().FormatDate());
        table.addCell(doklady[i].dppd().FormatDate());
        table.addCell(doklady[i].popis());
        table.addCell(doklady[i].zdroj());
        table.addCell(doklady[i].dic());
        for(size_t k=0; k<dph.GetCount(); k++)
        {
            if(k+1 != dph.GetCount())
            {
                table.addCell(wxString::Format("%.2f", dph[k].zaklad()), wxTEXT_ALIGNMENT_RIGHT);
                celkem[2*k] += dph[k].zaklad();
                table.addCell(wxString::Format("%.2f", dph[k].dph()), wxTEXT_ALIGNMENT_RIGHT);
                celkem[2*k+1] += dph[k].dph();
                table.addCell(wxString::Format("%d%%", dph[k].sazba()), wxTEXT_ALIGNMENT_RIGHT);
            }
            else
            {
                table.addCell(wxString::Format("%.2f", dph[k].zaklad()), wxTEXT_ALIGNMENT_RIGHT);
                celkem[2*k] += dph[k].zaklad();
            }
        }
        table.addCell(wxString::Format("%.2f", doklady[i].celkem()), wxTEXT_ALIGNMENT_RIGHT);
        celkem[2*dph.GetCount()-1] += doklady[i].celkem();
    }
    table.setColumnWidth(3, 30);
    table.refreshCellsSize();
    m_tables->Add(table);
}
