/***************************************************************
 * Name:      veta.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2016
 * License:   GPL3
 **************************************************************/

#include "veta.h"
#include <wx/arrimpl.cpp>

WX_DEFINE_OBJARRAY(dxVetaArray)

Veta::Veta()
    : m_id(-1), m_name(wxEmptyString), m_minOccurs(0), m_maxOccurs(0)
{
}

Veta::Veta(int id, const wxString &name, int minOccurs, int maxOccurs)
    : m_id(id), m_name(name), m_minOccurs(minOccurs), m_maxOccurs(maxOccurs)
{
}

wxString Veta::name() const
{
    return m_name;
}

void Veta::setName(const wxString &name)
{
    m_name = name;
}

int Veta::minOccurs() const
{
    return m_minOccurs;
}

void Veta::setMinOccurs(int minOccurs)
{
    m_minOccurs = minOccurs;
}

int Veta::maxOccurs() const
{
    return m_maxOccurs;
}

void Veta::setMaxOccurs(int maxOccurs)
{
    m_maxOccurs = maxOccurs;
}

wxXmlNode *Veta::createXmlNode() const
{
    wxXmlNode *node = new wxXmlNode(wxXML_ELEMENT_NODE, m_name);
    for(size_t i=0; i<m_atributy.GetCount(); i++)
    {
        if(m_atributy[i].isForXml())
        {
            node->AddAttribute(m_atributy[i].name(), m_atributy[i].value());
        }
    }
    return node;
}

dxAtributArray Veta::atributy() const
{
    return m_atributy;
}

void Veta::setAtributy(const dxAtributArray &atributy)
{
    m_atributy = atributy;
}

void Veta::setAtributValue(const wxString &name, const wxString &value)
{
    for(size_t i=0; i<m_atributy.GetCount(); i++)
    {
        if(m_atributy[i].name() == name)
        {
            m_atributy[i].setValue(value);
            break;
        }
    }
}

void Veta::setAtributValue(const wxString &name, int value)
{
    for(size_t i=0; i<m_atributy.GetCount(); i++)
    {
        if(m_atributy[i].name() == name)
        {
            m_atributy[i].setValue(value);
            break;
        }
    }
}

void Veta::setAtributValue(const wxString &name, double value)
{
    for(size_t i=0; i<m_atributy.GetCount(); i++)
    {
        if(m_atributy[i].name() == name)
        {
            m_atributy[i].setValue(value);
            break;
        }
    }
}

void Veta::setAtributValue(const wxString &name, const wxDateTime &value)
{
    for(size_t i=0; i<m_atributy.GetCount(); i++)
    {
        if(m_atributy[i].name() == name)
        {
            m_atributy[i].setValue(value);
            break;
        }
    }
}

wxString Veta::atributValue(const wxString &name) const
{
    for(size_t i=0; i<m_atributy.GetCount(); i++)
    {
        if(m_atributy[i].name() == name)
        {
            return m_atributy[i].value();
        }
    }
    return wxEmptyString;
}

int Veta::atributValueInt(const wxString &name) const
{
    for(size_t i=0; i<m_atributy.GetCount(); i++)
    {
        if(m_atributy[i].name() == name)
        {
            return m_atributy[i].valueInt();
        }
    }
    return 0;
}

double Veta::atributValueDouble(const wxString &name) const
{
    for(size_t i=0; i<m_atributy.GetCount(); i++)
    {
        if(m_atributy[i].name() == name)
        {
            return m_atributy[i].valueDouble();
        }
    }
    return 0.0;
}

wxDateTime Veta::atributValueDatum(const wxString &name) const
{
    for(size_t i=0; i<m_atributy.GetCount(); i++)
    {
        if(m_atributy[i].name() == name)
        {
            return m_atributy[i].valueDatum();
        }
    }
    return wxDateTime();
}

int Veta::id() const
{
    return m_id;
}

void Veta::setId(int id)
{
    m_id = id;
}

bool Veta::isForXml()
{
    for(size_t i=0; i<m_atributy.GetCount(); i++)
    {
        if(m_atributy[i].isForXml())
        {
            return true;
        }
    }
    return false;
}

void Veta::clearData()
{
    for(size_t i=0; i<m_atributy.GetCount(); i++)
    {
        m_atributy[i].clearValue();
    }
}

void Veta::clearAtributValue(const wxString &name)
{
    for(size_t i=0; i<m_atributy.GetCount(); i++)
    {
        if(m_atributy[i].name() == name)
        {
            m_atributy[i].clearValue();
            break;
        }
    }
}
