/***************************************************************
 * Name:      dokumentdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include <wx/fontdata.h>
#include <wx/fontdlg.h>
#include <wx/colordlg.h>
#include "dokumentdialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(DokumentDialog, wxDialog)

BEGIN_EVENT_TABLE(DokumentDialog, wxDialog)
    EVT_BUTTON(wxID_OK, DokumentDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, DokumentDialog::OnCancel)
    EVT_CHOICE(ID_DOKUMENT_FOOTER, DokumentDialog::OnFooter)
    EVT_BUTTON(ID_DOKUMENT_STYLE, DokumentDialog::OnFont)
    EVT_BUTTON(ID_DOKUMENT_FONTCOLOR, DokumentDialog::OnFontColor)
    EVT_BUTTON(ID_DOKUMENT_BACKCOLOR, DokumentDialog::OnBackColor)
END_EVENT_TABLE()

DokumentDialog::DokumentDialog(wxWindow *parent, const Document &doc)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Vlastnosti šablony")), m_document(doc),
        m_element(TextElement(1, "Text", wxPoint(0,0), 600, wxTEXT_ALIGNMENT_LEFT, doc.style()))
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

Document DokumentDialog::document() const
{
    return m_document;
}

void DokumentDialog::setDocument(const Document &document)
{
    m_document = document;
}

void DokumentDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxStaticBoxSizer *box = new wxStaticBoxSizer(wxHORIZONTAL, this, "Okraje");
    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Levý okraj")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_marginLeft = new wxSpinCtrl(this);
    m_marginLeft->SetRange(0,210);
    gridSizer->Add(m_marginLeft, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Horní okraj")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_marginTop = new wxSpinCtrl(this);
    m_marginTop->SetRange(10,297);
    gridSizer->Add(m_marginTop, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pravý okraj")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_marginRight = new wxSpinCtrl(this);
    m_marginRight->SetRange(0,210);
    gridSizer->Add(m_marginRight, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Spodní okraj")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_marginBottom = new wxSpinCtrl(this);
    m_marginBottom->SetRange(10,297);
    gridSizer->Add(m_marginBottom, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(0, 5, wxGBPosition(4,0));
    box->AddSpacer(5);
    box->Add(gridSizer, 1, wxGROW);
    box->AddSpacer(5);
    m_mainSizer->Add(box, 0, wxALL, 5);

    box = new wxStaticBoxSizer(wxHORIZONTAL, this, wxString::FromUTF8("Hlavička"));
    gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Levý text")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_headerLeft = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_headerLeft, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Prostřední text")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_headerCenter = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_headerCenter, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pravý text")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_headerRight = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_headerRight, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Podtržení")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_headerLine = new wxCheckBox(this, wxID_ANY, wxEmptyString);
    gridSizer->Add(m_headerLine, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(0, 5, wxGBPosition(4,0));
    box->AddSpacer(5);
    box->Add(gridSizer, 1, wxGROW);
    box->AddSpacer(5);
    m_mainSizer->Add(box, 0, wxALL, 5);

    box = new wxStaticBoxSizer(wxHORIZONTAL, this, wxString::FromUTF8("Patička"));
    gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Typ"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_footer = new wxChoice(this, ID_DOKUMENT_FOOTER);
    m_footer->Append(wxString::FromUTF8("Číslování stránek"));
    m_footer->Append(wxString::FromUTF8("Vlastní text"));
    m_footer->Append(wxString::FromUTF8("Bez patičky"));
    gridSizer->Add(m_footer, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Vlastní text")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_footerText = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_footerText, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(0, 5, wxGBPosition(2,0));
    box->AddSpacer(5);
    box->Add(gridSizer, 1, wxGROW);
    box->AddSpacer(5);
    m_mainSizer->Add(box, 0, wxALL, 5);

    box = new wxStaticBoxSizer(wxHORIZONTAL, this, wxString::FromUTF8("Výchozí styl"));
    gridSizer = new wxGridBagSizer(5, 5);
    m_fontSelect = new wxButton(this, ID_DOKUMENT_STYLE, "Font");
    gridSizer->Add(m_fontSelect, wxGBPosition(0,0), wxDefaultSpan, wxEXPAND);
    m_fontColorSelect = new wxButton(this, ID_DOKUMENT_FONTCOLOR, "Barva textu");
    gridSizer->Add(m_fontColorSelect, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    m_backColorSelect = new wxButton(this, ID_DOKUMENT_BACKCOLOR, wxString::FromUTF8("Barva pozadí"));
    gridSizer->Add(m_backColorSelect, wxGBPosition(0,2), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(0, 5, wxGBPosition(1,0));
    box->AddSpacer(5);
    box->Add(gridSizer, 1, wxGROW);
    box->AddSpacer(5);
    m_mainSizer->Add(box, 0, wxALL, 5);
    m_preview = new TextCanvas(this);
    m_mainSizer->Add(m_preview, 0, wxALL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void DokumentDialog::fillControls()
{
    m_marginLeft->SetValue(m_document.marginLeft());
    m_marginTop->SetValue(m_document.marginTop());
    m_marginRight->SetValue(m_document.marginRight());
    m_marginBottom->SetValue(m_document.marginBottom());

    m_headerLeft->SetValue(m_document.headerLeftText());
    m_headerCenter->SetValue(m_document.headerCenterText());
    m_headerRight->SetValue(m_document.headerRightText());
    m_headerLine->SetValue(m_document.headerLine());

    m_footer->SetSelection(m_document.footer());
    m_footerText->SetValue(m_document.footerText());
    if(m_document.footer() != dxPrint::F_TEXT) m_footerText->Enable(false);

    setFontText(m_document.style().font());
}

void DokumentDialog::setFontText(const wxFont& font)
{
    wxString text = font.GetFaceName()+wxString::Format(" %d", font.GetPointSize());
    m_element.setValue(text);
    wxScreenDC dc;
    int width, height;
    dc.SetFont(font);
    dc.GetTextExtent(text, &width, &height);
    m_preview->SetMinSize(wxSize(width,height)+wxSize(2*IMAGE_Y_OFFSET,2*IMAGE_Y_OFFSET));
    m_preview->SetSize(wxSize(width,height)+wxSize(2*IMAGE_Y_OFFSET,2*IMAGE_Y_OFFSET));
    m_preview->setElement(m_element);
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    m_preview->Refresh();
}

void DokumentDialog::OnOk(wxCommandEvent& /*event*/)
{
    m_document.setMarginLeft(m_marginLeft->GetValue());
    m_document.setMarginTop(m_marginTop->GetValue());
    m_document.setMarginRight(m_marginRight->GetValue());
    m_document.setMarginBottom(m_marginBottom->GetValue());

    m_document.setHeaderLeftText(m_headerLeft->GetValue());
    m_document.setHeaderCenterText(m_headerCenter->GetValue());
    m_document.setHeaderRightText(m_headerRight->GetValue());
    m_document.setHeaderLine(m_headerLine->IsChecked());

    m_document.setFooter(static_cast<dxPrint::Footer>(m_footer->GetSelection()));
    m_document.setFooterText(m_footerText->GetValue());

    EndModal(wxID_OK);
}

void DokumentDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void DokumentDialog::OnFooter(wxCommandEvent& /*event*/)
{
    if(m_footer->GetSelection() == 1)
    {
        m_footerText->Enable(true);
    }
    else
    {
        m_footerText->Enable(false);
        m_footerText->SetValue("");
    }
}

void DokumentDialog::OnFont(wxCommandEvent& /*event*/)
{
    wxFontData data;
    data.SetInitialFont(m_document.style().font());
    data.SetColour(m_document.style().fontColour());
    wxFontDialog dialog(this, data);
    if(dialog.ShowModal() == wxID_OK)
    {
        wxFontData retData = dialog.GetFontData();
        TextStyle style = m_document.style();
        style.setFont(retData.GetChosenFont());
        style.setFontColour(retData.GetColour());
        m_element.setStyle(style);
        m_document.setStyle(style);
        setFontText(retData.GetChosenFont());
    }
}

void DokumentDialog::OnFontColor(wxCommandEvent& /*event*/)
{
    wxColourData data;
    data.SetColour(m_document.style().fontColour());
    wxColourDialog dialog(this, &data);
    dialog.SetTitle("Vyberte barvu fontu");
    if(dialog.ShowModal() == wxID_OK)
    {
        data = dialog.GetColourData();
        TextStyle style = m_document.style();
        style.setFontColour(data.GetColour());
        m_element.setStyle(style);
        m_document.setStyle(style);
        m_preview->setElement(m_element);
        m_preview->Refresh();
    }
}

void DokumentDialog::OnBackColor(wxCommandEvent& /*event*/)
{
    wxColourData data;
    data.SetColour(m_document.style().backColour());
    wxColourDialog dialog(this, &data);
    dialog.SetTitle(wxString::FromUTF8("Vyberte barvu pozadí"));
    if(dialog.ShowModal() == wxID_OK)
    {
        data = dialog.GetColourData();
        TextStyle style = m_document.style();
        style.setBackColour(data.GetColour());
        m_element.setStyle(style);
        m_document.setStyle(style);
        m_preview->setElement(m_element);
        m_preview->Refresh();
    }
}
