/***************************************************************
 * Name:      imagedialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include "imagedialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(ImageDialog, wxDialog)

BEGIN_EVENT_TABLE(ImageDialog, wxDialog)
    EVT_BUTTON(wxID_OK, ImageDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, ImageDialog::OnCancel)
    EVT_CHOICE(ID_IMAGE_ALIGNMENT, ImageDialog::OnAlignment)
    EVT_BUTTON(ID_IMAGE_PATH, ImageDialog::OnPath)
END_EVENT_TABLE()

//Novy obrazek
ImageDialog::ImageDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Vlastnosti obrázku")), m_element(ImageElement())
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    m_alignment->SetSelection(1);
    m_element.setAlignment(wxTEXT_ALIGNMENT_CENTRE);
}

//Uprava obrazku
ImageDialog::ImageDialog(wxWindow *parent, const ImageElement &element)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Vlastnosti obrázku")), m_element(element)
{
    createControls();
    fillControls();
    m_preview->SetMinSize(m_element.imagePxSize()+wxSize(40,20));
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    m_preview->Refresh();
}

ImageElement ImageDialog::element() const
{
    return m_element;
}

void ImageDialog::setElement(const ImageElement &element)
{
    m_element = element;
}

void ImageDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Sekce"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_section = new wxChoice(this, ID_LINE_SECTION);
    m_section->Append(wxString::FromUTF8("Hlavička 1"));
    m_section->Append(wxString::FromUTF8("Hlavička 2"));
    m_section->Append(wxString::FromUTF8("Hlavička 3"));
    m_section->Append(wxString::FromUTF8("Položky"));
    m_section->Append("Celkem");
    m_section->Append("DPH");
    m_section->Append(wxString::FromUTF8("Patička 3"));
    m_section->Append(wxString::FromUTF8("Patička 2"));
    m_section->Append(wxString::FromUTF8("Patička 1"));
    m_section->Append(wxString::FromUTF8("Žádná"));
    m_section->SetSelection(10);
    gridSizer->Add(m_section, wxGBPosition(0,1), wxGBSpan(1,2), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pozice obrázku - Xová souřadnice")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_posX = new wxSpinCtrl(this);
    m_posX->SetRange(0,210);
    gridSizer->Add(m_posX, wxGBPosition(1,1), wxGBSpan(1,2), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pozice obrázku - Yová souřadnice")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_posY = new wxSpinCtrl(this);
    m_posY->SetRange(0,297);
    gridSizer->Add(m_posY, wxGBPosition(2,1), wxGBSpan(1,2), wxEXPAND);
    m_imageWidthTxt = new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Šířka obrázku"));
    gridSizer->Add(m_imageWidthTxt, wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_imageWidth = new wxSpinCtrl(this);
    m_imageWidth->SetRange(0,210);
    m_imageWidth->SetValue(100);
    gridSizer->Add(m_imageWidth, wxGBPosition(3,1), wxGBSpan(1,2), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Cesta"), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_path = new wxTextCtrl(this, wxID_ANY, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_READONLY);
    gridSizer->Add(m_path, wxGBPosition(4,1), wxDefaultSpan, wxALL|wxEXPAND);
    m_pathSelect = new wxButton(this, ID_IMAGE_PATH, "...");
    m_pathSelect->SetMaxSize(wxSize(35,-1));
    gridSizer->Add(m_pathSelect, wxGBPosition(4,2), wxDefaultSpan);
    m_alignmentTxt = new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Zarovnání"));
    gridSizer->Add(m_alignmentTxt, wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_alignment = new wxChoice(this, ID_IMAGE_ALIGNMENT);
    m_alignment->Append("Doleva");
    m_alignment->Append(wxString::FromUTF8("Na střed"));
    m_alignment->Append("Doprava");
    gridSizer->Add(m_alignment, wxGBPosition(5,1), wxGBSpan(1,2), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Náhled")), wxGBPosition(6,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_preview = new ImageCanvas(this);
    gridSizer->Add(m_preview, wxGBPosition(7,0), wxGBSpan(1,3), wxALL|wxEXPAND);
    gridSizer->SetEmptyCellSize(wxSize(0,0));
    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void ImageDialog::fillControls()
{
    m_posX->SetValue(m_element.pos().x);
    m_posY->SetValue(m_element.pos().y);
    m_imageWidth->SetValue(m_element.width());
    m_path->SetValue(m_element.path());
    m_path->SetToolTip(m_element.path());
    m_alignment->SetSelection(m_element.alignment()-1);
    m_section->SetSelection(m_element.section());
    m_preview->setElement(m_element);
    if(m_element.isqrcode())
    {
        m_imageWidthTxt->Hide();
        m_imageWidth->Hide();
        m_alignmentTxt->Hide();
        m_alignment->Hide();
    }
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void ImageDialog::OnOk(wxCommandEvent& /*event*/)
{
    m_element.setPage(1);
    m_element.setPos(wxPoint(m_posX->GetValue(), m_posY->GetValue()));
    m_element.setWidth(m_imageWidth->GetValue());
    m_element.setAlignment(static_cast<wxTextAttrAlignment>(m_alignment->GetSelection()+1));
    m_element.setPath(m_path->GetValue());
    if(m_path->GetValue() == "{qr-platba}")
    {
        m_element.setIsqrcode(true);
        m_element.setQrcodetxt("SPD*1.0*ACC:CZ2806000000000168540115*AM:336.00*CC:CZK*MSG:PLATBA ZA ZBOZI*X-VS:1234567890");
    }
    else
    {
        m_element.setIsqrcode(false);
    }
    m_element.setSection(static_cast<DOCSECTION>(m_section->GetSelection()));
    EndModal(wxID_OK);
}

void ImageDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void ImageDialog::OnAlignment(wxCommandEvent& /*event*/)
{
    m_preview->SetAlignment(static_cast<wxTextAttrAlignment>((m_alignment->GetSelection()+1)));
    m_preview->Refresh();
}

void ImageDialog::OnPath(wxCommandEvent& /*event*/)
{
    const wxString choices[] = { "Logo firmy", wxString::FromUTF8("Razítko firmy"), "QR Platba", "Ze souboru" } ;
    wxSingleChoiceDialog vyber(this, "Cesta", wxString::FromUTF8("Cesta k obrázku"), WXSIZEOF(choices), choices);
    if(vyber.ShowModal() == wxID_OK)
    {
        if(vyber.GetSelection() == 0)
        {
            setPath("{firma-logo}");
        }
        else if(vyber.GetSelection() == 1)
        {
            setPath("{firma-razitko}");
        }
        else if(vyber.GetSelection() == 2)
        {
            setPath("{qr-platba}");
        }
        else
        {
            wxFileDialog dialog(this, wxString::FromUTF8("Otevřít obrázek"), wxGetHomeDir(), wxEmptyString, wxString::FromUTF8("Soubory obrázků (*.gif;*.png;*.jpg)|*.gif;*.png;*.jpg"),
                                wxFD_OPEN|wxFD_FILE_MUST_EXIST);
            if(dialog.ShowModal() == wxID_OK)
            {
                wxString path = dialog.GetPath();
                if(!path.empty())
                {
                    setPath(path);
                }
            }
        }
    }
}

void ImageDialog::setPath(const wxString &path)
{
    m_path->SetValue(path);
    m_path->SetToolTip(path);
    m_element.setPath(path);
    if(path == "{qr-platba}")
    {
        m_element.setIsqrcode(true);
        m_element.setQrcodetxt("SPD*1.0*ACC:CZ2806000000000168540115*AM:336.00*CC:CZK*MSG:PLATBA ZA ZBOZI*X-VS:1234567890");
        m_imageWidthTxt->Hide();
        m_imageWidth->Hide();
        m_alignmentTxt->Hide();
        m_alignment->Hide();
    }
    else
    {
        m_element.setIsqrcode(false);
        m_imageWidthTxt->Show();
        m_imageWidth->Show();
        m_alignmentTxt->Show();
        m_alignment->Show();
    }
    m_preview->SetMinSize(m_element.imagePxSize()+wxSize(2*IMAGE_X_OFFSET,2*IMAGE_Y_OFFSET));
    m_preview->setElement(m_element);
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}
