/***************************************************************
 * Name:      linedialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include <wx/colordlg.h>
#include "linedialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(LineDialog, wxDialog)

BEGIN_EVENT_TABLE(LineDialog, wxDialog)
    EVT_BUTTON(wxID_OK, LineDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, LineDialog::OnCancel)
    EVT_CHOICE(ID_LINE_STYLE, LineDialog::OnStyle)
    EVT_BUTTON(ID_LINE_COLOR, LineDialog::OnColor)
    EVT_SPINCTRL(ID_LINE_WIDTH, LineDialog::OnWidth)
    EVT_CHOICE(ID_LINE_SECTION, LineDialog::OnSection)
    EVT_CHECKBOX(ID_LINE_ANCHOR, LineDialog::OnAnchor)
    EVT_CHECKBOX(ID_LINE_BORDERBOTTOM, LineDialog::OnBorderBottom)
    EVT_CHECKBOX(ID_LINE_BORDERTOP, LineDialog::OnBorderTop)
END_EVENT_TABLE()

//Nova line
LineDialog::LineDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Vlastnosti čáry")), m_element(LineElement())
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

//Uprava line
LineDialog::LineDialog(wxWindow *parent, const LineElement &element)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Vlastnosti čáry")), m_element(element)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    m_preview->Refresh();
}

LineElement LineDialog::element() const
{
    return m_element;
}

void LineDialog::setElement(const LineElement &element)
{
    m_element = element;
}

void LineDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Sekce"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_section = new wxChoice(this, ID_LINE_SECTION);
    m_section->Append(wxString::FromUTF8("Hlavička 1"));
    m_section->Append(wxString::FromUTF8("Hlavička 2"));
    m_section->Append(wxString::FromUTF8("Hlavička 3"));
    m_section->Append(wxString::FromUTF8("Položky"));
    m_section->Append("Celkem");
    m_section->Append("DPH");
    m_section->Append(wxString::FromUTF8("Patička 3"));
    m_section->Append(wxString::FromUTF8("Patička 2"));
    m_section->Append(wxString::FromUTF8("Patička 1"));
    m_section->Append(wxString::FromUTF8("Žádná"));
    m_section->SetSelection(10);
    gridSizer->Add(m_section, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Počátek čáry - Xová souřadnice")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_startX = new wxSpinCtrl(this);
    m_startX->SetRange(0,210);
    gridSizer->Add(m_startX, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Počátek čáry - Yová souřadnice")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_startY = new wxSpinCtrl(this);
    m_startY->SetRange(0,297);
    gridSizer->Add(m_startY, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Konec čáry - Xová souřadnice")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_endX = new wxSpinCtrl(this);
    m_endX->SetRange(0,210);
    gridSizer->Add(m_endX, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Konec čáry - Yová souřadnice")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_endY = new wxSpinCtrl(this);
    m_endY->SetRange(0,297);
    gridSizer->Add(m_endY, wxGBPosition(4,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Styl čáry")), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_style = new wxChoice(this, ID_LINE_STYLE);
    m_style->Append(wxString::FromUTF8("Plná čára"));
    m_style->Append(wxString::FromUTF8("Tečkovaná čára"));
    m_style->Append(wxString::FromUTF8("Dlouze čárkovaná čára"));
    m_style->Append(wxString::FromUTF8("Krátce čárkovaná čára"));
    m_style->Append(wxString::FromUTF8("Čerchovaná čára"));
    m_style->SetSelection(0);
    gridSizer->Add(m_style, wxGBPosition(5,1), wxDefaultSpan, wxEXPAND);
    m_colorSelect = new wxButton(this, ID_LINE_COLOR, wxString::FromUTF8("Barva čáry"));
    gridSizer->Add(m_colorSelect, wxGBPosition(6,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Šířka čáry")), wxGBPosition(7,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_lineWidth = new wxSpinCtrl(this, ID_LINE_WIDTH);
    m_lineWidth->SetRange(1,20);
    gridSizer->Add(m_lineWidth, wxGBPosition(7,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Přichytit k sekci")), wxGBPosition(8,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_anchor = new wxCheckBox(this, ID_LINE_ANCHOR, wxEmptyString);
    m_anchor->Enable(false);
    gridSizer->Add(m_anchor, wxGBPosition(8,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Horní okraj sekce")), wxGBPosition(9,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_borderTop = new wxCheckBox(this, ID_LINE_BORDERTOP, wxEmptyString);
    m_borderTop->Enable(false);
    gridSizer->Add(m_borderTop, wxGBPosition(9,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Spodní okraj sekce")), wxGBPosition(10,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_borderBottom = new wxCheckBox(this, ID_LINE_BORDERBOTTOM, wxEmptyString);
    m_borderBottom->Enable(false);
    gridSizer->Add(m_borderBottom, wxGBPosition(10,1), wxDefaultSpan, wxEXPAND);
    m_preview = new LineCanvas(this);
    m_preview->SetMinSize(wxSize(250, 40));
    gridSizer->Add(m_preview, wxGBPosition(11,0), wxGBSpan(1,2), wxALL|wxEXPAND);
    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void LineDialog::fillControls()
{
    m_startX->SetValue(m_element.start().x);
    m_startY->SetValue(m_element.start().y);
    m_endX->SetValue(m_element.end().x);
    m_endY->SetValue(m_element.end().y);
    LineStyle style = m_element.lineStyle();
    m_lineWidth->SetValue(style.width());
    m_style->SetSelection(style.style()-100);
    m_preview->setElement(m_element);
    m_section->SetSelection(m_element.section());
    m_anchor->SetValue(m_element.anchor());
    m_borderBottom->SetValue(m_element.borderBottom());
    m_borderTop->SetValue(m_element.borderTop());
    wxCommandEvent dummy;
    OnSection(dummy);
}

void LineDialog::OnOk(wxCommandEvent& /*event*/)
{
    m_element.setPage(1);
    m_element.setStart(wxPoint(m_startX->GetValue(), m_startY->GetValue()));
    m_element.setEnd(wxPoint(m_endX->GetValue(), m_endY->GetValue()));
    LineStyle style = m_element.lineStyle();
    style.setWidth(m_lineWidth->GetValue());
    style.setStyle(static_cast<wxPenStyle>(m_style->GetSelection()+100));
    m_element.setLineStyle(style);
    m_element.setSection(static_cast<DOCSECTION>(m_section->GetSelection()));
    m_element.setAnchor(m_anchor->IsChecked());
    m_element.setBorderBottom(m_borderBottom->IsChecked());
    m_element.setBorderTop(m_borderTop->IsChecked());
    EndModal(wxID_OK);
}

void LineDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void LineDialog::OnStyle(wxCommandEvent& /*event*/)
{
    LineStyle style = m_element.lineStyle();
    style.setStyle(static_cast<wxPenStyle>(m_style->GetSelection()+100));
    m_element.setLineStyle(style);
    m_preview->setElement(m_element);
    m_preview->Refresh();
}

void LineDialog::OnColor(wxCommandEvent& /*event*/)
{
    wxColourData data;
    data.SetColour(m_element.lineStyle().lineColour());
    wxColourDialog dialog(this, &data);
    dialog.SetTitle(wxString::FromUTF8("Vyberte barvu čáry"));
    if(dialog.ShowModal() == wxID_OK)
    {
        data = dialog.GetColourData();
        LineStyle style = m_element.lineStyle();
        style.setLineColour(data.GetColour());
        m_element.setLineStyle(style);
        m_preview->setElement(m_element);
        m_preview->Refresh();
    }
}

void LineDialog::OnWidth(wxSpinEvent& /*event*/)
{
    LineStyle style = m_element.lineStyle();
    style.setWidth(m_lineWidth->GetValue());
    m_element.setLineStyle(style);
    m_preview->setElement(m_element);
    m_preview->Refresh();
}

void LineDialog::OnSection(wxCommandEvent& /*event*/)
{
    if(m_section->GetSelection() == 9)
    {
        m_anchor->Enable(false);
        m_anchor->SetValue(false);
        m_borderBottom->Enable(false);
        m_borderBottom->SetValue(false);
        m_borderTop->Enable(false);
        m_borderTop->SetValue(false);
    }
    else
    {
        m_anchor->Enable(true);
        m_borderBottom->Enable(true);
        m_borderTop->Enable(true);
    }
}

void LineDialog::OnAnchor(wxCommandEvent& /*event*/)
{
    if(m_anchor->IsChecked())
    {
        m_borderBottom->Enable(false);
        m_borderBottom->SetValue(false);
        m_borderTop->Enable(false);
        m_borderTop->SetValue(false);
    }
    else
    {
        m_borderBottom->Enable(true);
        m_borderTop->Enable(true);
    }
}

void LineDialog::OnBorderBottom(wxCommandEvent& /*event*/)
{
    if(m_borderBottom->IsChecked())
    {
        m_anchor->Enable(false);
        m_anchor->SetValue(false);
        m_borderTop->Enable(false);
        m_borderTop->SetValue(false);
    }
    else
    {
        m_anchor->Enable(true);
        m_borderTop->Enable(true);
    }
}

void LineDialog::OnBorderTop(wxCommandEvent& /*event*/)
{
    if(m_borderTop->IsChecked())
    {
        m_anchor->Enable(false);
        m_anchor->SetValue(false);
        m_borderBottom->Enable(false);
        m_borderBottom->SetValue(false);
    }
    else
    {
        m_anchor->Enable(true);
        m_borderBottom->Enable(true);
    }
}
