/***************************************************************
 * Name:      paragraphdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include <wx/fontdata.h>
#include <wx/fontdlg.h>
#include <wx/colordlg.h>
#include "paragraphdialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"
#include "tags.h"

IMPLEMENT_DYNAMIC_CLASS(ParagraphDialog, wxDialog)

BEGIN_EVENT_TABLE(ParagraphDialog, wxDialog)
    EVT_BUTTON(wxID_OK, ParagraphDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, ParagraphDialog::OnCancel)
    EVT_CHOICE(ID_PARAGRAPH_ALIGNMENT, ParagraphDialog::OnAlignment)
    EVT_BUTTON(ID_PARAGRAPH_FONT, ParagraphDialog::OnFont)
    EVT_BUTTON(ID_PARAGRAPH_FONTCOLOR, ParagraphDialog::OnFontColor)
    EVT_BUTTON(ID_PARAGRAPH_BACKCOLOR, ParagraphDialog::OnBackColor)
    EVT_TEXT(ID_PARAGRAPH_TEXT, ParagraphDialog::OnText)
    EVT_SPINCTRL(ID_PARAGRAPH_WIDTH, ParagraphDialog::OnWidth)
    EVT_CHECKBOX(ID_PARAGRAPH_BORDER, ParagraphDialog::OnBorder)
    EVT_CHOICE(ID_PARAGRAPH_BORDERSTYLE, ParagraphDialog::OnBorderStyle)
    EVT_BUTTON(ID_PARAGRAPH_BORDERCOLOR, ParagraphDialog::OnBorderColor)
    EVT_SPINCTRL(ID_PARAGRAPH_BORDERWIDTH, ParagraphDialog::OnBorderWidth)
    EVT_BUTTON(ID_PARAGRAPH_KEYWORD, ParagraphDialog::OnKeyword)
END_EVENT_TABLE()

//Novy text
ParagraphDialog::ParagraphDialog(wxWindow *parent, const TextStyle &style)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Vlastnosti textového odstavce"))
{
    wxArrayString values;
    values.Add("Text1");
    values.Add("Text2");
    m_element = TextParagraphElement(1, values, wxPoint(0,0), 100, wxTEXT_ALIGNMENT_LEFT, style);
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

//Uprava textu
ParagraphDialog::ParagraphDialog(wxWindow *parent, const TextParagraphElement &element)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Vlastnosti textového odstavce")), m_element(element)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

TextParagraphElement ParagraphDialog::element() const
{
    return m_element;
}

void ParagraphDialog::setElement(const TextParagraphElement &element)
{
    m_element = element;
}

void ParagraphDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Sekce"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_section = new wxChoice(this, ID_LINE_SECTION);
    m_section->Append(wxString::FromUTF8("Hlavička 1"));
    m_section->Append(wxString::FromUTF8("Hlavička 2"));
    m_section->Append(wxString::FromUTF8("Hlavička 3"));
    m_section->Append(wxString::FromUTF8("Položky"));
    m_section->Append("Celkem");
    m_section->Append("DPH");
    m_section->Append(wxString::FromUTF8("Patička 3"));
    m_section->Append(wxString::FromUTF8("Patička 2"));
    m_section->Append(wxString::FromUTF8("Patička 1"));
    m_section->Append(wxString::FromUTF8("Žádná"));
    m_section->SetSelection(10);
    gridSizer->Add(m_section, wxGBPosition(0,1), wxGBSpan(1,3), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pozice odstavce - Xová souřadnice")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_posX = new wxSpinCtrl(this);
    m_posX->SetRange(0,210);
    gridSizer->Add(m_posX, wxGBPosition(1,1), wxGBSpan(1,3), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Pozice odstavce - Yová souřadnice")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_posY = new wxSpinCtrl(this);
    m_posY->SetRange(0,297);
    gridSizer->Add(m_posY, wxGBPosition(2,1), wxGBSpan(1,3), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Šířka odstavce")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_paragraphWidth = new wxSpinCtrl(this, ID_PARAGRAPH_WIDTH);
    m_paragraphWidth->SetRange(0,210);
    gridSizer->Add(m_paragraphWidth, wxGBPosition(3,1), wxGBSpan(1,3), wxEXPAND);
    m_text = new wxTextCtrl(this, ID_PARAGRAPH_TEXT, wxEmptyString, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE);
    m_text->SetMinSize(wxSize(-1,100));
    gridSizer->Add(m_text, wxGBPosition(4,0), wxGBSpan(1,3), wxEXPAND);
    m_keyword = new wxButton(this, ID_PARAGRAPH_KEYWORD, "...");
    m_keyword->SetToolTip(wxString::FromUTF8("Vložit klíčové slovo"));
    gridSizer->Add(m_keyword, wxGBPosition(4,3), wxDefaultSpan);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Zarovnání")), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_alignment = new wxChoice(this, ID_PARAGRAPH_ALIGNMENT);
    m_alignment->Append("Doleva");
    m_alignment->Append(wxString::FromUTF8("Na střed"));
    m_alignment->Append("Doprava");
    gridSizer->Add(m_alignment, wxGBPosition(5,1), wxGBSpan(1,3), wxEXPAND);
    m_fontSelect = new wxButton(this, ID_PARAGRAPH_FONT, "Font");
    gridSizer->Add(m_fontSelect, wxGBPosition(6,1), wxDefaultSpan, wxEXPAND);
    m_fontColorSelect = new wxButton(this, ID_PARAGRAPH_FONTCOLOR, "Barva textu");
    gridSizer->Add(m_fontColorSelect, wxGBPosition(6,2), wxDefaultSpan, wxEXPAND);
    m_backColorSelect = new wxButton(this, ID_PARAGRAPH_BACKCOLOR, wxString::FromUTF8("Barva pozadí"));
    gridSizer->Add(m_backColorSelect, wxGBPosition(6,3), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Okraj"), wxGBPosition(7,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_border = new wxCheckBox(this, ID_PARAGRAPH_BORDER, wxEmptyString);
    gridSizer->Add(m_border, wxGBPosition(7,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Styl okraje"), wxGBPosition(8,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_borderStyle = new wxChoice(this, ID_PARAGRAPH_BORDERSTYLE);
    m_borderStyle->Append(wxString::FromUTF8("Plná čára"));
    m_borderStyle->Append(wxString::FromUTF8("Tečkovaná čára"));
    m_borderStyle->Append(wxString::FromUTF8("Dlouze čárkovaná čára"));
    m_borderStyle->Append(wxString::FromUTF8("Krátce čárkovaná čára"));
    m_borderStyle->Append(wxString::FromUTF8("Čerchovaná čára"));
    m_borderStyle->SetSelection(0);
    m_borderStyle->Enable(false);
    gridSizer->Add(m_borderStyle, wxGBPosition(8,1), wxGBSpan(1,3), wxEXPAND);
    m_borderColor = new wxButton(this, ID_PARAGRAPH_BORDERCOLOR, "Barva okraje");
    m_borderColor->Enable(false);
    gridSizer->Add(m_borderColor, wxGBPosition(9,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Šířka okraje")), wxGBPosition(10,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_borderWidth = new wxSpinCtrl(this, ID_PARAGRAPH_BORDERWIDTH);
    m_borderWidth->SetRange(1,20);
    m_borderWidth->Enable(false);
    gridSizer->Add(m_borderWidth, wxGBPosition(10,1), wxGBSpan(1,3), wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Náhled")), wxGBPosition(11,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_mainSizer->Add(gridSizer, 1, wxLEFT|wxTOP|wxRIGHT, 5);
    m_preview = new ParagraphCanvas(this);
    m_mainSizer->Add(m_preview, 0, wxALL, 2);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void ParagraphDialog::fillControls()
{
    m_posX->SetValue(m_element.pos().x);
    m_posY->SetValue(m_element.pos().y);
    m_paragraphWidth->SetValue(m_element.width());
    m_text->SetValue(wxJoin(m_element.value(), '\n'));
    m_alignment->SetSelection(m_element.alignment()-1);
    m_border->SetValue(m_element.border());
    wxCommandEvent ev;
    OnBorder(ev);
    LineStyle style = m_element.borderStyle();
    m_borderWidth->SetValue(style.width());
    m_borderStyle->SetSelection(style.style()-100);
    setPreviewSize();
    m_preview->setElement(m_element);
    m_section->SetSelection(m_element.section());
}

void ParagraphDialog::OnOk(wxCommandEvent& /*event*/)
{
    m_element.setPage(1);
    m_element.setPos(wxPoint(m_posX->GetValue(), m_posY->GetValue()));
    m_element.setWidth(m_paragraphWidth->GetValue());
    m_element.setAlignment(static_cast<wxTextAttrAlignment>(m_alignment->GetSelection()+1));
    m_element.setValue(wxSplit(m_text->GetValue(), '\n'));
    m_element.setBorder(m_border->IsChecked());
    LineStyle style = m_element.borderStyle();
    style.setWidth(m_borderWidth->GetValue());
    style.setStyle(static_cast<wxPenStyle>(m_borderStyle->GetSelection()+100));
    m_element.setBorderStyle(style);
    m_element.setSection(static_cast<DOCSECTION>(m_section->GetSelection()));
    EndModal(wxID_OK);
}

void ParagraphDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void ParagraphDialog::OnAlignment(wxCommandEvent& /*event*/)
{
    m_element.setAlignment(static_cast<wxTextAttrAlignment>(m_alignment->GetSelection()+1));
    m_preview->SetAlignment(static_cast<wxTextAttrAlignment>(m_alignment->GetSelection()+1));
    m_preview->Refresh();
}

void ParagraphDialog::OnFont(wxCommandEvent& /*event*/)
{
    wxFontData data;
    data.SetInitialFont(m_element.style().font());
    data.SetColour(m_element.style().fontColour());
    wxFontDialog dialog(this, data);
    if(dialog.ShowModal() == wxID_OK)
    {
        wxFontData retData = dialog.GetFontData();
        TextStyle style = m_element.style();
        style.setFont(retData.GetChosenFont());
        style.setFontColour(retData.GetColour());
        m_element.setStyle(style);
        setPreviewSize();
        m_preview->setElement(m_element);
        m_mainSizer->Layout();
        m_mainSizer->Fit(this);
        m_mainSizer->SetSizeHints(this);
        m_preview->Refresh();
    }
}

void ParagraphDialog::OnFontColor(wxCommandEvent& /*event*/)
{
    wxColourData data;
    data.SetColour(m_element.style().fontColour());
    wxColourDialog dialog(this, &data);
    dialog.SetTitle("Vyberte barvu fontu");
    if(dialog.ShowModal() == wxID_OK)
    {
        data = dialog.GetColourData();
        TextStyle style = m_element.style();
        style.setFontColour(data.GetColour());
        m_element.setStyle(style);
        m_preview->setElement(m_element);
        m_preview->Refresh();
    }
}

void ParagraphDialog::OnBackColor(wxCommandEvent& /*event*/)
{
    wxColourData data;
    data.SetColour(m_element.style().backColour());
    wxColourDialog dialog(this, &data);
    dialog.SetTitle(wxString::FromUTF8("Vyberte barvu pozadí"));
    if(dialog.ShowModal() == wxID_OK)
    {
        data = dialog.GetColourData();
        TextStyle style = m_element.style();
        style.setBackColour(data.GetColour());
        m_element.setStyle(style);
        m_preview->setElement(m_element);
        m_preview->Refresh();
    }
}

void ParagraphDialog::OnText(wxCommandEvent& /*event*/)
{
    m_element.setValue(wxSplit(m_text->GetValue(), '\n'));
    setPreviewSize();
    m_preview->setElement(m_element);
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    m_preview->Refresh();
}

void ParagraphDialog::OnWidth(wxSpinEvent& /*event*/)
{
    m_element.setWidth(m_paragraphWidth->GetValue());
    setPreviewSize();
    m_preview->setElement(m_element);
    m_mainSizer->Layout();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    m_preview->Refresh();
}

void ParagraphDialog::OnBorder(wxCommandEvent& /*event*/)
{
    if(m_border->IsChecked())
    {
        m_borderStyle->Enable(true);
        m_borderWidth->Enable(true);
        m_borderColor->Enable(true);
    }
    else
    {
        m_borderStyle->Enable(false);
        m_borderWidth->Enable(false);
        m_borderColor->Enable(false);
    }
    m_element.setBorder(m_border->IsChecked());
    m_preview->setElement(m_element);
    m_preview->Refresh();
}

void ParagraphDialog::OnBorderStyle(wxCommandEvent& /*event*/)
{
    LineStyle style = m_element.borderStyle();
    style.setStyle(static_cast<wxPenStyle>(m_borderStyle->GetSelection()+100));
    m_element.setBorderStyle(style);
    m_preview->setElement(m_element);
    m_preview->Refresh();
}

void ParagraphDialog::OnBorderWidth(wxSpinEvent& /*event*/)
{
    LineStyle style = m_element.borderStyle();
    style.setWidth(m_borderWidth->GetValue());
    m_element.setBorderStyle(style);
    m_preview->setElement(m_element);
    m_preview->Refresh();
}

void ParagraphDialog::OnBorderColor(wxCommandEvent& /*event*/)
{
    wxColourData data;
    data.SetColour(m_element.borderStyle().lineColour());
    wxColourDialog dialog(this, &data);
    dialog.SetTitle("Vyberte barvu okraje");
    if(dialog.ShowModal() == wxID_OK)
    {
        data = dialog.GetColourData();
        LineStyle style = m_element.borderStyle();
        style.setLineColour(data.GetColour());
        m_element.setBorderStyle(style);
        m_preview->setElement(m_element);
        m_preview->Refresh();
    }
}

void ParagraphDialog::setPreviewSize()
{
    wxScreenDC dc;
    int width = dxutils::mmToPx(m_paragraphWidth->GetValue(), &dc, true);
    m_preview->SetMinSize(wxSize(width,m_element.height(true))+wxSize(2*IMAGE_Y_OFFSET,2*IMAGE_Y_OFFSET));
    m_preview->SetSize(wxSize(width,m_element.height(true))+wxSize(2*IMAGE_Y_OFFSET,2*IMAGE_Y_OFFSET));
}

void ParagraphDialog::OnKeyword(wxCommandEvent& /*event*/)
{
    wxArrayString choices;
    for(size_t i=0; i<numFakturaTags; i++)
    {
        choices.Add(fakturaTags[i]);
    }
    for(size_t i=0; i<numFirmaTags; i++)
    {
        choices.Add(firmaTags[i]);
    }
    for(size_t i=0; i<numKontaktTags; i++)
    {
        choices.Add(kontaktTags[i]);
    }
    wxSingleChoiceDialog vyber(this, wxString::FromUTF8("Vyberte klíčové slovo"), wxString::FromUTF8("Klíčové slovo"), choices);
    if(vyber.ShowModal() == wxID_OK)
    {
        m_text->AppendText(vyber.GetStringSelection());
    }
}
