/***************************************************************
 * Name:      rectdialog.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include <wx/colordlg.h>
#include "rectdialog.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

IMPLEMENT_DYNAMIC_CLASS(RectDialog, wxDialog)

BEGIN_EVENT_TABLE(RectDialog, wxDialog)
    EVT_BUTTON(wxID_OK, RectDialog::OnOk)
    EVT_BUTTON(wxID_CANCEL, RectDialog::OnCancel)
    EVT_CHOICE(ID_RECT_STYLE, RectDialog::OnStyle)
    EVT_BUTTON(ID_RECT_COLOR, RectDialog::OnColor)
    EVT_SPINCTRL(ID_RECT_WIDTH, RectDialog::OnWidth)
    EVT_CHOICE(ID_RECT_SECTION, RectDialog::OnSection)
    EVT_SPINCTRL(ID_RECT_FILLET, RectDialog::OnFillet)
END_EVENT_TABLE()

//Novy rectangle
RectDialog::RectDialog(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Vlastnosti obdelníku")), m_element(RectangleElement())
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

//Uprava rectangle
RectDialog::RectDialog(wxWindow *parent, const RectangleElement &element)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Vlastnosti obdelníku")), m_element(element)
{
    createControls();
    fillControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
    m_preview->Refresh();
}

RectangleElement RectDialog::element() const
{
    return m_element;
}

void RectDialog::setElement(const RectangleElement &element)
{
    m_element = element;
}

void RectDialog::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Sekce"), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_section = new wxChoice(this, ID_RECT_SECTION);
    m_section->Append(wxString::FromUTF8("Hlavička 1"));
    m_section->Append(wxString::FromUTF8("Hlavička 2"));
    m_section->Append(wxString::FromUTF8("Hlavička 3"));
    m_section->Append(wxString::FromUTF8("Položky"));
    m_section->Append("Celkem");
    m_section->Append("DPH");
    m_section->Append(wxString::FromUTF8("Patička 3"));
    m_section->Append(wxString::FromUTF8("Patička 2"));
    m_section->Append(wxString::FromUTF8("Patička 1"));
    m_section->Append(wxString::FromUTF8("Žádná"));
    m_section->SetSelection(10);
    gridSizer->Add(m_section, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Počátek obdelníku - Xová souřadnice")), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_startX = new wxSpinCtrl(this);
    m_startX->SetRange(0,210);
    gridSizer->Add(m_startX, wxGBPosition(1,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Počátek obdelníku - Yová souřadnice")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_startY = new wxSpinCtrl(this);
    m_startY->SetRange(0,297);
    gridSizer->Add(m_startY, wxGBPosition(2,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Šířka obdelníku")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_endX = new wxSpinCtrl(this);
    m_endX->SetRange(0,210);
    gridSizer->Add(m_endX, wxGBPosition(3,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Výška obdelníku")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_endY = new wxSpinCtrl(this);
    m_endY->SetRange(0,297);
    gridSizer->Add(m_endY, wxGBPosition(4,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Styl čáry")), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_style = new wxChoice(this, ID_RECT_STYLE);
    m_style->Append(wxString::FromUTF8("Plná čára"));
    m_style->Append(wxString::FromUTF8("Tečkovaná čára"));
    m_style->Append(wxString::FromUTF8("Dlouze čárkovaná čára"));
    m_style->Append(wxString::FromUTF8("Krátce čárkovaná čára"));
    m_style->Append(wxString::FromUTF8("Čerchovaná čára"));
    m_style->SetSelection(0);
    gridSizer->Add(m_style, wxGBPosition(5,1), wxDefaultSpan, wxEXPAND);
    m_colorSelect = new wxButton(this, ID_RECT_COLOR, wxString::FromUTF8("Barva čáry"));
    gridSizer->Add(m_colorSelect, wxGBPosition(6,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Šířka čáry")), wxGBPosition(7,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_rectWidth = new wxSpinCtrl(this, ID_RECT_WIDTH);
    m_rectWidth->SetRange(1,20);
    gridSizer->Add(m_rectWidth, wxGBPosition(7,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Přichytit k sekci")), wxGBPosition(8,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_anchor = new wxCheckBox(this, ID_LINE_ANCHOR, wxEmptyString);
    m_anchor->Enable(false);
    gridSizer->Add(m_anchor, wxGBPosition(8,1), wxDefaultSpan, wxEXPAND);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Zaoblení")), wxGBPosition(9,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_fillet = new wxSpinCtrl(this, ID_RECT_FILLET);
    m_fillet->SetRange(0,40);
    gridSizer->Add(m_fillet, wxGBPosition(9,1), wxDefaultSpan, wxEXPAND);
    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);
    m_preview = new RectangleCanvas(this);
    m_preview->SetMinSize(wxSize(2*IMAGE_X_OFFSET+360,2*IMAGE_Y_OFFSET+160));
    m_mainSizer->Add(m_preview, 0, wxALL, 2);

    wxStdDialogButtonSizer *btnSizer = new wxStdDialogButtonSizer();
    wxButton* okButton = new wxButton(this, wxID_OK, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(okButton);
    wxButton* cancelButton = new wxButton(this, wxID_CANCEL, "", wxDefaultPosition, wxDefaultSize, 0);
    btnSizer->AddButton(cancelButton);
    btnSizer->Realize();
    m_mainSizer->Add(btnSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void RectDialog::fillControls()
{
    m_startX->SetValue(m_element.start().x);
    m_startY->SetValue(m_element.start().y);
    m_endX->SetValue(m_element.size().x);
    m_endY->SetValue(m_element.size().y);
    LineStyle style = m_element.lineStyle();
    m_rectWidth->SetValue(style.width());
    m_style->SetSelection(style.style()-100);
    m_preview->setElement(m_element);
    m_section->SetSelection(m_element.section());
    m_anchor->SetValue(m_element.anchor());
    m_fillet->SetValue(m_element.fillet());
    wxCommandEvent dummy;
    OnSection(dummy);
}

void RectDialog::OnOk(wxCommandEvent& /*event*/)
{
    m_element.setPage(1);
    m_element.setStart(wxPoint(m_startX->GetValue(), m_startY->GetValue()));
    m_element.setSize(wxSize(m_endX->GetValue(), m_endY->GetValue()));
    LineStyle style = m_element.lineStyle();
    style.setWidth(m_rectWidth->GetValue());
    style.setStyle(static_cast<wxPenStyle>(m_style->GetSelection()+100));
    m_element.setLineStyle(style);
    m_element.setSection(static_cast<DOCSECTION>(m_section->GetSelection()));
    m_element.setAnchor(m_anchor->IsChecked());
    m_element.setFillet(m_fillet->GetValue());
    EndModal(wxID_OK);
}

void RectDialog::OnCancel(wxCommandEvent& /*event*/)
{
    EndModal(wxID_CANCEL);
}

void RectDialog::OnStyle(wxCommandEvent& /*event*/)
{
    LineStyle style = m_element.lineStyle();
    style.setStyle(static_cast<wxPenStyle>(m_style->GetSelection()+100));
    m_element.setLineStyle(style);
    m_preview->setElement(m_element);
    m_preview->Refresh();
}

void RectDialog::OnColor(wxCommandEvent& /*event*/)
{
    wxColourData data;
    data.SetColour(m_element.lineStyle().lineColour());
    wxColourDialog dialog(this, &data);
    dialog.SetTitle(wxString::FromUTF8("Vyberte barvu čáry"));
    if(dialog.ShowModal() == wxID_OK)
    {
        data = dialog.GetColourData();
        LineStyle style = m_element.lineStyle();
        style.setLineColour(data.GetColour());
        m_element.setLineStyle(style);
        m_preview->setElement(m_element);
        m_preview->Refresh();
    }
}

void RectDialog::OnWidth(wxSpinEvent& /*event*/)
{
    LineStyle style = m_element.lineStyle();
    style.setWidth(m_rectWidth->GetValue());
    m_element.setLineStyle(style);
    m_preview->setElement(m_element);
    m_preview->Refresh();
}

void RectDialog::OnSection(wxCommandEvent& /*event*/)
{
    if(m_section->GetSelection() == 9)
    {
        m_anchor->Enable(false);
        m_anchor->SetValue(false);
    }
    else
    {
        m_anchor->Enable(true);
    }
}

void RectDialog::OnFillet(wxSpinEvent &/*event*/)
{
    m_element.setFillet(m_fillet->GetValue());
    m_preview->setElement(m_element);
    m_preview->Refresh();
}
