/***************************************************************
 * Name:      templateeditor.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2015
 * License:   GPL3
 **************************************************************/

#include <wx/dcbuffer.h>
#include "templateeditor.h"
#include "dxdefs.h"
#include "utils/dxutils.h"
#include "dokumentdialog.h"
#include "linedialog.h"
#include "textdialog.h"
#include "imagedialog.h"
#include "templateprint.h"
#include "paragraphdialog.h"
#include "rowdialog.h"
#include "rectdialog.h"
#include "model/dbmodel.h"
#include "resources/images/open.xpm"
#include "resources/images/save.xpm"
#include "resources/images/saveas.xpm"
#include "resources/images/seznam.xpm"
#include "resources/images/preview.xpm"
#include "resources/images/add.xpm"
#include "resources/images/edit.xpm"
#include "resources/images/copy.xpm"
#include "resources/images/delete.xpm"
#include "resources/images/clear.xpm"

IMPLEMENT_DYNAMIC_CLASS(TemplateEditor, wxDialog)

BEGIN_EVENT_TABLE(TemplateEditor, wxDialog)
    EVT_MENU(wxID_OPEN, TemplateEditor::OnOpen)
    EVT_MENU(wxID_SAVE, TemplateEditor::OnSave)
    EVT_MENU(wxID_SAVEAS, TemplateEditor::OnSaveAs)
    EVT_MENU(wxID_CLEAR, TemplateEditor::OnClear)
    EVT_MENU(wxID_PREVIEW, TemplateEditor::OnPreview)
    EVT_CLOSE(TemplateEditor::OnClose)
    EVT_LISTBOX_DCLICK(ID_TEMPLATEEDITOR_LIST, TemplateEditor::OnItemActivated)
    EVT_LISTBOX(ID_TEMPLATEEDITOR_LIST, TemplateEditor::OnItemSelected)
    EVT_MENU(ID_TEMPLATEEDITOR_ADD, TemplateEditor::OnAddElement)
    EVT_MENU(ID_TEMPLATEEDITOR_EDIT, TemplateEditor::OnEditElement)
    EVT_MENU(ID_TEMPLATEEDITOR_COPY, TemplateEditor::OnCopyElement)
    EVT_MENU(ID_TEMPLATEEDITOR_DELETE, TemplateEditor::OnDeleteElement)
    EVT_MENU(ID_TEMPLATEEDITOR_DOCUMENT, TemplateEditor::OnDocumentProperties)
END_EVENT_TABLE()

TemplateEditor::TemplateEditor(wxWindow *parent)
    : wxDialog(parent, wxID_ANY, wxString::FromUTF8("Editor šablon"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE|wxRESIZE_BORDER), m_selectedId(-1),
      m_edited(false), m_filename("")
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    m_bar = new dxToolBar(this);
    m_bar->AddTool(wxID_OPEN, wxEmptyString, wxBitmap(open_xpm), dxutils::GreyScaledIcon(open_xpm), wxITEM_NORMAL, wxString::FromUTF8("Otevřít šablonu"));
    m_bar->AddTool(wxID_SAVE, wxEmptyString, wxBitmap(save_xpm), dxutils::GreyScaledIcon(save_xpm), wxITEM_NORMAL, wxString::FromUTF8("Uložit"));
    m_bar->AddTool(wxID_SAVEAS, wxEmptyString, wxBitmap(saveas_xpm), dxutils::GreyScaledIcon(saveas_xpm), wxITEM_NORMAL, wxString::FromUTF8("Uložit jako"));
    m_bar->AddTool(wxID_CLEAR, wxEmptyString, wxBitmap(clear_xpm), dxutils::GreyScaledIcon(clear_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vyčistit šablonu"));
    m_bar->AddSeparator();
    m_bar->AddTool(wxID_PREVIEW, wxEmptyString, wxBitmap(preview_xpm), dxutils::GreyScaledIcon(preview_xpm), wxITEM_NORMAL, wxString::FromUTF8("Náhled šablony"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_TEMPLATEEDITOR_DOCUMENT, wxEmptyString, wxBitmap(seznam_xpm), dxutils::GreyScaledIcon(seznam_xpm), wxITEM_NORMAL, wxString::FromUTF8("Vlastnosti šablony"));
    m_bar->AddSeparator();
    m_bar->AddTool(ID_TEMPLATEEDITOR_ADD, wxEmptyString, wxBitmap(add_xpm), dxutils::GreyScaledIcon(add_xpm), wxITEM_NORMAL, wxString::FromUTF8("Přidat prvek"));
    m_bar->AddTool(ID_TEMPLATEEDITOR_EDIT, wxEmptyString, wxBitmap(edit_xpm), dxutils::GreyScaledIcon(edit_xpm), wxITEM_NORMAL, "Upravit prvek");
    m_bar->AddTool(ID_TEMPLATEEDITOR_COPY, wxEmptyString, wxBitmap(copy_xpm), dxutils::GreyScaledIcon(copy_xpm), wxITEM_NORMAL, wxString::FromUTF8("Kopírovat prvek"));
    m_bar->AddTool(ID_TEMPLATEEDITOR_DELETE, wxEmptyString, wxBitmap(delete_xpm), dxutils::GreyScaledIcon(delete_xpm), wxITEM_NORMAL, "Smazat prvek");
    m_bar->Realize();
    m_mainSizer->Add(m_bar, 0, wxALL|wxEXPAND, 5);
    wxBoxSizer *sizer1 = new wxBoxSizer(wxHORIZONTAL);
    m_canvas = new wxPanel(this, wxID_ANY, wxDefaultPosition, wxSize(423, 2*PAGE_HEIGHT+3), wxDOUBLE_BORDER|wxTAB_TRAVERSAL);
    m_canvas->SetBackgroundStyle(wxBG_STYLE_PAINT);
    sizer1->Add(m_canvas, 1, wxALL|wxEXPAND, 5);
    wxBoxSizer *sizer2 = new wxBoxSizer(wxVERTICAL);
    sizer2->Add(new wxStaticText(this, wxID_ANY, "Prvky:"), 0, wxALL|wxEXPAND, 5);
    m_list = new wxListBox(this, ID_TEMPLATEEDITOR_LIST, wxDefaultPosition, wxSize(150, -1), 0, nullptr, wxLB_SINGLE);
    m_list->SetMinSize(wxSize(150, -1));
    sizer2->Add(m_list, 1, wxALL|wxEXPAND, 5);
    sizer1->Add(sizer2, 1, wxALL|wxEXPAND, 2);
    m_mainSizer->Add(sizer1, 1, wxALL|wxEXPAND, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);

    fillList();

    m_list->Connect(wxEVT_RIGHT_DOWN, wxMouseEventHandler(TemplateEditor::OnItemRightClick), nullptr, this);
    m_canvas->Connect(wxEVT_PAINT, wxPaintEventHandler(TemplateEditor::OnPaint), nullptr, this);
    this->Connect(wxEVT_SIZE, wxSizeEventHandler(TemplateEditor::OnSize));
}

void TemplateEditor::OnOpen(wxCommandEvent& /*event*/)
{
    const wxString choices[] = { wxString::FromUTF8("Výchozí šablona programu"), "Ze souboru" } ;
    wxSingleChoiceDialog vyber(this, wxString::FromUTF8("Vyberte typ šablony"), wxString::FromUTF8("Typ šablony"), WXSIZEOF(choices), choices);
    if(vyber.ShowModal() == wxID_OK)
    {
        if(vyber.GetSelection() == 0)
        {
            if(idbmodel->isFirmaPlatce()) createPlatceTemplate();
            else createNeplataceTemplate();
        }
        else
        {
            wxFileDialog dialog(m_parent, wxString::FromUTF8("Otevřít šablonu"), wxGetHomeDir(), wxEmptyString, "XML soubor (*.xml)|*.xml", wxFD_OPEN|wxFD_FILE_MUST_EXIST);
            if(dialog.ShowModal() == wxID_OK)
            {
                wxString path = dialog.GetPath();
                if(!path.empty())
                {
                    m_document.readFromXml(path);
                    m_filename = path;
                }
            }
        }
    }
    fillList();
}

void TemplateEditor::OnSave(wxCommandEvent& event)
{
    if(m_filename.empty())
    {
        OnSaveAs(event);
    }
    else
    {
        m_document.saveToXml(m_filename);
        m_edited = false;
    }
}

void TemplateEditor::OnSaveAs(wxCommandEvent& /*event*/)
{
    wxFileDialog dialog(m_parent, wxString::FromUTF8("Uložit šablonu jako"), wxGetHomeDir(), wxEmptyString, "XML soubor (*.xml)|*.xml", wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dialog.ShowModal() == wxID_OK)
    {
        wxString path = dialog.GetPath();
        if(!path.empty())
        {
            dxutils::correctEmptyFileExt("xml", path);
            m_document.saveToXml(path);
            m_filename = path;
            m_edited = false;
        }
    }
}

void TemplateEditor::OnClear(wxCommandEvent& /*event*/)
{
    if(m_document.getElementsCount()) m_edited = true;
    m_document.clearElements();
    fillList();
}

void TemplateEditor::OnPreview(wxCommandEvent& /*event*/)
{
    TemplatePrint dialog(this, wxString::FromUTF8("Šablona"), m_document);
    dialog.Preview();
}

void TemplateEditor::OnClose(wxCloseEvent& /*event*/)
{
    if(m_edited)
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Šablona byla změněna, chcete ji uložit?"), "Dotaz", wxCENTRE|wxYES_NO|wxICON_QUESTION);
        if(dialog.ShowModal() == wxID_YES)
        {
            wxCommandEvent ev;
            OnSave(ev);
        }
    }
    EndModal(wxID_OK);
}

void TemplateEditor::OnItemRightClick(wxMouseEvent &event)
{
    m_list->Select(m_list->HitTest(event.GetPosition()));
    setSelectedId(m_list->HitTest(event.GetPosition()));
    wxMenu popup;
    popup.Append(ID_TEMPLATEEDITOR_ADD, wxString::FromUTF8("Přidat prvek"));
    if(m_selectedId != -1)
    {
        popup.AppendSeparator();
        popup.Append(ID_TEMPLATEEDITOR_EDIT, "Upravit prvek");
        popup.Append(ID_TEMPLATEEDITOR_COPY, wxString::FromUTF8("Kopírovat prvek"));
        popup.Append(ID_TEMPLATEEDITOR_DELETE, "Smazat prvek");
    }
    PopupMenu(&popup, event.GetPosition()+m_list->GetPosition());
}

void TemplateEditor::OnItemActivated(wxCommandEvent &event)
{
    setSelectedId(event.GetInt());
    OnEditElement(event);
}

void TemplateEditor::OnItemSelected(wxCommandEvent &event)
{
    setSelectedId(event.GetInt());
}

void TemplateEditor::OnEditElement(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    Element *element = m_document.element(static_cast<size_t>(m_selectedId));
    switch(element->elementType()) {
    case IMAGE:
    {
        ImageElement *elp = static_cast<ImageElement*>(m_document.element(static_cast<size_t>(m_selectedId)));
        ImageDialog dialog(this, *elp);
        if(dialog.ShowModal() == wxID_OK)
        {
            ImageElement el = dialog.element();
            elp->setPos(el.pos());
            elp->setPath(el.path());
            elp->setIsqrcode(el.isqrcode());
            elp->setQrcodetxt(el.qrcodetxt());
            elp->setWidth(el.width());
            elp->setAlignment(el.alignment());
            elp->setSection(el.section());
            m_edited = true;
            fillList();
        }
        break;
    }
    case LINE:
    {
        LineElement *elp = static_cast<LineElement*>(m_document.element(static_cast<size_t>(m_selectedId)));
        LineDialog dialog(this, *elp);
        if(dialog.ShowModal() == wxID_OK)
        {
            LineElement el = dialog.element();
            elp->setStart(el.start());
            elp->setEnd(el.end());
            elp->setLineStyle(el.lineStyle());
            elp->setSection(el.section());
            elp->setAnchor(el.anchor());
            elp->setBorderBottom(el.borderBottom());
            elp->setBorderTop(el.borderTop());
            m_edited = true;
            fillList();
        }
        break;
    }
    case RECTANGLE:
    {
        RectangleElement *elp = static_cast<RectangleElement*>(m_document.element(static_cast<size_t>(m_selectedId)));
        RectDialog dialog(this, *elp);
        if(dialog.ShowModal() == wxID_OK)
        {
            RectangleElement el = dialog.element();
            elp->setStart(el.start());
            elp->setSize(el.size());
            elp->setLineStyle(el.lineStyle());
            elp->setSection(el.section());
            elp->setAnchor(el.anchor());
            elp->setFillet(el.fillet());
            m_edited = true;
            fillList();
        }
        break;
    }
    case TABLEROW:
    {
        TableRowElement *elp = static_cast<TableRowElement*>(m_document.element(static_cast<size_t>(m_selectedId)));
        RowDialog dialog(this, *elp);
        if(dialog.ShowModal() == wxID_OK)
        {
            TableRowElement el = dialog.element();
            elp->setPos(el.pos());
            elp->setWidth(el.width());
            elp->setValue(el.value());
            elp->setAlignment(el.alignment());
            elp->setStyle(el.style());
            elp->setBorder(el.border());
            elp->setBorderStyle(el.borderStyle());
            elp->setSection(el.section());
            m_edited = true;
            fillList();
        }
        break;
    }
    case TEXT:
    {
        TextElement *elp = static_cast<TextElement*>(m_document.element(static_cast<size_t>(m_selectedId)));
        TextDialog dialog(this, *elp);
        if(dialog.ShowModal() == wxID_OK)
        {
            TextElement el = dialog.element();
            elp->setPos(el.pos());
            elp->setWidth(el.width());
            elp->setValue(el.value());
            elp->setAlignment(el.alignment());
            elp->setStyle(el.style());
            elp->setSection(el.section());
            m_edited = true;
            fillList();
        }
        break;
    }
    case TEXTPARAGRAPH:
    {
        TextParagraphElement *elp = static_cast<TextParagraphElement*>(m_document.element(static_cast<size_t>(m_selectedId)));
        ParagraphDialog dialog(this, *elp);
        if(dialog.ShowModal() == wxID_OK)
        {
            TextParagraphElement el = dialog.element();
            elp->setPos(el.pos());
            elp->setWidth(el.width());
            elp->setValue(el.value());
            elp->setAlignment(el.alignment());
            elp->setStyle(el.style());
            elp->setBorder(el.border());
            elp->setBorderStyle(el.borderStyle());
            elp->setSection(el.section());
            m_edited = true;
            fillList();
        }
        break;
    }
    default: ;
    }
}

void TemplateEditor::OnAddElement(wxCommandEvent& /*event*/)
{
    const wxString choices[] = { wxString::FromUTF8("Čára"), "Text",  wxString::FromUTF8("Textový odstavec"), wxString::FromUTF8("Řádek tabulky"),
                                wxString::FromUTF8("Obrázek"), wxString::FromUTF8("Obdelník") } ;
    wxSingleChoiceDialog vyber(this, "Vyberte prvek", "Prvek", WXSIZEOF(choices), choices);
    if(vyber.ShowModal() == wxID_OK)
    {
        switch(vyber.GetSelection()) {
            case 0: addLine(); break;
            case 1: addText(); break;
            case 2: addParagraph(); break;
            case 3: addTableRow(); break;
            case 4: addImage(); break;
            case 5: addRectangle(); break;
        }
    }
}

void TemplateEditor::OnCopyElement(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    Element *element = m_document.element(static_cast<size_t>(m_selectedId));
    switch(element->elementType()) {
    case IMAGE:
    {
        ImageElement *elp = static_cast<ImageElement*>(m_document.element(static_cast<size_t>(m_selectedId)));
        ImageDialog dialog(this, *elp);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_document.addElement(dialog.element());
            m_document.clearElementSelection();
            m_edited = true;
            fillList();
        }
        break;
    }
    case LINE:
    {
        LineElement *elp = static_cast<LineElement*>(m_document.element(static_cast<size_t>(m_selectedId)));
        LineDialog dialog(this, *elp);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_document.addElement(dialog.element());
            m_document.clearElementSelection();
            m_edited = true;
            fillList();
        }
        break;
    }
    case RECTANGLE:
    {
        RectangleElement *elp = static_cast<RectangleElement*>(m_document.element(static_cast<size_t>(m_selectedId)));
        RectDialog dialog(this, *elp);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_document.addElement(dialog.element());
            m_document.clearElementSelection();
            m_edited = true;
            fillList();
        }
        break;
    }
    case TABLEROW:
    {
        TableRowElement *elp = static_cast<TableRowElement*>(m_document.element(static_cast<size_t>(m_selectedId)));
        RowDialog dialog(this, *elp);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_document.addElement(dialog.element());
            m_document.clearElementSelection();
            m_edited = true;
            fillList();
        }
        break;
    }
    case TEXT:
    {
        TextElement *elp = static_cast<TextElement*>(m_document.element(static_cast<size_t>(m_selectedId)));
        TextDialog dialog(this, *elp);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_document.addElement(dialog.element());
            m_document.clearElementSelection();
            m_edited = true;
            fillList();
        }
        break;
    }
    case TEXTPARAGRAPH:
    {
        TextParagraphElement *elp = static_cast<TextParagraphElement*>(m_document.element(static_cast<size_t>(m_selectedId)));
        ParagraphDialog dialog(this, *elp);
        if(dialog.ShowModal() == wxID_OK)
        {
            m_document.addElement(dialog.element());
            m_document.clearElementSelection();
            m_edited = true;
            fillList();
        }
        break;
    }
    default: ;
    }
}

void TemplateEditor::OnDeleteElement(wxCommandEvent& /*event*/)
{
    if(m_selectedId == -1) return;
    wxMessageDialog dialog(this, "Opravdu smazat prvek?", "Dotaz", wxOK|wxCANCEL|wxICON_QUESTION);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_document.deleteElement(static_cast<size_t>(m_selectedId));
        m_document.clearElementSelection();
        m_edited = true;
        fillList();
    }
}

void TemplateEditor::OnSize(wxSizeEvent &event)
{
    event.Skip();
#if defined (__WXMSW__)
    Refresh();
#endif
}

void TemplateEditor::OnPaint(wxPaintEvent& /*event*/)
{
    wxLogDebug("OnPaint");
    wxPaintDC dc(m_canvas);
    dc.SetBackground(*wxWHITE_BRUSH);
    dc.Clear();
    //TODO hack pro Gtk3
#ifdef __WXGTK3__
    double scale = 1.0*72./static_cast<double>(wxGetDisplayPPI().y);
#else
    double scale = 1.0;
#endif
    int pageWidth = 210;
    int pageHeight = PAGE_HEIGHT;
    int w, h;
    dc.GetSizeMM(&w, &h);
    double overallScaleX = scale * (w/static_cast<double>(pageWidth));
    double overallScaleY = scale * (h/static_cast<double>(pageHeight));
    dc.SetUserScale(wxMin(overallScaleX,overallScaleY), wxMin(overallScaleX,overallScaleY));
    double mmToLogical = (dc.GetPPI().x/(scale*25.4));
    drawMargins(&dc, mmToLogical);
    drawHeader(&dc, mmToLogical);
    drawFooter(&dc, mmToLogical);
    int posY = m_document.marginTop();
    int sheight = m_document.sectionHeight(HEADER1);
    drawElements(&dc, mmToLogical, m_document.elementsBySection(HEADER1), posY, sheight);
    posY += sheight;
    sheight = m_document.sectionHeight(HEADER2);
    drawElements(&dc, mmToLogical, m_document.elementsBySection(HEADER2), posY, sheight);
    posY += sheight;
    sheight = m_document.sectionHeight(HEADER3);
    drawElements(&dc, mmToLogical, m_document.elementsBySection(HEADER3), posY, sheight);
    posY += sheight;
    sheight = m_document.sectionHeight(ITEMS);
    drawElements(&dc, mmToLogical, m_document.elementsBySection(ITEMS), posY, sheight);
    posY += sheight;
    sheight = m_document.sectionHeight(SUM);
    drawElements(&dc, mmToLogical, m_document.elementsBySection(SUM), posY, sheight);
    posY += sheight;
    sheight = m_document.sectionHeight(DPH);
    drawElements(&dc, mmToLogical, m_document.elementsBySection(DPH), posY, sheight);
    sheight = m_document.sectionHeight(FOOTER1);
    posY = PAGE_HEIGHT-m_document.marginBottom()-sheight;
    drawElements(&dc, mmToLogical, m_document.elementsBySection(FOOTER1), posY, sheight);
    sheight = m_document.sectionHeight(FOOTER2);
    posY -= sheight;
    drawElements(&dc, mmToLogical, m_document.elementsBySection(FOOTER2), posY, sheight);
    sheight = m_document.sectionHeight(FOOTER3);
    posY -= sheight;
    drawElements(&dc, mmToLogical, m_document.elementsBySection(FOOTER3), posY, sheight);
    drawElements(&dc, mmToLogical, m_document.elementsBySection(ANYSECTION), 0, 0);
}

void TemplateEditor::OnDocumentProperties(wxCommandEvent& /*event*/)
{
    DokumentDialog dialog(this, m_document);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_edited = true;
        m_document = dialog.document();
        m_canvas->Refresh();
    }
}

void TemplateEditor::setSelectedId(long selectedIndex)
{
    if(selectedIndex < 0)
        m_selectedId = -1;
    else
    {
        m_document.setElementSelection(static_cast<size_t>(m_selectedId), false);
        m_selectedId = selectedIndex;
        m_document.setElementSelection(static_cast<size_t>(m_selectedId), true);
        m_canvas->Refresh();
    }
}

void TemplateEditor::fillList()
{
    m_list->Clear();
    m_document.sortElements();
    m_document.clearElementSelection();
    dxElementsArray elements = m_document.elements();
    for(size_t i=0; i<elements.GetCount(); i++)
    {
        switch(elements[i]->elementType()) {
        case IMAGE: m_list->Append(wxString::FromUTF8("Obrázek")); break;
        case RECTANGLE: m_list->Append(wxString::FromUTF8("Obdelník")); break;
        case LINE: m_list->Append(wxString::FromUTF8("Čára")); break;
        case TABLEROW: m_list->Append(wxString::FromUTF8("Řádek tabulky")); break;
        case TEXT: m_list->Append("Text"); break;
        case TEXTPARAGRAPH: m_list->Append(wxString::FromUTF8("Textový odstavec")); break;
        default: m_list->Append("prd");
        }
    }
    m_canvas->Refresh();
    if(elements.GetCount())
    {
        m_list->Select(0);
        setSelectedId(0);
    }
    else
    {
        m_selectedId = -1;
    }
    enableTools();
}

void TemplateEditor::drawMargins(wxDC *dc, double mmToLogical)
{
    dc->SetPen(*wxBLACK_PEN);
    dc->SetBrush(*wxTRANSPARENT_BRUSH);
    dc->DrawRectangle(0,0,static_cast<int>(210*mmToLogical), static_cast<int>(PAGE_HEIGHT*mmToLogical));
    dc->SetPen(wxPen(*wxBLACK, 1, wxPENSTYLE_SHORT_DASH));
    dc->DrawLine(wxPoint(static_cast<int>(m_document.marginLeft()*mmToLogical),static_cast<int>(0*mmToLogical)),
                 wxPoint(static_cast<int>(m_document.marginLeft()*mmToLogical),static_cast<int>(PAGE_HEIGHT*mmToLogical)));
    dc->DrawLine(wxPoint(0,static_cast<int>(m_document.marginTop()*mmToLogical)),
                 wxPoint(static_cast<int>(210*mmToLogical),static_cast<int>(m_document.marginTop()*mmToLogical)));
    dc->DrawLine(wxPoint(static_cast<int>((210-m_document.marginRight())*mmToLogical),0),
                 wxPoint(static_cast<int>((210-m_document.marginRight())*mmToLogical),static_cast<int>(PAGE_HEIGHT*mmToLogical)));
    dc->DrawLine(wxPoint(0,static_cast<int>((PAGE_HEIGHT-m_document.marginBottom())*mmToLogical)),
                 wxPoint(static_cast<int>(210*mmToLogical),static_cast<int>((PAGE_HEIGHT-m_document.marginBottom())*mmToLogical)));
}

void TemplateEditor::drawHeader(wxDC *dc, double mmToLogical)
{
    int pageWidthMM = 210;
    wxSize textSize;
    int leftMargin = 10;
    int rightMargin = 10;
    int topMargin = 10;
    double topMarginLogical = (mmToLogical*topMargin);
    double xPos;
    dc->SetFont(wxFont(wxFontInfo(10).FaceName(FACENAME)));
    //Vykresleni left header
    if(!m_document.headerLeftText().IsEmpty())
    {
        textSize = dc->GetTextExtent(m_document.headerLeftText());
        xPos = (leftMargin*mmToLogical);
        dc->DrawText(m_document.headerLeftText(), static_cast<int>(xPos), static_cast<int>(topMarginLogical-textSize.y));
    }
    //Vykresleni center header
    if(!m_document.headerCenterText().IsEmpty())
    {
        textSize = dc->GetTextExtent(m_document.headerCenterText());
        xPos = (((((pageWidthMM - leftMargin - rightMargin)/2.0)+leftMargin)*mmToLogical) - (textSize.x/2.0));
        dc->DrawText(m_document.headerCenterText(), static_cast<int>(xPos), static_cast<int>(topMarginLogical-textSize.y));
    }
    //Vykresleni right header
    if(!m_document.headerRightText().IsEmpty())
    {
        textSize = dc->GetTextExtent(m_document.headerRightText());
        xPos = ((pageWidthMM - rightMargin)*mmToLogical - textSize.x);
        dc->DrawText(m_document.headerRightText(), static_cast<int>(xPos), static_cast<int>(topMarginLogical-textSize.y));
    }
    //Vykresleni cary
    if(m_document.headerLine())
    {
        double leftMarginLogical = (mmToLogical*leftMargin);
        double rightMarginLogical = (mmToLogical*(pageWidthMM - rightMargin));
        dc->SetPen(*wxBLACK_PEN);
        dc->DrawLine(static_cast<int>(leftMarginLogical), static_cast<int>(topMarginLogical), static_cast<int>(rightMarginLogical), static_cast<int>(topMarginLogical));
    }
}

void TemplateEditor::drawFooter(wxDC *dc, double mmToLogical)
{
    if(m_document.footer() == dxPrint::F_NONE)
        return;
    wxString text = m_document.footer()==dxPrint::F_TEXT?m_document.footerText():wxString::FromUTF8("Stránka 1 z 1");
    if(text.IsEmpty())
        return;
    int pageWidthMM = 210, pageHeightMM = PAGE_HEIGHT;
    wxSize textSize = dc->GetTextExtent(text);
    int leftMargin = 10;
    int rightMargin = 10;
    int bottomMargin = 10;
    double bottomMarginLogical = (mmToLogical*(pageHeightMM-bottomMargin));
    double xPos = (((((pageWidthMM - leftMargin - rightMargin)/2.0)+leftMargin)*mmToLogical) - (textSize.x/2.0));
    dc->SetFont(wxFont(wxFontInfo(10).FaceName(FACENAME)));
    dc->DrawText(text, static_cast<int>(xPos), static_cast<int>(bottomMarginLogical));
}

void TemplateEditor::enableTools()
{
    if(m_selectedId == -1)
    {
        m_bar->EnableTool(ID_TEMPLATEEDITOR_EDIT, false);
        m_bar->EnableTool(ID_TEMPLATEEDITOR_COPY, false);
        m_bar->EnableTool(ID_TEMPLATEEDITOR_DELETE, false);
    }
    else
    {
        m_bar->EnableTool(ID_TEMPLATEEDITOR_EDIT, true);
        m_bar->EnableTool(ID_TEMPLATEEDITOR_COPY, true);
        m_bar->EnableTool(ID_TEMPLATEEDITOR_DELETE, true);
    }
}

void TemplateEditor::addLine()
{
    LineDialog dialog(this);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_document.addElement(dialog.element());
        m_edited = true;
        fillList();
    }
}

void TemplateEditor::addRectangle()
{
    RectDialog dialog(this);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_document.addElement(dialog.element());
        m_edited = true;
        fillList();
    }
}

void TemplateEditor::addText()
{
    TextDialog dialog(this, m_document.style());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_document.addElement(dialog.element());
        m_edited = true;
        fillList();
    }
}

void TemplateEditor::addParagraph()
{
    ParagraphDialog dialog(this, m_document.style());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_document.addElement(dialog.element());
        m_edited = true;
        fillList();
    }
}

void TemplateEditor::addTableRow()
{
    RowDialog dialog(this, m_document.style());
    if(dialog.ShowModal() == wxID_OK)
    {
        m_document.addElement(dialog.element());
        m_edited = true;
        fillList();
    }
}

void TemplateEditor::addImage()
{
    ImageDialog dialog(this);
    if(dialog.ShowModal() == wxID_OK)
    {
        m_document.addElement(dialog.element());
        m_edited = true;
        fillList();
    }
}

void TemplateEditor::createNeplataceTemplate()
{
    m_document = Document();
    int pageWidth = 210;
    int halfPage = wxRound(pageWidth/2.0);
    int pageHeight = PAGE_HEIGHT;
    int marginLeft = m_document.marginLeft();
    int marginTop = m_document.marginTop();
    int marginRight = m_document.marginRight();
    int marginBottom = m_document.marginBottom();
    m_document.addElement(LineElement(1, wxPoint(marginLeft,marginTop), wxPoint(pageWidth-marginRight,marginTop),LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document.addElement(LineElement(1, wxPoint(marginLeft,marginTop), wxPoint(marginLeft,pageHeight-marginBottom),LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document.addElement(LineElement(1, wxPoint(pageWidth-marginRight,marginTop), wxPoint(pageWidth-marginRight,pageHeight-marginBottom),LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document.addElement(LineElement(1, wxPoint(marginLeft,pageHeight-marginBottom), wxPoint(pageWidth-marginRight,pageHeight-marginBottom),LineStyle(*wxBLACK, BOLD_WIDTH)));

    TextElement text = TextElement(1, wxString::FromUTF8("Faktura - daňový doklad č. {faktura-cislo}"), wxPoint(marginLeft,0),
                                   pageWidth-marginLeft-marginRight, wxTEXT_ALIGNMENT_RIGHT, TextStyle(wxFont(wxFontInfo(16).FaceName(FACENAME).Bold())));
    text.setSection(HEADER1);
    m_document.addElement(text);
    LineElement line = LineElement(1, wxPoint(marginLeft,0), wxPoint(pageWidth-marginRight,0));
    line.setSection(HEADER1);
    line.setBorderBottom(true);
    m_document.addElement(line);

    int firmaHeight = 0;
    ImageElement image = ImageElement(1, "{firma-logo}", wxPoint(marginLeft,1), halfPage-marginLeft);
    image.setSection(HEADER2);
    m_document.addElement(image);
    firmaHeight = image.height()+1;
    text = TextElement(1, "Dodavatel:", wxPoint(marginLeft,firmaHeight), halfPage-marginLeft, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    text.setSection(HEADER2);
    m_document.addElement(text);
    firmaHeight += text.height();
    wxArrayString firma;
    firma.Add("{firma-nazev}");
    firma.Add("{firma-ulice}");
    firma.Add("{firma-psc} {firma-misto}");
    firma.Add("{firma-zeme}");
    firma.Add(wxString::FromUTF8("IČ: {firma-ic}"));
    firma.Add("Telefon: {firma-telefon}");
    firma.Add("E-mail: {firma-email}");
    firma.Add("WWW: {firma-www}");
    firma.Add(wxString::FromUTF8("Neplátce DPH"));
    firma.Add("Registrace: {firma-registrace}");
    TextParagraphElement paragraph = TextParagraphElement(1, firma, wxPoint(marginLeft,firmaHeight), halfPage-marginLeft);
    paragraph.setStyle(m_document.style());
    paragraph.setSection(HEADER2);
    m_document.addElement(paragraph);
    int kontaktHeight = 0;
    text = TextElement(1, wxString::FromUTF8("Odběratel:"), wxPoint(halfPage,0), halfPage-marginRight, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    text.setSection(HEADER2);
    m_document.addElement(text);
    wxArrayString kontakt;
    kontakt.Add("{kontakt-nazev}");
    kontakt.Add("{kontakt-ulice}");
    kontakt.Add("{kontakt-psc} {kontakt-misto}");
    kontakt.Add("{kontakt-zeme}");
    int adresaY = 37-marginTop-m_document.sectionHeight(HEADER1);
    paragraph = TextParagraphElement(1, kontakt, wxPoint(110,adresaY), pageWidth-marginRight-110);
    paragraph.setStyle(m_document.style());
    paragraph.setSection(HEADER2);
    m_document.addElement(paragraph);
    kontaktHeight = adresaY+paragraph.height();
    kontakt.Clear();
    kontakt.Add(" ");
    kontakt.Add(" ");
    kontakt.Add(wxString::FromUTF8("IČ: {kontakt-ic}"));
    paragraph = TextParagraphElement(1, kontakt, wxPoint(halfPage,kontaktHeight), halfPage-marginRight);
    paragraph.setStyle(m_document.style());
    paragraph.setSection(HEADER2);
    m_document.addElement(paragraph);
    line = LineElement(1, wxPoint(halfPage,0), wxPoint(halfPage,0));
    line.setAnchor(true);
    line.setSection(HEADER2);
    m_document.addElement(line);
    line = LineElement(1, wxPoint(marginLeft,0), wxPoint(pageWidth-marginRight,0));
    line.setBorderBottom(true);
    line.setSection(HEADER2);
    m_document.addElement(line);

    text = TextElement(1, wxString::FromUTF8("Platební podmínky:"), wxPoint(marginLeft,0), halfPage-marginLeft, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    text.setSection(HEADER3);
    m_document.addElement(text);
    wxArrayString podminky;
    podminky.Add(wxString::FromUTF8("Datum vystavení: {faktura-vystaveni}"));
    podminky.Add("Datum splatnosti: {faktura-splatnost}");
    podminky.Add(wxString::FromUTF8("Forma úhrady: {faktura-uhrada}"));
    podminky.Add(wxString::FromUTF8("Bankovní spojení: {faktura-banka}"));
    podminky.Add(wxString::FromUTF8("Variabilní symbol: {faktura-vs}"));
    podminky.Add(wxString::FromUTF8("Specifický symbol: {faktura-ss}"));
    podminky.Add(wxString::FromUTF8("Konstantní symbol: {faktura-ks}"));
    paragraph = TextParagraphElement(1, podminky, wxPoint(marginLeft,text.height()), pageWidth-marginLeft-marginRight);
    paragraph.setStyle(m_document.style());
    paragraph.setSection(HEADER3);
    m_document.addElement(paragraph);
    line = LineElement(1, wxPoint(marginLeft,0), wxPoint(pageWidth-marginRight,0));
    line.setBorderBottom(true);
    line.setSection(HEADER3);
    m_document.addElement(line);

    text = TextElement(1, wxString::FromUTF8("Fakturujeme vám:"), wxPoint(marginLeft,0), halfPage-marginLeft, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    text.setSection(ITEMS);
    m_document.addElement(text);
    int itemsHeight = text.height();
    wxArrayString texty;
    texty.Add("Popis");
    texty.Add(wxString::FromUTF8("Množství (mj)"));
    texty.Add("Cena za mj ({faktura-mena})");
    texty.Add("Cena celkem ({faktura-mena})");
    wxArrayInt width;
    width.Add(pageWidth-marginLeft-marginRight-95);
    width.Add(30);
    width.Add(30);
    width.Add(35);
    wxArrayInt alig;
    alig.Add(wxTEXT_ALIGNMENT_LEFT);
    alig.Add(wxTEXT_ALIGNMENT_CENTER);
    alig.Add(wxTEXT_ALIGNMENT_CENTER);
    alig.Add(wxTEXT_ALIGNMENT_CENTER);
    TableRowElement row = TableRowElement(1, texty, wxPoint(marginLeft,itemsHeight), width, alig);
    row.setStyle(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Italic())));
    row.setSection(ITEMS);
    m_document.addElement(row);
    itemsHeight += row.height();
    texty.Clear();
    texty.Add("{polozka-popis}");
    texty.Add("{polozka-pocet}{polozka-mj}");
    texty.Add("{polozka-cenamj}");
    texty.Add("{polozka-celkem}");
    row = TableRowElement(1, texty, wxPoint(marginLeft,itemsHeight), width, alig);
    row.setStyle(m_document.style());
    row.setSection(ITEMS);
    m_document.addElement(row);

    text = TextElement(1, wxString::FromUTF8("CELKEM K ÚHRADĚ: {faktura-celkem} {faktura-mena}"), wxPoint(marginLeft,3), pageWidth-marginLeft-marginRight, wxTEXT_ALIGNMENT_CENTER,
                       TextStyle(wxFont(wxFontInfo(14).FaceName(FACENAME).Bold())));
    text.setSection(SUM);
    m_document.addElement(text);

    int footerHeight = 0;
    image = ImageElement(1, "{firma-razitko}", wxPoint(halfPage,0), halfPage-marginRight, wxTEXT_ALIGNMENT_CENTER);
    image.setSection(FOOTER1);
    m_document.addElement(image);
    footerHeight += image.height()+2;
    if(footerHeight < 3)
    {
        footerHeight += 28;
    }
    image = ImageElement(1, "SPD*1.0*ACC:CZ2806000000000168540115*AM:336.00*CC:CZK*MSG:PLATBA ZA ZBOZI*X-VS:1234567890", wxPoint(marginLeft+2,2));
    image.setSection(FOOTER1);
    m_document.addElement(image);
    if(footerHeight < image.height()+4)
    {
        footerHeight = image.height()+4;
    }
    text = TextElement(1, wxString::FromUTF8("razítko, podpis"), wxPoint(halfPage,footerHeight), halfPage-marginRight, wxTEXT_ALIGNMENT_CENTER);
    text.setStyle(m_document.style());
    text.setSection(FOOTER1);
    m_document.addElement(text);
    line = LineElement(1, wxPoint(marginLeft,0), wxPoint(pageWidth-marginRight,0));
    line.setSection(FOOTER1);
    line.setBorderTop(true);
    m_document.addElement(line);
}

void TemplateEditor::createPlatceTemplate()
{
    m_document = Document();
    int pageWidth = 210;
    int halfPage = wxRound(pageWidth/2.0);
    int pageHeight = PAGE_HEIGHT;
    int marginLeft = m_document.marginLeft();
    int marginTop = m_document.marginTop();
    int marginRight = m_document.marginRight();
    int marginBottom = m_document.marginBottom();
    m_document.addElement(LineElement(1, wxPoint(marginLeft,marginTop), wxPoint(pageWidth-marginRight,marginTop),LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document.addElement(LineElement(1, wxPoint(marginLeft,marginTop), wxPoint(marginLeft,pageHeight-marginBottom),LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document.addElement(LineElement(1, wxPoint(pageWidth-marginRight,marginTop), wxPoint(pageWidth-marginRight,pageHeight-marginBottom),LineStyle(*wxBLACK, BOLD_WIDTH)));
    m_document.addElement(LineElement(1, wxPoint(marginLeft,pageHeight-marginBottom), wxPoint(pageWidth-marginRight,pageHeight-marginBottom),LineStyle(*wxBLACK, BOLD_WIDTH)));

    TextElement text = TextElement(1, wxString::FromUTF8("Faktura - daňový doklad č. {faktura-cislo}"), wxPoint(marginLeft,0),
            pageWidth-marginLeft-marginRight, wxTEXT_ALIGNMENT_RIGHT, TextStyle(wxFont(wxFontInfo(16).FaceName(FACENAME).Bold())));
    text.setSection(HEADER1);
    m_document.addElement(text);
    LineElement line = LineElement(1, wxPoint(marginLeft,0), wxPoint(pageWidth-marginRight,0));
    line.setSection(HEADER1);
    line.setBorderBottom(true);
    m_document.addElement(line);

    int firmaHeight = 0;
    ImageElement image = ImageElement(1, "{firma-logo}", wxPoint(marginLeft,1), halfPage-marginLeft);
    image.setSection(HEADER2);
    m_document.addElement(image);
    firmaHeight = image.height()+1;
    text = TextElement(1, "Dodavatel:", wxPoint(marginLeft,firmaHeight), halfPage-marginLeft, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    text.setSection(HEADER2);
    m_document.addElement(text);
    firmaHeight += text.height();
    wxArrayString firma;
    firma.Add("{firma-nazev}");
    firma.Add("{firma-ulice}");
    firma.Add("{firma-psc} {firma-misto}");
    firma.Add("{firma-zeme}");
    firma.Add(wxString::FromUTF8("IČ: {firma-ic}"));
    firma.Add(wxString::FromUTF8("DIČ: {firma-dic}"));
    firma.Add("Telefon: {firma-telefon}");
    firma.Add("E-mail: {firma-email}");
    firma.Add("WWW: {firma-www}");
    firma.Add("Registrace: {firma-registrace}");
    TextParagraphElement paragraph = TextParagraphElement(1, firma, wxPoint(marginLeft,firmaHeight), halfPage-marginLeft);
    paragraph.setStyle(m_document.style());
    paragraph.setSection(HEADER2);
    m_document.addElement(paragraph);
    int kontaktHeight = 0;
    text = TextElement(1, wxString::FromUTF8("Odběratel:"), wxPoint(halfPage,0), halfPage-marginRight, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    text.setSection(HEADER2);
    m_document.addElement(text);
    wxArrayString kontakt;
    kontakt.Add("{kontakt-nazev}");
    kontakt.Add("{kontakt-ulice}");
    kontakt.Add("{kontakt-psc} {kontakt-misto}");
    kontakt.Add("{kontakt-zeme}");
    int adresaY = 37-marginTop-m_document.sectionHeight(HEADER1);
    paragraph = TextParagraphElement(1, kontakt, wxPoint(110,adresaY), pageWidth-marginRight-110);
    paragraph.setStyle(m_document.style());
    paragraph.setSection(HEADER2);
    m_document.addElement(paragraph);
    kontaktHeight = adresaY+paragraph.height();
    kontakt.Clear();
    kontakt.Add(" ");
    kontakt.Add(" ");
    kontakt.Add(wxString::FromUTF8("IČ: {kontakt-ic}"));
    kontakt.Add(wxString::FromUTF8("DIČ: {kontakt-dic}"));
    paragraph = TextParagraphElement(1, kontakt, wxPoint(halfPage,kontaktHeight), halfPage-marginRight);
    paragraph.setStyle(m_document.style());
    paragraph.setSection(HEADER2);
    m_document.addElement(paragraph);
    line = LineElement(1, wxPoint(halfPage,0), wxPoint(halfPage,0));
    line.setAnchor(true);
    line.setSection(HEADER2);
    m_document.addElement(line);
    line = LineElement(1, wxPoint(marginLeft,0), wxPoint(pageWidth-marginRight,0));
    line.setBorderBottom(true);
    line.setSection(HEADER2);
    m_document.addElement(line);

    text = TextElement(1, wxString::FromUTF8("Platební podmínky:"), wxPoint(marginLeft,0), halfPage-marginLeft, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    text.setSection(HEADER3);
    m_document.addElement(text);
    wxArrayString podminky;
    podminky.Add(wxString::FromUTF8("Datum vystavení: {faktura-vystaveni}"));
    podminky.Add("Datum splatnosti: {faktura-splatnost}");
    podminky.Add(wxString::FromUTF8("Datum uskutečnění zdanitelného plnění: {faktura-duzp}"));
    podminky.Add(wxString::FromUTF8("Forma úhrady: {faktura-uhrada}"));
    podminky.Add(wxString::FromUTF8("Bankovní spojení: {faktura-banka}"));
    podminky.Add(wxString::FromUTF8("Variabilní symbol: {faktura-vs}"));
    podminky.Add(wxString::FromUTF8("Specifický symbol: {faktura-ss}"));
    podminky.Add(wxString::FromUTF8("Konstantní symbol: {faktura-ks}"));
    paragraph = TextParagraphElement(1, podminky, wxPoint(marginLeft,text.height()), pageWidth-marginLeft-marginRight);
    paragraph.setStyle(m_document.style());
    paragraph.setSection(HEADER3);
    m_document.addElement(paragraph);
    line = LineElement(1, wxPoint(marginLeft,0), wxPoint(pageWidth-marginRight,0));
    line.setBorderBottom(true);
    line.setSection(HEADER3);
    m_document.addElement(line);

    text = TextElement(1, wxString::FromUTF8("Fakturujeme vám:"), wxPoint(marginLeft,0), halfPage-marginLeft, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    text.setSection(ITEMS);
    m_document.addElement(text);
    int itemsHeight = text.height();
    wxArrayString texty;
    texty.Add("Popis");
    texty.Add(wxString::FromUTF8("Množství (mj)"));
    texty.Add("Cena za mj bez DPH ({faktura-mena})");
    texty.Add("Sazba DPH (%)");
    texty.Add("DPH ({faktura-mena})");
    texty.Add("Cena celkem ({faktura-mena})");
    wxArrayInt width;
    width.Add(pageWidth-marginLeft-marginRight-130);
    width.Add(20);
    width.Add(30);
    width.Add(20);
    width.Add(25);
    width.Add(35);
    wxArrayInt alig;
    alig.Add(wxTEXT_ALIGNMENT_LEFT);
    alig.Add(wxTEXT_ALIGNMENT_CENTER);
    alig.Add(wxTEXT_ALIGNMENT_CENTER);
    alig.Add(wxTEXT_ALIGNMENT_CENTER);
    alig.Add(wxTEXT_ALIGNMENT_CENTER);
    alig.Add(wxTEXT_ALIGNMENT_CENTER);
    TableRowElement row = TableRowElement(1, texty, wxPoint(marginLeft,itemsHeight), width, alig);
    row.setStyle(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Italic())));
    row.setSection(ITEMS);
    m_document.addElement(row);
    itemsHeight += row.height();
    texty.Clear();
    texty.Add("{polozka-popis}");
    texty.Add("{polozka-pocet}{polozka-mj}");
    texty.Add("{polozka-cenamj}");
    texty.Add("{polozka-sazba}");
    texty.Add("{polozka-dph}");
    texty.Add("{polozka-celkem}");
    row = TableRowElement(1, texty, wxPoint(marginLeft,itemsHeight), width, alig);
    row.setStyle(m_document.style());
    row.setSection(ITEMS);
    m_document.addElement(row);

    text = TextElement(1, wxString::FromUTF8("CELKEM K ÚHRADĚ: {faktura-celkem} {faktura-mena}"), wxPoint(marginLeft,3), pageWidth-marginLeft-marginRight, wxTEXT_ALIGNMENT_CENTER,
                       TextStyle(wxFont(wxFontInfo(14).FaceName(FACENAME).Bold())));
    text.setSection(SUM);
    m_document.addElement(text);

    line = LineElement(1, wxPoint(marginLeft,0), wxPoint(pageWidth-marginRight,0));
    line.setBorderTop(true);
    line.setSection(DPH);
    m_document.addElement(line);
    text = TextElement(1, "Rekapitulace DPH:", wxPoint(marginLeft,0), halfPage-marginLeft, wxTEXT_ALIGNMENT_LEFT, TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Bold())));
    text.setSection(DPH);
    m_document.addElement(text);
    int dphHeight = text.height();
    texty.Clear();
    width.Clear();
    alig.Clear();
    texty.Add("Sazba DPH");
    texty.Add(wxString::FromUTF8("Základ ({faktura-mena})"));
    texty.Add("DPH ({faktura-mena})");
    width.Add(63);
    width.Add(63);
    width.Add(pageWidth-marginLeft-marginRight-126);
    alig.Add(wxTEXT_ALIGNMENT_CENTER);
    alig.Add(wxTEXT_ALIGNMENT_CENTER);
    alig.Add(wxTEXT_ALIGNMENT_CENTER);
    row = TableRowElement(1, texty, wxPoint(marginLeft,dphHeight), width, alig);
    row.setStyle(TextStyle(wxFont(wxFontInfo(12).FaceName(FACENAME).Italic())));
    row.setSection(DPH);
    dphHeight += row.height();
    m_document.addElement(row);
    texty.Clear();
    texty.Add("{dph-sazba}");
    texty.Add("{dph-zaklad}");
    texty.Add("{dph-dph}");
    row = TableRowElement(1, texty, wxPoint(marginLeft,dphHeight), width, alig);
    row.setStyle(m_document.style());
    row.setSection(DPH);
    m_document.addElement(row);
    line = LineElement(1, wxPoint(marginLeft,0), wxPoint(pageWidth-marginRight,0));
    line.setBorderBottom(true);
    line.setSection(DPH);
    m_document.addElement(line);

    int footerHeight = 0;
    image = ImageElement(1, "{firma-razitko}", wxPoint(halfPage,0), halfPage-marginRight, wxTEXT_ALIGNMENT_CENTER);
    image.setSection(FOOTER1);
    m_document.addElement(image);
    footerHeight += image.height()+2;
    if(footerHeight < 3)
    {
        footerHeight += 28;
    }
    image = ImageElement(1, "SPD*1.0*ACC:CZ2806000000000168540115*AM:336.00*CC:CZK*MSG:PLATBA ZA ZBOZI*X-VS:1234567890", wxPoint(marginLeft+2,2));
    image.setSection(FOOTER1);
    m_document.addElement(image);
    if(footerHeight < image.height()+4)
    {
        footerHeight = image.height()+4;
    }
    text = TextElement(1, wxString::FromUTF8("razítko, podpis"), wxPoint(halfPage,footerHeight), halfPage-marginRight, wxTEXT_ALIGNMENT_CENTER);
    text.setStyle(m_document.style());
    text.setSection(FOOTER1);
    m_document.addElement(text);
    line = LineElement(1, wxPoint(marginLeft,0), wxPoint(pageWidth-marginRight,0));
    line.setSection(FOOTER1);
    line.setBorderTop(true);
    m_document.addElement(line);
}

void TemplateEditor::drawElements(wxDC *dc, double mmToLogical, dxElementsArray elements, int posY, int sectionHeight)
{
    for(size_t i=0; i<elements.GetCount(); i++)
    {
        if(elements[i]->elementType() == LINE)
        {
            LineElement el = *static_cast<LineElement*>(elements[i]);
            if(el.anchor())
            {
                el.setStart(wxPoint(el.start().x, posY));
                el.setEnd(wxPoint(el.end().x, posY+sectionHeight));
                el.DrawToDC(dc, mmToLogical);
                continue;
            }
            if(el.borderTop())
            {
                el.setStart(wxPoint(el.start().x, posY));
                el.setEnd(wxPoint(el.end().x, posY));
                el.DrawToDC(dc, mmToLogical);
                continue;
            }
            if(el.borderBottom())
            {
                el.setStart(wxPoint(el.start().x, posY+sectionHeight));
                el.setEnd(wxPoint(el.end().x, posY+sectionHeight));
                el.DrawToDC(dc, mmToLogical);
                continue;
            }
            el.DrawToDC(dc, mmToLogical, posY);
        }
        else if(elements[i]->elementType() == RECTANGLE)
        {
            RectangleElement el = *static_cast<RectangleElement*>(elements[i]);
            if(el.anchor())
            {
                el.setStart(wxPoint(el.start().x, posY));
                el.setSize(wxSize(el.size().x, sectionHeight));
                el.DrawToDC(dc, mmToLogical);
                continue;
            }
            el.DrawToDC(dc, mmToLogical, posY);
        }
        else
        {
            elements[i]->DrawToDC(dc, mmToLogical, posY);
        }
    }
}
