/***************************************************************
 * Name:      firmawizard.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include <wx/gbsizer.h>
#include <wx/splash.h>
#include "resources/images/splash.xpm"
#include "firmawizard.h"
#include "utils/dxutils.h"
#include "utils/dxsettings.h"

BEGIN_EVENT_TABLE(FirmaPage, wxWizardPageSimple)
    EVT_WIZARD_PAGE_CHANGING(wxID_ANY, FirmaPage::OnPageChanging)
END_EVENT_TABLE()

FirmaPage::FirmaPage(wxWizard *parent)
    : wxWizardPageSimple(parent)
{
    createControls();
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

void FirmaPage::createControls()
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);

    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Název")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_nazev = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_nazev, wxGBPosition(0,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Ulice"), wxGBPosition(1,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ulice = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_ulice, wxGBPosition(1,1));
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("čp")), wxGBPosition(1,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_cp = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_cp, wxGBPosition(1,3));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("PSČ")), wxGBPosition(2,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_psc = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_psc, wxGBPosition(2,1));
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Místo")), wxGBPosition(2,2), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_misto = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_misto, wxGBPosition(2,3));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Země")), wxGBPosition(3,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_zeme = new wxTextCtrl(this, wxID_ANY);
    m_zeme->SetValue(wxString::FromUTF8("Česká republika"));
    gridSizer->Add(m_zeme, wxGBPosition(3,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("IČ")), wxGBPosition(4,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_ic = new wxTextCtrl(this, wxID_ANY);
    m_ic->SetValidator(wxTextValidator(wxFILTER_DIGITS));
    gridSizer->Add(m_ic, wxGBPosition(4,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("DIČ")), wxGBPosition(5,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_dic = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_dic, wxGBPosition(5,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Registrace"), wxGBPosition(6,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_registrace = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_registrace, wxGBPosition(6,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Paušální výdaje")), wxGBPosition(7,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_pausal = new wxCheckBox(this, wxID_ANY, "");
    gridSizer->Add(m_pausal, wxGBPosition(7,1));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Plátce DPH")), wxGBPosition(8,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_platce = new wxCheckBox(this, wxID_ANY, "");
    gridSizer->Add(m_platce, wxGBPosition(8,1));

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Telefon"), wxGBPosition(9,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_telefon = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_telefon, wxGBPosition(9,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "E-mail"), wxGBPosition(10,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_email = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_email, wxGBPosition(10,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "WWW"), wxGBPosition(11,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_www = new wxTextCtrl(this, wxID_ANY);
    gridSizer->Add(m_www, wxGBPosition(11,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, "Logo"), wxGBPosition(12,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_logo = new wxFilePickerCtrl(this, wxID_ANY, wxEmptyString, "Vyberte logo", wxString::FromUTF8("Soubory obrázků (*.gif;*.png;*.jpg)|*.gif;*.png;*.jpg"));
    gridSizer->Add(m_logo, wxGBPosition(12,1), wxGBSpan(1,3), wxEXPAND);

    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Razítko")), wxGBPosition(13,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_razitko = new wxFilePickerCtrl(this, wxID_ANY, wxEmptyString, wxString::FromUTF8("Vyberte razítko"), wxString::FromUTF8("Soubory obrázků (*.gif;*.png;*.jpg)|*.gif;*.png;*.jpg"));
    gridSizer->Add(m_razitko, wxGBPosition(13,1), wxGBSpan(1,3), wxEXPAND);

    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL, 5);

    this->SetSizer(m_mainSizer);
}

void FirmaPage::OnPageChanging(wxWizardEvent &event)
{
    if(m_nazev->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("Název nemůže být prázdný"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_nazev->SetFocus();
        event.Veto();
        return;
    }
    if(m_ic->GetValue().IsEmpty())
    {
        wxMessageDialog dialog(this, wxString::FromUTF8("IČ nemůže být prázdné"), "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_ic->SetFocus();
        event.Veto();
        return;
    }
    if(dxsettings.firmaExists(m_nazev->GetValue()+"_"+m_ic->GetValue()))
    {
        wxString msg;
        msg << "Firma " << m_nazev->GetValue() << wxString::FromUTF8(", IČ ") << m_ic->GetValue() << wxString::FromUTF8(" už existuje");
        wxMessageDialog dialog(this, msg, "Informace", wxOK|wxICON_WARNING);
        dialog.ShowModal();
        m_nazev->GetValue();
        event.Veto();
        return;
    }
    if(!dxutils::checkIC(m_ic->GetValue()))
    {
        wxMessageBox(wxString::FromUTF8("IČ není platné"), "Informace");
    }
}

Adresa FirmaPage::getAdresa()
{
    return Adresa(1, m_ulice->GetValue(), m_cp->GetValue(), m_misto->GetValue(), m_psc->GetValue(), m_zeme->GetValue());
}

Firma FirmaPage::getFirma() const
{
    return Firma(m_nazev->GetValue(), m_ic->GetValue(), m_dic->GetValue(), m_registrace->GetValue(), m_platce->IsChecked(), m_pausal->IsChecked(),
                 m_telefon->GetValue(), m_email->GetValue(), m_www->GetValue(), m_logo->GetPath(), m_razitko->GetPath(), 1);
}

IMPLEMENT_DYNAMIC_CLASS(FirmaWizard, wxWizard)

BEGIN_EVENT_TABLE(FirmaWizard, wxWizard)
    EVT_WIZARD_FINISHED(wxID_ANY, FirmaWizard::OnFinished)
    EVT_WIZARD_CANCEL(wxID_ANY, FirmaWizard::OnCancel)
END_EVENT_TABLE()

FirmaWizard::FirmaWizard(wxWindow *parent, bool firstrun)
    : wxWizard(parent, wxID_ANY, wxString::FromUTF8("Průvodce přidáním firmy")), m_firstrun(firstrun), m_success(false)
{
    m_page1 = new wxWizardPageSimple(this);
    new wxStaticText(m_page1, wxID_ANY,
                     wxString::FromUTF8("Průvodce Vás provede nastavením firmy\na účetního roku"),
                     wxPoint(5,5));

    m_page2 = new FirmaPage(this);
    m_page1->SetNext(m_page2);

    m_page3 = new ObdobiPage(this);
    m_page2->SetNext(m_page3);
    m_page3->SetPrev(m_page2);

    GetPageAreaSizer()->Add(m_page1);
    GetPageAreaSizer()->Add(m_page2);
    GetPageAreaSizer()->Add(m_page3);
}

void FirmaWizard::OnFinished(wxWizardEvent& /*event*/)
{
    wxSplashScreen *splash = nullptr;
    splash = new wxSplashScreen(wxBitmap(splash_xpm), wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_NO_TIMEOUT, 0, nullptr, wxID_ANY);
    wxYield();
    m_success = true;
    if(!m_firstrun) idbmodel->closeDatabase();
    Firma firma = m_page2->getFirma();
    wxString dbname = dxutils::getUniqueDbName();
    dxsettings.setDbname(dbname);
    dxsettings.addDbnamePair(dbname, firma.nazev()+"_"+firma.ic());
    idbmodel->newDatabase();
    idbmodel->addAdresa(m_page2->getAdresa());
    idbmodel->setFirma(firma);
    idbmodel->addObdobi(Obdobi(m_page3->getRok(), true));
    idbmodel->setCurrentRok(m_page3->getRok());
    splash->Close(true);
    wxDELETE(splash);
}

void FirmaWizard::OnCancel(wxWizardEvent &event)
{
    if(m_firstrun && wxMessageBox(wxString::FromUTF8("Opravdu chcete ukončit průvodce?\n(Aplikace se ukončí)"), "Dotaz",
                        wxICON_QUESTION | wxYES_NO, this) != wxYES )
    {
        event.Veto();
    }
}

bool FirmaWizard::success() const
{
    return m_success;
}

Adresa FirmaWizard::getAdresa()
{
    return m_page2->getAdresa();
}

Firma FirmaWizard::getFirma() const
{
    return m_page2->getFirma();
}

Obdobi *FirmaWizard::getObdobi()
{
    return new Obdobi(m_page3->getRok(), true);
}

ObdobiPage::ObdobiPage(wxWizard *parent)
    : wxWizardPageSimple(parent)
{
    m_mainSizer = new wxBoxSizer(wxVERTICAL);
    wxGridBagSizer *gridSizer = new wxGridBagSizer(5, 5);
    gridSizer->Add(new wxStaticText(this, wxID_ANY, wxString::FromUTF8("Účetní období")), wxGBPosition(0,0), wxDefaultSpan, wxALIGN_CENTER_VERTICAL);
    m_rok = new wxSpinCtrl(this);
    m_rok->SetRange(OBDOBI_MIN, wxDateTime::GetCurrentYear()+1);
    m_rok->SetValue(wxDateTime::GetCurrentYear());
    gridSizer->Add(m_rok, wxGBPosition(0,1), wxDefaultSpan, wxEXPAND);
    m_mainSizer->Add(gridSizer, 0, wxALL|wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL, 5);
    this->SetSizer(m_mainSizer);
    m_mainSizer->Fit(this);
    m_mainSizer->SetSizeHints(this);
}

int ObdobiPage::getRok() const
{
    return m_rok->GetValue();
}
