/***************************************************************
 * Name:      dbmodel.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef DBMODEL_H
#define DBMODEL_H

#include <wx/event.h>
#include <wx/dynarray.h>
#include "wx/wxsqlite3.h"
#include "sqlite3mc_amalgamation.h"
#include "dxdefs.h"
#include "data/adresa.h"
#include "data/firma.h"
#include "data/kontakt.h"
#include "data/obdobi.h"
#include "data/dphsazba.h"
#include "data/mena.h"
#include "data/deniksloupec.h"
#include "data/predkontace.h"
#include "data/denik.h"
#include "data/pokladna.h"
#include "data/banka.h"
#include "data/rada.h"
#include "data/kurz.h"
#include "data/fatyp.h"
#include "data/fauhrada.h"
#include "data/fapolozka.h"
#include "data/faktura.h"
#include "data/zavpoh.h"
#include "data/uhrada.h"
#include "data/odpiskupina.h"
#include "data/majetek.h"
#include "data/odpis.h"
#include "data/settings.h"
#include "data/smtp.h"
#include "data/odeslany.h"
#include "data/fadphsazba.h"
#include "data/dphdruh.h"
#include "data/priznani.h"
#include "data/ulozene.h"
#include "data/dphdoklad.h"
#include "data/sklad.h"
#include "data/zasoba.h"
#include "data/pohybzasob.h"
#include "data/inventura.h"
#include "data/ivpolozka.h"
#include "data/ridic.h"
#include "data/phm.h"
#include "data/druhvozidla.h"
#include "data/vozidlo.h"
#include "data/jizda.h"

WX_DECLARE_OBJARRAY(Adresa, dxAdresaArray);
WX_DECLARE_OBJARRAY(Kontakt, dxKontaktArray);
WX_DECLARE_OBJARRAY(KontaktView, dxKontaktViewArray);
WX_DECLARE_OBJARRAY(Obdobi, dxObdobiArray);
WX_DECLARE_OBJARRAY(DphSazba, dxDphSazbaArray);
WX_DECLARE_OBJARRAY(Mena, dxMenaArray);
WX_DECLARE_OBJARRAY(DenikSloupec, dxDenikSloupecArray);
WX_DECLARE_OBJARRAY(Predkontace, dxPredkontaceArray);
WX_DECLARE_OBJARRAY(Denik, dxDenikArray);
WX_DECLARE_OBJARRAY(DenikView, dxDenikViewArray);
WX_DECLARE_OBJARRAY(Pokladna, dxPokladnaArray);
WX_DECLARE_OBJARRAY(Banka, dxBankaArray);
WX_DECLARE_OBJARRAY(Rada, dxRadaArray);
WX_DECLARE_OBJARRAY(Kurz, dxKurzArray);
WX_DECLARE_OBJARRAY(FaTyp, dxFaTypArray);
WX_DECLARE_OBJARRAY(FaUhrada, dxFaUhradaArray);
WX_DECLARE_OBJARRAY(FaPolozka, dxFaPolozkaArray);
WX_DECLARE_OBJARRAY(Faktura, dxFakturaArray);
WX_DECLARE_OBJARRAY(FakturaView, dxFakturaViewArray);
WX_DECLARE_OBJARRAY(ZavPoh, dxZavPohArray);
WX_DECLARE_OBJARRAY(ZavPohView, dxZavPohViewArray);
WX_DECLARE_OBJARRAY(Uhrada, dxUhradaArray);
WX_DECLARE_OBJARRAY(Odpiskupina, dxOdpiskupinaArray);
WX_DECLARE_OBJARRAY(Majetek, dxMajetekArray);
WX_DECLARE_OBJARRAY(MajetekView, dxMajetekViewArray);
WX_DECLARE_OBJARRAY(Odpis, dxOdpisArray);
WX_DECLARE_OBJARRAY(Settings, dxSettingsArray);
WX_DECLARE_OBJARRAY(Odeslany, dxOdeslanyArray);
WX_DECLARE_OBJARRAY(FaDphSazba, dxFaDphSazbaArray);
WX_DECLARE_OBJARRAY(DphDruh, dxDphDruhArray);
WX_DECLARE_OBJARRAY(Ulozene, dxUlozeneArray);
WX_DECLARE_OBJARRAY(DphDoklad, dxDphDokladArray);
WX_DECLARE_OBJARRAY(Sklad, dxSkladArray);
WX_DECLARE_OBJARRAY(Zasoba, dxZasobaArray);
WX_DECLARE_OBJARRAY(ZasobaView, dxZasobaViewArray);
WX_DECLARE_OBJARRAY(Pohybzasob, dxPohybzasobArray);
WX_DECLARE_OBJARRAY(Inventura, dxInventuraArray);
WX_DECLARE_OBJARRAY(IvPolozka, dxIvPolozkaArray);
WX_DECLARE_OBJARRAY(Ridic, dxRidicArray);
WX_DECLARE_OBJARRAY(Phm, dxPhmArray);
WX_DECLARE_OBJARRAY(DruhVozidla, dxDruhVozidlaArray);
WX_DECLARE_OBJARRAY(Vozidlo, dxVozidloArray);
WX_DECLARE_OBJARRAY(JizdaView, dxJizdaViewArray);
WX_DECLARE_OBJARRAY(Jizda, dxJizdaArray);

wxDECLARE_EVENT(MODEL_UPDATED, wxCommandEvent);
wxDECLARE_EVENT(MODEL_MSG, wxCommandEvent);

#define idbmodel DbModel::instance()

class DbModel
{
public:
    DbModel();
    explicit DbModel(wxWindow *parent);
    virtual ~DbModel();

    //pro sortovani dxDenikArray
    enum DSort
    {
        DS_DATUM = 0,
        DS_CISLO,
        DS_POPIS,
        DS_CASTKA,
        DS_PRIJEM,
        DS_POKLADNA,
        DS_BANKA,
        DS_VYPIS,
        DS_SLOUPEC,
        DS_PAUSAL
    };
    //pro sortovani dxFakturaArray a dxZavPohArray
    enum FZPSort
    {
        FZP_DOKLAD = 0,
        FZP_POPIS,
        FZP_PROTISTRANA,
        FZP_VYSTAVENA,
        FZP_SPLATNA,
        FZP_CELKEM,
        FZP_UHRADIT,
        FZP_MENA,
        FZP_DUZP
    };
    //pro sortovani dxMajetekArray
    enum MSort
    {
        MS_CISLO = 0,
        MS_NAZEV,
        MS_TYP,
        MS_ODPIS,
        MS_PORIZEN,
        MS_PORCENA,
        MS_ZUSCENA
    };
    //pro sortovani dxOdeslanyArray
    enum OSort
    {
        OS_CAS = 0,
        OS_FAKTURA,
        OS_PRIJEMCE
    };
    //pro sortovani dxZasobaArray
    enum ZSort
    {
        ZB_CISLO = 0,
        ZB_NAZEV,
        ZB_STAV,
        ZB_HODNOTA,
        ZB_SKLAD
    };
    //pro sortovani dxJizdaViewArray
    enum JSort
    {
        JA_DATUM = 0,
        JA_CESTA,
        JA_UCEL,
        JA_KM,
        JA_VOZIDLO,
        JA_RIDIC
    };

    static DbModel *instance();
    void initModel();
    void shutdownModel();
    wxString getVersion();
    bool openDatabase();
    bool newDatabase();
    bool closeDatabase();
    int getLastId(const wxString &table);
    void fillData();
    void updateDbFirma();

    //getters , setters apod.
    void addAdresa(const Adresa &adresa);
    void editAdresa(const Adresa &adresa);
    Adresa getAdresa(int id);
    bool deleteAdresa(int id);
    bool hasAdresa(int id);

    Firma firma() const;
    bool isFirmaPlatce();
    bool isFirmaPausal();
    void setFirma(const Firma &firma);
    wxString firmaLogo() const;
    wxString firmaRazitko() const;

    dxKontaktArray kontakty();
    void addKontakt(const Kontakt &kontakt);
    void editKontakt(const Kontakt &kontakt);
    Kontakt getKontakt(int id);
    bool deleteKontakt(int id);
    bool hasKontakt(int id);
    bool hasKontakt(const wxString &nazev);
    int getKontaktIdByNazev(const wxString &nazev);
    wxString getKontaktEmail(int id);
    int copyKontakt(int id);
    bool hasKontaktDic(int id);
    wxString kontaktDic(int id);
    dxKontaktViewArray kontaktyview();

    void addObdobi(const Obdobi &obdobi);
    size_t getObdobiCount();
    bool deleteObdobi(int rok);
    bool hasObdobi(int rok);
    bool hasObdobiGreaterThan(int rok);
    wxArrayString getObdobiAsStrings(int greater = -1);
    int currentRok() const;
    void setCurrentRok(int rok);
    int maxRok();
    //pro setCurrentRok nastavit pocatecni stav zasob
    void updateStavZasob(int zasobaid = -1);

    dxDphSazbaArray dphsazby();
    DphSazba getDphSazba(int id);
    bool hasDphSazba(int id);
    int getDphSazbaValue(int id);
    int getDphSazbaId(int value);
    int zakladniSazba(int rok=-1);
    int prvniSnizenaSazba(int rok=-1);
    int druhaSnizenaSazba(int rok=-1);

    dxDphDruhArray dphdruhy();
    DphDruh getDphDruh(int id);
    dxDphDruhArray dphdruhyNabizenePrijem();
    dxDphDruhArray dphdruhyNabizeneVydaj();
    bool hasDphDruh(int id);
    bool hasDphDruh(const wxString &zkratka);
    int getDphDruhIdByZkratka(const wxString &zkratka);
    wxArrayString getDphDruhKpp();
    wxArrayString getDphDruhRadky();
    wxArrayString getDphDruhKh();
    wxArrayInt getDphDruhIdByRadek(const wxString &radky);
    wxArrayInt getDphDruhIdByRadek(const wxArrayString &radky);
    wxArrayInt getDphDruhIdByKh(const wxString &kh);
    void nabizetDphDruh(int id);
    bool someDokladForDph(const wxDateTime &start, const wxDateTime &end);
    bool someDokladForDph(int id, const wxDateTime &start, const wxDateTime &end, bool mustDic = false);
    bool someDokladForDph(wxArrayInt ids, const wxDateTime &start, const wxDateTime &end, bool mustDic = false);
    /** Pro rezim prenesene danove povinosti */
    bool dphPlatiZakaznik(int id);
    bool kracenyDphDruh(int id);
    bool someDokladForDphSh(const wxDateTime &start, const wxDateTime &end);
    bool someDokladForDphKh(const wxDateTime &start, const wxDateTime &end);
    bool someDokladForDphKhA1(const wxDateTime &start, const wxDateTime &end);
    dxDphDokladArray dokladyForDph(int dphdruhid, const wxDateTime &start, const wxDateTime &end, bool mustDic = true);
    dxDphDokladArray dokladyForDph(wxArrayInt dphdruhids, const wxDateTime &start, const wxDateTime &end, bool mustDic = true);
    bool isDphSazbySameForDoklad(dxDphForDokladArray arr1, dxDphForDokladArray arr2);
    bool isDphDruhValidForDatum(const wxString &zkratka, const wxDateTime &datum);

    dxMenaArray meny();
    Mena getMena(int id);
    Mena getMena(const wxString &kod);
    Mena itemMena(size_t index);
    bool hasMena(int id);
    bool hasMena(const wxString &kod);
    wxString zkratkaMena(int id);
    wxString nazevMena(int id);
    wxString kodMena(int id);

    DenikSloupec getDenikSloupec(int sloupec);
    bool hasDenikSloupec(int sloupec);
    dxDenikSloupecArray deniksloupcePrijem();
    dxDenikSloupecArray deniksloupceVydaj();

    dxPredkontaceArray predkontace();
    void addPredkontace(const Predkontace &predkontace);
    void editPredkontace(const Predkontace &predkontace);
    Predkontace getPredkontace(int id);
    Predkontace getPredkontace(const wxString &zkratka);
    bool deletePredkontace(int id);
    bool hasPredkontace(int id);
    bool hasPredkontace(const wxString &zkratka);
    bool isPredkontacePrijem(int id);
    bool isPredkontaceDanova(int id);
    dxPredkontaceArray predkontacePrijem();
    dxPredkontaceArray predkontaceVydaj();
    int getPredkontaceIdByZkratka(const wxString &zkratka);
    int copyPredkontace(int id);
    bool isPredkontaceUsed(int id);
    int firstPredkontace();

    dxDenikArray denik();
    void addDenik(const Denik &denik);
    void editDenik(const Denik &denik);
    Denik getDenik(int id);
    bool deleteDenik(int id);
    bool hasDenik(int id);
    bool isDenikPrijem(int id);
    bool isDenikDanovy(int id);
    dxDenikArray denikPokladna();
    dxDenikArray denikPokladnaById(int pokladnaid);
    dxDenikArray denikBanka();
    dxDenikArray denikBankaById(int bankaid);
    int getLastVypis(int bankaid);
    bool isDenikUhrada(int id);
    int copyDenik(int id);
    dxDenikArray danovePrijmyVydajeForRok(int rok);
    int denikSloupecByDenikId(int id);
    dxDenikViewArray denikview();
    dxDenikViewArray denikviewPokladna();
    dxDenikViewArray denikviewBanka();

    dxPokladnaArray pokladny();
    void addPokladna(const Pokladna &pokladna);
    void editPokladna(const Pokladna &pokladna);
    Pokladna getPokladna(int id);
    Pokladna getPokladna(const wxString &zkratka);
    bool deletePokladna(int id);
    bool hasPokladna(int id);
    bool hasPokladna(const wxString &zkratka);
    int getPokladnaIdByZkratka(const wxString &zkratka);
    double getPokladnaStav(int id);
    /**
     * @brief getPokladnaZustatek
     * @param id
     * @return zustatek pokladny na zacatku odbodi
     */
    double getPokladnaZustatek(int id);
    double getPokladnyStav(const wxDateTime &toDate);
    bool isPokladnaUsed(int id);
    wxString getPokladnaZkratka(int id);

    dxBankaArray banky();
    void addBanka(const Banka &banka);
    void editBanka(const Banka &banka);
    Banka getBanka(int id);
    Banka getBanka(const wxString &zkratka);
    bool deleteBanka(int id);
    bool hasBanka(int id);
    bool hasBanka(const wxString &zkratka);
    int getBankaIdByZkratka(const wxString &zkratka);
    double getBankaStav(int id);
    /**
     * @brief getBankaZustatek
     * @param id
     * @return zustatek bankovniho uctu na zacatku odbodi
     */
    double getBankaZustatek(int id);
    double getBankyStav(const wxDateTime &toDate);
    bool isBankaUsed(int id);
    wxString getBankaZkratka(int id);

    wxArrayString kodyBank(bool aktivni);
    wxArrayString kodyBankSNazvem(bool aktivni);
    bool hasKod(const wxString &kod);
    wxString bicByKod(const wxString &kod);

    dxRadaArray rady();
    void editRada(const Rada &rada);
    Rada getRada(const wxString &id);
    bool hasRada(const wxString &id);
    void increaseRadaCislo(const wxString &id);
    wxString getCisloDokladu(const wxString &id);
    int getRadaCislo(const wxString &id);
    //prenastaveni cisel rad pro currentrok
    void setDefaultRadaCislo();

    void addKurz(const Kurz &kurz);
    Kurz getKurz(int id);
    Kurz getKurz(const wxString &kod, const wxDateTime &datum = wxDateTime::Today());
    bool hasKurz(int id);
    bool hasKurz(const wxString &kod);
    bool hasKurz(const wxString &kod, const wxDateTime &datum);
    bool hasJednotnyKurz(int rok, const wxString &kod);
    double getJednotnyKurz(const wxString &kod, int rok);

    FaTyp itemFaTyp(size_t index);
    wxArrayString getFaTypyAsStrings();

    dxFaUhradaArray fauhrady();
    void addFaUhrada(const FaUhrada &fauhrada);
    FaUhrada getFaUhrada(int id);
    /**
     * @brief getLastFaUhrada
     * @return FaUhrada s max(id)
     */
    FaUhrada getLastFaUhrada();
    bool hasFaUhrada(int id);
    bool hasFaUhrada(const wxString &uhrada);
    int getFaUhradaByPopis(const wxString &uhrada);
    bool deleteFauhrada(int id);
    bool faUhradaPokladnou(int id);
    bool isFauhradaUsed(int id);
    int firstFaUhrada();

    void addFaPolozka(const FaPolozka &fapolozka);
    void editFaPolozka(const FaPolozka &fapolozka);
    FaPolozka getFaPolozka(int id);
    bool deleteFaPolozka(int id);
    bool hasFaPolozka(int id);
    dxFaPolozkaArray fapolozkyByFakturaId(int fakturaid);
    dxFaPolozkaArray fapolozkyByZavpohId(int zavpohid);
    double dphFaktury(int fakturaid);
    double dphZavPoh(int zavpohid);
    bool deleteFaPolozkyByFakturaId(int fakturaid);
    bool deleteFaPolozkyByZavpohId(int zavpohid);

    void addFaktura(const Faktura &faktura, bool fireUpdate=true);
    void editFaktura(const Faktura &faktura);
    Faktura getFaktura(int id);
    bool deleteFaktura(int id, bool fireUpdate=true);
    bool hasFaktura(int id);
    bool isFakturaVydana(int id);
    dxFakturaArray vystavene();
    dxFakturaArray prijate();
    /* return id nove faktury */
    int copyFaktura(int id);
    dxFaDphSazbaArray dphFakturaSouhrn(int id, bool inCzk=false);
    wxString getFakturaCisloById(int id);
    dxFakturaViewArray vystaveneview();
    dxFakturaViewArray prijateview();
    wxDateTime getFakturaVystaveni(int id);
    wxDateTime getFakturaDuzp(int id);

    void addZavPoh(const ZavPoh &zavpoh, bool fireUpdate=true);
    void editZavPoh(const ZavPoh &zavpoh);
    ZavPoh getZavPoh(int id);
    bool deleteZavPoh(int id, bool fireUpdate=true);
    bool hasZavPoh(int id);
    dxZavPohArray zavazky();
    dxZavPohArray pohledavky();
    int copyZavPoh(int id);
    dxFaDphSazbaArray dphZavPohSouhrn(int id, bool inCzk=false);
    dxZavPohViewArray zavazkyview();
    dxZavPohViewArray pohledavkyview();
    wxDateTime getZavPohDuzp(int id);

    //jsou nejake neuhrazene faktury,zavazky,pohledavky v cizi mene
    bool hasPohledavkyDluhyCizi(int rok);
    bool hasJednotneKurzyForPohledavkyDluhy(int rok);
    double getPohledavkyStav(int rok);
    double getZavazkyStav(int rok);

    void addUhrada(const Uhrada &uhrada);
    bool hasUhrada(int id);
    double uhrazenoByFakturaId(int fakturaid);
    double uhrazenoByZavpohId(int zavpohid);
    bool isFakturaUhrazena(int fakturaid);
    bool isFakturaPosplatnosti(int fakturaid);
    bool isZavpohUhrazena(int zavpohid);
    bool isZavpohPosplatnosti(int zavpohid);
    bool deleteUhradaByFakturaId(int fakturaid);
    bool deleteUhradaByZavpohId(int zavpohid);
    bool someUhradaForFaktura(int fakturaid);
    bool someUhradaForZavpoh(int zavpohid);
    dxDenikArray denikUhradForFaktura(int fakturaid);
    dxDenikArray denikUhradForZavpoh(int zavpohid);
    bool someNeuhrazenaFakturaOrZavpoh();
    dxFakturaViewArray neuhrazeneFakturyView();
    dxZavPohViewArray neuhrazeneZavpohyView();

    dxOdpiskupinaArray odpiskupiny();
    bool hasOdpiskupina(int id);
    Odpiskupina getOdpiskupina(int id);

    dxMajetekArray majetek();
    void addMajetek(const Majetek &majetek);
    void editMajetek(const Majetek &majetek);
    Majetek getMajetek(int id);
    bool deleteMajetek(int id);
    bool hasMajetek(int id);
    double getZustatkovaCenaMajetku(int id);
    int copyMajetek(int id);
    double getMajetekStav(const wxDateTime &toDate, bool hmotny);
    wxString majetekTypAsString(int typ);
    wxString majetekOdpisAsString(int odpis);
    dxMajetekViewArray majetekview();

    void addOdpis(const Odpis &odpis);
    bool hasOdpis(int id);
    bool hasOdpis(int majetekid, int rok);
    double getOdpisy(int majetekid, int kroku);
    double calculateRovnomernyOdpis(double cena, double koeficient);
    double calculateZrychlenyOdpis(double cena, double koeficient, int rok, int majetekid, int porizen);
    void calculateOdpisy();
    dxOdpisArray odpisyMajetku(int majetekid, int kroku=-1);

    void addSettings(const Settings &settings);
    void editSettings(const Settings &settings);
    bool hasSettings(int rok);
    int getSplatnost(int rok);

    Smtp smtp() const;
    void setSmtp(const Smtp &smtp);
    void setSmtpSender(const wxString &sender);

    dxOdeslanyArray odeslane();
    void addOdeslany(const Odeslany &odeslany);
    bool hasOdeslany(int id);

    dxUlozeneArray ulozene();
    void addUlozene(const Ulozene &ulozene);
    void editUlozene(const Ulozene &ulozene);
    Ulozene getUlozene(int id);
    int getUlozeneId(int typ, const wxString &forma, int obdobi, const wxDateTime &from, const wxDateTime &to);
    size_t getUlozeneCount();
    bool deleteUlozene(int id);
    bool hasUlozene(int id);
    bool hasUlozene(int typ, const wxString &forma, int obdobi, const wxDateTime &from, const wxDateTime &to);
    bool someUlozeneDphShv();
    dxUlozeneArray ulozeneDphShv();
    bool someUlozeneDphDp3(int rok, bool radne = true);
    dxUlozeneArray ulozeneDphDp3(int rok, bool radne = true);
    wxString ulozeneXml(int id);

    Priznani priznani() const;
    void setPriznani(const Priznani &priznani);

    dxSkladArray sklady();
    void addSklad(const Sklad &sklad);
    void editSklad(const Sklad &sklad);
    Sklad getSklad(int id);
    Sklad getSklad(const wxString &nazev);
    size_t getSkladyCount();
    bool deleteSklad(int id);
    Sklad firstSklad();
    bool hasSklad(int id);
    bool hasSklad(const wxString &nazev);
    wxString getSkladNazevById(int id);
    bool isSkladUsed(int id);

    dxZasobaArray zasoby();
    void addZasoba(const Zasoba &zasoba);
    void editZasoba(const Zasoba &zasoba);
    Zasoba getZasoba(int id);
    size_t getZasobyCount();
    size_t getZasobyCountBySkladId(int skladid);
    bool deleteZasoba(int id);
    bool hasZasoba(int id);
    /* return id nove zasoby */
    int copyZasoba(int id);
    void getZasobaStav(bool fifo, dxPohybzasobArray pohyby, double *pocet, double *celkem);
    void getZasobaStav(int id, const wxDateTime &datum, double *pocet, double *celkem);
    double stavZasobyToDate(int id, const wxDateTime &datum);
    dxZasobaArray zasobyBySkladId(int skladid);
    bool zasobyOnSklad(int skladid);
    void changeZasobaOcenovani(int id, int ocenovani);
    double zasobaNakupniCena(int id);
    double getZasobyStav(const wxDateTime &toDate);
    void setZasobaStav(int id, double pocet, double celkem);
    dxZasobaViewArray zasobyview();

    dxInventuraArray inventury();
    void addInventura(const Inventura &inventura);
    void editInventura(const Inventura &inventura);
    Inventura getInventura(int id);
    bool deleteInventura(int id);
    bool hasInventura(int id);
    wxString getInventuraCisloById(int id);
    bool inventuraIncludedZasoba(int id, int zasobaid);
    wxDateTime getInventuraLastDateForZasoba(int zasobaid);
    wxDateTime getInventuraLastDateForSklad(int skladid);

    void addIvPolozka(const IvPolozka &ivpolozka);
    void editIvPolozka(const IvPolozka &ivpolozka);
    bool hasIvPolozka(int id);
    dxIvPolozkaArray ivpolozkyByInventuraId(int inventuraid);

    void addPohybzasob(const Pohybzasob &pohybzasob);
    void editPohybzasob(const Pohybzasob &pohybzasob);
    Pohybzasob getPohybzasob(int id);
    Pohybzasob getPohybzasobByInventura(int zasobaid, int inventuraid);
    Pohybzasob getPohybzasobByFaktura(int zasobaid, int fakturaid);
    bool deletePohybzasob(int id);
    bool hasPohybzasob(int id);
    dxPohybzasobArray pohybyzasobByZasobaId(int zasobaid);
    bool deletePohybyzasobByInventuraId(int inventuraid);
    bool deletePohybyzasobByFakturaId(int fakturaid, bool deleteDB=false);
    void editPohybzasobDatum(int zasobaid, int fakturaid, const wxDateTime &datum);

    dxRidicArray ridicy();
    void addRidic(const Ridic &ridic);
    Ridic getRidic(int id);
    Ridic getRidic(const wxString &jmeno);
    bool hasRidic(int id);
    bool hasRidic(const wxString &jmeno);
    wxString jmenoRidic(int id);

    dxPhmArray phm();
    Phm getPhm(int id);
    Phm getPhm(const wxString &jmeno);
    bool hasPhm(int id);
    bool hasPhm(const wxString &jmeno);
    wxString jmenoPhm(int id);

    double cenaPhm(int phmid, const wxDateTime &datum);

    dxDruhVozidlaArray druhvozidel();
    DruhVozidla getDruhVozidla(int id);
    DruhVozidla getDruhVozidla(const wxString &jmeno);
    bool hasDruhVozidla(int id);
    bool hasDruhVozidla(const wxString &jmeno);
    wxString jmenoDruhVozidla(int id);

    double nahradaKm(int druhvozidlaid, int rok);

    dxVozidloArray vozidla();
    void addVozidlo(const Vozidlo &vozidlo);
    void editVozidlo(const Vozidlo &vozidlo);
    Vozidlo getVozidlo(int id);
    size_t getVozidlaCount();
    Vozidlo firstVozidlo();
    bool deleteVozidlo(int id);
    bool hasVozidlo(int id);
    bool hasVozidlo(const wxString &rz);
    int copyVozidlo(int id);
    int getVozidloIdByRz(const wxString &rz);
    double nahradaKmForVozidlo(int id);

    dxJizdaViewArray jizdyview();
    void addJizda(const Jizda &jizda);
    void editJizda(const Jizda &jizda);
    int copyJizda(int id);
    Jizda getJizda(int id);
    bool deleteJizda(int id);
    bool hasJizda(int id);
    wxArrayString cesty();
    size_t cestyCount();
    wxArrayString ucely();
    size_t ucelyCount();
    int tachometrEnd(int vozidloid);
    dxJizdaArray jizdyByVozidlo(int vozidloid);

    //Texty for HomePage
    wxString homepagePrijmy();
    wxString homepageVydaje();
    wxString homepageVydane();
    wxString homepagePrijate();
    wxString homepagePohledavky();
    wxString homepageZavazky();

    //sort funkce
    void sortDenik(dxDenikArray &arr, DSort sort, bool asc);
    void sortDenikView(dxDenikViewArray &arr, DSort sort, bool asc);
    void sortFakturyView(dxFakturaViewArray &arr, FZPSort sort, bool asc);
    void sortZavPohView(dxZavPohViewArray &arr, FZPSort sort, bool asc);
    void sortMajetekView(dxMajetekViewArray &arr, MSort sort, bool asc);
    void sortFaPolozkyByDphSazba(dxFaPolozkaArray &arr);
    void sortFaPolozky(dxFaPolozkaArray &arr);
    void sortOdeslany(dxOdeslanyArray &arr, OSort sort, bool asc);
    void sortUlozene(dxUlozeneArray &arr);
    void sortDphDoklad(dxDphDokladArray &arr);
    void sortZasobyView(dxZasobaViewArray &arr, ZSort sort, bool asc);
    void sortJizdaView(dxJizdaViewArray &arr, JSort sort, bool asc);
    void sortDphDruh(dxDphDruhArray &arr, int sort, bool asc);
protected:
    void createDatabase();
    void createDatabaseTable(const char *table[], bool onlyInsert = false);
private:
    static DbModel *m_this;
    wxWindow *m_parent;
    wxSQLite3Database m_db;
    Firma m_firma;
    int m_currentRok;
    Smtp m_smtp;
    Priznani m_priznani;

    void checkDb();
    //checkTable fce return true jestlize ze tabulka je ok
    bool checkTableVersion();
    bool checkTableAdresa();
    bool checkTableKontakt();
    bool checkTableFirma();
    bool checkTableObdobi();
    bool checkTableDphsazba();
    bool checkTableDphdruh();
    bool checkTableMena();
    bool checkTableKurz();
    bool checkTablePokladna();
    bool checkTableBanka();
    bool checkTableBankovniustav();
    bool checkTableDeniksloupec();
    bool checkTablePredkontace();
    bool checkTableDenik();
    bool checkTableRada();
    bool checkTableFatyp();
    bool checkTableFauhrada();
    bool checkTableFaktura();
    bool checkTableZavpoh();
    bool checkTableFapolozka();
    bool checkTableUhrada();
    bool checkTableOdpiskupina();
    bool checkTableMajetek();
    bool checkTableOdpis();
    bool checkTableSettings();
    bool checkTableSmtp();
    bool checkTableOdeslany();
    bool checkTablePriznani();
    bool checkTableUlozene();
    bool checkTableSklad();
    bool checkTableZasoba();
    bool checkTableInventura();
    bool checkTableIvpolozka();
    bool checkTablePohybzasob();
    bool checkTableRidic();
    bool checkTablePhm();
    bool checkTableCenaphm();
    bool checkTableDruhVozidla();
    bool checkTableNahradakm();
    bool checkTableVozidlo();
    bool checkTableJizda();
    void fillFirma();
    void fillSmtp();
    void fillPriznani();
    void fireUpdated(int id);
    void fireMsg(const wxString &msg);
    int getCisloZDokladu(const wxString &doklad, int prefix, int last);
    wxArrayString tableColumns(const wxString &name);
    wxArrayString tableKeys(const wxString &name);
    bool hasTableColumn(const wxString &name, const wxString &column);
    void clearTable(const wxString &name);
    int tableRows(const wxString &name);
    int rowsForInsertToTable(const char *table[]);
    bool hasTableId(const wxString &table, const wxString &idcolumn, int id);
    bool hasTableId(const wxString &table, const wxString &idcolumn, const wxString &id);
    bool hasSameColumns(const wxString &table1, const wxString &table2);
    void tryExecuteUpdate(const wxString &sql);
};

#endif  /* DBMODEL_H */

