/***************************************************************
 * Name:      dxprint.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef DXPRINT_H
#define DXPRINT_H

#include <wx/print.h>
#include <wx/printdlg.h>

#ifdef __WXMAC__
    #define PAGE_OFFSET 5
    #define PAGE_HEIGHT 276
#else
    #define PAGE_OFFSET 0
    #define PAGE_HEIGHT 297
#endif
#define PREVIEW_ZOOM 100
#define PREVIEW_WIDTH 880
#define PREVIEW_WIDTHL 1280
#define PREVIEW_HEIGHT 650

class dxPrint: public wxPrintout
{
public:
    explicit dxPrint(const wxString &title);

    enum Footer {
        F_NUMBERING = 0,
        F_TEXT,
        F_NONE
    };

    virtual void OnBeginPrinting();
    virtual bool OnPrintPage(int page);
    virtual bool HasPage(int page);
    virtual bool OnBeginDocument(int startPage, int endPage);
    virtual void GetPageInfo(int *minPage, int *maxPage, int *selPageFrom, int *selPageTo);
    void performPageSetup();
    wxPrintData printData() const;
    int pages() const;
    void setPages(int pages);
    wxString headerLeftText() const;
    void setHeaderLeftText(const wxString &headerLeftText);
    wxString headerCenterText() const;
    void setHeaderCenterText(const wxString &headerCenterText);
    wxString headerRightText() const;
    void setHeaderRightText(const wxString &headerRightText);
    bool headerLine() const;
    void setHeaderLine(bool headerLine);
    int marginLeft() const;
    void setMarginLeft(int marginLeft);
    int marginRight() const;
    void setMarginRight(int marginRight);
    int marginTop() const;
    void setMarginTop(int marginTop);
    int marginBottom() const;
    void setMarginBottom(int marginBottom);
    Footer footer() const;
    void setFooter(const Footer &footer);
    wxString footerText() const;
    void setFooterText(const wxString &footerText);
    wxPrintOrientation paperOrientation() const;
    void setPaperOrientation(const wxPrintOrientation &paperOrientation);
protected:
    wxPageSetupDialogData m_pageSetup;
    wxPaperSize m_paperSize;
    wxPrintOrientation m_paperOrientation;
    int m_pages;
    int m_marginLeft;
    int m_marginRight;
    int m_marginTop;
    int m_marginBottom;
    Footer m_footer;
    wxString m_footerText;
    wxString m_headerLeftText;
    wxString m_headerCenterText;
    wxString m_headerRightText;
    bool m_headerLine;
    double m_mmToLogical;

    void drawFooter(int page, wxDC *dc, double mmToLogical);
    void drawHeader(wxDC *dc, double mmToLogical);
};

#endif  /* DXPRINT_H */

