/***************************************************************
 * Name:      styles.cpp
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#include "styles.h"
#include "utils/dxutils.h"

TextStyle::TextStyle(wxFont font, wxColour fontColour, wxColour backColour)
    : m_font(font), m_fontColour(fontColour), m_backColour(backColour)
{
}

wxFont TextStyle::font() const
{
    return m_font;
}

void TextStyle::setFont(const wxFont &font)
{
    m_font = font;
}

wxColour TextStyle::fontColour() const
{
    return m_fontColour;
}

void TextStyle::setFontColour(const wxColour &fontColour)
{
    m_fontColour = fontColour;
}

wxColour TextStyle::backColour() const
{
    return m_backColour;
}

void TextStyle::setBackColour(const wxColour &backColour)
{
    m_backColour = backColour;
}

wxXmlNode *TextStyle::createXmlNode() const
{
    wxXmlNode *node = new wxXmlNode(wxXML_ELEMENT_NODE, "TextStyle");
    node->AddAttribute("Font", dxutils::fontToString(m_font));
    node->AddAttribute("FontColour", m_fontColour.GetAsString());
    node->AddAttribute("BackColour", m_backColour.GetAsString());
    return node;
}

bool TextStyle::fromXmlNode(const wxXmlNode *node)
{
    if(!node) return false;
    if(node->GetName() != "TextStyle") return false;
    m_font = dxutils::fontFromString(node->GetAttribute("Font", dxutils::fontToString(m_font)));
    m_fontColour.Set(node->GetAttribute("FontColour", "black"));
    m_backColour.Set(node->GetAttribute("BackColour", "white"));
    return true;
}


LineStyle::LineStyle(wxColour lineColour, int width, wxPenStyle style)
    : m_lineColour(lineColour), m_width(width), m_style(style)
{
}

wxColour LineStyle::lineColour() const
{
    return m_lineColour;
}

void LineStyle::setLineColour(const wxColour &lineColour)
{
    m_lineColour = lineColour;
}

int LineStyle::width() const
{
    return m_width;
}

void LineStyle::setWidth(int width)
{
    m_width = width;
}

wxPenStyle LineStyle::style() const
{
    return m_style;
}

void LineStyle::setStyle(const wxPenStyle &style)
{
    m_style = style;
}

wxXmlNode *LineStyle::createXmlNode() const
{
    wxXmlNode *node = new wxXmlNode(wxXML_ELEMENT_NODE, "LineStyle");
    node->AddAttribute("Colour", m_lineColour.GetAsString());
    node->AddAttribute("Width", wxString::Format("%d", m_width));
    node->AddAttribute("Style", wxString::Format("%d", m_style));
    return node;
}

bool LineStyle::fromXmlNode(const wxXmlNode *node)
{
    if(!node) return false;
    if(node->GetName() != "LineStyle") return false;
    m_lineColour.Set(node->GetAttribute("Colour", "black"));
    m_width = wxAtoi(node->GetAttribute("Width", "1"));
    m_style = static_cast<wxPenStyle>(wxAtoi(node->GetAttribute("Style", "100")));
    return true;
}
