/***************************************************************
 * Name:      dxsettings.h
 * Author:    David Vachulka (archdvx@dxsolutions.org)
 * Copyright: 2014
 * License:   GPL3
 **************************************************************/

#ifndef DXSETTINGS_H
#define DXSETTINGS_H

#ifndef WX_PRECOMP
    #include <wx/wx.h>
#endif
#include "dxdefs.h"

#define dxsettings dxSettings::instance()

class dxSettings
{
public:
    dxSettings();
    dxSettings(const dxSettings &s);
    ~dxSettings();

    static dxSettings &instance();
    void load();
    void save();

    int x() const;
    void setX(int x);
    int y() const;
    void setY(int y);
    int w() const;
    void setW(int w);
    int h() const;
    void setH(int h);
    bool maximized() const;
    void setMaximized(bool maximized);
    DBMODEL dbmodel() const;
    void setDbmodel(DBMODEL dbmodel);
    wxString dbname() const;
    void setDbname(const wxString &dbname);
    void setDbname();
    dxStringMap dbmap() const;
    void setDbmap(const dxStringMap &dbmap);
    wxArrayString firmy();
    int dbcount() const;
    void setDbcount(int dbcount);
    void addDbnamePair(const wxString &dbname, const wxString &firma);
    /*!
     * \brief editDbnamePair
     * \param dbname
     * \param firma = nazev_ic
     */
    void editDbnamePair(const wxString &dbname, const wxString &firma);
    /*!
     * \brief firmaExists
     * \param firma = nazev_ic
     */
    bool firmaExists(const wxString &firma);
    /*!
     * \brief dbnameByFirma
     * \param firma = nazev_ic
     */
    wxString dbnameByFirma(const wxString &firma);
    /*!
     * \brief deleteFirma
     * \param firma = nazev_ic
     */
    bool deleteFirma(const wxString &firma);
    wxString auiperspective() const;
    void setAuiperspective(const wxString &auiperspective);
    int splatnost() const;
    void setSplatnost(int splatnost);
    wxDateTime datum() const;
    void setDatum(const wxDateTime &datum);
    wxString fakturaSablona() const;
    void setFakturaSablona(const wxString &fakturaSablona);
    wxString zalohaSablona() const;
    void setZalohaSablona(const wxString &zalohaSablona);
    wxDateTime lastBackup() const;
    void setLastBackup(const wxDateTime &lastBackup);
    wxString backupPath() const;
    void setBackupPath(const wxString &backupPath);
private:
    //GUI values
    int m_x;
    int m_y;
    int m_w;
    int m_h;
    bool m_maximized;
    wxString m_auiperspective;
    //DB values
    int m_dbcount;
    DBMODEL m_dbmodel;
    wxString m_dbname;
    dxStringMap m_dbmap;
    //Ucetni values
    int m_splatnost;
    //Ostatni
    wxDateTime m_datum; //aktualni datum aplikace
    wxString m_fakturaSablona; //fa vydana
    wxString m_zalohaSablona; //zalohava fa vydana
    wxDateTime m_lastBackup;
    wxString m_backupPath;
};

#endif /* DXSETTINGS_H */
